%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Jennifer Jackson 2017                                                        %
% A Biodiversity Approach to Cyber Security                                    %
% PhD Complexity Science                                                       %
% University of Warwick                                                        %
% http://wrap.warwick.ac.uk/98458                                              %
% This software is distributed under the  Creative Commons Attribution License %
% https://creativecommons.org/licenses/by/4.0/                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
classdef DataClass
    
    properties
    end
    
    methods
    end
    %static methods that can be called without instantiating a class
    methods (Static)
        %******************************************************
        %TOP LEVEL FUNCTIONS FOR THE FAV SCORE ALGORITHM
        %******************************************************
        %******************************************************
        %Initialisations for a single simulation for dynamic diversity
        %******************************************************
        function DataSimInit(handles,ihandles)
            if strcmp(ihandles.algModelS,'Favourability Score')
            %get data handles
             diversityModelParameters_FavScoreFigData = guidata(handles.diversityModelParameters_FavScoreFig); %load sub gui data
             %get data
            [ihandles.dataSet ihandles.alignData] = DataClass.getDatasetInfo(handles);
            [ihandles.SW_str_apps ihandles.SW_str_serv ihandles.SW_str_os ihandles.SW_str_dr ...
            ihandles.SW_freq_apps ihandles.SW_freq_serv ihandles.SW_freq_os ihandles.SW_freq_dr ...
            ihandles.CSW1 ihandles.CSW2 ihandles.CSW3 ihandles.CSW4 ihandles.CSW5 ihandles.CSW6 ihandles.CSW7 ihandles.CSW8 ihandles.CSW9 ihandles.CSW10 ihandles.CSWGroup ...
            ihandles.Genotype_apps ihandles.Genotype_serv ihandles.Genotype_os ihandles.Genotype_dr ...
            ihandles.E_apps ihandles.E_serv ihandles.E_os ihandles.E_dr ...
            ihandles.E_apps2 ihandles.E_serv2 ihandles.E_os2 ihandles.E_dr2 ...
            ihandles.CD_apps ihandles.CD_serv ihandles.CD_os ihandles.CD_dr ...
            ihandles.CD_apps2 ihandles.CD_serv2 ihandles.CD_os2 ihandles.CD_dr2 ...
            ihandles.ID_apps ihandles.ID_serv ihandles.ID_os ihandles.ID_dr ...
            ihandles.ID_apps2 ihandles.ID_serv2 ihandles.ID_os2 ihandles.ID_dr2 ...
             ihandles.IDtimeArray1 ihandles.IDtimeArray2 ...
            ] = getData(ihandles.dataSet,ihandles.alignData);
            %concat data
            ihandles.textdata = [ihandles.SW_freq_apps; ihandles.SW_freq_serv; ihandles.SW_freq_os; ihandles.SW_freq_dr];
            ihandles.Genotype = [ihandles.Genotype_apps ihandles.Genotype_serv ihandles.Genotype_os ihandles.Genotype_dr];
            ihandles.E = [ihandles.E_apps ihandles.E_serv ihandles.E_os ihandles.E_dr]'; %efficiency ratings
            ihandles.E2 = [ihandles.E_apps2 ihandles.E_serv2 ihandles.E_os2 ihandles.E_dr2]'; %efficiency ratings
            ihandles.CD = [ihandles.CD_apps ihandles.CD_serv ihandles.CD_os ihandles.CD_dr]'; %community desirability ratings
            ihandles.CD2 = [ihandles.CD_apps2 ihandles.CD_serv2 ihandles.CD_os2 ihandles.CD_dr2]'; %community desirability ratings2
            ihandles.ID = [ihandles.ID_apps ihandles.ID_serv ihandles.ID_os ihandles.ID_dr]'; %individual desirability ratings
            ihandles.ID2 = [ihandles.ID_apps2 ihandles.ID_serv2 ihandles.ID_os2 ihandles.ID_dr2]'; %individual desirability ratings
            %get size of data
            [ihandles.numNodes ihandles.len_apps ihandles.len_serv ihandles.len_os ihandles.len_dr] = DataClass.getGenoSize(ihandles.Genotype_apps,ihandles.Genotype_serv,ihandles.Genotype_os,ihandles.Genotype_dr);
            ihandles.len_sw = ihandles.len_apps+ihandles.len_serv+ihandles.len_os+ihandles.len_dr;
            [ihandles.SizeCD_apps ihandles.SizeCD_serv ihandles.SizeCD_os ihandles.SizeCD_dr ihandles.tallyLength ihandles.numOS ihandles.CDSize] = DataClass.getCDSize(ihandles.CD_apps,ihandles.CD_serv,ihandles.CD_os,ihandles.CD_dr,ihandles.CD,ihandles.CSWGroup);
            %generate dynamic weights
            [ihandles.divWeights ihandles.divWeightsE ihandles.comWeights ihandles.comWeightsE ihandles.indWeights ihandles.indWeightsE ihandles.qualWeights ihandles.qualWeightsE] = DataClass.getDynamicWeights(ihandles.numNodes,diversityModelParameters_FavScoreFigData); 
            %generate dynamic desirabilities
            [ihandles.comDes ihandles.comDesE ihandles.indDes ihandles.indDesE ihandles.qualDes ihandles.qualDesE] = DataClass.getDynamicDes(ihandles.numNodes,diversityModelParameters_FavScoreFigData);
            %align dynamic desirability data if required
            [ihandles.comDes,ihandles.indDes,ihandles.qualDes] = DataClass.alignDesDataAll(ihandles.numNodes,handles,ihandles.comDes,ihandles.indDes,ihandles.qualDes,ihandles.len_apps,ihandles.len_serv,ihandles.len_os,ihandles.len_dr,...
            ihandles.CSW1,ihandles.CSW2,ihandles.CSW3,ihandles.CSW4,ihandles.CSW5,ihandles.CSW6,ihandles.CSW7,ihandles.CSW8,ihandles.CSW9,ihandles.CSW10);
            %replicate CSW (compatability filters) to be the same size as Fav 
            ihandles.CSWGrouprep = DataClass.repCSW(ihandles.numNodes,ihandles.CDSize,ihandles.numOS,ihandles.CSWGroup);
            %get fixed encounter probability
           % ihandlesprobabilityDistr=ihandles.probabilityDistr
           % ihandles.probabilityDistr  = DataClass.getFixedEncounterProb(handles, ihandles.numNodes, ihandles.probabilityDistr);
            % ihandlesprobabilityDistr=ihandles.probabilityDistr
            end %strcmp
        end
         %******************************************************
        %Initialisations for a single run
        %******************************************************
        function DataRunInit(handles,ihandles,ohandles)
            %run initialisations
            %no changes 
        end
        %******************************************************
        %Algorithm code for a single time step
        %******************************************************
        function DataTloop(handles,ihandles,ohandles,t)
        end
        %*****************************************************
        %Gat data set info
        %*****************************************************
        function [dataSet alignData] = getDatasetInfo(handles)
            dataSet = get(handles.popupmenuInputDataset, 'Value');
            alignData = get(handles.checkboxAlignData, 'Value');
        end
        function [numNodes len_apps len_serv len_os len_dr] = getGenoSize(Genotype_apps,Genotype_serv,Genotype_os,Genotype_dr)
            numNodes = size(Genotype_apps,1)
            len_apps = size(Genotype_apps,2);
            len_serv = size(Genotype_serv,2);
            len_os = size(Genotype_os,2);
            len_dr = size(Genotype_dr,2);     
        end
        function [SizeCD_apps SizeCD_serv SizeCD_os SizeCD_dr tallyLength numOS CDSize] = getCDSize(CD_apps,CD_serv,CD_os,CD_dr,CD,CSWGroup)
            SizeCD_apps = size(CD_apps);
            SizeCD_serv = size(CD_serv);
            SizeCD_os = size(CD_os);
            SizeCD_dr = size(CD_dr);
            CDSize = size(CD);      
            tallyLength = CDSize(1);
            numOS = size(CSWGroup); %number of operating systems
            numOS = numOS(1);
        end
          function [divWeights divWeightsE comWeights comWeightsE indWeights indWeightsE qualWeights qualWeightsE] = getDynamicWeights(numNodes,handles)
            caseNo = 1; %caseNo 0 call weights from function, caseN0 = 1 % call weights from file
            %quality weighting
            handlesDisplay = handles
            qualWeightFile = get(handles.editWeightFileQual, 'String');
            qualWeights = getQualWeights(numNodes,caseNo,qualWeightFile);
            qualWeightsE = get(handles.checkboxDynQual, 'Value');
            %community desirability weighting
            comWeightFile = get(handles.editWeightFileCom, 'String');
            comWeights = getComWeights(numNodes,caseNo,comWeightFile);
            comWeightsE = get(handles.checkboxDynCom, 'Value');
            %individual desirability weighting
            indWeightFile = get(handles.editWeightFileInd, 'String');
            indWeights = getIndWeights(numNodes,caseNo,indWeightFile);
            indWeightsE = get(handles.checkboxDynInd, 'Value');
            %diversity weighting
            divWeightFile = get(handles.editWeightFileDiv, 'String'); 
            divWeights = getDivWeights(numNodes,caseNo,divWeightFile);
            divWeightsE = get(handles.checkboxDynDiv, 'Value');
        end
        function [comDes comDesE indDes indDesE qualDes qualDesE] = getDynamicDes(numNodes,handles)
            caseNo = 1; %caseNo 0 call weights from function, caseN0 = 1 % call weights from file
            %community desirabilities
            comDesFile = get(handles.editDesComFilename, 'String');
            comDes = getComDes(numNodes,caseNo,comDesFile);
            comDesE = get(handles.checkComDes, 'Value');
            %individual desirabilities
            indDesFile = get(handles.editDesIndFilename, 'String');
            indDes = getIndDes(numNodes,caseNo,indDesFile);
            indDesE = get(handles.checkIndDes, 'Value');
           %Quality desirabilities
            qualDesFile = get(handles.editDesQualFilename, 'String');
            qualDes = getQualDes(numNodes,caseNo,qualDesFile);
            qualDesE = get(handles.checkQualDes, 'Value');
        end
            %***************************************************************************
    %Data alignment and normalisation functions for dynamic desirabilities
    %****************************************************************************
    function [comDes,indDes,qualDes] = alignDesDataAll(numNodes,handles,comDes,indDes,qualDes,len_apps,len_serv,len_os,len_dr,...
            CSW1,CSW2,CSW3,CSW4,CSW5,CSW6,CSW7,CSW8,CSW9,CSW10)
        %manipulate data into correct format
        alignData=get(handles.checkboxAlignData, 'Value');
        qualDes_temp = qualDes;
        comDes_temp = comDes;
        indDes_temp = indDes;
        numChangesCom = size(comDes,2);
        numChangesInd = size(indDes,2);
        numChangesQual = size(qualDes,2);
        %Quality
        %get numChanges and call function
        for i = 1:numNodes
            for j = 1:numChangesQual
                E_apps(1:len_apps) = qualDes(i,j,2:len_apps+1);
                E_serv(1:len_serv)= qualDes(i,j,len_apps+2:len_apps+1+len_serv);
                E_os(1:len_os)= qualDes(i,j,len_apps+2+len_serv:len_apps+1+len_serv+len_os);
                E_dr(1:len_dr)= qualDes(i,j,len_apps+2+len_serv+len_os:end);
                [E_apps,E_serv,E_os,E_dr] = DataClass.alignDesDataC(alignData,E_apps,E_serv,E_os,E_dr,CSW1,CSW2,CSW3,CSW4,CSW5,CSW6,CSW7,CSW8,CSW9,CSW10);
                qualDes_temp(i,j,2:end) = [E_apps E_serv E_os E_dr];
            end
        end
        qualDes=qualDes_temp;
        %sumApps=sum(qualDes(1,1,2:9))%test for 1 node
        %community
        %get numChanges and call function
        for i = 1:numNodes
            for j = 1:numChangesCom
                CD_apps(1:len_apps) = comDes(i,j,2:len_apps+1);
                CD_serv(1:len_serv)= comDes(i,j,len_apps+2:len_apps+1+len_serv);
                CD_os(1:len_os)= comDes(i,j,len_apps+2+len_serv:len_apps+1+len_serv+len_os);
                CD_dr(1:len_dr)= comDes(i,j,len_apps+2+len_serv+len_os:end);
                [CD_apps,CD_serv,CD_os,CD_dr] = DataClass.alignDesDataC(alignData,CD_apps,CD_serv,CD_os,CD_dr,CSW1,CSW2,CSW3,CSW4,CSW5,CSW6,CSW7,CSW8,CSW9,CSW10);
                comDes_temp(i,j,2:end) = [CD_apps,CD_serv,CD_os,CD_dr];
            end
        end
        comDes=comDes_temp;
        %sumApps=sum(comDes(1,1,2:9))%test for node 1
        %Individual
        %get numChanges and call function
        for i = 1:numNodes
            for j = 1:numChangesInd
                ID_apps(1:len_apps) = indDes(i,j,2:len_apps+1);
                ID_serv(1:len_serv)= indDes(i,j,len_apps+2:len_apps+1+len_serv);
                ID_os(1:len_os)= indDes(i,j,len_apps+2+len_serv:len_apps+1+len_serv+len_os);
                ID_dr(1:len_dr)= indDes(i,j,len_apps+2+len_serv+len_os:end);
                sizeID_apps = size(ID_apps);
                sizeID_serv = size(ID_serv);
                sizeID_os = size(ID_os);
                sizeID_dr = size(ID_dr);
                [ID_apps,ID_serv,ID_os,ID_dr] = DataClass.alignDesDataC(alignData,ID_apps,ID_serv,ID_os,ID_dr,CSW1,CSW2,CSW3,CSW4,CSW5,CSW6,CSW7,CSW8,CSW9,CSW10);
                indDes_temp(i,j,2:end) = [ID_apps,ID_serv,ID_os,ID_dr];
            end
        end
        indDes=indDes_temp;
        %sumApps=sum(indDes(1,1,2:9))%test for node 1
    end
    function [CD_apps,CD_serv,CD_os,CD_dr] = alignDesDataC(alignData,CD_apps,CD_serv,CD_os,CD_dr,...
            CSW1,CSW2,CSW3,CSW4,CSW5,CSW6,CSW7,CSW8,CSW9,CSW10)
    %set normalisation of data 1->normalise to 1, 100->normalise to 100
    %calcS.m divides CD and ID by 100 so assumes normalisation to 100
    normValCD = 100;
    %CD
    %normalise arrays to sum to 1 for unfiltered version
    CD_appsUFilt=normArray(CD_apps,normValCD);%shouldn't need 100, but doesn't draw line on graph correctly otherwise
    CD_servUFilt=normArray(CD_serv,normValCD);
    CD_osUFilt=normArray(CD_os,normValCD);
    CD_drUFilt=normArray(CD_dr,normValCD);
    %normalise to OS for filtered version
    [CD_appsFilt,CD_servFilt,CD_osFilt,CD_drFilt] = normToOS(normValCD, ...
        CSW1,CSW2,CSW3,CSW4,CSW5,CSW6,CSW7,CSW8,CSW9,CSW10, ...
        CD_apps,CD_serv,CD_os,CD_dr);
    %reassign
    if alignData == 1
        %normalise and align to OS
        CD_apps = CD_appsFilt; 
        CD_serv = CD_servFilt;
        CD_os = CD_osFilt;
        CD_dr = CD_drFilt;
    else
        %just normalise
        CD_apps = CD_appsUFilt; 
        CD_serv = CD_servUFilt;
        CD_os = CD_osUFilt;
        CD_dr = CD_drUFilt;
    end
    CD_sumApps = sum(CD_apps);
    end
    function CSWGrouprep = repCSW(numNodes,CDSize,numOS,CSWGroup)
       %replicate CSW (compatability filters) to be the same size as Fav 
       CSWGrouprep = zeros(numNodes,CDSize(1),numOS);
       for m = 1:numOS
           CSWGrouprep(:,:,m) = repmat(CSWGroup(m,:),numNodes,1);
       end;           
    end

    
    
    
    
        
    end
end

