%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Jennifer Jackson 2017                                                        %
% A Biodiversity Approach to Cyber Security                                    %
% PhD Complexity Science                                                       %
% University of Warwick                                                        %
% http://wrap.warwick.ac.uk/98458                                              %
% This software is distributed under the  Creative Commons Attribution License %
% https://creativecommons.org/licenses/by/4.0/                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
classdef DynAlgClass
    
    properties
    end
    
    methods
    end
    %static methods that can be called without instantiating a class
    methods (Static)
        %******************************************************
        %TOP LEVEL FUNCTIONS FOR THE FAV SCORE ALGORITHM
        %******************************************************
        %******************************************************
        %Initialisations for a single simulation
        %******************************************************
        function AlgModelFavScoreSimInit(handles,ihandles,ohandles)
            if strcmp(ihandles.algModelS,'Favourability Score')
            %set up inputs
            diversityModelParameters_FavScoreFigData = guidata(handles.diversityModelParameters_FavScoreFig); %load sub gui data
            %get current settings from GUI (static parameters)
            [ihandles.turnOnSol1 ihandles.turnOnSol2 ihandles.turnOnSol3 ihandles.turnOnSol4 ihandles.turnOnSol5] = DynAlgClass.getAlgChoice(diversityModelParameters_FavScoreFigData);
           % [ihandles.numRunsPerSim ihandles.numTimeSteps] = DynAlgClass.getSimInfo(handles);
            [ihandles.minEncounters ihandles.maxEncounters ihandles.distStr] = DynAlgClass.getEncounterInfo(diversityModelParameters_FavScoreFigData);
            [ihandles.minstd4Tolerance ihandles.maxstd4Tolerance ihandles.minstd5Tolerance ihandles.maxstd5Tolerance ihandles.sampleSize ...
                   ihandles.restartUpdateTest ihandles.stopUpdate4 ihandles.restartUpdate4 ihandles.stopUpdate5 ihandles.restartUpdate5] = DynAlgClass.getStopStartInfo(diversityModelParameters_FavScoreFigData, ihandles.maxEncounters);
            [ihandles.ChangeDesirabilitiesFlag ihandles.tChange1Enable ihandles.tChange1 ihandles.tChange2Enable ihandles.tChange2] = DynAlgClass.getComDesFlags(diversityModelParameters_FavScoreFigData);
            [ihandles.ChangeIndDesirabilitiesFlag ihandles.tChangeInd1Enable ihandles.tChangeInd1 ihandles.tChangeInd2Enable ihandles.tChangeInd2] = DynAlgClass.getIndDesFlags(diversityModelParameters_FavScoreFigData);
            [ihandles.ChangeEffFlag ihandles.tChangeEff1Enable ihandles.tChangeEff1 ihandles.tChangeEff2Enable ihandles.tChangeEff2] = DynAlgClass.getEffDesFlags(diversityModelParameters_FavScoreFigData);
            [ihandles.We ihandles.Wcd ihandles.Wid ihandles.wdiv] = DynAlgClass.getWeights(diversityModelParameters_FavScoreFigData);
            
            %initialise outputs
            %[ohandles.Nei_sol1 ohandles.Nei_sol2 ohandles.Nei_sol3 ohandles.Nei_sol4 ohandles.Nei_sol5] = DynAlgClass.initSimDivOutputs(ihandles.numRunsPerSim, ihandles.numTimeSteps);
            [ohandles.nodesDiversity4 ohandles.nodesMean4 ohandles.nodesStd4 ohandles.updateStopDisplay4 ohandles.minStd4Curve ohandles.updateStopDisplay5 ohandles.nodesStd5 ohandles.minStd5Curve ...
                ohandles.lastStoppedStdMatrix ohandles.lastStoppedStdMatrix2 ohandles.Nei_indDynamic] = DynAlgClass.initSimOutputs(ihandles.numRunsPerSim, ihandles.numNodes, ihandles.numTimeSteps);
            end %strcmp
        end
        %******************************************************
        %Initialisations for a single run
        %******************************************************
        function AlgModelFavScoreRunInit(handles,ihandles,rhandles)
        if strcmp(ihandles.algModelS,'Favourability Score')
        %Total score - can be fixed for a given simulation
        %reset weight size
        diversityModelParameters_FavScoreFigData = guidata(handles.diversityModelParameters_FavScoreFig); %load sub gui data
        [ihandles.We ihandles.Wcd ihandles.Wid ihandles.wdiv] = DynAlgClass.getWeights(diversityModelParameters_FavScoreFigData);
        %initialise calcS
        rhandles.S = calcS(ihandles.We, ihandles.E, ihandles.Wcd, ihandles.CD, ihandles.Wid, ihandles.ID);
        %for calcS changing during simulation
        [rhandles.S_CD rhandles.S_ID rhandles.S_ID_apps rhandles.S_ID_os rhandles.S_ID_serv rhandles.S_ID_dr rhandles.S_E] = DynAlgClass.initRunVariablesCalcS(ihandles.CD,ihandles.ID,ihandles.ID_apps,ihandles.ID_os,ihandles.ID_serv,ihandles.ID_dr,ihandles.E);

        %recording individual diversity dynamically over time
        rhandles.Nei_indDynamic = zeros(1,ihandles.numTimeSteps);
        rhandles.counterEncounters = zeros(ihandles.numNodes,1); 
        rhandles.updateYes = zeros(ihandles.numNodes,1); %this is used
        
        %recording genotype updates to be used as input to the state
        %machine for removing infection whenever a new genotype is loaded
        %which is used to represent a VM being destroyed and created
        rhandles.updateGenotypeYes = zeros(ihandles.numNodes,1);

        %bespoke solutions 1 to 3 - no longer used
        if or(or(ihandles.turnOnSol1 == 1,ihandles.turnOnSol2 == 1),ihandles.turnOnSol3 == 1)
        [rhandles.updateStop1 rhandles.updateStop2 rhandles.updateStop3] = DynAlgClass.initRunVariablesBespokeSols(ihandles.numNodes);
        end
        if ihandles.turnOnSol1 == 1
        [rhandles.GenotypeSol1 rhandles.GenotypeSol1_apps rhandles.GenotypeSol1_serv rhandles.GenotypeSol1_os rhandles.GenotypeSol1_dr] = DynAlgClass.initRunVariablesGenotype(ihandles.Genotype,ihandles.Genotype_apps,ihandles.Genotype_serv,ihandles.Genotype_os,ihandles.Genotype_dr); 
        end
        if ihandles.turnOnSol2 == 1
        [rhandles.GenotypeSol2 rhandles.GenotypeSol2_apps rhandles.GenotypeSol2_serv rhandles.GenotypeSol2_os rhandles.GenotypeSol2_dr] = DynAlgClass.initRunVariablesGenotype(ihandles.Genotype,ihandles.Genotype_apps,ihandles.Genotype_serv,ihandles.Genotype_os,ihandles.Genotype_dr); 
        end
        if ihandles.turnOnSol3 == 1
        [rhandles.GenotypeSol3 rhandles.GenotypeSol3_apps rhandles.GenotypeSol3_serv rhandles.GenotypeSol3_os rhandles.GenotypeSol3_dr] = DynAlgClass.initRunVariablesGenotype(ihandles.Genotype,ihandles.Genotype_apps,ihandles.Genotype_serv,ihandles.Genotype_os,ihandles.Genotype_dr); 
        end
        %solution 4
        [rhandles.updateStop4 rhandles.reachTolFlag4 rhandles.minStd4 rhandles.maxStd4 rhandles.minStd4Curve rhandles.nodesDiversity4 rhandles.updateStopDisplay4 ...
        rhandles.nodesMean4 rhandles.nodesMean4Filt rhandles.nodesStd4 rhandles.nodesStd4Filt rhandles.displayStopVal rhandles.displayStartVal rhandles.updateStopDisplayFlagOld4 ...
        rhandles.lastStoppedStd rhandles.lastStoppedStdMatrix rhandles.lastStoppedStdMatrix2 rhandles.nodesDiversity4_buffer] = DynAlgClass.initRunVariablesSol4(ihandles.numNodes,ihandles.minstd4Tolerance,ihandles.maxstd4Tolerance,ihandles.numTimeSteps,ihandles.sampleSize);
        [rhandles.GenotypeSol4 rhandles.GenotypeSol4_apps rhandles.GenotypeSol4_serv rhandles.GenotypeSol4_os rhandles.GenotypeSol4_dr] = DynAlgClass.initRunVariablesGenotype(ihandles.Genotype,ihandles.Genotype_apps,ihandles.Genotype_serv,ihandles.Genotype_os,ihandles.Genotype_dr);    

        %solution 5
        [rhandles.updateStop5 rhandles.reachTolFlag5 rhandles.minStd5 rhandles.maxStd5 rhandles.minStd5Curve rhandles.nodesDiversity5 rhandles.updateStopDisplay5 ...
        rhandles.nodesMean5 rhandles.nodesStd5 rhandles.displayStopVal5 rhandles.displayStartVal5 rhandles.updateStopDisplayFlagOld5 rhandles.lastStoppedStd5 rhandles.nodesDiversity5_buffer] = DynAlgClass.initRunVariablesSol5(ihandles.numNodes,ihandles.minstd5Tolerance,ihandles.maxstd5Tolerance,ihandles.numTimeSteps,ihandles.sampleSize);
        [rhandles.GenotypeSol5 rhandles.GenotypeSol5_apps rhandles.GenotypeSol5_serv rhandles.GenotypeSol5_os rhandles.GenotypeSol5_dr] = DynAlgClass.initRunVariablesGenotype(ihandles.Genotype,ihandles.Genotype_apps,ihandles.Genotype_serv,ihandles.Genotype_os,ihandles.Genotype_dr); 

        %for counting software
        [rhandles.tallyUpdates rhandles.tallySol1 rhandles.tallySol2 rhandles.tallySol3 rhandles.tallySol4 rhandles.tallySol5] = DynAlgClass.initRunVariablesTally(ihandles.numNodes,ihandles.tallyLength);
        %initialise dynamic variables for dynamic weights and dynamic desirabilities
        [rhandles.divWeightsValueOld,rhandles.comWeightsValueOld,rhandles.indWeightsValueOld,rhandles.qualWeightsValueOld,...
                    rhandles.calcSE_Comw,rhandles.calcSE_Indw,rhandles.calcSE_Qualw,rhandles.comDesValueOld,rhandles.indDesValueOld,rhandles.qualDesValueOld,...
                    rhandles.calcSE_Comd,rhandles.calcSE_Indd,rhandles.calcSE_Quald,ihandles.We,ihandles.Wcd,ihandles.Wid,rhandles.S_CD,rhandles.S_E] = DynAlgClass.initRunVariablesDynamic(ihandles.numNodes,ihandles.len_sw,ihandles.We,ihandles.Wcd,ihandles.Wid,rhandles.S_CD,rhandles.S_E);

        %set up type of encounters & number of encounters before an update
        plotData=0;
        rhandles.numEncounters = stochasticEncounters(ihandles.distStr,ihandles.numNodes,ihandles.minEncounters,ihandles.maxEncounters,plotData);
        end %str cmp
        end
        
        function AlgModelFavScoreAssignOutputs(runNumber,ihandles,rhandles,ohandles)
            %from ihandles
            %ohandles.numNodes=ihandles.numNodes;
            %ohandles.numRunsPerSim=ihandles.numRunsPerSim;
            %ohandles.numTimeSteps=ihandles.numTimeSteps;
            ohandles.len_apps=ihandles.len_apps;
            ohandles.len_serv=ihandles.len_serv;
            ohandles.len_os=ihandles.len_os;
            ohandles.len_dr=ihandles.len_dr;
            ohandles.CD_apps=ihandles.CD_apps;
            ohandles.CD_serv=ihandles.CD_serv;
            ohandles.CD_os=ihandles.CD_os;
            ohandles.CD_dr=ihandles.CD_dr;
            ohandles.ChangeDesirabilitiesFlag=ihandles.ChangeDesirabilitiesFlag;
            ohandles.tChange1=ihandles.tChange1;
            ohandles.tChange2=ihandles.tChange2;
            ohandles.CD_apps2=ihandles.CD_apps2;
            ohandles.CD_serv2=ihandles.CD_serv2;
            ohandles.CD_os2=ihandles.CD_os2;
            ohandles.CD_dr2=ihandles.CD_dr2;
            ohandles.ChangeIndDesirabilitiesFlag=ihandles.ChangeIndDesirabilitiesFlag;
            ohandles.tChangeInd1=ihandles.tChangeInd1;
            ohandles.tChangeInd2=ihandles.tChangeInd2;
            ohandles.ID2=ihandles.ID2;
            ohandles.ID_apps2=ihandles.ID_apps2;
            ohandles.ID_serv2=ihandles.ID_serv2;
            ohandles.ID_os2=ihandles.ID_os2;
            ohandles.ID_dr2=ihandles.ID_dr2;
            ohandles.E_apps=ihandles.E_apps;
            ohandles.E_serv=ihandles.E_serv;
            ohandles.E_os=ihandles.E_os;
            ohandles.E_dr=ihandles.E_dr;
            ohandles.ChangeEffFlag=ihandles.ChangeEffFlag;
            ohandles.tChangeEff1=ihandles.tChangeEff1;
            ohandles.tChangeEff2=ihandles.tChangeEff2;
            ohandles.E_apps2=ihandles.E_apps2;
            ohandles.E_serv2=ihandles.E_serv2;
            ohandles.E_os2=ihandles.E_os2;
            ohandles.E_dr2=ihandles.E_dr2;
            ohandles.Genotype=ihandles.Genotype;
            ohandles.Genotype_apps=ihandles.Genotype_apps;
            ohandles.Genotype_serv=ihandles.Genotype_serv;
            ohandles.Genotype_os=ihandles.Genotype_os;
            ohandles.Genotype_dr=ihandles.Genotype_dr ;
            ohandles.textdata=ihandles.textdata;
            ohandles.ID=ihandles.ID;
            ohandles.ID_apps=ihandles.ID_apps;
            ohandles.ID_serv=ihandles.ID_serv;
            ohandles.ID_os=ihandles.ID_os;
            ohandles.ID_dr=ihandles.ID_dr;
            ohandles.minstd4Tolerance=ihandles.minstd4Tolerance;
            ohandles.maxstd4Tolerance=ihandles.maxstd4Tolerance;
            ohandles.minstd5Tolerance=ihandles.minstd5Tolerance;
            ohandles.maxstd5Tolerance=ihandles.maxstd5Tolerance;
            ohandles.turnOnSol1=ihandles.turnOnSol1;
            ohandles.turnOnSol2=ihandles.turnOnSol2;
            ohandles.turnOnSol3=ihandles.turnOnSol3;
            ohandles.turnOnSol4=ihandles.turnOnSol4;
            ohandles.turnOnSol5=ihandles.turnOnSol5;
            ohandles.stdMargin=ihandles.stdMargin;
            ohandles.singleStopMode = ihandles.singleStopMode;
            ohandles.restartMode = ihandles.restartMode;  
            ohandles.ChangeOwnIndDesirabilitiesFlag = ihandles.ChangeOwnIndDesirabilitiesFlag;            
            %from rhandles
            ohandles.nodesDiversity4(runNumber,:,:)=rhandles.nodesDiversity4;
            %ohandles.nodesMean4=rhandles.nodesMean4;
            ohandles.nodesMean4(runNumber,:,:)=rhandles.nodesMean4;
            %ohandles.nodesStd4=rhandles.nodesStd4;
            ohandles.nodesStd4(runNumber,:,:)=rhandles.nodesStd4;
            %ohandles.updateStopDisplay4=rhandles.updateStopDisplay4;
            ohandles.updateStopDisplay4(runNumber,:,:)=rhandles.updateStopDisplay4;
            %ohandles.nodesStd4=rhandles.nodesStd4;
            ohandles.nodesStd4(runNumber,:,:)=rhandles.nodesStd4;
            %ohandles.minStd4Curve=rhandles.minStd4Curve;
            ohandles.minStd4Curve(runNumber,:,:)=rhandles.minStd4Curve;
            %ohandles.updateStopDisplay5=rhandles.updateStopDisplay5;
            ohandles.updateStopDisplay5(runNumber,:,:)=rhandles.updateStopDisplay5;
            %ohandles.nodesStd5=rhandles.nodesStd5;
            ohandles.nodesStd5(runNumber,:,:)=rhandles.nodesStd5;
            %ohandles.minStd5Curve=rhandles.minStd5Curve;  
            ohandles.minStd5Curve(runNumber,:,:)=rhandles.minStd5Curve;
            %ohandles.lastStoppedStdMatrix=rhandles.lastStoppedStdMatrix;
            ohandles.lastStoppedStdMatrix(runNumber,:,:)=rhandles.lastStoppedStdMatrix;
            %ohandles.lastStoppedStdMatrix2=rhandles.lastStoppedStdMatrix2;
            ohandles.lastStoppedStdMatrix2(runNumber,:,:)=rhandles.lastStoppedStdMatrix2;
            %ohandles.Nei_indDynamic=rhandles.Nei_indDynamic;
            ohandles.Nei_indDynamic(runNumber,:,:)=rhandles.Nei_indDynamic;
        end
        %******************************************************
        %Algorithm code for a single time step
        %******************************************************
        %function DynAlgClass.AlgModelFavScoreTloop(t,ihandles,rhandles,ohandles,currentSimNumber)
            
       %end
        %******************************************************
        %Get simulation settings (static parameters)
        %******************************************************
        function [turnOnSol1 turnOnSol2 turnOnSol3 turnOnSol4 turnOnSol5] = getAlgChoice(handles)
            %flags to turn on or off solutions - Algorithm choice
            %plot options include:
            %1.plot solutions 1 to 5
            %2.plot solutions 4 & 5 only
            %3.plot solution 4 only
        if get(handles.radiobuttonSelectAlg4Only, 'Value')==1
            turnOnSol1 = 0;
            turnOnSol2 = 0;
            turnOnSol3 = 0;
            turnOnSol4 = 1;
            turnOnSol5 = 0;
        elseif get(handles.radiobuttonSelectAlg4And5, 'Value')==1
            turnOnSol1 = 0;
            turnOnSol2 = 0;
            turnOnSol3 = 0;
            turnOnSol4 = 1;
            turnOnSol5 = 1;
        elseif get(handles.radiobuttonSelectAlg1To5, 'Value')==1
            turnOnSol1 = 1;
            turnOnSol2 = 1;
            turnOnSol3 = 1;
            turnOnSol4 = 1;
            turnOnSol5 = 1;
        end;
        end
       % function [numSims numRuns] = getSimInfo(handles)
       %     %number of simulations(number of simulation runs)
       %     numSims = get(handles.editNumRuns, 'Value');
       %     %number of runs per simulation(number of time steps per simulation run)
       %     numRuns = get(handles.editNumTimeSteps, 'Value');
       % end
        function [minEncounters maxEncounters distStr] = getEncounterInfo(handles)
            %for stochastic encounters set number of encounters different for each node
            %according to a specific distribution
            minEncounters = get(handles.editMinEncounters, 'Value');
            maxEncounters = get(handles.editMaxEncounters, 'Value');
            %can be set to 'fixed' 'normal' 'uniform'
            %when set to 'fixed' the maxEncounters number will be used as the fixed
            %number
            encounterType = get(handles.popupmenuEncounterType, 'Value');
            if encounterType ==1
            distStr = 'fixed'; %fixed number of encounters;
            elseif encounterType ==2
            distStr = 'normal'; %normal number of encounters;
            elseif encounterType ==3
            distStr = 'uniform'; %uniform number of encounters;
            else
            disp('error encounterType')
            end;
        end
        function [minstd4Tolerance maxstd4Tolerance minstd5Tolerance maxstd5Tolerance sampleSize ...
                   restartUpdateTest stopUpdate4 restartUpdate4 stopUpdate5 restartUpdate5] = getStopStartInfo(handles, maxEncounters)
            %***************************************************************************
            %stopping the genotype update procedure after stabilising for solution 4 & 5
            minstd4Tolerance = get(handles.editAlg4Minstd, 'Value'); %tolerance on the std of when to stop updating
            maxstd4Tolerance = get(handles.editAlg4MaxStd, 'Value'); %tolerance on the std of when to stop updating
            minstd5Tolerance = get(handles.editAlg5MinStd, 'Value'); %tolerance on the std of when to stop updating
            maxstd5Tolerance = get(handles.editAlg5MaxStd, 'Value'); %tolerance on the std of when to stop updating
            %(std of last 100*maxEncounters samples)
            sampleSize = get(handles.editUpdateSampleSize, 'Value')*maxEncounters;
            alg4Update = get(handles.popupmenuAlg4UpdateType, 'Value');
            alg5Update = get(handles.popupmenuAlg5UpdateType, 'Value');
            %alg4*******************
            restartUpdateTest=0;
            if alg4Update==1
            %continuous update
            stopUpdate4 = 0;
            restartUpdate4 = 0;
            elseif alg4Update==2
            %Single stop update when reached std tolerance and sample size
            stopUpdate4 = 1 ;%set to 1 to stop the nodes updating when reached the stdTolerance
            restartUpdate4 = 0;
            elseif alg4Update==3
            %multiple stop update when reached std tolerance and sample size
            stopUpdate4 = 0;
            restartUpdate4 = 1 ;%set to 1 to stop updating when minimum std found, and then update again if a lower std is found
            end
            %alg5*****************
            if alg5Update==1
                %continuous update
                stopUpdate5 = 0;
                restartUpdate5 = 0;
            elseif alg5Update==2
                %Single stop update when reached std tolerance and sample size
                stopUpdate5 = 1; %set to 1 to stop the nodes updating when reached the stdTolerance
                restartUpdate5 = 0;
            elseif alg5Update==3
                %multiple stop update when reached std tolerance and sample size
                stopUpdate5 = 0;
                restartUpdate5 = 1; %set to 1 to stop updating when minimum std found, and then update again if a lower std is found  
            end
        end
        function [ChangeDesirabilitiesFlag tChange1Enable tChange1 tChange2Enable tChange2] = getComDesFlags(handles)
            %change community desirabilities during the simulation
            %flag to indicate sim should change the desirabilitiies
            disp('get flags from gui')
            ChangeDesirabilitiesFlag = get(handles.checkboxChangeCommDes, 'Value');
            %time of changes
            tChange1Enable = get(handles.checkboxCommTime1, 'Value');
            tChange1 = get(handles.editCommTime1, 'Value');
            tChange2Enable = get(handles.checkboxCommTime2, 'Value');
            tChange2 = get(handles.editCommTime2, 'Value');
        end
        function [ChangeIndDesirabilitiesFlag tChangeInd1Enable tChangeInd1 tChangeInd2Enable tChangeInd2] = getIndDesFlags(handles)
            ChangeIndDesirabilitiesFlag = get(handles.checkboxChangeIndDes, 'Value');
            %time of changes
            tChangeInd1Enable = get(handles.checkboxIndTime1, 'Value');
            tChangeInd1 = get(handles.editIndTime1, 'Value');
            tChangeInd2Enable = get(handles.checkboxIndTime2, 'Value');
            tChangeInd2 = get(handles.editIndTime2, 'Value');
        end
        function [ChangeEffFlag tChangeEff1Enable tChangeEff1 tChangeEff2Enable tChangeEff2] = getEffDesFlags(handles)
            ChangeEffFlag = get(handles.checkboxChangeEff, 'Value');
            %time of changes
            tChangeEff1Enable = get(handles.checkboxEffTime1, 'Value');
            tChangeEff1 = get(handles.editEffTime1, 'Value');
            tChangeEff2Enable = get(handles.checkboxEffTime2, 'Value');
            tChangeEff2 = get(handles.editEffTime2, 'Value');
        end
        function [We Wcd Wid wdiv] = getWeights(handles)
            %efficiency weighting
            We = get(handles.editEffWeight, 'Value');
            %community desirability weighting
            Wcd = get(handles.editCommWeight, 'Value');
            %individual desirability weighting
            Wid = get(handles.editIndWeight, 'Value');
            %diversity weighting, set to 0 to turn off diversity
            wdiv = get(handles.editDiversityWeight, 'Value');
            %wdiv = 0.00000001
        end
        function [divWeights divWeightsE comWeights comWeightsE indWeights indWeightsE qualWeights qualWeightsE] = getDynamicWeights(numNodes,handles)
            caseNo = 1; %caseNo 0 call weights from function, caseN0 = 1 % call weights from file
            %quality weighting
            qualWeightFile = get(handles.editWeightFileQual, 'String');
            qualWeights = getQualWeights(numNodes,caseNo,qualWeightFile);
            qualWeightsE = get(handles.checkboxDynQual, 'Value');
            %community desirability weighting
            comWeightFile = get(handles.editWeightFileCom, 'String');
            comWeights = getComWeights(numNodes,caseNo,comWeightFile);
            comWeightsE = get(handles.checkboxDynCom, 'Value');
            %individual desirability weighting
            indWeightFile = get(handles.editWeightFileInd, 'String');
            indWeights = getIndWeights(numNodes,caseNo,indWeightFile);
            indWeightsE = get(handles.checkboxDynInd, 'Value');
            %diversity weighting
            divWeightFile = get(handles.editWeightFileDiv, 'String'); 
            divWeights = getDivWeights(numNodes,caseNo,divWeightFile);
            divWeightsE = get(handles.checkboxDynDiv, 'Value');
        end
        function [comDes comDesE indDes indDesE qualDes qualDesE] = getDynamicDes(numNodes,handles)
            caseNo = 1; %caseNo 0 call weights from function, caseN0 = 1 % call weights from file
            %community desirabilities
            comDesFile = get(handles.editDesComFilename, 'String');
            comDes = getComDes(numNodes,caseNo,comDesFile);
            comDesE = get(handles.checkComDes, 'Value');
            %individual desirabilities
            indDesFile = get(handles.editDesIndFilename, 'String');
            indDes = getIndDes(numNodes,caseNo,indDesFile);
            indDesE = get(handles.checkIndDes, 'Value');
           %Quality desirabilities
            qualDesFile = get(handles.editDesQualFilename, 'String');
            qualDes = getQualDes(numNodes,caseNo,qualDesFile);
            qualDesE = get(handles.checkQualDes, 'Value');
        end
        function [dataSet alignData] = getDatasetInfo(handles)
            dataSet = get(handles.popupmenuInputDataset, 'Value');
            alignData = get(handles.checkboxAlignData, 'Value');
        end
        %********************************************************
        %Dataset functions
        %********************************************************
        function [numNodes len_apps len_serv len_os len_dr] = getGenoSize(Genotype_apps,Genotype_serv,Genotype_os,Genotype_dr)
            numNodes = size(Genotype_apps,1)
            len_apps = size(Genotype_apps,2);
            len_serv = size(Genotype_serv,2);
            len_os = size(Genotype_os,2);
            len_dr = size(Genotype_dr,2);     
        end
        function [SizeCD_apps SizeCD_serv SizeCD_os SizeCD_dr tallyLength numOS CDSize] = getCDSize(CD_apps,CD_serv,CD_os,CD_dr,CD,CSWGroup)
            SizeCD_apps = size(CD_apps);
            SizeCD_serv = size(CD_serv);
            SizeCD_os = size(CD_os);
            SizeCD_dr = size(CD_dr);
            CDSize = size(CD);      
            tallyLength = CDSize(1);
            numOS = size(CSWGroup); %number of operating systems
            numOS = numOS(1);
        end
        %**************************************************************
        %Initialisation functions for a SIMULATION
        %INITIALISATION OF MATRICES AND VARIABLES
        %***************************************************************
        %THIS FUNCTION IS NO LONGER USED
       % function [probabilityDistr randomNumber Nei_sol1 Nei_sol2 Nei_sol3 Nei_sol4 Nei_sol5 CSWGrouprep] = initSimVariables(numNodes,numSims,numRuns,CDSize,numOS,CSWGroup)
       %     %number of encounters before an update
       %     %numEncounters = zeros(numNodes,1);
       %     %probability of an encounter
       %     probabilityDistr = zeros(numNodes,1);
       %     %probability of an encounter
       %     probabilityDistr(:) = 0.5;%currently fixed, could be stochastic %WRITE FUNCTION TO GO HERE  
       %     
       %     %uniform random number to decide whether to meet another node
       %     randomNumber = zeros(numNodes,1);
       %     %diversity index
       %     %Nei_sol1 = zeros(numSims,numRuns); 
       %     %Nei_sol2 = zeros(numSims,numRuns); 
       %     %Nei_sol3 = zeros(numSims,numRuns);
       %     %NOT USED
       %     Nei_sol1 = 0; %NOT USED
       %     Nei_sol2 = 0; %NOT USED
       %     Nei_sol3 = 0;%NOT USED
       %     Nei_sol4 = zeros(numSims,numRuns); 
       %     Nei_sol5 = zeros(numSims,numRuns);
       %     %replicate CSW (compatability filters) to be the same size as Fav 
       %     CSWGrouprep = zeros(numNodes,CDSize(1),numOS);
       %     for m = 1:numOS
       %         CSWGrouprep(:,:,m) = repmat(CSWGroup(m,:),numNodes,1);
       %     end;           
       % end
      %  function [Nei_sol1 Nei_sol2 Nei_sol3 Nei_sol4 Nei_sol5] = initSimDivOutputs(numRunsPerSim, numTimeSteps)
      %      %diversity index
      %      Nei_sol1 = zeros(numRunsPerSim,numTimeSteps); 
      %      Nei_sol2 = zeros(numRunsPerSim,numTimeSteps); 
      %      Nei_sol3 = zeros(numRunsPerSim,numTimeSteps); 
      %      Nei_sol4 = zeros(numRunsPerSim,numTimeSteps); 
      %      Nei_sol5 = zeros(numRunsPerSim,numTimeSteps);   
      %  end
        function [nodesDiversity4 nodesMean4 nodesStd4 updateStopDisplay4 minStd4Curve updateStopDisplay5 nodesStd5 minStd5Curve ...
                lastStoppedStdMatrix lastStoppedStdMatrix2 Nei_indDynamic] = initSimOutputs(numRunsPerSim, numNodes, numTimeSteps)
            %diversity index
            %nodesDiversity4 = zeros(numRunsPerSim,numNodes,numTimeSteps+1); 
            %nodesMean4 = zeros(numRunsPerSim,numNodes,numTimeSteps+1);
            %nodesStd4 = zeros(numRunsPerSim,numNodes,numTimeSteps+1);
            nodesDiversity4 = zeros(numRunsPerSim,numNodes,numTimeSteps); %updated 24th April 2016
            nodesMean4 = zeros(numRunsPerSim,numNodes,numTimeSteps); %updated 24th April 2016
            nodesStd4 = zeros(numRunsPerSim,numNodes,numTimeSteps); %updated 24th April 2016
            updateStopDisplay4 = ones(numRunsPerSim,numTimeSteps,numNodes);
            minStd4Curve = zeros(numRunsPerSim,numNodes,1);
            updateStopDisplay5 = ones(numRunsPerSim,numTimeSteps,numNodes);
            %nodesStd5 = zeros(numRunsPerSim,numNodes,numTimeSteps+1);
            nodesStd5 = zeros(numRunsPerSim,numNodes,numTimeSteps);%updated 24th April 2016
            minStd5Curve = zeros(numRunsPerSim,numNodes,1);
            lastStoppedStdMatrix = ones(numRunsPerSim,numTimeSteps,numNodes);
            lastStoppedStdMatrix2 = ones(numRunsPerSim,numTimeSteps,numNodes);
            Nei_indDynamic = zeros(numRunsPerSim,1,numTimeSteps);
        end
        %**************************************************************
        %Initialisation functions for a RUN
        %INITIALISATION OF MATRICES AND VARIABLES
        %***************************************************************
        function [S_CD S_ID S_ID_apps S_ID_os S_ID_serv S_ID_dr S_E] = initRunVariablesCalcS(CD,ID,ID_apps,ID_os,ID_serv,ID_dr,E)
            S_CD = CD;
            S_ID = ID;
            S_ID_apps = ID_apps;
            S_ID_os = ID_os;
            S_ID_serv = ID_serv;
            S_ID_dr = ID_dr;
            S_E = E;         
        end
        function [tallyUpdates tallySol1 tallySol2 tallySol3 tallySol4 tallySol5] = initRunVariablesTally(numNodes,tallyLength)
            %count the sw/hw variants encountered
            tallyUpdates = zeros(numNodes,1);
            tallySol1 = zeros(numNodes,tallyLength);
            tallySol2 = zeros(numNodes,tallyLength);
            tallySol3 = zeros(numNodes,tallyLength);
            tallySol4 = zeros(numNodes,tallyLength);
            tallySol5 = zeros(numNodes,tallyLength);
        end
        function [updateStop1 updateStop2 updateStop3] = initRunVariablesBespokeSols(numNodes)
            updateStop1 = ones(numNodes,1); %flag to stop updating
            updateStop2 = ones(numNodes,1);
            updateStop3 = ones(numNodes,1); %flag to stop updating          
        end
        function [updateStop4 reachTolFlag4 minStd4 maxStd4 minStd4Curve nodesDiversity4 ...
                updateStopDisplay4 nodesMean4 nodesMean4Filt nodesStd4 nodesStd4Filt ...
                displayStopVal4 displayStartVal4 updateStopDisplayFlagOld4 ...
                lastStoppedStd lastStoppedStdMatrix lastStoppedStdMatrix2 nodesDiversity4_buffer] = initRunVariablesSol4(numNodes,minstd4Tolerance,maxstd4Tolerance,numRuns,sampleSize)
            %Filtered case
            updateStop4 = ones(numNodes,1); %flag to stop updating
            reachTolFlag4 = zeros(numNodes,1); %flag to indicate when reached std tolerance for multiple stopping
            minStd4 = repmat(minstd4Tolerance,numNodes,1); %record minimum std value for each node, start with the tolerance number, for multiple stopping
            maxStd4 = repmat(maxstd4Tolerance,numNodes,1); %record minimum std value for each node, start with the tolerance number, for multiple stopping
            minStd4Curve = repmat(minstd4Tolerance,numNodes,1); %for graph display only
           % nodesDiversity4 = zeros(numNodes,numRuns+1);
             nodesDiversity4 = zeros(numNodes,numRuns); %updated 24th Apr 2016
           nodesDiversity4_buffer = zeros(numNodes,sampleSize); %added 24th Apr 2016
            updateStopDisplay4 = ones(numRuns,numNodes); % for display on graph when stop
            %nodesMean4 = zeros(numNodes, numRuns+1);
            %nodesMean4Filt = zeros(numNodes, numRuns+1);
            %nodesStd4= zeros(numNodes, numRuns+1);
            %nodesStd4Filt= zeros(numNodes, numRuns+1);
            nodesMean4 = zeros(numNodes, numRuns); %updated 24th Apr 2016
            nodesMean4Filt = zeros(numNodes, numRuns); %updated 24th Apr 2016
            nodesStd4= zeros(numNodes, numRuns); %updated 24th Apr 2016
            nodesStd4Filt= zeros(numNodes, numRuns); %updated 24th Apr 2016
            %solution 4 single stop variables
            displayStopVal4 = 0.4;
            displayStartVal4 = 0.6;
            updateStopDisplayFlagOld4(1:numRuns, 1:numNodes) = displayStartVal4;
            lastStoppedStd = ones(numNodes).*displayStartVal4;
            lastStoppedStdMatrix = ones(numRuns,numNodes).*displayStartVal4;
            lastStoppedStdMatrix2 = ones(numRuns,numNodes).*displayStartVal4;
        end
        function [updateStop5 reachTolFlag5 minStd5 maxStd5 minStd5Curve nodesDiversity5 ...
                updateStopDisplay5 nodesMean5 nodesStd5 displayStopVal5 displayStartVal5 updateStopDisplayFlagOld5 lastStoppedStd5 nodesDiversity5_buffer] = initRunVariablesSol5(numNodes,minstd5Tolerance,maxstd5Tolerance,numRuns,sampleSize)
            %Unfiltered case
            updateStop5 = ones(numNodes,1);
            reachTolFlag5 = zeros(numNodes,1);
            minStd5 = repmat(minstd5Tolerance,numNodes,1); %record minimum std value for each node, start with the tolerance number, for multiple stopping
            maxStd5 = repmat(maxstd5Tolerance,numNodes,1); %record minimum std value for each node, start with the tolerance number, for multiple stopping
            minStd5Curve = repmat(minstd5Tolerance,numNodes,1); %for graph display only
            %nodesDiversity5 = zeros(numNodes,numRuns+1);
            nodesDiversity5 = zeros(numNodes,numRuns); %updated 24th Apr 2016
            nodesDiversity5_buffer = zeros(numNodes,sampleSize); %added 24th Apr 2016
            updateStopDisplay5 = ones(numRuns,numNodes);
            %nodesMean5 = zeros(numNodes, numRuns+1);
            %nodesStd5= zeros(numNodes, numRuns+1);
            nodesMean5 = zeros(numNodes, numRuns);%updated 24th Apr 2016
            nodesStd5= zeros(numNodes, numRuns);%updated 24th Apr 2016
            displayStopVal5 = 0.4;
            displayStartVal5 = 0.6;
            updateStopDisplayFlagOld5(1:numRuns, 1:numNodes) = displayStartVal5;
            lastStoppedStd5 = ones(numNodes).*displayStartVal5;
        end
        function [GenotypeSol GenotypeSol_apps GenotypeSol_serv GenotypeSol_os GenotypeSol_dr] = initRunVariablesGenotype(Genotype,Genotype_apps,Genotype_serv,Genotype_os,Genotype_dr)
            GenotypeSol = Genotype;
            GenotypeSol_apps = Genotype_apps;
            GenotypeSol_serv = Genotype_serv;
            GenotypeSol_os = Genotype_os;
            GenotypeSol_dr = Genotype_dr;
        end
        function [divWeightsValueOld,comWeightsValueOld,indWeightsValueOld,qualWeightsValueOld,...
                    calcSE_Comw,calcSE_Indw,calcSE_Qualw,comDesValueOld,indDesValueOld,qualDesValueOld,...
                    calcSE_Comd,calcSE_Indd,calcSE_Quald,We,Wcd,Wid,S_CD,S_E] = initRunVariablesDynamic(numNodes,len_sw,We,Wcd,Wid,S_CD,S_E)
            %dynamic weights
            divWeightsValueOld = zeros(numNodes,1);
            comWeightsValueOld = zeros(numNodes,1);
            indWeightsValueOld = zeros(numNodes,1);
            qualWeightsValueOld = zeros(numNodes,1);
            calcSE_Comw =0; %enable calcS - comm weights
            calcSE_Indw =0; %enable calcS - ind weights
            calcSE_Qualw=0; %enable calcS - qual weights
            %dynamic desirabilities
            comDesValueOld = zeros(numNodes,1,len_sw);
            indDesValueOld = zeros(numNodes,1,len_sw);
            qualDesValueOld = zeros(numNodes,1,len_sw);
            calcSE_Comd =0; %enable calcS - comm des
            calcSE_Indd=0; %enable calcS - ind des
            calcSE_Quald=0;%enable calcS - qual des
            %for dynamic S calculation
            %extend for dynamic manipulation for all nodes
            We = repmat(We,len_sw,numNodes); %TEMP REPMAT
            Wcd = repmat(Wcd,len_sw,numNodes); %TEMP REPMAT
            S_CD = repmat(S_CD,1,numNodes); %TEMP REPMAT
            S_E = repmat(S_E,1,numNodes);  %TEMP REPMAT
            Wid = repmat(Wid,len_sw,numNodes); %TEMP REPMAT
        end
        function  [nodesDiversity4 nodesDiversity5 nodesDiversity4_buffer nodesDiversity5_buffer] = initRunVariablesOutputs(numNodes,numTimeSteps,sampleSize)
            nodesDiversity4 = zeros(numNodes,numTimeSteps);
            nodesDiversity5 = zeros(numNodes,numTimeSteps);
            nodesDiversity4_buffer = zeros(numNodes,sampleSize); 
            nodesDiversity5_buffer =zeros(numNodes,sampleSize); 
        end
        %**********************************************************************
        %functions to support dynamic changing of weights
        %**********************************************************************
        function arrayOfWeights = genArrayStructOfWeights(numNodes,numChanges)
             s = struct('changeEnable',{},'changeTime',{},'changeValue',{});
             s(1).changeEnable = zeros(1,numChanges);
             s(1).changeTime = zeros(1,numChanges);
             s(1).changeValue = zeros(1,numChanges);
             
             for i = 2:numNodes
             s(i) = s(i-1);
             end
             arrayOfWeights = s;
        end
        function arrayOfWeights = setArrayStructOfWeights(numNodes,numChanges,nodeRangeT1,nodeRangeT2,changeEnable,changeTime,changeValue)
            %set a certain number of nodes with the same weight values at a
            %specific time
            %NOTE THIS IS CALL BY VALUE, MAY NEED CALL BY REF (handles)
             s = DynAlgClass.genArrayStructOfWeights(numNodes,numChanges);            
             for i = nodeRangeT1:nodeRangeT2
             s(i).changeEnable = changeEnable;
             s(i).changeTime = changeTime;
             s(i).changeValue = changeValue;
             end
             arrayOfWeights = s;
        end
        function weights = getWeightsFromFile(fileName,numNodes)
            fileNameD=fileName
            fid = fopen(fileName, 'r');
                    if fid < 0
                    %file doesn't exist
                    else
                    text = fgetl(fid);
                    text = fgetl(fid);
                    numChanges = str2num(fgetl(fid));
                    %weights = zeros(numNodes,numChanges,numChanges);
                    while ~feof(fid)
                    text = fgetl(fid);
                    nodesStr = fgetl(fid);
                    if strcmp('all',nodesStr)
                        %disp('apply to all nodes')
                        text = fgetl(fid);
                        weightsS = fgetl(fid);
                        weightsV = str2num(weightsS);
                        sizewV = size(weightsV,2);
                        weightsTemp = zeros(1,numChanges,sizewV); % set up 1 node
                        weightsTemp(1,:,:) = weightsV;
                        weights = repmat(weightsTemp,[numNodes,1,1]); %extend to all nodes
                    else
                        nodeVal = str2num(nodesStr);
                        if size(nodeVal,2)>1
                        %disp('apply to multiple nodes')
                        text = fgetl(fid);
                        weightsS = fgetl(fid);
                        weightsV = str2num(weightsS);
                        for i = nodeVal(1) : nodeVal(end)
                         weights(i,:,:) = weightsV;   
                        end
                        else
                        %disp('apply to a single node')
                        text = fgetl(fid);
                        weightsS = fgetl(fid);
                        weightsV = str2num(weightsS);
                        weights(nodeVal,:,:) = weightsV;
                        end
                    end   
                    end
                    fclose(fid);
                    end;
        end
        function weightValues = getWeightValuesRepmat(changeNumber,weights,len_sw)
            weightValues_temp = weights(:,changeNumber,2);
            weightValues = repmat(weightValues_temp,1,len_sw);
        end
        function weightValues = getWeightValues(changeNumber,weights)
            weightValues = weights(:,changeNumber,2:end);
        end
        function weightTimes = getWeightTimes(changeNumber,weights)
            weightTimes = weights(:,changeNumber,1);
        end
        function [calcSE changeWeightValues] = changeWeights(t,changeNumber,weights,weightValuesOld)
                 %change weights of diversity
                 weightTimes = DynAlgClass.getWeightTimes(changeNumber,weights);
                 weightValuesNew = DynAlgClass.getWeightValues(changeNumber,weights);
                 %ttime = t;
                 change = (weightTimes-t)==0; %get array of zeros and 1's indicating which node matches time t
                 calcSE = sum(change)>0; %for any change enable recalc of S
                 %changeSize = size(change);%temp for testing
                 tempSize = zeros(1,1,size(weightValuesNew,3)); %make an array the correct size to repmat
                 change = repmat(change,size(tempSize));
                 %calcSE_2 = sum(change,2)>0
                 changeInv = change<1; %get inverse array
                 %newChangeSize = size(change) %temp for testing
                 %calcSESize = size(calcSE)
                 %weightOldsize = size(weightValuesOld)
                 %weightNewsize = size(weightValuesNew) %temp for testing
                 tempNew = change.*weightValuesNew; %get new values for current time
                 tempOld = changeInv.*weightValuesOld; %keep old values not ready for change
                 changeWeightValues = tempNew + tempOld;
        end
        function [calcSE changeWeightValues] = changeAllWeights(t,weights,weightValuesOld)
                    numChanges = size(weights,2);
                    weightValuesOld_temp=weightValuesOld; 
                    calcSE_temp2 = 0;
                    for i = 1:numChanges
                    [calcSE_temp changeWeightValues] = DynAlgClass.changeWeights(t,i,weights,weightValuesOld_temp);
                    calcSE_temp2 = calcSE_temp+calcSE_temp2;
                    weightValuesOld_temp = changeWeightValues;
                    end
                    calcSE= calcSE_temp2>0;
        end
        %*******************************************************************************
        %Functions to support dynamic changing of desirabilities
        %*******************************************************************************
        function desirabilities = getIndDesFromFile(fileName,numNodes) 
            fid = fopen(fileName, 'r');
                    if fid < 0
                    %file doesn't exist
                    else
                    text = fgetl(fid);
                    text = fgetl(fid);
                    numChanges = str2num(fgetl(fid));
                    while ~feof(fid)
                    text = fgetl(fid);
                    nodesStr = fgetl(fid);
                    nodeVal = str2num(nodesStr);
                    if strcmp('all',nodesStr)
                        %disp('apply to all nodes')
                        text = fgetl(fid); %desirabilities
                         %*****************************
                        timeS = fgetl(fid); %time string
                        timeV(1) = str2num(timeS); %time value
                        %************************************
                        ID = DynAlgClass.getIDdataFromFile(fid);
                        %************************************
                        %extend ID to fit number of requested nodes
                        weightsTemp(:,1,:) = DynAlgClass.extendID(ID,numNodes);
                        %*********************************************
                        for j = 2 : numChanges
                            %have to do seperate first because do not know size beforehand
                            text = fgetl(fid); %desirabilities
                            %*****************************
                            timeS = fgetl(fid); %time string
                            timeV(j) = str2num(timeS); %time value
                            %************************************
                            ID = DynAlgClass.getIDdataFromFile(fid);
                            %********************************************
                            %extend ID to fit number of requested nodes
                            weightsTemp(:,j,:) = DynAlgClass.extendID(ID,numNodes);   
                        end %end j
                        timeVrep = repmat(timeV,numNodes,1);%all nodes change at same time
                        weightsTemp(:,:,1)=timeVrep ; 
                        desirabilities = weightsTemp;
                    else 
                        if size(nodeVal,2)>1
                        %disp('apply to multiple nodes')
                        for j = 1 : numChanges
                        text = fgetl(fid); %desirabilities
                        %****************************************
                        %assign new times
                        timeS = fgetl(fid); %time function
                        timeV = str2num(timeS); %time function
                        numNodesChange = size(nodeVal,2);
                        %*****************************************
                        %assign new weights
                        ID = DynAlgClass.getIDdataFromFile(fid);
                        %extend ID to fit number of requested nodes
                        ID3(:,j,:) = DynAlgClass.extendID(ID,numNodesChange);
                        %*****************************************
                        if size(timeV,2)<numNodesChange %if number of times is less than num nodes to change, apply time to all nodes
                            timeVrep = repmat(timeV,numNodesChange,1);%all nodes change at same time
                        else
                            timeVrep = timeV'; %otherwise get individual times for each node
                        end
                        if numNodesChange > numNodes %if number to change is greater than number of actual nodes
                            timeVrepNew = timeVrep(1:numNodes);
                            endVal = numNodes-nodeVal(1)+1;
                            weightsTemp(nodeVal(1):nodeVal(endVal),j,1) = timeVrepNew(1:endVal) ;
                            weightsTemp(nodeVal(1):nodeVal(endVal),j,2:end) = ID3(1:endVal,j,2:end);
                        else
                            timeVrepNew = timeVrep; %otherwise assign directly
                            weightsTemp(nodeVal(1):nodeVal(end),j,1) = timeVrepNew; 
                            weightsTemp(nodeVal(1):nodeVal(end),j,2:end) = ID3(:,j,2:end);
                        end
                        %********************************************
                        end %end j
                        desirabilities = weightsTemp;
                        else
                         %disp('apply to single nodes')  
                        for j = 1 : numChanges
                            text = fgetl(fid); %desirabilities
                            %****************************************
                            %assign new times
                            timeS = fgetl(fid); %time function
                            timeV = str2num(timeS); %time function
                            numNodesChange = size(nodeVal,2);
                            %*****************************************
                            %assign new weights
                            ID = DynAlgClass.getIDdataFromFile(fid);
                            %*****************************************
                            weightsTemp(nodeVal,j,1) = timeV;
                            weightsTemp(nodeVal,j,2:end) = ID;
                        end %end j loop
                        desirabilities = weightsTemp;
                        end %end if
                    end   
                    end
                    fclose(fid);
                    end;
        end
        function IDExtend = extendID(ID,numNodes)
        %************************************
        %extend ID to fit number of requested nodes
        numNodesFile = size(ID,1); %ID specified for a number of nodes
        sizeReps = floor(numNodes/numNodesFile); %number to replicate
        sizeSpare = numNodes - sizeReps*numNodesFile ; %number spare
        IDTemp = zeros(numNodes,1,size(ID,2)+1); % set up matrix size
        for i = 1:sizeReps
            IDTemp(1+(i-1)*numNodesFile:i*numNodesFile,1,2:end) = ID;
        end
        for i = 1:sizeSpare
            IDTemp(sizeReps*numNodesFile+i,1,2:end) = ID(i,:);
        end
        %*********************************************
        IDExtend = IDTemp;
        end
        function IDdata = getIDdataFromFile(fid)
          %************************************
          text = fgetl(fid); %ID_apps text
          ID_appsS = fgetl(fid); %ID_apps string
          ID_appsV = str2num(ID_appsS); %ID_apps value
          sizeID_appsV = size(ID_appsV,2);%ID_apps size
          text = fgetl(fid); %ID_serv text
          ID_servS = fgetl(fid); %ID_serv string
          ID_servV = str2num(ID_servS); %ID_serv value
          sizeID_servV = size(ID_servV,2);%ID_serv size
          text = fgetl(fid); %ID_os text
          ID_osS = fgetl(fid); %ID_os string
          ID_osV = str2num(ID_osS); %ID_os value
          sizeID_osV = size(ID_osV,2);%ID_os size
          text = fgetl(fid); %ID_dr text
          ID_drS = fgetl(fid); %ID_dr string
          ID_drV = str2num(ID_drS); %ID_dr value
          sizeID_drV = size(ID_drV,2);%ID_dr size
          %********************************************
          IDdata = [ID_appsV ID_servV ID_osV ID_drV];
        end
    %***************************************************************************
    %Data alignment and normalisation functions for dynamic desirabilities
    %****************************************************************************
    function [comDes,indDes,qualDes] = alignDesDataAll(numNodes,handles,comDes,indDes,qualDes,len_apps,len_serv,len_os,len_dr,...
            CSW1,CSW2,CSW3,CSW4,CSW5,CSW6,CSW7,CSW8,CSW9,CSW10)
        %manipulate data into correct format
        alignData=get(handles.checkboxAlignData, 'Value');
        qualDes_temp = qualDes;
        comDes_temp = comDes;
        indDes_temp = indDes;
        numChangesCom = size(comDes,2);
        numChangesInd = size(indDes,2);
        numChangesQual = size(qualDes,2);
        %Quality
        %get numChanges and call function
        for i = 1:numNodes
            for j = 1:numChangesQual
                E_apps(1:len_apps) = qualDes(i,j,2:len_apps+1);
                E_serv(1:len_serv)= qualDes(i,j,len_apps+2:len_apps+1+len_serv);
                E_os(1:len_os)= qualDes(i,j,len_apps+2+len_serv:len_apps+1+len_serv+len_os);
                E_dr(1:len_dr)= qualDes(i,j,len_apps+2+len_serv+len_os:end);
                [E_apps,E_serv,E_os,E_dr] = DynAlgClass.alignDesDataC(alignData,E_apps,E_serv,E_os,E_dr,CSW1,CSW2,CSW3,CSW4,CSW5,CSW6,CSW7,CSW8,CSW9,CSW10);
                qualDes_temp(i,j,2:end) = [E_apps E_serv E_os E_dr];
            end
        end
        qualDes=qualDes_temp;
        %sumApps=sum(qualDes(1,1,2:9))%test for 1 node
        %community
        %get numChanges and call function
        for i = 1:numNodes
            for j = 1:numChangesCom
                CD_apps(1:len_apps) = comDes(i,j,2:len_apps+1);
                CD_serv(1:len_serv)= comDes(i,j,len_apps+2:len_apps+1+len_serv);
                CD_os(1:len_os)= comDes(i,j,len_apps+2+len_serv:len_apps+1+len_serv+len_os);
                CD_dr(1:len_dr)= comDes(i,j,len_apps+2+len_serv+len_os:end);
                [CD_apps,CD_serv,CD_os,CD_dr] = DynAlgClass.alignDesDataC(alignData,CD_apps,CD_serv,CD_os,CD_dr,CSW1,CSW2,CSW3,CSW4,CSW5,CSW6,CSW7,CSW8,CSW9,CSW10);
                comDes_temp(i,j,2:end) = [CD_apps,CD_serv,CD_os,CD_dr];
            end
        end
        comDes=comDes_temp;
        %sumApps=sum(comDes(1,1,2:9))%test for node 1
        %Individual
        %get numChanges and call function
        for i = 1:numNodes
            for j = 1:numChangesInd
                ID_apps(1:len_apps) = indDes(i,j,2:len_apps+1);
                ID_serv(1:len_serv)= indDes(i,j,len_apps+2:len_apps+1+len_serv);
                ID_os(1:len_os)= indDes(i,j,len_apps+2+len_serv:len_apps+1+len_serv+len_os);
                ID_dr(1:len_dr)= indDes(i,j,len_apps+2+len_serv+len_os:end);
                sizeID_apps = size(ID_apps);
                sizeID_serv = size(ID_serv);
                sizeID_os = size(ID_os);
                sizeID_dr = size(ID_dr);
                [ID_apps,ID_serv,ID_os,ID_dr] = DynAlgClass.alignDesDataC(alignData,ID_apps,ID_serv,ID_os,ID_dr,CSW1,CSW2,CSW3,CSW4,CSW5,CSW6,CSW7,CSW8,CSW9,CSW10);
                indDes_temp(i,j,2:end) = [ID_apps,ID_serv,ID_os,ID_dr];
            end
        end
        indDes=indDes_temp;
        %sumApps=sum(indDes(1,1,2:9))%test for node 1
    end
    function [CD_apps,CD_serv,CD_os,CD_dr] = alignDesDataC(alignData,CD_apps,CD_serv,CD_os,CD_dr,...
            CSW1,CSW2,CSW3,CSW4,CSW5,CSW6,CSW7,CSW8,CSW9,CSW10)
    %set normalisation of data 1->normalise to 1, 100->normalise to 100
    %calcS.m divides CD and ID by 100 so assumes normalisation to 100
    normValCD = 100;
    %CD
    %normalise arrays to sum to 1 for unfiltered version
    CD_appsUFilt=normArray(CD_apps,normValCD);%shouldn't need 100, but doesn't draw line on graph correctly otherwise
    CD_servUFilt=normArray(CD_serv,normValCD);
    CD_osUFilt=normArray(CD_os,normValCD);
    CD_drUFilt=normArray(CD_dr,normValCD);
    %normalise to OS for filtered version
    [CD_appsFilt,CD_servFilt,CD_osFilt,CD_drFilt] = normToOS(normValCD, ...
        CSW1,CSW2,CSW3,CSW4,CSW5,CSW6,CSW7,CSW8,CSW9,CSW10, ...
        CD_apps,CD_serv,CD_os,CD_dr);
    %reassign
    if alignData == 1
        %normalise and align to OS
        CD_apps = CD_appsFilt; 
        CD_serv = CD_servFilt;
        CD_os = CD_osFilt;
        CD_dr = CD_drFilt;
    else
        %just normalise
        CD_apps = CD_appsUFilt; 
        CD_serv = CD_servUFilt;
        CD_os = CD_osUFilt;
        CD_dr = CD_drUFilt;
    end
    CD_sumApps = sum(CD_apps);
    end
    %******************************************************************
    % changing weights and desirabilities at a certain time step - OLD METHODS
    %******************************************************************
    %change weights and desirabilities - OLD METHOD
    function ChangeWeightsAndDesirabilities(t,ihandles,rhandles)
     %change weights of diversity - OLD METHOD
     DynAlgClass.ChangeWeightsOfDiversity(t,ihandles)   
     %change weights of IndDesirabilities on at a certain time OLD METHOD -STILL WORKS
     DynAlgClass.ChangeWeightsIndDesirabilities(t,ihandles)
     %Change community level desirabilities OLD METHOD - STILL WORKS
     DynAlgClass.ChangeCommDesirabilities(t,ihandles,rhandles)
     %Change ALL individual level desirabilities - OLD METHOD - STILL WORKS
     DynAlgClass.ChangeAllIndDesirabilities(t,ihandles,rhandles)
     %Change ONLY OWN individual level desirabilities - OLD METHOD STILL WORKS
     DynAlgClass.ChangeOwnIndDesirabilities(t,ihandles,rhandles)
     %Change efficiencies  -OLD METHOD - STILL WORKS
     DynAlgClass.ChangeEfficienceies(t,ihandles,rhandles)
    end 
     %change weights of diversity - OLD METHOD
     function ChangeWeightsOfDiversity(t,ihandles)
     if t == ihandles.wdivUpdateTime1
             tD=t %display t
             ihandles.wdiv = ihandles.wdivUpdateValue1
     elseif t== ihandles.wdivUpdateTime2
             tD=t %display t
             ihandles.wdiv = ihandles.wdivUpdateValue2   
     elseif t== ihandles.wdivUpdateTime3
             tD=t %display t
             ihandles.wdiv = ihandles.wdivUpdateValue3 
     end
     end %func
     %change weights of IndDesirabilities on at a certain time OLD METHOD -STILL WORKS
     function ChangeWeightsIndDesirabilities(t,ihandles)
     if t == ihandles.WidUpdateTime1
             tD=t %display t
             ihandles.Wid = ihandles.WidUpdateValue1
     elseif t== ihandles.WidUpdateTime2
             tD=t %display t
             ihandles.Wid = ihandles.WidUpdateValue2    
     end
     end %func
     %Change community level desirabilities OLD METHOD - STILL WORKS
     function ChangeCommDesirabilities(t,ihandles,rhandles)
     if ihandles.ChangeDesirabilitiesFlag == 1
        if (t == ihandles.tChange1) && (ihandles.tChange1Enable ==1) 
            %recalculate total score with new community level desirabilities
            rhandles.S_CD = repmat(ihandles.CD2,1,ihandles.numNodes);
            disp('change CD2')
            rhandles.S = calcSDyn(ihandles.We, rhandles.S_E, ihandles.Wcd, rhandles.S_CD, ihandles.Wid, rhandles.S_ID);
        end; 
        if (t == ihandles.tChange2) && (ihandles.tChange2Enable ==1) 
            %recalculate total score with new community level desirabilities
            rhandles.S_CD = repmat(ihandles.CD,1,ihandles.numNodes);
            disp('change CD')
            rhandles.S = calcSDyn(ihandles.We, rhandles.S_E, ihandles.Wcd, rhandles.S_CD, ihandles.Wid, rhandles.S_ID);
        end;
     end;
     end %func
     %Change ALL individual level desirabilities - OLD METHOD - STILL WORKS
     function ChangeAllIndDesirabilities(t,ihandles,rhandles)
     if ihandles.ChangeIndDesirabilitiesFlag == 1
        if (t == ihandles.tChangeInd1) && (ihandles.tChangeInd1Enable ==1) 
            %recalculate total score with new individual level desirabilities
            rhandles.S_ID = ihandles.ID2;
            disp('change ID2')
            rhandles.S = calcSDyn(ihandles.We, rhandles.S_E, ihandles.Wcd, rhandles.S_CD, ihandles.Wid, rhandles.S_ID);
            %SDisplay = S(1:5,1:10)
        end; 
        if (t == ihandles.tChangeInd2) && (ihandles.tChangeInd2Enable ==1) 
            %recalculate total score with new individual level desirabilities
            rhandles.S_ID = ihandles.ID;
            disp('change ID')
            rhandles.S = calcSDyn(ihandles.We, rhandles.S_E, ihandles.Wcd, rhandles.S_CD, ihandles.Wid, rhandles.S_ID);
        end;
     end;
     end %func
     %Change ONLY OWN individual level desirabilities - OLD METHOD STILL WORKS
     function ChangeOwnIndDesirabilities(t,ihandles,rhandles)
     if ihandles.ChangeOwnIndDesirabilitiesFlag == 1
         %change weights
         if t == ihandles.wdivUpdateTime1
             ihandles.wdiv = ihandles.wdivUpdateValue1;
             rhandles.S = calcSDyn(ihandles.We, rhandles.S_E, ihandles.Wcd, rhandles.S_CD, ihandles.Wid, rhandles.S_ID);
         elseif t == ihandles.wdivUpdateTime2
             ihandles.wdiv = ihandles.wdivUpdateValue2;
             rhandles.S = calcSDyn(ihandles.We, rhandles.S_E, ihandles.Wcd, rhandles.S_CD, ihandles.Wid, rhandles.S_ID);
         elseif t == ihandles.wdivUpdateTime3
             ihandles.wdiv = ihandles.wdivUpdateValue3;
             rhandles.S = calcSDyn(ihandles.We, rhandles.S_E, ihandles.Wcd, rhandles.S_CD, ihandles.Wid, rhandles.S_ID);
         end
         if t == ihandles.WidUpdateTime1
             ihandles.Wid = ihandles.WidUpdateValue1;
             rhandles.S = calcSDyn(ihandles.We, rhandles.S_E, ihandles.Wcd, rhandles.S_CD, ihandles.Wid, rhandles.S_ID);
         elseif t == WidUpdateTime2
             ihandles.Wid = ihandles.WidUpdateValue2;
             rhandles.S = calcSDyn(ihandles.We, rhandles.S_E, ihandles.Wcd, rhandles.S_CD, ihandles.Wid, rhandles.S_ID);
         end
        for i = 1:ihandles.numNodes  
        if (t == ihandles.IDtimeArray1(i)) 
            %recalculate total score with new individual level desirabilities
            rhandles.S_ID_apps(i,:) = ihandles.ID_apps2(i,:);
            rhandles.S_ID_os(i,:) = ihandles.ID_os2(i,:);
            rhandles.S_ID_serv(i,:) = ihandles.ID_serv2(i,:);
            rhandles.S_ID_dr(i,:) = ihandles.ID_dr2(i,:);            
            rhandles.S_ID = [rhandles.S_ID_apps rhandles.S_ID_serv rhandles.S_ID_os rhandles.S_ID_dr]'; 
            rhandles.S = calcSDyn(ihandles.We, rhandles.S_E, ihandles.Wcd, rhandles.S_CD, ihandles.Wid, rhandles.S_ID);
        end; 
        if (t == ihandles.IDtimeArray2(i)) 
            %recalculate total score with new individual level desirabilities
            rhandles.S_ID_apps(i,:) = ihandles.ID_apps(i,:);
            rhandles.S_ID_os(i,:) = ihandles.ID_os(i,:);
            rhandles.S_ID_serv(i,:) = ihandles.ID_serv(i,:);
            rhandles.S_ID_dr(i,:) = ihandles.ID_dr(i,:);            
            rhandles.S_ID = [rhandles.S_ID_apps rhandles.S_ID_serv rhandles.S_ID_os rhandles.S_ID_dr]';
            %disp('change ID')
            rhandles.S = calcSDyn(ihandles.We, rhandles.S_E, ihandles.Wcd, rhandles.S_CD, ihandles.Wid, rhandles.S_ID);
        end;
        end; %end for i loop
        %recalculate dynamic diversity of individuals
        rhandles.Nei_indDynamic(t) = variantFreq4(ihandles.textdata, rhandles.S_ID, 0);
     end; %end if flag
    end %func
    %Change efficiencies  -OLD METHOD - STILL WORKS
    function ChangeEfficienceies(t,ihandles,rhandles)
     if ihandles.ChangeEffFlag == 1
        if (t == ihandles.tChangeEff1) && (ihandles.tChangeEff1Enable ==1) 
            %recalculate total score with new individual level desirabilities
            rhandles.S_E = repmat(ihandles.E2,1,ihandles.numNodes);
            disp('change E2')
            tDisplay = t
            rhandles.S = calcSDyn(ihandles.We, rhandles.S_E, ihandles.Wcd, rhandles.S_CD, ihandles.Wid, rhandles.S_ID);
        end; 
        if (t == ihandles.tChangeEff2) && (ihandles.tChangeEff2Enable ==1) 
            %recalculate total score with new individual level desirabilities
            rhandles.S_E = repmat(ihandles.E,1,ihandles.numNodes);
            tDisplay=t
            disp('change E')
            rhandles.S = calcSDyn(ihandles.We, rhandles.S_E, ihandles.Wcd, rhandles.S_CD, ihandles.Wid, rhandles.S_ID);
        end;
     end;
    end %func
    %******************************************************************
    % changing weights and desirabilities at a certain time step - NEW METHOD
    %******************************************************************
    function ChangeWeightsAndDesirabilitiesNEW(t,ihandles,rhandles)
         %Maximise diversity - dynamic weights only
      %check if fixed or dynamic weights
     if ihandles.divWeightsE == 1    
     [rhandles.calcSE_Divw changeDivWeightValues] = DynAlgClass.changeAllWeights(t,ihandles.divWeights,rhandles.divWeightsValueOld);
     rhandles.divWeightsValueOld = changeDivWeightValues;
     changeDivWeightValuesRep = repmat(changeDivWeightValues,1,ihandles.len_sw);
     %if dynamic weights required change to this
     ihandles.wdiv = changeDivWeightValuesRep; %size (ihandles.numNodes,len_sw) %needs to be this way round as S is transposed in equation
     sizewdiv = size(ihandles.wdiv);
     end
     %*********************************************************************
     %Maximise comm - Dynamic weights
     %check if fixed or dynamic weights
     if ihandles.comWeightsE == 1         
        [rhandles.calcSE_Comw changeComWeightValues] = DynAlgClass.changeAllWeights(t,ihandles.comWeights,rhandles.comWeightsValueOld);
        rhandles.comWeightsValueOld = changeComWeightValues;
        changeComWeightValuesRep = repmat(changeComWeightValues,1,ihandles.len_sw); %make (ihandles.numNodes,1,len_sw)
        %if dynamic weights required change to this
        ihandles.Wcd = changeComWeightValuesRep'; %size (len_sw, ihandles.numNodes)%needs to be this way round as S is transposed to wdiv
        sizeWcd = size(ihandles.Wcd);
        %need to recalculate S to take affect
     end
     %*********************************************************************
     %Maximise comm - Dynamic desirabilities
     %check if fixed or dynamic desirabilities  
     if ihandles.comDesE==1
          sizecomWeights = size(ihandles.comWeights);
          sizecomDes = size(ihandles.comDes);
        [rhandles.calcSE_Comd changeComDesValues] = DynAlgClass.changeAllWeights(t,ihandles.comDes,rhandles.comDesValueOld);
        rhandles.comDesValueOld = changeComDesValues;
        %changeComDesValuesRep = repmat(changeComDesValues,1,len_sw)
        %if dynamic weights required change to this
        S_CD_temp(1:ihandles.numNodes,1:ihandles.len_sw) = changeComDesValues(1:ihandles.numNodes,1,1:ihandles.len_sw);
        rhandles.S_CD = S_CD_temp';
        %need to recalculate S to take affect
     end
     %*********************************************************************
     %Maximise Ind - Dynamic weights
     if ihandles.indWeightsE == 1
     [rhandles.calcSE_Indw changeIndWeightValues] = DynAlgClass.changeAllWeights(t,ihandles.indWeights,rhandles.indWeightsValueOld);
     rhandles.indWeightsValueOld = changeIndWeightValues;
     changeIndWeightValuesRep = repmat(ihandles.changeIndWeightValues,1,ihandles.len_sw);
     %check if fixed or dynamic weights   
         %if dynamic weights required change to this
         ihandles.Wid = changeIndWeightValuesRep';
     end     
     %*********************************************************************
     %Maximise Ind - Dynamic desirabilities
     %check if fixed or dynamic desirabilities  
     if ihandles.indDesE==1
        [rhandles.calcSE_Indd changeIndDesValues] = DynAlgClass.changeAllWeights(t,ihandles.indDes,rhandles.indDesValueOld);
        rhandles.indDesValueOld = changeIndDesValues;
        %if dynamic weights required change to this
        S_ID_temp(1:ihandles.numNodes,1:ihandles.len_sw) = changeIndDesValues(1:ihandles.numNodes,1,1:ihandles.len_sw);
        rhandles.S_ID = S_ID_temp';
        %need to recalculate S to take affect
     end
     %*********************************************************************
     %Maximise qual - Dynamic weights
     %check if fixed or dynamic weights
     if ihandles.qualWeightsE == 1
     [rhandles.calcSE_Qualw changeQualWeightValues] = DynAlgClass.changeAllWeights(t,ihandles.qualWeights,rhandles.qualWeightsValueOld);
     rhandles.qualWeightsValueOld = changeQualWeightValues;
     changeQualWeightValuesRep = repmat(changeQualWeightValues,1,ihandles.len_sw);
         %if dynamic weights required change to this
         ihandles.We = changeQualWeightValuesRep';
     end 
     %*********************************************************************
     %Maximise qual - Dynamic desirabilities
     %check if fixed or dynamic desirabilities  
     if ihandles.qualDesE==1
        [rhandles.calcSE_Quald changeQualDesValues] = DynAlgClass.changeAllWeights(t,ihandles.qualDes,rhandles.qualDesValueOld);
        rhandles.qualDesValueOld = changeQualDesValues;
        %if dynamic weights required change to this
        S_E_temp(1:ihandles.numNodes,1:ihandles.len_sw) = changeQualDesValues(1:ihandles.numNodes,1,1:ihandles.len_sw);
        rhandles.S_E = S_E_temp';
        %need to recalculate S to take affect
     end
     %*********************************************************************
     %If any dynamic settings set(except max diversity), recalculate S, second output from function that
     %defines if an S update is required.
        %recalculate S 
        calcSE = (rhandles.calcSE_Comw + rhandles.calcSE_Comd + rhandles.calcSE_Indw + rhandles.calcSE_Indd + rhandles.calcSE_Qualw + rhandles.calcSE_Quald)>0; %check if update required
        if calcSE == 1
         tUpdateSTime = t
         rhandles.S = calcSDyn(ihandles.We, rhandles.S_E, ihandles.Wcd, rhandles.S_CD, ihandles.Wid, rhandles.S_ID);    
        end
    end %func
    %***********************************************************************
    %STOP-START GENOTYPE UPDATE PROCESS FUNCTIONS
    %***********************************************************************
    %*******************************************
     %STOPPING THE UPDATE PROCESS
     %Code for solution 4 and 5 only
     %*******************************************
     function StopUpdateProcess(t,lengthCounter,updateYes,ihandles,rhandles)
         %******************
         %sol 4
         %******************
     %need to wait a length of sample size before stopping
     %ihandlessampleSizeD=ihandles.sampleSize 
     %if lengthCounter >ihandles.sampleSize 
     %start = lengthCounter-(ihandles.sampleSize)
     %else
     %    start = 1
     %end;
     %stop = lengthCounter+1
     stop=ihandles.sampleSize;
     if t < ihandles.sampleSize 
         start = ihandles.sampleSize-t;
     else
         start = 1;
     end
     %*****************************
      %record the diversity each node sees for solution 4 only
     newValue4(:,1)= nodesNei(rhandles.tallySol4, rhandles.numEncounters, ihandles.len_apps, ihandles.len_serv, ihandles.len_os, ihandles.len_dr, updateYes, rhandles.nodesDiversity4_buffer(:,end));
     rhandles.nodesDiversity4_buffer(:,1:end-1) = rhandles.nodesDiversity4_buffer(:,2:end); %shift values in buffer
     rhandles.nodesDiversity4_buffer(:,end) = newValue4(:,1); %assign buffer with new value
     
     rhandles.nodesDiversity4(:,lengthCounter) = newValue4(:,1); 
     rhandles.nodesMean4(:,lengthCounter) = mean(rhandles.nodesDiversity4_buffer(:,start:stop),2); %added 24th april 2016
     rhandles.nodesStd4(:,lengthCounter) = std(rhandles.nodesDiversity4_buffer(:,start:stop),0,2); %added 24th april 2016

     %******************************
     %record the diversity each node sees for solution 4 only
     %size(nodesDiversityOld4)
     %nodesDiversityOld4 = rhandles.nodesDiversity4(:,lengthCounter);
     %disp('StopUpdateProcess')
     %lengthCounterD=lengthCounter
     %rhandles.nodesDiversity4(:,lengthCounter+1) = nodesNei(rhandles.tallySol4, rhandles.numEncounters, ihandles.len_apps, ihandles.len_serv, ihandles.len_os, ihandles.len_dr, updateYes, nodesDiversityOld4);
     %buffer diversity values
     %bufferPointerD=bufferPointer;
     %rhandles.nodesDiversity4_buffer(:,bufferPointer) = rhandles.nodesDiversity4(:,lengthCounter+1); %added 24th april 2016
    %rhandlesnodesDiversity4D=rhandles.nodesDiversity4(:,lengthCounter+1);
     %rhandlesnodesDiversity4_bufferD=rhandles.nodesDiversity4_buffer;
     %record the running mean of the diversity each node sees for sol 4
     %record the running standard deviation each node sees for solution 4
     %(dim 1 does the columns, dim 2 does the rows)
     %rhandles.nodesMean4(:,lengthCounter+1) = mean(rhandles.nodesDiversity4(:,start:stop),2); 
     %rhandles.nodesStd4(:,lengthCounter+1) = std(rhandles.nodesDiversity4(:,start:stop),0,2);
     %rhandles.nodesMean4(:,lengthCounter+1) = mean( rhandles.nodesDiversity4_buffer(:,start:stop),2); %added 24th april 2016
     %rhandles.nodesStd4(:,lengthCounter+1) = std( rhandles.nodesDiversity4_buffer(:,start:stop),0,2); %added 24th april 2016
     %****************Single stop update solution 4 only
     if ihandles.stopUpdate4 == 1    
      if ihandles.singleStopMode == 1 ||ihandles.singleStopMode == 2
      [rhandles.updateStop4 updateStopDisplayFlag4 lastStoppedStdNew] = singleStopUpdating(ihandles.singleStopMode,t,lengthCounter,ihandles.sampleSize,rhandles.nodesStd4,ihandles.numNodes,rhandles.updateStopDisplayFlagOld4,rhandles.displayStopVal,rhandles.displayStartVal,ihandles.minstd4Tolerance,rhandles.lastStoppedStd);           
        rhandles.updateStopDisplayFlagOld4(lengthCounter,:) = updateStopDisplayFlag4(lengthCounter,:); %record value for next time round      
        rhandles.updateStopDisplay4(lengthCounter,:) = updateStopDisplayFlag4(lengthCounter,:); %record valuefor displaying on graph    
      elseif ihandles.singleStopMode == 3 ||ihandles.singleStopMode == 4
       [rhandles.updateStop4 updateStopDisplayFlag4 lastStoppedStdNew] = singleStopUpdating(ihandles.singleStopMode,t,lengthCounter,ihandles.sampleSize,rhandles.nodesStd4,ihandles.numNodes,rhandles.updateStopDisplayFlagOld4,rhandles.displayStopVal,rhandles.displayStartVal,ihandles.minstd4Tolerance,rhandles.lastStoppedStd);           
        rhandles.updateStopDisplayFlagOld4(lengthCounter,:) = updateStopDisplayFlag4(lengthCounter,:); %record value for next time round      
        rhandles.updateStopDisplay4(lengthCounter,:) = updateStopDisplayFlag4(lengthCounter,:); %record valuefor displaying on graph 
        %restart condition
        updateStop4Old = rhandles.updateStop4;
        [updateStopDisplayFlag4 rhandles.updateStop4] = restart(ihandles.singleStopMode,ihandles.stopUpdate4,ihandles.numNodes,ihandles.sampleSize,t,lengthCounter,ihandles.tChange1Enable,ihandles.tChange1,ihandles.tChange2Enable,ihandles.tChange2,rhandles.updateStopDisplayFlagOld4,updateStop4Old,rhandles.displayStopVal,rhandles.displayStartVal,...
                                                        rhandles.nodesStd4,ihandles.maxStdTolerance,rhandles.lastStoppedStd,ihandles.stdMargin,...
                                                        ihandles.ChangeDesirabilitiesFlag,ihandles.ChangeEffFlag,ihandles.ChangeIndDesirabilitiesFlag,ihandles.ChangeOwnIndDesirabilitiesFlag, ...
                                                        ihandles.tChangeEff1Enable, ihandles.tChangeEff2Enable, ihandles.tChangeEff1, ihandles.tChangeEff2, ...
                                                        ihandles.tChangeInd1Enable, ihandles.tChangeInd2Enable, ihandles.tChangeInd1,ihandles.tChangeInd2, ...
                                                        ihandles.IDtimeArray1, ihandles.IDtimeArray2,ihandles.wdivUpdateTime1,ihandles.wdivUpdateTime2,ihandles.wdivUpdateTime3,ihandles.WidUpdateTime1,ihandles.WidUpdateTime2);                                                                              
        %update flags
        rhandles.updateStopDisplayFlagOld4(lengthCounter,:) = updateStopDisplayFlag4(lengthCounter,:); %record value for next time round      
        rhandles.updateStopDisplay4(lengthCounter,:) = updateStopDisplayFlag4(lengthCounter,:); %record valuefor displaying on graph  
      elseif ihandles.singleStopMode == 5   
        [rhandles.updateStop4 updateStopDisplayFlag4 lastStoppedStdNew] = singleStopUpdating(ihandles.singleStopMode,t,lengthCounter,ihandles.sampleSize,rhandles.nodesStd4,ihandles.numNodes,rhandles.updateStopDisplayFlagOld4,rhandles.displayStopVal,rhandles.displayStartVal,ihandles.minstd4Tolerance,rhandles.lastStoppedStd);           
        rhandles.updateStopDisplayFlagOld4(lengthCounter,:) = updateStopDisplayFlag4(lengthCounter,:); %record value for next time round      
        rhandles.updateStopDisplay4(lengthCounter,:) = updateStopDisplayFlag4(lengthCounter,:); %record valuefor displaying on graph 
        %restart condition
        updateStop4Old = rhandles.updateStop4;
        [updateStopDisplayFlag4 rhandles.updateStop4] = restart(ihandles.singleStopMode,ihandles.stopUpdate4,ihandles.numNodes,ihandles.sampleSize,t,lengthCounter,ihandles.tChange1Enable,ihandles.tChange1,ihandles.tChange2Enable,ihandles.tChange2,rhandles.updateStopDisplayFlagOld4,updateStop4Old,rhandles.displayStopVal,rhandles.displayStartVal,...
                                                        rhandles.nodesStd4,ihandles.maxStdTolerance,rhandles.lastStoppedStd,ihandles.stdMargin,...
                                                        ihandles.ChangeDesirabilitiesFlag,ihandles.ChangeEffFlag,ihandles.ChangeIndDesirabilitiesFlag,ihandles.ChangeOwnIndDesirabilitiesFlag, ...
                                                        ihandles.tChangeEff1Enable, ihandles.tChangeEff2Enable, ihandles.tChangeEff1, ihandles.tChangeEff2, ...
                                                        ihandles.tChangeInd1Enable, ihandles.tChangeInd2Enable, ihandles.tChangeInd1,ihandles.tChangeInd2, ...
                                                        ihandles.IDtimeArray1, ihandles.IDtimeArray2,ihandles.wdivUpdateTime1,ihandles.wdivUpdateTime2,ihandles.wdivUpdateTime3,ihandles.WidUpdateTime1,ihandles.WidUpdateTime2);
        %update flags
        rhandles.updateStopDisplayFlagOld4(lengthCounter,:) = updateStopDisplayFlag4(lengthCounter,:); %record value for next time round      
        rhandles.updateStopDisplay4(lengthCounter,:) = updateStopDisplayFlag4(lengthCounter,:); %record valuefor displaying on graph  
      end %if
     end %if
     %****************Multiple stop update solution 4 only
     if ihandles.restartUpdate4 == 1
        [rhandles.updateStop4 updateStopDisplayFlag4 lastStoppedStdNew] = singleStopUpdating(ihandles.restartMode,t,lengthCounter,ihandles.sampleSize,rhandles.nodesStd4,ihandles.numNodes,rhandles.updateStopDisplayFlagOld4,rhandles.displayStopVal,rhandles.displayStartVal,ihandles.minstd4Tolerance,rhandles.lastStoppedStd);           
        rhandles.updateStopDisplayFlagOld4(lengthCounter,:) = updateStopDisplayFlag4(lengthCounter,:); %record value for next time round      
        rhandles.updateStopDisplay4(lengthCounter,:) = updateStopDisplayFlag4(lengthCounter,:); %record valuefor displaying on graph 
        rhandles.lastStoppedStd = lastStoppedStdNew;
        updateStop4Old = rhandles.updateStop4;
        %restart condition
        [updateStopDisplayFlag4 rhandles.updateStop4] = restart(ihandles.restartMode,ihandles.restartUpdate4,ihandles.numNodes,ihandles.sampleSize,t,lengthCounter,ihandles.tChange1Enable,ihandles.tChange1,ihandles.tChange2Enable,ihandles.tChange2,rhandles.updateStopDisplayFlagOld4,updateStop4Old,rhandles.displayStopVal,rhandles.displayStartVal,...
                                                        rhandles.nodesStd4,ihandles.maxstd4Tolerance,lastStoppedStdNew,ihandles.stdMargin,...
                                                        ihandles.ChangeDesirabilitiesFlag,ihandles.ChangeEffFlag,ihandles.ChangeIndDesirabilitiesFlag,ihandles.ChangeOwnIndDesirabilitiesFlag, ...
                                                        ihandles.tChangeEff1Enable, ihandles.tChangeEff2Enable, ihandles.tChangeEff1, ihandles.tChangeEff2, ...
                                                        ihandles.tChangeInd1Enable, ihandles.tChangeInd2Enable, ihandles.tChangeInd1,ihandles.tChangeInd2, ...
                                                        ihandles.IDtimeArray1, ihandles.IDtimeArray2,ihandles.wdivUpdateTime1,ihandles.wdivUpdateTime2,ihandles.wdivUpdateTime3,ihandles.WidUpdateTime1,ihandles.WidUpdateTime2);
        %update flags
        rhandles.updateStopDisplayFlagOld4(lengthCounter,:) = updateStopDisplayFlag4(lengthCounter,:); %record value for next time round      
        rhandles.updateStopDisplay4(lengthCounter,:) = updateStopDisplayFlag4(lengthCounter,:); %record valuefor displaying on graph
        %check last stopped std value-THIS CODE IS FOR TESTING PURPOSES ONLY FOR MODE6******
        rhandles.lastStoppedStdMatrix(lengthCounter,:) = lastStoppedStdNew;
        if t > 1
            for i = 1: ihandles.numNodes
                 if rhandles.updateStopDisplay4(lengthCounter-1,i)==rhandles.displayStartVal && rhandles.updateStopDisplay4(lengthCounter,i) == rhandles.displayStopVal
                    rhandles.lastStoppedStdMatrix2(lengthCounter,i) = lastStoppedStdNew(i);
                 else
                    rhandles.lastStoppedStdMatrix2(lengthCounter,i) = rhandles.lastStoppedStdMatrix2(lengthCounter-1,i);
                 end%end if
            end %end i loop
        end%end t
        %**********************************************
     end
     %*******************************************************
     %****************Multiple stop update solution 4 only     
     if ihandles.restartUpdateTest == 1
     %record the diversity each node sees for solution 4 only
     for i = 1: ihandles.numNodes
     % check to see if surpassed sample size
     if (t >ihandles.sampleSize)
      %***minimum update
      if rhandles.reachTolFlag4(i) == 0 && (rhandles.nodesStd4(i,lengthCounter+1) <= rhandles.minStd4(i)) && (rhandles.nodesStd4(i,lengthCounter+1) < rhandles.maxStd4(i))  
          rhandles.minStd4(i) = rhandles.nodesStd4(i,lengthCounter+1); %set new minimum
          rhandles.minStd4Curve(i,lengthCounter) = rhandles.nodesStd4(i,lengthCounter+1);
          rhandles.reachTolFlag4(i) = 1; %surpassed tolerance
      elseif rhandles.reachTolFlag4(i) == 1 && (rhandles.nodesStd4(i,lengthCounter+1) <= rhandles.minStd4(i)) && (rhandles.nodesStd4(i,lengthCounter+1) < rhandles.maxStd4(i))  
          rhandles.minStd4(i) = rhandles.nodesStd4(i,lengthCounter+1); %set new minimum
           rhandles.minStd4Curve(i,lengthCounter) = rhandles.nodesStd4(i,lengthCounter+1);
          rhandles.updateStop4(i) = 1; %continue updating
          rhandles.updateStopDisplay4(lengthCounter,i)= 1; %continue updating
      elseif rhandles.reachTolFlag4(i) == 1 && (rhandles.nodesStd4(i,lengthCounter+1) > rhandles.minStd4(i)) && (rhandles.nodesStd4(i,lengthCounter+1) < rhandles.maxStd4(i)) 
         rhandles.updateStop4(i) = 0; %stop updating
         rhandles.updateStopDisplay4(lengthCounter,i)= 0; %stop updating
         rhandles.minStd4Curve(i,lengthCounter) = rhandles.minStd4Curve(i,lengthCounter-1);
      %***maximum update - update again and reset if std goes back up again
      elseif rhandles.reachTolFlag4(i) == 1 && (rhandles.nodesStd4(i,lengthCounter+1) > rhandles.minStd4(i)) && (rhandles.nodesStd4(i,lengthCounter+1) > rhandles.maxStd4(i))  
          rhandles.updateStop4(i) = 1; %start updating again
          rhandles.updateStopDisplay4(lengthCounter,i)= 1; %start updating again
         %reset min tolerance and tolFlag
          rhandles.minStd4(i) = ihandles.minstd4Tolerance;
           rhandles.minStd4Curve(i,lengthCounter) = ihandles.minstd4Tolerance;
          rhandles.reachTolFlag4(i) = 0;
      else
          rhandles.minStd4Curve(i,lengthCounter) = rhandles.minStd4Curve(i,lengthCounter-1);
      end; %end stop update logic
     else
         rhandles.minStd4Curve(i,lengthCounter) = ihandles.minstd4Tolerance;
     end; %end sample size
     end; %end i loop
     end; % end restartUpdate4
     %***********************************************
     %Sol 5
     %******************************
       stop=ihandles.sampleSize;
     if t < ihandles.sampleSize 
         start = ihandles.sampleSize-t;
     else
         start = 1;
     end
     %*****************************
      %record the diversity each node sees for solution 4 only
     newValue5(:,1)= nodesNei(rhandles.tallySol5, rhandles.numEncounters, ihandles.len_apps, ihandles.len_serv, ihandles.len_os, ihandles.len_dr, updateYes, rhandles.nodesDiversity5_buffer(:,end));
     rhandles.nodesDiversity5_buffer(:,1:end-1) = rhandles.nodesDiversity5_buffer(:,2:end); %shift values in buffer
     rhandles.nodesDiversity5_buffer(:,end) = newValue5(:,1); %assign buffer with new value
     
     rhandles.nodesDiversity5(:,lengthCounter) = newValue5(:,1); 
     rhandles.nodesMean5(:,lengthCounter) = mean(rhandles.nodesDiversity5_buffer(:,start:stop),2); %added 24th april 2016
     rhandles.nodesStd5(:,lengthCounter) = std(rhandles.nodesDiversity5_buffer(:,start:stop),0,2); %added 24th april 2016

     %if lengthCounter >ihandles.sampleSize 
     %start = lengthCounter-(ihandles.sampleSize);
     %else
     %    start = 1;
     %end;
     %stop = lengthCounter+1;
     %nodesDiversityOld5 = rhandles.nodesDiversity5(:,lengthCounter);
     %rhandles.nodesDiversity5(:,lengthCounter+1) = nodesNei(rhandles.tallySol5, rhandles.numEncounters, ihandles.len_apps, ihandles.len_serv, ihandles.len_os, ihandles.len_dr, updateYes, nodesDiversityOld5);
     %record the running mean of the diversity each node sees for sol 5
     %record the running standard deviation each node sees for solution 5
     %(dim 1 does the columns, dim 2 does the rows)
     %rhandles.nodesMean5(:,lengthCounter+1) = mean(rhandles.nodesDiversity5(:,start:stop),2); 
     %rhandles.nodesStd5(:,lengthCounter+1) = std(rhandles.nodesDiversity5(:,start:stop),0,2);
     %****************Single stop update solution 5 only
     if ihandles.stopUpdate5 == 1                 
      [rhandles.updateStop5 updateStopDisplayFlag5 lastStoppedStdNew] = singleStopUpdating(ihandles.singleStopMode,t,lengthCounter,ihandles.sampleSize,rhandles.nodesStd5,ihandles.numNodes,rhandles.updateStopDisplayFlagOld5,rhandles.displayStopVal,rhandles.displayStartVal,ihandles.minstd5Tolerance,rhandles.lastStoppedStd5);           
        rhandles.updateStopDisplayFlagOld5(lengthCounter,:) = updateStopDisplayFlag5(lengthCounter,:); %record value for next time round      
        rhandles.updateStopDisplay5(lengthCounter,:) = updateStopDisplayFlag5(lengthCounter,:); %record valuefor displaying on graph        
     end
     %***********************************************
     %****************Multiple stop update solution 5 only
     if ihandles.restartUpdate5 == 1
     %record the diversity each node sees for solution 5 only
     %nodesDiversityOld5 = ihandles.nodesDiversity5(:,t);
     %ihandles.nodesDiversity5(:,t+1) = nodesNei(tallySol5, rhandles.numEncounters, len_apps, len_serv, len_os, len_dr, updateYes, nodesDiversityOld5);
     %record the running standard deviation each node sees for solution 4
     %nodesMean5(:,t+1) = mean(ihandles.nodesDiversity5(:,start:stop),2); 
     %nodesStd5(:,t+1) = std(ihandles.nodesDiversity5(:,start:stop),0,2);
     for i = 1: ihandles.numNodes
     % check to see if surpassed sample size
     if (t >ihandles.sampleSize)
      %***minimum update
      if rhandles.reachTolFlag5(i) == 0 && (rhandles.nodesStd5(i,lengthCounter+1) <= rhandles.minStd5(i)) && (rhandles.nodesStd5(i,lengthCounter+1) < rhandles.maxStd5(i))  
          rhandles.minStd5(i) = rhandles.nodesStd5(i,lengthCounter+1); %set new minimum
          rhandles.minStd5Curve(i,lengthCounter) = rhandles.nodesStd5(i,lengthCounter+1);
          rhandles.reachTolFlag5(i) = 1; %surpassed tolerance
      elseif rhandles.reachTolFlag5(i) == 1 && (rhandles.nodesStd5(i,lengthCounter+1) <= rhandles.minStd5(i)) && (rhandles.nodesStd5(i,lengthCounter+1) < rhandles.maxStd5(i))  
          rhandles.minStd5(i) = rhandles.nodesStd5(i,lengthCounter+1); %set new minimum
           rhandles.minStd5Curve(i,lengthCounter) = rhandles.nodesStd5(i,lengthCounter+1);
          rhandles.updateStop5(i) = 1; %continue updating
          rhandles.updateStopDisplay5(lengthCounter,i)= 1; %continue updating
      elseif rhandles.reachTolFlag5(i) == 1 && (rhandles.nodesStd5(i,lengthCounter+1) > rhandles.minStd5(i)) && (rhandles.nodesStd5(i,lengthCounter+1) < rhandles.maxStd5(i)) 
         rhandles.updateStop5(i) = 0; %stop updating
         rhandles.updateStopDisplay5(lengthCounter,i)= 0; %stop updating
         rhandles.minStd5Curve(i,lengthCounter) = rhandles.minStd5Curve(i,lengthCounter-1);
      %***maximum update - update again and reset if std goes back up again
      elseif rhandles.reachTolFlag5(i) == 1 && (rhandles.nodesStd5(i,lengthCounter+1) > rhandles.minStd5(i)) && (rhandles.nodesStd5(i,lengthCounter+1) > rhandles.maxStd5(i))  
          rhandles.updateStop5(i) = 1; %start updating again
          rhandles.updateStopDisplay5(lengthCounter,i)= 1; %start updating again
          %reset min tolerance and tolFlag
          rhandles.minStd5(i) = ihandles.minstd5Tolerance;
           rhandles.minStd5Curve(i,lengthCounter) = ihandles.minstd5Tolerance;
          rhandles.reachTolFlag5(i) = 0;
      else
          rhandles.minStd5Curve(i,lengthCounter) = rhandles.minStd5Curve(i,lengthCounter-1);
      end; %end stop update logic
     else
         rhandles.minStd5Curve(i,lengthCounter) = ihandles.minstd5Tolerance;
     end; %end sample size
     end; %end i loop
     end; % end restartUpdate4
     end; %func
     %***********************************************
     % Functions for updating genotypes
     %**********************************************
     %**********************************************************************
     % Update Genotypes for all solutions
     %**********************************************************************
     %solution 1
     function updateGenotypesAllSols(t,updateYes,ihandles,rhandles)
     if ihandles.turnOnSol1 == 1
     solNum=1;
     [rhandles.Genotype_1New rhandles.Genotype_apps1New rhandles.Genotype_serv1New rhandles.Genotype_os1New rhandles.Genotype_dr1New] = ...
                    updateGenotype(t,rhandles.tallySol1, solNum, ihandles.tallyLength, updateYes, rhandles.updateStop1, ihandles.numNodes, rhandles.numEncounters, ihandles.tolerance, ...
                    ihandles.We, ihandles.Wcd, ihandles.Wid, ihandles.wdiv, rhandles.S, ihandles.CSWGrouprep, ihandles.numOS, ihandles.CDSize, ...
                    ihandles.CSW1, ihandles.CSW2, ihandles.CSW3, ihandles.CSW4, ihandles.CSW5, ihandles.CSW6, ihandles.CSW7, ihandles.CSW8, ihandles.CSW9, ihandles.CSW10, ... 
                    ihandles.len_apps, ihandles.len_serv, ihandles.len_os, ihandles.len_dr, ...
                    ihandles.SizeCD_apps, ihandles.SizeCD_serv, ihandles.SizeCD_os, ihandles.SizeCD_dr, ...
                    rhandles.GenotypeSol1_apps, rhandles.GenotypeSol1_serv, rhandles.GenotypeSol1_os, rhandles.GenotypeSol1_dr);
     end; 
     %solution 2
     if ihandles.turnOnSol2 == 1
     solNum=2;
     [rhandles.Genotype_2New rhandles.Genotype_apps2New rhandles.Genotype_serv2New rhandles.Genotype_os2New rhandles.Genotype_dr2New] = ...
                    updateGenotype(t,rhandles.tallySol2, solNum, ihandles.tallyLength, updateYes, rhandles.updateStop2, ihandles.numNodes, rhandles.numEncounters, ihandles.tolerance, ...
                    ihandles.We, ihandles.Wcd, ihandles.Wid, ihandles.wdiv, rhandles.S, ihandles.CSWGrouprep, ihandles.numOS, ihandles.CDSize, ...
                    ihandles.CSW1, ihandles.CSW2, ihandles.CSW3, ihandles.CSW4, ihandles.CSW5, ihandles.CSW6, ihandles.CSW7, ihandles.CSW8, ihandles.CSW9, ihandles.CSW10, ... 
                    ihandles.len_apps, ihandles.len_serv, ihandles.len_os, ihandles.len_dr, ...
                    ihandles.SizeCD_apps, ihandles.SizeCD_serv, ihandles.SizeCD_os, ihandles.SizeCD_dr, ...
                    rhandles.GenotypeSol2_apps, rhandles.GenotypeSol2_serv, rhandles.GenotypeSol2_os, rhandles.GenotypeSol2_dr);
     end;           
     %solution 3
     if ihandles.turnOnSol3 == 1
     solNum=3;
     [rhandles.Genotype_3New rhandles.Genotype_apps3New rhandles.Genotype_serv3New rhandles.Genotype_os3New rhandles.Genotype_dr3New] = ...
                    updateGenotype(t,rhandles.tallySol3, solNum, ihandles.tallyLength, updateYes, rhandles.updateStop3, ihandles.numNodes, rhandles.numEncounters, ihandles.tolerance, ...
                    ihandles.We, ihandles.Wcd, ihandles.Wid, ihandles.wdiv, rhandles.S, ihandles.CSWGrouprep, ihandles.numOS, ihandles.CDSize, ...
                    ihandles.CSW1, ihandles.CSW2, ihandles.CSW3, ihandles.CSW4, ihandles.CSW5, ihandles.CSW6, ihandles.CSW7, ihandles.CSW8, ihandles.CSW9, ihandles.CSW10, ... 
                    ihandles.len_apps, ihandles.len_serv, ihandles.len_os, ihandles.len_dr, ...
                    ihandles.SizeCD_apps, ihandles.SizeCD_serv, ihandles.SizeCD_os, ihandles.SizeCD_dr, ...
                    rhandles.GenotypeSol3_apps, rhandles.GenotypeSol3_serv, rhandles.GenotypeSol3_os, rhandles.GenotypeSol3_dr);
     end; 
     %solution 4
     if ihandles.turnOnSol4 == 1
     solNum=6;
     [rhandles.Genotype_4New rhandles.Genotype_apps4New rhandles.Genotype_serv4New rhandles.Genotype_os4New rhandles.Genotype_dr4New] = ...
                    updateGenotype(t,rhandles.tallySol4, solNum, ihandles.tallyLength, updateYes, rhandles.updateStop4, ihandles.numNodes, rhandles.numEncounters, ihandles.tolerance, ...
                    ihandles.We, ihandles.Wcd, ihandles.Wid, ihandles.wdiv, rhandles.S, ihandles.CSWGrouprep, ihandles.numOS, ihandles.CDSize, ...
                    ihandles.CSW1, ihandles.CSW2, ihandles.CSW3, ihandles.CSW4, ihandles.CSW5, ihandles.CSW6, ihandles.CSW7, ihandles.CSW8, ihandles.CSW9, ihandles.CSW10, ... 
                    ihandles.len_apps, ihandles.len_serv, ihandles.len_os, ihandles.len_dr, ...
                    ihandles.SizeCD_apps, ihandles.SizeCD_serv, ihandles.SizeCD_os, ihandles.SizeCD_dr, ...
                    rhandles.GenotypeSol4_apps, rhandles.GenotypeSol4_serv, rhandles.GenotypeSol4_os, rhandles.GenotypeSol4_dr);
 
     end;
     %solution 5
     if ihandles.turnOnSol5 == 1
     solNum=7;
     [rhandles.Genotype_5New rhandles.Genotype_apps5New rhandles.Genotype_serv5New rhandles.Genotype_os5New rhandles.Genotype_dr5New] = ...
                    updateGenotype(t,rhandles.tallySol5, solNum, ihandles.tallyLength, updateYes, rhandles.updateStop5, ihandles.numNodes, rhandles.numEncounters, ihandles.tolerance, ...
                    ihandles.We, ihandles.Wcd, ihandles.Wid, ihandles.wdiv, rhandles.S, ihandles.CSWGrouprep, ihandles.numOS, ihandles.CDSize, ...
                    ihandles.CSW1, ihandles.CSW2, ihandles.CSW3, ihandles.CSW4, ihandles.CSW5, ihandles.CSW6, ihandles.CSW7, ihandles.CSW8, ihandles.CSW9, ihandles.CSW10, ... 
                    ihandles.len_apps, ihandles.len_serv, ihandles.len_os, ihandles.len_dr, ...
                    ihandles.SizeCD_apps, ihandles.SizeCD_serv, ihandles.SizeCD_os, ihandles.SizeCD_dr, ...
                    rhandles.GenotypeSol5_apps, rhandles.GenotypeSol5_serv, rhandles.GenotypeSol5_os, rhandles.GenotypeSol5_dr); 
     %Genotype_apps5New
     end;
    end; %func
    %******************************************************************
    % functions for assigning outputs
    %******************************************************************
     %******************************************************************
     % reasign new output for all solutions
     %********************************************************************
     function reasignNewOutputs(t,psim,ihandles,rhandles,ohandles)
     %solution 1
     if ihandles.turnOnSol1 == 1
     rhandles.GenotypeSol1 = rhandles.Genotype_1New;
     rhandles.GenotypeSol1_apps = rhandles.Genotype_apps1New;
     rhandles.GenotypeSol1_serv = rhandles.Genotype_serv1New;
     rhandles.GenotypeSol1_os = rhandles.Genotype_os1New;
     rhandles.GenotypeSol1_dr = rhandles.Genotype_dr1New;
     %ohandles.GenoSim_sol1(psim,t,:,:) = rhandles.Genotype_1New; %added 31st March 2016
     ohandles.Nei_sol1(psim,t) = variantFreq4(ihandles.textdata, rhandles.Genotype_1New', 0); %diversity index
     end;
     %solution 2
     if ihandles.turnOnSol2 == 1
     rhandles.GenotypeSol2 = rhandles.Genotype_2New;
     rhandles.GenotypeSol2_apps = rhandles.Genotype_apps2New;
     rhandles.GenotypeSol2_serv = rhandles.Genotype_serv2New;
     rhandles.GenotypeSol2_os = rhandles.Genotype_os2New;
     rhandles.GenotypeSol2_dr = rhandles.Genotype_dr2New;
     %ohandles.GenoSim_sol2(psim,t,:,:) = rhandles.Genotype_2New; %added 31st March 2016
     ohandles.Nei_sol2(psim,t) = variantFreq4(ihandles.textdata, rhandles.Genotype_2New', 0);%diversity index
     end;
     %solution 3
     if ihandles.turnOnSol3 == 1
     rhandles.GenotypeSol3 = rhandles.Genotype_3New;
     rhandles.GenotypeSol3_apps = rhandles.Genotype_apps3New;
     rhandles.GenotypeSol3_serv = rhandles.Genotype_serv3New;
     rhandles.GenotypeSol3_os = rhandles.Genotype_os3New;
     rhandles.GenotypeSol3_dr = rhandles.Genotype_dr3New;
     %ohandles.GenoSim_sol3(psim,t,:,:) = rhandles.Genotype_3New; %added 31st March 2016
     ohandles.Nei_sol3(psim,t) = variantFreq4(ihandles.textdata, rhandles.Genotype_3New', 0);%diversity index
     end;
     %solution 4
     if ihandles.turnOnSol4 == 1
     rhandles.GenotypeSol4 = rhandles.Genotype_4New;
     rhandles.GenotypeSol4_apps = rhandles.Genotype_apps4New;
     rhandles.GenotypeSol4_serv = rhandles.Genotype_serv4New;
     rhandles.GenotypeSol4_os = rhandles.Genotype_os4New;
     rhandles.GenotypeSol4_dr = rhandles.Genotype_dr4New;
     ohandles.GenoSim_sol4(psim,t,:,:) = rhandles.Genotype_4New; %added 31st March 2016
     ohandles.Nei_sol4(psim,t) = variantFreq4(ihandles.textdata, rhandles.Genotype_4New', 0);%diversity index
     end;
     %solution 5
     if ihandles.turnOnSol5 == 1
     rhandles.GenotypeSol5 = rhandles.Genotype_5New;
     rhandles.GenotypeSol5_apps = rhandles.Genotype_apps5New;
     rhandles.GenotypeSol5_serv = rhandles.Genotype_serv5New;
     rhandles.GenotypeSol5_os = rhandles.Genotype_os5New;
     rhandles.GenotypeSol5_dr = rhandles.Genotype_dr5New;
     ohandles.GenoSim_sol5(psim,t,:,:) = rhandles.Genotype_5New; %added 31st March 2016
     ohandles.Nei_sol5(psim,t) = variantFreq4(ihandles.textdata, rhandles.Genotype_5New', 0);%diversity index
     end;
    end; %func

    %****************************************************************
    end %end methods (static)
end %end class def
