%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Jennifer Jackson 2017                                                        %
% A Biodiversity Approach to Cyber Security                                    %
% PhD Complexity Science                                                       %
% University of Warwick                                                        %
% http://wrap.warwick.ac.uk/98458                                              %
% This software is distributed under the  Creative Commons Attribution License %
% https://creativecommons.org/licenses/by/4.0/                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%function outputs = DynAlgStochastic(handles,currentSimNumber,networkModelEnable,probOfEncounter)
function outputs = DynAlgStochastic(handles,currentSimNumber)
%Stochastic update in time version (run time parameters)
outputs = 0
singleStopMode = 4
restartMode = 5
maxStdTolerance = 0.1 %for single stop mode only
stdMargin = 0.01 %for mode 6 multiple restart
ChangeOwnIndDesirabilitiesFlag = 0 %to enable
%need to have an enable flag for wdiv - TO DO
%wdivUpdateTime1=1
%wdivUpdateValue1=1
%wdivUpdateTime2=4000
%wdivUpdateValue2=0.3
%wdivUpdateTime3=5000
%wdivUpdateValue3=0
wdivUpdateTime1=0;
wdivUpdateValue1=0;
wdivUpdateTime2=0;
wdivUpdateValue2=0;
wdivUpdateTime3=0;
wdivUpdateValue3=0;

%flag for ownInd to turn on and off
%WidUpdateTime1=1
%WidUpdateValue1=0
%WidUpdateTime2=1000
%WidUpdateValue2=1

WidUpdateTime1=0;
WidUpdateValue1=0;
WidUpdateTime2=0;
WidUpdateValue2=0;


%%%%%%%%%%%%%%%%%%%%
%VARIABLE PARAMETERS
%%%%%%%%%%%%%%%%%%%%
%get current settings (static parameters)
[turnOnSol1 turnOnSol2 turnOnSol3 turnOnSol4 turnOnSol5] = DynAlgClass.getAlgChoice(handles);
[numSims numRuns] = DynAlgClass.getSimInfo(handles);
[minEncounters maxEncounters distStr] = DynAlgClass.getEncounterInfo(handles);
[minstd4Tolerance maxstd4Tolerance minstd5Tolerance maxstd5Tolerance sampleSize ...
                   restartUpdateTest stopUpdate4 restartUpdate4 stopUpdate5 restartUpdate5] = DynAlgClass.getStopStartInfo(handles, maxEncounters);
[ChangeDesirabilitiesFlag tChange1Enable tChange1 tChange2Enable tChange2] = DynAlgClass.getComDesFlags(handles);
[ChangeIndDesirabilitiesFlag tChangeInd1Enable tChangeInd1 tChangeInd2Enable tChangeInd2] = DynAlgClass.getIndDesFlags(handles);
[ChangeEffFlag tChangeEff1Enable tChangeEff1 tChangeEff2Enable tChangeEff2] = DynAlgClass.getEffDesFlags(handles);
[We Wcd Wid wdiv] = DynAlgClass.getWeights(handles);

%[dataSet alignData] = DynAlgClass.getDatasetInfo(handles);

%tolerance for choosing random maximum out of a possible many choices for solution 1 and 2
tolerance = 0.2;

%%%%%%%%%%
%TEST DATA
%%%%%%%%%%
[dataSet alignData] = DataClass.getDatasetInfo(handles);
[SW_str_apps SW_str_serv SW_str_os SW_str_dr ...
 SW_freq_apps SW_freq_serv SW_freq_os SW_freq_dr ...
 CSW1 CSW2 CSW3 CSW4 CSW5 CSW6 CSW7 CSW8 CSW9 CSW10 CSWGroup ...
 Genotype_apps Genotype_serv Genotype_os Genotype_dr ...
 E_apps E_serv E_os E_dr ...
 E_apps2 E_serv2 E_os2 E_dr2 ...
 CD_apps CD_serv CD_os CD_dr ...
 CD_apps2 CD_serv2 CD_os2 CD_dr2 ...
 ID_apps ID_serv ID_os ID_dr ...
 ID_apps2 ID_serv2 ID_os2 ID_dr2 ...
 IDtimeArray1 IDtimeArray2 ...
 ] = getData(dataSet,alignData);
%concat data
textdata = [SW_freq_apps; SW_freq_serv; SW_freq_os; SW_freq_dr];
Genotype = [Genotype_apps Genotype_serv Genotype_os Genotype_dr];
E = [E_apps E_serv E_os E_dr]'; %efficiency ratings
E2 = [E_apps2 E_serv2 E_os2 E_dr2]'; %efficiency ratings
CD = [CD_apps CD_serv CD_os CD_dr]'; %community desirability ratings
CD2 = [CD_apps2 CD_serv2 CD_os2 CD_dr2]'; %community desirability ratings2
ID = [ID_apps ID_serv ID_os ID_dr]'; %individual desirability ratings
ID2 = [ID_apps2 ID_serv2 ID_os2 ID_dr2]'; %individual desirability ratings
%get size of data
[numNodes len_apps len_serv len_os len_dr] = DataClass.getGenoSize(Genotype_apps,Genotype_serv,Genotype_os,Genotype_dr);
len_sw = len_apps+len_serv+len_os+len_dr;
[SizeCD_apps SizeCD_serv SizeCD_os SizeCD_dr tallyLength numOS CDSize] = DataClass.getCDSize(CD_apps,CD_serv,CD_os,CD_dr,CD,CSWGroup);
%generate dynamic weights
[divWeights divWeightsE comWeights comWeightsE indWeights indWeightsE qualWeights qualWeightsE] = DataClass.getDynamicWeights(numNodes,handles); 
%generate dynamic desirabilities
[comDes comDesE indDes indDesE qualDes qualDesE] = DataClass.getDynamicDes(numNodes,handles);
%align dynamic desirability data if required
[comDes,indDes,qualDes] = DataClass.alignDesDataAll(numNodes,handles,comDes,indDes,qualDes,len_apps,len_serv,len_os,len_dr,...
            CSW1,CSW2,CSW3,CSW4,CSW5,CSW6,CSW7,CSW8,CSW9,CSW10);
        
%NOT INCLUDED IN DYNALG CLASS YET        
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%INITIALISATION OF MATRICES AND VARIABLES FOR A SIMULATION
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[probabilityDistr randomNumber Nei_sol1 Nei_sol2 Nei_sol3 Nei_sol4 Nei_sol5 CSWGrouprep] = DynAlgClass.initSimVariables(numNodes,numSims,numRuns,CDSize,numOS,CSWGroup);

%*************************
%INCLUDE WIRELESS MODEL
%*************************
%if networkModelEnable ==1
%    probabilityDistr(:) = probOfEncounter; %taken from wireless bluetooth model 
%end

%set up type of encounters & number of encounters before an update
%plotData=0;
%numEncounters = stochasticEncounters(distStr,numNodes,minEncounters,maxEncounters,plotData);

%%%%%%%%%%%
%MAIN CODE
%%%%%%%%%%%
%reset number encountered
 for psim = 1: numSims %numSims here actually means number of individual runs
    
 %set up type of encounters & number of encounters before an update
plotData=0;
numEncounters = stochasticEncounters(distStr,numNodes,minEncounters,maxEncounters,plotData);    
     
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %INITIALISATION OF MATRICES AND VARIABLES FOR A RUN
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   
 %Total score - can be fixed for a given simulation
 S = calcS(We, E, Wcd, CD, Wid, ID);
 %for calcS changing during simulation
[S_CD S_ID S_ID_apps S_ID_os S_ID_serv S_ID_dr S_E] = DynAlgClass.initRunVariablesCalcS(CD,ID,ID_apps,ID_os,ID_serv,ID_dr,E);

 %recording individual diversity dynamically over time
 Nei_indDynamic = zeros(1,numRuns);
 counterEncounters = zeros(numNodes,1); 
 updateYes = zeros(numNodes,1); %this is used

%bespoke solutions 1 to 3 - no longer used
[updateStop1 updateStop2 updateStop3] = DynAlgClass.initRunVariablesBespokeSols(numNodes);
[GenotypeSol1 GenotypeSol1_apps GenotypeSol1_serv GenotypeSol1_os GenotypeSol1_dr] = DynAlgClass.initRunVariablesGenotype(Genotype,Genotype_apps,Genotype_serv,Genotype_os,Genotype_dr); 
[GenotypeSol2 GenotypeSol2_apps GenotypeSol2_serv GenotypeSol2_os GenotypeSol2_dr] = DynAlgClass.initRunVariablesGenotype(Genotype,Genotype_apps,Genotype_serv,Genotype_os,Genotype_dr); 
[GenotypeSol3 GenotypeSol3_apps GenotypeSol3_serv GenotypeSol3_os GenotypeSol3_dr] = DynAlgClass.initRunVariablesGenotype(Genotype,Genotype_apps,Genotype_serv,Genotype_os,Genotype_dr); 

%solution 4
[updateStop4 reachTolFlag4 minStd4 maxStd4 minStd4Curve nodesDiversity4 updateStopDisplay4 ...
    nodesMean4 nodesMean4Filt nodesStd4 nodesStd4Filt displayStopVal displayStartVal updateStopDisplayFlagOld4 ...
    lastStoppedStd lastStoppedStdMatrix lastStoppedStdMatrix2] = DynAlgClass.initRunVariablesSol4(numNodes,minstd4Tolerance,maxstd4Tolerance,numRuns);
[GenotypeSol4 GenotypeSol4_apps GenotypeSol4_serv GenotypeSol4_os GenotypeSol4_dr] = DynAlgClass.initRunVariablesGenotype(Genotype,Genotype_apps,Genotype_serv,Genotype_os,Genotype_dr);    

%solution 5
[updateStop5 reachTolFlag5 minStd5 maxStd5 minStd5Curve nodesDiversity5 updateStopDisplay5 ...
    nodesMean5 nodesStd5 displayStopVal5 displayStartVal5 updateStopDisplayFlagOld5 lastStoppedStd5] = DynAlgClass.initRunVariablesSol5(numNodes,minstd5Tolerance,maxstd5Tolerance,numRuns);
[GenotypeSol5 GenotypeSol5_apps GenotypeSol5_serv GenotypeSol5_os GenotypeSol5_dr] = DynAlgClass.initRunVariablesGenotype(Genotype,Genotype_apps,Genotype_serv,Genotype_os,Genotype_dr); 

%for counting software
[tallyUpdates tallySol1 tallySol2 tallySol3 tallySol4 tallySol5] = DynAlgClass.initRunVariablesTally(numNodes,tallyLength);
%initialise dynamic variables for dynamic weights and dynamic desirabilities
[divWeightsValueOld,comWeightsValueOld,indWeightsValueOld,qualWeightsValueOld,...
                    calcSE_Comw,calcSE_Indw,calcSE_Qualw,comDesValueOld,indDesValueOld,qualDesValueOld,...
                    calcSE_Comd,calcSE_Indd,calcSE_Quald,We,Wcd,Wid,S_CD,S_E] = DynAlgClass.initRunVariablesDynamic(numNodes,len_sw,We,Wcd,Wid,S_CD,S_E);


%*********************
 %numRuns in this case is number of time steps    
 for t = 1:numRuns
     %choose a uniform random number to decide whether to meet another node
     randomNumber = rand(numNodes,1);
     %reset array that indicates there  has been an encounter
     encounterYes = zeros(numNodes,1);
     %for measuring timing of how long a sim has left to run
     simTimeStatus(t, numRuns)
     %*********************************************************************
     %change weights of diversity - OLD METHOD
     if t == wdivUpdateTime1
             tD=t %display t
             wdiv = wdivUpdateValue1
     elseif t== wdivUpdateTime2
             tD=t %display t
             wdiv = wdivUpdateValue2   
     elseif t== wdivUpdateTime3
             tD=t %display t
             wdiv = wdivUpdateValue3 
     end
     %*********************************************************************
     %Change dynamic weights and desirabilities - NEW METHOD
     %*********************************************************************
     %Maximise diversity - dynamic weights only
      %check if fixed or dynamic weights
     if divWeightsE == 1    
     [calcSE_Divw changeDivWeightValues] = DynAlgClass.changeAllWeights(t,divWeights,divWeightsValueOld);
     divWeightsValueOld = changeDivWeightValues;
     changeDivWeightValuesRep = repmat(changeDivWeightValues,1,len_sw);
     %if dynamic weights required change to this
     wdiv = changeDivWeightValuesRep; %size (numNodes,len_sw) %needs to be this way round as S is transposed in equation
     sizewdiv = size(wdiv);
     end
     %*********************************************************************
     %Maximise comm - Dynamic weights
     %check if fixed or dynamic weights
     if comWeightsE == 1         
        [calcSE_Comw changeComWeightValues] = DynAlgClass.changeAllWeights(t,comWeights,comWeightsValueOld);
        comWeightsValueOld = changeComWeightValues;
        changeComWeightValuesRep = repmat(changeComWeightValues,1,len_sw); %make (numNodes,1,len_sw)
        %if dynamic weights required change to this
        Wcd = changeComWeightValuesRep'; %size (len_sw, numNodes)%needs to be this way round as S is transposed to wdiv
        sizeWcd = size(Wcd);
        %need to recalculate S to take affect
     end
     %*********************************************************************
     %Maximise comm - Dynamic desirabilities
     %check if fixed or dynamic desirabilities  
     if comDesE==1
          sizecomWeights = size(comWeights);
          sizecomDes = size(comDes);
        [calcSE_Comd changeComDesValues] = DynAlgClass.changeAllWeights(t,comDes,comDesValueOld);
        comDesValueOld = changeComDesValues;
        %changeComDesValuesRep = repmat(changeComDesValues,1,len_sw)
        %if dynamic weights required change to this
        S_CD_temp(1:numNodes,1:len_sw) = changeComDesValues(1:numNodes,1,1:len_sw);
        S_CD = S_CD_temp';
        %need to recalculate S to take affect
     end
     %*********************************************************************
     %Maximise Ind - Dynamic weights
     if indWeightsE == 1
     [calcSE_Indw changeIndWeightValues] = DynAlgClass.changeAllWeights(t,indWeights,indWeightsValueOld);
     indWeightsValueOld = changeIndWeightValues;
     changeIndWeightValuesRep = repmat(changeIndWeightValues,1,len_sw);
     %check if fixed or dynamic weights   
         %if dynamic weights required change to this
         Wid = changeIndWeightValuesRep';
     end     
     %*********************************************************************
     %Maximise Ind - Dynamic desirabilities
     %check if fixed or dynamic desirabilities  
     if indDesE==1
        [calcSE_Indd changeIndDesValues] = DynAlgClass.changeAllWeights(t,indDes,indDesValueOld);
        indDesValueOld = changeIndDesValues;
        %if dynamic weights required change to this
        S_ID_temp(1:numNodes,1:len_sw) = changeIndDesValues(1:numNodes,1,1:len_sw);
        S_ID = S_ID_temp';
        %need to recalculate S to take affect
     end
     %*********************************************************************
     %Maximise qual - Dynamic weights
     %check if fixed or dynamic weights
     if qualWeightsE == 1
     [calcSE_Qualw changeQualWeightValues] = DynAlgClass.changeAllWeights(t,qualWeights,qualWeightsValueOld);
     qualWeightsValueOld = changeQualWeightValues;
     changeQualWeightValuesRep = repmat(changeQualWeightValues,1,len_sw);
         %if dynamic weights required change to this
         We = changeQualWeightValuesRep';
     end 
     %*********************************************************************
     %Maximise qual - Dynamic desirabilities
     %check if fixed or dynamic desirabilities  
     if qualDesE==1
        [calcSE_Quald changeQualDesValues] = DynAlgClass.changeAllWeights(t,qualDes,qualDesValueOld);
        qualDesValueOld = changeQualDesValues;
        %if dynamic weights required change to this
        S_E_temp(1:numNodes,1:len_sw) = changeQualDesValues(1:numNodes,1,1:len_sw);
        S_E = S_E_temp';
        %need to recalculate S to take affect
     end
     %*********************************************************************
     %If any dynamic settings set(except max diversity), recalculate S, second output from function that
     %defines if an S update is required.
        %recalculate S 
        calcSE = (calcSE_Comw + calcSE_Comd + calcSE_Indw + calcSE_Indd + calcSE_Qualw + calcSE_Quald)>0; %check if update required
        if calcSE == 1
         tUpdateSTime = t
         S = calcSDyn(We, S_E, Wcd, S_CD, Wid, S_ID);    
        end
     
     %*********************************************************************
     %change weights of IndDesirabilities on at a certain time OLD METHOD -STILL WORKS
     if t == WidUpdateTime1
             tD=t %display t
             Wid = WidUpdateValue1
     elseif t== WidUpdateTime2
             tD=t %display t
             Wid = WidUpdateValue2    
     end
     %*********************************************************************
     %*********************************************************************
     %Change community level desirabilities OLD METHOD - STILL WORKS
     if ChangeDesirabilitiesFlag == 1
        if (t == tChange1) && (tChange1Enable ==1) 
            %recalculate total score with new community level desirabilities
            S_CD = repmat(CD2,1,numNodes);
            disp('change CD2')
      %S_IDSize=size(S_ID)
    %CDSize=size(S_CD)
    %S_ESize=size(S_E)
    %weSize=size(We)
    %wcdSize=size(Wcd)
    %widSize=size(Wid)
            S = calcSDyn(We, S_E, Wcd, S_CD, Wid, S_ID);
        end; 
        if (t == tChange2) && (tChange2Enable ==1) 
            %recalculate total score with new community level desirabilities
            S_CD = repmat(CD,1,numNodes);
            disp('change CD')
            S = calcSDyn(We, S_E, Wcd, S_CD, Wid, S_ID);
        end;
     end;
     %*********************************************************************
          %*********************************************************************
     %Change ALL individual level desirabilities - OLD METHOD - STILL WORKS
     if ChangeIndDesirabilitiesFlag == 1
        if (t == tChangeInd1) && (tChangeInd1Enable ==1) 
            %recalculate total score with new individual level desirabilities
            S_ID = ID2;
            disp('change ID2')
            %IDDisplay = ID(1:5,1:10)
            %ID2Display = ID2(1:5,1:10)
            %SDisplay = S(1:5,1:10)
            %size(We)
            %size(S_E)
            %size(Wcd)
            %size(S_CD)
            %size(Wid)
            %size(S_ID)
            S = calcSDyn(We, S_E, Wcd, S_CD, Wid, S_ID);
            %SDisplay = S(1:5,1:10)
        end; 
        if (t == tChangeInd2) && (tChangeInd2Enable ==1) 
            %recalculate total score with new individual level desirabilities
            S_ID = ID;
            disp('change ID')
            S = calcSDyn(We, S_E, Wcd, S_CD, Wid, S_ID);
        end;
     end;
     %*********************************************************************
           %*********************************************************************
     %Change ONLY OWN individual level desirabilities - OLD METHOD STILL WORKS
     if ChangeOwnIndDesirabilitiesFlag == 1
         %change weights
         if t == wdivUpdateTime1
             wdiv = wdivUpdateValue1;
             S = calcSDyn(We, S_E, Wcd, S_CD, Wid, S_ID);
         elseif t == wdivUpdateTime2
             wdiv = wdivUpdateValue2;
             S = calcSDyn(We, S_E, Wcd, S_CD, Wid, S_ID);
         elseif t == wdivUpdateTime3
             wdiv = wdivUpdateValue3;
             S = calcSDyn(We, S_E, Wcd, S_CD, Wid, S_ID);
         end
         if t == WidUpdateTime1
             Wid = WidUpdateValue1;
             S = calcSDyn(We, S_E, Wcd, S_CD, Wid, S_ID);
         elseif t == WidUpdateTime2
             Wid = WidUpdateValue2;
             S = calcSDyn(We, S_E, Wcd, S_CD, Wid, S_ID);
         end
        for i = 1:numNodes  
        if (t == IDtimeArray1(i)) 
            %recalculate total score with new individual level desirabilities
            S_ID_apps(i,:) = ID_apps2(i,:);
            S_ID_os(i,:) = ID_os2(i,:);
            S_ID_serv(i,:) = ID_serv2(i,:);
            S_ID_dr(i,:) = ID_dr2(i,:);            
            S_ID = [S_ID_apps S_ID_serv S_ID_os S_ID_dr]'; 
            %test
            %disp('change ID2')
            %tD=t
            %IDDisplay = ID(1:5,1:10)
            %S_IDDisplay = S_ID(1:5,1:10)
            %SDisplay = S(1:5,1:10)
            S = calcSDyn(We, S_E, Wcd, S_CD, Wid, S_ID);
            %SDisplay = S(1:5,1:10)
        end; 
        if (t == IDtimeArray2(i)) 
            %recalculate total score with new individual level desirabilities
            S_ID_apps(i,:) = ID_apps(i,:);
            S_ID_os(i,:) = ID_os(i,:);
            S_ID_serv(i,:) = ID_serv(i,:);
            S_ID_dr(i,:) = ID_dr(i,:);            
            S_ID = [S_ID_apps S_ID_serv S_ID_os S_ID_dr]';
            %disp('change ID')
            S = calcSDyn(We, S_E, Wcd, S_CD, Wid, S_ID);
        end;
        end; %end for i loop
        %recalculate dynamic diversity of individuals
        Nei_indDynamic(t) = variantFreq4(textdata, S_ID, 0);
        
     end; %end if flag
     %*********************************************************************
     %*********************************************************************
     %Change efficiencies  -OLD METHOD - STILL WORKS
     if ChangeEffFlag == 1
        if (t == tChangeEff1) && (tChangeEff1Enable ==1) 
            %recalculate total score with new individual level desirabilities
            S_E = repmat(E2,1,numNodes);
            disp('change E2')
            tDisplay = t
            S = calcSDyn(We, S_E, Wcd, S_CD, Wid, S_ID);
        end; 
        if (t == tChangeEff2) && (tChangeEff2Enable ==1) 
            %recalculate total score with new individual level desirabilities
            S_E = repmat(E,1,numNodes);
            tDisplay=t
            disp('change E')
            S = calcSDyn(We, S_E, Wcd, S_CD, Wid, S_ID);
        end;
     end;
     %*********************************************************************    
     
     %update time progress
     if t == fix(numRuns/2)
         numTotSims = num2str(get(handles.popupmenuNumSims, 'Value'))%total number of simulations
         prog=num2str(((psim-1)*(100/numSims)) + (50/numSims));
         %prog = num2str(50 * psim/numSims)
         
         progressStr = [prog '%  ' num2str(currentSimNumber) ' of ' numTotSims]
         set(handles.editSimProgress, 'String' ,progressStr);
         pause(0.001)%time to update screen
     end
     if t == numRuns
         numTotSims = num2str(get(handles.popupmenuNumSims, 'Value'))%total number of simulations
         prog=num2str(((psim-1)*(100/numSims)) + (100/numSims));
         %prog = num2str(50 * psim/numSims)
         
         progressStr = [prog '%  ' num2str(currentSimNumber) ' of ' numTotSims]
         set(handles.editSimProgress, 'String' ,progressStr);
         pause(0.001)%time to update screen
     end
     
     %*********************************************************************
     %do for each node like a for and an if statement
     %encounter a node if less than probability number
     %(randomNumber < probabilityDistr) equals 1 if true
     encounterYes = (randomNumber < probabilityDistr);
     encounterYesRep = repmat(encounterYes,1,tallyLength);
     %generate a random node to encounter , but must not encounter itself
     randomEncounter = randi([1 numNodes],numNodes,1);
     for i = 1:numNodes
         while randomEncounter(i) == i
             randomEncounter(i) = randi(numNodes,1);
         end;
     end;
     %only tally those that have had an encounter
     tallySol1 = (GenotypeSol1(randomEncounter,:)).*encounterYesRep + tallySol1;
     tallySol2 = (GenotypeSol2(randomEncounter,:)).*encounterYesRep + tallySol2;
     tallySol3 = (GenotypeSol3(randomEncounter,:)).*encounterYesRep + tallySol3;
     tallySol4 = (GenotypeSol4(randomEncounter,:)).*encounterYesRep + tallySol4;
     tallySol5 = (GenotypeSol5(randomEncounter,:)).*encounterYesRep + tallySol5;
     %increment counters - only increment if encountered another node
     counterEncounters = counterEncounters + (1.*encounterYes);
     %numEncountersDISPLAY = numEncounters
     %check counters for update (if true will set to 1)
     updateYes = (counterEncounters == numEncounters);
     %*******************************************
     %STOPPING THE UPDATE PROCESS
     %Code for solution 4 and 5 only
     %*******************************************
     if t >sampleSize 
     start = t-(sampleSize);
     else
         start = 1;
     end;
     stop = t+1;
     %record the diversity each node sees for solution 4 only
     nodesDiversityOld4 = nodesDiversity4(:,t);
     nodesDiversity4(:,t+1) = nodesNei(tallySol4, numEncounters, len_apps, len_serv, len_os, len_dr, updateYes, nodesDiversityOld4);
     %record the running mean of the diversity each node sees for sol 4
     %record the running standard deviation each node sees for solution 4
     %(dim 1 does the columns, dim 2 does the rows)
     nodesMean4(:,t+1) = mean(nodesDiversity4(:,start:stop),2); 
     nodesStd4(:,t+1) = std(nodesDiversity4(:,start:stop),0,2);
     
     %nodesDiversityOld4T = nodesDiversity4(:,t);
     %nodesDiversityOld4TD = nodesDiversityOld4T(1:5,1:end)
     %[nodesStd4T nodesMean4T nodesDiversity4T] = localDiversity(t,sampleSize,nodesDiversityOld4T,tallySol4,numEncounters,len_apps,len_serv,len_os,len_dr,updateYes);
     
     
     %****************Single stop update solution 4 only
     if stopUpdate4 == 1    
      if singleStopMode == 1 ||singleStopMode == 2
      [updateStop4 updateStopDisplayFlag4 lastStoppedStdNew] = singleStopUpdating(singleStopMode,t,sampleSize,nodesStd4,numNodes,updateStopDisplayFlagOld4,displayStopVal,displayStartVal,minstd4Tolerance,lastStoppedStd);           
        updateStopDisplayFlagOld4(t,:) = updateStopDisplayFlag4(t,:); %record value for next time round      
        updateStopDisplay4(t,:) = updateStopDisplayFlag4(t,:); %record valuefor displaying on graph    
      elseif singleStopMode == 3 ||singleStopMode == 4
       [updateStop4 updateStopDisplayFlag4 lastStoppedStdNew] = singleStopUpdating(singleStopMode,t,sampleSize,nodesStd4,numNodes,updateStopDisplayFlagOld4,displayStopVal,displayStartVal,minstd4Tolerance,lastStoppedStd);           
        updateStopDisplayFlagOld4(t,:) = updateStopDisplayFlag4(t,:); %record value for next time round      
        updateStopDisplay4(t,:) = updateStopDisplayFlag4(t,:); %record valuefor displaying on graph 
        
        updateStop4Old = updateStop4;
        [updateStopDisplayFlag4 updateStop4] = restart(singleStopMode,stopUpdate4,numNodes,sampleSize,t,tChange1Enable,tChange1,tChange2Enable,tChange2,updateStopDisplayFlagOld4,updateStop4Old,displayStopVal,displayStartVal,...
                                                        nodesStd4,maxStdTolerance,lastStoppedStd,stdMargin,...
                                                        ChangeDesirabilitiesFlag,ChangeEffFlag,ChangeIndDesirabilitiesFlag,ChangeOwnIndDesirabilitiesFlag, ...
                                                        tChangeEff1Enable, tChangeEff2Enable, tChangeEff1, tChangeEff2, ...
                                                        tChangeInd1Enable, tChangeInd2Enable, tChangeInd1,tChangeInd2, ...
                                                        IDtimeArray1, IDtimeArray2,wdivUpdateTime1,wdivUpdateTime2,wdivUpdateTime3,WidUpdateTime1,WidUpdateTime2);
                                                    
                                                    
                                                       
                                                    
                                                    
        %update flags
        updateStopDisplayFlagOld4(t,:) = updateStopDisplayFlag4(t,:); %record value for next time round      
        updateStopDisplay4(t,:) = updateStopDisplayFlag4(t,:); %record valuefor displaying on graph  
      elseif singleStopMode == 5   
        [updateStop4 updateStopDisplayFlag4 lastStoppedStdNew] = singleStopUpdating(singleStopMode,t,sampleSize,nodesStd4,numNodes,updateStopDisplayFlagOld4,displayStopVal,displayStartVal,minstd4Tolerance,lastStoppedStd);           
        updateStopDisplayFlagOld4(t,:) = updateStopDisplayFlag4(t,:); %record value for next time round      
        updateStopDisplay4(t,:) = updateStopDisplayFlag4(t,:); %record valuefor displaying on graph 
        
        updateStop4Old = updateStop4;
        [updateStopDisplayFlag4 updateStop4] = restart(singleStopMode,stopUpdate4,numNodes,sampleSize,t,tChange1Enable,tChange1,tChange2Enable,tChange2,updateStopDisplayFlagOld4,updateStop4Old,displayStopVal,displayStartVal,...
                                                        nodesStd4,maxStdTolerance,lastStoppedStd,stdMargin,...
                                                        ChangeDesirabilitiesFlag,ChangeEffFlag,ChangeIndDesirabilitiesFlag,ChangeOwnIndDesirabilitiesFlag, ...
                                                        tChangeEff1Enable, tChangeEff2Enable, tChangeEff1, tChangeEff2, ...
                                                        tChangeInd1Enable, tChangeInd2Enable, tChangeInd1,tChangeInd2, ...
                                                        IDtimeArray1, IDtimeArray2,wdivUpdateTime1,wdivUpdateTime2,wdivUpdateTime3,WidUpdateTime1,WidUpdateTime2);
        %update flags
        updateStopDisplayFlagOld4(t,:) = updateStopDisplayFlag4(t,:); %record value for next time round      
        updateStopDisplay4(t,:) = updateStopDisplayFlag4(t,:); %record valuefor displaying on graph  
      end %if
     end %if
     %****************Multiple stop update solution 4 only
     if restartUpdate4 == 1
        [updateStop4 updateStopDisplayFlag4 lastStoppedStdNew] = singleStopUpdating(restartMode,t,sampleSize,nodesStd4,numNodes,updateStopDisplayFlagOld4,displayStopVal,displayStartVal,minstd4Tolerance,lastStoppedStd);           
        updateStopDisplayFlagOld4(t,:) = updateStopDisplayFlag4(t,:); %record value for next time round      
        updateStopDisplay4(t,:) = updateStopDisplayFlag4(t,:); %record valuefor displaying on graph 
        lastStoppedStd = lastStoppedStdNew;

        updateStop4Old = updateStop4;
        %restartModeD=restartMode
        [updateStopDisplayFlag4 updateStop4] = restart(restartMode,restartUpdate4,numNodes,sampleSize,t,tChange1Enable,tChange1,tChange2Enable,tChange2,updateStopDisplayFlagOld4,updateStop4Old,displayStopVal,displayStartVal,...
                                                        nodesStd4,maxstd4Tolerance,lastStoppedStdNew,stdMargin,...
                                                        ChangeDesirabilitiesFlag,ChangeEffFlag,ChangeIndDesirabilitiesFlag,ChangeOwnIndDesirabilitiesFlag, ...
                                                        tChangeEff1Enable, tChangeEff2Enable, tChangeEff1, tChangeEff2, ...
                                                        tChangeInd1Enable, tChangeInd2Enable, tChangeInd1,tChangeInd2, ...
                                                        IDtimeArray1, IDtimeArray2,wdivUpdateTime1,wdivUpdateTime2,wdivUpdateTime3,WidUpdateTime1,WidUpdateTime2);
        %update flags
        updateStopDisplayFlagOld4(t,:) = updateStopDisplayFlag4(t,:); %record value for next time round      
        updateStopDisplay4(t,:) = updateStopDisplayFlag4(t,:); %record valuefor displaying on graph
 

        %check last stopped std value-THIS CODE IS FOR TESTING PURPOSES ONLY FOR MODE6******
        lastStoppedStdMatrix(t,:) = lastStoppedStdNew;
        if t > 1
            for i = 1: numNodes
                 if updateStopDisplay4(t-1,i)==displayStartVal && updateStopDisplay4(t,i) == displayStopVal
                    lastStoppedStdMatrix2(t,i) = lastStoppedStdNew(i);
                 else
                    lastStoppedStdMatrix2(t,i) = lastStoppedStdMatrix2(t-1,i);
                 end%end if
            end %end i loop
        end%end t
        %**********************************************

     end
     %*******************************************************
     %****************Multiple stop update solution 4 only
   %  if restartUpdate4 == 1
   %  [nodesMeanNew4 nodesStdNew4 updateStopNew4 updateStopDisplayNew4 minStdCurveNew4, minStdNew4, reachTolFlagNew4 nodesDiversityNew4] = multipleStopUpdating(t, sampleSize, ...
   %                   numNodes, nodesDiversity4, ...
   %                   tallySol4, numEncounters, len_apps, len_serv, len_os, len_dr, updateYes, ...
   %                   minstd4Tolerance, minStd4, maxStd4,reachTolFlag4, updateStop4, updateStopDisplay4, minStd4Curve);
   %  end;  
   %  reachTolFlag4 = reachTolFlagNew4;
   %  updateStop4 = updateStopNew4;
   %  updateStopDisplay4(t+1,:) = updateStopDisplayNew4(t,:);
   %  minStd4Curve(:,t+1) = minStdCurveNew4(:,t);
   %  minStd4 = minStdNew4;
   %  nodesDiversity4(:,t+1) = nodesDiversityNew4(:,t+1);
   %  nodesMean4(:,t+1) = nodesMeanNew4(:,t+1);
   %   nodesStd4(:,t+1) = nodesStdNew4(:,t+1);      
     if restartUpdateTest == 1
     %record the diversity each node sees for solution 4 only
     %nodesDiversityOld4 = nodesDiversity4(:,t);
     %nodesDiversity4(:,t+1) = nodesNei(tallySol4, numEncounters, len_apps, len_serv, len_os, len_dr, updateYes, nodesDiversityOld4);
     %record the running standard deviation each node sees for solution 4
     %nodesMean4(:,t+1) = mean(nodesDiversity4(:,start:stop),2); 
     %nodesStd4(:,t+1) = std(nodesDiversity4(:,start:stop),0,2);
     for i = 1: numNodes
     % check to see if surpassed sample size
     if (t >sampleSize)
      %***minimum update
      if reachTolFlag4(i) == 0 && (nodesStd4(i,t+1) <= minStd4(i)) && (nodesStd4(i,t+1) < maxStd4(i))  
          minStd4(i) = nodesStd4(i,t+1); %set new minimum
          minStd4Curve(i,t) = nodesStd4(i,t+1);
          reachTolFlag4(i) = 1; %surpassed tolerance
      elseif reachTolFlag4(i) == 1 && (nodesStd4(i,t+1) <= minStd4(i)) && (nodesStd4(i,t+1) < maxStd4(i))  
          minStd4(i) = nodesStd4(i,t+1); %set new minimum
           minStd4Curve(i,t) = nodesStd4(i,t+1);
          updateStop4(i) = 1; %continue updating
          updateStopDisplay4(t,i)= 1; %continue updating
      elseif reachTolFlag4(i) == 1 && (nodesStd4(i,t+1) > minStd4(i)) && (nodesStd4(i,t+1) < maxStd4(i)) 
         updateStop4(i) = 0; %stop updating
         updateStopDisplay4(t,i)= 0; %stop updating
         minStd4Curve(i,t) = minStd4Curve(i,t-1);
      %***maximum update - update again and reset if std goes back up again
      elseif reachTolFlag4(i) == 1 && (nodesStd4(i,t+1) > minStd4(i)) && (nodesStd4(i,t+1) > maxStd4(i))  
          updateStop4(i) = 1; %start updating again
          updateStopDisplay4(t,i)= 1; %start updating again
         %reset min tolerance and tolFlag
          minStd4(i) = minstd4Tolerance;
           minStd4Curve(i,t) = minstd4Tolerance;
          reachTolFlag4(i) = 0;
      else
          minStd4Curve(i,t) = minStd4Curve(i,t-1);
      end; %end stop update logic
     else
         minStd4Curve(i,t) = minstd4Tolerance;
     end; %end sample size
     end; %end i loop
     end; % end restartUpdate4
     
     %***********************************************
     if t >sampleSize 
     start = t-(sampleSize);
     else
         start = 1;
     end;
     stop = t+1;
     nodesDiversityOld5 = nodesDiversity5(:,t);
     nodesDiversity5(:,t+1) = nodesNei(tallySol5, numEncounters, len_apps, len_serv, len_os, len_dr, updateYes, nodesDiversityOld5);
     %record the running mean of the diversity each node sees for sol 5
     %record the running standard deviation each node sees for solution 5
     %(dim 1 does the columns, dim 2 does the rows)
     nodesMean5(:,t+1) = mean(nodesDiversity5(:,start:stop),2); 
     nodesStd5(:,t+1) = std(nodesDiversity5(:,start:stop),0,2);
     
     
     %****************Single stop update solution 5 only
     if stopUpdate5 == 1                 
      [updateStop5 updateStopDisplayFlag5 lastStoppedStdNew] = singleStopUpdating(singleStopMode,t,sampleSize,nodesStd5,numNodes,updateStopDisplayFlagOld5,displayStopVal,displayStartVal,minstd5Tolerance,lastStoppedStd5);           
        updateStopDisplayFlagOld5(t,:) = updateStopDisplayFlag5(t,:); %record value for next time round      
        updateStopDisplay5(t,:) = updateStopDisplayFlag5(t,:); %record valuefor displaying on graph        
     
     
     
     
          %record the diversity each node sees for solution 5 only
     %nodesDiversityOld5 = nodesDiversity5(:,t);
     %nodesDiversity5(:,t+1) = nodesNei(tallySol5, numEncounters, len_apps, len_serv, len_os, len_dr, updateYes, nodesDiversityOld5);
     %record the running mean of the diversity each node sees for sol 5
     %record the running standard deviation each node sees for solution 5
     %(dim 1 does the columns, dim 2 does the rows)
     %nodesMean5(:,t+1) = mean(nodesDiversity5(:,start:stop),2); 
     %nodesStd5(:,t+1) = std(nodesDiversity5(:,start:stop),0,2);
    % for i = 1: numNodes
     % if standard deviation less than the tolerance and surpassed sample
     % size, then stop updating
    %   if (nodesStd5(i,t+1) < minstd5Tolerance) && (t >sampleSize)
     %    updateStop5(i) = 0;
    %     updateStopDisplay5(t,i)= 0;
    %   end; %end nodesStd
   % end; %end i loop
 end
     %***********************************************
     %****************Multiple stop update solution 5 only
     if restartUpdate5 == 1
     %record the diversity each node sees for solution 5 only
     %nodesDiversityOld5 = nodesDiversity5(:,t);
     %nodesDiversity5(:,t+1) = nodesNei(tallySol5, numEncounters, len_apps, len_serv, len_os, len_dr, updateYes, nodesDiversityOld5);
     %record the running standard deviation each node sees for solution 4
     %nodesMean5(:,t+1) = mean(nodesDiversity5(:,start:stop),2); 
     %nodesStd5(:,t+1) = std(nodesDiversity5(:,start:stop),0,2);
     for i = 1: numNodes
     % check to see if surpassed sample size
     if (t >sampleSize)
      %***minimum update
      if reachTolFlag5(i) == 0 && (nodesStd5(i,t+1) <= minStd5(i)) && (nodesStd5(i,t+1) < maxStd5(i))  
          minStd5(i) = nodesStd5(i,t+1); %set new minimum
          minStd5Curve(i,t) = nodesStd5(i,t+1);
          reachTolFlag5(i) = 1; %surpassed tolerance
      elseif reachTolFlag5(i) == 1 && (nodesStd5(i,t+1) <= minStd5(i)) && (nodesStd5(i,t+1) < maxStd5(i))  
          minStd5(i) = nodesStd5(i,t+1); %set new minimum
           minStd5Curve(i,t) = nodesStd5(i,t+1);
          updateStop5(i) = 1; %continue updating
          updateStopDisplay5(t,i)= 1; %continue updating
      elseif reachTolFlag5(i) == 1 && (nodesStd5(i,t+1) > minStd5(i)) && (nodesStd5(i,t+1) < maxStd5(i)) 
         updateStop5(i) = 0; %stop updating
         updateStopDisplay5(t,i)= 0; %stop updating
         minStd5Curve(i,t) = minStd5Curve(i,t-1);
      %***maximum update - update again and reset if std goes back up again
      elseif reachTolFlag5(i) == 1 && (nodesStd5(i,t+1) > minStd5(i)) && (nodesStd5(i,t+1) > maxStd5(i))  
          updateStop5(i) = 1; %start updating again
          updateStopDisplay5(t,i)= 1; %start updating again
          %reset min tolerance and tolFlag
          minStd5(i) = minstd5Tolerance;
           minStd5Curve(i,t) = minstd5Tolerance;
          reachTolFlag5(i) = 0;
      else
          minStd5Curve(i,t) = minStd5Curve(i,t-1);
      end; %end stop update logic
     else
         minStd5Curve(i,t) = minstd5Tolerance;
     end; %end sample size
     end; %end i loop
     end; % end restartUpdate4
     %***********************************************
     
     
     %keep a running total of the number of updates performed
     tallyUpdates = tallyUpdates + updateYes;
     %run update procedure if necessary
     %ADD CODE HERE
     %solution 1
     if turnOnSol1 == 1
     solNum=1;
     [Genotype_1New Genotype_apps1New Genotype_serv1New Genotype_os1New Genotype_dr1New] = ...
                    updateGenotype(tallySol1, solNum, tallyLength, updateYes, updateStop1, numNodes, numEncounters, tolerance, ...
                    We, Wcd, Wid, wdiv, S, CSWGrouprep, numOS, CDSize, ...
                    CSW1, CSW2, CSW3, CSW4, CSW5, CSW6, CSW7, CSW8, CSW9, CSW10, ... 
                    len_apps, len_serv, len_os, len_dr, ...
                    SizeCD_apps, SizeCD_serv, SizeCD_os, SizeCD_dr, ...
                    GenotypeSol1_apps, GenotypeSol1_serv, GenotypeSol1_os, GenotypeSol1_dr);
     end; 
     %solution 2
     if turnOnSol2 == 1
     solNum=2;
     [Genotype_2New Genotype_apps2New Genotype_serv2New Genotype_os2New Genotype_dr2New] = ...
                    updateGenotype(tallySol2, solNum, tallyLength, updateYes, updateStop2, numNodes, numEncounters, tolerance, ...
                    We, Wcd, Wid, wdiv, S, CSWGrouprep, numOS, CDSize, ...
                    CSW1, CSW2, CSW3, CSW4, CSW5, CSW6, CSW7, CSW8, CSW9, CSW10, ... 
                    len_apps, len_serv, len_os, len_dr, ...
                    SizeCD_apps, SizeCD_serv, SizeCD_os, SizeCD_dr, ...
                    GenotypeSol2_apps, GenotypeSol2_serv, GenotypeSol2_os, GenotypeSol2_dr);
     end;           
     %solution 3
     if turnOnSol3 == 1
     solNum=3;
     [Genotype_3New Genotype_apps3New Genotype_serv3New Genotype_os3New Genotype_dr3New] = ...
                    updateGenotype(tallySol3, solNum, tallyLength, updateYes, updateStop3, numNodes, numEncounters, tolerance, ...
                    We, Wcd, Wid, wdiv, S, CSWGrouprep, numOS, CDSize, ...
                    CSW1, CSW2, CSW3, CSW4, CSW5, CSW6, CSW7, CSW8, CSW9, CSW10, ... 
                    len_apps, len_serv, len_os, len_dr, ...
                    SizeCD_apps, SizeCD_serv, SizeCD_os, SizeCD_dr, ...
                    GenotypeSol3_apps, GenotypeSol3_serv, GenotypeSol3_os, GenotypeSol3_dr);
     end; 
     %solution 4
     if turnOnSol4 == 1
     solNum=6;
     [Genotype_4New Genotype_apps4New Genotype_serv4New Genotype_os4New Genotype_dr4New] = ...
                    updateGenotype(tallySol4, solNum, tallyLength, updateYes, updateStop4, numNodes, numEncounters, tolerance, ...
                    We, Wcd, Wid, wdiv, S, CSWGrouprep, numOS, CDSize, ...
                    CSW1, CSW2, CSW3, CSW4, CSW5, CSW6, CSW7, CSW8, CSW9, CSW10, ... 
                    len_apps, len_serv, len_os, len_dr, ...
                    SizeCD_apps, SizeCD_serv, SizeCD_os, SizeCD_dr, ...
                    GenotypeSol4_apps, GenotypeSol4_serv, GenotypeSol4_os, GenotypeSol4_dr);
 
     end;
     %solution 5
     if turnOnSol5 == 1
     solNum=7;
     [Genotype_5New Genotype_apps5New Genotype_serv5New Genotype_os5New Genotype_dr5New] = ...
                    updateGenotype(tallySol5, solNum, tallyLength, updateYes, updateStop5, numNodes, numEncounters, tolerance, ...
                    We, Wcd, Wid, wdiv, S, CSWGrouprep, numOS, CDSize, ...
                    CSW1, CSW2, CSW3, CSW4, CSW5, CSW6, CSW7, CSW8, CSW9, CSW10, ... 
                    len_apps, len_serv, len_os, len_dr, ...
                    SizeCD_apps, SizeCD_serv, SizeCD_os, SizeCD_dr, ...
                    GenotypeSol5_apps, GenotypeSol5_serv, GenotypeSol5_os, GenotypeSol5_dr); 
     %Genotype_apps5New
     end;
     %reset counters if needed, if reset resetCounters will be set to zero
     resetCounters = 1 - updateYes;
     counterEncounters = counterEncounters .* resetCounters;
     %reset the tallySol if needed
     resetCountersRep = repmat(resetCounters,1,tallyLength);
     tallySol1 = tallySol1 .*resetCountersRep;
     tallySol2 = tallySol2 .*resetCountersRep;
     tallySol3 = tallySol3 .*resetCountersRep;
     tallySol4 = tallySol4 .*resetCountersRep;
     tallySol5 = tallySol5 .*resetCountersRep;
     
     %reasign new output   
     %solution 1
     if turnOnSol1 == 1
     GenotypeSol1 = Genotype_1New;
     GenotypeSol1_apps = Genotype_apps1New;
     GenotypeSol1_serv = Genotype_serv1New;
     GenotypeSol1_os = Genotype_os1New;
     GenotypeSol1_dr = Genotype_dr1New;
     Nei_sol1(psim,t) = variantFreq4(textdata, Genotype_1New', 0); %diversity index
     end;
     %solution 2
     if turnOnSol2 == 1
     GenotypeSol2 = Genotype_2New;
     GenotypeSol2_apps = Genotype_apps2New;
     GenotypeSol2_serv = Genotype_serv2New;
     GenotypeSol2_os = Genotype_os2New;
     GenotypeSol2_dr = Genotype_dr2New;
     Nei_sol2(psim,t) = variantFreq4(textdata, Genotype_2New', 0);%diversity index
     end;
     %solution 3
     if turnOnSol3 == 1
     GenotypeSol3 = Genotype_3New;
     GenotypeSol3_apps = Genotype_apps3New;
     GenotypeSol3_serv = Genotype_serv3New;
     GenotypeSol3_os = Genotype_os3New;
     GenotypeSol3_dr = Genotype_dr3New;
     Nei_sol3(psim,t) = variantFreq4(textdata, Genotype_3New', 0);%diversity index
     end;
     %solution 4
     if turnOnSol4 == 1
     GenotypeSol4 = Genotype_4New;
     GenotypeSol4_apps = Genotype_apps4New;
     GenotypeSol4_serv = Genotype_serv4New;
     GenotypeSol4_os = Genotype_os4New;
     GenotypeSol4_dr = Genotype_dr4New;
     Nei_sol4(psim,t) = variantFreq4(textdata, Genotype_4New', 0);%diversity index
     end;
     %solution 5
     if turnOnSol5 == 1
     GenotypeSol5 = Genotype_5New;
     GenotypeSol5_apps = Genotype_apps5New;
     GenotypeSol5_serv = Genotype_serv5New;
     GenotypeSol5_os = Genotype_os5New;
     GenotypeSol5_dr = Genotype_dr5New;
     Nei_sol5(psim,t) = variantFreq4(textdata, Genotype_5New', 0);%diversity index
     end;
           
 end; %end t
 end; %end psim

 %return output data
 outputs.numNodes=numNodes;
 outputs.numRuns=numSims;
 outputs.numTimeSteps=numRuns;
 outputs.len_apps=len_apps;
 outputs.len_serv=len_serv;
 outputs.len_os=len_os;
 outputs.len_dr=len_dr;
 outputs.CD_apps=CD_apps;
 outputs.CD_serv=CD_serv;
 outputs.CD_os=CD_os;
 outputs.CD_dr=CD_dr;
 outputs.ChangeDesirabilitiesFlag=ChangeDesirabilitiesFlag;
 outputs.tChange1=tChange1;
 outputs.tChange2=tChange2;
 outputs.CD_apps2=CD_apps2;
 outputs.CD_serv2=CD_serv2;
 outputs.CD_os2=CD_os2;
 outputs.CD_dr2=CD_dr2;
 outputs.ChangeIndDesirabilitiesFlag=ChangeIndDesirabilitiesFlag;
 outputs.tChangeInd1=tChangeInd1;
 outputs.tChangeInd2=tChangeInd2;
 outputs.ID2=ID2;
 outputs.ID_apps2=ID_apps2;
 outputs.ID_serv2=ID_serv2;
 outputs.ID_os2=ID_os2;
 outputs.ID_dr2=ID_dr2;
 outputs.E_apps=E_apps;
 outputs.E_serv=E_serv;
 outputs.E_os=E_os;
 outputs.E_dr=E_dr;
 outputs.ChangeEffFlag=ChangeEffFlag;
 outputs.tChangeEff1=tChangeEff1;
 outputs.tChangeEff2=tChangeEff2;
 outputs.E_apps2=E_apps2;
 outputs.E_serv2=E_serv2;
 outputs.E_os2=E_os2;
 outputs.E_dr2=E_dr2;
 outputs.Genotype=Genotype;
 outputs.Genotype_apps=Genotype_apps;
 outputs.Genotype_serv=Genotype_serv;
 outputs.Genotype_os=Genotype_os;
 outputs.Genotype_dr=Genotype_dr; 
 outputs.textdata=textdata;
 outputs.ID=ID;
 outputs.ID_apps=ID_apps;
 outputs.ID_serv=ID_serv;
 outputs.ID_os=ID_os;
 outputs.ID_dr=ID_dr;
 outputs.Nei_sol1=Nei_sol1;
 outputs.Nei_sol2=Nei_sol2;
 outputs.Nei_sol3=Nei_sol3;
 outputs.Nei_sol4=Nei_sol4;
 outputs.Nei_sol5=Nei_sol5;
 outputs.nodesDiversity4=nodesDiversity4;
 outputs.nodesMean4=nodesMean4;
 outputs.nodesStd4=nodesStd4;
 outputs.updateStopDisplay4=updateStopDisplay4;
 outputs.nodesStd4=nodesStd4;
 outputs.minstd4Tolerance=minstd4Tolerance;
 outputs.maxstd4Tolerance=maxstd4Tolerance;
 outputs.minStd4Curve=minStd4Curve;
 outputs.updateStopDisplay5=updateStopDisplay5;
 outputs.nodesStd5=nodesStd5;
 outputs.minstd5Tolerance=minstd5Tolerance;
 outputs.maxstd5Tolerance=maxstd5Tolerance;
 outputs.minStd5Curve=minStd5Curve;
 outputs.turnOnSol1=turnOnSol1;
 outputs.turnOnSol2=turnOnSol2;
 outputs.turnOnSol3=turnOnSol3;
 outputs.turnOnSol4=turnOnSol4;
 outputs.turnOnSol5=turnOnSol5;
 
 %NEW OUTPUTS NOT YET AUTO SAVED TO FILE
 outputs.lastStoppedStdMatrix=lastStoppedStdMatrix;
 outputs.lastStoppedStdMatrix2=lastStoppedStdMatrix2;
 outputs.stdMargin=stdMargin;
 outputs.Nei_indDynamic=Nei_indDynamic;
 outputs.singleStopMode = singleStopMode;
 outputs.restartMode = restartMode;
 outputs.ChangeOwnIndDesirabilitiesFlag = ChangeOwnIndDesirabilitiesFlag;

 
 
 %plot data output 
% [NeiAbsMax Nei_orig Nei_ind Nei_comm Nei_eff ] = ...
%    plotOutput(saveGraphs, saveGraphsDirectoryName, numNodes, numSims, numRuns, ...
%               len_apps, len_serv, len_os, len_dr, ...
%               CD_apps,CD_serv,CD_os,CD_dr, ...
%               ChangeDesirabilitiesFlag, tChange1, tChange2,CD_apps2,CD_serv2,CD_os2,CD_dr2, ...
%               E_apps,E_serv,E_os,E_dr, ...
%               Genotype, textdata, ID, ...
%               Nei_sol1, Nei_sol2, Nei_sol3, Nei_sol4, Nei_sol5, ...
%               nodesDiversity4, nodesMean4, nodesStd4, ...
%               updateStopDisplay4, nodesStd4, minstd4Tolerance, maxstd4Tolerance, minStd4Curve, ...
%               updateStopDisplay5, nodesStd5, minstd5Tolerance, maxstd5Tolerance, minStd5Curve, ...
%               turnOnSol1, turnOnSol2, turnOnSol3, turnOnSol4, turnOnSol5);
           
%save data         
%good = saveOutput(numNodes,saveData, saveDataDirectoryName,nodesDiversity4)
          