%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Jennifer Jackson 2017                                                        %
% A Biodiversity Approach to Cyber Security                                    %
% PhD Complexity Science                                                       %
% University of Warwick                                                        %
% http://wrap.warwick.ac.uk/98458                                              %
% This software is distributed under the  Creative Commons Attribution License %
% https://creativecommons.org/licenses/by/4.0/                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function DynamicDiversityBatchMode()
close all;
clear all;
runTests = ones(1,7) %(1,numTests total)
%runTests(1) = 0 % 0 means don't run that test
%runTests(2) = 0 % 0 means don't run that test
%directory = 'settings\dynamicDiversity\'
directory = 'settings\dynamicDiversity\' %PC directory
tic
figh = DiversityModelGUI; %load program

%test settings files to run
%**************************
%Test 1
%continuous mode, random encounter
%**************************
if runTests(1) == 1
testFile(1).Filename = [directory 'Test1_0_02\settings'];
testFile(2).Filename = [directory 'Test1_0_04\settings'];
testFile(3).Filename= [directory 'Test1_0_06\settings'];
testFile(4).Filename = [directory 'Test1_0_08\settings'];
testFile(5).Filename = [directory 'Test1_0_1\settings'];

numTests = size(testFile,2)

for testNumber = 1:numTests
    run1Instance(figh,testFile(testNumber).Filename)
end
disp('completed test 1')
end

%**************************
%Test 2
%single stop mode, random encounter
%**************************
if runTests(2) == 1
clear testFile
testFile(1).Filename = [directory 'Test2_0_02\settings'];
testFile(2).Filename = [directory 'Test2_0_04\settings'];
testFile(3).Filename= [directory 'Test2_0_06\settings'];
testFile(4).Filename = [directory 'Test2_0_08\settings'];
testFile(5).Filename = [directory 'Test2_0_1\settings'];

numTests = size(testFile,2)
%numTests = 2 %for overide

for testNumber = 1:numTests
    run1Instance(figh,testFile(testNumber).Filename)
end
disp('completed test 2')
end



%**************************
%Test 3
%continuous, wireless bluetooth
%**************************
if runTests(3) == 1
clear testFile
testFile(1).Filename = [directory 'Test3_WBT_10\settings'];

numTests = size(testFile,2)
%numTests = 2 %for overide

for testNumber = 1:numTests
    run1Instance(figh,testFile(testNumber).Filename)
end
disp('completed test 3')
end

%**************************
%Test 4
%single stop, wireless bluetooth
%**************************
if runTests(4) == 1
clear testFile
testFile(1).Filename = [directory 'Test4_WBT_10\settings'];

numTests = size(testFile,2)
%numTests = 2 %for overide

for testNumber = 1:numTests
    run1Instance(figh,testFile(testNumber).Filename)
end
disp('completed test 4')
end


%**************************
%Test 5
%continuous, random waypoint
%**************************
if runTests(5) == 1
clear testFile
testFile(1).Filename = [directory 'Test5_RWP_rr\settings'];

numTests = size(testFile,2)
%numTests = 2 %for overide

for testNumber = 1:numTests
    run1Instance(figh,testFile(testNumber).Filename)
end
disp('completed test 5')
end

%**************************
%Test 6
%Single stop, random waypoint
%**************************
if runTests(6) == 1
clear testFile
testFile(1).Filename = [directory 'Test6_RWP_rr\settings'];

numTests = size(testFile,2)
%numTests = 2 %for overide

for testNumber = 1:numTests
    run1Instance(figh,testFile(testNumber).Filename)
end
disp('completed test 6')
end

%**************************
%Test 7
%Continuous, random encounter, vary the number of encounters before an
%update
%**************************
if runTests(7) == 1
clear testFile
testFile(1).Filename = [directory 'Test7_0_1_2e\settings'];
testFile(2).Filename = [directory 'Test7_0_1_10e\settings'];
testFile(3).Filename= [directory 'Test7_0_1_30e\settings'];
testFile(4).Filename = [directory 'Test7_0_1_50e\settings'];
testFile(5).Filename = [directory 'Test7_0_1_100e\settings'];
testFile(6).Filename = [directory 'Test7_0_02_2e\settings'];
testFile(7).Filename = [directory 'Test7_0_02_10e\settings'];
testFile(8).Filename= [directory 'Test7_0_02_30e\settings'];
testFile(9).Filename = [directory 'Test7_0_02_50e\settings'];
testFile(10).Filename = [directory 'Test7_0_02_100e\settings'];

numTests = size(testFile,2)
%numTests = 2 %for overide

for testNumber = 1:numTests
    run1Instance(figh,testFile(testNumber).Filename)
end
disp('completed test 7')
end


toc
%close program

end



%***************
%sub functions
%***************
function run1Instance(figh,testFile)
%***********************************
%Run 1 program simulation
%***********************************
DiversityModelGUI('LoadSettings_Callback',figh,[],guidata(figh))
%DiversityModelGUI('LoadSettings_Callback',fighdata.LoadSettings,[],guidata(figh))
fighdata = guidata(figh); %load main gui data
loadSettingsData = guidata(fighdata.loadSettingsFig); %load settings data
editLoadSettings=get(loadSettingsData.editLoadSettings,'String');
%set new settings file
editLoadSettings = testFile;
set(loadSettingsData.editLoadSettings,'String',editLoadSettings);
editLoadSettings=get(loadSettingsData.editLoadSettings,'String')%check
%OPEN SETTINGS FILE
%loadSettings('pushbuttonOpen_Callback',fighdata.loadSettingsFig,[],guidata(fighdata.loadSettingsFig))
loadSettings('pushbuttonOpen_Callback',loadSettingsData.pushbuttonOpen,[],guidata(fighdata.loadSettingsFig))
%RUN SIMULATION
numSimsHandles = fighdata.popupmenuNumSims;
%pushbuttonRunSimulations_Callback(hObject, eventdata, handles)
%hobject = pushbuttonRunSimulations: 332.0018
disp('batchmode pushbuttonrunSims')
fighdata.pushbuttonRunSimulations;
DiversityModelGUI('pushbuttonRunSimulations_Callback',fighdata.pushbuttonRunSimulations,[],fighdata)
end