%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Jennifer Jackson 2017                                                        %
% A Biodiversity Approach to Cyber Security                                    %
% PhD Complexity Science                                                       %
% University of Warwick                                                        %
% http://wrap.warwick.ac.uk/98458                                              %
% This software is distributed under the  Creative Commons Attribution License %
% https://creativecommons.org/licenses/by/4.0/                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
classdef MeasureDiversityClass
%*************************************************************************
%functions to support diversity measurements
%*************************************************************************

    properties
    end
    
    methods
    end
    %static methods that can be called without instantiating a class    
    methods (Static)
 %***********************************************************************
 % Function to get handle data and include in the model
 %***********************************************************************
 function MeasDivSimInit(handles,ihandles,ohandles)
     ihandlesnumRunsPerSim = ihandles.numRunsPerSim;
     ihandlesnumTimeSteps = ihandles.numTimeSteps;
      [ohandles.Nei_sol1 ohandles.Nei_sol2 ohandles.Nei_sol3 ohandles.Nei_sol4 ohandles.Nei_sol5] = MeasureDiversityClass.initSimDivOutputs(ihandles.numRunsPerSim, ihandles.numTimeSteps); 
      [ohandles.neiIndexSingleLocus_sol1 ohandles.neiIndexSingleLocus_sol2 ohandles.neiIndexSingleLocus_sol3 ohandles.neiIndexSingleLocus_sol4 ohandles.neiIndexSingleLocus_sol5] = MeasureDiversityClass.initSimDivOutputsLocus(ihandles.numRunsPerSim, ihandles.numTimeSteps,ihandles.numLoci);  
        numVariants = ihandles.len_apps + ihandles.len_serv + ihandles.len_os + ihandles.len_dr 
      [ohandles.GenoSim_sol4 ohandles.GenoSim_sol5] = MeasureDiversityClass.initSimGenoOutputs(ihandles.numRunsPerSim, ihandles.numTimeSteps, ihandles.numNodes,numVariants); %added 31st March 2016
 end
 function [Nei_sol1 Nei_sol2 Nei_sol3 Nei_sol4 Nei_sol5] = initSimDivOutputs(numRunsPerSim, numTimeSteps)
   %diversity index
  % Nei_sol1 = zeros(numRunsPerSim,numTimeSteps); 
  % Nei_sol2 = zeros(numRunsPerSim,numTimeSteps); 
  % Nei_sol3 = zeros(numRunsPerSim,numTimeSteps); 
   Nei_sol1 = 0;    %NOT USED
   Nei_sol2 = 0;   %NOT USED
   Nei_sol3 = 0;   %NOT USED
   Nei_sol4 = zeros(numRunsPerSim,numTimeSteps); 
   Nei_sol5 = zeros(numRunsPerSim,numTimeSteps); 
 end
  function [GenoSim_sol4 GenoSim_sol5] = initSimGenoOutputs(numRunsPerSim, numTimeSteps,numNodes,numVariants)
   %diversity index
   %GenoSim_sol1 = zeros(numRunsPerSim,numTimeSteps,numNodes,numVariants); 
   %GenoSim_sol2 = zeros(numRunsPerSim,numTimeSteps,numNodes,numVariants); 
   %GenoSim_sol3 = zeros(numRunsPerSim,numTimeSteps,numNodes,numVariants); 
   GenoSim_sol4 = zeros(numRunsPerSim,numTimeSteps,numNodes,numVariants); 
   GenoSim_sol5 = zeros(numRunsPerSim,numTimeSteps,numNodes,numVariants);   
 end
 function [SingleLocus_sol1 SingleLocus_sol2 SingleLocus_sol3 SingleLocus_sol4 SingleLocus_sol5] = initSimDivOutputsLocus(numRunsPerSim, numTimeSteps,numLoci)   
    %SingleLocus_sol1 = zeros(numRunsPerSim,numTimeSteps,numLoci);
    %SingleLocus_sol2 = zeros(numRunsPerSim,numTimeSteps,numLoci);
    %SingleLocus_sol3 = zeros(numRunsPerSim,numTimeSteps,numLoci);
      SingleLocus_sol1 = 0;%NOT USED
    SingleLocus_sol2 = 0%NOT USED
    SingleLocus_sol3 = 0;%NOT USED
    
    SingleLocus_sol4 = zeros(numRunsPerSim,numTimeSteps,numLoci);
    SingleLocus_sol5 = zeros(numRunsPerSim,numTimeSteps,numLoci);
 end
 function MeasDivRunInit(ihandles,rhandles,ohandles)
     %run initialisations
 end
 
 function [ neiIndex neiIndexSingleLocus ] = getNeiIndex( genotypeMatrix, numAlleles )
    %calculates the nei index
    %genotypeMatrix rows:number of nodes, columns:number of alleles in 1 loci
    %test
    %genotypeMatrix = zeros(10,26);
    %for i = 1:8
    %genotypeMatrix(i,i) = 1;
    %genotypeMatrix(i,i+8) = 1;
    %genotypeMatrix(i,i+18) = 1;
    %end
    %genotypeMatrix(1:5,17) = 1;
    %genotypeMatrix(6:10,18) = 1;
    %genotypeMatrix(9:10,1) = 1;
    %genotypeMatrix(9:10,9) = 1;
    %genotypeMatrix(9:10,26) = 1;
    %genotypeMatrix
    %numAlleles = [8 8 2 8]

    %code
    %get start and end points of loci
    [startP endP numLoci] = MeasureDiversityClass.getStartAndEndLoci(numAlleles);
    neiIndexSingleLocus= zeros(1,numLoci);
    for j=1:numLoci
        neiIndexSingleLocus(j) = MeasureDiversityClass.getNeiIndexSingleLocus( genotypeMatrix(:,startP(j):endP(j)));
    end
    neiIndex = sum(neiIndexSingleLocus)./numLoci;
 end
 function [ neiIndexSingleLocus ] = getNeiIndexSingleLocus( genotypeMatrix )
    %calculates the nei index
    %genotypeMatrix rows:number of nodes, columns:number of alleles in 1 loci
    %test
    %genotypeMatrix = zeros(10,8);
    %for i = 1:8
    %genotypeMatrix(i,i) = 1;
    %end
    %genotypeMatrix(9:10,1) = 1;
    %genotypeMatrix
    %code
    [numPop numAlleles] = size(genotypeMatrix);
    frequencies = (sum(genotypeMatrix))./numPop;
    sqFreq = frequencies.*frequencies;
    Ps = sum(sqFreq);
    neiIndexSingleLocus = 1-Ps;

 end
 function [ startP endP numLoci] = getStartAndEndLoci( numAlleles )
    %numAlleles = [8 8 2 8]
    numLoci = size(numAlleles,2);
    endPi = zeros(1,numLoci+1);
    endP = zeros(1,numLoci);
    startP = zeros(1,numLoci);
    for i = 1:numLoci
        endPi(i+1) = endPi(i)+numAlleles(i);
        startP(i) = endPi(i)+1;
    end
    endP(1,:) = endPi(2:end);
 end

 
    end %static
end %class