%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Jennifer Jackson 2017                                                        %
% A Biodiversity Approach to Cyber Security                                    %
% PhD Complexity Science                                                       %
% University of Warwick                                                        %
% http://wrap.warwick.ac.uk/98458                                              %
% This software is distributed under the  Creative Commons Attribution License %
% https://creativecommons.org/licenses/by/4.0/                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
classdef NodeEncounterClass
    %functions to support node encounters, includes random encounters and
    %random waypoint
    properties
    end
    
    methods
    end
    methods (Static)
        function nodeEncounterSimInit(mainhandles,ihandles)
           %get fixed encounter probability
            NodeEncounterClass.getFixedEncounterProb(mainhandles, ihandles);   
        end
        function nodeEncounterRunInit(ihandles,rhandles)
            rhandles.randomEncounter = zeros(ihandles.numNodes,1);
            rhandles.encounterYes  = zeros(ihandles.numNodes,1);
            rhandles.sumEncounterYesM = zeros(ihandles.numTimeSteps,ihandles.numNodes); %full time matrix need to set this properly
        end
           %function probabilityDistr  = getFixedEncounterProb(handles, numNodes, probabilityDistrOld)
    function getFixedEncounterProb(mainhandles,ihandles)    
       checkboxNetworkModelEnable = get(mainhandles.checkboxNetworkModelEnable,'Value');
       popupmenuNetworkModelV = get(mainhandles.popupmenuNetworkModel,'Value');
        handles = guidata(mainhandles.networkModelParameters_randomEncounterFig); %load data
       %if or(checkboxNetworkModelEnable == 0, and(checkboxNetworkModelEnable == 1 ,popupmenuNetworkModel > 1))
       networkModelS = GUIClass.getNetworkModelString(popupmenuNetworkModelV)
        if and(checkboxNetworkModelEnable,strcmp(networkModelS,'Random Encounter'))
       %probability of an encounter
       ihandles.probabilityDistr = zeros(ihandles.numNodes,1);
       %probability of an encounter
       %ihandles.probabilityDistr(:) = 0.5;%currently fixed, could be stochastic 
        ihandles.probabilityDistr(:) = get(handles.editRandomEncounterProb, 'Value');%
       else
         %probabilityDistr = probabilityDistrOld  
       end
   end
        function nodeEncounter(t,ihandles,rhandles)
        %*********************************************************************
        %Node encounter
        %*********************************************************************
        %choose a uniform random number to decide whether to meet another node
        if and(ihandles.networkModelEnable,or(strcmp(ihandles.networkModelS,'Random Encounter'),strcmp(ihandles.networkModelS,'Wireless Bluetooth')));
        randomNumber = rand(ihandles.numNodes,1);
        %encounter a node if less than probability number
        %(randomNumber < ihandles.probabilityDistr) equals 1 if true
        rhandles.encounterYes = (randomNumber < ihandles.probabilityDistr);
        %generate a random node to encounter , but must not encounter itself
        rhandles.randomEncounter = randi([1 ihandles.numNodes],ihandles.numNodes,1);
        for i = 1:ihandles.numNodes
            while rhandles.randomEncounter(i) == i
             rhandles.randomEncounter(i) = randi(ihandles.numNodes,1);
            end;
        end;
                        
         %added to keep track of number of encounters - Test
         %only
        rhandles.sumEncounterYesM(t,:) = rhandles.encounterYes';
        rhandles.sumEncounterYes = sum(rhandles.sumEncounterYesM,1);
        %test
        %displayencounterYes = rhandles.encounterYes
        %displayrandomEncounter = rhandles.randomEncounter
        %displaysumM = rhandles.sumEncounterYesM;
        %displaysum = rhandles.sumEncounterYes
        end %if
        end; %func
        
        
        %***************************************************************
        %Random waypoint functions
        %***************************************************************
        %Initialisation functions*****************************************
        function RWPSimInit(mainhandles, ihandles)   
            %function for initialising for a simulation
            %1.get main handles data
            ihandles.s_input.NB_NODES = ihandles.numNodes;
            ihandles.s_input.SIMULATION_TIME = get(mainhandles.editNumTimeSteps, 'Value');  
            %2.get data from RWP GUI handles
            handles = guidata(mainhandles.networkModelParameters_randomWaypointFig); %load data
            ihandles.PlotWayPoints = get(handles.checkboxPlotWayPoints, 'Value');
            popupmenuRWPTxRadiusV = get(handles.popupmenuRWPTxRadius,'Value');
            ihandles.RWPrange = GUIClass.getTxRadius(popupmenuRWPTxRadiusV);
            popupmenuRWPEncounterChoiceV = get(handles.popupmenuRWPEncounterChoice,'Value');
            ihandles.RWPmode = GUIClass.getRWPmodeStr(popupmenuRWPEncounterChoiceV);
            editRWPPosXminV = get(handles.editRWPPosXmin,'Value');
            editRWPPosXmaxV = get(handles.editRWPPosXmax,'Value');
            editRWPPosYminV = get(handles.editRWPPosYmin,'Value');
            editRWPPosYmaxV = get(handles.editRWPPosYmax,'Value');
            editRWPSpeedminV = get(handles.editRWPSpeedmin,'Value');
            editRWPSpeedmaxV = get(handles.editRWPSpeedmax,'Value');
            editRWPPauseminV = get(handles.editRWPPausemin,'Value');
            editRWPPausemaxV = get(handles.editRWPPausemax,'Value');
            editRWPWalkTimeminV = get(handles.editRWPWalkTimemin,'Value');
            editRWPWalkTimemaxV = get(handles.editRWPWalkTimemax,'Value');
            editRWPDirectionminV = get(handles.editRWPDirectionmin,'Value');
            editRWPDirectionmaxV = get(handles.editRWPDirectionmax,'Value');
            %3.set random waypoint parameters
            ihandles.s_input.V_POSITION_X_INTERVAL = [editRWPPosXminV editRWPPosXmaxV];
            ihandles.s_input.V_POSITION_Y_INTERVAL = [editRWPPosYminV editRWPPosYmaxV];
            ihandles.s_input.V_SPEED_INTERVAL = [editRWPSpeedminV editRWPSpeedmaxV];
            ihandles.s_input.V_PAUSE_INTERVAL = [editRWPPauseminV editRWPPausemaxV];
            ihandles.s_input.V_WALK_INTERVAL = [editRWPWalkTimeminV editRWPWalkTimemaxV];
            ihandles.s_input.V_DIRECTION_INTERVAL = [editRWPDirectionminV editRWPDirectionmaxV];
            %4.compute random waypoints if enabled
            NodeEncounterClass.RWPComputeRandomWayPoints(ihandles);
        end %func
        function RWPRunInit(ihandles,rhandles)
            %function for initialising 1 run of a simulation
            if and(ihandles.networkModelEnable,strcmp(ihandles.networkModelS,'Random Waypoint'));
            rhandles.RWPSumEncounterYesM = zeros(ihandles.numTimeSteps,ihandles.numNodes);
            rhandles.RWPNodeTagged = zeros(4,ihandles.numNodes); %1 endTime 2 tag 3 nodeNumberTagged 4 virusTransmitted -difficult to do matrix operations with struct
            for nodeNumber = 1: ihandles.numNodes
                rhandles.RWPNodeTagged(rhandles.RWPnodeNumberTagged,nodeNumber) = nodeNumber;
            end
            %Independent geno-exploit alg
            rhandles.endTimeTxGenotype = zeros(1,ihandles.numNodes);
            rhandles.endTimeTxExploit = zeros(1,ihandles.numNodes);
            rhandles.endTimeTimeOutTxGenotype = zeros(ihandles.numNodes,ihandles.numNodes);
            rhandles.tagState = ones(1,ihandles.numNodes);
            rhandles.nodesTagged = zeros(ihandles.numNodes,ihandles.numNodes);
            rhandles.txGenotypeEncounterYes = zeros(1,ihandles.numNodes);
            rhandles.txExploitEncounterYes = zeros(1,ihandles.numNodes);   
            end %if
            
        end
        function RWPComputeRandomWayPoints(ihandles)
            %function for precomputing the random waypoints ready for a
            %simulation
            %check if enabled
            if and(ihandles.networkModelEnable,strcmp(ihandles.networkModelS,'Random Waypoint'));
                disp('computing random waypoints.....')
                tic
                s_input = ihandles.s_input;
                s_mobility = Generate_Mobility(s_input);
                timeStep = 1 ; %0.1;%(s) %time interval originally set at 0.1
                [vs_node] = test_Animate(s_mobility,s_input,timeStep,ihandles.PlotWayPoints);
                %save precomputed x and y waypoint positions
                for nodeIndex = 1:s_mobility.NB_NODES
                    ihandles.vs_node(nodeIndex).v_x = vs_node(nodeIndex).v_x;
                    ihandles.vs_node(nodeIndex).v_y = vs_node(nodeIndex).v_y;
                end           
                disp('finished computing random waypoints.....')
                toc
            else
                %zero waypoint data
                simTime = ihandles.s_input.SIMULATION_TIME+1
                vxy = zeros(1,simTime);
                vs_nodeT.vxy = vxy;
                for nodeIndex = 1:ihandles.numNodes
                    ihandles.vs_node(nodeIndex).v_x = vs_nodeT.vxy;
                    ihandles.vs_node(nodeIndex).v_y = vs_nodeT.vxy;
                end  
            end %if
        end %func
        %Main Random Waypoint Function************************************
        function RWPNodeEncounterTloop(t,lengthCounter,ihandles,rhandles)
            %check if enabled
            if and(ihandles.networkModelEnable,strcmp(ihandles.networkModelS,'Random Waypoint'));
            %1.get the node positions for one time step
                for nodeIndex = 1:ihandles.s_input.NB_NODES
                    nodes.x(nodeIndex) = ihandles.vs_node(nodeIndex).v_x(t);
                    nodes.y(nodeIndex) = ihandles.vs_node(nodeIndex).v_y(t);
                end
            %2.find nodes in range and determine if there will be an encounter and/or exploit transmission
              %identify RWP mode
            if or(strcmp(ihandles.RWPmode,'Nearest in range'),strcmp(ihandles.RWPmode,'Random in range'))
                [RWPencounterYes,RWPnodesEncountered] = NodeEncounterClass.getNearestNode(nodes,ihandles.RWPrange,ihandles.RWPmode);
                %assign outputs correctly
                rhandles.RWPencounterYes = RWPencounterYes'; %only used in the random waypoint functions
                rhandles.RWPnodesEncountered = RWPnodesEncountered'; %only used in the random waypoint functions
                %rhandles.RWPSumEncounterYesM(t,:) = RWPencounterYes(1,:); %used to count number of encounters during the simulation
                rhandles.RWPSumEncounterYesM(lengthCounter,:) = RWPencounterYes(1,:); %used to count number of encounters during the simulation
                rhandles.RWPSumEncounterYes = sum(rhandles.RWPSumEncounterYesM,1);
                rhandles.randomEncounter = rhandles.RWPnodesEncountered; %used by the genotype diversity algorithm, used by the checkExploitRoute function
                rhandles.encounterYes = rhandles.RWPencounterYes;  %used by the genotype diversity algorithm
                %checkExploitRoute takes in encounterYes as a variable which can be set in attackModelRunTloop if saved first as
                %a rhandles variable
            elseif or(strcmp(ihandles.RWPmode,'Available in range'),strcmp(ihandles.RWPmode,'Available in range with a virus tx time'))
                [RWPencounterYes,RWPnodesEncountered] = NodeEncounterClass.getNearestNodeTagged(t,ihandles,rhandles,nodes,ihandles.RWPrange,ihandles.RWPmode);
                %assign outputs correctly
                rhandles.RWPencounterYes = RWPencounterYes'; %only used in the random waypoint functions
                rhandles.RWPnodesEncountered = RWPnodesEncountered'; %only used in the random waypoint functions
                %rhandles.RWPSumEncounterYesM(t,:) = RWPencounterYes(1,:); %used to count number of encounters during the simulation
                rhandles.RWPSumEncounterYesM(lengthCounter,:) = RWPencounterYes(1,:); %used to count number of encounters during the simulation
                rhandles.RWPSumEncounterYes = sum(rhandles.RWPSumEncounterYesM,1);
                rhandles.randomEncounter = rhandles.RWPnodesEncountered; %used by the genotype diversity algorithm, used by the checkExploitRoute function
                rhandles.encounterYes = rhandles.RWPencounterYes;  %used by the genotype diversity algorithm
                %checkExploitRoute takes in encounterYes as a variable which can be set in attackModelRunTloop if saved first as
                %a rhandles variable
            elseif strcmp(ihandles.RWPmode,'Independent geno-exploit')
                %disp('running Independent geno-exploit RWP encounter option ....')
                %tD = t %testing only
                [txGenotypeEncounterYes,txExploitEncounterYes,encounterNodes] = NodeEncounterClass.getIndGenoExp(t,ihandles,rhandles,nodes,ihandles.RWPrange,ihandles.RWPmode);
                %assign outputs correctly
                %encounter nodes is currently a matrix and needs turning into a number array
                [value index] = sort(encounterNodes);
                sortedAndReducedEncounterNodes =  value(end,:).*index(end,:);
                %number array should have no zeros in it for genotype alg,encounterYes used as enable
                zerosToOnes = sortedAndReducedEncounterNodes<1;
                encounters = sortedAndReducedEncounterNodes+zerosToOnes;
                rhandles.randomEncounter = encounters'; %used by the genotype diversity algorithm, used by the checkExploitRoute function
                rhandles.encounterYes = txGenotypeEncounterYes';  %used by the genotype diversity algorithm
                %checkExploitRoute takes in encounterYes as a variable which can be set in attackModelRunTloop if saved first as
                %an rhandles variable: txExploitEncounterYes is saved as rhandles.txExploitEncounterYes
                %randomEncounterD = rhandles.randomEncounter %testing only
                %encounterYesD = rhandles.encounterYes %testing only
            end
            end %if
        end %func
        %Functions for 'Random in range' and Nearest in range'*************
        function [encounterYes,nodesEncountered] = getNearestNode(nodes,range,mode)
            %get a node to encounter
            %1.First work out which nodes are actually in range
            [nodesInRange,nodesInRangeNumberM,numNodesInRange,nodesInRangeFlag,nearestNodes,randomNodes] = NodeEncounterClass.getNodesInRangePlus(nodes,range);
            %2.Then decide which node to actually encounter, this can be either the 1.nearest or 2.random one in range       
            [encounterYes,nodesEncountered] = NodeEncounterClass.getNodesEncounter(mode,nodesInRange,numNodesInRange,nodesInRangeFlag,nearestNodes,randomNodes);
        end %func
        function [nodesInRange,nodesInRangeNumberM,numNodesInRange,nodesInRangeFlag,nearestNodes,randomNodes] = getNodesInRangePlus(nodes, range)
            %get nodes in range
            [sqrDistance,nodesInRange,nodesInRangeShift,nodesInRangeNumberM,sortNodesInRangeNumber,numNodesInRange,nodesInRangeFlag] = NodeEncounterClass.getNodesInRange(nodes,range);
            lenx = size(nodes.x,2);
            %Get nearest node
            [nearestNodesValue, nearestNodesIndex] = min(sqrDistance(1:end-1,:));
            %need to get in correct format
            nearestNodes = NodeEncounterClass.formatNodesIndex(nearestNodesIndex,lenx);
            %Choose a random node in range
            randomNumberMatrix = rand(lenx,lenx);
            MaskedRandomNumberMatrix = randomNumberMatrix.*nodesInRange;
            [randomNodesValue,randomNodesIndex] = max(MaskedRandomNumberMatrix);
            %need to get in correct format
            randomNodes = NodeEncounterClass.formatNodesIndex(randomNodesIndex,lenx);
        end %func
        function [encounterYes,nodesEncountered] = getNodesEncounter(mode,nodesInRange,numNodesInRange,nodesInRangeFlag,nearestNodes,randomNodes)
            %choose selection process
            if strcmp(mode, 'Nearest in range')
                %disp('Nearest in range')
                %choose nearest node if in range
                encounterYes = nodesInRangeFlag;
                %nodesEncountered = nearestNodes.*nodesInRangeFlag;
                nodesEncountered = nearestNodes;
            elseif strcmp(mode,'Random in range')
                 %disp('Random in range')
            %choose random nearest node if more than 1 in range
            encounterYes = nodesInRangeFlag;
            %nodesEncountered = randomNodes.*nodesInRangeFlag;
            nodesEncountered = randomNodes;
            end
        end %func
        %Functions for 'Available in range' and 'Available in range with a virus tx time *******************************
        function [encounterYes,nodesEncountered] = getNearestNodeTagged(t,ihandles,rhandles,nodes,range,mode)
            %get a node to encounter
            %1.First work out which nodes are actually in range
            [sqrDistance,nodesInRange,nodesInRangeShift,nodesInRangeNumberM,sortNodesInRangeNumber,numNodesInRange,nodesInRangeFlag] = NodeEncounterClass.getNodesInRange(nodes,range);
            %2.Check which nodes have been tagged, which can send exploit info, and which are just an encounter with genotype information
            if strcmp(mode, 'Available in range with a virus tx time')
                %disp('running checkTagging')
                [identNewNode] = NodeEncounterClass.checkTagging(t,rhandles,nodesInRange,nodesInRangeNumberM,numNodesInRange,nodesInRangeFlag);
            elseif strcmp(mode, 'Available in range')
                 identNewNode = ones(1,ihandles.numNodes); %all nodes should check for a new node to encounter
            end
            %3.dont choose nodes that are already tagged, or chosen by another node, - only choose those that are available for communication
            [encounterYes nodesEncountered] = NodeEncounterClass.getAvailableNodes(rhandles,nodesInRangeNumberM,identNewNode);
            %4.tag new nodes encountered
            if strcmp(mode, 'Available in range with a virus tx time')
                %disp('running addTagging')
                NodeEncounterClass.addTagging(ihandles,rhandles,t,encounterYes,nodesEncountered);
            end
            %testing
            tD=t;
            %disp('output of getNearestNodeTagged') 
            disptag = rhandles.RWPNodeTagged(rhandles.RWPtag,:);
            dispvirustx = rhandles.RWPNodeTagged(rhandles.RWPvirusTransmitted,:);
            dispnumber = rhandles.RWPNodeTagged(rhandles.RWPnodeNumberTagged,:);
            dispendTime = rhandles.RWPNodeTagged(rhandles.RWPendTime,:);
        end %func
        function [identNewNode] = checkTagging(t,rhandles,nodesInRange,nodesInRangeNumberM,numNodesInRange,nodesInRangeFlag)
            numNodes = size(nodesInRange,2)
            %reset tx virus flag
            rhandles.RWPNodeTagged(rhandles.RWPvirusTransmitted,:) = 0;
            identNewNode = zeros(1,numNodes);
            %1.compare node numbers in matrix against tagged numbers(node tagged to be in range previously)
            for nodeNumber = 1: numNodes
                %need and in here somewhere to check if tagged and check end time
                match(:,nodeNumber) = (nodesInRangeNumberM(:,nodeNumber) == rhandles.RWPNodeTagged(rhandles.RWPnodeNumberTagged,nodeNumber)); %number matches , need and here to and the tag
                matchSum(nodeNumber) = sum(match(:,nodeNumber)); %match against number
                %if tagged and same node still in range and >= end time, release tag and send virus
                option1 = and(rhandles.RWPNodeTagged(rhandles.RWPtag,nodeNumber),and(matchSum(nodeNumber),t >= rhandles.RWPNodeTagged(rhandles.RWPendTime,nodeNumber)));
                if and(rhandles.RWPNodeTagged(rhandles.RWPtag,nodeNumber),and(matchSum(nodeNumber),t >= rhandles.RWPNodeTagged(rhandles.RWPendTime,nodeNumber)))
                    %disp('release tag and send virus')
                    %nodeNumber
                    %rhandles.RWPNodeTagged(rhandles.RWPtag,nodeNumber);
                    rhandles.RWPNodeTagged(rhandles.RWPtag,nodeNumber) = 0;
                    %rhandles.RWPNodeTagged(rhandles.RWPvirusTransmitted,nodeNumber);
                    rhandles.RWPNodeTagged(rhandles.RWPvirusTransmitted,nodeNumber) = 1;
                %elseif tagged and same node NOT still in range, release tag but don't send virus,identify new node in range and tag
                option2 = and(rhandles.RWPNodeTagged(rhandles.RWPtag,nodeNumber),matchSum(nodeNumber)==0);
                elseif and(rhandles.RWPNodeTagged(rhandles.RWPtag,nodeNumber),matchSum(nodeNumber)==0)
                    %disp('release tag, identify a new node in range and tag')
                    %nodeNumber
                    rhandles.RWPNodeTagged(rhandles.RWPtag,nodeNumber);
                    rhandles.RWPNodeTagged(rhandles.RWPtag,nodeNumber) = 0;
                    identNewNode(nodeNumber) = 1;
                %elseif not tagged identify new node in range and tag
                option3 = rhandles.RWPNodeTagged(rhandles.RWPtag,nodeNumber)==0;
                elseif (rhandles.RWPNodeTagged(rhandles.RWPtag,nodeNumber)==0)
                    %disp('identify a new node in range and tag')
                    %nodeNumber
                    identNewNode(nodeNumber) = 1;
                else
                    %disp('keep tag and do nothing')
                    %nodeNumber
                end %if
            end %for
            %disp('output of check tagging') %testing
            %match
            %matchSum   
            %disptag = rhandles.RWPNodeTagged(rhandles.RWPtag,:)
            %dispvirustx = rhandles.RWPNodeTagged(rhandles.RWPvirusTransmitted,:)
            %dispnumber = rhandles.RWPNodeTagged(rhandles.RWPnodeNumberTagged,:)
            %dispendTime = rhandles.RWPNodeTagged(rhandles.RWPendTime,:)
        end %func
        function [chosenFlag chosenNodes] = getAvailableNodes(rhandles,nodesInRangeNumberM,identNewNode)
            numNodes = size(nodesInRangeNumberM,2);
            %exclude nodes that are already tagged from the selection list
            %so dont have transmissions at the same time
            %init
            taggedNodesOrdered = zeros(1,numNodes); %indicates which nodes are tagged (including those already), arranged in order  [node1 node2 ... noden] 1-tagged 0-untagged
            chosenNodes = ones(1,numNodes); %node number chosen to be encountered - nodeEncounter
            chosenFlag = zeros(1,numNodes); %flag indicating a node has been chosen - encounterYes
            %code
            maskedTagged = rhandles.RWPNodeTagged(rhandles.RWPnodeNumberTagged,:) .* rhandles.RWPNodeTagged(rhandles.RWPtag,:);
            %1.sort nodes in range for processing later
            sortNodesInRangeNumberM = sort(nodesInRangeNumberM);
            %2.arrange tagged nodes in order
             for i = 1: numNodes
                if maskedTagged(i) > 0    
                taggedNodesOrdered(maskedTagged(i)) = 1;
                end
             end
            %3.scroll through the nodes in range and select one that is not
            %already tagged, if a node is selected, tag it, to stop others
            %from selecting it at the same time
            for i = 1:numNodes
                if identNewNode(i) ==1 %only do if need to encounter another node
                    num = numNodes;
                    while and(and(sortNodesInRangeNumberM(num,i) > 0 ,num > 0),chosenFlag(1,i)==0) %scroll through the nodes in range only
                        if taggedNodesOrdered(sortNodesInRangeNumberM(num,i)) == 0;  %select only if not already tagged
                            chosenNodes(1,i) = sortNodesInRangeNumberM(num,i);
                            taggedNodesOrdered(sortNodesInRangeNumberM(num,i)) = 1;
                            chosenFlag(1,i)=1;
                        end
                        num = num-1;
                    end
                end
            end
           % taggedNodesOrderedD=taggedNodesOrdered %testing
           % chosenNodesD=chosenNodes %testing
           % chosenFlagD=chosenFlag  %testing
           % AllNodesNumber = maskedTagged + chosenNodes.*chosenFlag
           % AllNodesTagged = rhandles.RWPNodeTagged(rhandles.RWPtag,:) + chosenFlag 
        end %end func
        function addTagging(ihandles,rhandles,t,encounterYes,nodesEncountered)
            %add tagging to new nodes in range, whilst preserving current tagging status of unreleased nodes
            oldTag = rhandles.RWPNodeTagged(rhandles.RWPtag,:)
            timeToTransmitVirus = ihandles.timeToTxVirus .* GUIClass.convertTimeUnits(ihandles.timeToTxVirusUnits)
            %set tag, node number and end time
            rhandles.RWPNodeTagged(rhandles.RWPtag,:)= oldTag + encounterYes;
            rhandles.RWPNodeTagged(rhandles.RWPnodeNumberTagged,:)= rhandles.RWPNodeTagged(rhandles.RWPnodeNumberTagged,:).* rhandles.RWPNodeTagged(rhandles.RWPvirusTransmitted,:) + rhandles.RWPNodeTagged(rhandles.RWPnodeNumberTagged,:).* oldTag + nodesEncountered.*encounterYes;
            rhandles.RWPNodeTagged(rhandles.RWPendTime,:)= rhandles.RWPNodeTagged(rhandles.RWPendTime,:).* oldTag + (t+timeToTransmitVirus).*encounterYes;
        end
        %***********Functions for 'Independent geno-exploit'***************
        function [txGenotypeEncounterYes,txExploitEncounterYes,encounterNodes] = getIndGenoExp(t,ihandles,rhandles,nodes,range,mode)
            %get a node to encounter
            %1.First work out which nodes are actually in range
            [sqrDistance,nodesInRange,nodesInRangeShift,nodesInRangeNumberM,sortNodesInRangeNumber,numNodesInRange,nodesInRangeFlag] = NodeEncounterClass.getNodesInRange(nodes,range);
            %sortNodesInRangeNumber %testing only
            %2.Do state machine
            [txGenotypeEncounterYes,txExploitEncounterYes,encounterNodes] = NodeEncounterClass.getNextStateTagState(t,ihandles,rhandles,sortNodesInRangeNumber,nodesInRangeShift);
            %3.Work out encounter outputs
        
        end
        function [txGenotypeEncounterYes,txExploitEncounterYes,encounterNodes] = getNextStateTagState(t,ihandles,rhandles,sortNodesInRangeNumber,nodesInRangeShift)
            %***************key pre-statemachine checks********************************
            [newNodeIdentified chosenNodes] = NodeEncounterClass.getAvailableNodesInRange(ihandles.numNodes,rhandles.nodesTagged,sortNodesInRangeNumber);
            %check if tagged still in range
            [taggedNodeStillInRange] = NodeEncounterClass.checkTaggedInRange(rhandles.nodesTagged,nodesInRangeShift);
            %check if tagged out of range
            [taggedNodeOutOfRange] = NodeEncounterClass.checkTaggedOutOfRange(rhandles.nodesTagged,nodesInRangeShift);
            %check if taggedNode - endTimeTimeOutTxGenotype is in reset
            [taggedNodeTimeOutReset] = NodeEncounterClass.checkTimeOutTxGenotype(rhandles.nodesTagged,rhandles.endTimeTimeOutTxGenotype);
            %check  tagged node has not reached end time of endTimeTimeOutTxGenotype
            activeEndTimeTimeOutTxGenotype = NodeEncounterClass.checkTimeOutTxGenotypeNotReached(rhandles.nodesTagged,rhandles.endTimeTimeOutTxGenotype);

            %tagState statemachine
            %********************************************************************************
            %state 1 not tagged
            %**********************************************************************************
            %1.check if in state '1 - not tagged' AND new nodes to tag -> change to state '2 - tagged'
            %set endtxgenotype and endtxexploit
            changeState_NotTaggedToTagged = and((rhandles.tagState==1),(newNodeIdentified));%mask of nodes to change
            nextState_NotTaggedToTagged = changeState_NotTaggedToTagged*2; %2 means state 2
            %********************************************************************************
            %state 2 tagged
            %**********************************************************************************
            %1. still in range and endTimeTimeOutTxGenotype ==0 and reached endTimeTxgenotype-> change to state 3 - tx genotype
            changeState_TaggedToTxGenotype = and(and(and((rhandles.tagState==2),(taggedNodeStillInRange)),taggedNodeTimeOutReset),rhandles.endTimeTxGenotype<=t);
            nextState_TaggedToTxGenotype = changeState_TaggedToTxGenotype.*3; %3 means state 3
            %2.  still in range and NOT reached endTimeTimeOutTxGenotype, but reached endTimeTxexploit-> change to state 4 - tx exploit
            changeState_TaggedToTxExploit = and(and(and((rhandles.tagState==2),(taggedNodeStillInRange)),activeEndTimeTimeOutTxGenotype),rhandles.endTimeTxExploit<=t);
            nextState_TaggedToTxExploit = changeState_TaggedToTxExploit.*4; %4 means state 4
            %3.  -> change to state 1 - tagged node out of range
            changeState_TaggedToNotTagged = and((rhandles.tagState==2),(taggedNodeOutOfRange));%mask of nodes to change
            releaseTags__TaggedToNotTagged = repmat((changeState_TaggedToNotTagged),[ihandles.numNodes,1]);
            nextState_TaggedToNotTagged = changeState_TaggedToNotTagged.*1; %1 means state 1
            %********************************************************************************
            %state 3 tagged & tx genotype
            %**********************************************************************************
            %1. still in range and reached endTimeTxExploit-> change to state 4 - txExploit
            changeState_TxGenotypeToTxExploit = and(and((rhandles.tagState==3),(taggedNodeStillInRange)),rhandles.endTimeTxExploit<=t);
            nextState_TxGenotypeToTxExploit = changeState_TxGenotypeToTxExploit.*4; %4 means state 4
            %2.  -> change to state 1 - tagged node out of range
            changeState_TxGenotypeToNotTagged = and((rhandles.tagState==3),(taggedNodeOutOfRange));%mask of nodes to change
            releaseTags__TxGenotypeToNotTagged = repmat((changeState_TxGenotypeToNotTagged),[ihandles.numNodes,1]);
            nextState_TxGenotypeToNotTagged = changeState_TxGenotypeToNotTagged.*1; %1 means state 1
            %********************************************************************************
            %state 4 tagged & tx exploit
            %**********************************************************************************
            %1.-> change to state 1 - All change after tx exploit tagged out of range
            changeState_TxExploitToNotTagged = (rhandles.tagState==4);%mask of nodes to change
            releaseTags__TxExploitToNotTagged = repmat((changeState_TxExploitToNotTagged),[ihandles.numNodes,1]);
            nextState_TxExploitToNotTagged = changeState_TxExploitToNotTagged.*1; %1 means state 1
            %*********************************************************************************
            %work out next state
            %*********************************************************************************
            NOTchangeState = (changeState_NotTaggedToTagged + changeState_TaggedToTxGenotype + changeState_TaggedToTxExploit + ...
                            changeState_TaggedToNotTagged + changeState_TxGenotypeToTxExploit + changeState_TxGenotypeToNotTagged + ...
                            changeState_TxExploitToNotTagged -1).*(-1);%inv of change state
            nextState_NoChange = NOTchangeState.*rhandles.tagState;
            nextStateNumber = nextState_NotTaggedToTagged + nextState_TaggedToTxGenotype + nextState_TaggedToTxExploit + ...
                            + nextState_TaggedToNotTagged + nextState_TxGenotypeToTxExploit + nextState_TxGenotypeToNotTagged + ...
                            nextState_TxExploitToNotTagged + nextState_NoChange;
            %*********************************************************************************
            %Post statemachine set new variables 
            %*********************************************************************************
            %update genotype time out
            timeOutPeriod = ihandles.timeOutPeriodTxGenotype.*GUIClass.convertTimeUnits(ihandles.timeOutPeriodTxGenotypeUnits);
            rhandles.endTimeTimeOutTxGenotype = NodeEncounterClass.setTimeOut(ihandles.numNodes,t,timeOutPeriod,changeState_TaggedToTxGenotype,rhandles.nodesTagged,rhandles.endTimeTimeOutTxGenotype);
            %endTimeTimeOutTxGenotypeD = rhandles.endTimeTimeOutTxGenotype %testing only
            %Add tagging
            %nodesTaggedD = rhandles.nodesTagged; %testing only
            releaseTags = releaseTags__TaggedToNotTagged + releaseTags__TxGenotypeToNotTagged + releaseTags__TxExploitToNotTagged; %release these tags
            tagsToAdd = NodeEncounterClass.addTags(ihandles.numNodes,chosenNodes); %add these ones
            tagsToStayTheSame = rhandles.nodesTagged.*((releaseTags + tagsToAdd )<1); %keep these ones
            rhandles.nodesTagged = tagsToAdd + tagsToStayTheSame;
            %nodesTaggedD = rhandles.nodesTagged %testing only
            %set endtxgenotype
            %endTimeTxGenotypeD = rhandles.endTimeTxGenotype; %testing only
            keepEndTimeTxGenotype = (releaseTags(1,:)<1).*rhandles.endTimeTxGenotype; %keep these ones
            addEndTimeTxGenotype = newNodeIdentified.*(t+(ihandles.timeToTxGenotype.*GUIClass.convertTimeUnits(ihandles.timeToTxGenotypeUnits)));%add these ones
            rhandles.endTimeTxGenotype = keepEndTimeTxGenotype + addEndTimeTxGenotype;
            %endTimeTxGenotypeD = rhandles.endTimeTxGenotype %testing only
            %set endExploitTime assuming genotype not going to be sent first
            %endTimeTxExploitD = rhandles.endTimeTxExploit; %testing only
            keepEndTimeTxExploit = (releaseTags(1,:)<1).*rhandles.endTimeTxExploit; %keep these ones
            AddEndTimeTxExploit = newNodeIdentified.*(t+(ihandles.timeToTxVirus.*GUIClass.convertTimeUnits(ihandles.timeToTxVirusUnits)));%add these ones
            rhandles.endTimeTxExploit = keepEndTimeTxExploit + AddEndTimeTxExploit;
            %endTimeTxExploitD=rhandles.endTimeTxExploit; %testing only
            %update endExploitTime if sent genotype first
            keepEndTimeTxExploit2 = (changeState_TaggedToTxGenotype < 1).*rhandles.endTimeTxExploit;
            AddEndTimeTxExploit2 = changeState_TaggedToTxGenotype.*(t+(ihandles.timeToTxVirus.*GUIClass.convertTimeUnits(ihandles.timeToTxVirusUnits)));
            rhandles.endTimeTxExploit = keepEndTimeTxExploit2 + AddEndTimeTxExploit2;
             %endTimeTxExploitD=rhandles.endTimeTxExploit %testing only
            %check endTimeTimeOutTxGenotype against t and reset if reached
            %endTimeTimeOutTxGenotypeD = rhandles.endTimeTimeOutTxGenotype; %testing only
            keepEndTimeTimeOutTxGenotype = rhandles.endTimeTimeOutTxGenotype>t;
            rhandles.endTimeTimeOutTxGenotype = rhandles.endTimeTimeOutTxGenotype.*keepEndTimeTimeOutTxGenotype;
            %endTimeTimeOutTxGenotypeD = rhandles.endTimeTimeOutTxGenotype %testing only
            %next state
            rhandles.tagState = nextStateNumber;
            %*************************************
            %Assign outputs
            %*************************************         
            rhandles.txGenotypeEncounterYes = changeState_TaggedToTxGenotype;
            rhandles.txExploitEncounterYes = or(changeState_TaggedToTxExploit,changeState_TxGenotypeToTxExploit);
            
            txGenotypeEncounterYes = rhandles.txGenotypeEncounterYes;
            txExploitEncounterYes = rhandles.txExploitEncounterYes;
            encounterNodes = rhandles.nodesTagged;
        end %end func
        %**********sub functions************
        function tagsToAdd = addTags(numNodes,chosenNodes)
            tagsToAdd = zeros(numNodes,numNodes);
            for i = 1:numNodes
                if chosenNodes(i)>0
                tagsToAdd(chosenNodes(i),i)=1;
                end
            end
        end %end func
        function activeEndTimeTimeOutTxGenotype = checkTimeOutTxGenotypeNotReached(nodesTagged,endTimeTimeOutTxGenotype)
            %check if not reached end time
            match = and(nodesTagged,endTimeTimeOutTxGenotype>0);
            activeEndTimeTimeOutTxGenotype = sum(match,1);%sum down columns
        end %end func
        function timeOutTxGenotype = setTimeOut(numNodes,t,timeOutPeriod,changeState_TaggedToTxGenotype,nodesTagged,timeOutTxGenotypeOld)
            changeState_TaggedToTxGenotypeRep = repmat(changeState_TaggedToTxGenotype,[numNodes,1]);
            timeOutTxGenotype_temp = changeState_TaggedToTxGenotypeRep .* nodesTagged .* (t+timeOutPeriod);
            timeOutTxGenotype = timeOutTxGenotype_temp + timeOutTxGenotypeOld; %put this under rhandles
        end %end func
        function [taggedNodeStillInRange] = checkTaggedInRange(nodesTagged,nodesInRangeShift)
            match = and(nodesTagged,nodesInRangeShift);
            taggedNodeStillInRange=sum(match,1);
        end %end func
        function [taggedNodeOutOfRange] = checkTaggedOutOfRange(nodesTagged,nodesInRangeShift)
            match = and(nodesTagged,nodesInRangeShift<1);
            taggedNodeOutOfRange=sum(match,1);
        end %end func
        function[taggedNodeTimeOutReset] = checkTimeOutTxGenotype(nodesTagged,endTimeTimeOutTxGenotype)
            match = and(nodesTagged,endTimeTimeOutTxGenotype==0);
            taggedNodeTimeOutReset=sum(match,1);
        end %end func
        function [chosenFlag chosenNodes] = getAvailableNodesInRange(numNodes,nodesTagged,sortNodesInRangeNumber)
            %taggedNodesOrdered = zeros(1,numNodes);
            chosenNodes = zeros(1,numNodes); %node number chosen to be encountered - nodeEncounter
            chosenFlag = zeros(1,numNodes); %flag indicating a node has been chosen - encounterYes
            %nodesTagged - numNodes by numNodes matrix, each column is the node, each row
            %is what the node sees about other nodes, i.e which node is tagged for
            %communicating with
            taggedNodesOrdered = sum((nodesTagged >=1),2); %sum across rows to determine which node has been tagged already
            %nodesAvailableOrdered = nodesTaggedOrdered <1 %invert to find out which nodes are available
            identNewNode = sum((nodesTagged >=1),1) <1; %sum across columns and invert to find out which nodes are trying to identify a node to tag 
            %scroll through the nodes in range and select one that is not
            %already tagged, if a node is selected, tag it, to stop others
            %from selecting it at the same time
            for i = 1:numNodes
                if identNewNode(i) ==1 %only do if need to encounter another node
                    num = numNodes;
                    while and(and(sortNodesInRangeNumber(num,i) > 0 ,num > 0),chosenFlag(1,i)==0) %scroll through the nodes in range only
                        if taggedNodesOrdered(sortNodesInRangeNumber(num,i)) == 0;  %select only if not already tagged
                            chosenNodes(1,i) = sortNodesInRangeNumber(num,i);
                            taggedNodesOrdered(sortNodesInRangeNumber(num,i)) = 1;
                            chosenFlag(1,i)=1;
                        end
                        num = num-1;
                    end
                end
            end
            %taggedNodesOrderedD=taggedNodesOrdered %testing
        end%end func
        
        %********Random waypoint common functions**************************
        function [sqrDistance,nodesInRange,nodesInRangeShift,nodesInRangeNumber,sortNodesInRangeNumber,numNodesInRange,nodesInRangeFlag] = getNodesInRange(nodes,range)
            % 1.square of distance between nodes
            % xdiff=|x2-x1|
            % ydiff=|y2-y1|
            % (xdiff)^2 + (ydiff)^2
            lenx = size(nodes.x,2);
            numNodes = lenx;
            sqrDistance = zeros(lenx,lenx);
            for a = 1:lenx
                b=a+1;
                c=a-1;
                %x
                xShifted(1:lenx-a) = nodes.x(b:end);
                xShifted(lenx-c:lenx) = nodes.x(1:a);
                xdiff = xShifted-nodes.x;
                xsq = xdiff.^2;
                %y
                yShifted(1:lenx-a) = nodes.y(b:end);
                yShifted(lenx-c:lenx) = nodes.y(1:a);
                ydiff = yShifted-nodes.y;
                ysq = ydiff.^2;
                %square of distance
                sqrDistance(a,:) = xsq+ysq;
            end
            %2.Work out which nodes are in range
            nodesInRange = sqrDistance(:,:)<=range.^2; %before self
            nodesInRange(end,:) = 0; %exclude self
            numNodesInRange = sum(nodesInRange);
            nodesInRangeFlag = numNodesInRange>=1;
            %3.convert nodes in range matrix of 1's and 0's into to a matrix containing the actual node(s) number in range
            %*****************more efficient method
            nodesInRangeShift = zeros(numNodes,numNodes);
            nodesInRangeNumber = zeros(numNodes,numNodes);
            nodeNumbers = 1:numNodes;
            for i = 1:numNodes
                nodesInRangeShift(:,i) = circshift(nodesInRange(:,i), [i, 0]); %shift so node number flags are aligned in rows
                nodesInRangeNumber(:,i) = nodesInRangeShift(:,i) .* nodeNumbers'; %get the node numbers
            end
            sortNodesInRangeNumber = sort(nodesInRangeNumber, 1, 'ascend'); %sort nodes in order
            %*****************Old method
            %nodesInRangeNumberM = zeros(numNodes,numNodes);
            %for nodeNumber = 1: numNodes
            %    for nodeIndex= 1:numNodes
            %        nodesInRangeNumberM(nodeIndex,nodeNumber) = (nodesInRange(nodeIndex,nodeNumber)+nodeIndex+nodeNumber-1).*nodesInRange(nodeIndex,nodeNumber);
            %        if nodesInRangeNumberM(nodeIndex,nodeNumber) > numNodes
            %            nodesInRangeNumberM(nodeIndex,nodeNumber) = rem(nodesInRangeNumberM(nodeIndex,nodeNumber),numNodes);
            %        end
            %    end
            %end
            %nodesInRangeNumberM
            
            %*********************
        end %end func
        function nearestNodes = formatNodesIndex(nearestNodesIndex,lenx)
            for a= 1:lenx
                index(a) = nearestNodesIndex(a)+a;
                if index(a) > lenx
                    index(a) = rem(index(a),lenx);
                end
            end
            nearestNodes = index;
        end% func
        %*****************************************************************
        % Random Waypoint animation showing nodes infected
        % upgrade later to show other node states in different colours
        % blue - not infected
        % red - infected
        %*****************************************************************
        function RWP_animateAttack(ihandles,rhandles,PlotWayPoints)
        %check need to run this function
        if and(ihandles.networkModelEnable,strcmp(ihandles.networkModelS,'Random Waypoint'))
            %time scale
            time_step = 1;
            v_t = 0:time_step:ihandles.numTimeSteps;
            %get saved precomputed x and y waypoint positions
             for nodeIndex = 1:ihandles.numNodes
                vs_node(nodeIndex).v_x = ihandles.vs_node(nodeIndex).v_x;
                vs_node(nodeIndex).v_y = ihandles.vs_node(nodeIndex).v_y;
             end 
    %*************************************************
    %only plot if selected
    %*************************************************
    if PlotWayPoints == 1
    figure;
    hold on;
    for nodeIndex = 1:ihandles.numNodes
        %rhandles.nodeStateAtTimeStr(1,nodeIndex)
        nodeInfected(nodeIndex) = strcmp(rhandles.nodeStateAtTimeStr(1,nodeIndex),'I');
        if nodeInfected(nodeIndex)==1
           vh_node_pos(nodeIndex) = plot(vs_node(nodeIndex).v_x(1),vs_node(nodeIndex).v_y(1),'*','color',[1 0 0]); 
        else
        vh_node_pos(nodeIndex) = plot(vs_node(nodeIndex).v_x(1),vs_node(nodeIndex).v_y(1),'*','color',[0.3 0.3 1]);
        end
    end
    title(cat(2,'Simulation time (sec): ',num2str(ihandles.numTimeSteps)));
    xlabel('X (meters)');
    ylabel('Y (meters)');
    title('Radom Waypoint mobility');
    ht = text(min(vs_node(1).v_x),max(vs_node(1).v_y),cat(2,'Time (sec) = 0'));
    %axis([min(vs_node(1).v_x) max(vs_node(1).v_x) min(vs_node(1).v_y) max(vs_node(1).v_y)]);
    xmin = ihandles.s_input.V_POSITION_X_INTERVAL(1);
    xmax = ihandles.s_input.V_POSITION_X_INTERVAL(2);
    ymin = ihandles.s_input.V_POSITION_Y_INTERVAL(1);
    ymax = ihandles.s_input.V_POSITION_Y_INTERVAL(2);
    axis([xmin xmax ymin ymax]);
    hold off;
    for timeIndex = 1:ihandles.numTimeSteps ;%length(v_t);
        t = v_t(timeIndex);
        set(ht,'String',cat(2,'Time (sec) = ',num2str(t,4)));
        for nodeIndex = 1:ihandles.numNodes
            nodeInfected(nodeIndex) = strcmp(rhandles.nodeStateAtTimeStr(timeIndex,nodeIndex),'I');
            if nodeInfected(nodeIndex)==1
                %vh_node_pos(nodeIndex) = plot(vs_node(nodeIndex).v_x(timeIndex),vs_node(nodeIndex).v_y(timeIndex),'*','color',[1 0 0]);
                set(vh_node_pos(nodeIndex),'color',[1 0 0]); %change to red
            end
                set(vh_node_pos(nodeIndex),'XData',vs_node(nodeIndex).v_x(timeIndex),'YData',vs_node(nodeIndex).v_y(timeIndex));
            %[1 1 0] y yellow
            %[1 0 1] m magenta
            %[0 1 1] c cyan
            %[1 0 0] r red
            %[0 1 0] g green
            %[0 0 1] b blue
            %[1 1 1] w white
            %[0 0 0] k black
        end
        drawnow;
    end
   end %end if
        end %if rwp selected check
   end %func
   function RWP_animateAttackPlayback(outputs,runNumber)
            %time scale
            time_step = 1;
            v_t = 0:time_step:outputs.numTimeSteps;
            %get saved precomputed x and y waypoint positions
             for nodeIndex = 1:outputs.numNodes
                vs_node(nodeIndex).v_x = outputs.vs_node(nodeIndex).v_x;
                maxX(nodeIndex) = max(vs_node(nodeIndex).v_x);
                vs_node(nodeIndex).v_y = outputs.vs_node(nodeIndex).v_y;
                maxY(nodeIndex) = max(vs_node(nodeIndex).v_y);
             end 
    %*************************************************
    %plot
    %*************************************************
    figure;
    hold on;
    for nodeIndex = 1:outputs.numNodes
        temp3.nodeStateAtTimeStr = outputs.allRunsNodeStateAtTime(runNumber).nodeStateAtTimeStr;
        const = char(temp3.nodeStateAtTimeStr(1,nodeIndex));
        [plotColour nodeShape] = NodeEncounterClass.getPlotColour(const);
        vh_node_pos(nodeIndex) = plot(vs_node(nodeIndex).v_x(1),vs_node(nodeIndex).v_y(1),nodeShape,'color',plotColour); 
    end
    title(cat(2,'Simulation time (sec): ',num2str(outputs.numTimeSteps)));
    xlabel('X (meters)');
    ylabel('Y (meters)');
    titleGraph = ['Random Waypoint mobility - run number:' ,num2str(runNumber)]
    title(titleGraph);
    ht = text(min(vs_node(1).v_x),max(vs_node(1).v_y),cat(2,'Time (sec) = 0'));
    %axis([min(vs_node(1).v_x) max(vs_node(1).v_x) min(vs_node(1).v_y) max(vs_node(1).v_y)]);
    xmin = 0; %ihandles.s_input.V_POSITION_X_INTERVAL(1);
    xmax = round(max(maxX)); %ihandles.s_input.V_POSITION_X_INTERVAL(2);
    ymin = 0; %ihandles.s_input.V_POSITION_Y_INTERVAL(1);
    ymax = round(max(maxY)); %ihandles.s_input.V_POSITION_Y_INTERVAL(2);
    axis([xmin xmax ymin ymax]);
    hold off;
    for timeIndex = 1:outputs.numTimeSteps ;%length(v_t);
        t = v_t(timeIndex);
        set(ht,'String',cat(2,'Time (sec) = ',num2str(t,4)));
        for nodeIndex = 1:outputs.numNodes
           const = char(temp3.nodeStateAtTimeStr(timeIndex,nodeIndex)); %get state of each node at each time step
          [plotColour nodeShape] = NodeEncounterClass.getPlotColour(const); %plot type on graph
            set(vh_node_pos(nodeIndex),'Marker',nodeShape); %change marker
            set(vh_node_pos(nodeIndex),'color',plotColour); %change colour
            set(vh_node_pos(nodeIndex),'XData',vs_node(nodeIndex).v_x(timeIndex),'YData',vs_node(nodeIndex).v_y(timeIndex))
            %[1 1 0] y yellow
            %[1 0 1] m magenta
            %[0 1 1] c cyan
            %[1 0 0] r red
            %[0 1 0] g green
            %[0 0 1] b blue
            %[1 1 1] w white
            %[0 0 0] k black
        end
        drawnow;
    end  %if
    end %func
    function RWPAssignOutputs(ihandles,ohandles)
        ohandles.vs_node = ihandles.vs_node;
    end
    function [plotColour nodeShape] = getPlotColour(nodeStateAtTimeStr)
            %colours
            %[1 1 0] y yellow
            %[1 0 1] m magenta
            %[0 1 1] c cyan
            %[1 0 0] r red
            %[0 1 0] g green
            %[0 0 1] b blue
            %[1 1 1] w white
            %[0 0 0] k black
            %shapes
            
        switch(nodeStateAtTimeStr)
            case 'S'
                plotColour = [0 0 1]; %blue
                nodeShape = '*';
            case 'E'
                plotColour = [1 0 1]; %magenta
                nodeShape = '*' ;
            case 'P'
                plotColour = [0 1 0]; %green
                nodeShape = 'o'; %can't get infected, patch installed
            case 'T'
                plotColour = [1 1 0]; %yellow
                nodeShape = '*' ;
            case 'I'
                plotColour = [1 0 0]; %red
                nodeShape = '*' ;
            case 'C'
                plotColour = [1 0 0]; %red
                nodeShape = 'o'; %offline
            case 'O'
                plotColour = [0 0 1]; %blue
                nodeShape = 'o' ;%offline
            case 'X'
                plotColour = [1 0 1]; %magenta
                nodeShape = 'o'; %offline
            case 'R'
                plotColour = [0 1 0]; %green
                nodeShape = '*' ;
        end
    end
    
    
    %*************************************************************************    
    end; %method
    
end

