%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Jennifer Jackson 2017                                                        %
% A Biodiversity Approach to Cyber Security                                    %
% PhD Complexity Science                                                       %
% University of Warwick                                                        %
% http://wrap.warwick.ac.uk/98458                                              %
% This software is distributed under the  Creative Commons Attribution License %
% https://creativecommons.org/licenses/by/4.0/                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
classdef StaticDiversityClass
%*************************************************************************
%List of functions to support static diversity
%*************************************************************************
%functions listed here:

%
%
%
%*************************************************************************
    properties
    end
    
    methods
    end
    %static methods that can be called without instantiating a class    
    methods (Static)
 %***********************************************************************
 % Function to get handle data and include in the model
 %***********************************************************************
 function StaticDataSimInit(handles,ihandles,ohandles)
     %get data handles
     if strcmp(ihandles.algModelS,'Static Diversity')
       disp('getting static data now')      
             %get true data
             [ihandles.dataSet] = StaticDiversityClass.getDatasetInfo(handles);
             [ihandles.Genotype ihandles.Genotype_apps ihandles.Genotype_serv ihandles.Genotype_os ihandles.Genotype_dr] = StaticDiversityClass.getStaticData(ihandles.dataSet,handles);
            %get size of data
            numNodes = size(ihandles.Genotype,1);
            len_apps = size(ihandles.Genotype_apps,2);
            len_serv = size(ihandles.Genotype_serv,2);
            len_os = size(ihandles.Genotype_os,2);
            len_dr = size(ihandles.Genotype_dr,2);
            ihandles.numNodes = numNodes;
            ihandles.len_apps = len_apps;
            ihandles.len_serv = len_serv;
            ihandles.len_os = len_os;
            ihandles.len_dr = len_dr;
           % [ihandles.numNodes ihandles.len_apps ihandles.len_serv ihandles.len_os ihandles.len_dr] = DataClass.getGenoSize(ihandles.Genotype_apps,ihandles.Genotype_serv,ihandles.Genotype_os,ihandles.Genotype_dr)
            ihandles.len_sw = ihandles.len_apps+ihandles.len_serv+ihandles.len_os+ihandles.len_dr
            %get dummy data
             StaticDiversityClass.initDummyData(ihandles, ohandles);
     end %strcmp
 end
 function StaticDiversitySimInit(handles,ihandles,ohandles)
      if strcmp(ihandles.algModelS,'Static Diversity')
          diversityModelParameters_FavScoreFigData = guidata(handles.diversityModelParameters_FavScoreFig); %load sub gui data
            %set up inputs
            %get current settings from GUI (static parameters)
            [ihandles.turnOnSol1 ihandles.turnOnSol2 ihandles.turnOnSol3 ihandles.turnOnSol4 ihandles.turnOnSol5] = DynAlgClass.getAlgChoice(diversityModelParameters_FavScoreFigData);
           % [ihandles.numRunsPerSim ihandles.numTimeSteps] = DynAlgClass.getSimInfo(handles);
            [ihandles.minEncounters ihandles.maxEncounters ihandles.distStr] = DynAlgClass.getEncounterInfo(diversityModelParameters_FavScoreFigData);
            [ihandles.minstd4Tolerance ihandles.maxstd4Tolerance ihandles.minstd5Tolerance ihandles.maxstd5Tolerance ihandles.sampleSize ...
                   ihandles.restartUpdateTest ihandles.stopUpdate4 ihandles.restartUpdate4 ihandles.stopUpdate5 ihandles.restartUpdate5] = DynAlgClass.getStopStartInfo(diversityModelParameters_FavScoreFigData, ihandles.maxEncounters);
            [ihandles.ChangeDesirabilitiesFlag ihandles.tChange1Enable ihandles.tChange1 ihandles.tChange2Enable ihandles.tChange2] = DynAlgClass.getComDesFlags(diversityModelParameters_FavScoreFigData);
            [ihandles.ChangeIndDesirabilitiesFlag ihandles.tChangeInd1Enable ihandles.tChangeInd1 ihandles.tChangeInd2Enable ihandles.tChangeInd2] = DynAlgClass.getIndDesFlags(diversityModelParameters_FavScoreFigData);
            [ihandles.ChangeEffFlag ihandles.tChangeEff1Enable ihandles.tChangeEff1 ihandles.tChangeEff2Enable ihandles.tChangeEff2] = DynAlgClass.getEffDesFlags(diversityModelParameters_FavScoreFigData);
            [ihandles.We ihandles.Wcd ihandles.Wid ihandles.wdiv] = DynAlgClass.getWeights(diversityModelParameters_FavScoreFigData);
             %initialise outputs   
            [ohandles.nodesDiversity4 ohandles.nodesMean4 ohandles.nodesStd4 ohandles.updateStopDisplay4 ohandles.minStd4Curve ohandles.updateStopDisplay5 ohandles.nodesStd5 ohandles.minStd5Curve ...
                ohandles.lastStoppedStdMatrix ohandles.lastStoppedStdMatrix2 ohandles.Nei_indDynamic] = DynAlgClass.initSimOutputs(ihandles.numRunsPerSim, ihandles.numNodes, ihandles.numTimeSteps);
     end %strcmp
 end
 function initDummyData(ihandles, ohandles)

        for i = 1:ihandles.len_apps
            ihandles.SW_str_apps(i) = 'a';
            ihandles.SW_freq_apps(i,:) = 'aV';
            CSW_str_apps(i) = 'a';
        end
        for i = 1:ihandles.len_serv
            ihandles.SW_str_serv(i) = 'a';
            ihandles.SW_freq_serv(i,:) = 'sV';
            CSW_str_serv(i) = 'a';
        end
        for i = 1:ihandles.len_os
            ihandles.SW_str_os(i) = 'a';
            ihandles.SW_freq_os(i,:) = 'oV';
            CSW_str_os(i) = 'a';
        end
        for i = 1:ihandles.len_dr
            ihandles.SW_str_dr(i) = 'a';
            ihandles.SW_freq_dr(i,:) = 'dV';
            CSW_str_dr(i) = 'a';
        end
        CSW = [CSW_str_apps'; CSW_str_serv'; CSW_str_os'; CSW_str_dr'];
        %create two filters
        ihandles.CSW1 = zeros(size(CSW));
        ihandles.CSW2 = zeros(size(CSW));
        ihandles.CSW3 = zeros(size(CSW));
        ihandles.CSW4 = zeros(size(CSW));
        ihandles.CSW5 = zeros(size(CSW));
        ihandles.CSW6 = zeros(size(CSW));
        ihandles.CSW7 = zeros(size(CSW));
        ihandles.CSW8 = zeros(size(CSW));
        ihandles.CSW9 = zeros(size(CSW));
        ihandles.CSW10 = zeros(size(CSW));
        CSWGroup(1,:) = ihandles.CSW1;
        CSWGroup(2,:) = ihandles.CSW2;
        ihandles.CSWGroup = CSWGroup;
        
        ihandles.E_apps = zeros(1,ihandles.len_apps).*10;
        ihandles.E_apps2 = zeros(1,ihandles.len_apps);
        ihandles.CD_apps2 = zeros(1,ihandles.len_apps);
        ihandles.E_serv = zeros(1,ihandles.len_serv).*10;
        ihandles.E_serv2 = zeros(1,ihandles.len_serv);
        ihandles.CD_serv2 = zeros(1,ihandles.len_serv);
        ihandles.E_os = zeros(1,ihandles.len_os).*10;
        ihandles.E_os2 = zeros(1,ihandles.len_os);
        ihandles.CD_os2 = zeros(1,ihandles.len_os);
        ihandles.E_dr = zeros(1,ihandles.len_dr).*10;
        ihandles.E_dr2 = zeros(1,ihandles.len_dr);
        
        ihandles.CD_dr2 = zeros(1,ihandles.len_dr);
        
        ihandles.CD_apps = ones(1,ihandles.len_apps).*1;
        ihandles.CD_serv = ones(1,ihandles.len_serv).*1;
        ihandles.CD_os = ones(1,ihandles.len_os).*1;
        ihandles.CD_dr = ones(1,ihandles.len_dr).*1;
        
        %concat data
     ihandles.textdata = [ihandles.SW_freq_apps; ihandles.SW_freq_serv; ihandles.SW_freq_os; ihandles.SW_freq_dr];
     

        ihandles.E = [ihandles.E_apps ihandles.E_serv ihandles.E_os ihandles.E_dr]'; %efficiency ratings
        ihandles.E2 = [ihandles.E_apps2 ihandles.E_serv2 ihandles.E_os2 ihandles.E_dr2]'; %efficiency ratings
        ihandles.CD = [ihandles.CD_apps ihandles.CD_serv ihandles.CD_os ihandles.CD_dr]'; %community desirability ratings
        ihandles.CD2 = [ihandles.CD_apps2 ihandles.CD_serv2 ihandles.CD_os2 ihandles.CD_dr2]'; %community desirability ratings2
        ihandles.ID = [ihandles.ID_apps ihandles.ID_serv ihandles.ID_os ihandles.ID_dr]'; %individual desirability ratings
        ihandles.ID2 = [ihandles.ID_apps2 ihandles.ID_serv2 ihandles.ID_os2 ihandles.ID_dr2]'; %individual desirability ratings
        
        ihandles.SizeCD_apps = size(ihandles.CD_apps);
        ihandles.SizeCD_serv = size(ihandles.CD_serv);
        ihandles.SizeCD_os = size(ihandles.CD_os);
        ihandles.SizeCD_dr = size(ihandles.CD_dr);
        ihandles.CDSize = size(ihandles.CD);      
        ihandles.tallyLength = ihandles.CDSize(1);
        ihandles.numOS = ihandles.len_os %number of operating systems
     
        ihandles.ID_apps = ones(ihandles.numNodes,ihandles.len_apps);
        ihandles.ID_serv = ones(ihandles.numNodes,ihandles.len_serv);
        ihandles.ID_os = ones(ihandles.numNodes,ihandles.len_os);
        ihandles.ID_dr = ones(ihandles.numNodes,ihandles.len_dr);
        ihandles.ID_apps2 = ones(ihandles.numNodes,ihandles.len_apps);
        ihandles.ID_serv2 = ones(ihandles.numNodes,ihandles.len_serv);
        ihandles.ID_os2 = ones(ihandles.numNodes,ihandles.len_os);
        ihandles.ID_dr2 = ones(ihandles.numNodes,ihandles.len_dr);
                
        ohandles.ID_apps = ones(ihandles.numNodes,ihandles.len_apps);
        ohandles.ID_serv = ones(ihandles.numNodes,ihandles.len_serv);
        ohandles.ID_os = ones(ihandles.numNodes,ihandles.len_os);
        ohandles.ID_dr = ones(ihandles.numNodes,ihandles.len_dr);
        ohandles.ID_apps2 = ones(ihandles.numNodes,ihandles.len_apps);
        ohandles.ID_serv2 = ones(ihandles.numNodes,ihandles.len_serv);
        ohandles.ID_os2 = ones(ihandles.numNodes,ihandles.len_os);
        ohandles.ID_dr2 = ones(ihandles.numNodes,ihandles.len_dr);
        
        ohandles.textdata = [ihandles.SW_freq_apps; ihandles.SW_freq_serv; ihandles.SW_freq_os; ihandles.SW_freq_dr];
        
        ihandles.ID = [ihandles.ID_apps ihandles.ID_serv ihandles.ID_os ihandles.ID_dr]'; %individual desirability ratings
        ihandles.ID2 = [ihandles.ID_apps2 ihandles.ID_serv2 ihandles.ID_os2 ihandles.ID_dr2]'; %individual desirability ratings
 
 end
             %*****************************************************
        %Get data set info
        %*****************************************************
        function [dataSet] = getDatasetInfo(handles)
            dataSet = get(handles.popupmenuStaticDataset, 'Value');
        end
 function StaticRunInit(ihandles,rhandles,ohandles)
     %run initialisations
      if strcmp(ihandles.algModelS,'Static Diversity')
        %recording individual diversity dynamically over time
        rhandles.Nei_indDynamic = zeros(1,ihandles.numTimeSteps);  
        rhandles.counterEncounters = zeros(ihandles.numNodes,1); 
        rhandles.updateYes = zeros(ihandles.numNodes,1); %this is used
        
        %recording genotype updates to be used as input to the state
        %machine for removing infection whenever a new genotype is loaded
        %which is used to represent a VM being destroyed and created
        rhandles.updateGenotypeYes = zeros(ihandles.numNodes,1);

        %bespoke solutions 1 to 3 - no longer used
        [rhandles.updateStop1 rhandles.updateStop2 rhandles.updateStop3] = DynAlgClass.initRunVariablesBespokeSols(ihandles.numNodes);
        [rhandles.GenotypeSol1 rhandles.GenotypeSol1_apps rhandles.GenotypeSol1_serv rhandles.GenotypeSol1_os rhandles.GenotypeSol1_dr] = DynAlgClass.initRunVariablesGenotype(ihandles.Genotype,ihandles.Genotype_apps,ihandles.Genotype_serv,ihandles.Genotype_os,ihandles.Genotype_dr); 
        [rhandles.GenotypeSol2 rhandles.GenotypeSol2_apps rhandles.GenotypeSol2_serv rhandles.GenotypeSol2_os rhandles.GenotypeSol2_dr] = DynAlgClass.initRunVariablesGenotype(ihandles.Genotype,ihandles.Genotype_apps,ihandles.Genotype_serv,ihandles.Genotype_os,ihandles.Genotype_dr); 
        [rhandles.GenotypeSol3 rhandles.GenotypeSol3_apps rhandles.GenotypeSol3_serv rhandles.GenotypeSol3_os rhandles.GenotypeSol3_dr] = DynAlgClass.initRunVariablesGenotype(ihandles.Genotype,ihandles.Genotype_apps,ihandles.Genotype_serv,ihandles.Genotype_os,ihandles.Genotype_dr); 

        %solution 4
        [rhandles.updateStop4 rhandles.reachTolFlag4 rhandles.minStd4 rhandles.maxStd4 rhandles.minStd4Curve rhandles.nodesDiversity4 rhandles.updateStopDisplay4 ...
        rhandles.nodesMean4 rhandles.nodesMean4Filt rhandles.nodesStd4 rhandles.nodesStd4Filt rhandles.displayStopVal rhandles.displayStartVal rhandles.updateStopDisplayFlagOld4 ...
        rhandles.lastStoppedStd rhandles.lastStoppedStdMatrix rhandles.lastStoppedStdMatrix2] = DynAlgClass.initRunVariablesSol4(ihandles.numNodes,ihandles.minstd4Tolerance,ihandles.maxstd4Tolerance,ihandles.numTimeSteps,ihandles.sampleSize);
        [rhandles.GenotypeSol4 rhandles.GenotypeSol4_apps rhandles.GenotypeSol4_serv rhandles.GenotypeSol4_os rhandles.GenotypeSol4_dr] = DynAlgClass.initRunVariablesGenotype(ihandles.Genotype,ihandles.Genotype_apps,ihandles.Genotype_serv,ihandles.Genotype_os,ihandles.Genotype_dr);    

        %solution 5
        [rhandles.updateStop5 rhandles.reachTolFlag5 rhandles.minStd5 rhandles.maxStd5 rhandles.minStd5Curve rhandles.nodesDiversity5 rhandles.updateStopDisplay5 ...
        rhandles.nodesMean5 rhandles.nodesStd5 rhandles.displayStopVal5 rhandles.displayStartVal5 rhandles.updateStopDisplayFlagOld5 rhandles.lastStoppedStd5] = DynAlgClass.initRunVariablesSol5(ihandles.numNodes,ihandles.minstd5Tolerance,ihandles.maxstd5Tolerance,ihandles.numTimeSteps,ihandles.sampleSize);
        [rhandles.GenotypeSol5 rhandles.GenotypeSol5_apps rhandles.GenotypeSol5_serv rhandles.GenotypeSol5_os rhandles.GenotypeSol5_dr] = DynAlgClass.initRunVariablesGenotype(ihandles.Genotype,ihandles.Genotype_apps,ihandles.Genotype_serv,ihandles.Genotype_os,ihandles.Genotype_dr); 

        %for counting software
        [rhandles.tallyUpdates rhandles.tallySol1 rhandles.tallySol2 rhandles.tallySol3 rhandles.tallySol4 rhandles.tallySol5] = DynAlgClass.initRunVariablesTally(ihandles.numNodes,ihandles.tallyLength);
        %initialise dynamic variables for dynamic weights and dynamic desirabilities
%        [rhandles.divWeightsValueOld,rhandles.comWeightsValueOld,rhandles.indWeightsValueOld,rhandles.qualWeightsValueOld,...
%                    rhandles.calcSE_Comw,rhandles.calcSE_Indw,rhandles.calcSE_Qualw,rhandles.comDesValueOld,rhandles.indDesValueOld,rhandles.qualDesValueOld,...
%                    rhandles.calcSE_Comd,rhandles.calcSE_Indd,rhandles.calcSE_Quald,ihandles.We,ihandles.Wcd,ihandles.Wid,rhandles.S_CD,rhandles.S_E] = DynAlgClass.initRunVariablesDynamic(ihandles.numNodes,ihandles.len_sw,ihandles.We,ihandles.Wcd,ihandles.Wid,rhandles.S_CD,rhandles.S_E);
        %set up type of encounters & number of encounters before an update
        plotData=0;
        rhandles.numEncounters = stochasticEncounters(ihandles.distStr,ihandles.numNodes,ihandles.minEncounters,ihandles.maxEncounters,plotData); 
      end
 end
 %function to generate static genotype data using a random distribution -
 %5th Jan 2017
 function [genotypeMatrix] = getStaticDataRandom(numNodes,numVariantsArray)
 numLoci = size(numVariantsArray,2);
 lenGenotype = sum(numVariantsArray);
%generate genotypes    
genotypeMatrix = zeros(numNodes,lenGenotype);%binary form of genotype matrix
genotypeNode = zeros(numLoci,numNodes);
for j = 1:numLoci
 genotypeNode(j,:) = randi(numVariantsArray(j),[1,numNodes]); %generate variant number from a random distribution
end
%***
%convert to binary matrix
for i = 1:numNodes
for j = 1:numLoci
start = sum(numVariantsArray(1:j))-numVariantsArray(j);
genotypeMatrix(i,(start+genotypeNode(j,i)))=1;
end
end
%***
 end %end func
 
 
 %function to generate static genotype data
 function [genotype Genotype_apps Genotype_serv Genotype_os Genotype_dr] = getStaticData(dataSetNumber,handles)
     numNodes = 400;
     switch dataSetNumber
        case 1 %max diversity for [8 8 2 8]
        numVariantsArray = [8 8 2 8]
        [genotype neiIndex neiIndexSingleLocus ] = StaticDiversityClass.genMaxDiversityGenotypes(numNodes,numVariantsArray);
        [Genotype_apps Genotype_serv Genotype_os Genotype_dr] = StaticDiversityClass.seperateGenotype(genotype,numVariantsArray);
        
        case 2 %max diversity for [8 8 8 8]
        numVariantsArray = [8 8 8 8]
        [genotype neiIndex neiIndexSingleLocus ] = StaticDiversityClass.genMaxDiversityGenotypes(numNodes,numVariantsArray);
        [Genotype_apps Genotype_serv Genotype_os Genotype_dr] = StaticDiversityClass.seperateGenotype(genotype,numVariantsArray);
        
        case 3 %freq controlled for [8 - - -], %max div for [- 8 2 8] 
        numVariantsArray = [8 8 2 8]
        freqArrayOneLocus = fix([1/8 1/8 1/8 1/8 1/8 1/8 1/8 1/8].*numNodes)
        [genotype neiIndex neiIndexSingleLocus ]  = StaticDiversityClass.genFreqDiversity(freqArrayOneLocus,numNodes,numVariantsArray);        
        [Genotype_apps Genotype_serv Genotype_os Genotype_dr] = StaticDiversityClass.seperateGenotype(genotype,numVariantsArray);    
        
        case 4 %freq controlled for [8 - - -], %max div for [- 8 2 8] 
        numVariantsArray = [8 8 2 8]
        fl = 6/7 %8-2 = 6, with 7 variants left
        freqArrayOneLocus = fix([2/8 fl/8 fl/8 fl/8 fl/8 fl/8 fl/8 fl/8].*numNodes)
        sumFreqArray = sum(freqArrayOneLocus)
        [genotype neiIndex neiIndexSingleLocus ]  = StaticDiversityClass.genFreqDiversity(freqArrayOneLocus,numNodes,numVariantsArray);        
        [Genotype_apps Genotype_serv Genotype_os Genotype_dr] = StaticDiversityClass.seperateGenotype(genotype,numVariantsArray);          
        
        case 5 %freq controlled for [8 - - -], %max div for [- 8 2 8] 
        numVariantsArray = [8 8 2 8]
        fl = 5/7 %8-3 = 5, with 7 variants left
        freqArrayOneLocus = fix([3/8 fl/8 fl/8 fl/8 fl/8 fl/8 fl/8 fl/8].*numNodes)
        sumFreqArray = sum(freqArrayOneLocus)
        [genotype neiIndex neiIndexSingleLocus ]  = StaticDiversityClass.genFreqDiversity(freqArrayOneLocus,numNodes,numVariantsArray);        
        [Genotype_apps Genotype_serv Genotype_os Genotype_dr] = StaticDiversityClass.seperateGenotype(genotype,numVariantsArray);          

        case 6 %freq controlled for [8 - - -], %max div for [- 8 2 8] 
        numVariantsArray = [8 8 2 8]
        fl = 4/7
        freqArrayOneLocus = fix([4/8 fl/8 fl/8 fl/8 fl/8 fl/8 fl/8 fl/8].*numNodes)
        sumFreqArray = sum(freqArrayOneLocus)
        [genotype neiIndex neiIndexSingleLocus ]  = StaticDiversityClass.genFreqDiversity(freqArrayOneLocus,numNodes,numVariantsArray);        
        [Genotype_apps Genotype_serv Genotype_os Genotype_dr] = StaticDiversityClass.seperateGenotype(genotype,numVariantsArray);          
        
        case 7 %freq controlled for [8 - - -], %max div for [- 8 2 8] 
        numVariantsArray = [8 8 2 8]
        fl = 3/7
        freqArrayOneLocus = fix([5/8 fl/8 fl/8 fl/8 fl/8 fl/8 fl/8 fl/8].*numNodes)
        sumFreqArray = sum(freqArrayOneLocus)
        [genotype neiIndex neiIndexSingleLocus ]  = StaticDiversityClass.genFreqDiversity(freqArrayOneLocus,numNodes,numVariantsArray);        
        [Genotype_apps Genotype_serv Genotype_os Genotype_dr] = StaticDiversityClass.seperateGenotype(genotype,numVariantsArray);          
             
        case 8 %freq controlled for [8 - - -], %max div for [- 8 2 8] 
        numVariantsArray = [8 8 2 8]
        fl = 2/7
        freqArrayOneLocus = fix([6/8 fl/8 fl/8 fl/8 fl/8 fl/8 fl/8 fl/8].*numNodes)
        sumFreqArray = sum(freqArrayOneLocus)
        [genotype neiIndex neiIndexSingleLocus ]  = StaticDiversityClass.genFreqDiversity(freqArrayOneLocus,numNodes,numVariantsArray);        
        [Genotype_apps Genotype_serv Genotype_os Genotype_dr] = StaticDiversityClass.seperateGenotype(genotype,numVariantsArray);          
        
        case 9 %freq controlled for [8 - - -], %max div for [- 8 2 8] 
        numVariantsArray = [8 8 2 8]
        fl = 1/7
        freqArrayOneLocus = fix([7/8 fl/8 fl/8 fl/8 fl/8 fl/8 fl/8 fl/8].*numNodes)
        sumFreqArray = sum(freqArrayOneLocus)
        [genotype neiIndex neiIndexSingleLocus ]  = StaticDiversityClass.genFreqDiversity(freqArrayOneLocus,numNodes,numVariantsArray);        
        [Genotype_apps Genotype_serv Genotype_os Genotype_dr] = StaticDiversityClass.seperateGenotype(genotype,numVariantsArray);          
          
        case 10 %freq controlled for [8 - - -], %max div for [- 8 2 8] 
        numVariantsArray = [8 8 2 8]
        fl = 0/7
        freqArrayOneLocus = fix([8/8 fl/8 fl/8 fl/8 fl/8 fl/8 fl/8 fl/8].*numNodes)
        sumFreqArray = sum(freqArrayOneLocus)
        [genotype neiIndex neiIndexSingleLocus ]  = StaticDiversityClass.genFreqDiversity(freqArrayOneLocus,numNodes,numVariantsArray);        
        [Genotype_apps Genotype_serv Genotype_os Genotype_dr] = StaticDiversityClass.seperateGenotype(genotype,numVariantsArray);    
        
         case 11 %random assignment to maximise variant diversity
             %get numLoci and numVariantsArray from sub GUI
             disp('random variant assignment')
              randomAssignmentParametersFigData = guidata(handles.randomAssignmentParametersFig) %load sub gui data
              numNodes = get(randomAssignmentParametersFigData.editNumNodes, 'Value')
              numVariantsArray = get(randomAssignmentParametersFigData.editVarArray, 'Value')
             %generate genotypes
             [genotype] = StaticDiversityClass.getStaticDataRandom(numNodes,numVariantsArray);
             [Genotype_apps Genotype_serv Genotype_os Genotype_dr] = StaticDiversityClass.seperateGenotype(genotype,numVariantsArray); %4 loci
     end
 end
 function  [genotype neiIndex neiIndexSingleLocus ]  = genFreqDiversity(freqArray,numNodes,numVariantsArray)
       %init
        lenArray = sum(numVariantsArray);
        [startP endP numLoci] = StaticDiversityClass.getStartAndEndLoci(numVariantsArray);
        genotype= zeros(numNodes,lenArray);
       %freq div
        numVariants = numVariantsArray(1);
        genotype(:,startP(1):endP(1)) = StaticDiversityClass.genFreqDiversityOneLocus(freqArray,numNodes,numVariants);
       %max div loci
         for j=2:numLoci
            genotype(:,startP(j):endP(j)) = StaticDiversityClass.genMaxDiversityOneLocus(numNodes,numVariantsArray(j));
         end
        %check nei
        [ neiIndex neiIndexSingleLocus ] = StaticDiversityClass.checkNei(genotype, numVariantsArray);
 end
 function genotypeMatrixPerc = genFreqDiversityOneLocus(freqArray,numNodes,numVariants)
    genotypeMatrixPerc = zeros(numNodes,numVariants);
    c=0;
    for varNumber = 1:numVariants-1
        for j = 1:freqArray(varNumber)
            c=c+1;
        genotypeMatrixPerc(c,varNumber)=1;
        end
    end
    %last one encase not exact frequency scaling
     varNumber = numVariants
        for j = 1:numNodes-c
            c=c+1;
        genotypeMatrixPerc(c,varNumber)=1;
        end
    
    
 end
 function [Genotype_apps Genotype_serv Genotype_os Genotype_dr] = seperateGenotype(genotype,numVariantsArray)
     start1 = 1; start2 = numVariantsArray(1)+1; start3 = numVariantsArray(1)+ numVariantsArray(2)+1;
     start4 = numVariantsArray(1)+ numVariantsArray(2)+numVariantsArray(3)+1;
     Genotype_apps = genotype(:,start1:start2-1);
     Genotype_serv = genotype(:,start2:start3-1);
     Genotype_os = genotype(:,start3:start4-1);
     Genotype_dr = genotype(:,start4:end);
     
     %re-written 5th Jan 2017
    % [ startP endP numLoci] = StaticDiversityClass.getStartAndEndLoci( numVariantsArray )
    % Genotype_apps = genotype(:,startP(1):endP(1));
    % Genotype_serv = genotype(:,startP(2):endP(2));
    % Genotype_os = genotype(:,startP(3):endP(3));
    % Genotype_dr = genotype(:,startP(4):endP(4));
     
 end

 function StaticDiversityTLoop(t,handles,ihandles,rhandles,ohandles,psim)
       %run function if enabled
     if strcmp(ihandles.algModelS,'Static Diversity')
     %*********************************************************************
     %change weights and desirabilities 
     %*********************************************************************
     %DynAlgClass.ChangeWeightsAndDesirabilities(t,ihandles,rhandles); %OLD METHOD, will still work
     %DynAlgClass.ChangeWeightsAndDesirabilitiesNEW(t,ihandles,rhandles); %NEW METHOD
     %*********************************************************************
     % Tally software encountered
     %*********************************************************************
     %only tally those that have had an encounter
     encounterYesRep = repmat(rhandles.encounterYes,1,ihandles.tallyLength);
     rhandles.tallySol1 = (rhandles.GenotypeSol1(rhandles.randomEncounter,:)).*encounterYesRep + rhandles.tallySol1;
     rhandles.tallySol2 = (rhandles.GenotypeSol2(rhandles.randomEncounter,:)).*encounterYesRep + rhandles.tallySol2;
     rhandles.tallySol3 = (rhandles.GenotypeSol3(rhandles.randomEncounter,:)).*encounterYesRep + rhandles.tallySol3;
     rhandles.tallySol4 = (rhandles.GenotypeSol4(rhandles.randomEncounter,:)).*encounterYesRep + rhandles.tallySol4;
     rhandles.tallySol5 = (rhandles.GenotypeSol5(rhandles.randomEncounter,:)).*encounterYesRep + rhandles.tallySol5;
     %**********************************************************************
     % Check when to update the genotype
     %***********************************************************************
     %increment counters - only increment if encountered another node
     rhandles.counterEncounters = rhandles.counterEncounters + (1.*rhandles.encounterYes);
     %check counters for update (if true will set to 1)
     updateYes = (rhandles.counterEncounters == rhandles.numEncounters);
     rhandles.updateGenotypeYes = updateYes;
     %**********************************************************************
     % Stop-start the update process (sol 4 and 5 only)
     %**********************************************************************
     %DynAlgClass.StopUpdateProcess(t,updateYes,ihandles,rhandles)
     %*********************************************************************
     % keep a running total of the number of updates performed
     %**********************************************************************
     rhandles.tallyUpdates = rhandles.tallyUpdates + updateYes;
     %**********************************************************************
     % Update Genotypes for all solutions
     %**********************************************************************
     %updateYes(:) = 0;
     %DynAlgClass.updateGenotypesAllSols(t,updateYes,ihandles,rhandles)
     %*********************************************************************
     % Reset counters and tallys
     %*********************************************************************
     %reset counters if needed, if reset resetCounters will be set to zero
     resetCounters = 1 - updateYes;
     rhandles.counterEncounters = rhandles.counterEncounters .* resetCounters;
     %reset the tallySol if needed
     resetCountersRep = repmat(resetCounters,1,ihandles.tallyLength);
     rhandles.tallySol1 = rhandles.tallySol1 .*resetCountersRep;
     rhandles.tallySol2 = rhandles.tallySol2 .*resetCountersRep;
     rhandles.tallySol3 = rhandles.tallySol3 .*resetCountersRep;
     rhandles.tallySol4 = rhandles.tallySol4 .*resetCountersRep;
     rhandles.tallySol5 = rhandles.tallySol5 .*resetCountersRep;
     %******************************************************************
     % reasign new output for all solutions
     %********************************************************************
     StaticDiversityClass.reasignNewOutputs(t,psim,ihandles,rhandles,ohandles)
     end%strcmp
 end
     %******************************************************************
     % reasign new output for all solutions
     %********************************************************************
     function reasignNewOutputs(t,psim,ihandles,rhandles,ohandles)
     %solution 1
     if ihandles.turnOnSol1 == 1
      numAlleles = [ihandles.len_apps ihandles.len_serv ihandles.len_os ihandles.len_dr];
     [ohandles.Nei_sol1(psim,t) ohandles.neiIndexSingleLocus_sol1(psim,t,:) ] = MeasureDiversityClass.getNeiIndex( rhandles.GenotypeSol1, numAlleles );
     end;
     %solution 2
     if ihandles.turnOnSol2 == 1
      numAlleles = [ihandles.len_apps ihandles.len_serv ihandles.len_os ihandles.len_dr];
     [ohandles.Nei_sol2(psim,t) ohandles.neiIndexSingleLocus_sol2(psim,t,:) ] = MeasureDiversityClass.getNeiIndex( rhandles.GenotypeSol2, numAlleles );
     end;
     %solution 3
     if ihandles.turnOnSol3 == 1
      numAlleles = [ihandles.len_apps ihandles.len_serv ihandles.len_os ihandles.len_dr];
     [ohandles.Nei_sol3(psim,t) ohandles.neiIndexSingleLocus_sol3(psim,t,:) ] = MeasureDiversityClass.getNeiIndex( rhandles.GenotypeSol3, numAlleles );
     end;
     %solution 4
     if ihandles.turnOnSol4 == 1
      numAlleles = [ihandles.len_apps ihandles.len_serv ihandles.len_os ihandles.len_dr];
     [ohandles.Nei_sol4(psim,t) ohandles.neiIndexSingleLocus_sol4(psim,t,:)] = MeasureDiversityClass.getNeiIndex( rhandles.GenotypeSol4, numAlleles );
     %neiIndexSingleLocus2(psim,t,:) = neiIndexSingleLocus
     ohandles.GenoSim_sol4(psim,t,:,:) = rhandles.GenotypeSol4; %added 31st March 2016
     end;
     %solution 5
     if ihandles.turnOnSol5 == 1
      numAlleles = [ihandles.len_apps ihandles.len_serv ihandles.len_os ihandles.len_dr];
     [ohandles.Nei_sol5(psim,t) ohandles.neiIndexSingleLocus_sol5(psim,t,:) ] = MeasureDiversityClass.getNeiIndex( rhandles.GenotypeSol5, numAlleles );
     ohandles.GenoSim_sol5(psim,t,:,:) = rhandles.GenotypeSol5; %added 31st March 2016
     end;
    end; %func

 function [genotype neiIndex neiIndexSingleLocus ] = genMaxDiversityGenotypes(numNodes,numVariantsArray)
     %numVariantsArray = [8 8 2 8]
         %get start and end points of loci
     lenArray = sum(numVariantsArray);
    [startP endP numLoci] = StaticDiversityClass.getStartAndEndLoci(numVariantsArray);
    genotype= zeros(numNodes,lenArray);
    for j=1:numLoci
        %genotype_temp(:,:,j) = StaticDiversityClass.genMaxDiversityOneLocus(numNodes,numVariantsArray(j))
        genotype(:,startP(j):endP(j)) = StaticDiversityClass.genMaxDiversityOneLocus(numNodes,numVariantsArray(j));
    end
    [ neiIndex neiIndexSingleLocus ] = StaticDiversityClass.checkNei(genotype, numVariantsArray);
 end
 function oneLocus = genMaxDiversityOneLocus(numNodes,numVariants)
    %TEST1
    %numNodes = 50
    %numVariants = 5
    oneLocus = zeros(numNodes,numVariants); %popNumber,numberOfAlleles,testNumber
    for a = 1:numVariants
        oneLocus((a:numVariants:end),a)=1;
    end
 end
  function [ startP endP numLoci] = getStartAndEndLoci( numAlleles )
    %numAlleles = [8 8 2 8]
    numLoci = size(numAlleles,2);
    endPi = zeros(1,numLoci+1);
    endP = zeros(1,numLoci);
    startP = zeros(1,numLoci);
    for i = 1:numLoci
        endPi(i+1) = endPi(i)+numAlleles(i);
        startP(i) = endPi(i)+1;
    end
    endP(1,:) = endPi(2:end);
  end
 
 function [ neiIndex neiIndexSingleLocus ] = checkNei(genotypeMatrix, numAlleles)
     [ neiIndex neiIndexSingleLocus ] = MeasureDiversityClass.getNeiIndex( genotypeMatrix, numAlleles )
 end
 
 function plotStatic(saveGraphs, closeAllGraphs, saveGraphsDirectoryName, simName, numNodes, numSims, numRuns, ...
               len_apps, len_serv, len_os, len_dr, ...
               Nei_sol1, Nei_sol2, Nei_sol3, Nei_sol4, Nei_sol5, ...
               turnOnSol1, turnOnSol2, turnOnSol3, turnOnSol4, turnOnSol5,...
               sumOfStates,beta)
    
    %choose to do beta graph, although bear in mind beta value not right as doesn't take into account actual number susceptible
    doBetaGraph = 0; %0 don't save
           
    %maximum possible diversity for number of nodes and variants
    len = size(Nei_sol4); %length of data
    NeiAbsMax = neiMax4(len_apps, len_serv, len_os, len_dr, numNodes);
    Nei_MaxL = repmat(NeiAbsMax,1,len(2)+1); %extend to length of data
    h1DivIndex = figure;
    plot(Nei_MaxL,'k')
    hold on;
    for psim = 1:numSims %for each run
        %plot options include:
    %1.plot solutions 1 to 5
    %2.plot solutions 4 & 5 only
    %3.plot solution 4 only (default option)
    if turnOnSol1==1 && turnOnSol2==1 && turnOnSol3==1 && turnOnSol4==1 && turnOnSol5==1
        plot(Nei_sol1(psim,:),'r--')
        hold on;
        plot(Nei_sol2(psim,:),'r')
        hold on;
        plot(Nei_sol3(psim,:),'c')
        meanSol3 = mean(Nei_sol3(psim,:))
        hold on;
    end;
    %plot solution 4
    plot(Nei_sol4(psim,:),'m')
    meanSol4 = mean(Nei_sol4(psim,:))
    hold on;
    %plot solution 5
    if turnOnSol4==1 && turnOnSol5==1
        plot(Nei_sol5(psim,:),'b')
        meanSol5 = mean(Nei_sol5(psim,:))
        hold on;
    end;
    end;
    xlabel('number of time steps')
    axis([0 numRuns 0 1])
    ylabel('diversity index')
    %legend
    if turnOnSol1==1 && turnOnSol2==1 && turnOnSol3==1 && turnOnSol4==1 && turnOnSol5==1
        legend('maximum','sol1', 'sol2', 'sol3', 'filtering', 'no filtering')
    elseif turnOnSol4==1 && turnOnSol5==0
        legend('maximum','filtering')
    elseif turnOnSol4==1 && turnOnSol5==1
        legend('maximum','filtering', 'no filtering')
    end
    
       %************************************************************
   % Attack graphs
   %************************************************************
    attackdataSize = size(sumOfStates) %size of data
    colourPlot = {'b' 'r' 'g' 'c' 'r--' 'm' 'k' 'y'}
    numRuns = attackdataSize(1)
    numStates = attackdataSize(2)
    numTimeSteps = attackdataSize(3)

    h11Attack = figure;
    %graphs
        for psim = 1:numRuns %for each run
            for stateNumber = 1:numStates %for each state
            %plot solution 4 only
             %b(1,:) = a(1,1,:)this code works
            plotStates(1,:) = sumOfStates(psim,stateNumber,:);
            plot(plotStates,char(colourPlot(stateNumber)))
            hold('on'); 
            end
        end
    
        xlabel('Number of Time Steps')
        ylabel('Number of Nodes')
        hold('on'); 
        %legend
            if numStates == 2
                legend('Susceptibles','Infected')
            elseif numStates ==3
                legend('Susceptibles','Infected','Recovered')
            elseif numStates == 4
                legend('Susceptibles','Exposed','Infected','Recovered')
            else
                legend('Susceptibles','Exposed','Prevented','Treated','Infected','Contained','Offline','eXposed off-line')
            end
            hold('on');
   %********************************
   %beta graphs   
   %*******************************
   if doBetaGraph == 1
   %beta test for 1 run only
        h12Beta = figure;
        betaSize = size(beta)
        plot(beta(psim,:))
        hold on
        golayFiltOrder=3;
        sampleSize=21;
        y=sgolayfilt(beta(1,:),golayFiltOrder,sampleSize,[],2);
        %mean of beta
        betaNaN = isnan(beta) + isinf(beta);%check for NaNs and infs to exclude
        sortBetaNaN = sort(betaNaN);%put NaNs at end of list
        sortBeta = sort(beta);%put NaNs at end of list
        [maxV maxi] = max(sortBetaNaN);%get index of first NaN where data finishes
        meanBeta = mean(sortBeta(1:maxi-1))%calculate the mean of non NaN data points
        meanBetarep = repmat(meanBeta,size(beta,2));
        %this way of working out mean just gets NaN
        %meanb=mean(outputs.beta(1,900:1500))
        %meanbrep = repmat(meanb,size(outputs.beta,2));
       % meany=mean(y(900:1500))
       %mean of y (filtered)
        yNaN = isnan(y) + isinf(y);%check for NaNs and infs to exclude
        sortyNaN = sort(yNaN);%put NaNs at end of list
        sorty = sort(y);%put NaNs at end of list
        [maxVy maxiy] = max(sortyNaN);%get index of first NaN where data finishes
        meany = mean(sorty(2:maxiy-1))%calculate the mean of non NaN data points
        meanyrep = repmat(meany,size(y,2));
       
        %meany=mean(y)
        %meanyrep = repmat(meany,size(y,2));
        
        plot(y(1,:),'k')
         hold on;
        plot(meanBetarep,'g')
        hold on;
        plot(meanyrep,'r')
        %hold on;
        legend('beta','filtered beta','mean beta', 'mean filtered');
   end
   %********************************
   %save graphs   
   %*******************************
   if saveGraphs ==1
   t = datestr(clock, 30);
   dateStr = [t];
   graphName1a = ['\' simName '_h1DivIndex.fig'];
   graphName1b = ['\' simName '_h1DivIndex.emf'];    
   graphName11a = ['\' simName 'h11Attack.fig'];
   graphName11b = ['\' simName 'h11Attack.emf'];  
   if doBetaGraph == 1
   graphName12a = ['\' simName 'h12Beta.fig'];
   graphName12b = ['\' simName 'h12Beta.emf']; 
   end
   filename1a = [saveGraphsDirectoryName graphName1a];
   filename1b = [saveGraphsDirectoryName graphName1b];
   filename11a = [saveGraphsDirectoryName graphName11a];
   filename11b = [saveGraphsDirectoryName graphName11b];
   if doBetaGraph == 1
   filename12a = [saveGraphsDirectoryName graphName12a];
   filename12b = [saveGraphsDirectoryName graphName12b];  
   end
   saveas(h1DivIndex,filename1a);
   saveas(h1DivIndex,filename1b);
   saveas(h11Attack,filename11a);
   saveas(h11Attack,filename11b);
   if doBetaGraph == 1
   saveas(h12Beta,filename12a);
   saveas(h12Beta,filename12b);
   end
    if closeAllGraphs ==1
       %close all graphs after saving
       close(h1DivIndex);
       close(h11Attack);
       if doBetaGraph==1; close(h12Beta); end;
   end;
 end
   end %func
 
    end %static
end %class
 
 
 
 
 
 
 
 
 
 
 
 
 
 