%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Jennifer Jackson 2017                                                        %
% A Biodiversity Approach to Cyber Security                                    %
% PhD Complexity Science                                                       %
% University of Warwick                                                        %
% http://wrap.warwick.ac.uk/98458                                              %
% This software is distributed under the  Creative Commons Attribution License %
% https://creativecommons.org/licenses/by/4.0/                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
classdef WirelessClass
%*************************************************************************
%List of functions to support wirelss network models
%*************************************************************************
%functions listed here:
%SEPTICOX Bluetooth model functions to model wireless networks
%function beta = calcContactRate(numNodes,linkProb,connectivity)
%function connectivity = calcConnectivity(density,rangeTx,timeStep,Tvirus)
%function linkProb = calcLinkProb(Tvirus,rangeTx,nodeVelocity,vMin,vMax,drawPlot)
%function y = atty(v,t,R,phi,v1,a,b)
%
%
%
%*************************************************************************
    properties
    end
    
    methods
    end
    %static methods that can be called without instantiating a class    
    methods (Static)
        %test to modify call by reference
        function testInput(inputHandles)
            inputHandles.density = 3;
        end
 %***********************************************************************
 % Function to get handle data and include in the model
 %***********************************************************************
 function WBSimInit(mainhandles,ihandles)
     %sim initialisations
     handles = guidata(mainhandles.networkModelParameters_wirelessBluetoothFig); %load data
     %wireless bluetooth
     ihandles.density =get(handles.editDensity,'Value');
     ihandles.avVelocity =get(handles.editAvVelocity,'Value');
     popupmenuTxRadiusSelection =get(handles.popupmenuTxRadius,'Value');
     ihandles.TxRadius = GUIClass.getTxRadius(popupmenuTxRadiusSelection);
     ihandles.Tvirus =get(handles.editTVirus,'Value');  
     guidata(mainhandles.networkModelParameters_wirelessBluetoothFig, handles); %save data
     drawPlot = 0;
     checkboxNetworkModelEnable = get(mainhandles.checkboxNetworkModelEnable,'Value');
     popupmenuNetworkModelV = get(mainhandles.popupmenuNetworkModel,'Value');
     networkModelS = GUIClass.getNetworkModelString(popupmenuNetworkModelV)
     if and(checkboxNetworkModelEnable,strcmp(networkModelS,'Wireless Bluetooth'))
     %if and(checkboxNetworkModelEnable == 1 ,popupmenuNetworkModel == 1)
     probOfEncounter = WirelessClass.bluetoothModel_betaCalc(ihandles.density,ihandles.avVelocity,ihandles.TxRadius,ihandles.Tvirus,drawPlot)
     %testing
     ihandlesdensity=ihandles.density
     ihandlesavVelocity=ihandles.avVelocity
     ihandlesTxRadius=ihandles.TxRadius
     ihandlesTvirus=ihandles.Tvirus
     
     ihandles.probabilityDistr = zeros(ihandles.numNodes,1);
     ihandles.probabilityDistr(:) = probOfEncounter;  
     end
 end
 function WBRunInit(ihandles,rhandles,ohandles)
     %run initialisations
 end

 %***********************************************************************
 %Bluetooth front end model for calculating probabilities of encounters
 %and virus transmission
 %***********************************************************************
 function beta = bluetoothModel_betaCalc(density,nodeVelocity,rangeTx,Tvirus,drawPlot) 
     %drawPlot = 1;
     vMin = 0.01;
     vMax = nodeVelocity*2; %assumes uniform distribution of velocities
numNodes = 400; %arbitary number as not actually used
    timeStep=2*rangeTx/nodeVelocity; %time for node to move full distance away
    connectivity = WirelessClass.calcConnectivity(density,rangeTx,timeStep,Tvirus)/timeStep;
    [linkProb PLink] = WirelessClass.calcLinkProb(Tvirus,rangeTx,nodeVelocity,vMin,vMax,drawPlot);
    [beta_nProportional beta_nProportionalLink] = WirelessClass.calcContactRate(numNodes,linkProb,connectivity,PLink); %includes the proportional bit of numNodes (1/N) * I
    %don't need to include the proportional bit for this model as infection
    %transfer is based on an actual contact rather than proportional assumption
    beta = real(beta_nProportional.*numNodes) %cancels out effect of the number of nodes used in the original SEPTICOX model
    betaLink = real(beta_nProportionalLink.*numNodes); %cancels out effect of the number of nodes used in the original SEPTICOX model
    if drawPlot ==1
        t=1:100;
        figure
        plot(t,betaLink);
        hold on
        plot(t(Tvirus),betaLink(Tvirus), 'm+');
        hold on;
        title('Probability of virus transmission beta (average across all nodes')
        xlabel('time needed for virus transmission')
        ylabel('probability of transmission')
    end
 end
 %****************************************************************
        
        
%******************************************************************
%Calculate contact rate
%******************************************************************
function [beta betaLink] = calcContactRate(numNodes,linkProb,connectivity,PLink)
%check for classes with no nodes in them
bi = linkProb.*connectivity
beta = (1./numNodes).*linkProb.*connectivity;
%for k=1:length(numNodes)
%if beta(k) == inf
%    beta(k) = 0 
%end
%end
betaLink = (1./numNodes).*PLink.*connectivity;

end    
%******************************************************************
%Calculate average connectivity (possible number of links in range)
%kav = min{7,(density*area of transmission) - its own possible link}
%******************************************************************
function connectivity = calcConnectivity(density,rangeTx,timeStep,Tvirus)
%average number of nodes in transmission range
%nodesTRange = (density.*pi.*rangeTx.^2) - 1;
%nodesTRange = (density.*pi.*rangeTx.^2)
nodesTRange = (density.*4.*rangeTx.^2)%changed JTD 3rd October 2011
%check to see if nodesTRange is less than 1
%if nodesTRange < 1
%    nodesTRange = 0;
%end
%average possible connectivity, note that piconets can only have a maximum
%of 8 nodes or 7 links
%connectivityMax = min(7,nodesTRange)
%note that can only transmit a certain number of viruses in a given time
%step as the node can not transmit simultaneously. The number will depend
%upon the time step length and the virus transmission time.
%maximum possible transmissions in a single time step
maxTransmissions = timeStep./Tvirus
connectivity = min(nodesTRange,maxTransmissions)
end

%calculate the probability of virus transmission across a single link
function [linkProb PLink] = calcLinkProb(Tvirus,rangeTx,nodeVelocity,vMin,vMax,drawPlot)
%Inputs
%1.Tvirus - time needed to transmit the virus across a link, minimum of 1
%second
%2.rangeTx - transmission radius of nodes
%3.nodeVelocity - average velocity of current node
%4.vMin - minimum velocity of current node, must be at least 0.01
%5.vMax - maximum velocity of current node
%6.drawPlot - specifies if a graph showing the CDF should be drawn

%parameters to delete
%rangeTx = 250; %transmission radius
%Tvirus = 3 ; %time for virus to transmit to a node
%nodeVelocity = 10;
%vMin = 0.01;
%vMax = 10;
%drawPlot=1;

%Parameters to change in code
%v1=nodeVelocity;
%R=rangeTx;
%a=vMin;
%b=vMax;

%*********************************
%fixed parameters
%*********************************
%integral bounds for node velocity:
%minimum relative velocity between current node and another to
%form link, must be at least 0.01
vlow=0.01;
%maximum relative velocity between current node and another to
%form link,
vupp=2.*rangeTx./Tvirus;
%integral bounds for phi - angle between two nodes:
%minimum angle between current node and another to
%form link, must be at least 0.01
philow=0.01;
%maximum angle between current node and another to
%form link
phiupp=pi;

%******************************************************************
%Calculate the probability of virus transmission for a single link
%******************************************************************
numClasses = length(Tvirus)
for i = 1:numClasses
%define annonymous function, v and phi are the integration parameters
F=@(v,phi)WirelessClass.atty(v,Tvirus(i),rangeTx(i),phi,nodeVelocity(i),vMin(i),vMax(i));
%numerically integrate over all possible velocity values and angles
Q=dblquad(F,vlow,vupp(i),philow,phiupp);
%link Probability
linkProb(i) = ((1./(pi.*(vMax(i)-vMin(i)))).*Q);
end
%*****************************************************
%Draw extra graphs if requested
%*****************************************************
if drawPlot == 1
for t=1:100
    %define annonymous function, v and phi are the integration parameters
    F=@(v,phi)WirelessClass.atty(v,t,rangeTx,phi,nodeVelocity,vMin,vMax);
    %numerically integrate over all possible velocity values and angles for
    %each time step
    Q=dblquad(F,vlow,vupp,philow,phiupp);
    %link CDF
    CDFlink(t) = 1 - ((1./(pi.*(vMax-vMin))).*Q);
    %probability of virus transmission
    PLink(t) = (1./(pi.*(vMax-vMin))).*Q;
end
s=1:100;
%Draw CDF for a given velocity
figure;
plot(s,CDFlink,'b');
hold on;
title('CDF of link lifetime')
%Draw virus transmission probability for a given velocity
figure
plot(s,PLink,'m');
hold on;
plot(s(Tvirus),PLink(Tvirus),'b+');
hold on;
title('Probability of virus transmission across a single link - PLink')
xlabel('time needed for virus transmission')
ylabel('probability of transmission')
else
    PLink = zeros(1,100);
    CDFlink = zeros(1,100);
end
end
%**********************************************************
function y = atty(v,t,R,phi,v1,a,b)
gh=sqrt(1-((v.*t)./(2.*R)).^2);
gj = WirelessClass.g(v,phi,v1,a,b);
y = v.*gh.*gj;
end
%**********************************************************
function y = g(v,phi,v1,a,b)
h=sqrt(v.^2+v1.^2+(2.*v.*v1.*cos(phi)));
y1=((h-a) > 0);%heaviside without zero test
y2=((h-b) > 0);%heaviside without zero test
y=(y1-y2)./(h+realmin);
end
%**********************************************************
    end %end static methods
    
end

