%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Jennifer Jackson 2017                                                        %
% A Biodiversity Approach to Cyber Security                                    %
% PhD Complexity Science                                                       %
% University of Warwick                                                        %
% http://wrap.warwick.ac.uk/98458                                              %
% This software is distributed under the  Creative Commons Attribution License %
% https://creativecommons.org/licenses/by/4.0/                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function displayGUIGraph(outputs,handles)
%display graph in axes
%1.outputs - output data to display
%2.handles - handles info

graphType = get(handles.popupmenuGraphType,'Value');

%**************************************************************************
%pre-graph calculations
%work out some original diversity metrics, and that of the different output
%solutions and plot them on a graph
plotData = 0; %for plotting sub functions
len = size(outputs.Nei_sol4); %length of data

%maximum possible diversity for number of nodes and variants
NeiAbsMax = neiMax4(outputs.len_apps, outputs.len_serv, outputs.len_os, outputs.len_dr, outputs.numNodes)
Nei_MaxL = repmat(NeiAbsMax,1,len(2)+1); %extend to length of data

%original diversity of the network (saved as an output)
Genotype = [outputs.Genotype_apps outputs.Genotype_serv outputs.Genotype_os outputs.Genotype_dr];
data = Genotype';
%Nei_orig = variantFreq4(outputs.textdata, data, plotData)

numAlleles = [outputs.len_apps outputs.len_serv outputs.len_os outputs.len_dr];
[Nei_orig Nei_orig_singleLocus] = MeasureDiversityClass.getNeiIndex(Genotype, numAlleles);
    
Nei_origL = repmat(Nei_orig,1,len(2)+1); %extend to length of data


%STILL SEEMS TO DISPLAY THIS EVEN WHEN RUNNING STATIC DIVERSITY
%individual desirabilities diversity
data = outputs.ID;

Nei_ind = variantFreq4(outputs.textdata, data, plotData)
Nei_indLog = log10(1-Nei_ind)
Nei_indL = repmat(Nei_ind,1,len(2)+1);  %extend to length of data
if outputs.ChangeIndDesirabilitiesFlag == 1
%if change part way through
data2 = outputs.ID2;
Nei_ind2 = variantFreq4(outputs.textdata, data2, plotData)
Nei_indL(1,outputs.tChangeInd1+1:outputs.tChangeInd2) = Nei_ind2;
end

%diversity of the community desirabilities
Nei_comm = communityDiversity(outputs.CD_apps,outputs.CD_serv,outputs.CD_os,outputs.CD_dr)
Nei_comLog = log10(1-Nei_comm)
Nei_commL = repmat(Nei_comm,1,len(2)+1); %extend to length of data
%if change part way through
if outputs.ChangeDesirabilitiesFlag == 1
Nei_comm2 = communityDiversity(outputs.CD_apps2,outputs.CD_serv2,outputs.CD_os2,outputs.CD_dr2)
Nei_comLog2 = log10(1-Nei_comm2)
Nei_commL2 = repmat(Nei_comm2,1,outputs.tChange2-outputs.tChange1); %extend to length of data
%if change back again
Nei_comm3 = communityDiversity(outputs.CD_apps,outputs.CD_serv,outputs.CD_os,outputs.CD_dr)
Nei_comLog3 = log10(1-Nei_comm3)
Nei_commL3 = repmat(Nei_comm3,1,len(2)+1-outputs.tChange2); %extend to length of data
size(Nei_commL2)
size(Nei_commL3)
size(Nei_commL(outputs.tChange1+1:outputs.tChange2))
size(Nei_commL(outputs.tChange2+1:len(2)+1))
Nei_commL(1,outputs.tChange1+1:outputs.tChange2) = Nei_commL2;
Nei_commL(1,outputs.tChange2+1:len(2)+1) = Nei_commL3;
end;

%diversity of the efficiency
%high efficiency diversity means all the efficiency ratings are similar.
%low efficiency diversity means one, or a small proportion are far more
%efficient than the rest
Nei_eff = efficiencyDiversity(outputs.E_apps,outputs.E_serv,outputs.E_os,outputs.E_dr)
Nei_effL = repmat(Nei_eff,1,len(2)+1);
%if change part way through
if outputs.ChangeEffFlag == 1
Nei_eff2 = efficiencyDiversity(outputs.E_apps2,outputs.E_serv2,outputs.E_os2,outputs.E_dr2);
Nei_eff2Log2 = log10(1-Nei_eff2);
Nei_eff2L2 = repmat(Nei_eff2,1,outputs.tChangeEff2-outputs.tChangeEff1); %extend to length of data
%if change back again
Nei_eff3 = efficiencyDiversity(outputs.E_apps,outputs.E_serv,outputs.E_os,outputs.E_dr);
Nei_eff3Log3 = log10(1-Nei_eff3);
Nei_eff3L3 = repmat(Nei_eff3,1,len(2)+1-outputs.tChangeEff2); %extend to length of data
size(Nei_eff2L2);
size(Nei_eff3L3);
size(Nei_effL(outputs.tChangeEff1+1:outputs.tChangeEff2));
size(Nei_effL(outputs.tChangeEff2+1:len(2)+1));
Nei_effL(1,outputs.tChangeEff1+1:outputs.tChangeEff2) = Nei_eff2L2;
Nei_effL(1,outputs.tChangeEff2+1:len(2)+1) = Nei_eff3L3;
end;

%get solutions to start from original diversity
%Nei_sol1_temp(1:outputs.numRuns,1) = Nei_orig;
%Nei_sol1_temp(:,2:outputs.numTimeSteps+1)=outputs.Nei_sol1;
%Nei_sol1=Nei_sol1_temp;

%Nei_sol2_temp(1:outputs.numRuns,1) = Nei_orig;
%Nei_sol2_temp(:,2:outputs.numTimeSteps+1)=outputs.Nei_sol2;
%Nei_sol2=Nei_sol2_temp;

%Nei_sol3_temp(1:outputs.numRuns,1) = Nei_orig;
%sizeoutputsNei_sol3 = size(outputs.Nei_sol3)
%Nei_sol3_temp(:,2:outputs.numTimeSteps+1)=outputs.Nei_sol3;
%Nei_sol3=Nei_sol3_temp;

Nei_sol4_temp(1:outputs.numRuns,1) = Nei_orig
sizeoutputsNei_sol4 = size(outputs.Nei_sol4)
sizeoutputsNei_sol5 = size(outputs.Nei_sol5)
outputs.numRuns
Nei_sol4_temp(:,2:outputs.numTimeSteps+1)=outputs.Nei_sol4;
Nei_sol4=Nei_sol4_temp;

Nei_sol5_temp(1:outputs.numRuns,1) = Nei_orig;
Nei_sol5_temp(:,2:outputs.numTimeSteps+1)=outputs.Nei_sol5;
Nei_sol5=Nei_sol5_temp;
%*****************************************
%release graph from previous data to refresh plot
hold(handles.axesGraph,'off') 
%graphs
graphTypeD=graphType
switch (graphType)
    case 1
        %'Diversity Index'
        %axesGraph
        %h1DivIndex = figure;
        %figure(handles.axesGraph)
         %plot output        
       %example from miniproject plot(handles.axes1,t./simTimeAxes,St,'b');
        
        %set(0,'CurrentFigure',handles.axesGraph)
        %clear('handles.axesGraph')
        plot(handles.axesGraph,Nei_MaxL,'k')
        hold(handles.axesGraph,'on'); 
        %hold on;
        plot(handles.axesGraph,Nei_origL,'g')
        hold(handles.axesGraph,'on'); 
        %hold on;
        plot(handles.axesGraph,Nei_commL,'b--')
        hold(handles.axesGraph,'on'); 
        %hold on;
        plot(handles.axesGraph,Nei_indL,'c--')
        hold(handles.axesGraph,'on'); 
        %hold on;
        plot(handles.axesGraph,Nei_effL,'g--')
        hold(handles.axesGraph,'on'); 
        %hold on;
        outputs.numRuns
        for psim = 1:outputs.numRuns
            %plot options include:
            %1.plot solutions 1 to 5
            %2.plot solutions 4 & 5 only
            %3.plot solution 4 only (default option)
            %if outputs.turnOnSol1==1 && outputs.turnOnSol2==1 && outputs.turnOnSol3==1 && outputs.turnOnSol4==1 && outputs.turnOnSol5==1
            %    plot(handles.axesGraph,Nei_sol1(psim,:),'r--')
            %    hold(handles.axesGraph,'on') ;
            %    plot(handles.axesGraph,Nei_sol2(psim,:),'r')
            %    hold(handles.axesGraph,'on'); 
            %    plot(handles.axesGraph,Nei_sol3(psim,:),'c')
            %    meanSol3 = mean(Nei_sol3(psim,:))
            %    hold(handles.axesGraph,'on'); 
            %end;
            %plot solution 4
            plot(handles.axesGraph,Nei_sol4(psim,:),'m')
            meanSol4 = mean(Nei_sol4(psim,:))
            hold(handles.axesGraph,'on'); 
            %plot solution 5
            if outputs.turnOnSol4==1 && outputs.turnOnSol5==1
                plot(handles.axesGraph,Nei_sol5(psim,:),'b')
                meanSol5 = mean(Nei_sol5(psim,:))
                hold(handles.axesGraph,'on'); 
            end;
        end;
        xlabel(handles.axesGraph,'Number of Time Steps')
        axis(handles.axesGraph,[0 outputs.numTimeSteps 0 1])
        ylabel(handles.axesGraph,'Diversity Index')
        hold(handles.axesGraph,'on'); 
        %legend
            if outputs.turnOnSol1==1 && outputs.turnOnSol2==1 && outputs.turnOnSol3==1 && outputs.turnOnSol4==1 && outputs.turnOnSol5==1
                legend(handles.axesGraph,'max','orig','comm','ind', 'eff', 'sol1', 'sol2', 'sol3', 'sol4', 'sol5')
            elseif outputs.turnOnSol4==1 && outputs.turnOnSol5==0
                legend(handles.axesGraph,'max','orig','comm','ind', 'eff', 'sol4')
            elseif outputs.turnOnSol4==1 && outputs.turnOnSol5==1
                legend(handles.axesGraph,'max','orig','comm','ind', 'eff', 'sol4', 'sol5')
            end
            hold(handles.axesGraph,'on'); 
    
    case 2
        %'Diversity Index Log'
        %h2DivIndexLog = figure;
        plot(handles.axesGraph,log10(1 -Nei_origL),'g')
        hold(handles.axesGraph,'on');
        plot(handles.axesGraph,log10(1 - Nei_commL),'b--')
        hold(handles.axesGraph,'on');
        plot(handles.axesGraph,log10(1 -Nei_indL),'c--')
        hold(handles.axesGraph,'on');
        plot(handles.axesGraph,log10(1 -Nei_effL),'g--')
        hold(handles.axesGraph,'on');
        plot(handles.axesGraph,log10(1 - Nei_MaxL),'k')
        hold(handles.axesGraph,'on');
        for psim = 1:outputs.numRuns
            %plot options include:
            %1.plot solutions 1 to 5
            %2.plot solutions 4 & 5 only
            %3.plot solution 4 only (default option)
           % if outputs.turnOnSol1==1 && outputs.turnOnSol2==1 && outputs.turnOnSol3==1 && outputs.turnOnSol4==1 && outputs.turnOnSol5==1
           %     plot(handles.axesGraph,log10(1- Nei_sol1(psim,:)),'r')
           %     hold(handles.axesGraph,'on');
           %     plot(handles.axesGraph,log10(1-Nei_sol2(psim,:)),'r--')
           %     hold(handles.axesGraph,'on');
           %     plot(handles.axesGraph,log10(1- Nei_sol3(psim,:)),'c')
           %     meanSol3 = mean(Nei_sol3(psim,:))
           %     hold(handles.axesGraph,'on');
           % end;
            %plot solution 4
            a = Nei_sol4(psim,:);
            loga = log10(1-(a));
            plot(handles.axesGraph,log10(1-(a)),'m')
            meanSol4 = mean(Nei_sol4(psim,:))
            meanSol4Log = mean(loga(1,:))
            hold(handles.axesGraph,'on');
            %plot solution 5
            if outputs.turnOnSol4==1 && outputs.turnOnSol5==1
                a = Nei_sol5(psim,:);
                loga = log10(1-(a));
                plot(handles.axesGraph,log10(1-(a)),'b')
                meanSol5 = mean(Nei_sol5(psim,:))
                meanSol5Log = mean(loga(1,:))
                hold(handles.axesGraph,'on');
            end;
        end;
        xlabel(handles.axesGraph,'Number of Time Steps')
        axis(handles.axesGraph,[0 outputs.numTimeSteps -1.4 0])
        ylabel(handles.axesGraph,'Log (1-Diversity Index)')
        %legend
            if outputs.turnOnSol1==1 && outputs.turnOnSol2==1 && outputs.turnOnSol3==1 && outputs.turnOnSol4==1 && outputs.turnOnSol5==1
                legend(handles.axesGraph,'orig','comm','ind', 'eff', 'max', 'sol1', 'sol2', 'sol3', 'sol4', 'sol5')
            elseif outputs.turnOnSol4==1 && outputs.turnOnSol5==0
                legend(handles.axesGraph,'orig','comm','ind', 'eff', 'max', 'sol4')
            elseif outputs.turnOnSol4==1 && outputs.turnOnSol5==1
                legend(handles.axesGraph,'orig','comm','ind', 'eff', 'max', 'sol4', 'sol5')
            end
    case 3
        %'Individual Diversities'
        %plot individual diversities of nodes
        %h3IndDiv = figure;
        sizenodesDiversity4 = size(outputs.nodesDiversity4);
        for psim = 1:outputs.numRuns
            nodesDiversity4(:,:) =   outputs.nodesDiversity4(psim,:,:);
            sizenodesDiversity4 = size(nodesDiversity4);
            plot(handles.axesGraph,nodesDiversity4(1,:),'g')
            hold(handles.axesGraph,'on');
            plot(handles.axesGraph,nodesDiversity4(2,:),'r')
            hold(handles.axesGraph,'on');
            plot(handles.axesGraph,nodesDiversity4(3,:),'b')
            hold(handles.axesGraph,'on');
            plot(handles.axesGraph,nodesDiversity4(4,:),'c')
            hold(handles.axesGraph,'on');
            plot(handles.axesGraph,nodesDiversity4(5,:),'m')
            hold(handles.axesGraph,'on');
            plot(handles.axesGraph,nodesDiversity4(6,:),'g')
            hold(handles.axesGraph,'on');
            plot(handles.axesGraph,nodesDiversity4(7,:),'r')
            hold(handles.axesGraph,'on');
            plot(handles.axesGraph,nodesDiversity4(8,:),'b')
            hold(handles.axesGraph,'on');
            plot(handles.axesGraph,nodesDiversity4(9,:),'c')
            hold(handles.axesGraph,'on');
            plot(handles.axesGraph,nodesDiversity4(10,:),'m')
            hold(handles.axesGraph,'on');
        end
            axis(handles.axesGraph,[0 outputs.numTimeSteps 0 1])
            xlabel(handles.axesGraph,'Number of Time Steps')
            ylabel(handles.axesGraph,'Individual Diversities')
    case 4
        %'Individual Diversities Log'
        %h4IndDivLog = figure;
        for psim = 1:outputs.numRuns
            nodesDiversity4(:,:) =  outputs.nodesDiversity4(psim,:,:);
            nodesDiversityLog = log10(1-nodesDiversity4);
            plot(handles.axesGraph,nodesDiversityLog(1,:),'g')
            hold(handles.axesGraph,'on');
            plot(handles.axesGraph,nodesDiversityLog(2,:),'r')
            hold(handles.axesGraph,'on');
            plot(handles.axesGraph,nodesDiversityLog(3,:),'b')
            hold(handles.axesGraph,'on');
            plot(handles.axesGraph,nodesDiversityLog(4,:),'c')
            hold(handles.axesGraph,'on');
            plot(handles.axesGraph,nodesDiversityLog(5,:),'m')
            hold(handles.axesGraph,'on');
            plot(handles.axesGraph,nodesDiversityLog(6,:),'g')
            hold(handles.axesGraph,'on');
            plot(handles.axesGraph,nodesDiversityLog(7,:),'r')
            hold(handles.axesGraph,'on');
            plot(handles.axesGraph,nodesDiversityLog(8,:),'b')
            hold(handles.axesGraph,'on');
            plot(handles.axesGraph,nodesDiversityLog(9,:),'c')
            hold(handles.axesGraph,'on');
            plot(handles.axesGraph,nodesDiversityLog(10,:),'m')
            hold(handles.axesGraph,'on');
        end
            axis(handles.axesGraph,[0 outputs.numTimeSteps -1.4 0])
            xlabel(handles.axesGraph,'Number of Time Steps')
            ylabel(handles.axesGraph,'Log10(1-Individual Diversities)')
    case 5
        %'Node 1 mean Alg 4'
        %plot the mean of the individual diversities
        %h5Means = figure;
        c='g';
         for psim = 1:outputs.numRuns
             if c == 'g'; c='r'; else c='g'; end;
            nodesMean4(:,:) =  outputs.nodesMean4(psim,:,:);
            plot(handles.axesGraph,nodesMean4(1,:),c)
            hold(handles.axesGraph,'on');
         end
        %axis([0 numRuns 0 1])
        xlabel(handles.axesGraph,'Number of Time Steps')
        ylabel(handles.axesGraph,'Mean of Individual Diversities')
    case 6
        %'Node 1 standard deviation Alg 4'
        %plot the std of the individual diversities
       %h6Stds = figure;
       c='g';
       for psim = 1:outputs.numRuns
           if c == 'g'; c='r'; else c='g'; end;
            nodesStd4(:,:) =  outputs.nodesStd4(psim,:,:);
            plot(handles.axesGraph,nodesStd4(1,:),c)
            hold(handles.axesGraph,'on');
       end
        %axis([0 numRuns 0 1])
        xlabel(handles.axesGraph,'Number of Time Steps')
        ylabel(handles.axesGraph,'Standard Deviation of Individual Diversities')
    case 7
        %'Node 1 stop/start update Alg 4'
           %figure for solution 4 showing the standard deviation and when the 
           %genotype has stopped updating for 1 node
           if outputs.turnOnSol4 == 1
                %h7Sol4StopStart=figure;
                %for i = 1:numNodes
                c='g';
                for psim = 1:outputs.numRuns
                    if c == 'g'; c='c'; else c='g'; end;
                    updateStopDisplay4(:,:) =  outputs.updateStopDisplay4(psim,:,:);
                    nodesStd4(:,:) =  outputs.nodesStd4(psim,:,:);
                    minStd4Curve(:,:) =  outputs.minStd4Curve(psim,:,:);
                    for i = 1:1
                    % 1 when updating, 0 when stopped
                    plot(handles.axesGraph,updateStopDisplay4(:,i));
                    hold(handles.axesGraph,'on');
                    %standard deviation
                    plot(handles.axesGraph,nodesStd4(i,:),c);
                    hold(handles.axesGraph,'on');
                    end;
                    %max and min lines on graph
                    minStd4Display = repmat(outputs.minstd4Tolerance,outputs.numTimeSteps,1);
                    maxStd4Display = repmat(outputs.maxstd4Tolerance,outputs.numTimeSteps,1);
                    %max std line
                    plot(handles.axesGraph,maxStd4Display,'m');
                    hold(handles.axesGraph,'on');
                    %tracking the minimum std threshold value as it changes
                    plot(handles.axesGraph,minStd4Curve(1,:),'y');
                    hold(handles.axesGraph,'on');
                    % original std min line
                    plot(handles.axesGraph,minStd4Display,'r');
                    hold(handles.axesGraph,'on');
               end;
           axis(handles.axesGraph,[0 outputs.numTimeSteps -0.1 1.1])
           xlabel(handles.axesGraph,'Number of Time Steps')
           ylabel(handles.axesGraph,'Standard Deviation of Individual Diversities')
           %title('solution 4 - stop/start update')
           legend(handles.axesGraph,'stop-start', 'std','maxStd','minStdTrack','minStd')
           end;
    case 8
           %'Node 1 stop/start update Alg 5'
           %figure for solution 5 showing the standard deviation and when the 
           %genotype has stopped updating for 1 node
           if outputs.turnOnSol5 == 1
          %h8Sol5StopStart = figure;
           %for i = 1:numNodes
                c='g';
                for psim = 1:outputs.numRuns
                    if c == 'g'; c='c'; else c='g'; end;
                    updateStopDisplay5(:,:) =  outputs.updateStopDisplay5(psim,:,:);
                    nodesStd5(:,:) =  outputs.nodesStd5(psim,:,:);
                    minStd5Curve(:,:) =  outputs.minStd5Curve(psim,:,:);
                    for i = 1:1
                    % 1 when updating, 0 when stopped
                    plot(handles.axesGraph,updateStopDisplay5(:,i));
                    hold(handles.axesGraph,'on');
                    plot(handles.axesGraph,nodesStd5(i,:),c);
                    hold(handles.axesGraph,'on');
                    end;
                    %max and min lines on graph
                    minStd5Display = repmat(outputs.minstd5Tolerance,outputs.numTimeSteps,1);
                    maxStd5Display = repmat(outputs.maxstd5Tolerance,outputs.numTimeSteps,1);
                    plot(handles.axesGraph,maxStd5Display,'m');
                    hold(handles.axesGraph,'on');
                    plot(handles.axesGraph,minStd5Curve(1,:),'y');
                    hold(handles.axesGraph,'on');
                    plot(handles.axesGraph,minStd5Display,'r');
                    hold(handles.axesGraph,'on');
                end
           axis(handles.axesGraph,[0 outputs.numTimeSteps -0.1 1.1])
           xlabel(handles.axesGraph,'Number of Time Steps')
           ylabel(handles.axesGraph,'Standard Deviation of Individual Diversities')
           %title('solution 5 - stop/start update')
           legend(handles.axesGraph,'stop-start', 'std','maxStd','minStdTrack','minStd')
           end;     
    otherwise
        disp('graph error')
end