%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Jennifer Jackson 2017                                                        %
% A Biodiversity Approach to Cyber Security                                    %
% PhD Complexity Science                                                       %
% University of Warwick                                                        %
% http://wrap.warwick.ac.uk/98458                                              %
% This software is distributed under the  Creative Commons Attribution License %
% https://creativecommons.org/licenses/by/4.0/                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function Nei = efficiencyDiversity(E_apps,E_serv,E_os,E_dr)
%high efficiency diversity means all the efficiency ratings are similar.
%low efficiency diversity means one, or a small proportion are far more
%efficient than the rest


%TEST INPUTS
%efficiency/quality ratings of software (1 to 100) 
%(1)inefficient (100)maximum efficiency
%E_apps = [65 75 55 50 60 60 60 80 20 20 65 75 55 50 60 60 60 80 20 20];
%E_serv = [65 66 67 63 59 62 72 77 50 50 65 75 55 50 60 60 60 80 20 20];
%E_os = [50 50 50 50 50 50 50 50 50 50];
%E_dr = [70 80 60 75 69 68 63 58 65 65 65 75 55 50 60 60 60 80 20 20];

%MAIN CODE

%algorithm
%work out the frequencies of each
freqvarApps = E_apps./sum(E_apps);
freqvarServ = E_serv./sum(E_serv);
freqvarOS = E_os./sum(E_os);
freqvarDr = E_dr./sum(E_dr);

%determine (h) heterozygosity = 1-sum(freq.^2)
hApps = 1 - sum(freqvarApps.^2);
hServ = 1 - sum(freqvarServ.^2);
hOS = 1 - sum(freqvarOS.^2);
hDr = 1 - sum(freqvarDr.^2);

%determine Nei genetic diversity index sum(h)/number of loci
Nei = (hApps + hServ + hOS + hDr)/4;
