%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Jennifer Jackson 2017                                                        %
% A Biodiversity Approach to Cyber Security                                    %
% PhD Complexity Science                                                       %
% University of Warwick                                                        %
% http://wrap.warwick.ac.uk/98458                                              %
% This software is distributed under the  Creative Commons Attribution License %
% https://creativecommons.org/licenses/by/4.0/                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%********************************************************************
%Dynamic Weights File
%Weights for maximise diversity
%********************************************************************
%1. FORMAT
%set matrix size: weights = zeros(numNodes,numChanges,numChanges)
%                           = zeros(numNodes,numTimes, numValues)

%*********************************************************************
%2. SET VALUES:

%example to set individual node:
%weights(2,:,:) = [10 0.2; 20 0.25]
%node 2 set for time1 = 10, value1 = 0.2, time2 = 20, value2 = 0.25

%example to copy node values:
% divWeights(3:4,:,:) = divWeights(1:2,:,:)

%example to set multiple nodes the same value using repmat:
%B = repmat(A,[m n p...]) produces a multidimensional array B composed of 
%copies of A. The size of B is [size(A,1)*m, size(A,2)*n, size(A,3)*p, ...].
% c = zeros(1,2,2) - set up 1 node
% c(1,:,:) = [10 0.2; 20 0.25] - set value
% d = repmat(c,[5,1,1]) - extend to all 5 nodes
%************************************************************************

function comWeights = getComWeights(numNodes,caseNo,weightFile)
switch caseNo
    case 1
        %get values from file
        directorynameWeights = GUIClass.getDirectoryNameWeights();
        fileName = [directorynameWeights weightFile];
        comWeights = DynAlgClass.getWeightsFromFile(fileName,numNodes);
    otherwise
        %get values from function
        numChanges = 2;
        comWeights = zeros(numNodes,numChanges,numChanges);

        %Set all nodes the same
        c = zeros(1,2,2); % set up 1 node
        c(1,:,:) = [1 0.2; 20 0.25];
        d = repmat(c,[numNodes,1,1]); %extend to all nodes
        comWeights = d;
end
end

