%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Jennifer Jackson 2017                                                        %
% A Biodiversity Approach to Cyber Security                                    %
% PhD Complexity Science                                                       %
% University of Warwick                                                        %
% http://wrap.warwick.ac.uk/98458                                              %
% This software is distributed under the  Creative Commons Attribution License %
% https://creativecommons.org/licenses/by/4.0/                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function NeiAbsMax = neiMax4(len_apps, len_serv, len_os, len_dr, numNodes)
%works with 4 different locus: apps, serv, OS, dr
%(1) finds the absolute maximum diversity value 

Va=len_apps; 
Vs=len_serv; 
Vo=len_os; 
Vd=len_dr; 
N=numNodes;
 
 %Absolute maximum diversity per locus
 %if N (numNodes) < V (NumVariants), then h per locus = 1 - 1/N
 %if N > V, then h = 1 - [(V-r)f^2 + r(f+1)^2 ]/N^2
 %locusApps
 fa=fix(N/Va); %round down to nearest integer
 ra=rem(N,Va); %remainder
 if N < Va
     ha = 1 - 1/N;
 else
     htemp1 = (Va-ra)*fa^2;
     htemp2 = ra*(fa+1)^2;
     ha = 1 - (htemp1 + htemp2)/N^2;
 end;
 %locusServ
 fs=fix(N/Vs);
 rs=rem(N,Vs);
 if N < Vs
     hs = 1 - 1/N;
 else
     htemp1 = (Vs-rs)*fs^2;
     htemp2 = rs*(fs+1)^2;
     hs = 1 - (htemp1 + htemp2)/N^2;
 end;
 %locusOS
 fo=fix(N/Vo);
 ro=rem(N,Vo);
 if N < Vo
     ho = 1 - 1/N;
 else
     htemp1 = (Vo-ro)*fo^2;
     htemp2 = ro*(fo+1)^2;
     ho = 1 - (htemp1 + htemp2)/N^2;
 end;
 %locusDr
 fd=fix(N/Vd);
 rd=rem(N,Vd);
 if N < Vd
     hd = 1 - 1/N;
 else
     htemp1 = (Vd-rd)*fd^2;
     htemp2 = rd*(fd+1)^2;
     hd = 1 - (htemp1 + htemp2)/N^2;
 end;
 
 NeiAbsMax = (ha + hs + ho + hd)/4;
 





