%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Jennifer Jackson 2017                                                        %
% A Biodiversity Approach to Cyber Security                                    %
% PhD Complexity Science                                                       %
% University of Warwick                                                        %
% http://wrap.warwick.ac.uk/98458                                              %
% This software is distributed under the  Creative Commons Attribution License %
% https://creativecommons.org/licenses/by/4.0/                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [N_apps,N_serv,N_os,N_dr] = normToOS( normVal,...
    CSW1,CSW2,CSW3,CSW4,CSW5,CSW6,CSW7,CSW8,CSW9,CSW10, ...
    X_apps,X_serv,X_os,X_dr)
%normToOS
%This function normalises the efficiency, community, or individual software
%input data to the compatable operating systems. This is then used by the
%main code to determine the expected diversity of the filtered version.
%This is then drawn on the graph along with the ideal unfiltered case.

%normVal - can be 1(normalise to 1) or 100 (normalise to 100)
% if it is set incorrectly it will automatically normalise to 1

%check number of OS
numOS = length(X_os);

%1.normalise to 100 if not done so already
sumX_apps = sum(X_apps);
sumX_serv = sum(X_serv);
sumX_os = sum(X_os);
sumX_dr = sum(X_dr);
X_apps = (X_apps./sumX_apps .* 100);
X_serv = (X_serv./sumX_serv .* 100);
X_os = (X_os./sumX_os .* 100);
X_dr = (X_dr./sumX_dr .* 100);
%check add up to 100
sumX_apps = sum(X_apps);
sumX_serv = sum(X_serv);
sumX_os = sum(X_os);
sumX_dr = sum(X_dr);

%apps
%filters out apps to OS1 to 10
CSW1_apps = CSW1(1:length(X_apps)) .* X_apps; %os1
CSW1_appsSum= sum(CSW1_apps); %sum efficiencies
CSW2_apps = CSW2(1:length(X_apps)) .* X_apps; %os2
CSW2_appsSum= sum(CSW2_apps); %sum efficiencies
CSW3_apps = CSW3(1:length(X_apps)) .* X_apps; %os3
CSW3_appsSum= sum(CSW3_apps); %sum efficiencies
CSW4_apps = CSW4(1:length(X_apps)) .* X_apps; %os4
CSW4_appsSum= sum(CSW4_apps); %sum efficiencies
CSW5_apps = CSW5(1:length(X_apps)) .* X_apps; %os5
CSW5_appsSum= sum(CSW5_apps); %sum efficiencies
CSW6_apps = CSW6(1:length(X_apps)) .* X_apps; %os6
CSW6_appsSum= sum(CSW6_apps); %sum efficiencies
CSW7_apps = CSW7(1:length(X_apps)) .* X_apps; %os7
CSW7_appsSum= sum(CSW7_apps); %sum efficiencies
CSW8_apps = CSW8(1:length(X_apps)) .* X_apps; %os8
CSW8_appsSum= sum(CSW8_apps); %sum efficiencies
CSW9_apps = CSW9(1:length(X_apps)) .* X_apps; %os9
CSW9_appsSum= sum(CSW9_apps); %sum efficiencies
CSW10_apps = CSW10(1:length(X_apps)) .* X_apps; %os10
CSW10_appsSum= sum(CSW10_apps); %sum efficiencies
%normalise apps to OS
CSW1_appsNorm = (CSW1_apps./CSW1_appsSum).* X_os(1);
if numOS > 1
CSW2_appsNorm = (CSW2_apps./CSW2_appsSum).* X_os(2);
end
if numOS > 2
CSW3_appsNorm = (CSW3_apps./CSW3_appsSum).* X_os(3);
end
if numOS > 3
CSW4_appsNorm = (CSW4_apps./CSW4_appsSum).* X_os(4);
end
if numOS > 4
CSW5_appsNorm = (CSW5_apps./CSW5_appsSum).* X_os(5);
end
if numOS > 5
CSW6_appsNorm = (CSW6_apps./CSW6_appsSum).* X_os(6);
end
if numOS > 6
CSW7_appsNorm = (CSW7_apps./CSW7_appsSum).* X_os(7);
end
if numOS > 7
CSW8_appsNorm = (CSW8_apps./CSW8_appsSum).* X_os(8);
end
if numOS > 8
CSW9_appsNorm = (CSW9_apps./CSW9_appsSum).* X_os(9);
end
if numOS > 9
CSW10_appsNorm = (CSW10_apps./CSW10_appsSum).* X_os(10);
end
%return apps to 1 array
N_apps = CSW1_appsNorm;
if numOS > 1
N_apps = CSW1_appsNorm + CSW2_appsNorm;
end
if numOS > 2
N_apps = CSW1_appsNorm + CSW2_appsNorm + CSW3_appsNorm;
end
if numOS > 3
N_apps = CSW1_appsNorm + CSW2_appsNorm + CSW3_appsNorm + CSW4_appsNorm;
end
if numOS > 4
N_apps = CSW1_appsNorm + CSW2_appsNorm + CSW3_appsNorm + CSW4_appsNorm + ...
    CSW5_appsNorm;
end
if numOS > 5
N_apps = CSW1_appsNorm + CSW2_appsNorm + CSW3_appsNorm + CSW4_appsNorm + ...
    CSW5_appsNorm + CSW6_appsNorm;
end
if numOS > 6
N_apps = CSW1_appsNorm + CSW2_appsNorm + CSW3_appsNorm + CSW4_appsNorm + ...
    CSW5_appsNorm + CSW6_appsNorm + CSW7_appsNorm;
end
if numOS > 7
N_apps = CSW1_appsNorm + CSW2_appsNorm + CSW3_appsNorm + CSW4_appsNorm + ...
    CSW5_appsNorm + CSW6_appsNorm + CSW7_appsNorm + CSW8_appsNorm;
end
if numOS > 8
N_apps = CSW1_appsNorm + CSW2_appsNorm + CSW3_appsNorm + CSW4_appsNorm + ...
    CSW5_appsNorm + CSW6_appsNorm + CSW7_appsNorm + CSW8_appsNorm + ...
    CSW9_appsNorm;
end
if numOS > 9
N_apps = CSW1_appsNorm + CSW2_appsNorm + CSW3_appsNorm + CSW4_appsNorm + ...
    CSW5_appsNorm + CSW6_appsNorm + CSW7_appsNorm + CSW8_appsNorm + ...
    CSW9_appsNorm + CSW10_appsNorm;
end
%check adds up to 100 still
sum(N_apps);
%check if need to normalise to 1
if normVal ~= 100
    N_apps = N_apps./100;
end


%os
N_os = X_os; %normalised to 100 already
%check if need to normalise to 1
if normVal ~= 100
    N_os = N_os./100;
end

%serv
%filters out serv to OS1 to 10
CSW1_serv = CSW1(length(X_apps)+1:length(X_apps)+length(X_serv)) .* X_serv; %os1
CSW1_servSum= sum(CSW1_serv); %sum efficiencies
CSW2_serv = CSW2(length(X_apps)+1:length(X_apps)+length(X_serv)) .* X_serv; %os2
CSW2_servSum= sum(CSW2_serv); %sum efficiencies
CSW3_serv = CSW3(length(X_apps)+1:length(X_apps)+length(X_serv)) .* X_serv; %os3
CSW3_servSum= sum(CSW3_serv); %sum efficiencies
CSW4_serv = CSW4(length(X_apps)+1:length(X_apps)+length(X_serv)) .* X_serv; %os4
CSW4_servSum= sum(CSW4_serv); %sum efficiencies
CSW5_serv = CSW5(length(X_apps)+1:length(X_apps)+length(X_serv)) .* X_serv; %os5
CSW5_servSum= sum(CSW5_serv); %sum efficiencies
CSW6_serv = CSW6(length(X_apps)+1:length(X_apps)+length(X_serv)) .* X_serv; %os6
CSW6_servSum= sum(CSW6_serv); %sum efficiencies
CSW7_serv = CSW7(length(X_apps)+1:length(X_apps)+length(X_serv)) .* X_serv; %os7
CSW7_servSum= sum(CSW7_serv); %sum efficiencies
CSW8_serv = CSW8(length(X_apps)+1:length(X_apps)+length(X_serv)) .* X_serv; %os8
CSW8_servSum= sum(CSW8_serv); %sum efficiencies
CSW9_serv = CSW9(length(X_apps)+1:length(X_apps)+length(X_serv)) .* X_serv; %os9
CSW9_servSum= sum(CSW9_serv); %sum efficiencies
CSW10_serv = CSW10(length(X_apps)+1:length(X_apps)+length(X_serv)) .* X_serv; %os10
CSW10_servSum= sum(CSW10_serv) ;%sum efficiencies
%normalise serv to OS
CSW1_servNorm = (CSW1_serv./CSW1_servSum).* X_os(1);
if numOS > 1
CSW2_servNorm = (CSW2_serv./CSW2_servSum).* X_os(2);
end
if numOS > 2
CSW3_servNorm = (CSW3_serv./CSW3_servSum).* X_os(3);
end
if numOS > 3
CSW4_servNorm = (CSW4_serv./CSW4_servSum).* X_os(4);
end
if numOS > 4
CSW5_servNorm = (CSW5_serv./CSW5_servSum).* X_os(5);
end
if numOS > 5
CSW6_servNorm = (CSW6_serv./CSW6_servSum).* X_os(6);
end
if numOS > 6
CSW7_servNorm = (CSW7_serv./CSW7_servSum).* X_os(7);
end
if numOS > 7
CSW8_servNorm = (CSW8_serv./CSW8_servSum).* X_os(8);
end
if numOS > 8
CSW9_servNorm = (CSW9_serv./CSW9_servSum).* X_os(9);
end
if numOS > 9
CSW10_servNorm = (CSW10_serv./CSW10_servSum).* X_os(10);
end
%return serv to 1 array
N_serv = CSW1_servNorm;
if numOS > 1
N_serv = CSW1_servNorm + CSW2_servNorm;
end
if numOS > 2
N_serv = CSW1_servNorm + CSW2_servNorm + CSW3_servNorm;
end
if numOS > 3
N_serv = CSW1_servNorm + CSW2_servNorm + CSW3_servNorm + CSW4_servNorm;
end
if numOS > 4
N_serv = CSW1_servNorm + CSW2_servNorm + CSW3_servNorm + CSW4_servNorm + ...
    CSW5_servNorm;
end
if numOS > 5
N_serv = CSW1_servNorm + CSW2_servNorm + CSW3_servNorm + CSW4_servNorm + ...
    CSW5_servNorm + CSW6_servNorm;
end
if numOS > 6
N_serv = CSW1_servNorm + CSW2_servNorm + CSW3_servNorm + CSW4_servNorm + ...
    CSW5_servNorm + CSW6_servNorm + CSW7_servNorm;
end
if numOS > 7
N_serv = CSW1_servNorm + CSW2_servNorm + CSW3_servNorm + CSW4_servNorm + ...
    CSW5_servNorm + CSW6_servNorm + CSW7_servNorm + CSW8_servNorm;
end
if numOS > 8
N_serv = CSW1_servNorm + CSW2_servNorm + CSW3_servNorm + CSW4_servNorm + ...
    CSW5_servNorm + CSW6_servNorm + CSW7_servNorm + CSW8_servNorm + ...
    CSW9_servNorm;
end
if numOS > 9
N_serv = CSW1_servNorm + CSW2_servNorm + CSW3_servNorm + CSW4_servNorm + ...
    CSW5_servNorm + CSW6_servNorm + CSW7_servNorm + CSW8_servNorm + ...
    CSW9_servNorm + CSW10_servNorm;
end
%check adds up to 100 still
sum(N_serv);

%check if need to normalise to 1
if normVal ~= 100
    N_serv = N_serv./100;
end

%dr
%filters out dr to OS1 to 10
CSW1_dr = CSW1(length(X_apps)+length(X_serv)+length(X_os)+1:end) .* X_dr; %os1
CSW1_drSum= sum(CSW1_dr); %sum efficiencies
CSW2_dr = CSW2(length(X_apps)+length(X_serv)+length(X_os)+1:end) .* X_dr; %os2
CSW2_drSum= sum(CSW2_dr); %sum efficiencies
CSW3_dr = CSW3(length(X_apps)+length(X_serv)+length(X_os)+1:end) .* X_dr; %os3
CSW3_drSum= sum(CSW3_dr); %sum efficiencies
CSW4_dr = CSW4(length(X_apps)+length(X_serv)+length(X_os)+1:end) .* X_dr; %os4
CSW4_drSum= sum(CSW4_dr); %sum efficiencies
CSW5_dr = CSW5(length(X_apps)+length(X_serv)+length(X_os)+1:end) .* X_dr; %os5
CSW5_drSum= sum(CSW5_dr); %sum efficiencies
CSW6_dr = CSW6(length(X_apps)+length(X_serv)+length(X_os)+1:end) .* X_dr; %os6
CSW6_drSum= sum(CSW6_dr); %sum efficiencies
CSW7_dr = CSW7(length(X_apps)+length(X_serv)+length(X_os)+1:end) .* X_dr; %os7
CSW7_drSum= sum(CSW7_dr); %sum efficiencies
CSW8_dr = CSW8(length(X_apps)+length(X_serv)+length(X_os)+1:end) .* X_dr; %os8
CSW8_drSum= sum(CSW8_dr); %sum efficiencies
CSW9_dr = CSW9(length(X_apps)+length(X_serv)+length(X_os)+1:end) .* X_dr; %os9
CSW9_drSum= sum(CSW9_dr); %sum efficiencies
CSW10_dr = CSW10(length(X_apps)+length(X_serv)+length(X_os)+1:end) .* X_dr; %os10
CSW10_drSum= sum(CSW10_dr); %sum efficiencies
%normalise dr to OS
CSW1_drNorm = (CSW1_dr./CSW1_drSum).* X_os(1);
if numOS > 1
CSW2_drNorm = (CSW2_dr./CSW2_drSum).* X_os(2);
end
if numOS > 2
CSW3_drNorm = (CSW3_dr./CSW3_drSum).* X_os(3);
end
if numOS > 3
CSW4_drNorm = (CSW4_dr./CSW4_drSum).* X_os(4);
end
if numOS > 4
CSW5_drNorm = (CSW5_dr./CSW5_drSum).* X_os(5);
end
if numOS > 5
CSW6_drNorm = (CSW6_dr./CSW6_drSum).* X_os(6);
end
if numOS > 6
CSW7_drNorm = (CSW7_dr./CSW7_drSum).* X_os(7);
end
if numOS > 7
CSW8_drNorm = (CSW8_dr./CSW8_drSum).* X_os(8);
end
if numOS > 8
CSW9_drNorm = (CSW9_dr./CSW9_drSum).* X_os(9);
end
if numOS > 9
CSW10_drNorm = (CSW10_dr./CSW10_drSum).* X_os(10);
end
%return apps to 1 array
N_dr = CSW1_drNorm;
if numOS > 1
N_dr = CSW1_drNorm + CSW2_drNorm;
end
if numOS > 2
N_dr = CSW1_drNorm + CSW2_drNorm + CSW3_drNorm;
end
if numOS > 3
N_dr = CSW1_drNorm + CSW2_drNorm + CSW3_drNorm + CSW4_drNorm;
end
if numOS > 4
N_dr = CSW1_drNorm + CSW2_drNorm + CSW3_drNorm + CSW4_drNorm + ...
    CSW5_drNorm;
end
if numOS > 5
N_dr = CSW1_drNorm + CSW2_drNorm + CSW3_drNorm + CSW4_drNorm + ...
    CSW5_drNorm + CSW6_drNorm;
end
if numOS > 6
N_dr = CSW1_drNorm + CSW2_drNorm + CSW3_drNorm + CSW4_drNorm + ...
    CSW5_drNorm + CSW6_drNorm + CSW7_drNorm;
end
if numOS > 7
N_dr = CSW1_drNorm + CSW2_drNorm + CSW3_drNorm + CSW4_drNorm + ...
    CSW5_drNorm + CSW6_drNorm + CSW7_drNorm + CSW8_drNorm;
end
if numOS > 8
N_dr = CSW1_drNorm + CSW2_drNorm + CSW3_drNorm + CSW4_drNorm + ...
    CSW5_drNorm + CSW6_drNorm + CSW7_drNorm + CSW8_drNorm + ...
    CSW9_drNorm;
end
if numOS > 9
N_dr = CSW1_drNorm + CSW2_drNorm + CSW3_drNorm + CSW4_drNorm + ...
    CSW5_drNorm + CSW6_drNorm + CSW7_drNorm + CSW8_drNorm + ...
    CSW9_drNorm + CSW10_drNorm;
end
%check adds up to 100 still
sum(N_dr);
%check if need to normalise to 1
if normVal ~= 100
    N_dr = N_dr./100;
end


end

