%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Jennifer Jackson 2017                                                        %
% A Biodiversity Approach to Cyber Security                                    %
% PhD Complexity Science                                                       %
% University of Warwick                                                        %
% http://wrap.warwick.ac.uk/98458                                              %
% This software is distributed under the  Creative Commons Attribution License %
% https://creativecommons.org/licenses/by/4.0/                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [N_apps,N_serv,N_os,N_dr] = normToOS2(normVal, ...
    CSW1,CSW2,CSW3,CSW4,CSW5,CSW6,CSW7,CSW8,CSW9,CSW10, ...
    X_apps,X_serv,X_os,X_dr)
%normToOS
%This function normalises the efficiency, community, or individual software
%input data to the compatable operating systems. This is then used by the
%main code to determine the expected diversity of the filtered version.
%This is then drawn on the graph along with the ideal unfiltered case.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%TESTING - limit to 5 nodes(rows) for testing
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%sizeX_apps = size(X_apps);
%sizeX_serv = size(X_apps);
%sizeX_os = size(X_apps);
%sizeX_dr = size(X_apps);
%disp('limit to 5 nodes(rows) for testing')
%X_apps = X_apps(1:5,:)
%X_serv = X_serv(1:5,:)
%X_os = X_os(1:5,:)
%X_dr = X_dr(1:5,:)

%sizeX_apps = size(X_apps);
%sizeX_serv = size(X_apps);
%sizeX_os = size(X_apps);
%sizeX_dr = size(X_apps);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%maximum number of operating systems catered for is 10
maxOS = 10;

%1.normalise to 100 if not done so already
sumX_apps = sum(X_apps,2); %sum along rows
sumX_serv = sum(X_serv,2);
sumX_os = sum(X_os,2);
sumX_dr = sum(X_dr,2);
%extend to all columns
sumX_appsExt = repmat(sumX_apps,1,size(X_apps,2)) ;
sumX_servExt = repmat(sumX_serv,1,size(X_serv,2));
sumX_osExt = repmat(sumX_os,1,size(X_os,2));
sumX_drExt = repmat(sumX_dr,1,size(X_dr,2));

X_apps = (X_apps./sumX_appsExt .* 100);
X_serv = (X_serv./sumX_servExt .* 100);
X_os = (X_os./sumX_osExt .* 100);
X_dr = (X_dr./sumX_drExt .* 100);
%check add up to 100 across rows

sumX_apps = sum(X_apps,2);
sumX_serv = sum(X_serv,2);
sumX_os = sum(X_os,2);
sumX_dr = sum(X_dr,2);

numOS = size(X_os);

%extend filters to all rows
CSW1_Ext = repmat(CSW1,size(X_apps,1),1);
CSW2_Ext = repmat(CSW2,size(X_apps,1),1);
CSW3_Ext = repmat(CSW3,size(X_apps,1),1);
CSW4_Ext = repmat(CSW4,size(X_apps,1),1);
CSW5_Ext = repmat(CSW5,size(X_apps,1),1);
CSW6_Ext = repmat(CSW6,size(X_apps,1),1);
CSW7_Ext = repmat(CSW7,size(X_apps,1),1);
CSW8_Ext = repmat(CSW8,size(X_apps,1),1);
CSW9_Ext = repmat(CSW9,size(X_apps,1),1);
CSW10_Ext = repmat(CSW10,size(X_apps,1),1);

%apps
%filters out apps to OS1 to 10
CSW1_apps = CSW1_Ext(:,1:size(X_apps,2)) .* X_apps; %os1
CSW1_appsSum= sum(CSW1_apps,2); %sum 
CSW2_apps = CSW2_Ext(:,1:size(X_apps,2)) .* X_apps; %os2
CSW2_appsSum= sum(CSW2_apps,2); %sum 
CSW3_apps = CSW3_Ext(:,1:size(X_apps,2)) .* X_apps; %os3
CSW3_appsSum= sum(CSW3_apps,2); %sum 
CSW4_apps = CSW4_Ext(:,1:size(X_apps,2)) .* X_apps; %os4
CSW4_appsSum= sum(CSW4_apps,2); %sum 
CSW5_apps = CSW5_Ext(:,1:size(X_apps,2)) .* X_apps; %os5
CSW5_appsSum= sum(CSW5_apps,2); %sum 
CSW6_apps = CSW6_Ext(:,1:size(X_apps,2)) .* X_apps; %os6
CSW6_appsSum= sum(CSW6_apps,2); %sum 
CSW7_apps = CSW7_Ext(:,1:size(X_apps,2)).* X_apps; %os7
CSW7_appsSum= sum(CSW7_apps,2); %sum 
CSW8_apps = CSW8_Ext(:,1:size(X_apps,2)) .* X_apps; %os8
CSW8_appsSum= sum(CSW8_apps,2); %sum 
CSW9_apps = CSW9_Ext(:,1:size(X_apps,2)) .* X_apps; %os9
CSW9_appsSum= sum(CSW9_apps,2); %sum 
CSW10_apps = CSW10_Ext(:,1:size(X_apps,2)) .* X_apps; %os10
CSW10_appsSum= sum(CSW10_apps,2); %sum 

%extend sums
CSW1_appsSumExt = repmat(CSW1_appsSum,1,size(X_apps,2));
CSW2_appsSumExt = repmat(CSW2_appsSum,1,size(X_apps,2));
CSW3_appsSumExt = repmat(CSW3_appsSum,1,size(X_apps,2));
CSW4_appsSumExt = repmat(CSW4_appsSum,1,size(X_apps,2));
CSW5_appsSumExt = repmat(CSW5_appsSum,1,size(X_apps,2));
CSW6_appsSumExt = repmat(CSW6_appsSum,1,size(X_apps,2));
CSW7_appsSumExt = repmat(CSW7_appsSum,1,size(X_apps,2));
CSW8_appsSumExt = repmat(CSW8_appsSum,1,size(X_apps,2));
CSW9_appsSumExt = repmat(CSW9_appsSum,1,size(X_apps,2));
CSW10_appsSumExt = repmat(CSW10_appsSum,1,size(X_apps,2));

%normalise apps to OS
CSW1_appsNorm = zeros(size(CSW1_apps));
CSW2_appsNorm = zeros(size(CSW2_apps));
CSW3_appsNorm = zeros(size(CSW3_apps));
CSW4_appsNorm = zeros(size(CSW4_apps));
CSW5_appsNorm = zeros(size(CSW5_apps));
CSW6_appsNorm = zeros(size(CSW6_apps));
CSW7_appsNorm = zeros(size(CSW7_apps));
CSW8_appsNorm = zeros(size(CSW8_apps));
CSW9_appsNorm = zeros(size(CSW9_apps));
CSW10_appsNorm = zeros(size(CSW10_apps));

X_osExt1 = repmat(X_os(:,1),1,size(CSW1_apps,2));
CSW1_appsNorm = (CSW1_apps./CSW1_appsSumExt).* X_osExt1;
if numOS > 1; X_osExt2 = repmat(X_os(:,2),1,size(CSW2_apps,2)); CSW2_appsNorm = (CSW2_apps./CSW2_appsSumExt).* X_osExt2; end
if numOS > 2; X_osExt3 = repmat(X_os(:,3),1,size(CSW3_apps,2)); CSW3_appsNorm = (CSW3_apps./CSW3_appsSumExt).* X_osExt3; end
if numOS > 3; X_osExt4 = repmat(X_os(:,4),1,size(CSW4_apps,2)); CSW4_appsNorm = (CSW4_apps./CSW4_appsSumExt).* X_osExt4; end
if numOS > 4; X_osExt5 = repmat(X_os(:,5),1,size(CSW5_apps,2)); CSW5_appsNorm = (CSW5_apps./CSW5_appsSumExt).* X_osExt5; end
if numOS > 5; X_osExt6 = repmat(X_os(:,6),1,size(CSW6_apps,2)); CSW6_appsNorm = (CSW6_apps./CSW6_appsSumExt).* X_osExt6; end
if numOS > 6; X_osExt7 = repmat(X_os(:,7),1,size(CSW7_apps,2)); CSW7_appsNorm = (CSW7_apps./CSW7_appsSumExt).* X_osExt7; end
if numOS > 7; X_osExt8 = repmat(X_os(:,8),1,size(CSW8_apps,2)); CSW8_appsNorm = (CSW8_apps./CSW8_appsSumExt).* X_osExt8; end
if numOS > 8; X_osExt9 = repmat(X_os(:,9),1,size(CSW9_apps,2)); CSW9_appsNorm = (CSW9_apps./CSW9_appsSumExt).* X_osExt9; end
if numOS > 9; X_osExt10 = repmat(X_os(:,10),1,size(CSW10_apps,2)); CSW10_appsNorm = (CSW10_apps./CSW10_appsSumExt).* X_osExt10; end
%return apps to 1 array
N_apps = CSW1_appsNorm + CSW2_appsNorm + CSW3_appsNorm + CSW4_appsNorm + ...
    CSW5_appsNorm + CSW6_appsNorm + CSW7_appsNorm + CSW8_appsNorm + ...
    CSW9_appsNorm + CSW10_appsNorm;
%check adds up to 100 still
sum_appsCheck = sum(N_apps,2);

%check if need to normalise to 1
if normVal ~= 100
    N_apps = N_apps./100;
    %sum_appsCheck = sum(N_apps,2)
end


%os
N_os = X_os; %normalised to 100 already
%check if need to normalise to 1
if normVal ~= 100
    N_os = N_os./100;
    %sum_osCheck = sum(N_os,2)
end


%serv
%filters out serv to OS1 to 10
start = size(X_apps,2) + 1;  %start after apps finished because part of same row
stop = size(X_apps,2) + size(X_serv,2); %stop when serv finish

CSW1_serv = CSW1_Ext(:,start:stop) .* X_serv; %os1
CSW1_servSum= sum(CSW1_serv,2); %sum 
CSW2_serv = CSW2_Ext(:,start:stop) .* X_serv; %os2
CSW2_servSum= sum(CSW2_serv,2); %sum efficiencies
CSW3_serv = CSW3_Ext(:,start:stop) .* X_serv; %os3
CSW3_servSum= sum(CSW3_serv,2); %sum efficiencies
CSW4_serv = CSW4_Ext(:,start:stop) .* X_serv; %os4
CSW4_servSum= sum(CSW4_serv,2); %sum efficiencies
CSW5_serv = CSW5_Ext(:,start:stop) .* X_serv; %os5
CSW5_servSum= sum(CSW5_serv,2); %sum efficiencies
CSW6_serv = CSW6_Ext(:,start:stop) .* X_serv; %os6
CSW6_servSum= sum(CSW6_serv,2); %sum efficiencies
CSW7_serv = CSW7_Ext(:,start:stop) .* X_serv; %os7
CSW7_servSum= sum(CSW7_serv,2); %sum efficiencies
CSW8_serv = CSW8_Ext(:,start:stop) .* X_serv; %os8
CSW8_servSum= sum(CSW8_serv,2); %sum efficiencies
CSW9_serv = CSW9_Ext(:,start:stop) .* X_serv; %os9
CSW9_servSum= sum(CSW9_serv,2); %sum efficiencies
CSW10_serv = CSW10_Ext(:,start:stop) .* X_serv; %os10
CSW10_servSum= sum(CSW10_serv,2) ;%sum efficiencies

%extend sums
CSW1_servSumExt = repmat(CSW1_servSum,1,size(X_serv,2)); 
CSW2_servSumExt = repmat(CSW2_servSum,1,size(X_serv,2));
CSW3_servSumExt = repmat(CSW3_servSum,1,size(X_serv,2));
CSW4_servSumExt = repmat(CSW4_servSum,1,size(X_serv,2));
CSW5_servSumExt = repmat(CSW5_servSum,1,size(X_serv,2));
CSW6_servSumExt = repmat(CSW6_servSum,1,size(X_serv,2));
CSW7_servSumExt = repmat(CSW7_servSum,1,size(X_serv,2));
CSW8_servSumExt = repmat(CSW8_servSum,1,size(X_serv,2));
CSW9_servSumExt = repmat(CSW9_servSum,1,size(X_serv,2));
CSW10_servSumExt = repmat(CSW10_servSum,1,size(X_serv,2));



%normalise serv to OS
CSW1_servNorm = zeros(size(CSW1_serv));
CSW2_servNorm = zeros(size(CSW2_serv));
CSW3_servNorm = zeros(size(CSW3_serv));
CSW4_servNorm = zeros(size(CSW4_serv));
CSW5_servNorm = zeros(size(CSW5_serv));
CSW6_servNorm = zeros(size(CSW6_serv));
CSW7_servNorm = zeros(size(CSW7_serv));
CSW8_servNorm = zeros(size(CSW8_serv));
CSW9_servNorm = zeros(size(CSW9_serv));
CSW10_servNorm = zeros(size(CSW10_serv));


X_osExt1 = repmat(X_os(:,1),1,size(CSW1_serv,2));
CSW1_servNorm = (CSW1_serv./CSW1_servSumExt).* X_osExt1;
if numOS > 1; X_osExt2 = repmat(X_os(:,2),1,size(CSW2_serv,2)); CSW2_servNorm = (CSW2_serv./CSW2_servSumExt).* X_osExt2; end
if numOS > 2; X_osExt3 = repmat(X_os(:,3),1,size(CSW3_serv,2)); CSW3_servNorm = (CSW3_serv./CSW3_servSumExt).* X_osExt3; end
if numOS > 3; X_osExt4 = repmat(X_os(:,4),1,size(CSW4_serv,2)); CSW4_servNorm = (CSW4_serv./CSW4_servSumExt).* X_osExt4; end
if numOS > 4; X_osExt5 = repmat(X_os(:,5),1,size(CSW5_serv,2)); CSW5_servNorm = (CSW5_serv./CSW5_servSumExt).* X_osExt5; end
if numOS > 5; X_osExt6 = repmat(X_os(:,6),1,size(CSW6_serv,2)); CSW6_servNorm = (CSW6_serv./CSW6_servSumExt).* X_osExt6; end
if numOS > 6; X_osExt7 = repmat(X_os(:,7),1,size(CSW7_serv,2)); CSW7_servNorm = (CSW7_serv./CSW7_servSumExt).* X_osExt7; end
if numOS > 7; X_osExt8 = repmat(X_os(:,8),1,size(CSW8_serv,2)); CSW8_servNorm = (CSW8_serv./CSW8_servSumExt).* X_osExt8; end
if numOS > 8; X_osExt9 = repmat(X_os(:,9),1,size(CSW9_serv,2)); CSW9_servNorm = (CSW9_serv./CSW9_servSumExt).* X_osExt9; end
if numOS > 9; X_osExt10 = repmat(X_os(:,10),1,size(CSW10_serv,2)); CSW10_servNorm = (CSW10_serv./CSW10_servSumExt).* X_osExt10; end

%return serv to 1 array
N_serv = CSW1_servNorm + CSW2_servNorm + CSW3_servNorm + CSW4_servNorm + ...
    CSW5_servNorm + CSW6_servNorm + CSW7_servNorm + CSW8_servNorm + ...
    CSW9_servNorm + CSW10_servNorm;
%check adds up to 100 still
servNormCheck = sum(N_serv,2);

%check if need to normalise to 1
if normVal ~= 100
    N_serv = N_serv./100;
   % sum_servCheck = sum(N_serv,2)
end

%dr
%filters out dr to OS1 to 10
start = size(X_apps,2) + size(X_serv,2) + size(X_os,2) + 1;


CSW1_dr = CSW1_Ext(:,start:end) .* X_dr; %os1
CSW1_drSum= sum(CSW1_dr,2); %sum efficiencies
CSW2_dr = CSW2_Ext(:,start:end) .* X_dr; %os2
CSW2_drSum= sum(CSW2_dr,2); %sum efficiencies
CSW3_dr = CSW3_Ext(:,start:end) .* X_dr; %os3
CSW3_drSum= sum(CSW3_dr,2); %sum efficiencies
CSW4_dr = CSW4_Ext(:,start:end) .* X_dr; %os4
CSW4_drSum= sum(CSW4_dr,2); %sum efficiencies
CSW5_dr = CSW5_Ext(:,start:end) .* X_dr; %os5
CSW5_drSum= sum(CSW5_dr,2); %sum efficiencies
CSW6_dr = CSW6_Ext(:,start:end) .* X_dr; %os6
CSW6_drSum= sum(CSW6_dr,2); %sum efficiencies
CSW7_dr = CSW7_Ext(:,start:end) .* X_dr; %os7
CSW7_drSum= sum(CSW7_dr,2); %sum efficiencies
CSW8_dr = CSW8_Ext(:,start:end) .* X_dr; %os8
CSW8_drSum= sum(CSW8_dr,2); %sum efficiencies
CSW9_dr = CSW9_Ext(:,start:end) .* X_dr; %os9
CSW9_drSum= sum(CSW9_dr,2); %sum efficiencies
CSW10_dr = CSW10_Ext(:,start:end) .* X_dr; %os10
CSW10_drSum= sum(CSW10_dr,2); %sum efficiencies

%extend sums
CSW1_drSumExt = repmat(CSW1_drSum,1,size(X_dr,2));
CSW2_drSumExt = repmat(CSW2_drSum,1,size(X_dr,2));
CSW3_drSumExt = repmat(CSW3_drSum,1,size(X_dr,2));
CSW4_drSumExt = repmat(CSW4_drSum,1,size(X_dr,2));
CSW5_drSumExt = repmat(CSW5_drSum,1,size(X_dr,2));
CSW6_drSumExt = repmat(CSW6_drSum,1,size(X_dr,2));
CSW7_drSumExt = repmat(CSW7_drSum,1,size(X_dr,2));
CSW8_drSumExt = repmat(CSW8_drSum,1,size(X_dr,2));
CSW9_drSumExt = repmat(CSW9_drSum,1,size(X_dr,2));
CSW10_drSumExt = repmat(CSW10_drSum,1,size(X_dr,2));

%normalise dr to OS
CSW1_drNorm = zeros(size(CSW1_dr));
CSW2_drNorm = zeros(size(CSW2_dr));
CSW3_drNorm = zeros(size(CSW3_dr));
CSW4_drNorm = zeros(size(CSW4_dr));
CSW5_drNorm = zeros(size(CSW5_dr));
CSW6_drNorm = zeros(size(CSW6_dr));
CSW7_drNorm = zeros(size(CSW7_dr));
CSW8_drNorm = zeros(size(CSW8_dr));
CSW9_drNorm = zeros(size(CSW9_dr));
CSW10_drNorm = zeros(size(CSW10_dr));

X_osExt1 = repmat(X_os(:,1),1,size(CSW1_dr,2));
CSW1_drNorm = (CSW1_dr./CSW1_drSumExt).* X_osExt1;
if numOS > 1; X_osExt2 = repmat(X_os(:,2),1,size(CSW2_dr,2)); CSW2_drNorm = (CSW2_dr./CSW2_drSumExt).* X_osExt2; end
if numOS > 2; X_osExt3 = repmat(X_os(:,3),1,size(CSW3_dr,2)); CSW3_drNorm = (CSW3_dr./CSW3_drSumExt).* X_osExt3; end
if numOS > 3; X_osExt4 = repmat(X_os(:,4),1,size(CSW4_dr,2)); CSW4_drNorm = (CSW4_dr./CSW4_drSumExt).* X_osExt4; end
if numOS > 4; X_osExt5 = repmat(X_os(:,5),1,size(CSW5_dr,2)); CSW5_drNorm = (CSW5_dr./CSW5_drSumExt).* X_osExt5; end
if numOS > 5; X_osExt6 = repmat(X_os(:,6),1,size(CSW6_dr,2)); CSW6_drNorm = (CSW6_dr./CSW6_drSumExt).* X_osExt6; end
if numOS > 6; X_osExt7 = repmat(X_os(:,7),1,size(CSW7_dr,2)); CSW7_drNorm = (CSW7_dr./CSW7_drSumExt).* X_osExt7; end
if numOS > 7; X_osExt8 = repmat(X_os(:,8),1,size(CSW8_dr,2)); CSW8_drNorm = (CSW8_dr./CSW8_drSumExt).* X_osExt8; end
if numOS > 8; X_osExt9 = repmat(X_os(:,9),1,size(CSW9_dr,2)); CSW9_drNorm = (CSW9_dr./CSW9_drSumExt).* X_osExt9; end
if numOS > 9; X_osExt10 = repmat(X_os(:,10),1,size(CSW10_dr,2)); CSW10_drNorm = (CSW10_dr./CSW10_drSumExt).* X_osExt10; end


%return apps to 1 array
N_dr = CSW1_drNorm + CSW2_drNorm + CSW3_drNorm + CSW4_drNorm + ...
    CSW5_drNorm + CSW6_drNorm + CSW7_drNorm + CSW8_drNorm + ...
    CSW9_drNorm + CSW10_drNorm;
%check adds up to 100 still
sum_drNormCheck = sum(N_dr,2);

%check if need to normalise to 1
if normVal ~= 100
    N_dr = N_dr./100;
    %sum_drCheck = sum(N_dr,2)
end


%CHECKED UP TO HERE IS OK
end



