%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Jennifer Jackson 2017                                                        %
% A Biodiversity Approach to Cyber Security                                    %
% PhD Complexity Science                                                       %
% University of Warwick                                                        %
% http://wrap.warwick.ac.uk/98458                                              %
% This software is distributed under the  Creative Commons Attribution License %
% https://creativecommons.org/licenses/by/4.0/                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [NeiAbsMax Nei_orig Nei_ind Nei_comm Nei_eff ] = ...
    plotOutput(saveGraphs, closeAllGraphs, saveGraphsDirectoryName, simName, numNodes, numSims, numRuns, ...
               len_apps, len_serv, len_os, len_dr, ...
               CD_apps,CD_serv,CD_os,CD_dr, ...
               ChangeDesirabilitiesFlag, tChange1, tChange2,CD_apps2,CD_serv2,CD_os2,CD_dr2, ...
               E_apps,E_serv,E_os,E_dr, ...
               ChangeEffFlag, tChangeEff1, tChangeEff2,E_apps2,E_serv2,E_os2,E_dr2, ...
               origGenotype, textData, ID, ...
               ChangeIndDesirabilitiesFlag, ID2, tChangeInd1, tChangeInd2, ...
               Nei_sol1, Nei_sol2, Nei_sol3, Nei_sol4, Nei_sol5, ...
               nodesDiversity,nodesMean, nodesStd, ...
               updateStopDisplay4, nodesStd4, minstd4Tolerance, maxstd4Tolerance, minStd4Curve, ...
               updateStopDisplay5, nodesStd5, minstd5Tolerance, maxstd5Tolerance, minStd5Curve, ...
               turnOnSol1, turnOnSol2, turnOnSol3, turnOnSol4, turnOnSol5,...
               lastStoppedStdMatrix,lastStoppedStdMatrix2,stdMargin,Nei_indDynamic,...
               numStates,sumOfStates)
                                       
%work out some original diversity metrics, and that of the different output
%solutions and plot them on a graph
sampleSize=10; %3*40; %NOTE THIS NEEDS TO BE AN INPUT TO THE FUNCTION
golayFiltOrder=3;
diffSampleInterval=5;%20;

plotData = 0; %for plotting sub functions
len = size(Nei_sol4); %length of data

%maximum possible diversity for number of nodes and variants
NeiAbsMax = neiMax4(len_apps, len_serv, len_os, len_dr, numNodes);
Nei_MaxL = repmat(NeiAbsMax,1,len(2)+1); %extend to length of data

%original diversity of the network
data = origGenotype';
Nei_orig = variantFreq4(textData, data, plotData);
Nei_origL = repmat(Nei_orig,1,len(2)+1); %extend to length of data
 
%individual desirabilities diversity
data = ID;
Nei_ind = variantFreq4(textData, data, plotData);
Nei_indLog = log10(1-Nei_ind);
Nei_indL = repmat(Nei_ind,1,len(2)+1);  %extend to length of data
%if change part way through
if ChangeIndDesirabilitiesFlag ==1
data2 = ID2;
Nei_ind2 = variantFreq4(textData, data2, plotData);
Nei_indL(1,tChangeInd1+1:tChangeInd2) = Nei_ind2;
end

%diversity of the community desirabilities
Nei_comm = communityDiversity(CD_apps,CD_serv,CD_os,CD_dr);
Nei_comLog = log10(1-Nei_comm);
Nei_commL = repmat(Nei_comm,1,len(2)+1); %extend to length of data
%if change part way through
if ChangeDesirabilitiesFlag == 1
Nei_comm2 = communityDiversity(CD_apps2,CD_serv2,CD_os2,CD_dr2);
Nei_comLog2 = log10(1-Nei_comm2);
Nei_commL2 = repmat(Nei_comm2,1,tChange2-tChange1); %extend to length of data
%if change back again
Nei_comm3 = communityDiversity(CD_apps,CD_serv,CD_os,CD_dr);
Nei_comLog3 = log10(1-Nei_comm3);
Nei_commL3 = repmat(Nei_comm3,1,len(2)+1-tChange2); %extend to length of data
size(Nei_commL2);
size(Nei_commL3);
size(Nei_commL(tChange1+1:tChange2));
size(Nei_commL(tChange2+1:len(2)+1));
Nei_commL(1,tChange1+1:tChange2) = Nei_commL2;
Nei_commL(1,tChange2+1:len(2)+1) = Nei_commL3;
end;

%diversity of the efficiency
%high efficiency diversity means all the efficiency ratings are similar.
%low efficiency diversity means one, or a small proportion are far more
%efficient than the rest
Nei_eff = efficiencyDiversity(E_apps,E_serv,E_os,E_dr);
Nei_effL = repmat(Nei_eff,1,len(2)+1);
%if change part way through
if ChangeEffFlag == 1
Nei_eff2 = efficiencyDiversity(E_apps2,E_serv2,E_os2,E_dr2);
Nei_eff2Log2 = log10(1-Nei_eff2);
Nei_eff2L2 = repmat(Nei_eff2,1,tChangeEff2-tChangeEff1); %extend to length of data
%if change back again
Nei_eff3 = efficiencyDiversity(E_apps,E_serv,E_os,E_dr);
Nei_eff3Log3 = log10(1-Nei_eff3);
Nei_eff3L3 = repmat(Nei_eff3,1,len(2)+1-tChangeEff2); %extend to length of data
size(Nei_eff2L2);
size(Nei_eff3L3);
size(Nei_effL(tChangeEff1+1:tChangeEff2));
size(Nei_effL(tChangeEff2+1:len(2)+1));
Nei_effL(1,tChangeEff1+1:tChangeEff2) = Nei_eff2L2;
Nei_effL(1,tChangeEff2+1:len(2)+1) = Nei_eff3L3;
end;

%get solutions to start from original diversity
Nei_sol1_temp(1:numSims,1) = Nei_orig;
Nei_sol1_temp(:,2:numRuns+1)=Nei_sol1;
Nei_sol1=Nei_sol1_temp;

Nei_sol2_temp(1:numSims,1) = Nei_orig;
Nei_sol2_temp(:,2:numRuns+1)=Nei_sol2;
Nei_sol2=Nei_sol2_temp;

Nei_sol3_temp(1:numSims,1) = Nei_orig;
Nei_sol3_temp(:,2:numRuns+1)=Nei_sol3;
Nei_sol3=Nei_sol3_temp;

Nei_sol4_temp(1:numSims,1) = Nei_orig;
Nei_sol4_temp(:,2:numRuns+1)=Nei_sol4;
Nei_sol4=Nei_sol4_temp;

Nei_sol5_temp(1:numSims,1) = Nei_orig;
Nei_sol5_temp(:,2:numRuns+1)=Nei_sol5;
Nei_sol5=Nei_sol5_temp;

h1DivIndex = figure;
plot(Nei_MaxL,'k')
hold on;
plot(Nei_origL,'g')
hold on;
plot(Nei_commL,'b--')
hold on;
plot(Nei_indL,'c--')
hold on;
sizeNei_indL = size(Nei_indL)
sizeNei_indDynamic = size(Nei_indDynamic)
for psim = 1:numSims
dyn(:,:) = Nei_indDynamic(psim,:,:);
sizeDyn = size(dyn)
plot(dyn,'c--')
hold on;
end;%psim
plot(Nei_effL,'g--')
hold on;
for psim = 1:numSims
        %plot options include:
    %1.plot solutions 1 to 5
    %2.plot solutions 4 & 5 only
    %3.plot solution 4 only (default option)
    if turnOnSol1==1 && turnOnSol2==1 && turnOnSol3==1 && turnOnSol4==1 && turnOnSol5==1
        plot(Nei_sol1(psim,:),'r--')
        hold on;
        plot(Nei_sol2(psim,:),'r')
        hold on;
        plot(Nei_sol3(psim,:),'c')
        meanSol3 = mean(Nei_sol3(psim,:))
        hold on;
    end;
    %plot solution 4
    plot(Nei_sol4(psim,:),'m')
    meanSol4 = mean(Nei_sol4(psim,:))
    hold on;
    %plot solution 5
    if turnOnSol4==1 && turnOnSol5==1
        plot(Nei_sol5(psim,:),'b')
        meanSol5 = mean(Nei_sol5(psim,:))
        hold on;
    end;
end;
xlabel('number of time steps')
axis([0 numRuns 0 1])
ylabel('diversity index')
%legend
    if turnOnSol1==1 && turnOnSol2==1 && turnOnSol3==1 && turnOnSol4==1 && turnOnSol5==1
        legend('maximum','original','community','individual','dynInd', 'efficiency', 'sol1', 'sol2', 'sol3', 'filtering', 'no filtering')
    elseif turnOnSol4==1 && turnOnSol5==0
        legend('maximum','original','community','individual','dynInd', 'efficiency', 'filtering')
    elseif turnOnSol4==1 && turnOnSol5==1
        legend('maximum','original','community','individual','dynInd', 'efficiency', 'filtering', 'no filtering')
    end

%h2DivIndexLog = figure;
%plot(log10(1 -Nei_origL),'g')
%hold on;
%plot(log10(1 - Nei_commL),'b--')
%hold on;
%plot(log10(1 -Nei_indL),'c--')
%hold on;
%plot(log10(1 -Nei_effL),'g--')
%hold on;
%plot(log10(1 - Nei_MaxL),'k')
%hold on;
%for psim = 1:numSims
%    %plot options include:
%    %1.plot solutions 1 to 5
%    %2.plot solutions 4 & 5 only
%    %3.plot solution 4 only (default option)
%    if turnOnSol1==1 && turnOnSol2==1 && turnOnSol3==1 && turnOnSol4==1 && turnOnSol5==1
%        plot(log10(1- Nei_sol1(psim,:)),'r')
%        hold on;
%        plot(log10(1-Nei_sol2(psim,:)),'r--')
%        hold on;
%        plot(log10(1- Nei_sol3(psim,:)),'c')
%        meanSol3 = mean(Nei_sol3(psim,:));
%        hold on;
%    end;
%    %plot solution 4
%    a = Nei_sol4(psim,:);
%    loga = log10(1-(a));
%    plot(log10(1-(a)),'m')
%    meanSol4 = mean(Nei_sol4(psim,:));
%    meanSol4Log = mean(loga(1,:));
%    hold on;
%    %plot solution 5
%    if turnOnSol4==1 && turnOnSol5==1
%        a = Nei_sol5(psim,:);
%        loga = log10(1-(a));
%        plot(log10(1-(a)),'b')
%        meanSol5 = mean(Nei_sol5(psim,:));
%        meanSol5Log = mean(loga(1,:));
%        hold on;
%    end;
%end;
%xlabel('number of time steps')
%axis([0 numRuns -1.4 0])
%ylabel('log (1-diversity index)')
%%legend
 %   if turnOnSol1==1 && turnOnSol2==1 && turnOnSol3==1 && turnOnSol4==1 && turnOnSol5==1
 %       legend('original','community','individual', 'efficiency', 'maximum', 'sol1', 'sol2', 'sol3', 'filtering', 'no filtering')
 %   elseif turnOnSol4==1 && turnOnSol5==0
 %       legend('original','community','individual', 'efficiency', 'maximum', 'filtering')
 %   elseif turnOnSol4==1 && turnOnSol5==1
 %       legend('original','community','individual', 'efficiency', 'maximum', 'filtering', 'no filtering')
 %   end
    


%plot individual diversities of nodes
h3IndDiv = figure;
        for psim = 1:numSims
            nodesDiversity4(:,:) =   nodesDiversity(psim,:,:);
            sizenodesDiversity4 = size(nodesDiversity4)
            plot(nodesDiversity4(1,:),'g')
            hold on;
            plot(nodesDiversity4(2,:),'r')
            hold on;
            plot(nodesDiversity4(3,:),'b')
            hold on;
            plot(nodesDiversity4(4,:),'c')
            hold on;
            plot(nodesDiversity4(5,:),'m')
            hold on;
            plot(nodesDiversity4(6,:),'g')
            hold on;
            plot(nodesDiversity4(7,:),'r')
            hold on;
            plot(nodesDiversity4(8,:),'b')
            hold on;
            plot(nodesDiversity4(9,:),'c')
            hold on;
            plot(nodesDiversity4(10,:),'m')
            hold on;
        end
axis([0 numRuns 0 1])
xlabel('number of time steps')
ylabel('individual diversities')

%%log graph
%h4IndDivLog = figure;
%    for psim = 1:numSims
%        nodesDiversity4(:,:) =   nodesDiversity(psim,:,:);
%        nodesDiversityLog = log10(1-nodesDiversity4);
%        plot(nodesDiversityLog(1,:),'g')
%        hold on;
%    end
%axis([0 numRuns -1.4 0])
%xlabel('number of time steps')
%ylabel('log10(1-individual diversities)')

%plot the mean of an individual diversity
h5Means = figure;
    for psim = 1:numSims
        nodesMean4(:,:) =   nodesMean(psim,:,:);
        sizem4 = size(nodesMean4)
        diffSampleIntervalD=diffSampleInterval
        numRunsD=numRuns
        plot(nodesMean4(1,:),'g')
        hold on;
        sampleSizeDisplay=sampleSize*5
        %y=sgolayfilt(nodesMean4,golayFiltOrder,sampleSize*5+1,[],2);
        %plot(y(1,:),'k')
        %hold on;
        newStart=1;
        nodesMean4(1,newStart:diffSampleInterval:numRuns)
        b=diff(nodesMean4(1,newStart:diffSampleInterval:numRuns),1,2);
        b2=abs(b);
        runLength = size(b2,2)*diffSampleInterval;
        %s=1:diffSampleInterval:numRuns-1;
        s=newStart:diffSampleInterval:runLength;
        sizes = size(s)
        sizeb = size(b2)
        plot(s,b2,'m')
        hold on
        %c=diff(y(1,newStart:diffSampleInterval:numRuns+1),1,2);
        %c2=abs(c);
        %plot(s,c2,'c')
        %hold on;
end
%axis([0 numRuns 0 1])
xlabel('number of time steps')
ylabel('mean of individual diversities')

%plot the mean of an individual diversity
h5_sumStarted = figure;
h5_1Means = figure;
h5_2Means = figure;
h5_3Means = figure;
h5_4Means = figure;
h5_5Means = figure;
h5_6Means = figure;
h5_7Means = figure;
h5_8Means = figure;
h5_9Means = figure;
h5_10Means = figure;


%psim loop
for psim = 1:numSims
        nodesMean4(:,:) =   nodesMean(psim,:,:);
        updateStopDisplay(:,:) =   updateStopDisplay4(psim,:,:);
        
startval=0.6;
stopval=0.4;
stoppingMean = ones(numNodes,numRuns).*nodesMean4(1,1);
for i = 1:numNodes
  for j = 2:numRuns
    if  (updateStopDisplay(j-1,i)==startval && updateStopDisplay(j,i)==stopval)
      stoppingMean(i,j) = nodesMean4(i,j);
    else
      stoppingMean(i,j) = stoppingMean(i,j-1);
    end
  end
end
startingMean = ones(numNodes,numRuns).*stopval;
startingTol = 0.07;
for i = 1:numNodes
  for j = 2:numRuns
      diffMeans = abs(nodesMean4(i,j)-stoppingMean(i,j));
    if  (stoppingMean(i,j)>0 && (diffMeans > startingTol))
      startingMean(i,j) = startval;
    else
      startingMean(i,j) = startingMean(i,j-1);
    end
  end
end
startingMean2 = (startingMean-0.4).*5; %1 if on, and 0 if off
sumStarted = sum(startingMean2,1); %sum value for each time step

figure(h5_sumStarted);
plot(sumStarted) %figure 6
hold on;


%h5_1Means = figure;
figure(h5_1Means);
plot(nodesMean4(1,:),'g') %figure 7
hold on;
plot(stoppingMean(1,:),'k')
hold on;
plot(startingMean(1,:),'c')
hold on;
%axis([0 numRuns 0 1])
xlabel('number of time steps')
ylabel('mean of individual diversities and stopping mean node 1')

%h5_2Means = figure;
figure(h5_2Means);
plot(nodesMean4(2,:),'g') %figure 8
hold on;
plot(stoppingMean(2,:),'k')
hold on;
plot(startingMean(2,:),'c')
hold on;
%axis([0 numRuns 0 1])
xlabel('number of time steps')
ylabel('mean of individual diversities and stopping mean node 2')

%h5_3Means = figure;
figure(h5_3Means);
plot(nodesMean4(3,:),'g')
hold on;
plot(stoppingMean(3,:),'k')
hold on;
plot(startingMean(3,:),'c')
hold on;
%axis([0 numRuns 0 1])
xlabel('number of time steps')
ylabel('mean of individual diversities and stopping mean node 3')

%h5_4Means = figure;
figure(h5_4Means);
plot(nodesMean4(4,:),'g')
hold on;
plot(stoppingMean(4,:),'k')
hold on;
plot(startingMean(4,:),'c')
hold on;
%axis([0 numRuns 0 1])
xlabel('number of time steps')
ylabel('mean of individual diversities and stopping mean node 4')

%h5_5Means = figure;
figure(h5_5Means);
plot(nodesMean4(5,:),'g')
hold on;
plot(stoppingMean(5,:),'k')
hold on;
plot(startingMean(5,:),'c')
hold on;
%axis([0 numRuns 0 1])
xlabel('number of time steps')
ylabel('mean of individual diversities and stopping mean node 5')

%h5_6Means = figure;
figure(h5_6Means);
plot(nodesMean4(6,:),'g')
hold on;
plot(stoppingMean(6,:),'k')
hold on;
plot(startingMean(6,:),'c')
hold on;
%axis([0 numRuns 0 1])
xlabel('number of time steps')
ylabel('mean of individual diversities and stopping mean node 6')

%h5_7Means = figure;
figure(h5_7Means);
plot(nodesMean4(7,:),'g')
hold on;
plot(stoppingMean(7,:),'k')
hold on;
plot(startingMean(7,:),'c')
hold on;
%axis([0 numRuns 0 1])
xlabel('number of time steps')
ylabel('mean of individual diversities and stopping mean node 7')

%h5_8Means = figure;
figure(h5_8Means);
plot(nodesMean4(8,:),'g')
hold on;
plot(stoppingMean(8,:),'k')
hold on;
plot(startingMean(8,:),'c')
hold on;
%axis([0 numRuns 0 1])
xlabel('number of time steps')
ylabel('mean of individual diversities and stopping mean node 8')

%h5_9Means = figure;
figure(h5_9Means);
plot(nodesMean4(9,:),'g')
hold on;
plot(stoppingMean(9,:),'k')
hold on;
plot(startingMean(9,:),'c')
hold on;
%axis([0 numRuns 0 1])
xlabel('number of time steps')
ylabel('mean of individual diversities and stopping mean node 9')

%h5_10Means = figure;
figure(h5_10Means);
plot(nodesMean4(10,:),'g')
hold on;
plot(stoppingMean(10,:),'k')
hold on;
plot(startingMean(10,:),'c')
hold on;
%axis([0 numRuns 0 1])
xlabel('number of time steps')
ylabel('mean of individual diversities and stopping mean node 10')

end; %end psim loop
%*****************************

%plot the std of an individual diversity
h6Stds = figure;
for psim = 1:numSims
        nodesStdt(:,:) =   nodesStd(psim,:,:);
        plot(nodesStdt(1,:),'g')
        hold on;
        %y=sgolayfilt(nodesStdt,golayFiltOrder,sampleSize*5+1,[],2);
        %plot(y(1,:),'k')
        %hold on;
        b=diff(nodesStdt(1,1:diffSampleInterval:numRuns),1,2);
        b2=abs(b);
        runLength = size(b2,2)*diffSampleInterval;
        s=1:diffSampleInterval:runLength;
        plot(s,b2,'m')
        hold on
        %c=diff(y(1,1:diffSampleInterval:numRuns+1),1,2);
        %c2=abs(c);
        %plot(s,c2,'c')
        %hold on;
end; %psim
%axis([0 numRuns 0 1])
xlabel('number of time steps')
ylabel('standard deviation of individual diversities')

   %figure for solution 4 showing the standard deviation and when the 
   %genotype has stopped updating for 1 node
   if turnOnSol4 == 1
  h7Sol4StopStart=figure;
        for psim = 1:numSims
            nodesStdt(:,:) =   nodesStd4(psim,:,:);
            updateStopDisplayt(:,:) =   updateStopDisplay4(psim,:,:);
            lastStoppedStdMatrix2t(:,:) =   lastStoppedStdMatrix2(psim,:,:);
            lastStoppedStdMatrixt(:,:) =   lastStoppedStdMatrix(psim,:,:);
            minStd4Curvet(:,:) =   minStd4Curve(psim,:,:);
   
            %for i = 1:numNodes
            for i = 1:1
                % 1 when updating, 0 when stopped
                plot(updateStopDisplayt(:,i));
                hold on
                %standard deviation
                plot(nodesStdt(i,:),'g');
                hold on
                plot(lastStoppedStdMatrix2t(:,i),'b');
                hold on
                lastStoppedMargin2(:,i) = lastStoppedStdMatrix2t(:,i) + stdMargin;
                plot(lastStoppedMargin2(:,i),'c');
                hold on
                plot(lastStoppedStdMatrixt(:,i),'y');
                hold on
            end;
            %max and min lines on graph
            minStd4Display = repmat(minstd4Tolerance,numRuns,1);
            maxStd4Display = repmat(maxstd4Tolerance,numRuns,1);
            hold on
            %max std line
            plot(maxStd4Display,'m');
            hold on
            %tracking the minimum std threshold value as it changes
            plot(minStd4Curvet(1,:),'y');
            hold on
            % original std min line
            plot(minStd4Display,'r');
            hold on
        end ; %psim
   %axis([0 numRuns -0.1 1.1])
   xlabel('number of time steps')
   ylabel('standard deviation of individual diversities')
   title('solution 4 - stop/start update')
   %legend('stop-start', 'std','maxStd','minStdTrack','minStd')
   end;
    
   
   %****************************************************************
   %more nodes added - 12th January 2015
      %figure for solution 4 showing the standard deviation and when the 
   %genotype has stopped updating for 1 node
   if turnOnSol4 == 1
  h7_2Sol4StopStart=figure;
          for psim = 1:numSims
            nodesStdt(:,:) =   nodesStd4(psim,:,:);
            updateStopDisplayt(:,:) =   updateStopDisplay4(psim,:,:);
            lastStoppedStdMatrix2t(:,:) =   lastStoppedStdMatrix2(psim,:,:);
            lastStoppedStdMatrixt(:,:) =   lastStoppedStdMatrix(psim,:,:);
            minStd4Curvet(:,:) =   minStd4Curve(psim,:,:);
   %for i = 1:numNodes
   for i = 2:2
    % 1 when updating, 0 when stopped
    plot(updateStopDisplayt(:,i));
    hold on
    %standard deviation
    plot(nodesStdt(i,:),'g');
    hold on
    plot(lastStoppedStdMatrix2t(:,i),'b');
    hold on
    lastStoppedMargin2(:,i) = lastStoppedStdMatrix2t(:,i) + stdMargin;
    plot(lastStoppedMargin2(:,i),'c');
    hold on
      plot(lastStoppedStdMatrixt(:,i),'y');
    hold on
   end;
   %max and min lines on graph
   minStd4Display = repmat(minstd4Tolerance,numRuns,1);
   maxStd4Display = repmat(maxstd4Tolerance,numRuns,1);
   hold on
   %max std line
   plot(maxStd4Display,'m');
   hold on
   %tracking the minimum std threshold value as it changes
   plot(minStd4Curvet(1,:),'y');
   hold on
   % original std min line
   plot(minStd4Display,'r');
   hold on
   end; %psim
   %axis([0 numRuns -0.1 1.1])
   xlabel('number of time steps')
   ylabel('standard deviation of individual diversities')
   title('solution 4 - stop/start update 2')
   %legend('stop-start', 'std','maxStd','minStdTrack','minStd')
   end;
   

   %*******************************************************************
      %figure for solution 4 showing the standard deviation and when the 
   %genotype has stopped updating for 1 node
   if turnOnSol4 == 1
  h7_3Sol4StopStart=figure;
    for psim = 1:numSims
            nodesStdt(:,:) =   nodesStd4(psim,:,:);
            updateStopDisplayt(:,:) =   updateStopDisplay4(psim,:,:);
            lastStoppedStdMatrix2t(:,:) =   lastStoppedStdMatrix2(psim,:,:);
            lastStoppedStdMatrixt(:,:) =   lastStoppedStdMatrix(psim,:,:);
            minStd4Curvet(:,:) =   minStd4Curve(psim,:,:);
   
            %for i = 1:numNodes
            for i = 3:3
                % 1 when updating, 0 when stopped
                plot(updateStopDisplayt(:,i));
                hold on
                %standard deviation
                plot(nodesStdt(i,:),'g');
                hold on
                plot(lastStoppedStdMatrix2t(:,i),'b');
                hold on
                lastStoppedMargin2(:,i) = lastStoppedStdMatrix2t(:,i) + stdMargin;
                plot(lastStoppedMargin2(:,i),'c');
                hold on
                plot(lastStoppedStdMatrixt(:,i),'y');
                hold on
            end;
            %max and min lines on graph
            minStd4Display = repmat(minstd4Tolerance,numRuns,1);
            maxStd4Display = repmat(maxstd4Tolerance,numRuns,1);
            hold on
            %max std line
            plot(maxStd4Display,'m');
            hold on
            %tracking the minimum std threshold value as it changes
            plot(minStd4Curvet(1,:),'y');
            hold on
            % original std min line
            plot(minStd4Display,'r');
            hold on
    end; %psim
   %axis([0 numRuns -0.1 1.1])
   xlabel('number of time steps')
   ylabel('standard deviation of individual diversities')
   title('solution 4 - stop/start update 3')
   %legend('stop-start', 'std','maxStd','minStdTrack','minStd')
   end;
   
   %******************************************************************* 
      %figure for solution 4 showing the standard deviation and when the 
   %genotype has stopped updating for 1 node
   if turnOnSol4 == 1
  h7_4Sol4StopStart=figure;
      for psim = 1:numSims
            nodesStdt(:,:) =   nodesStd4(psim,:,:);
            updateStopDisplayt(:,:) =   updateStopDisplay4(psim,:,:);
            lastStoppedStdMatrix2t(:,:) =   lastStoppedStdMatrix2(psim,:,:);
            lastStoppedStdMatrixt(:,:) =   lastStoppedStdMatrix(psim,:,:);
            minStd4Curvet(:,:) =   minStd4Curve(psim,:,:);
            %for i = 1:numNodes
            for i = 4:4
                % 1 when updating, 0 when stopped
                plot(updateStopDisplayt(:,i));
                hold on
                %standard deviation
                plot(nodesStdt(i,:),'g');
                hold on
                plot(lastStoppedStdMatrix2t(:,i),'b');
                hold on
                lastStoppedMargin2(:,i) = lastStoppedStdMatrix2t(:,i) + stdMargin;
                plot(lastStoppedMargin2(:,i),'c');
                hold on
                plot(lastStoppedStdMatrixt(:,i),'y');
                hold on
            end;
            %max and min lines on graph
            minStd4Display = repmat(minstd4Tolerance,numRuns,1);
            maxStd4Display = repmat(maxstd4Tolerance,numRuns,1);
            hold on
            %max std line
            plot(maxStd4Display,'m');
            hold on
            %tracking the minimum std threshold value as it changes
            plot(minStd4Curvet(1,:),'y');
            hold on
            % original std min line
            plot(minStd4Display,'r');
            hold on
    end; %psim
   %axis([0 numRuns -0.1 1.1])
   xlabel('number of time steps')
   ylabel('standard deviation of individual diversities')
   title('solution 4 - stop/start update 4')
   %legend('stop-start', 'std','maxStd','minStdTrack','minStd')
   end;
   
   %***************************************************************
      %figure for solution 4 showing the standard deviation and when the 
   %genotype has stopped updating for 1 node
   if turnOnSol4 == 1
  h7_5Sol4StopStart=figure;
        for psim = 1:numSims
            nodesStdt(:,:) =   nodesStd4(psim,:,:);
            updateStopDisplayt(:,:) =   updateStopDisplay4(psim,:,:);
            lastStoppedStdMatrix2t(:,:) =   lastStoppedStdMatrix2(psim,:,:);
            lastStoppedStdMatrixt(:,:) =   lastStoppedStdMatrix(psim,:,:);
            minStd4Curvet(:,:) =   minStd4Curve(psim,:,:);
            %for i = 1:numNodes
            for i = 5:5
                % 1 when updating, 0 when stopped
                plot(updateStopDisplayt(:,i));
                hold on
                %standard deviation
                plot(nodesStdt(i,:),'g');
                hold on
                plot(lastStoppedStdMatrix2t(:,i),'b');
                hold on
                lastStoppedMargin2(:,i) = lastStoppedStdMatrix2t(:,i) + stdMargin;
                plot(lastStoppedMargin2(:,i),'c');
                hold on
                plot(lastStoppedStdMatrixt(:,i),'y');
                hold on
            end;
            %max and min lines on graph
             minStd4Display = repmat(minstd4Tolerance,numRuns,1);
            maxStd4Display = repmat(maxstd4Tolerance,numRuns,1);
            hold on
            %max std line
            plot(maxStd4Display,'m');
            hold on
            %tracking the minimum std threshold value as it changes
            plot(minStd4Curvet(1,:),'y');
            hold on
            % original std min line
            plot(minStd4Display,'r');
            hold on
         end;%psim
   
   %axis([0 numRuns -0.1 1.1])
   xlabel('number of time steps')
   ylabel('standard deviation of individual diversities')
   title('solution 4 - stop/start update 5')
   %legend('stop-start', 'std','maxStd','minStdTrack','minStd')
   end;

   %*************************************************************
       %figure for solution 4 showing the standard deviation and when the 
   %genotype has stopped updating for 1 node
   if turnOnSol4 == 1
  h7_6Sol4StopStart=figure;
          for psim = 1:numSims
            nodesStdt(:,:) =   nodesStd4(psim,:,:);
            updateStopDisplayt(:,:) =   updateStopDisplay4(psim,:,:);
            lastStoppedStdMatrix2t(:,:) =   lastStoppedStdMatrix2(psim,:,:);
            lastStoppedStdMatrixt(:,:) =   lastStoppedStdMatrix(psim,:,:);
            minStd4Curvet(:,:) =   minStd4Curve(psim,:,:);
            %for i = 1:numNodes
            for i = 6:6
                % 1 when updating, 0 when stopped
                plot(updateStopDisplayt(:,i));
                hold on
                %standard deviation
                plot(nodesStdt(i,:),'g');
                hold on
                plot(lastStoppedStdMatrix2t(:,i),'b');
                hold on
                lastStoppedMargin2(:,i) = lastStoppedStdMatrix2t(:,i) + stdMargin;
                plot(lastStoppedMargin2(:,i),'c');
                hold on
                plot(lastStoppedStdMatrixt(:,i),'y');
                hold on
            end;
            %max and min lines on graph
            minStd4Display = repmat(minstd4Tolerance,numRuns,1);
            maxStd4Display = repmat(maxstd4Tolerance,numRuns,1);
            hold on
            %max std line
            plot(maxStd4Display,'m');
            hold on
            %tracking the minimum std threshold value as it changes
            plot(minStd4Curvet(1,:),'y');
            hold on
            % original std min line
            plot(minStd4Display,'r');
            hold on
    end;%psim
   %axis([0 numRuns -0.1 1.1])
   xlabel('number of time steps')
   ylabel('standard deviation of individual diversities')
   title('solution 4 - stop/start update 6')
   %legend('stop-start', 'std','maxStd','minStdTrack','minStd')
   end;
      
   %*************************************************************
        %figure for solution 4 showing the standard deviation and when the 
   %genotype has stopped updating for 1 node
   if turnOnSol4 == 1
  h7_7Sol4StopStart=figure;
            for psim = 1:numSims
            nodesStdt(:,:) =   nodesStd4(psim,:,:);
            updateStopDisplayt(:,:) =   updateStopDisplay4(psim,:,:);
            lastStoppedStdMatrix2t(:,:) =   lastStoppedStdMatrix2(psim,:,:);
            lastStoppedStdMatrixt(:,:) =   lastStoppedStdMatrix(psim,:,:);
            minStd4Curvet(:,:) =   minStd4Curve(psim,:,:);
            %for i = 1:numNodes
            for i = 7:7
                % 1 when updating, 0 when stopped
                plot(updateStopDisplayt(:,i));
                hold on
                %standard deviation
                plot(nodesStdt(i,:),'g');
                hold on
                plot(lastStoppedStdMatrix2t(:,i),'b');
                hold on
                lastStoppedMargin2(:,i) = lastStoppedStdMatrix2t(:,i) + stdMargin;
                plot(lastStoppedMargin2(:,i),'c');
                hold on
                plot(lastStoppedStdMatrixt(:,i),'y');
                hold on
            end;
            %max and min lines on graph
            minStd4Display = repmat(minstd4Tolerance,numRuns,1);
            maxStd4Display = repmat(maxstd4Tolerance,numRuns,1);
            hold on
            %max std line
            plot(maxStd4Display,'m');
            hold on
            %tracking the minimum std threshold value as it changes
            plot(minStd4Curvet(1,:),'y');
            hold on
            % original std min line
            plot(minStd4Display,'r');
            hold on
    end;%psim
   %axis([0 numRuns -0.1 1.1])
   xlabel('number of time steps')
   ylabel('standard deviation of individual diversities')
   title('solution 4 - stop/start update 7')
   %legend('stop-start', 'std','maxStd','minStdTrack','minStd')
   end;

   %*************************************************************
        %figure for solution 4 showing the standard deviation and when the 
   %genotype has stopped updating for 1 node
   if turnOnSol4 == 1
  h7_8Sol4StopStart=figure;
              for psim = 1:numSims
            nodesStdt(:,:) =   nodesStd4(psim,:,:);
            updateStopDisplayt(:,:) =   updateStopDisplay4(psim,:,:);
            lastStoppedStdMatrix2t(:,:) =   lastStoppedStdMatrix2(psim,:,:);
            lastStoppedStdMatrixt(:,:) =   lastStoppedStdMatrix(psim,:,:);
            minStd4Curvet(:,:) =   minStd4Curve(psim,:,:);
            %for i = 1:numNodes
            for i = 8:8
                % 1 when updating, 0 when stopped
                plot(updateStopDisplayt(:,i));
                hold on
                %standard deviation
                plot(nodesStdt(i,:),'g');
                hold on
                plot(lastStoppedStdMatrix2t(:,i),'b');
                hold on
                lastStoppedMargin2(:,i) = lastStoppedStdMatrix2t(:,i) + stdMargin;
                plot(lastStoppedMargin2(:,i),'c');
                hold on
                plot(lastStoppedStdMatrixt(:,i),'y');
                hold on
            end;
            %max and min lines on graph
            minStd4Display = repmat(minstd4Tolerance,numRuns,1);
            maxStd4Display = repmat(maxstd4Tolerance,numRuns,1);
            hold on
            %max std line
            plot(maxStd4Display,'m');
            hold on
            %tracking the minimum std threshold value as it changes
            plot(minStd4Curvet(1,:),'y');
            hold on
            % original std min line
            plot(minStd4Display,'r');
            hold on
    end;%psim
   %axis([0 numRuns -0.1 1.1])
   xlabel('number of time steps')
   ylabel('standard deviation of individual diversities')
   title('solution 4 - stop/start update 8')
   %legend('stop-start', 'std','maxStd','minStdTrack','minStd')
   end;
  
   %*************************************************************
        %figure for solution 4 showing the standard deviation and when the 
   %genotype has stopped updating for 1 node
   if turnOnSol4 == 1
  h7_9Sol4StopStart=figure;
            for psim = 1:numSims
            nodesStdt(:,:) =   nodesStd4(psim,:,:);
            updateStopDisplayt(:,:) =   updateStopDisplay4(psim,:,:);
            lastStoppedStdMatrix2t(:,:) =   lastStoppedStdMatrix2(psim,:,:);
            lastStoppedStdMatrixt(:,:) =   lastStoppedStdMatrix(psim,:,:);
            minStd4Curvet(:,:) =   minStd4Curve(psim,:,:);
            %for i = 1:numNodes
                for i = 9:9
                % 1 when updating, 0 when stopped
                plot(updateStopDisplayt(:,i));
                hold on
                %standard deviation
                plot(nodesStdt(i,:),'g');
                hold on
                plot(lastStoppedStdMatrix2t(:,i),'b');
                hold on
                lastStoppedMargin2(:,i) = lastStoppedStdMatrix2t(:,i) + stdMargin;
                plot(lastStoppedMargin2(:,i),'c');
                hold on
                plot(lastStoppedStdMatrixt(:,i),'y');
                hold on
            end;
            %max and min lines on graph
                minStd4Display = repmat(minstd4Tolerance,numRuns,1);
            maxStd4Display = repmat(maxstd4Tolerance,numRuns,1);
            hold on
            %max std line
            plot(maxStd4Display,'m');
            hold on
            %tracking the minimum std threshold value as it changes
            plot(minStd4Curvet(1,:),'y');
            hold on
            % original std min line
            plot(minStd4Display,'r');
            hold on
            end;%psim
            %axis([0 numRuns -0.1 1.1])
            xlabel('number of time steps')
            ylabel('standard deviation of individual diversities')
            title('solution 4 - stop/start update 9')
            %legend('stop-start', 'std','maxStd','minStdTrack','minStd')
    end;

   %*************************************************************
        %figure for solution 4 showing the standard deviation and when the 
   %genotype has stopped updating for 1 node
   if turnOnSol4 == 1
  h7_10Sol4StopStart=figure;
            for psim = 1:numSims
            nodesStdt(:,:) =   nodesStd4(psim,:,:);
            updateStopDisplayt(:,:) =   updateStopDisplay4(psim,:,:);
            lastStoppedStdMatrix2t(:,:) =   lastStoppedStdMatrix2(psim,:,:);
            lastStoppedStdMatrixt(:,:) =   lastStoppedStdMatrix(psim,:,:);
            minStd4Curvet(:,:) =   minStd4Curve(psim,:,:);
            %for i = 1:numNodes
            for i = 10:10
                % 1 when updating, 0 when stopped
                plot(updateStopDisplayt(:,i));
                hold on
                %standard deviation
                plot(nodesStdt(i,:),'g');
                hold on
                plot(lastStoppedStdMatrix2t(:,i),'b');
                hold on
                lastStoppedMargin2(:,i) = lastStoppedStdMatrix2t(:,i) + stdMargin;
                plot(lastStoppedMargin2(:,i),'c');
                hold on
                plot(lastStoppedStdMatrixt(:,i),'y');
                hold on
            end;
            %max and min lines on graph
            minStd4Display = repmat(minstd4Tolerance,numRuns,1);
            maxStd4Display = repmat(maxstd4Tolerance,numRuns,1);
            hold on
            %max std line
            plot(maxStd4Display,'m');
            hold on
            %tracking the minimum std threshold value as it changes
            plot(minStd4Curvet(1,:),'y');
            hold on
            % original std min line
            plot(minStd4Display,'r');
            hold on
        end;%psim
   %axis([0 numRuns -0.1 1.1])
   xlabel('number of time steps')
   ylabel('standard deviation of individual diversities')
   title('solution 4 - stop/start update 10')
   %legend('stop-start', 'std','maxStd','minStdTrack','minStd')
   end;
   %*************************************************************

   %figure for solution 5 showing the standard deviation and when the 
   %genotype has stopped updating for 1 node
   if turnOnSol5 == 1
  h8Sol5StopStart = figure;
         for psim = 1:numSims
            nodesStdt(:,:) =   nodesStd5(psim,:,:);
            updateStopDisplayt(:,:) =   updateStopDisplay5(psim,:,:);
            minStd5Curvet(:,:) =   minStd5Curve(psim,:,:);
            %for i = 1:numNodes
            for i = 1:1
                % 1 when updating, 0 when stopped
                plot(updateStopDisplayt(:,i));
                hold on
                plot(nodesStdt(i,:),'g');
            end;
            %max and min lines on graph
            minStd5Display = repmat(minstd5Tolerance,numRuns,1);
            maxStd5Display = repmat(maxstd5Tolerance,numRuns,1);
            hold on
            plot(maxStd5Display,'m');
            hold on
            plot(minStd5Curvet(1,:),'y');
            hold on
            plot(minStd5Display,'r');
            hold on
        end;%psim
   axis([0 numRuns -0.1 1.1])
   xlabel('number of time steps')
   ylabel('standard deviation of individual diversities')
   title('solution 5 - stop/start update')
   legend('stop-start', 'std','maxStd','minStdTrack','minStd')
   end;
   
   %how many nodes are still updating sol4
    h9Sol4StopStart = figure;
   startVal=0.6; %THESE NEED TO BE PASSED DOWN INTO FUNCTION
   stopVal=0.4; %THESE NEED TO BE PASSED DOWN INTO FUNCTION
   for psim = 1:numSims
        updateStopDisplayt(:,:) =   updateStopDisplay4(psim,:,:);
       
        a = updateStopDisplayt;
        b = abs(a-startVal);
        sizeb_h9 = size(b)
        c = startVal-stopVal;
        d = sum(b,2)./c;
        %for i = 1:numNodes
        plot(d,'c');
        hold on 
    end;%psim
   xlabel('number of time steps')
   ylabel('number of nodes stopped updating')
   title('solution 4 - stop/start update')
   
   
   %extra graphs
   %justthe nodes supposed to be turning on for ind desirabilities
   %nodes 201 to xxx
    h9_2Sol4StopStart = figure;
   startVal=0.6; %THESE NEED TO BE PASSED DOWN INTO FUNCTION
   stopVal=0.4; %THESE NEED TO BE PASSED DOWN INTO FUNCTION
      for psim = 1:numSims
        updateStopDisplayt(:,:) =   updateStopDisplay4(psim,:,:);
        a = updateStopDisplayt;
        b = abs(a-startVal);
        %b2 = b(:,201:400);
        starting=(numNodes/2)+1; %TEMP 23rd March 2015 as above code only worked with 400 nodes
        b2 = b(:,starting:numNodes);
        sizeb2_h92 = size(b2)
        c = startVal-stopVal;
        d = sum(b2,2)./c;
        %for i = 1:numNodes
        plot(d,'c');
        hold on 
      end;%psim
    xlabel('number of time steps')
   ylabel('number of nodes stopped updating that turn on for ind')
   title('solution 4 - stop/start update')
   
   %nodes left that need to detect the changes
   %nodes 1 to 200, and xxx to 400
      h9_3Sol4StopStart = figure;
   startVal=0.6; %THESE NEED TO BE PASSED DOWN INTO FUNCTION
   stopVal=0.4; %THESE NEED TO BE PASSED DOWN INTO FUNCTION
   for psim = 1:numSims
        updateStopDisplayt(:,:) =   updateStopDisplay4(psim,:,:);
        a = updateStopDisplayt;
        b = abs(a-startVal);
        % b2 = b(:,1:200);
            b2 = b(:,1:starting-1); %TEMP 23rd March 2015 as above code only worked with 200 nodes
        sizeb2_h92 = size(b2)
        c = startVal-stopVal;
        d = sum(b2,2)./c;
        %for i = 1:numNodes
        plot(d,'c');
        hold on 
   end;%psim
   xlabel('number of time steps')
   ylabel('number of nodes stopped updating not turned on for ind')
   title('solution 4 - stop/start update')
   
   
    %UPTO HERE
    %plot error from the mean sol4
   %meanSolution4 = mean(Nei_sol4(1,:));
   %commError = Nei_commL - Nei_sol4(1,:);
   %meanError1(1:tChange1) = abs(mean(commError(1:tChange1)));
   %meanError2(tChange1+1:tChange2) = abs(mean(commError(tChange1+1:tChange2)));
   %meanError3(tChange2+1:numRuns) = abs(mean(commError(tChange2+1:numRuns)));
   %h10Sol4CommError = figure;
   %plot(commError,'b')
   %hold on
   %plot(meanError1,'r')
   %hold on
   %plot(meanError2,'g')
   %hold on
   %plot(meanError3,'c')
   %hold on 
   %xlabel('number of time steps')
   %ylabel('error')
   %title('solution 4 - error from the mean diversity of comm')
    
    
    
   %plot error from the mean sol4
   h10Sol4CommError = figure;
    for psim = 1:numSims
        meanSolution4 = mean(Nei_sol4(psim,:));
        commError = Nei_commL - Nei_sol4(psim,:);
        meanError1(1:tChange1) = abs(mean(commError(1:tChange1)));
        meanError2(tChange1+1:tChange2) = abs(mean(commError(tChange1+1:tChange2)));
        meanError3(tChange2+1:numRuns) = abs(mean(commError(tChange2+1:numRuns)));
   
        plot(commError,'b')
        hold on
        plot(meanError1,'r')
        hold on
        plot(meanError2,'g')
        hold on
        %plot(meanError3,'c')
        %hold on 
    end;%psim
   xlabel('number of time steps')
   ylabel('error')
   title('solution 4 - error from the mean diversity of comm')
   
   %************************************************************
   % Attack graphs
   %************************************************************
attackdataSize = size(sumOfStates) %size of data
colourPlot = {'b' 'r' 'g' 'c' 'r--' 'm' 'k' 'y'}
numRuns = attackdataSize(1)
numStates = attackdataSize(2)
numTimeSteps = attackdataSize(3)

h11Attack = figure;
%graphs
        for psim = 1:numRuns %for each run
            for stateNumber = 1:numStates %for each state
            %plot solution 4 only
             %b(1,:) = a(1,1,:)this code works
            plotStates(1,:) = sumOfStates(psim,stateNumber,:);
            plot(plotStates,char(colourPlot(stateNumber)))
            hold('on'); 
            end
        end

        xlabel('Number of Time Steps')
        ylabel('Number of Nodes')
        hold('on'); 
        %legend
            if numStates == 2
                legend('Susceptibles','Infected')
            elseif numStates ==3
                legend('Susceptibles','Infected','Recovered')
            elseif numStates == 4
                legend('Susceptibles','Exposed','Infected','Recovered')
            else
                legend('Susceptibles','Exposed','Prevented','Treated','Infected','Contained','Offline','eXposed off-line')
            end
            hold('on'); 
   
   %save graphs
   if saveGraphs ==1
   t = datestr(clock, 30);
   dateStr = [t];
   graphName1a = ['\' simName '_h1DivIndex.fig'];
   graphName1b = ['\' simName '_h1DivIndex.emf'];
   graphName2a = ['\' simName '_h2DivIndexLog.fig'];
   graphName2b = ['\' simName '_h2DivIndexLog.emf'];
   graphName3a = ['\' simName '_h3IndDiv.fig'];
   graphName3b = ['\' simName '_h3IndDiv.emf'];
   graphName4a = ['\' simName '_h4IndDivLog.fig'];
   graphName4b = ['\' simName '_h4IndDivLog.emf'];
   graphName5a = ['\' simName '_h5Means.fig'];
   graphName5b = ['\' simName '_h5Means.emf'];
   
   graphName5a_sumStarted = ['\' simName '_h5_sumStarted.fig'];
   graphName5b_sumStarted = ['\' simName '_h5_sumStarted.emf'];
   graphName5a_1 = ['\' simName '_h5_1Means.fig'];
   graphName5b_1 = ['\' simName '_h5_1Means.emf'];
   graphName5a_2 = ['\' simName '_h5_2Means.fig'];
   graphName5b_2 = ['\' simName '_h5_2Means.emf'];
   graphName5a_3 = ['\' simName '_h5_3Means.fig'];
   graphName5b_3 = ['\' simName '_h5_3Means.emf'];
   graphName5a_4 = ['\' simName '_h5_4Means.fig'];
   graphName5b_4 = ['\' simName '_h5_4Means.emf'];
   graphName5a_5 = ['\' simName '_h5_5Means.fig'];
   graphName5b_5 = ['\' simName '_h5_5Means.emf'];
   graphName5a_6 = ['\' simName '_h5_6Means.fig'];
   graphName5b_6 = ['\' simName '_h5_6Means.emf'];
   graphName5a_7 = ['\' simName '_h5_7Means.fig'];
   graphName5b_7 = ['\' simName '_h5_7Means.emf'];
   graphName5a_8 = ['\' simName '_h5_8Means.fig'];
   graphName5b_8 = ['\' simName '_h5_8Means.emf'];
   graphName5a_9 = ['\' simName '_h5_9Means.fig'];
   graphName5b_9 = ['\' simName '_h5_9Means.emf'];
   graphName5a_10 = ['\' simName '_h5_10Means.fig'];
   graphName5b_10 = ['\' simName '_h5_10Means.emf'];
   
   graphName6a = ['\' simName '_h6Stds.fig'];
   graphName6b = ['\' simName '_h6Stds.emf'];
   graphName7a = ['\' simName '_h7Sol4StopStart.fig'];
   graphName7b = ['\' simName '_h7Sol4StopStart.emf'];
   %**************added 12th Jan 2015*************
   graphName7a_2 = ['\' simName '_h7_2Sol4StopStart.fig'];
   graphName7b_2 = ['\' simName '_h7_2Sol4StopStart.emf'];
   graphName7a_3 = ['\' simName '_h7_3Sol4StopStart.fig'];
   graphName7b_3 = ['\' simName '_h7_3Sol4StopStart.emf'];
   graphName7a_4 = ['\' simName '_h7_4Sol4StopStart.fig'];
   graphName7b_4 = ['\' simName '_h7_4Sol4StopStart.emf'];
   graphName7a_5 = ['\' simName '_h7_5Sol4StopStart.fig'];
   graphName7b_5 = ['\' simName '_h7_5Sol4StopStart.emf'];
   graphName7a_6 = ['\' simName '_h7_6Sol4StopStart.fig'];
   graphName7b_6 = ['\' simName '_h7_6Sol4StopStart.emf'];
   graphName7a_7 = ['\' simName '_h7_7Sol4StopStart.fig'];
   graphName7b_7 = ['\' simName '_h7_7Sol4StopStart.emf'];
   graphName7a_8 = ['\' simName '_h7_8Sol4StopStart.fig'];
   graphName7b_8 = ['\' simName '_h7_8Sol4StopStart.emf'];
   graphName7a_9 = ['\' simName '_h7_9Sol4StopStart.fig'];
   graphName7b_9 = ['\' simName '_h7_9Sol4StopStart.emf'];
   graphName7a_10 = ['\' simName '_h7_10Sol4StopStart.fig'];
   graphName7b_10 = ['\' simName '_h7_10Sol4StopStart.emf'];
   %**********************************************
   graphName8a = ['\' simName '_h8Sol5StopStart.fig'];
   graphName8b = ['\' simName '_h8Sol5StopStart.emf'];
   graphName9a = ['\' simName '_h9Sol4StopStart.fig'];
   graphName9b = ['\' simName '_h9Sol4StopStart.emf'];
   %*********************added 14th Jan
   graphName9a_2 = ['\' simName '_h9_2Sol4StopStart.fig'];
   graphName9b_2 = ['\' simName '_h9_2Sol4StopStart.emf'];
   graphName9a_3 = ['\' simName '_h9_3Sol4StopStart.fig'];
   graphName9b_3 = ['\' simName '_h9_3Sol4StopStart.emf'];
   %**********************************
   graphName10a = ['\' simName 'h10Sol4CommError.fig'];
   graphName10b = ['\' simName 'h10Sol4CommError.emf']; 
   graphName11a = ['\' simName 'h11Attack.fig'];
   graphName11b = ['\' simName 'h11Attack.emf'];

   filename1a = [saveGraphsDirectoryName graphName1a];
   filename1b = [saveGraphsDirectoryName graphName1b];
   filename2a = [saveGraphsDirectoryName graphName2a];
   filename2b = [saveGraphsDirectoryName graphName2b];
   filename3a = [saveGraphsDirectoryName graphName3a];
   filename3b = [saveGraphsDirectoryName graphName3b];
   filename4a = [saveGraphsDirectoryName graphName4a];
   filename4b = [saveGraphsDirectoryName graphName4b];
   filename5a = [saveGraphsDirectoryName graphName5a];
   filename5b = [saveGraphsDirectoryName graphName5b];
   
   filename5a_sumStarted = [saveGraphsDirectoryName graphName5a_sumStarted];
   filename5b_sumStarted = [saveGraphsDirectoryName graphName5b_sumStarted];
   filename5a_1 = [saveGraphsDirectoryName graphName5a_1];
   filename5b_1 = [saveGraphsDirectoryName graphName5b_1];
   filename5a_2 = [saveGraphsDirectoryName graphName5a_2];
   filename5b_2 = [saveGraphsDirectoryName graphName5b_2]; 
   filename5a_3 = [saveGraphsDirectoryName graphName5a_3];
   filename5b_3 = [saveGraphsDirectoryName graphName5b_3];
   filename5a_4 = [saveGraphsDirectoryName graphName5a_4];
   filename5b_4 = [saveGraphsDirectoryName graphName5b_4];
   filename5a_5 = [saveGraphsDirectoryName graphName5a_5];
   filename5b_5 = [saveGraphsDirectoryName graphName5b_5];
   filename5a_6 = [saveGraphsDirectoryName graphName5a_6];
   filename5b_6 = [saveGraphsDirectoryName graphName5b_6];
   filename5a_7 = [saveGraphsDirectoryName graphName5a_7];
   filename5b_7 = [saveGraphsDirectoryName graphName5b_7];
   filename5a_8 = [saveGraphsDirectoryName graphName5a_8];
   filename5b_8 = [saveGraphsDirectoryName graphName5b_8];
   filename5a_9 = [saveGraphsDirectoryName graphName5a_9];
   filename5b_9 = [saveGraphsDirectoryName graphName5b_9];
   filename5a_10 = [saveGraphsDirectoryName graphName5a_10];
   filename5b_10 = [saveGraphsDirectoryName graphName5b_10];
   
   filename6a = [saveGraphsDirectoryName graphName6a];
   filename6b = [saveGraphsDirectoryName graphName6b];
   filename7a = [saveGraphsDirectoryName graphName7a];
   filename7b = [saveGraphsDirectoryName graphName7b];
    %**************added 12th Jan 2015*************
   filename7a_2 = [saveGraphsDirectoryName graphName7a_2];
   filename7b_2 = [saveGraphsDirectoryName graphName7b_2];
   filename7a_3 = [saveGraphsDirectoryName graphName7a_3];
   filename7b_3 = [saveGraphsDirectoryName graphName7b_3];
   filename7a_4 = [saveGraphsDirectoryName graphName7a_4];
   filename7b_4 = [saveGraphsDirectoryName graphName7b_4];
   filename7a_5 = [saveGraphsDirectoryName graphName7a_5];
   filename7b_5 = [saveGraphsDirectoryName graphName7b_5];
   filename7a_6 = [saveGraphsDirectoryName graphName7a_6];
   filename7b_6 = [saveGraphsDirectoryName graphName7b_6];
   filename7a_7 = [saveGraphsDirectoryName graphName7a_7];
   filename7b_7 = [saveGraphsDirectoryName graphName7b_7];
   filename7a_8 = [saveGraphsDirectoryName graphName7a_8];
   filename7b_8 = [saveGraphsDirectoryName graphName7b_8];
   filename7a_9 = [saveGraphsDirectoryName graphName7a_9];
   filename7b_9 = [saveGraphsDirectoryName graphName7b_9];
   filename7a_10 = [saveGraphsDirectoryName graphName7a_10];
   filename7b_10 = [saveGraphsDirectoryName graphName7b_10]; 
    %***********************************************
   filename8a = [saveGraphsDirectoryName graphName8a];
   filename8b = [saveGraphsDirectoryName graphName8b]; 
   filename9a = [saveGraphsDirectoryName graphName9a];
   filename9b = [saveGraphsDirectoryName graphName9b];
   %*********added 14th Jan
   filename9a_2 = [saveGraphsDirectoryName graphName9a_2];
   filename9b_2 = [saveGraphsDirectoryName graphName9b_2];
   filename9a_3 = [saveGraphsDirectoryName graphName9a_3];
   filename9b_3 = [saveGraphsDirectoryName graphName9b_3];
   %*************************************************
   filename10a = [saveGraphsDirectoryName graphName10a];
   filename10b = [saveGraphsDirectoryName graphName10b];
   filename11a = [saveGraphsDirectoryName graphName11a];
   filename11b = [saveGraphsDirectoryName graphName11b];
   
   saveas(h1DivIndex,filename1a);
   saveas(h1DivIndex,filename1b);
  % saveas(h2DivIndexLog,filename2a);
  % saveas(h2DivIndexLog,filename2b);
   saveas(h3IndDiv,filename3a);
   saveas(h3IndDiv,filename3b);
  % saveas(h4IndDivLog,filename4a);
  % saveas(h4IndDivLog,filename4b);
   saveas(h5Means,filename5a);
   saveas(h5Means,filename5b);
   saveas(h5_sumStarted,filename5a_sumStarted);
   saveas(h5_sumStarted,filename5b_sumStarted);
   saveas(h5_1Means,filename5a_1);
   saveas(h5_1Means,filename5b_1);   
   saveas(h5_2Means,filename5a_2);
   saveas(h5_2Means,filename5b_2);
   saveas(h5_3Means,filename5a_3);
   saveas(h5_3Means,filename5b_3);
   saveas(h5_4Means,filename5a_4);
   saveas(h5_4Means,filename5b_4);
   saveas(h5_5Means,filename5a_5);
   saveas(h5_5Means,filename5b_5);
   saveas(h5_6Means,filename5a_6);
   saveas(h5_6Means,filename5b_6);
   saveas(h5_7Means,filename5a_7);
   saveas(h5_7Means,filename5b_7);
   saveas(h5_8Means,filename5a_8);
   saveas(h5_8Means,filename5b_8);
   saveas(h5_9Means,filename5a_9);
   saveas(h5_9Means,filename5b_9);
   saveas(h5_10Means,filename5a_10);
   saveas(h5_10Means,filename5b_10);

   
   saveas(h6Stds,filename6a);
   saveas(h6Stds,filename6b);
   saveas(h7Sol4StopStart,filename7a);
   saveas(h7Sol4StopStart,filename7b);
   %**************added 12th Jan 2015*************
   saveas(h7_2Sol4StopStart,filename7a_2);
   saveas(h7_2Sol4StopStart,filename7b_2);
   saveas(h7_3Sol4StopStart,filename7a_3);
   saveas(h7_3Sol4StopStart,filename7b_3);
   saveas(h7_4Sol4StopStart,filename7a_4);
   saveas(h7_4Sol4StopStart,filename7b_4);
   saveas(h7_5Sol4StopStart,filename7a_5);
   saveas(h7_5Sol4StopStart,filename7b_5);  
   saveas(h7_6Sol4StopStart,filename7a_6);
   saveas(h7_6Sol4StopStart,filename7b_6);
   saveas(h7_7Sol4StopStart,filename7a_7);
   saveas(h7_7Sol4StopStart,filename7b_7);
   saveas(h7_8Sol4StopStart,filename7a_8);
   saveas(h7_8Sol4StopStart,filename7b_8);
   saveas(h7_9Sol4StopStart,filename7a_9);
   saveas(h7_9Sol4StopStart,filename7b_9);
   saveas(h7_10Sol4StopStart,filename7a_10);
   saveas(h7_10Sol4StopStart,filename7b_10);
   %**********************************************
   saveas(h8Sol5StopStart,filename8a);
   saveas(h8Sol5StopStart,filename8b);
   saveas(h9Sol4StopStart,filename9a);
   saveas(h9Sol4StopStart,filename9b);
   %**********************added 14th Jan 2015
   saveas(h9_2Sol4StopStart,filename9a_2);
   saveas(h9_2Sol4StopStart,filename9b_2);
   saveas(h9_3Sol4StopStart,filename9a_3);
   saveas(h9_3Sol4StopStart,filename9b_3);
   %**********************************************
   saveas(h10Sol4CommError,filename10a);
   saveas(h10Sol4CommError,filename10b);
   saveas(h11Attack,filename11a);
   saveas(h11Attack,filename11b);   
   
   if closeAllGraphs ==1
       %close all graphs after saving
       close(h1DivIndex);
    %   close(h2DivIndexLog);
       close(h3IndDiv);
    %   close(h4IndDivLog);
       close(h5Means);
       close(h5_sumStarted);
       close(h5_1Means);
       close(h5_2Means);
       close(h5_3Means);
       close(h5_4Means);
       close(h5_5Means);
       close(h5_6Means);
       close(h5_7Means);
       close(h5_8Means);
       close(h5_9Means);
       close(h5_10Means);
       
       close(h6Stds);
       close(h7Sol4StopStart);
       %**************added 12th Jan 2015*************
       close(h7_2Sol4StopStart);
       close(h7_3Sol4StopStart);
       close(h7_4Sol4StopStart);
       close(h7_5Sol4StopStart);
       close(h7_6Sol4StopStart);
       close(h7_7Sol4StopStart);
       close(h7_8Sol4StopStart);
       close(h7_9Sol4StopStart);  
       close(h7_10Sol4StopStart); 
       %**********************************************
       close(h8Sol5StopStart);
       close(h9Sol4StopStart);
       %****************added 14th Jan
       close(h9_2Sol4StopStart);
       close(h9_3Sol4StopStart);
       %********************
       close(h10Sol4CommError);
       close(h11Attack);
   end;
   
   end;
   
   
   


