%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Jennifer Jackson 2017                                                        %
% A Biodiversity Approach to Cyber Security                                    %
% PhD Complexity Science                                                       %
% University of Warwick                                                        %
% http://wrap.warwick.ac.uk/98458                                              %
% This software is distributed under the  Creative Commons Attribution License %
% https://creativecommons.org/licenses/by/4.0/                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%change number of encounters according to a probability distribution
%change when have an encounter according to a probability distribution
%test file
function  numEncounters = stochasticEncounters(distStr,numNodes,minEncounters,maxEncounters, plotData)
%Inputs
%(1)distStr - a)'normal', b)'uniform' c) 'fixed'
%(2)numNodes - number of nodes
%(3)minEncounters - minimum number of encounters
%(4)maxEncounters - maximum number of encounters/fixed number of encounters
%(5)plotData - a)1 plot distributions b)0 don't plot distributions

%test inputs
%clear all;
%close all;

%distStr = 'uniform'
%numNodes = 20;
%minEncounters = 1;
%maxEncounters = 10;
%plotData = 1;

%%%%%%%%%%
%Main Code
%%%%%%%%%%

switch distStr
    case 'normal'
    %probability of the number of encounters before an update
    %normal/gaussian parameters
    meanE = ((maxEncounters - minEncounters)/2) + minEncounters;
    standardDeviationE = (maxEncounters - meanE)/3;
    %choose a set of random numbers from a normal distribution
    rn = round(meanE + standardDeviationE.*randn(numNodes,1));
    %clip top
    ctop= (rn <= maxEncounters) .* rn + (rn > maxEncounters) .* maxEncounters .* ones(size(rn));
    %clip bottom
    cbot= (ctop >= minEncounters) .* ctop + (ctop < minEncounters) .* minEncounters .* ones(size(ctop));
    %output data
    numEncounters = cbot;

    %plot distributions
    if plotData == 1
    %pre-clip    
    figure
    hist(rn,2*maxEncounters);
    mean(rn);
    std(rn);
    %after clip top
    figure;
    hist(ctop,2*maxEncounters);
    %after clip bottom
    figure;
    hist(cbot,maxEncounters - minEncounters);
    hhist = hist(cbot,maxEncounters - minEncounters);
    sumhist=sum(hhist);
    end;

    case 'uniform'
    %uniform distribution
    x = minEncounters:1:maxEncounters;
    ru = randi([minEncounters maxEncounters],numNodes,1);
    numEncounters = ru;
    %plot distributions
    if plotData == 1
    figure
    hist(ru,x);
    shist=hist(ru,x);
    sumShist = sum(shist);
    end;
    
    case 'fixed'
        %fixed number of encounters for each node
 numEncounters = zeros(numNodes,1);
 numEncounters(:,:) = maxEncounters;
end; %switch