%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Jennifer Jackson 2017                                                        %
% A Biodiversity Approach to Cyber Security                                    %
% PhD Complexity Science                                                       %
% University of Warwick                                                        %
% http://wrap.warwick.ac.uk/98458                                              %
% This software is distributed under the  Creative Commons Attribution License %
% https://creativecommons.org/licenses/by/4.0/                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [nodesMean nodesStd updateStopFlag updateStopDisplayFlag minStdCurve] = multipleStopUpdating(t, sampleSize, ...
                      numNodes, stopUpdate, restartUpdate, nodesDiversity, ...
                      tallySol, numEncounters, len_apps, len_serv, len_os, len_dr, updateYes, ...
                      minstdTolerance, minStd, maxStd, updateStopDisplay)

%*******************************************
     %STOPPING THE UPDATE PROCESS
     %Code for solution 4 and 5 only
     %*******************************************
     if t > sampleSize 
     start = t-(sampleSize);
     else
         start = 1;
     end;
     stop = t+1;
        
     
     %****************Single stop update solution 4 only
     if stopUpdate == 1
     %record the diversity each node sees for solution 4 only
     nodesDiversityOld = nodesDiversity(:,t);
     nodesDiversity(:,t+1) = nodesNei(tallySol, numEncounters, len_apps, len_serv, len_os, len_dr, updateYes, nodesDiversityOld);
     %record the running mean of the diversity each node sees for sol 4
     %record the running standard deviation each node sees for solution 4
     %(dim 1 does the columns, dim 2 does the rows)
     nodesMean(:,t+1) = mean(nodesDiversity(:,start:stop),2); 
     nodesStd(:,t+1) = std(nodesDiversity(:,start:stop),0,2);
     for i = 1: numNodes
     % if standard deviation less than the tolerance and surpassed sample
     % size, then stop updating
      if (nodesStd(i,t+1) < minstdTolerance) && (t >sampleSize)
         updateStopFlag(i) = 0;
         updateStopDisplayFlag(t,i)= 0;
      else
         updateStopFlag(i) = 1;
         updateStopDisplayFlag(t,i)= 1;
       end; %end nodesStd
     end; %end i loop
     end; %end stopUpdate
     %***********************************************
     %****************Multiple stop update 
     if restartUpdate == 1
     %record the diversity each node sees for solution 4 only
     nodesDiversityOld = nodesDiversity(:,t);
     nodesDiversity(:,t+1) = nodesNei(tallySol, numEncounters, len_apps, len_serv, len_os, len_dr, updateYes, nodesDiversityOld);
     %record the running standard deviation each node sees for solution 
     nodesMean(:,t+1) = mean(nodesDiversity(:,start:stop),2); 
     nodesStd(:,t+1) = std(nodesDiversity(:,start:stop),0,2);
     for i = 1: numNodes
     % check to see if surpassed sample size
     if (t >sampleSize)
      %***minimum update
      if reachTolFlag(i) == 0 && (nodesStd(i,t+1) <= minStd(i)) && (nodesStd(i,t+1) < maxStd4(i))  
          minStd(i) = nodesStd(i,t+1); %set new minimum
          minStdCurve(i,t) = nodesStd(i,t+1);
          reachTolFlag(i) = 1; %surpassed tolerance
      elseif reachTolFlag(i) == 1 && (nodesStd(i,t+1) <= minStd(i)) && (nodesStd(i,t+1) < maxStd(i))  
          minStd(i) = nodesStd(i,t+1); %set new minimum
           minStdCurve(i,t) = nodesStd(i,t+1);
          updateStop(i) = 1; %continue updating
          updateStopDisplay(t,i)= 1; %continue updating
      elseif reachTolFlag(i) == 1 && (nodesStd(i,t+1) > minStd(i)) && (nodesStd(i,t+1) < maxStd(i)) 
         updateStop(i) = 0; %stop updating
         updateStopDisplay(t,i)= 0; %stop updating
         minStdCurve(i,t) = minStdCurve(i,t-1);
      %***maximum update - update again and reset if std goes back up again
      elseif reachTolFlag(i) == 1 && (nodesStd(i,t+1) > minStd(i)) && (nodesStd(i,t+1) > maxStd(i))  
          updateStop(i) = 1; %start updating again
          updateStopDisplay(t,i)= 1; %start updating again
          %reset min tolerance and tolFlag
          minStd(i) = minstdTolerance;
           minStdCurve(i,t) = minstdTolerance;
          reachTolFlag(i) = 0;
      else
          minStdCurve(i,t) = minStdCurve(i,t-1);
      end; %end stop update logic
     else
         minStdCurve(i,t) = minstdTolerance;
     end; %end sample size
     end; %end i loop
     end; % end restartUpdate