%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Jennifer Jackson 2017                                                        %
% A Biodiversity Approach to Cyber Security                                    %
% PhD Complexity Science                                                       %
% University of Warwick                                                        %
% http://wrap.warwick.ac.uk/98458                                              %
% This software is distributed under the  Creative Commons Attribution License %
% https://creativecommons.org/licenses/by/4.0/                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function diversityOutput = tallyEncounters(typeEncounter, numFixedEncounters, minEncounters, maxEncounters, distStr, ...
                                        numNodes, Genotype, tallyLength)
%tallys up encounters, either fixed or stochastically set
%Inputs
%(1) typeEncounter -  a)'fixed'  b)'stochastic'
%(2) numFixedEncounters - when 'fixed' type selected this is the number of
%encounters each node must count before making a decision
%(3) minEncounters - when 'stochastic' type selected this is the minmimun
%number of encounters for setting up the distribution
%(4) maxEncounters - when 'stochastic' type selected this is the maximum
%number of encounters for setting up the distribution
%(5) distStr - when 'stochastic' type selected this specifies the type of
%stochastic distribution and can be 'normal' or 'uniform'
%(6) numNodes
%(7) Genotype matrix
%(8) tallyLength (CDSize(1))

%%%%%%%%%%
%TEST DATA
%%%%%%%%%%
%typeEncounter = 'stochastic'
%distStr = 'normal'
%       numFixedEncounters = 3
%       maxEncounters = 10
%       minEncounters = 5
%       [SW_str_apps SW_str_serv SW_str_os SW_str_dr ...
%          SW_freq_apps SW_freq_serv SW_freq_os SW_freq_dr ...
%          CSW1 CSW2 CSW3 CSW4 CSW5 CSW6 CSW7 CSW8 CSW9 CSW10 ...
%          Genotype_apps Genotype_serv Genotype_os Genotype_dr ...
%          E_apps E_serv E_os E_dr ...
%          CD_apps CD_serv CD_os CD_dr ...
%          ID_apps ID_serv ID_os ID_dr ...
%        ] = getData(9);
%       Genotype = [Genotype_apps Genotype_serv Genotype_os Genotype_dr];
%       CD = [CD_apps CD_serv CD_os CD_dr]'; %community desirability ratings
      
%      genoSize = size(Genotype);
%      numNodes= genoSize(1);
%      CDSize = size(CD);      
%      tallyLength = CDSize(1);
      

      
  %initialisation
  GenotypeSol1 = Genotype;    
  tallySol1 = zeros(numNodes,tallyLength); %solution 1
  tallySol2 = zeros(numNodes,tallyLength); %solution 2
  
        
switch typeEncounter
        
        %%%%%%%%%%%
        %FIXED
        %%%%%%%%%%
        case 'fixed'
        %creates a matrix of random numbers
        %choose a genotype node at random except for itself
        %will choose a number between 0 and 10 so have to -1, and then
        %+1 after rounded
        %creates a matrix of random numbers
        randIndex = round(rand(numNodes,numFixedEncounters)*(numNodes-1))+1;
        %choose a number between 1 and numNodes, but must not be k
                for k = 1:numNodes 
                    for j = 1:numFixedEncounters
                    if randIndex(k,j) == k
                        randIndex(k,j) = randIndex(k,j)-1;
                        if randIndex(k,j) == 0
                            randIndex(k,j) = 2;
                        end;
                    end;
                    end; %end j
                end; %end k 
                randIndex;
        
        %tally up the software present, do for each node
        for k = 1:numNodes 
            for j = 1:numFixedEncounters
                SWIndex = randIndex(k,j);        
                tallySol1(k,:) = GenotypeSol1(SWIndex,:) + tallySol1(k,:);
            end; %end j
        end; %end k
        
        %calculate the diversity score
        diversitySol1 = 1 - tallySol1(:,:)./numFixedEncounters;
        diversityOutput = diversitySol1;
        
        %%%%%%%%%%%%%%%       
        %STOCHASTIC%%%       
        %%%%%%%%%%%%%%%       
        case 'stochastic'
        %change number of encounters according to a probability
        %distribution, fixed for each node
        numEncountersSt = stochasticEncounters(distStr,numNodes,minEncounters,maxEncounters, 0);
        %creates a matrix of random numbers
        randIndex2 = round(rand(numNodes,maxEncounters)*(numNodes-1))+1;
        %choose a number between 1 and numNodes, but must not be k
        for k = 1:numNodes 
            for j = 1:maxEncounters
            if randIndex2(k,j) == k
                randIndex2(k,j) = randIndex2(k,j)-1;
                if randIndex2(k,j) == 0
                    randIndex2(k,j) = 2;
                end;
            end;
            end; %end j
        end; %end k 
        randIndex2;
        
        %tally up the software present, do for each node
        %when stochastic the number will be different
        for k = 1:numNodes 
            for j = 1:numEncountersSt(k)
                SWIndex = randIndex2(k,j);        
                tallySol2(k,:) = GenotypeSol1(SWIndex,:) + tallySol2(k,:);
            end; %end j
        end; %end k
        
        %calculate the diversity score
        for k = 1:numNodes           
        diversitySol2(k,:) = 1 - tallySol2(k,:)./numEncountersSt(k,1);
        end;
        diversityOutput = diversitySol2;
        end; %end switch