%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Jennifer Jackson 2017                                                        %
% A Biodiversity Approach to Cyber Security                                    %
% PhD Complexity Science                                                       %
% University of Warwick                                                        %
% http://wrap.warwick.ac.uk/98458                                              %
% This software is distributed under the  Creative Commons Attribution License %
% https://creativecommons.org/licenses/by/4.0/                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [Genotype_New Genotype_appsNew Genotype_servNew Genotype_osNew Genotype_drNew] = ...
                                 updateGenotype(t,tallySol, solNum, tallyLength, updateYes, updateStop, numNodes, numEncounters, tolerance,...
                                 we, wcd, wid, wdiv, S, CSWGrouprep, numOS, CDSize, ...
                                 CSW1, CSW2, CSW3, CSW4, CSW5, CSW6, CSW7, CSW8, CSW9, CSW10, ... 
                                 len_apps, len_serv, len_os, len_dr, ...
                                 SizeCD_apps, SizeCD_serv, SizeCD_os, SizeCD_dr, ...
                                 Genotype_apps, Genotype_serv, Genotype_os, Genotype_dr)
% INPUTS
% 1) tallySol - current tally of the software encounters
% 2) solNum - solution number as this will effect the calculation
% 3) tallyLength - length of the tally solution
% 4) updateYes - 1 indicates if a node should perform an update
% 5) numNodes - number of nodes
% 6) numEncounters - number of encounters for each node before an update
% 7) wdiv - diversity weighting
% 8) S - total fixed score based on desirabilities

%calculate the diversity score
numEncountersRep = repmat(numEncounters,1,tallyLength);
diversitySol_temp = 1 - tallySol./numEncountersRep;
%renormalise to 1 across each of the 4 loci - added 14th Jan 2015

%    diversitySolDisplay = diversitySol_temp(1,:) %1 node
%    sizeDiv = size(diversitySol_temp)

apps = normArray2(diversitySol_temp(:,1:len_apps),1);
end1 = len_apps+len_serv;
serv = normArray2(diversitySol_temp(:,len_apps+1:end1),1);
end2 = len_apps+len_serv+len_os;
os = normArray2(diversitySol_temp(:,end1+1:end2),1);
dr = normArray2(diversitySol_temp(:,end2+1:end),1);
diversitySolT = [apps serv os dr];
%diversitySolTDisplay = diversitySolT(1,:)

diversitySol=diversitySolT;
%numEncountersRepDISPLAY = numEncountersRep(:,1)
%tallySolDISPLAY = tallySol(:,17:18)
%diversitySolDISPLAY = diversitySol(:,17:18)

%calculate the desirability score
tallySol;
updateYes;
tallySum = sum(tallySol,2);
tallySumRep = repmat(tallySum,1,tallyLength);
tallyPercent = tallySol./tallySumRep;
desirabilityInt = S' - tallyPercent;
clipValues = (desirabilityInt > 0);
desirabilityInt = desirabilityInt.*clipValues; %get rid of negative numbers
%ridNans = 1 - isnan(desirabilityInt);
desirabilityIntSum = sum(desirabilityInt,2);
desirabilityIntSumRep = repmat(desirabilityIntSum,1,tallyLength);
desirabilitySol = desirabilityInt./desirabilityIntSumRep;
sum(desirabilitySol,2);

switch solNum
    %%%%%%%%%%%%%%%%
    %Solution 1
    %%%%%%%%%%%%%%%
    case 1
    %generate solutions using compatability filters and compare
    %sum up the max values from each of the two solutions and choose
    %the one with the highest summed desirability score
    
    %calculate the favourability of next chosen genotype
  
    %FavSol4 = (S') + diversitySol.*wdiv;
    %FavSol4 = desirabilitySol + diversitySol.*wdiv;
   %FavSol4 =  (S' .* diversitySol) + diversitySol; 
   FavSol4 =  (S' .* diversitySol.*wdiv) + diversitySol.*wdiv;
    if wdiv == 0
        disp('WARNING SOLUTION 1 - setting wdiv to 0 will cause the favourability score to be 0')
    end;
    %check for zero favourability
    if FavSol4 == 0
        disp('WARNING SOLUTION 1 - favourability score is 0, there will be no update of genotypes')
        %dont change the genotype, keep the current one
        Genotype_appsNew = Genotype_apps;
        Genotype_servNew = Genotype_serv;
        Genotype_osNew = Genotype_os;
        Genotype_drNew = Genotype_dr;
        %output from solution (1)
        Genotype_New = [Genotype_appsNew Genotype_servNew Genotype_osNew Genotype_drNew];
        return; %return out of function
    end;
    
    %solutions using compatability filters
    Fav4OSGroup = zeros(numNodes,CDSize(1),numOS);
    for m = 1:numOS
     Fav4OSGroup(:,:,m) = FavSol4 .* CSWGrouprep(:,:,m);
    end;
    
    %split back into seperate matrix to choose values based on
    %probabilities
    Fav4OSGroup_apps = zeros(numNodes,len_apps,numOS); 
    Fav4OSGroup_serv = zeros(numNodes,len_serv,numOS);
    Fav4OSGroup_os = zeros(numNodes,len_os,numOS);
    Fav4OSGroup_dr = zeros(numNodes,len_dr,numOS);
    for m = 1:numOS
      Fav4OSGroup_apps(:,:,m) = Fav4OSGroup(:,1:SizeCD_apps(2),m);
      Fav4OSGroup_serv(:,:,m) = Fav4OSGroup(:,SizeCD_apps(2)+1:SizeCD_apps(2)+SizeCD_serv(2),m);
      Fav4OSGroup_os(:,:,m) = Fav4OSGroup(:,SizeCD_apps(2)+1+SizeCD_serv(2):SizeCD_apps(2)+SizeCD_serv(2)+SizeCD_os(2),m);
      Fav4OSGroup_dr(:,:,m) = Fav4OSGroup(:,SizeCD_apps(2)+1+SizeCD_serv(2)+SizeCD_os(2):SizeCD_apps(2)+SizeCD_serv(2)+SizeCD_os(2)+SizeCD_dr(2),m);
     end;

     %get values for each OS config  
     for m = 1:numOS
     [maxFav4OSGroup_apps(:,:,m),maxFav4OSGroup_appsI(:,:,m)]  = maxRand(Fav4OSGroup_apps(:,:,m),2,tolerance);
     [maxFav4OSGroup_serv(:,:,m),maxFav4OSGroup_servI(:,:,m)]  = maxRand(Fav4OSGroup_serv(:,:,m),2,tolerance);
     [maxFav4OSGroup_os(:,:,m),maxFav4OSGroup_osI(:,:,m)]  = maxRand(Fav4OSGroup_os(:,:,m),2,tolerance);
     %because this is looking at specific OS then really index values should all
     %equal the number of the OS and there is only 1 value to choose
     %from.
     maxFav4OSGroup_osI(:,:,m) = m;
     [maxFav4OSGroup_dr(:,:,m),maxFav4OSGroup_drI(:,:,m)]  = maxRand(Fav4OSGroup_dr(:,:,m),2,tolerance);           
     end;
        
     %sum the max values for each node for each OS to see which solution to choose 
     for m = 1:numOS
      sum4GroupMax(:,:,m) = maxFav4OSGroup_apps(:,:,m) + maxFav4OSGroup_serv(:,:,m) + maxFav4OSGroup_os(:,:,m) + maxFav4OSGroup_dr(:,:,m);
     end;
        
     %combine into single matrix to get the max values
     sum4GroupCat = [];
     for m = 1:numOS
      sum4GroupCat = [sum4GroupCat sum4GroupMax(:,:,m)];
     end;
        
     %get the max values
     [sumC4, sumI4] = probSol(sum4GroupCat);
       
     %create matrices and reset to zero
     Genotype_apps1New4 = zeros(size(Genotype_apps)); 
     Genotype_serv1New4 = zeros(size(Genotype_serv));
     Genotype_os1New4 = zeros(size(Genotype_os));
     Genotype_dr1New4 = zeros(size(Genotype_dr));
     
     %populate with 1's
        for h = 1:numNodes
            for j = 1: numOS
                 if sumI4(h) == j
                %choose OS solution
                Genotype_apps1New4(h,maxFav4OSGroup_appsI(h,1,j)) = 1;
                Genotype_serv1New4(h,maxFav4OSGroup_servI(h,1,j)) = 1;
                Genotype_os1New4(h,maxFav4OSGroup_osI(h,1,j)) = 1;
                Genotype_dr1New4(h,maxFav4OSGroup_drI(h,1,j)) = 1;
                end; %end if sumI
            end; %end for j
        end; %end for h 
        
        
        
     %only update the genotypes for those that are ready to be updated
     for h = 1:numNodes
         if updateYes(h) == 1 
             Genotype_appsNew(h,:) = Genotype_apps1New4(h,:);
             Genotype_servNew(h,:) = Genotype_serv1New4(h,:);
             Genotype_osNew(h,:) = Genotype_os1New4(h,:);
             Genotype_drNew(h,:) = Genotype_dr1New4(h,:);
         else
             Genotype_appsNew(h,:) = Genotype_apps(h,:);
             Genotype_servNew(h,:) = Genotype_serv(h,:);
             Genotype_osNew(h,:) = Genotype_os(h,:);
             Genotype_drNew(h,:) = Genotype_dr(h,:);
         end;    
     end;
         
     %output from solution (1)
     Genotype_New = [Genotype_appsNew Genotype_servNew Genotype_osNew Genotype_drNew];
     
   %%%%%%%%%%%%%%%%%%%%%%
   %Solution 2
   %%%%%%%%%%%%%%%%%%%%%%
    case 2
        %generate 1 solution
        %Solution with the maximum OS desirability number as priority
        %followed by the compatible maximum SW desirability
        %find the most desirable OS
        
        %calculate the favourability of next chosen genotype
        FavSol2 =  (S' .* diversitySol.*wdiv) + diversitySol.*wdiv;
        if wdiv == 0
            disp('WARNING SOLUTION 2 - setting wdiv to 0 will cause the favourability score to be 0')
        end;
        %check for zero favourability
        if FavSol2 == 0
            disp('WARNING SOLUTION 2 - favourability score is 0, there will be no update of genotypes')
            %dont change the genotype, keep the current one
            Genotype_appsNew = Genotype_apps;
            Genotype_servNew = Genotype_serv;
            Genotype_osNew = Genotype_os;
            Genotype_drNew = Genotype_dr;
            %output from solution (2)
            Genotype_New = [Genotype_appsNew Genotype_servNew Genotype_osNew Genotype_drNew];
            return; %return out of function
        end;
   
        FavOS_os = FavSol2(:,SizeCD_apps(2)+1+SizeCD_serv(2):SizeCD_apps(2)+SizeCD_serv(2)+SizeCD_os(2));
        %tells you which filter to use on which node
        %[maxFav maxI] =  max(FavOS_os,[],2);
        [maxFav maxI] =  maxRand(FavOS_os,2,tolerance);
        for f = 1:numNodes
            % use OS1 filter
            if maxI(f) == 1
            FavNew(f,:) = FavSol2(f,:).*CSW1;
            % use OS2 filter
            elseif maxI(f) == 2
            FavNew(f,:) = FavSol2(f,:).*CSW2;
            %use OS3 filter
            elseif maxI(f) == 3
            FavNew(f,:) = FavSol2(f,:).*CSW3;
            elseif maxI(f) == 4
            FavNew(f,:) = FavSol2(f,:).*CSW4;
            elseif maxI(f) == 5
            FavNew(f,:) = FavSol2(f,:).*CSW5;
            elseif maxI(f) == 6
            FavNew(f,:) = FavSol2(f,:).*CSW6;
            elseif maxI(f) == 7
            FavNew(f,:) = FavSol2(f,:).*CSW7;
            elseif maxI(f) == 8
            FavNew(f,:) = FavSol2(f,:).*CSW8;
            elseif maxI(f) == 9
            FavNew(f,:) = FavSol2(f,:).*CSW9;
            else
            FavNew(f,:) = FavSol2(f,:).*CSW10;    
            end;
        end;
        
        
        %split back into seperate matrix to choose max values of the
        %software
        FavNew_apps = FavNew(:,1:SizeCD_apps(2));
        FavNew_serv = FavNew(:,SizeCD_apps(2)+1:SizeCD_apps(2)+SizeCD_serv(2));
        FavNew_os = FavNew(:,SizeCD_apps(2)+1+SizeCD_serv(2):SizeCD_apps(2)+SizeCD_serv(2)+SizeCD_os(2));
        FavNew_dr = FavNew(:,SizeCD_apps(2)+1+SizeCD_serv(2)+SizeCD_os(2):SizeCD_apps(2)+SizeCD_serv(2)+SizeCD_os(2)+SizeCD_dr(2));
        %get max values
        %choose random maximum number out of possible choices
        %dont always want to choose the first
        [maxFavNew_apps,maxFavNew_appsI]  = maxRand(FavNew_apps,2,tolerance);
        [maxFavNew_serv,maxFavNew_servI]  = maxRand(FavNew_serv,2,tolerance);
        [maxFavNew_os,maxFavNew_osI]  = maxRand(FavNew_os,2,tolerance);
        [maxFavNew_dr,maxFavNew_drI]  = maxRand(FavNew_dr,2,tolerance);
        %get most desirable solution
       %create matrices and reset to zero
       Genotype_appsNew = zeros(size(Genotype_apps)); 
       Genotype_servNew = zeros(size(Genotype_serv));
       Genotype_osNew = zeros(size(Genotype_os));
       Genotype_drNew = zeros(size(Genotype_dr));
        %populate with 1's
        for h = 1:numNodes
        Genotype_appsNew(h,maxFavNew_appsI(h)) = 1;
        Genotype_servNew(h,maxFavNew_servI(h)) = 1;
        Genotype_osNew(h,maxFavNew_osI(h)) = 1;
        Genotype_drNew(h,maxFavNew_drI(h)) = 1;
        end;       
        
        %output from solution (2)
        Genotype_New = [Genotype_appsNew Genotype_servNew Genotype_osNew Genotype_drNew];
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
     
     
   %%%%%%%%%%%%%%%%%%%%%%
   %Solution 3
   %%%%%%%%%%%%%%%%%%%%%
    case 3
    %generate solutions using compatability filters and compare
    %sum up the max values from each of the two solutions and choose
    %the one with the highest summed desirability score
    
    %calculate the favourability of next chosen genotype
    %FavSol4 = desirabilitySol + diversitySol.*wdiv;
   FavSol4 =  (S' .* diversitySol.*wdiv) + diversitySol.*wdiv;
       if wdiv == 0
        disp('WARNING SOLUTION 3 - setting wdiv to 0 will cause the favourability score to be 0')
    end;
    %check for zero favourability
    if FavSol4 == 0
        disp('WARNING SOLUTION 3 - favourability score is 0, there will be no update of genotypes')
        %dont change the genotype, keep the current one
        Genotype_appsNew = Genotype_apps;
        Genotype_servNew = Genotype_serv;
        Genotype_osNew = Genotype_os;
        Genotype_drNew = Genotype_dr;
        %output from solution (3)
        Genotype_New = [Genotype_appsNew Genotype_servNew Genotype_osNew Genotype_drNew];
        return; %return out of function
    end;
    
    %solutions using compatability filters
    Fav4OSGroup = zeros(numNodes,CDSize(1),numOS);
    for m = 1:numOS
     Fav4OSGroup(:,:,m) = FavSol4 .* CSWGrouprep(:,:,m);
    end;
    
    %split back into seperate matrix to choose values based on
    %probabilities
    Fav4OSGroup_apps = zeros(numNodes,len_apps,numOS); 
    Fav4OSGroup_serv = zeros(numNodes,len_serv,numOS);
    Fav4OSGroup_os = zeros(numNodes,len_os,numOS);
    Fav4OSGroup_dr = zeros(numNodes,len_dr,numOS);
    for m = 1:numOS
      Fav4OSGroup_apps(:,:,m) = Fav4OSGroup(:,1:SizeCD_apps(2),m);
      Fav4OSGroup_serv(:,:,m) = Fav4OSGroup(:,SizeCD_apps(2)+1:SizeCD_apps(2)+SizeCD_serv(2),m);
      Fav4OSGroup_os(:,:,m) = Fav4OSGroup(:,SizeCD_apps(2)+1+SizeCD_serv(2):SizeCD_apps(2)+SizeCD_serv(2)+SizeCD_os(2),m);
      Fav4OSGroup_dr(:,:,m) = Fav4OSGroup(:,SizeCD_apps(2)+1+SizeCD_serv(2)+SizeCD_os(2):SizeCD_apps(2)+SizeCD_serv(2)+SizeCD_os(2)+SizeCD_dr(2),m);
     end;

     %get values for each OS config  
     for m = 1:numOS
     [maxFav4OSGroup_apps(:,:,m),maxFav4OSGroup_appsI(:,:,m)]  = probSol(Fav4OSGroup_apps(:,:,m));
     [maxFav4OSGroup_serv(:,:,m),maxFav4OSGroup_servI(:,:,m)]  = probSol(Fav4OSGroup_serv(:,:,m));
     [maxFav4OSGroup_os(:,:,m),maxFav4OSGroup_osI(:,:,m)]  = probSol(Fav4OSGroup_os(:,:,m));
     %because this is looking at specific OS then really index values should all
     %equal the number of the OS and there is only 1 value to choose
     %from.
     maxFav4OSGroup_osI(:,:,m) = m;
     [maxFav4OSGroup_dr(:,:,m),maxFav4OSGroup_drI(:,:,m)]  = probSol(Fav4OSGroup_dr(:,:,m)) ;          
     end;
        
     %sum the max values for each node for each OS to see which solution to choose 
     for m = 1:numOS
      sum4GroupMax(:,:,m) = maxFav4OSGroup_apps(:,:,m) + maxFav4OSGroup_serv(:,:,m) + maxFav4OSGroup_os(:,:,m) + maxFav4OSGroup_dr(:,:,m);
     end;
        
     %combine into single matrix to get the max values
     sum4GroupCat = [];
     for m = 1:numOS
      sum4GroupCat = [sum4GroupCat sum4GroupMax(:,:,m)];
     end;
        
     %get the max values
     [sumC4, sumI4] = probSol(sum4GroupCat);
       
     %create matrices and reset to zero
     Genotype_apps1New4 = zeros(size(Genotype_apps)); 
     Genotype_serv1New4 = zeros(size(Genotype_serv));
     Genotype_os1New4 = zeros(size(Genotype_os));
     Genotype_dr1New4 = zeros(size(Genotype_dr));
     
     %populate with 1's
        for h = 1:numNodes
            for j = 1: numOS
                 if sumI4(h) == j
                  
                %choose OS solution
                Genotype_apps1New4(h,maxFav4OSGroup_appsI(h,1,j)) = 1;
                Genotype_serv1New4(h,maxFav4OSGroup_servI(h,1,j)) = 1;
                Genotype_os1New4(h,maxFav4OSGroup_osI(h,1,j)) = 1;
                Genotype_dr1New4(h,maxFav4OSGroup_drI(h,1,j)) = 1;
                end; %end if sumI
            end; %end for j
        end; %end for h 
        
        
        
     %only update the genotypes for those that are ready to be updated
     for h = 1:numNodes
         if updateYes(h) == 1 
             Genotype_appsNew(h,:) = Genotype_apps1New4(h,:);
             Genotype_servNew(h,:) = Genotype_serv1New4(h,:);
             Genotype_osNew(h,:) = Genotype_os1New4(h,:);
             Genotype_drNew(h,:) = Genotype_dr1New4(h,:);
         else
             Genotype_appsNew(h,:) = Genotype_apps(h,:);
             Genotype_servNew(h,:) = Genotype_serv(h,:);
             Genotype_osNew(h,:) = Genotype_os(h,:);
             Genotype_drNew(h,:) = Genotype_dr(h,:);
         end;    
     end;
         
     %output from solution (3)
     Genotype_New = [Genotype_appsNew Genotype_servNew Genotype_osNew Genotype_drNew];
 
     
   %%%%%%%%%%%%%%%%%%%%%%
   %Solution 4 - NO LONGER USED - SEE CASE 6
   %%%%%%%%%%%%%%%%%%%%%
    case 4
    %generate solutions using compatability filters and compare
    %sum up the max values from each of the two solutions and choose
    %the one with the highest summed desirability score
    
    %calculate the favourability of next chosen genotype
    FavSol4 = (S') + diversitySol.*wdiv;
    %FavSol4 = desirabilitySol + diversitySol.*wdiv;
    %check for zero favourability
    if FavSol4 == 0
        disp('WARNING SOLUTION 4 - favourability score is 0, there will be no update of genotypes')
        %dont change the genotype, keep the current one
        Genotype_appsNew = Genotype_apps;
        Genotype_servNew = Genotype_serv;
        Genotype_osNew = Genotype_os;
        Genotype_drNew = Genotype_dr;
        %output from solution (4)
        Genotype_New = [Genotype_appsNew Genotype_servNew Genotype_osNew Genotype_drNew];
        return; %return out of function
    end;
    
    %solutions using compatability filters
    Fav4OSGroup = zeros(numNodes,CDSize(1),numOS);
    for m = 1:numOS
     Fav4OSGroup(:,:,m) = FavSol4 .* CSWGrouprep(:,:,m);
    end;
    
    %split back into seperate matrix to choose values based on
    %probabilities
    Fav4OSGroup_apps = zeros(numNodes,len_apps,numOS); 
    Fav4OSGroup_serv = zeros(numNodes,len_serv,numOS);
    Fav4OSGroup_os = zeros(numNodes,len_os,numOS);
    Fav4OSGroup_dr = zeros(numNodes,len_dr,numOS);
    for m = 1:numOS
      Fav4OSGroup_apps(:,:,m) = Fav4OSGroup(:,1:SizeCD_apps(2),m);
      Fav4OSGroup_serv(:,:,m) = Fav4OSGroup(:,SizeCD_apps(2)+1:SizeCD_apps(2)+SizeCD_serv(2),m);
      Fav4OSGroup_os(:,:,m) = Fav4OSGroup(:,SizeCD_apps(2)+1+SizeCD_serv(2):SizeCD_apps(2)+SizeCD_serv(2)+SizeCD_os(2),m);
      Fav4OSGroup_dr(:,:,m) = Fav4OSGroup(:,SizeCD_apps(2)+1+SizeCD_serv(2)+SizeCD_os(2):SizeCD_apps(2)+SizeCD_serv(2)+SizeCD_os(2)+SizeCD_dr(2),m);
     end;

     %get values for each OS config  
     for m = 1:numOS
     [maxFav4OSGroup_apps(:,:,m),maxFav4OSGroup_appsI(:,:,m)]  = probSol(Fav4OSGroup_apps(:,:,m));
     [maxFav4OSGroup_serv(:,:,m),maxFav4OSGroup_servI(:,:,m)]  = probSol(Fav4OSGroup_serv(:,:,m));
     [maxFav4OSGroup_os(:,:,m),maxFav4OSGroup_osI(:,:,m)]  = probSol(Fav4OSGroup_os(:,:,m));
     %because this is looking at specific OS then really index values should all
     %equal the number of the OS and there is only 1 value to choose
     %from.
     %maxFav4OSGroup_osI(:,:,m) = m %jtd commented out 14th April 2014 as
     %doesnt alter function of code
     [maxFav4OSGroup_dr(:,:,m),maxFav4OSGroup_drI(:,:,m)]  = probSol(Fav4OSGroup_dr(:,:,m)) ;         
     end;
        
     %sum the max values for each node for each OS to see which solution to choose 
     for m = 1:numOS
      sum4GroupMax(:,:,m) = maxFav4OSGroup_apps(:,:,m) + maxFav4OSGroup_serv(:,:,m) + maxFav4OSGroup_os(:,:,m) + maxFav4OSGroup_dr(:,:,m);
     end;
        
     %combine into single matrix to get the max values
     sum4GroupCat = [];
     for m = 1:numOS
      sum4GroupCat = [sum4GroupCat sum4GroupMax(:,:,m)];
     end;
        
     %get the max values
     [sumC4, sumI4] = probSol(sum4GroupCat);
       
     %create matrices and reset to zero
     Genotype_apps1New4 = zeros(size(Genotype_apps)); 
     Genotype_serv1New4 = zeros(size(Genotype_serv));
     Genotype_os1New4 = zeros(size(Genotype_os));
     Genotype_dr1New4 = zeros(size(Genotype_dr));
     
     %populate with 1's
        for h = 1:numNodes
            for j = 1: numOS
                 if sumI4(h) == j
                  
                %choose OS solution
                Genotype_apps1New4(h,maxFav4OSGroup_appsI(h,1,j)) = 1;
                Genotype_serv1New4(h,maxFav4OSGroup_servI(h,1,j)) = 1;
                Genotype_os1New4(h,maxFav4OSGroup_osI(h,1,j)) = 1;
                Genotype_dr1New4(h,maxFav4OSGroup_drI(h,1,j)) = 1;
                end; %end if sumI
            end; %end for j
        end; %end for h 
        
        
        
     %only update the genotypes for those that are ready to be updated
     for h = 1:numNodes
         if (updateYes(h) == 1) && (updateStop(h) == 1)
             Genotype_appsNew(h,:) = Genotype_apps1New4(h,:);
             Genotype_servNew(h,:) = Genotype_serv1New4(h,:);
             Genotype_osNew(h,:) = Genotype_os1New4(h,:);
             Genotype_drNew(h,:) = Genotype_dr1New4(h,:);
         else
             Genotype_appsNew(h,:) = Genotype_apps(h,:);
             Genotype_servNew(h,:) = Genotype_serv(h,:);
             Genotype_osNew(h,:) = Genotype_os(h,:);
             Genotype_drNew(h,:) = Genotype_dr(h,:);
         end;    
     end;
         
     %output from solution (4)
     Genotype_New = [Genotype_appsNew Genotype_servNew Genotype_osNew Genotype_drNew];
 
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %NO LONGER USED - SEE CASE 7
    % solution 4 but with compatbility filtering turned off
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%     
    case 5

    %generate solutions using compatability filters and compare
    %sum up the max values from each of the solutions and choose
    % one using probability
    
    %calculate the favourability of next chosen genotype
    FavSol4 = (S') + diversitySol.*wdiv;
    %FavSol4 = desirabilitySol + diversitySol.*wdiv;
    %check for zero favourability
    if FavSol4 == 0
        disp('WARNING SOLUTION 5 - favourability score is 0, there will be no update of genotypes')
        %dont change the genotype, keep the current one
        Genotype_appsNew = Genotype_apps;
        Genotype_servNew = Genotype_serv;
        Genotype_osNew = Genotype_os;
        Genotype_drNew = Genotype_dr;
        %output from solution (5)
        Genotype_New = [Genotype_appsNew Genotype_servNew Genotype_osNew Genotype_drNew];
        return; %return out of function
    end;
    numOS=1; %WHY IS THIS SET TO 1? - This is set to 1 because there is no 
    %filtering into seperate solution sets, there is just 1 big solution set
    %Dont use compatability filters
    Fav4OSGroup(:,:,1) = FavSol4;
    
    %split back into seperate matrix to choose values based on
    %probabilities
    Fav4OSGroup_apps = zeros(numNodes,len_apps,numOS); 
    Fav4OSGroup_serv = zeros(numNodes,len_serv,numOS);
    Fav4OSGroup_os = zeros(numNodes,len_os,numOS);
    Fav4OSGroup_dr = zeros(numNodes,len_dr,numOS);
    for m = 1:numOS
      Fav4OSGroup_apps(:,:,m) = Fav4OSGroup(:,1:SizeCD_apps(2),m);
      Fav4OSGroup_serv(:,:,m) = Fav4OSGroup(:,SizeCD_apps(2)+1:SizeCD_apps(2)+SizeCD_serv(2),m);
      Fav4OSGroup_os(:,:,m) = Fav4OSGroup(:,SizeCD_apps(2)+1+SizeCD_serv(2):SizeCD_apps(2)+SizeCD_serv(2)+SizeCD_os(2),m);
      Fav4OSGroup_dr(:,:,m) = Fav4OSGroup(:,SizeCD_apps(2)+1+SizeCD_serv(2)+SizeCD_os(2):SizeCD_apps(2)+SizeCD_serv(2)+SizeCD_os(2)+SizeCD_dr(2),m);
     end;
     %get values for one OS solution set  
     for m = 1:numOS
     [maxFav4OSGroup_apps(:,:,m),maxFav4OSGroup_appsI(:,:,m)]  = probSol(Fav4OSGroup_apps(:,:,m));
     [maxFav4OSGroup_serv(:,:,m),maxFav4OSGroup_servI(:,:,m)]  = probSol(Fav4OSGroup_serv(:,:,m));
     [maxFav4OSGroup_os(:,:,m),maxFav4OSGroup_osI(:,:,m)]  = probSol(Fav4OSGroup_os(:,:,m));
     [maxFav4OSGroup_dr(:,:,m),maxFav4OSGroup_drI(:,:,m)]  = probSol(Fav4OSGroup_dr(:,:,m)) ;          
     end;
     %DONT THINK NEED THIS BIT
     %sum the max values for each node to see which solution to choose 
     for m = 1:numOS
      sum4GroupMax(:,:,m) = maxFav4OSGroup_apps(:,:,m) + maxFav4OSGroup_serv(:,:,m) + maxFav4OSGroup_os(:,:,m) + maxFav4OSGroup_dr(:,:,m);
     end;
      %DONT THINK NEED THIS BIT   
     %combine into single matrix to get the max values
     sum4GroupCat = [];
     for m = 1:numOS
      sum4GroupCat = [sum4GroupCat sum4GroupMax(:,:,m)];
     end;
        %DONT THINK NEED THIS BIT AS ALREADY HAVE THE CHOSEN ONES FROM EACH LOCUS
        %JUST REPLICATING BY DOING THIS AGAIN - CHECK SIM OUTPUT - SHOULD
        %GET SAME RESULT
     %get the max values
     [sumC4, sumI4] = probSol(sum4GroupCat);
       
     %create matrices and reset to zero
     Genotype_apps1New4 = zeros(size(Genotype_apps)); 
     Genotype_serv1New4 = zeros(size(Genotype_serv));
     Genotype_os1New4 = zeros(size(Genotype_os));
     Genotype_dr1New4 = zeros(size(Genotype_dr));
     
     %populate with 1's
        for h = 1:numNodes
            for j = 1: numOS
                 if sumI4(h) == j
                  
                %choose OS solution
                Genotype_apps1New4(h,maxFav4OSGroup_appsI(h,1,j)) = 1;
                Genotype_serv1New4(h,maxFav4OSGroup_servI(h,1,j)) = 1;
                Genotype_os1New4(h,maxFav4OSGroup_osI(h,1,j)) = 1;
                Genotype_dr1New4(h,maxFav4OSGroup_drI(h,1,j)) = 1;
                end; %end if sumI
            end; %end for j
        end; %end for h 
        
     %only update the genotypes for those that are ready to be updated
     for h = 1:numNodes
         if (updateYes(h) == 1) && (updateStop(h) == 1)

             Genotype_appsNew(h,:) = Genotype_apps1New4(h,:);
             Genotype_servNew(h,:) = Genotype_serv1New4(h,:);
             Genotype_osNew(h,:) = Genotype_os1New4(h,:);
             Genotype_drNew(h,:) = Genotype_dr1New4(h,:);
         else
             
             Genotype_appsNew(h,:) = Genotype_apps(h,:);
             Genotype_servNew(h,:) = Genotype_serv(h,:);
             Genotype_osNew(h,:) = Genotype_os(h,:);
             Genotype_drNew(h,:) = Genotype_dr(h,:);
         end;    
     end;

         
     %output from solution (5)
     Genotype_New = [Genotype_appsNew Genotype_servNew Genotype_osNew Genotype_drNew];
     
      %%%%%%%%%%%%%%%%%%%%%%
   %Solution 6
   %%%%%%%%%%%%%%%%%%%%%
    case 6
    %SAME AS SOLUTION 4 BUT CHOOSE OS FIRST USING PROBABILITY, THEN CHOOSE
    %THE OTHER VARIANTS
    
    %calculate the favourability of next chosen genotype
    %diversitySolDisp = diversitySol
    %wdivDisp = wdiv

    %SSize=size(S')
    %solSize=size(diversitySol)
    %wdivSize=size(wdiv)
    %weSize=size(we)
    %wcdSize=size(wcd)
    %widSize=size(wid)
   %FavSol4 = (S') + diversitySol.*wdiv;
    FavSol4 = diversitySol.*wdiv;
    wtot = we' + wcd' + wid' + wdiv; %sum of weightings
    FavSol4 = FavSol4./wtot; %normalise to 1
    %**********************************************
    %Temp fix to blacklist software 0-blacklist, 1-keep
    %**********************************************
    vulnDisclosedTime = 14000; %start blacklisting when release
    if t > vulnDisclosedTime
    %SSize=size(S')
    %S'
    %[sizeFavSol4r c] = size(FavSol4)
    BL_apps = [0 1 1 1 1 1 1 1];
    BL_serv = [1 1 1 1 1 1 1 1];
    BL_os = [1 1];
    BL_dr = [1 1 1 1 1 1 1 1];
    BL_MASK = [BL_apps BL_serv BL_os BL_dr];
    BL_MASKrep=repmat(BL_MASK,[numNodes,1]);
    FavSol4 = FavSol4.*BL_MASKrep;
    end
    %***********************************************
    FavSol4Display = FavSol4(1:5,1:5);
    %check for zero favourability
    if FavSol4 == 0
        disp('WARNING SOLUTION 6 - favourability score is 0, there will be no update of genotypes')
        %dont change the genotype, keep the current one
        Genotype_appsNew = Genotype_apps;
        Genotype_servNew = Genotype_serv;
        Genotype_osNew = Genotype_os;
        Genotype_drNew = Genotype_dr;
        %output from solution
        Genotype_New = [Genotype_appsNew Genotype_servNew Genotype_osNew Genotype_drNew];
        return; %return out of function
    end;
    
    %choose OS first based on probabilities
      FavSol4_os(:,:) = FavSol4(:,SizeCD_apps(2)+1+SizeCD_serv(2):SizeCD_apps(2)+SizeCD_serv(2)+SizeCD_os(2));
      [maxFav4_os(:,:),maxFav4_osI(:,:)] = probSol(FavSol4_os(:,:));
    %solutions using compatability filters
    Fav4OSGroup = zeros(numNodes,CDSize(1),numOS);
    for m = 1:numOS
     Fav4OSGroup(:,:,m) = FavSol4 .* CSWGrouprep(:,:,m);
    end;
    %choose solution set with chosen OS
    for n = 1:numNodes
        s=maxFav4_osI(n,:);
        chosenOSGroup(n,:) = Fav4OSGroup(n,:,s);
    end;
    
    
    %split back into seperate matrix 
    chosenOSGroup_apps = zeros(numNodes,len_apps); 
    chosenOSGroup_serv = zeros(numNodes,len_serv);
    chosenOSGroup_os = zeros(numNodes,len_os);
    chosenOSGroup_dr = zeros(numNodes,len_dr);
    
      chosenOSGroup_apps(:,:) = chosenOSGroup(:,1:SizeCD_apps(2));
      chosenOSGroup_serv(:,:) = chosenOSGroup(:,SizeCD_apps(2)+1:SizeCD_apps(2)+SizeCD_serv(2));
      chosenOSGroup_os(:,:) = chosenOSGroup(:,SizeCD_apps(2)+1+SizeCD_serv(2):SizeCD_apps(2)+SizeCD_serv(2)+SizeCD_os(2));
      chosenOSGroup_dr(:,:) = chosenOSGroup(:,SizeCD_apps(2)+1+SizeCD_serv(2)+SizeCD_os(2):SizeCD_apps(2)+SizeCD_serv(2)+SizeCD_os(2)+SizeCD_dr(2));

    %choose variants based on probabilities
     [maxchosenOSGroup_apps(:,:),maxchosenOSGroup_appsI(:,:)]  = probSol(chosenOSGroup_apps(:,:));
     [maxchosenOSGroup_serv(:,:),maxchosenOSGroup_servI(:,:)]  = probSol(chosenOSGroup_serv(:,:));
     [maxchosenOSGroup_os(:,:),maxchosenOSGroup_osI(:,:)]  = probSol(chosenOSGroup_os(:,:));
     [maxchosenOSGroup_dr(:,:),maxchosenOSGroup_drI(:,:)]  = probSol(chosenOSGroup_dr(:,:));
     
     %create matrices and reset to zero
     Genotype_apps1New4 = zeros(size(Genotype_apps)); 
     Genotype_serv1New4 = zeros(size(Genotype_serv));
     Genotype_os1New4 = zeros(size(Genotype_os));
     Genotype_dr1New4 = zeros(size(Genotype_dr));
    
     %populate with 1's using I values
     for h = 1:numNodes
          %choose OS solution
                Genotype_apps1New4(h,maxchosenOSGroup_appsI(h,1)) = 1;
                Genotype_serv1New4(h,maxchosenOSGroup_servI(h,1)) = 1;
                Genotype_os1New4(h,maxchosenOSGroup_osI(h,1)) = 1;
                Genotype_dr1New4(h,maxchosenOSGroup_drI(h,1)) = 1;
     end;  
        
     %only update the genotypes for those that are ready to be updated
     for h = 1:numNodes
         if (updateYes(h) == 1) && (updateStop(h) == 1)
             Genotype_appsNew(h,:) = Genotype_apps1New4(h,:);
             Genotype_servNew(h,:) = Genotype_serv1New4(h,:);
             Genotype_osNew(h,:) = Genotype_os1New4(h,:);
             Genotype_drNew(h,:) = Genotype_dr1New4(h,:);
         else
             Genotype_appsNew(h,:) = Genotype_apps(h,:);
             Genotype_servNew(h,:) = Genotype_serv(h,:);
             Genotype_osNew(h,:) = Genotype_os(h,:);
             Genotype_drNew(h,:) = Genotype_dr(h,:);
         end;    
     end;
         
     %output from solution (4)
     Genotype_New = [Genotype_appsNew Genotype_servNew Genotype_osNew Genotype_drNew];
     
     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % solution 5 but get rid of the sums, just use probability on each
    % locus
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%     
    case 7

    %generate solutions using compatability filters 
    %choose one variant from each locus using probability
    
    %calculate the favourability of next chosen genotype
    FavSol4 = (S') + diversitySol.*wdiv;
    wtot = we' + wcd' + wid' + wdiv;
    FavSol4 = FavSol4./wtot; %normalise to 1
    %FavSol4 = desirabilitySol + diversitySol.*wdiv;
    %check for zero favourability
    if FavSol4 == 0
        disp('WARNING SOLUTION 5 - favourability score is 0, there will be no update of genotypes')
        %dont change the genotype, keep the current one
        Genotype_appsNew = Genotype_apps;
        Genotype_servNew = Genotype_serv;
        Genotype_osNew = Genotype_os;
        Genotype_drNew = Genotype_dr;
        %output from solution (5)
        Genotype_New = [Genotype_appsNew Genotype_servNew Genotype_osNew Genotype_drNew];
        return; %return out of function
    end;
    numOS=1; %WHY IS THIS SET TO 1? - used for a for loop - This is set to 1 because there is no 
    %filtering into seperate solution sets, there is just 1 big solution set
    %Dont use compatability filters
    Fav4OSGroup(:,:,1) = FavSol4;
    
    %split back into seperate matrix to choose values based on
    %probabilities
    Fav4OSGroup_apps = zeros(numNodes,len_apps,numOS); 
    Fav4OSGroup_serv = zeros(numNodes,len_serv,numOS);
    Fav4OSGroup_os = zeros(numNodes,len_os,numOS);
    Fav4OSGroup_dr = zeros(numNodes,len_dr,numOS);
    for m = 1:numOS
      Fav4OSGroup_apps(:,:,m) = Fav4OSGroup(:,1:SizeCD_apps(2),m);
      Fav4OSGroup_serv(:,:,m) = Fav4OSGroup(:,SizeCD_apps(2)+1:SizeCD_apps(2)+SizeCD_serv(2),m);
      Fav4OSGroup_os(:,:,m) = Fav4OSGroup(:,SizeCD_apps(2)+1+SizeCD_serv(2):SizeCD_apps(2)+SizeCD_serv(2)+SizeCD_os(2),m);
      Fav4OSGroup_dr(:,:,m) = Fav4OSGroup(:,SizeCD_apps(2)+1+SizeCD_serv(2)+SizeCD_os(2):SizeCD_apps(2)+SizeCD_serv(2)+SizeCD_os(2)+SizeCD_dr(2),m);
     end;
     %get values for one OS solution set  
     for m = 1:numOS
     [maxFav4OSGroup_apps(:,:,m),maxFav4OSGroup_appsI(:,:,m)]  = probSol(Fav4OSGroup_apps(:,:,m));
     [maxFav4OSGroup_serv(:,:,m),maxFav4OSGroup_servI(:,:,m)]  = probSol(Fav4OSGroup_serv(:,:,m));
     [maxFav4OSGroup_os(:,:,m),maxFav4OSGroup_osI(:,:,m)]  = probSol(Fav4OSGroup_os(:,:,m));
     [maxFav4OSGroup_dr(:,:,m),maxFav4OSGroup_drI(:,:,m)]  = probSol(Fav4OSGroup_dr(:,:,m)) ;         
     end;
       
     %create matrices and reset to zero
     Genotype_apps1New4 = zeros(size(Genotype_apps)); 
     Genotype_serv1New4 = zeros(size(Genotype_serv));
     Genotype_os1New4 = zeros(size(Genotype_os));
     Genotype_dr1New4 = zeros(size(Genotype_dr));
     
      %populate with 1's using I values
     for h = 1:numNodes
          %choose OS solution
                Genotype_apps1New4(h,maxFav4OSGroup_appsI(h,1)) = 1;
                Genotype_serv1New4(h,maxFav4OSGroup_servI(h,1)) = 1;
                Genotype_os1New4(h,maxFav4OSGroup_osI(h,1)) = 1;
                Genotype_dr1New4(h,maxFav4OSGroup_drI(h,1)) = 1;
     end;  
        
     %only update the genotypes for those that are ready to be updated
     for h = 1:numNodes
         if (updateYes(h) == 1) && (updateStop(h) == 1)

             Genotype_appsNew(h,:) = Genotype_apps1New4(h,:);
             Genotype_servNew(h,:) = Genotype_serv1New4(h,:);
             Genotype_osNew(h,:) = Genotype_os1New4(h,:);
             Genotype_drNew(h,:) = Genotype_dr1New4(h,:);
         else
             
             Genotype_appsNew(h,:) = Genotype_apps(h,:);
             Genotype_servNew(h,:) = Genotype_serv(h,:);
             Genotype_osNew(h,:) = Genotype_os(h,:);
             Genotype_drNew(h,:) = Genotype_dr(h,:);
         end;    
     end;

         
     %output from solution (5)
     Genotype_New = [Genotype_appsNew Genotype_servNew Genotype_osNew Genotype_drNew];
     
 
       
    otherwise
        disp('SOLUTION NUMBER NOT VALID')
end; %end swith

