%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Jennifer Jackson 2017                                                        %
% A Biodiversity Approach to Cyber Security                                    %
% PhD Complexity Science                                                       %
% University of Warwick                                                        %
% http://wrap.warwick.ac.uk/98458                                              %
% This software is distributed under the  Creative Commons Attribution License %
% https://creativecommons.org/licenses/by/4.0/                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function Nei = variantFreq4(textdata, data, plotdata)
%works with 4 different locus: apps, serv, OS, dr
%dynAlg data only has major versions at the mo represented in the data
%plotData: 1 if plot data 0 if no plot data


%locus represents a type such as an operating system
%variant represents the different versions of a type i.e diff.operating systems
data;
[tm,tn] = size(textdata);
[m,n] = size(data);
%columns is nodes

%count number of variants in each locus

%locusApps
%'a' represents major variants
%'V' represents a minor versions
numVarApps = 0;
fApps = 1;
for i = 1:m
    if strcmp(textdata(i,1), 'a') && strcmp(textdata(i,2), 'V')
    numVarApps = numVarApps+1;
    freqvarApps(fApps) = sum(data(i,1:n));
    fApps = fApps+1;
end
end
totalvarAppsInUse = sum(freqvarApps);
freqvarApps = freqvarApps./totalvarAppsInUse;
%freqvarApps = freqvarApps./n;
if plotdata == 1 
figure
bar(freqvarApps)
title('frequency of Apps variants')
end;

%locusServ
%'s' represents major variants
%'V' represents a minor versions
numVarServ = 0;
fServ = 1;
for i = 1:m
    if strcmp(textdata(i,1), 's') && strcmp(textdata(i,2), 'V')
    numVarServ = numVarServ+1;
    freqvarServ(fServ) = sum(data(i,1:n));
    fServ = fServ+1;
end
end
totalvarServInUse = sum(freqvarServ);
freqvarServ = freqvarServ./totalvarServInUse;
if plotdata == 1
figure
bar(freqvarServ)
title('frequency of Serv variants')
end;


%locusOS
%'o' represents major variants, i.e Windows and Linux
%if more granularity required on versions, i.e windows vista, windows XP
%etc then..
%'V' represents a minor versions
numVarOS = 0;
fOS = 1;
for i = 1:m
    if strcmp(textdata(i,1), 'o') && strcmp(textdata(i,2), 'V')
    numVarOS = numVarOS+1;
    freqvarOS(fOS) = sum(data(i,1:n));
    fOS = fOS+1;
end
end
totalvarOSInUse = sum(freqvarOS);
freqvarOS = freqvarOS./totalvarOSInUse;
if plotdata == 1
figure
bar(freqvarOS)
title('frequency of OS variants')
end;


%locusDr
%'d' represents a major variant
%'V' represents a minor version
numVarDr = 0;
fDr = 1;
for i = 1:m
    if strcmp(textdata(i,1), 'd') && strcmp(textdata(i,2), 'V')
    numVarDr = numVarDr+1;
    freqvarDr(fDr) = sum(data(i,1:n));
    fDr = fDr+1;
end
end
freqvarDr;
totalvarDrInUse = sum(freqvarDr);
freqvarDr = freqvarDr./totalvarDrInUse;
%if freqvarIC > 1 %check for non-entries
if plotdata == 1
figure
bar(freqvarDr)
title('frequency of Dr variants')
end;

%determine (h) heterozygosity = 1-sum(freq.^2)
hApps = 1 - sum(freqvarApps.^2);
hServ = 1 - sum(freqvarServ.^2);
hOS = 1 - sum(freqvarOS.^2);
hDr = 1 - sum(freqvarDr.^2);


%determine Ae = 1/(1-h)
AeApps = 1/(1-hApps);
AeServ = 1/(1-hServ);
AeOS = 1/(1-hOS);
AeDr = 1/(1-hDr);

%determine Nei genetic diversity index sum(h)/number of loci
%if freqvarIC > 1 %check for non-entries
Nei = (hApps + hServ + hOS + hDr)/4;
%else
%    Nei = (hOS + hW)/2 %no IC data present
%end


