%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Jennifer Jackson 2017                                                        %
% A Biodiversity Approach to Cyber Security                                    %
% PhD Complexity Science                                                       %
% University of Warwick                                                        %
% http://wrap.warwick.ac.uk/98458                                              %
% This software is distributed under the  Creative Commons Attribution License %
% https://creativecommons.org/licenses/by/4.0/                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [ FinalSizeR stdR ] = finalSizeApprox( Sprop, Bn, gamma, VRmin, VRmax, numNodes)
%calculate approximate final size and std
R0 = Sprop.*Bn./gamma; %reproduction number
Ns = Sprop.*numNodes; %number of nodes in malware model

%final size approximation equation
for h = VRmin:VRmax
  text = ['1-x = exp(-1.0*(' num2str(Sprop(h)*Bn/gamma) ')*x)']; 
meanRTheory =  (solve(text, 'Real', true)).*Sprop(h).*numNodes; %multiply by the number of nodes only in the malware model
FinalSizeR(h) = max(double(meanRTheory));
end
%variance equation of the normal distribution
r_inf = FinalSizeR./Ns % final size at infinity as a proportion of the number of susceptible nodes
top_int = 1+((1-r_inf).*(R0.^2))
top = r_inf.*(1-r_inf).*(top_int)
bot = (1-((1-r_inf).*R0)).^2
var = (top./bot).*Ns %multiply by number of nodes only in the malware model
stdR = sqrt(var)
end

