%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Jennifer Jackson 2017                                                        %
% A Biodiversity Approach to Cyber Security                                    %
% PhD Complexity Science                                                       %
% University of Warwick                                                        %
% http://wrap.warwick.ac.uk/98458                                              %
% This software is distributed under the  Creative Commons Attribution License %
% https://creativecommons.org/licenses/by/4.0/                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
classdef rwpHandle < handle
    
    properties (Constant=true)
        MODE_RIR = 1;
        MODE_NIR = 2;
        MODE_AIR = 4;
    end;
    
    properties

        % References - simulation settings
        nodes       % Number of nodes (absolute)
        nodesln     % Number of limited nodes (absolute) - always first
        xmin        % Minimum x-axis location (du)
        xmax        % Maximum x-axis location (du)
        ymin        % Minimum y-axis location (du)
        ymax        % Maximum y-axis location (du)
        xminln      % Minimum x-axis location limited nodes (du)
        xmaxln      % Maximum x-axis location limited nodes (du)
        yminln      % Minimum y-axis location limited nodes (du)
        ymaxln      % Maximum y-axis location limited nodes (du)
        smin        % Minimum speed (du / tu)
        smax        % Maximum speed (du / tu)
        pmin        % Additional minimum pause between segments (integer tu)
        pmax        % Additional maximum pause between segments (integer tu)
        
        dinr        % Radial distance for in-range (du)
        dexceed     % Exceeding distance to use as a check (du)

        tinrg       % Total time in-range for genotype transfer (tu)
        tinrv       % Total time in-range for virus transfer (tu) 
        mode        % Waypoint option
      
        % References - check for end
        xlow        % Keep track of lowest x-coordinate waypoint
        xhigh       % Keep track of highest x-coordinate waypoint
        ylow        % Keep track of lowest y-coordinate waypoint
        yhigh       % Keep track of highest y-coordinate waypoint
        xlowln      % Keep track of lowest x-coordinate waypoint (ln)
        xhighln     % Keep track of highest x-coordinate waypoint (ln)
        ylowln      % Keep track of lowest y-coordinate waypoint (ln)
        yhighln     % Keep track of highest y-coordinate waypoint (ln)

        % References - calculated nodes
        nodesfr     % Number of full-range nodes
        
        % References - indices
        nodeslnminind   % Node limited-range min index
        nodeslnmaxind   % Node limited-range max index
        nodesfrminind   % Node full-range min index
        nodesfrmaxind   % Node full-range max index

        % References 1R x NC, one per node
        xminv   % Node array limited/full range min index x
        xmaxv   % Node array limited/full range max index x
        yminv   % Node array limited/full range min index y
        ymaxv   % Node array limited/full range max index y

        % Timestep
        t           % Timestep number
        
        % References 1R x NC, one per node
        lx          % Last timestep x-coordinate (du)
        ly          % Last timestep y-coordinate (du)
        cx          % Current timestep x-coordinate (du)
        cy          % Current timestep y-coordinate (du)
        tx          % Target x-coordinate (du)
        ty          % Target y-coordinate (du)
        dx          % Delta x-coordinate per timestep (du)
        dy          % Delta y-coordinate per timestep (du)
        sseg        % Current speed this segment (su  = du / tu)
        aseg        % Current direction this segment as angle (rad)
        dseg        % Current distance this segment (du)
        tseg        % Timesteps for this segment (absolute)
        pseg        % Pause value for this segment (integer tu)
        
        % References NR x NC, one per node pairing
        ddx         % Delta distance between Node N (R) and Node M (C) (du)
        ddy         % Delta distance between Node N (R) and Node M (C) (du)
        dd          % Delta distance between Node N (R) and Node M (C) (du)
        tir         % Time steps that Node N (R) and Node M (C) have been in range (dt)
        tag         % Tagged Node N (R) on Node M (C) (no units)
        
        % Eye matrix to prevent nodes seeing themselves at 0 distance
        eyeo        % Diagonal matrix with 'intmax' on diagonal entries
        
        % Matrix to convert one-hot row encoded tag to node number
        c2ind       % Duplicate column matrix to provide conversion
        
        % In Range List - 1R x NC array
        irl         % Cell array for holding variable length vectors
        irf         % Used as Boolean to show if in-range flagged for this timestep
        
        % Marked List - 1R x NC array
        mlg         % Array entries set at tagged node number if genotype transfer completes this cycle 
        mlv         % Array entries set at tagged node number if virus    transfer completes this cycle
        
        % Node list - list of node indices that are to be logged
        logena      % Array with indices of each node to log
 
        % Handle list - 1R x NC, one per node for each log file
        logTrack    % Array with file handle, where used, for track log
        logDist     % Array with file handle, where used, for distance log
        logTIR      % Array with file handle, where used, for timesteps-in-range log
        logGVFlag   % Array with file handle, where used, for genotype/virus flag log
        
        % Log directory
        logDir      % Directory for log files
        
    end
end