%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Jennifer Jackson 2017                                                        %
% A Biodiversity Approach to Cyber Security                                    %
% PhD Complexity Science                                                       %
% University of Warwick                                                        %
% http://wrap.warwick.ac.uk/98458                                              %
% This software is distributed under the  Creative Commons Attribution License %
% https://creativecommons.org/licenses/by/4.0/                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function rwpLog(rwph)
            
c1lu  = ['.', '+'];
c2lu  = ['.', '*'];
taglu = ['N', 'Y'];

% Iterate over nodes enabled for logging
if ~isempty(rwph.logena)
    for m = rwph.logena
        
        % Track log - individual entries logged
        fprintf(rwph.logTrack(m), '%6.0f,', rwph.t);
        fprintf(rwph.logTrack(m), '%10.3f,', rwph.lx(m));
        fprintf(rwph.logTrack(m), '%10.3f,', rwph.ly(m));
        fprintf(rwph.logTrack(m), '%10.3f,', rwph.cx(m));
        fprintf(rwph.logTrack(m), '%10.3f,', rwph.cy(m));
        fprintf(rwph.logTrack(m), '%10.3f,', rwph.tx(m));
        fprintf(rwph.logTrack(m), '%10.3f,', rwph.ty(m));
        fprintf(rwph.logTrack(m), '%10.3f,', rwph.dx(m));
        fprintf(rwph.logTrack(m), '%10.3f,', rwph.dy(m));
        fprintf(rwph.logTrack(m), '%10.3f,', rwph.sseg(m));
        fprintf(rwph.logTrack(m), '%10.3f,', rad2deg(rwph.aseg(m)));
        fprintf(rwph.logTrack(m), '%10.3f,', rwph.dseg(m));
        fprintf(rwph.logTrack(m), '%6.0f,', rwph.tseg(m));
        fprintf(rwph.logTrack(m), '%5.0f,', rwph.pseg(m));
        if rwph.irf(m) ~= 0
            fprintf(rwph.logTrack(m), '   1,');
            fprintf(rwph.logTrack(m), ' <');
            
            % Extract the indices of those in range - this will produce
            % an array with the indicies which Matlab can use directly
            % to index just those relevant parts of the array for d = ... 
            % and tir = ... below
            c1 = c1lu((rwph.mlg(m) > 0) + 1);
            c2 = c2lu((rwph.mlv(m) > 0) + 1);
            f1 = find(rwph.tag(m,:));
            if isempty(f1)
                f1s = '---';
            else
                f1s = sprintf('%03.0f', f1);
            end;
            fprintf(rwph.logTrack(m), '[%c%c:[%03.0f,%03.0f]:%s] ', c1, c2, rwph.mlg(m), rwph.mlv(m), f1s);
            for k = cell2mat(rwph.irl(m)),
                d = rwph.dd(m, k);
                tir = rwph.tir(m, k);
                tag = rwph.tag(m, k);
                tags = taglu(tag + 1);
                alsotagind = find(rwph.tag(:,k))'; % List of any others also tagging this target
                if isempty(alsotagind)
                    atis = '---';
                else
                    atis = sprintf('%03.0f ', alsotagind);
                end;
                %o = zeros(1, length(n) + length(d) + length(tir) + length(tag) + length(alsotagind));
                %o(1:5:end) = n;
                %o(2:5:end) = d;
                %o(3:5:end) = tir;
                %o(4:5:end) = tag;
                %o(5:5:end) = alsotagind;
                %fprintf(rwph.logTrack(m), '%.0f@%.3f=%03.0f-%1.0f-{%0.f}, ', o);
                fprintf(rwph.logTrack(m), '%3.0f@%.3f=%03.0f-%c-{%s}, ', k, d, tir, tags, atis);
            end;
            fprintf(rwph.logTrack(m), '>,');
        else
            fprintf(rwph.logTrack(m), '   0,');
            fprintf(rwph.logTrack(m), ' <NONE>\t\t\t\t\t\t\t\t,');
        end;
        fprintf(rwph.logTrack(m), '\n');
       
        % Distance log - dumps all columns in one go
        fprintf(rwph.logDist(m), '%6.0f,', rwph.t);
        fprintf(rwph.logDist(m), '%10.3f,', rwph.dd(m,:));
        fprintf(rwph.logDist(m), '\n');
        
        % TIR log - dumps all columns in one go
        fprintf(rwph.logTIR(m), '%6.0f,', rwph.t);
        fprintf(rwph.logTIR(m), '%3.0f,', rwph.tir(m,:));
        fprintf(rwph.logTIR(m), '\n');
        
        % Genotype / Virus flag log - dumps node's columns
        fprintf(rwph.logGVFlag(m), '%6.0f,', rwph.t);
        fprintf(rwph.logGVFlag(m), '%03.0f/%03.0f,', rwph.mlg(m), rwph.mlv(m));
        fprintf(rwph.logGVFlag(m), '\n');
        
    end;
end;


