%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Jennifer Jackson 2017                                                        %
% A Biodiversity Approach to Cyber Security                                    %
% PhD Complexity Science                                                       %
% University of Warwick                                                        %
% http://wrap.warwick.ac.uk/98458                                              %
% This software is distributed under the  Creative Commons Attribution License %
% https://creativecommons.org/licenses/by/4.0/                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function rwpPlot(ld, l, xl, xu, yl, yu, tmin, tmax)

% Set tmin and tmax to 0 to ignore the bound as all timesteps are positive

% LUTs - need to be cell arrays or it glues the entries together...
lui = {'k+', 'r+', 'g+', 'b+'};
luf = {'kx', 'rx', 'gx', 'bx'};
lur = {'ko', 'ro', 'go', 'bo'};
lun = {'k', 'r', 'g', 'b'};

% Setup graph
figure;
ns = '';
for k = 1:length(l)
    n = l(k);
    ns = sprintf('%s %i', ns, n);
end;
s = sprintf('RWP Nodes {%s}, Time=%i:%i', ns, tmin, tmax);
set(gcf,'Name', s);
title(s);
axis([xl xu yl yu]);
hold on;

% Iterate over node entries in the array provided
for k = 1:length(l)
    
    % Get the node number
    n = l(k);
    
    % Calculate the colour map
    q = mod(n - 1, length(lui)) + 1;
    
    % Open the file for this entry
    s = sprintf('%s\\rwpLogTrack-Node-%05i.txt', ld, n);
    fh = fopen(s, 'rt');
    if( fh < 0 )
        fprintf('Unable to open file for node : %3.0f', n);
        return;
    end;
    
    % Scan each line
    cl = fgetl(fh);
    while cl ~= -1
    
        % Skip comments
        [cmt,c,d1,d2] = sscanf(cl, '%s');
        if strncmp(cmt, '#', 1) == 0
        
            % Extract the following values - TS, <LX>, <LY>, <CX>, <CY>, TX, TY, 
            % DX, DY, SSEG, ASEG, DSEG, TSEG, PSEG, <IRL>
            [a,c,d1,d2] = sscanf(cl, '%i,%f,%f,%f,%f,%*f,%*f,%*f,%*f,%*f,%*f,%*f,%*f,%*f,%i');
            if c ~= 6
               fprintf('Unable to parse line');
               return;
            end;
                        
            % Extract values
            ts = a(1);
            lx = a(2);
            ly = a(3);
            cx = a(4);
            cy = a(5);
            irl = a(6);
 
            % Continue to next line if outside wanted boundary
            if (tmin == 0 || ts >= tmin) && (tmax == 0 || ts <= tmax)
               
                % Ignore the initial entry, if seen, since it is only
                % for setup - check for this after the wanted bounds check
                % to ensure that is enforced
                if ts ~= 0
                
                    % Log to console (debug)
                    % fprintf('%03.0f, %03.0f @ %5i = ', n, q, ts);
                    % fprintf('(%8.3f,%8.3f) -> (%8.3f,%8.3f) [%1i]\n', lx, ly, cx, cy, irl);

                    % Draw markers for first entry
                    if (tmin == 0 && ts == 1) || ts == tmin
                        plot([lx lx], [ly ly], lui{q});
                    end;

                    % If in range, draw a marker
                    if irl == 1
                        plot([cx cx], [cy cy], lur{q});
                    end;

                    % Normal segment
                    plot([lx cx], [ly cy], lun{q});
                    fxx = cx;
                    fy = cy;
                    
                end;
                    
            end;
                
        end;
        
        % Get the next line
        cl = fgetl(fh);
        
    end;
    
    % Final marker - draw from final in case time limits do
    % not plot some of the lines
    plot([fxx fxx], [fy fy], luf{q});
    
    % Close the file
    fclose(fh);
    
end;
    

