%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Jennifer Jackson 2017                                                        %
% A Biodiversity Approach to Cyber Security                                    %
% PhD Complexity Science                                                       %
% University of Warwick                                                        %
% http://wrap.warwick.ac.uk/98458                                              %
% This software is distributed under the  Creative Commons Attribution License %
% https://creativecommons.org/licenses/by/4.0/                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Clear everything
%close all;
%clear all;
%clc;

% Notify user of progress
fprintf('Setup\n');

% Create the reference class
rwph = rwpHandle;

% Setup simulation parameters
%
% -- Base parameters --
%
% du = distance units
% tu = time units
%
% -- Derived parameters --
%
% su = du / tu units
%
rwph.nodes    = 10;    % Number of nodes (absolute)
rwph.nodesln  = 0;     % Number of limited nodes (absolute) - always first
rwph.xmin     = 0;     % Minimum x-axis location (du)
rwph.xmax     = 100;   % Maximum x-axis location (du)
rwph.ymin     = 0;     % Minimum y-axis location (du)
rwph.ymax     = 100;   % Maximum y-axis location (du)
rwph.xminln   = 0;     % Minimum x-axis location limited nodes (du)
rwph.xmaxln   = 30;    % Maximum x-axis location limited nodes (du)
rwph.yminln   = 0;     % Minimum y-axis location limited nodes (du)
rwph.ymaxln   = 30;    % Maximum y-axis location limited nodes (du)
rwph.smin     = 0.25;  % Minimum speed (du / tu)
rwph.smax     = 2.00;  % Maximum speed (du / tu)
rwph.pmin     = 2;     % Minimum time for pause (integer tu)
rwph.pmax     = 4;     % Maximum time for pause (integer tu)
rwph.dinr     = 5;     % Radial distance for in-range (du)
rwph.dexceed  = 1000;  % Value larger than any possible distance (du)
rwph.tinrg    = 3;     % Time in-range for genotype transfer (tu)
rwph.tinrv    = 5;     % Time in-range for virus transfer (tu)

% Mode
% rwph.mode = rwph.MODE_RIR;      % Select Random in range mode
% rwph.mode = rwph.MODE_NIR;      % Select Nearest in range mode
rwph.mode = rwph.MODE_AIR;      % Select (Random) Available in range mode

% Log vector
rwph.logena = [1 2 7 8];

% Log directory
rwph.logDir = sprintf('%s\\log', pwd);

% Set iteration steps
l = 1000;

% Initialise the simulation
fprintf('Initialising\n');
rwpInit(rwph);

% Iterate the simulation
fprintf('Iterating\n');
for n = 1:l
    rwpIterate(rwph);
end;

% Cleanup
fprintf('Clean-up\n');
rwpCleanup(rwph);

% Log and validate waypoint extremes
fprintf('Extremes : (%.3f, %.3f) - (%.3f, %.3f)\n', ...
        rwph.xlow, rwph.ylow, rwph.xhigh, rwph.yhigh);
if( rwph.xlow  >= rwph.xmin && rwph.ylow  >= rwph.ymin && ...
    rwph.xhigh <= rwph.xmax && rwph.yhigh <= rwph.ymax)
    fprintf('\tOK - waypoints all within allowed range\n');
else
    fprintf('\tERROR - some waypoints exceed allowed range\n');
end;
if rwph.nodesln > 0
    fprintf('Extremes (ln) : (%.3f, %.3f) - (%.3f, %.3f)\n', ...
            rwph.xlowln, rwph.ylowln, rwph.xhighln, rwph.yhighln);
    if( rwph.xlowln  >= rwph.xminln && rwph.ylowln  >= rwph.yminln && ...
        rwph.xhighln <= rwph.xmaxln && rwph.yhighln <= rwph.ymaxln)
        fprintf('\tOK - limited node waypoints all within allowed range\n');
    else
        fprintf('\tERROR - some limited node waypoints exceed allowed range\n');
    end;
end;

% Plot all logged entries by reloading file data ONLY - can limit timestep
% when not set to zero
fprintf('Plotting\n');
rwpPlot(rwph.logDir, rwph.logena, rwph.xmin, rwph.xmax, rwph.ymin, rwph.ymax, 0, 0);
%rwpPlot(rwph.logDir, [1 2 5 6], rwph.xmin, rwph.xmax, rwph.ymin, rwph.ymax, 0, 0);
%rwpPlot(rwph.logDir, [1 2 5 6], rwph.xmin, rwph.xmax, rwph.ymin, rwph.ymax, 500, 550);

% End of simulation script
fprintf('End\n');


