%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Jennifer Jackson 2017                                                        %
% A Biodiversity Approach to Cyber Security                                    %
% PhD Complexity Science                                                       %
% University of Warwick                                                        %
% http://wrap.warwick.ac.uk/98458                                              %
% This software is distributed under the  Creative Commons Attribution License %
% https://creativecommons.org/licenses/by/4.0/                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [Ro St It Rt] = SIRDeterministic(beta, gamma, S, I, R, timeMax, drawPlots)
%Deterministic SIR model
%Difference equations:
%St+1 = St - beta*It*(St/N)
%It+1 = It + beta*It*(St/N) - v*It
%Rt+1 = Rt + v*It
%clear all;
%close all;

%set parameters
%beta = 0.02/2; %0.5; %probability of getting the disease
%D = 2; %0.0202; %0.000202;%no N %5; %duration of infection
%v = 0; %1/D; %rate of recovery

%set initial values of population
N = S; %total population size
It(1) = I; %10; %number of infected
St(1) = N - It(1); %number of susceptibles
Rt(1) = R; %number of recovered
%timeMax = 2000
%check reproduction number
Ro = beta/gamma;

t = [1:timeMax];

for i = 1:timeMax-1
St(i+1) = St(i) - beta*It(i)*(St(i)/N);
It(i+1) = It(i) + beta*It(i)*(St(i)/N) - gamma*It(i);
Rt(i+1) = Rt(i) + gamma*It(i);   
B(i)=-(St(i+1) - St(i))/(It(i)*St(i)/N); %measured beta
end

if drawPlots == 1
    resilience = St+Rt;
figure
plot(t,St,'b')
hold on;
plot(t,It,'r')
hold on;
plot(t,Rt,'g')
hold on;
plot(t,resilience,'c')
hold on;
legend('Susceptibles','Infected','Recovered', 'Resilience perturbation');
title('Matlab SIR model implementation using difference equations');
xlabel('time');
ylabel('population number');

%measured beta
%figure
%plot(B,'b')
end
end
