%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Jennifer Jackson 2017                                                        %
% A Biodiversity Approach to Cyber Security                                    %
% PhD Complexity Science                                                       %
% University of Warwick                                                        %
% http://wrap.warwick.ac.uk/98458                                              %
% This software is distributed under the  Creative Commons Attribution License %
% https://creativecommons.org/licenses/by/4.0/                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [ S I R ] = stochasticSIR(numNodes, beta, numTimesteps, dtt, gamma)
%discrete SIR Markov model
%initialisation values
N = numNodes;
I(1) = 1;
S(1) = N - I(1);
R(1) = N - I(1)-S(1);

for t = 1:numTimesteps-1
%define probabilities at each time step
prob1 = beta*S(t)*I(t)*dtt/N; %-1,1
prob2 = gamma*I(t)*dtt; %0,-1
prob3 = 1 - (prob1+prob2); %0,0
sumCheck = prob1+prob2+prob3; %should sum to 1
if (prob1+prob2 > 1); disp('dtt not set small enough'); end;%delta time check
u1=rand; % uniform random number
if u1 <= prob1
    S(t+1) = S(t)-1;
    I(t+1) = I(t)+1;
elseif and(u1 > prob1, u1 <=(prob1+prob2))
    S(t+1) = S(t);
    I(t+1) = I(t)-1;
else
    S(t+1) = S(t);
    I(t+1) = I(t);
end
R(t+1) = N - I(t+1)-S(t+1);
end
end

