%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Jennifer Jackson 2017                                                        %
% A Biodiversity Approach to Cyber Security                                    %
% PhD Complexity Science                                                       %
% University of Warwick                                                        %
% http://wrap.warwick.ac.uk/98458                                              %
% This software is distributed under the  Creative Commons Attribution License %
% https://creativecommons.org/licenses/by/4.0/                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [ output_args ] = testStochasticSIR( input_args )
%Test stochastic SIR function
close all;
clear all;
doTests = zeros(1,10) %tests with no genotypes

%***************

%***************
% SIR BASIC TESTS
%**************
%outputDirectory = 'stochasticSIRGraphs\' 
%doTests(1) = 1 %SI single beta with graphs- compare to deterministic
doTests(2) = 1 %SIR - compare to SIR deterministic
%doTests(3) = 1 %SIR - compare to SIR deterministic and agent

if doTests(1) == 1
outputDirectory = 'stochasticSIRGraphs\'
numNodes = 1000
beta = 0.3
gamma = 0
maxTime = 80;
numSims = 1000;

%test stochastic
dtt=0.01; % Time step
numTimesteps = floor(maxTime/dtt); %7000
time = 0:dtt:numTimesteps*dtt-dtt;
for i = 1:numSims
 [ S(i,:) I(i,:) R(i,:) ] = stochasticSIR(numNodes, beta, numTimesteps, dtt, gamma);
end

 figS = figure
 for i = 1:50
 plot(time,S(i,:),'k')
 hold on
 end
 xlabel('Time')
 ylabel('Susceptible')


 figI = figure
 for i = 1:50
 plot(time,I(i,:),'k')
 hold on
 end
 xlabel('Time')
 ylabel('Infected')
 
meanI=mean(I);
meanS=mean(S);
 %compare to deterministic
 S = numNodes
 I = 1
 R = 0
 timeMax = time(length(time));
 [Ro St It Rt] = SIRDeterministic(beta, gamma, S, I, R, timeMax, 0);
 step = timeMax / length(It);
 ts = [0:step:timeMax-step];
 size(ts)
 size(It)
 
 %plot 
 figmean = figure
 plot(time,meanS,'k--', 'Linewidth',1)
 hold on
  plot(time,meanI,'k', 'Linewidth',2)
 hold on
 plot(ts,It,'k.-', 'Linewidth',1)
 legend('S', 'I', 'Deterministic I')
 xlabel('Time')
 ylabel('Infected')
 
saveas(figS,[outputDirectory 'Test1figS.fig']);
saveas(figS,[outputDirectory 'Test1figS.emf']);
saveas(figI,[outputDirectory 'Test1figI.fig']);
saveas(figI,[outputDirectory 'Test1figI.emf']);
saveas(figmean,[outputDirectory 'Test1figmean.fig']);
saveas(figmean,[outputDirectory 'Test1figmean.emf']);
end%dotests1
%****************************************
%Test2
if doTests(2) == 1
close all;
outputDirectory = 'stochasticSIRGraphs\'
numNodes = 1000
beta = 0.3
gamma = 0.15
maxTime = 120;
numSims = 50;

%test stochastic
dtt=0.01; % Time step
numTimesteps = floor(maxTime/dtt); %7000
time = 0:dtt:numTimesteps*dtt-dtt;
for i = 1:numSims
 [ S(i,:) I(i,:) R(i,:) ] = stochasticSIR(numNodes, beta, numTimesteps, dtt, gamma);
end

 figS = figure
 for i = 1:50
 plot(time,S(i,:),'k')
 hold on
 end
 xlabel('Time')
 ylabel('Susceptible')


 figI = figure
 for i = 1:50
 plot(time,I(i,:),'k')
 hold on
 end
 xlabel('Time')
 ylabel('Infected')
 
  figR = figure
 for i = 1:50
 plot(time,R(i,:),'k')
 hold on
 end
 xlabel('Time')
 ylabel('Recovered')
 
meanI=mean(I);
meanS=mean(S);
 %compare to deterministic
 S = numNodes
 I = 1
 R = 0
 timeMax = time(length(time));
 [Ro St It Rt] = SIRDeterministic(beta, gamma, S, I, R, timeMax, 0);
 step = timeMax / length(It);
 ts = [0:step:timeMax-step];
 size(ts)
 size(It)
 %deterministic decimate
 decBy = 5;
timeScale = 1:decBy:ts
 %plot 
 figmean = figure
 plot(time,meanS,'k--', 'Linewidth',1)
 hold on
  plot(time,meanI,'k', 'Linewidth',2)
 hold on
 plot(ts(1:decBy:end),It(1:decBy:end),'k.-', 'Linewidth',1)
 legend('S', 'I', 'Deterministic I')
 xlabel('Time')
 ylabel('Infected')
 
saveas(figS,[outputDirectory 'Test2figS.fig']);
saveas(figS,[outputDirectory 'Test2figS.emf']);
saveas(figI,[outputDirectory 'Test2figI.fig']);
saveas(figI,[outputDirectory 'Test2figI.emf']);
saveas(figR,[outputDirectory 'Test2figR.fig']);
saveas(figR,[outputDirectory 'Test2figR.emf']);
saveas(figmean,[outputDirectory 'Test2figmean.fig']);
saveas(figmean,[outputDirectory 'Test2figmean.emf']);
end%dotests2



%*******************
%Test 3
if doTests(3) == 1
numNodes = 1000
beta = 0.3
gamma = 0
maxTime = 100;
numSims = 1000;

%test stochastic
dtt=0.01; % Time step
numTimesteps = floor(maxTime/dtt); %7000
time = 0:dtt:numTimesteps*dtt-dtt;
for i = 1:numSims
 [ S(i,:) I(i,:) R(i,:) ] = stochasticSIR(numNodes, beta, numTimesteps, dtt, gamma);
end
meanI=mean(I);
 
%compare to agent
for i = 1:numSims
 [Sa(i,:) Ia(i,:) Ra(i,:)] = agentSIR(numNodes, beta, maxTime-1, 0, gamma);
end
meanIa=mean(Ia);
 
 %compare to deterministic
 S = numNodes
 I = 1
 R = 0
 timeMax = time(length(time));
 [Ro St It Rt] = SIRDeterministic(beta, gamma, S, I, R, timeMax, 0);
 step = timeMax / length(It);
 ts = [0:step:timeMax-step];
 size(ts)
 size(It)
 
 %plot 
 figure
 plot(time,meanI,'k--', 'Linewidth',2)
 hold on
 size (meanIa)
 size(ts)
 plot(ts,meanIa,'k', 'Linewidth',2)
 plot(ts,It,'k.-', 'Linewidth',1)
 legend('stochastic', 'agent', 'deterministic')
 xlabel('Time')
 ylabel('Infected')
end%dotests2


end

