%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Jennifer Jackson 2017                                                        %
% A Biodiversity Approach to Cyber Security                                    %
% PhD Complexity Science                                                       %
% University of Warwick                                                        %
% http://wrap.warwick.ac.uk/98458                                              %
% This software is distributed under the  Creative Commons Attribution License %
% https://creativecommons.org/licenses/by/4.0/                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
classdef GUIClass
    %basic library of functions used within the GUI 
    
    %properties list
    properties
    end
    
    %ordinary functions and class constructor to go here
    methods
    end
    
    %static methods that can be called without instantiating a class
    methods (Static)
            %**********************************************************************************
            %Generate program limits
            %**********************************************************************************
             function maxNumSims = getMaxNumSims()
                %maximum number of simulations is currently set to 30
                maxNumSims = 30;
             end
             function fileExtension = getFileExtension(binmode)
                 if binmode == 1
                    fileExtension  = 'bin'; %need to change to 'bin'
                 else
                    fileExtension = 'txt';
                 end
             end
            %***********************************************************************************
            %Initialise and monitor simulation progress and times
            %***********************************************************************************
            function startTime = recordSimStart(hObject,handles)
                %record the start time of the simulation
                startTime = fix(clock);
                startTimeStr = [num2str(startTime(4)) ':' num2str(startTime(5)) ':' num2str(startTime(6))];
                set(handles.editSimStartTime, 'String' ,startTimeStr);
                endTimeStr = ['00' ':' '00' ':' '00'];
                set(handles.editSimEndTime, 'String' ,endTimeStr);
                %Update handles structure
                guidata(hObject, handles);
            end
            function recordSimEnd(hObject,handles)
                %record the end time of the simulation
                endTime = fix(clock);
                endTimeStr = [num2str(endTime(4)) ':' num2str(endTime(5)) ':' num2str(endTime(6))];
                set(handles.editSimEndTime, 'String' ,endTimeStr);
                %Update handles structure
                guidata(hObject, handles);
            end
            function recordProgressStart(hObject,handles)
                %record the start progress of the simulation
                numFiles = get(handles.popupmenuNumSims, 'Value');
                numSims = num2str(numFiles);
                progressStr = ['0%  0 of ' numSims]
                set(handles.editSimProgress, 'String' ,progressStr);
                %Update handles structure
                guidata(hObject, handles);
                pause(0.001)%time to update screen
            end
            function recordProgressEnd(hObject,handles)
                %record the end progress of the simulation
                numSims = num2str(get(handles.popupmenuNumSims, 'Value'));
                progressStr = ['100%  ' numSims ' of ' numSims]
                set(handles.editSimProgress, 'String' ,progressStr);
                %Update handles structure
                guidata(hObject, handles);
            end
            %**********************************************************************************
            %Generate directory names
            %**********************************************************************************
            function directorynameSettings = getDirectoryNameSettings()
                %directorynameSettings = 'settings\';
                directorynameSettings = 'settings\';
            end
            function directorynameOut = getDirectoryNameOut()
                %directorynameOut = 'outputs\';
                directorynameOut = 'outputs\';
            end       
            function directorynameWeights = getDirectoryNameWeights()
                %directorynameWeights = 'weights\';
                directorynameWeights='weights\';
            end
            function directorynameScripts = getDirectoryNameScripts()
                %directorynameScripts = 'scripts\';
                directorynameScripts='scripts\';
            end
            function directorynameDesirabilities = getDirectoryNameDesirabilities()
                %directorynameDesirabilities = 'desirabilities\';
                directorynameDesirabilities='desirabilities\';
            end
            %**********************************************************************************
            %Generate file names
            %**********************************************************************************
            function [settingsTempFileNames settingsSimFileNames outputsTempFileNames outputsSimFileNames directorynameOut folderName directoryName] = genAllFileNames(handles,startTime)
                %**************************************
                numFiles = get(handles.popupmenuNumSims, 'Value');
                %check file extensions to use for output data
                binmode=get(handles.checkboxbinmode,'Value'); %this is set for each setting but should be applied to all sims really
                fileSavePrecision= get(handles.popupmenuFileSavePrecision, 'Value') %check if save as mat extension (3)
                if fileSavePrecision < 3
                    fileExtension = GUIClass.getFileExtension(binmode);
                else
                    fileExtension = '.mat'; %save as mat files
                end
                %*****************************
                %names of settings temp files
                directorynameSettings = GUIClass.getDirectoryNameSettings();
                settingsTempFileNames = GUIClass.genSettingsTempFileNames(directorynameSettings,numFiles);
                %names of outputs temp files
                directorynameOut = GUIClass.getDirectoryNameOut();
                outputsTempFileNames = GUIClass.genOutputsTempFileNames(directorynameOut,numFiles,fileExtension);
                %names of automated outputs
                folderName=[num2str(startTime(1)) '-' num2str(startTime(2)) '-' num2str(startTime(3)) '-' num2str(startTime(4)) '-' num2str(startTime(5)) '-' num2str(startTime(6)) '\'];
                directoryName = [directorynameOut folderName];
                outputsSimFileNames = GUIClass.genOutputsSimFileNames(directoryName,numFiles,fileExtension);
                settingsSimFileNames = GUIClass.genSettingsSimFileNames(directoryName,numFiles);
            end
            function filenames = genSettingsTempFileNames(directoryname,numFiles)
            %call function by writing GUIClass.genSettingsTempFileNames(directoryname,numFiles)
            %directoryname = 'settings\';
                for i = 1:numFiles    
                %S = {'Hello' 'Goodbye'; 'Yes' 'No'} %cell array of strings
                %C = char('Hello','Goodbye','Yes','No') %character array of strings
                filenames(i) = cellstr([directoryname 'settingsTemp' int2str(i) '.txt']);           
                end
            end
            function filenames = genSettingsSimFileNames(directoryname,numFiles) %for autosave
                for i = 1:numFiles    
                filenames(i) = cellstr([directoryname 'settings_set' int2str(i) '.txt']);           
                end
            end
            function filenames = genSettingsSaveAsFileNames(editSaveAs,numFiles) %for manual save
                for i = 1:numFiles  
                filenames(i) = cellstr([editSaveAs '_set' int2str(i) '.txt']);           
                end
            end
            function filenames = genOutputsTempFileNames(directoryname,numFiles,extension)
            %extension can be 'bin' - binary or 'txt' - text
                for i = 1:numFiles    
                filenames(i) = cellstr([directoryname 'outputsTemp' int2str(i) '.' extension]);           
                end
            end
            function filenames = genOutputsSimFileNames(directoryname,numFiles,extension)
            %extension can be 'bin' - binary or 'txt' - text
                for i = 1:numFiles    
                filenames(i) = cellstr([directoryname 'outputs_sim' int2str(i) '.' extension]);           
                end
            end
            %added 20th April 2016
            function filenames = genOutputsTempFileNames_simHeaders(directoryname,numFiles,extension)
            %extension can be 'bin' - binary or 'txt' - text
                for i = 1:numFiles    
                filenames(i) = cellstr([directoryname 'outputsTemp_sim' int2str(i) 'Header.' extension]);           
                end
            end
        %    function filenames = genOutputsTempFileNames_runFiles(directoryname,numFiles,extension)
        %    %extension can be 'bin' - binary or 'txt' - text
        %        for i = 1:numFiles    %numSims
        %            for j = 1:numRuns
        %                filenames(i,j,1) = cellstr([directoryname 'outputsTemp_sim' int2str(i) 'Run' int2str(j) 'Nei_sol4' '.' extension]); 
        %                filenames(i,j,2) = cellstr([directoryname 'outputsTemp_sim' int2str(i) 'Run' int2str(j) 'Nei_sol5' '.' extension]); 
        %                filenames(i,j,3) = cellstr([directoryname 'outputsTemp_sim' int2str(i) 'Run' int2str(j) 'nodesDiversity4' '.' extension]); 
        %                filenames(i,j,4) = cellstr([directoryname 'outputsTemp_sim' int2str(i) 'Run' int2str(j) 'nodesMean4' '.' extension]); 
        %                filenames(i,j,5) = cellstr([directoryname 'outputsTemp_sim' int2str(i) 'Run' int2str(j) 'nodesStd4' '.' extension]); 
        %                filenames(i,j,6) = cellstr([directoryname 'outputsTemp_sim' int2str(i) 'Run' int2str(j) 'updateStopDisplay4' '.' extension]); 
        %                filenames(i,j,7) = cellstr([directoryname 'outputsTemp_sim' int2str(i) 'Run' int2str(j) 'minStd4Curve' '.' extension]); 
        %                filenames(i,j,8) = cellstr([directoryname 'outputsTemp_sim' int2str(i) 'Run' int2str(j) 'nodesStd5' '.' extension]); 
        %                filenames(i,j,9) = cellstr([directoryname 'outputsTemp_sim' int2str(i) 'Run' int2str(j) 'updateStopDisplay5' '.' extension]); 
        %                filenames(i,j,10) = cellstr([directoryname 'outputsTemp_sim' int2str(i) 'Run' int2str(j) 'minStd5Curve' '.' extension]);                     
        %                filenames(i,j,11) = cellstr([directoryname 'outputsTemp_sim' int2str(i) 'Run' int2str(j) 'lastStoppedStdMatrix' '.' extension]); 
        %                filenames(i,j,12) = cellstr([directoryname 'outputsTemp_sim' int2str(i) 'Run' int2str(j) 'lastStoppedStdMatrix2' '.' extension]);                    
        %                filenames(i,j,13) = cellstr([directoryname 'outputsTemp_sim' int2str(i) 'Run' int2str(j) 'Nei_indDynamic' '.' extension]); 
        %                filenames(i,j,14) = cellstr([directoryname 'outputsTemp_sim' int2str(i) 'Run' int2str(j) 'numStates' '.' extension]); 
        %                filenames(i,j,15) = cellstr([directoryname 'outputsTemp_sim' int2str(i) 'Run' int2str(j) 'sumOfStates' '.' extension]); 
        %                filenames(i,j,16) = cellstr([directoryname 'outputsTemp_sim' int2str(i) 'Run' int2str(j) 'nodeStateAtTimeStr' '.' extension]);
        %                filenames(i,j,17) = cellstr([directoryname 'outputsTemp_sim' int2str(i) 'Run' int2str(j) 'nodeStateAtTimeNumber' '.' extension]);
        %                filenames(i,j,18) = cellstr([directoryname 'outputsTemp_sim' int2str(i) 'Run' int2str(j) 'neiIndexSingleLocus_sol4' '.' extension]);
        %                filenames(i,j,19) = cellstr([directoryname 'outputsTemp_sim' int2str(i) 'Run' int2str(j) 'neiIndexSingleLocus_sol5' '.' extension]);
        %                filenames(i,j,20) = cellstr([directoryname 'outputsTemp_sim' int2str(i) 'Run' int2str(j) 'GenoSim_sol4' '.' extension]);
        %                filenames(i,j,21) = cellstr([directoryname 'outputsTemp_sim' int2str(i) 'Run' int2str(j) 'GenoSim_sol5' '.' extension]);
        %                filenames(i,j,22) = cellstr([directoryname 'outputsTemp_sim' int2str(i) 'Run' int2str(j) 'beta' '.' extension]);
        %            end
        %        end
        %    end
            function filenames = getTempRunFileNames(directoryname,simNumber,runNumber)
                        extension = 'bin';
                        filenames(1) = cellstr([directoryname 'outputsTemp_sim' int2str(simNumber) 'Run' int2str(runNumber) 'Nei_sol4' '.' extension]); 
                        filenames(2) = cellstr([directoryname 'outputsTemp_sim' int2str(simNumber) 'Run' int2str(runNumber) 'Nei_sol5' '.' extension]); 
                        filenames(3) = cellstr([directoryname 'outputsTemp_sim' int2str(simNumber) 'Run' int2str(runNumber) 'nodesDiversity4' '.' extension]); 
                        filenames(4) = cellstr([directoryname 'outputsTemp_sim' int2str(simNumber) 'Run' int2str(runNumber) 'nodesMean4' '.' extension]); 
                        filenames(5) = cellstr([directoryname 'outputsTemp_sim' int2str(simNumber) 'Run' int2str(runNumber) 'nodesStd4' '.' extension]); 
                        filenames(6) = cellstr([directoryname 'outputsTemp_sim' int2str(simNumber) 'Run' int2str(runNumber) 'updateStopDisplay4' '.' extension]); 
                        filenames(7) = cellstr([directoryname 'outputsTemp_sim' int2str(simNumber) 'Run' int2str(runNumber) 'minStd4Curve' '.' extension]); 
                        filenames(8) = cellstr([directoryname 'outputsTemp_sim' int2str(simNumber) 'Run' int2str(runNumber) 'nodesStd5' '.' extension]); 
                        filenames(9) = cellstr([directoryname 'outputsTemp_sim' int2str(simNumber) 'Run' int2str(runNumber) 'updateStopDisplay5' '.' extension]); 
                        filenames(10) = cellstr([directoryname 'outputsTemp_sim' int2str(simNumber) 'Run' int2str(runNumber) 'minStd5Curve' '.' extension]);                     
                        filenames(11) = cellstr([directoryname 'outputsTemp_sim' int2str(simNumber) 'Run' int2str(runNumber) 'lastStoppedStdMatrix' '.' extension]); 
                        filenames(12) = cellstr([directoryname 'outputsTemp_sim' int2str(simNumber) 'Run' int2str(runNumber) 'lastStoppedStdMatrix2' '.' extension]);                    
                        filenames(13) = cellstr([directoryname 'outputsTemp_sim' int2str(simNumber) 'Run' int2str(runNumber) 'Nei_indDynamic' '.' extension]);                                                   
                        filenames(14) = cellstr([directoryname 'outputsTemp_sim' int2str(simNumber) 'Run' int2str(runNumber) 'numStates' '.' extension]); 
                        filenames(15) = cellstr([directoryname 'outputsTemp_sim' int2str(simNumber) 'Run' int2str(runNumber) 'sumOfStates' '.' extension]); 
                        filenames(16) = cellstr([directoryname 'outputsTemp_sim' int2str(simNumber) 'Run' int2str(runNumber) 'nodeStateAtTimeStr' '.' extension]);
                        filenames(17) = cellstr([directoryname 'outputsTemp_sim' int2str(simNumber) 'Run' int2str(runNumber) 'nodeStateAtTimeNumber' '.' extension]);
                        filenames(18) = cellstr([directoryname 'outputsTemp_sim' int2str(simNumber) 'Run' int2str(runNumber) 'neiIndexSingleLocus_sol4' '.' extension]);
                        filenames(19) = cellstr([directoryname 'outputsTemp_sim' int2str(simNumber) 'Run' int2str(runNumber) 'neiIndexSingleLocus_sol5' '.' extension]);
                        filenames(20) = cellstr([directoryname 'outputsTemp_sim' int2str(simNumber) 'Run' int2str(runNumber) 'GenoSim_sol4' '.' extension]);
                        filenames(21) = cellstr([directoryname 'outputsTemp_sim' int2str(simNumber) 'Run' int2str(runNumber) 'GenoSim_sol5' '.' extension]);
                        filenames(22) = cellstr([directoryname 'outputsTemp_sim' int2str(simNumber) 'Run' int2str(runNumber) 'beta' '.' extension]);
             
            end
            function filenames = getTempFileNames_simHeaders(directoryname,simNumber)
            %extension can be 'bin' - binary or 'txt' - text 
            extension = 'bin';
                filenames = cellstr([directoryname 'outputsTemp_sim' int2str(simNumber) 'Header.' extension]);           
            end
            function filenames = getsimFileNames_simHeaders(directoryname,simNumber)
            %extension can be 'bin' - binary or 'txt' - text 
               extension = 'bin';
                filenames = cellstr([directoryname 'outputs_sim' int2str(simNumber) 'Header.' extension]);           
            end
            function filenames = genOutputsSimFileNames_simHeaders(directoryname,numFiles,extension)
            %extension can be 'bin' - binary or 'txt' - text
                for i = 1:numFiles    
                filenames(i) = cellstr([directoryname 'outputs_sim' int2str(i) 'Header.' extension]);           
                end
            end
          %  function filenames = genOutputsSimFileNames_runFiles(directoryname,numFiles,numRuns,extension) 
          %  %extension can be 'bin' - binary or 'txt' - text
          %      for i = 1:numFiles    
          %          for j = 1:numRuns
          %              filenames(i,j,1) = cellstr([directoryname 'outputs_sim' int2str(i) 'Run' int2str(j) 'Nei_sol4' '.' extension]); 
          %              filenames(i,j,2) = cellstr([directoryname 'outputs_sim' int2str(i) 'Run' int2str(j) 'Nei_sol5' '.' extension]); 
          %              filenames(i,j,3) = cellstr([directoryname 'outputs_sim' int2str(i) 'Run' int2str(j) 'nodesDiversity4' '.' extension]); 
          %              filenames(i,j,4) = cellstr([directoryname 'outputs_sim' int2str(i) 'Run' int2str(j) 'nodesMean4' '.' extension]); 
          %              filenames(i,j,5) = cellstr([directoryname 'outputs_sim' int2str(i) 'Run' int2str(j) 'nodesStd4' '.' extension]); 
          %              filenames(i,j,6) = cellstr([directoryname 'outputs_sim' int2str(i) 'Run' int2str(j) 'updateStopDisplay4' '.' extension]); 
          %              filenames(i,j,7) = cellstr([directoryname 'outputs_sim' int2str(i) 'Run' int2str(j) 'minStd4Curve' '.' extension]); 
          %              filenames(i,j,8) = cellstr([directoryname 'outputs_sim' int2str(i) 'Run' int2str(j) 'nodesStd5' '.' extension]); 
          %              filenames(i,j,9) = cellstr([directoryname 'outputs_sim' int2str(i) 'Run' int2str(j) 'updateStopDisplay5' '.' extension]); 
          %              filenames(i,j,10) = cellstr([directoryname 'outputs_sim' int2str(i) 'Run' int2str(j) 'minStd5Curve' '.' extension]);                     
          %              filenames(i,j,11) = cellstr([directoryname 'outputs_sim' int2str(i) 'Run' int2str(j) 'lastStoppedStdMatrix' '.' extension]); 
          %              filenames(i,j,12) = cellstr([directoryname 'outputs_sim' int2str(i) 'Run' int2str(j) 'lastStoppedStdMatrix2' '.' extension]);                    
          %              filenames(i,j,13) = cellstr([directoryname 'outputs_sim' int2str(i) 'Run' int2str(j) 'Nei_indDynamic' '.' extension]); 
          %              filenames(i,j,14) = cellstr([directoryname 'outputs_sim' int2str(i) 'Run' int2str(j) 'numStates' '.' extension]); 
          %              filenames(i,j,15) = cellstr([directoryname 'outputs_sim' int2str(i) 'Run' int2str(j) 'sumOfStates' '.' extension]); 
          %              filenames(i,j,16) = cellstr([directoryname 'outputs_sim' int2str(i) 'Run' int2str(j) 'nodeStateAtTimeStr' '.' extension]);
          %              filenames(i,j,17) = cellstr([directoryname 'outputs_sim' int2str(i) 'Run' int2str(j) 'nodeStateAtTimeNumber' '.' extension]);
          %              filenames(i,j,18) = cellstr([directoryname 'outputs_sim' int2str(i) 'Run' int2str(j) 'neiIndexSingleLocus_sol4' '.' extension]);
          %              filenames(i,j,19) = cellstr([directoryname 'outputs_sim' int2str(i) 'Run' int2str(j) 'neiIndexSingleLocus_sol5' '.' extension]);
          %              filenames(i,j,20) = cellstr([directoryname 'outputs_sim' int2str(i) 'Run' int2str(j) 'GenoSim_sol4' '.' extension]);
          %              filenames(i,j,21) = cellstr([directoryname 'outputs_sim' int2str(i) 'Run' int2str(j) 'GenoSim_sol5' '.' extension]);
          %              filenames(i,j,22) = cellstr([directoryname 'outputs_sim' int2str(i) 'Run' int2str(j) 'beta' '.' extension]);
          %          end
          %      end
          %  end
          function filenames = getOutputRunFileNames(directoryname,folderName,simNumber,runNumber)
                        extension = 'bin';
                        filenames(1) = cellstr([directoryname folderName 'outputs_sim' int2str(simNumber) 'Run' int2str(runNumber) 'Nei_sol4' '.' extension]); 
                        filenames(2) = cellstr([directoryname folderName 'outputs_sim' int2str(simNumber) 'Run' int2str(runNumber) 'Nei_sol5' '.' extension]); 
                        filenames(3) = cellstr([directoryname folderName 'outputs_sim' int2str(simNumber) 'Run' int2str(runNumber) 'nodesDiversity4' '.' extension]); 
                        filenames(4) = cellstr([directoryname folderName 'outputs_sim' int2str(simNumber) 'Run' int2str(runNumber) 'nodesMean4' '.' extension]); 
                        filenames(5) = cellstr([directoryname folderName 'outputs_sim' int2str(simNumber) 'Run' int2str(runNumber) 'nodesStd4' '.' extension]); 
                        filenames(6) = cellstr([directoryname folderName 'outputs_sim' int2str(simNumber) 'Run' int2str(runNumber) 'updateStopDisplay4' '.' extension]); 
                        filenames(7) = cellstr([directoryname folderName 'outputs_sim' int2str(simNumber) 'Run' int2str(runNumber) 'minStd4Curve' '.' extension]); 
                        filenames(8) = cellstr([directoryname folderName 'outputs_sim' int2str(simNumber) 'Run' int2str(runNumber) 'nodesStd5' '.' extension]); 
                        filenames(9) = cellstr([directoryname folderName 'outputs_sim' int2str(simNumber) 'Run' int2str(runNumber) 'updateStopDisplay5' '.' extension]); 
                        filenames(10) = cellstr([directoryname folderName 'outputs_sim' int2str(simNumber) 'Run' int2str(runNumber) 'minStd5Curve' '.' extension]);                     
                        filenames(11) = cellstr([directoryname folderName 'outputs_sim' int2str(simNumber) 'Run' int2str(runNumber) 'lastStoppedStdMatrix' '.' extension]); 
                        filenames(12) = cellstr([directoryname folderName 'outputs_sim' int2str(simNumber) 'Run' int2str(runNumber) 'lastStoppedStdMatrix2' '.' extension]);                    
                        filenames(13) = cellstr([directoryname folderName 'outputs_sim' int2str(simNumber) 'Run' int2str(runNumber) 'Nei_indDynamic' '.' extension]);                                                   
                        filenames(14) = cellstr([directoryname folderName 'outputs_sim' int2str(simNumber) 'Run' int2str(runNumber) 'numStates' '.' extension]); 
                        filenames(15) = cellstr([directoryname folderName 'outputs_sim' int2str(simNumber) 'Run' int2str(runNumber) 'sumOfStates' '.' extension]); 
                        filenames(16) = cellstr([directoryname folderName 'outputs_sim' int2str(simNumber) 'Run' int2str(runNumber) 'nodeStateAtTimeStr' '.' extension]);
                        filenames(17) = cellstr([directoryname folderName 'outputs_sim' int2str(simNumber) 'Run' int2str(runNumber) 'nodeStateAtTimeNumber' '.' extension]);
                        filenames(18) = cellstr([directoryname folderName 'outputs_sim' int2str(simNumber) 'Run' int2str(runNumber) 'neiIndexSingleLocus_sol4' '.' extension]);
                        filenames(19) = cellstr([directoryname folderName 'outputs_sim' int2str(simNumber) 'Run' int2str(runNumber) 'neiIndexSingleLocus_sol5' '.' extension]);
                        filenames(20) = cellstr([directoryname folderName 'outputs_sim' int2str(simNumber) 'Run' int2str(runNumber) 'GenoSim_sol4' '.' extension]);
                        filenames(21) = cellstr([directoryname folderName 'outputs_sim' int2str(simNumber) 'Run' int2str(runNumber) 'GenoSim_sol5' '.' extension]);
                        filenames(22) = cellstr([directoryname folderName 'outputs_sim' int2str(simNumber) 'Run' int2str(runNumber) 'beta' '.' extension]);
             
            end
            %***********************************************************************************
            %Save, load, and delete temporary settings files
            %************************************************************************************
            %find currently selected simulation and make sure settings saved first in temp file
            %assumes data comes from main GUI
            %NO LONGER USED
           % function saveSettingsTempFile(settingsTempFileNames,settingsSimNumber,handles)
           %     filenameSettingsSimNumber = char(settingsTempFileNames(settingsSimNumber));
           %     fidtemp = fopen(filenameSettingsSimNumber, 'w');
           %     fprintf(fidtemp, '%s\n', '%Dynamic Diversity Algorithm');
           %     simText = ['%Matlab settings for simulation ' num2str(settingsSimNumber)];
           %     fprintf(fidtemp, '%s\n', simText);
           %     %save all data
           %     %disp('saving temp file')                
           %     saveSettings1Sim(fidtemp, handles);
           %     %close file
           %    fclose(fidtemp);
          %  end
            %find currently selected simulation and make sure settings saved first in temp file
            %assumes data comes from multiple GUI's
            function saveSettingsTempFileMultipleGUIs(settingsTempFileNames,settingsSimNumber,handles)
                disp('saveSettingsTempFileMultipleGUIs')
                %assumes handles is the pointer to the main handles (i.e handles if called from main gui or handles.main if called from sub gui)
                filenameSettingsSimNumber = char(settingsTempFileNames(settingsSimNumber))
                fidtemp = fopen(filenameSettingsSimNumber, 'w');
                fprintf(fidtemp, '%s\n', '%Dynamic Diversity Algorithm');
                simText = ['%Matlab settings for simulation ' num2str(settingsSimNumber)];
                fprintf(fidtemp, '%s\n', simText);
                %save all data
                %disp('saving temp file')
                GUIClass.saveSettings1SimMain(fidtemp, handles);
                GUIClass.saveSettings1SimStatic(fidtemp, handles)%added 7th Jan 2017
                GUIClass.saveSettings1SimNetwork(fidtemp, handles);
                GUIClass.saveSettings1SimAttack(fidtemp, handles);
                GUIClass.saveSettings1SimExploit(fidtemp, handles);
                GUIClass.saveSettings1SimAlg(fidtemp, handles);
                GUIClass.saveSettings1SimPerf(fidtemp, handles);
                GUIClass.saveSettings1SimSaveParameters(fidtemp, handles);
                %close file
                fclose(fidtemp);
            end
            %find currently selected simulation and make sure settings saved first in temp file
            function loadSettingsTempFile(settingsTempFileNames,settingsSimNumber,handles)
                disp('loadSettingsTempFile')
                %load settings for number 
                fid2 = fopen(char(settingsTempFileNames(settingsSimNumber)), 'r');
                %check to see if file exists
                if fid2 == -1
                    %file doesn't exist yet
                    errorText = ['settings file ' settingsSimNumber ' doesnt exist yet'];
                    disp(errorText)
                else
                    headertext1 = fgetl(fid2);
                    headertext2 = fgetl(fid2);
                    loadSettings1Sim(fid2,handles);
                    fclose(fid2);
                end
            end
            function loadSettingsTempFileMultipleGUIs(settingsTempFileNames,settingsSimNumber,handles)
                disp('loadSettingsTempFileMultipleGUIs')
                %load settings for number 
                fidtemp = fopen(char(settingsTempFileNames(settingsSimNumber)), 'r');
                %check to see if file exists
                if fidtemp == -1
                    %file doesn't exist yet
                    errorText = ['settings file ' settingsSimNumber ' doesnt exist yet'];
                    disp(errorText)
                else
                    headertext1 = fgetl(fidtemp);
                    headertext2 = fgetl(fidtemp);
                    GUIClass.loadSettings1SimMain(fidtemp, handles);
                    GUIClass.loadSettings1SimStatic(fidtemp, handles); %added 7th Jan 2017
                    GUIClass.loadSettings1SimNetwork(fidtemp, handles);
                    GUIClass.loadSettings1SimAttack(fidtemp, handles);
                    GUIClass.loadSettings1SimExploit(fidtemp, handles);
                    GUIClass.loadSettings1SimAlg(fidtemp, handles);
                    GUIClass.loadSettings1SimPerf(fidtemp, handles);
                    GUIClass.loadSettings1SimSaveParameters(fidtemp, handles);
                    fclose(fidtemp);
                end
            end            
            function deleteAllSettingsTempFiles()
                %Delete all settings temp files
                %get max possible number of temp files
                numFiles = GUIClass.getMaxNumSims();
                %generate filename list
                directorynameSettings = GUIClass.getDirectoryNameSettings();
                settingsTempFileNames = GUIClass.genSettingsTempFileNames(directorynameSettings,numFiles);
                %delete all temp files
                for settingsSimNumber = 1 : numFiles
                    GUIClass.deleteSettingsTempFile(settingsTempFileNames,settingsSimNumber);
                end
                
            end
            function deleteSettingsTempFile(settingsTempFileNames,settingsSimNumber)
                %Delete a single settings temp file
                fid1=fopen(char(settingsTempFileNames(settingsSimNumber)),'r');
                if fid1 >= 3
                    fclose(fid1);
                    delete(char(settingsTempFileNames(settingsSimNumber)));
                end
            end
            function saveSettings1SimMain(fid, handles)
                %get data from boxes in main window
                %Multiple simulations
                numRunsV =get(handles.editNumRuns,'Value');
                numTimeStepsV =get(handles.editNumTimeSteps,'Value');
                autoSaveOutV =get(handles.checkboxSaveOutput,'Value');
                autoSaveGrV =get(handles.checkboxSaveGraphs,'Value');
                binmodeV =get(handles.checkboxbinmode,'Value');
                fileSaveOptionV =get(handles.popupmenuFileSaveOption,'Value'); %added 27th April 2016
                fileSavePrecisionV =get(handles.popupmenuFileSavePrecision,'Value'); %added 27th April 2016
                blockLengthV =get(handles.editBlockLength,'Value');%added 27th April 2016
                %simulation includes
                datasetEnableV =get(handles.checkboxDatasetEnable,'Value');
                datasetStaticEnableV =get(handles.checkboxStaticDatasetEnable,'Value');%added 31st March 2016
                networkModelEnableV =get(handles.checkboxNetworkModelEnable,'Value');
                attackModelEnableV =get(handles.checkboxAttackModelEnable,'Value');
                exploitEnableV = get(handles.checkboxExploitEnable,'Value');
                algEnableV =get(handles.checkboxAlgEnable,'Value');
                networkPerfEnableV =get(handles.checkboxNetworkPerfEnable,'Value');
                %simulation selections
                inputDatasetV =get(handles.popupmenuInputDataset,'Value');
                alignDataV =get(handles.checkboxAlignData,'Value');
                inputStaticDatasetV =get(handles.popupmenuStaticDataset,'Value'); %added 31st March 2016
                networkModelV =get(handles.popupmenuNetworkModel,'Value');
                networkModelS = GUIClass.getNetworkModelString(networkModelV);
                attackModelV =get(handles.popupmenuAttackModel,'Value');
                attackModelS = GUIClass.getAttackModelString(attackModelV);
                exploitModelV =get(handles.popupmenuExploitType,'Value');
                exploitModelS = GUIClass.getExploitModelString(exploitModelV);
                algorithmV =get(handles.popupmenuAlgorithm,'Value');
                algorithmS = GUIClass.getAlgModelString(algorithmV);
                networkPerformanceV =get(handles.popupmenuNetworkPerformance,'Value');
                networkPerformanceS = GUIClass.getPerfModelString(networkPerformanceV);
                %graphical output
                graphTypeV =get(handles.popupmenuGraphType,'Value');
                graphTypeS = GUIClass.getGraphString(graphTypeV);
                %save to file
                %Multiple simulations
                fprintf(fid, '%s\n', '*******************');
                fprintf(fid, '%s\n', '***Main GUI data***');
                fprintf(fid, '%s\n', '*******************');
                fprintf(fid, '%s\n', '***simulation');
                fprintf(fid, '%s\n', 'editNumRuns');
                fprintf(fid, '%d\n', numRunsV);
                fprintf(fid, '%s\n', 'editNumTimeSteps');
                fprintf(fid, '%d\n', numTimeStepsV);
                fprintf(fid, '%s\n', 'checkboxSaveOutput');
                fprintf(fid, '%d\n', autoSaveOutV);
                fprintf(fid, '%s\n', 'checkboxSaveGraphs');
                fprintf(fid, '%d\n', autoSaveGrV);
                fprintf(fid, '%s\n', 'binmode');
                fprintf(fid, '%d\n', binmodeV);
                fprintf(fid, '%s\n', 'fileSaveOption');%added 27th April 2016
                fprintf(fid, '%d\n', fileSaveOptionV);%added 27th April 2016
                fprintf(fid, '%s\n', 'fileSavePrecision');%added 27th April 2016
                fprintf(fid, '%d\n', fileSavePrecisionV);%added 27th April 2016
                fprintf(fid, '%s\n', 'blockLength');%added 27th April 2016
                fprintf(fid, '%d\n', blockLengthV);%added 27th April 2016
                %simulation includes
                fprintf(fid, '%s\n', '***simulation includes');
                fprintf(fid, '%s\n', 'datasetEnable');
                fprintf(fid, '%d\n', datasetEnableV); 
                fprintf(fid, '%s\n', 'datasetStaticEnable'); %added 31st March 2016
                fprintf(fid, '%d\n', datasetStaticEnableV);  %added 31st March 2016
                fprintf(fid, '%s\n', 'networkModelEnable');
                fprintf(fid, '%d\n', networkModelEnableV); 
                fprintf(fid, '%s\n', 'attackModelEnable');
                fprintf(fid, '%d\n', attackModelEnableV); 
                fprintf(fid, '%s\n', 'exploitEnable');
                fprintf(fid, '%d\n', exploitEnableV);
                fprintf(fid, '%s\n', 'algEnable');
                fprintf(fid, '%d\n', algEnableV); 
                fprintf(fid, '%s\n', 'networkPerfEnable');
                fprintf(fid, '%d\n', networkPerfEnableV); 
                %simulation selections
                fprintf(fid, '%s\n', '***simulation selections');
                fprintf(fid, '%s\n', 'inputDataset');
                fprintf(fid, '%d\n', inputDatasetV);
                fprintf(fid, '%s\n', 'checkboxAlignData');
                fprintf(fid, '%d\n', alignDataV);
                fprintf(fid, '%s\n', 'inputStaticDataset');%added 31st March 2016
                fprintf(fid, '%d\n', inputStaticDatasetV);%added 31st March 2016
                fprintf(fid, '%s\n', 'networkModel');
                fprintf(fid, '%d\n', networkModelV); 
                fprintf(fid, '%s\n', networkModelS);
                fprintf(fid, '%s\n', 'attackModel');
                fprintf(fid, '%d\n', attackModelV); 
                fprintf(fid, '%s\n', attackModelS);
                fprintf(fid, '%s\n', 'exploitModel');
                fprintf(fid, '%d\n', exploitModelV); 
                fprintf(fid, '%s\n', exploitModelS);                
                fprintf(fid, '%s\n', 'algorithm');
                fprintf(fid, '%d\n', algorithmV);
                fprintf(fid, '%s\n', algorithmS);
                fprintf(fid, '%s\n', 'networkPerformance');
                fprintf(fid, '%d\n', networkPerformanceV); 
                fprintf(fid, '%s\n', networkPerformanceS); 
                fprintf(fid, '%s\n', '***graphical output');
                fprintf(fid, '%s\n', 'graphType');
                fprintf(fid, '%d\n', graphTypeV);
                fprintf(fid, '%s\n', graphTypeS);
            end
            function saveSettings1SimStatic(fid, mainhandles)%save data%added 7th Jan 2017
                inputStaticDataSetV = get(mainhandles.popupmenuStaticDataset, 'Value');
                %save static diversity data settings
                %save to file
                fprintf(fid, '%s\n', '******************************************');
                fprintf(fid, '%s\n', '***Static Diversity - Random Assignment***');
                fprintf(fid, '%s\n', '******************************************');
                fprintf(fid, '%d\n', inputStaticDataSetV);
                switch(inputStaticDataSetV)
                    case 11
                    handles = guidata(mainhandles.randomAssignmentParametersFig); %load data
                    editNumNodesV =get(handles.editNumNodes,'Value');
                    editVarArrayV =get(handles.editVarArray,'Value');
                    fprintf(fid, '%s\n', 'numNodes');
                    fprintf(fid, '%d\n', editNumNodesV);
                    fprintf(fid, '%s\n', 'VarArrayV');
                    fprintf(fid, '%d\n', editVarArrayV);
                    guidata(mainhandles.randomAssignmentParametersFig, handles); 
                end
            end
            function saveSettings1SimNetwork(fid, mainhandles)
                networkModelV = get(mainhandles.popupmenuNetworkModel, 'Value');
                networkModelS = GUIClass.getNetworkModelString(networkModelV);
                %save network model settings
                %save to file
                fprintf(fid, '%s\n', '*******************');
                fprintf(fid, '%s\n', '***Network Model***');
                fprintf(fid, '%s\n', '*******************');
                fprintf(fid, '%s\n', networkModelS);
                switch(networkModelS)
                    case 'Random Encounter'
                    handles = guidata(mainhandles.networkModelParameters_randomEncounterFig); %load data
                    editRandomEncounterProbV =get(handles.editRandomEncounterProb,'Value');
                    fprintf(fid, '%s\n', 'RandomEncounterProb');
                    fprintf(fid, '%d\n', editRandomEncounterProbV);
                    guidata(mainhandles.networkModelParameters_randomEncounterFig, handles); %save data
                    case 'Wireless Bluetooth'
                    handles = guidata(mainhandles.networkModelParameters_wirelessBluetoothFig); %load data
                    %wireless bluetooth
                    editDensityV =get(handles.editDensity,'Value');
                    editAvVelocityV =get(handles.editAvVelocity,'Value');
                    popupmenuTxRadiusV =get(handles.popupmenuTxRadius,'Value');
                    editTVirusV =get(handles.editTVirus,'Value');
                    %save to file
                    %fprintf(fid, '%s\n', '***Wireless Bluetooth');
                    fprintf(fid, '%s\n', 'Density');
                    fprintf(fid, '%d\n', editDensityV); 
                    fprintf(fid, '%s\n', 'AvVelocity');
                    fprintf(fid, '%d\n', editAvVelocityV); 
                    fprintf(fid, '%s\n', 'TxRadius');
                    fprintf(fid, '%d\n', popupmenuTxRadiusV); 
                    fprintf(fid, '%s\n', 'TVirus');
                    fprintf(fid, '%d\n', editTVirusV);
                    guidata(mainhandles.networkModelParameters_wirelessBluetoothFig, handles); %save data
                    case 'Random Waypoint'
                    handles = guidata(mainhandles.networkModelParameters_randomWaypointFig); %load data
                    checkboxPlotWayPointsV = get(handles.checkboxPlotWayPoints,'Value');
                    popupmenuRWPTxRadiusV = get(handles.popupmenuRWPTxRadius,'Value');
                    popupmenuRWPTxRadiusS = num2str(GUIClass.getTxRadius(popupmenuRWPTxRadiusV));
                    popupmenuRWPEncounterChoiceV = get(handles.popupmenuRWPEncounterChoice,'Value');
                    popupmenuRWPEncounterChoiceS = GUIClass.getRWPmodeStr(popupmenuRWPEncounterChoiceV);
                    editRWPPosXminV = get(handles.editRWPPosXmin,'Value');
                    editRWPPosXmaxV = get(handles.editRWPPosXmax,'Value');
                    editRWPPosYminV = get(handles.editRWPPosYmin,'Value');
                    editRWPPosYmaxV = get(handles.editRWPPosYmax,'Value');
                    editRWPSpeedminV = get(handles.editRWPSpeedmin,'Value');
                    editRWPSpeedmaxV = get(handles.editRWPSpeedmax,'Value');
                    editRWPPauseminV = get(handles.editRWPPausemin,'Value');
                    editRWPPausemaxV = get(handles.editRWPPausemax,'Value');
                    editRWPWalkTimeminV = get(handles.editRWPWalkTimemin,'Value');
                    editRWPWalkTimemaxV = get(handles.editRWPWalkTimemax,'Value');
                    editRWPDirectionminV = get(handles.editRWPDirectionmin,'Value');
                    editRWPDirectionmaxV = get(handles.editRWPDirectionmax,'Value');
                    %save to file
                    fprintf(fid, '%s\n', 'PlotWayPoints');
                    fprintf(fid, '%d\n', checkboxPlotWayPointsV); 
                    fprintf(fid, '%s\n', 'RWPTxRadius');
                    fprintf(fid, '%d\n', popupmenuRWPTxRadiusV); 
                    fprintf(fid, '%s\n', popupmenuRWPTxRadiusS); 
                    fprintf(fid, '%s\n', 'RWPEncounterChoice');
                    fprintf(fid, '%d\n', popupmenuRWPEncounterChoiceV); 
                    fprintf(fid, '%s\n', popupmenuRWPEncounterChoiceS);
                    fprintf(fid, '%s\n', 'RWPPosXmin');
                    fprintf(fid, '%d\n', editRWPPosXminV);
                    fprintf(fid, '%s\n', 'RWPPosXmax');
                    fprintf(fid, '%d\n', editRWPPosXmaxV);                
                    fprintf(fid, '%s\n', 'RWPPosYmin');
                    fprintf(fid, '%d\n', editRWPPosYminV);
                    fprintf(fid, '%s\n', 'RWPPosYmax');
                    fprintf(fid, '%d\n', editRWPPosYmaxV);
                    fprintf(fid, '%s\n', 'RWPSpeedmin');
                    fprintf(fid, '%d\n', editRWPSpeedminV);
                    fprintf(fid, '%s\n', 'RWPSpeedmax');
                    fprintf(fid, '%d\n', editRWPSpeedmaxV);
                    fprintf(fid, '%s\n', 'RWPPausemin');
                    fprintf(fid, '%d\n', editRWPPauseminV);
                    fprintf(fid, '%s\n', 'RWPPausemax');
                    fprintf(fid, '%d\n', editRWPPausemaxV);
                    fprintf(fid, '%s\n', 'RWPWalkTimemin');
                    fprintf(fid, '%d\n', editRWPWalkTimeminV);
                    fprintf(fid, '%s\n', 'RWPWalkTimemax');
                    fprintf(fid, '%d\n', editRWPWalkTimemaxV);
                    fprintf(fid, '%s\n', 'RWPDirectionmin');
                    fprintf(fid, '%d\n', editRWPDirectionminV);
                    fprintf(fid, '%s\n', 'RWPDirectionmax');
                    fprintf(fid, '%d\n', editRWPDirectionmaxV);
                    guidata(mainhandles.networkModelParameters_randomWaypointFig, handles); %save data
                    otherwise
                    fprintf(fid, '%s\n', '***Error: Invalid network model!!!!');
                end
            end
            function saveSettings1SimAttack(fid, mainhandles)
                %save network model settings
                attackModelV = get(mainhandles.popupmenuAttackModel, 'Value');
                attackModelS = GUIClass.getAttackModelString(attackModelV);
                %save to file
                fprintf(fid, '%s\n', '*******************');
                fprintf(fid, '%s\n', '***Attack Model***');
                fprintf(fid, '%s\n', '*******************');
                fprintf(fid, '%s\n', attackModelS);
                switch(attackModelS)
                    case 'SI'
                    handles = guidata(mainhandles.attackModelParameters_SIFig); %load data  
                    fprintf(fid, '%s\n', '***No parameters to set'); 
                    guidata(mainhandles.attackModelParameters_SIFig, handles); %save data
                    case 'SIR'
                    handles = guidata(mainhandles.attackModelParameters_SIRFig); %load data 
                    editRecovProbV =get(handles.editRecovProb,'Value');
                    fprintf(fid, '%s\n', 'RecovProb');
                    fprintf(fid, '%d\n', editRecovProbV);
                    guidata(mainhandles.attackModelParameters_SIRFig, handles); %save data
                    case 'SEIR'
                    handles = guidata(mainhandles.attackModelParameters_SEIRFig); %load data
                   % popupmenuVirusTypeV =get(handles.popupmenuVirusType,'Value');
                   % editUserProbV =get(handles.editPei1,'Value');
                   % editLatTimeV =get(handles.editPei2,'Value');
                   % popupmenuLatUnitsV =get(handles.popupmenuLatUnits,'Value');
                    editRecoveryProbV =get(handles.editRecoveryProb,'Value');
                    %fprintf(fid, '%s\n', 'VirusType');
                    %fprintf(fid, '%d\n', popupmenuVirusTypeV);
                    %fprintf(fid, '%s\n', 'UserProb');
                    %fprintf(fid, '%d\n', editUserProbV);
                    %fprintf(fid, '%s\n', 'LatTime');
                    %fprintf(fid, '%d\n', editLatTimeV);
                    %fprintf(fid, '%s\n', 'LatUnits');
                    %fprintf(fid, '%d\n', popupmenuLatUnitsV);
                    fprintf(fid, '%s\n', 'RecovProb');
                    fprintf(fid, '%d\n', editRecoveryProbV);
                    guidata(mainhandles.attackModelParameters_SEIRFig, handles); %save data
                    case 'SEPTICOX'
                    handles = guidata(mainhandles.attackModelParameters_SEPTICOXFig); %load data
                   % popupmenuVirusTypeV =get(handles.popupmenuVirusType,'Value');
                   % editPei1V =get(handles.editPei1,'Value');
                   % editPei2V =get(handles.editPei2,'Value');
                   % popupmenuLUnitsV =get(handles.popupmenuLUnits,'Value');
                    editDevicesOfflineV =get(handles.editDevicesOffline,'Value')
                    editPos1V =get(handles.editPos1,'Value');
                    editPos2V =get(handles.editPos2,'Value');
                    editPso1V =get(handles.editPso1,'Value');
                    editPso2V =get(handles.editPso2,'Value');
                    editPexV =get(handles.editPex,'Value');
                    editPxeV =get(handles.editPxe,'Value');
                    editPto1V =get(handles.editPto1,'Value');
                    editPto2V =get(handles.editPto2,'Value');
                    editPspV =get(handles.editPsp,'Value');
                    editPetV =get(handles.editPet,'Value');
                    editPitV =get(handles.editPit,'Value');
                    editPtsV =get(handles.editPts,'Value');
                    editPtpV =get(handles.editPtp,'Value');
                    editPicV =get(handles.editPic,'Value');
                    editPciV =get(handles.editPci,'Value');
                    editFreqOnV =get(handles.editFreqOn,'Value');
                    editPeakOnV =get(handles.editPeakOn,'Value');
                    editSpreadOnV =get(handles.editSpreadOn,'Value');
                    editAmpOnV =get(handles.editAmpOn,'Value');
                    editFreqOffV =get(handles.editFreqOff,'Value');
                    editPeakOffV =get(handles.editPeakOff,'Value');
                    editSpreadOffV =get(handles.editSpreadOff,'Value');
                    editAmpOffV =get(handles.editAmpOff,'Value');
                    checkboxUsePeriodicV =get(handles.checkboxUsePeriodic,'Value');
                    checkboxPlotPeriodV =get(handles.checkboxPlotPeriod,'Value');
                    %fprintf(fid, '%s\n', 'VirusType');
                    %fprintf(fid, '%d\n', popupmenuVirusTypeV);
                    %fprintf(fid, '%s\n', 'Pei1');
                    %fprintf(fid, '%d\n', editPei1V);
                    %fprintf(fid, '%s\n', 'Pei2');
                    %fprintf(fid, '%d\n', editPei2V);
                    %fprintf(fid, '%s\n', 'LUnits');
                    %fprintf(fid, '%d\n', popupmenuLUnitsV);
                    fprintf(fid, '%s\n', 'DevicesOffline');
                    fprintf(fid, '%d\n', editDevicesOfflineV);
                    fprintf(fid, '%s\n', 'Pos1');
                    fprintf(fid, '%d\n', editPos1V);
                    fprintf(fid, '%s\n', 'Pos2');
                    fprintf(fid, '%d\n', editPos2V);
                    fprintf(fid, '%s\n', 'Pso1');
                    fprintf(fid, '%d\n', editPso1V);
                    fprintf(fid, '%s\n', 'Pso2');
                    fprintf(fid, '%d\n', editPso2V);
                    fprintf(fid, '%s\n', 'Pex');
                    fprintf(fid, '%d\n', editPexV);
                    fprintf(fid, '%s\n', 'Pxe');
                    fprintf(fid, '%d\n', editPxeV);
                    fprintf(fid, '%s\n', 'Pto1');
                    fprintf(fid, '%d\n', editPto1V);
                    fprintf(fid, '%s\n', 'Pto2');
                    fprintf(fid, '%d\n', editPto2V);
                    fprintf(fid, '%s\n', 'Psp');
                    fprintf(fid, '%d\n', editPspV);
                    fprintf(fid, '%s\n', 'Pet');
                    fprintf(fid, '%d\n', editPetV);
                    fprintf(fid, '%s\n', 'Pit');
                    fprintf(fid, '%d\n', editPitV);
                    fprintf(fid, '%s\n', 'Pts');
                    fprintf(fid, '%d\n', editPtsV);
                    fprintf(fid, '%s\n', 'Ptp');
                    fprintf(fid, '%d\n', editPtpV);
                    fprintf(fid, '%s\n', 'Pic');
                    fprintf(fid, '%d\n', editPicV);
                    fprintf(fid, '%s\n', 'Pci');
                    fprintf(fid, '%d\n', editPciV);
                    fprintf(fid, '%s\n', 'FreqOn');
                    fprintf(fid, '%d\n', editFreqOnV);
                    fprintf(fid, '%s\n', 'PeakOn');
                    fprintf(fid, '%d\n', editPeakOnV);
                    fprintf(fid, '%s\n', 'SpreadOn');
                    fprintf(fid, '%d\n', editSpreadOnV);
                    fprintf(fid, '%s\n', 'AmpOn');
                    fprintf(fid, '%d\n', editAmpOnV);
                    fprintf(fid, '%s\n', 'FreqOff');
                    fprintf(fid, '%d\n', editFreqOffV);
                    fprintf(fid, '%s\n', 'PeakOff');
                    fprintf(fid, '%d\n', editPeakOffV);
                    fprintf(fid, '%s\n', 'SpreadOff');
                    fprintf(fid, '%d\n', editSpreadOffV);
                    fprintf(fid, '%s\n', 'AmpOff');
                    fprintf(fid, '%d\n', editAmpOffV);
                    fprintf(fid, '%s\n', 'UsePeriodic');
                    fprintf(fid, '%d\n', checkboxUsePeriodicV);
                    fprintf(fid, '%s\n', 'PlotPeriod');
                    fprintf(fid, '%d\n', checkboxPlotPeriodV);
                    guidata(mainhandles.attackModelParameters_SEPTICOXFig, handles); %save data
                    otherwise
                    fprintf(fid, '%s\n', '***Error: Invalid attack model!!!!');   
                end 
            end
            function saveSettings1SimExploit(fid, mainhandles)
                %save to file
                fprintf(fid, '%s\n', '*******************');
                fprintf(fid, '%s\n', '***Edit Exploit***');
                fprintf(fid, '%s\n', '*******************');
                 handles = guidata(mainhandles.editExploitFig); %load dat
                 editExploitAppV=get(handles.editExploitApp,'String');
                 editExploitServV=get(handles.editExploitServ,'String');
                 editExploitOSV=get(handles.editExploitOS,'String');
                 editExploitDrV=get(handles.editExploitDr,'String');
                 popupmenuMalwareTypeV =get(handles.popupmenuMalwareType,'Value'); %added 7th Jan 2017
                 popupmenuVirusTypeV =get(handles.popupmenuVirusType,'Value');
                 editPei1V =get(handles.editPei1,'Value'); %prob of user download click
                 editPei2V =get(handles.editPei2,'Value'); %virus latency time
                 popupmenuLatUnitsV =get(handles.popupmenuLatUnits,'Value');
                 editInfectionTimeV = get(handles.editInfectionTime,'Value');
                 popupmenuInfectTimeUnitsV =get(handles.popupmenuInfectTimeUnits,'Value');
                 editTimeToTxVirusV = get(handles.editTimeToTxVirus,'Value');
                 popupmenuTransmitTimeUnitsV =get(handles.popupmenuTransmitTimeUnits,'Value');
                 editNumInitialInfectedV =get(handles.editNumInitialInfected,'Value');
                 editTimeToTxGenotypeV = get(handles.editTimeToTxGenotype, 'Value');
                 popupmenuTimeToTxGenotypeUnitsV = get(handles.popupmenuTimeToTxGenotypeUnits, 'Value');
                 editTimeOutPeriodTxGenotypeV = get(handles.editTimeOutPeriodTxGenotype, 'Value');
                 popupmenuTimeOutPeriodTxGenotypeUnitsV = get(handles.popupmenuTimeOutPeriodTxGenotypeUnits, 'Value');
                 
                 fprintf(fid, '%s\n', 'editExploitAppV');
                 fprintf(fid, '%s\n', editExploitAppV);
                 fprintf(fid, '%s\n', 'editExploitServV');
                 fprintf(fid, '%s\n', editExploitServV);
                 fprintf(fid, '%s\n', 'editExploitOSV');
                 fprintf(fid, '%s\n', editExploitOSV);
                 fprintf(fid, '%s\n', 'editExploitDrV');
                 fprintf(fid, '%s\n', editExploitDrV);
                 fprintf(fid, '%s\n', 'MalwareType(1 AND, 2 OR)'); %added 7th Jan 2017
                 fprintf(fid, '%d\n', popupmenuMalwareTypeV); %added 7th Jan 2017
                 fprintf(fid, '%s\n', 'VirusType');
                 fprintf(fid, '%d\n', popupmenuVirusTypeV);
                 fprintf(fid, '%s\n', 'Pei1');
                 fprintf(fid, '%d\n', editPei1V);
                 fprintf(fid, '%s\n', 'Pei2');
                 fprintf(fid, '%d\n', editPei2V);
                 fprintf(fid, '%s\n', 'LatUnits');
                 fprintf(fid, '%d\n', popupmenuLatUnitsV);
                 fprintf(fid, '%s\n', 'InfectionTime');
                 fprintf(fid, '%d\n', editInfectionTimeV);
                 fprintf(fid, '%s\n', 'editNumInitialInfectedV');
                 fprintf(fid, '%d\n', editNumInitialInfectedV);
                 fprintf(fid, '%s\n', 'InfectTimeUnits');
                 fprintf(fid, '%d\n', popupmenuInfectTimeUnitsV);
                 fprintf(fid, '%s\n', 'TimeToTxVirus');
                 fprintf(fid, '%d\n', editTimeToTxVirusV);
                 fprintf(fid, '%s\n', 'TransmitTimeUnits');
                 fprintf(fid, '%d\n', popupmenuTransmitTimeUnitsV);
                 fprintf(fid, '%s\n', 'TimeToTxGenotypeV');
                 fprintf(fid, '%d\n', editTimeToTxGenotypeV);
                 fprintf(fid, '%s\n', 'TimeToTxGenotypeUnitsV');
                 fprintf(fid, '%d\n', popupmenuTimeToTxGenotypeUnitsV);
                 fprintf(fid, '%s\n', 'TimeOutPeriodTxGenotypeV');
                 fprintf(fid, '%d\n', editTimeOutPeriodTxGenotypeV);
                 fprintf(fid, '%s\n', 'TimeOutPeriodTxGenotypeUnitsV');
                 fprintf(fid, '%d\n', popupmenuTimeOutPeriodTxGenotypeUnitsV);
                 guidata(mainhandles.editExploitFig, handles); %save data
            end
            function saveSettings1SimAlg(fid, mainhandles)
                diversityAlgV = get(mainhandles.popupmenuAlgorithm, 'Value');
                diversityAlgS = GUIClass.getAlgModelString(diversityAlgV);
                %save diversity model settings
                %save to file
                fprintf(fid, '%s\n', '*************************');
                fprintf(fid, '%s\n', '***Diversity Algorithm***');
                fprintf(fid, '%s\n', '*************************');
                fprintf(fid, '%s\n', diversityAlgS);
                switch(diversityAlgS)
                    case 'Favourability Score'
                    handles = guidata(mainhandles.diversityModelParameters_FavScoreFig); %load data
                    %select algorithms
                    radiobuttonSelectAlg1To5 =get(handles.radiobuttonSelectAlg1To5,'Value');
                    radiobuttonSelectAlg4And5 =get(handles.radiobuttonSelectAlg4And5,'Value');
                    radiobuttonSelectAlg4Only =get(handles.radiobuttonSelectAlg4Only,'Value');
                    %type of encounters
                    encounterTypeV=get(handles.popupmenuEncounterType,'Value');
                    encounterTypeS = GUIClass.getEncounterString(encounterTypeV);
                    minEncountersV =get(handles.editMinEncounters,'Value');
                    maxEncountersV =get(handles.editMaxEncounters,'Value');
                    encounterProbV = get(handles.editEncounterProb,'Value');
                    %weighting factors
                    diversityWeightV =get(handles.editDiversityWeight,'Value');
                    commWeightV =get(handles.editCommWeight,'Value');
                    indWeightV =get(handles.editIndWeight,'Value');
                    effWeightV =get(handles.editEffWeight,'Value');                  
                    %dynamic weighting factors
                    dynDivV =get(handles.checkboxDynDiv,'Value');
                    dynComV =get(handles.checkboxDynCom,'Value');
                    dynIndV =get(handles.checkboxDynInd,'Value');
                    dynQualV =get(handles.checkboxDynQual,'Value');
                    weightFileDiv =get(handles.editWeightFileDiv,'String');
                    weightFileCom =get(handles.editWeightFileCom,'String');
                    weightFileInd =get(handles.editWeightFileInd,'String');
                    weightFileQual =get(handles.editWeightFileQual,'String');
                    %stopping the genotype update process
                    updateSampleSizeV =get(handles.editUpdateSampleSize,'Value');
                    alg4UpdateTypeV =get(handles.popupmenuAlg4UpdateType,'Value');
                    alg4MinstdV =get(handles.editAlg4Minstd,'Value');
                    alg4MaxStdV =get(handles.editAlg4MaxStd,'Value');
                    alg5UpdateTypeV =get(handles.popupmenuAlg5UpdateType,'Value');
                    alg5MinStdV =get(handles.editAlg5MinStd,'Value');
                    alg5MaxStdV =get(handles.editAlg5MaxStd,'Value');
                    editAlg4MinstdE =get(handles.editAlg4Minstd, 'Enable');
                    editAlg4MaxStdE =get(handles.editAlg4MaxStd, 'Enable');
                    editAlg5MinStdE =get(handles.editAlg5MinStd, 'Enable');
                    editAlg5MaxStdE =get(handles.editAlg5MaxStd, 'Enable');
                    %Dynamic desirability changes
                    changeCommDesV =get(handles.checkboxChangeCommDes,'Value');
                    commTime1V =get(handles.editCommTime1,'Value');
                    checkboxCommTime1V =get(handles.checkboxCommTime1,'Value');
                    commTime2V =get(handles.editCommTime2,'Value');
                    checkboxCommTime2V =get(handles.checkboxCommTime2,'Value');
                    changeIndDesV =get(handles.checkboxChangeIndDes,'Value');
                    indTime1V =get(handles.editIndTime1,'Value');
                    checkboxIndTime1V =get(handles.checkboxIndTime1,'Value');
                    indTime2V =get(handles.editIndTime2,'Value');
                    checkboxIndTime2V =get(handles.checkboxIndTime2,'Value');
                    changeEffDesV =get(handles.checkboxChangeEff,'Value');
                    effTime1V =get(handles.editEffTime1,'Value');
                    checkboxEffTime1V =get(handles.checkboxEffTime1,'Value');
                    effTime2V =get(handles.editEffTime2,'Value');
                    checkboxEffTime2V =get(handles.checkboxEffTime2,'Value');
                    %Dynamic desirability changes from file
                    checkComDesV =get(handles.checkComDes,'Value');
                    checkIndDesV =get(handles.checkIndDes,'Value');
                    checkQualDesV =get(handles.checkQualDes,'Value');
                    editDesComFilenameS =get(handles.editDesComFilename,'String');
                    editDesIndFilenameS =get(handles.editDesIndFilename,'String');
                    editDesQualFilenameS =get(handles.editDesQualFilename,'String');
                    %save data to file
                    fprintf(fid, '%s\n', '***select algorithms');
                    fprintf(fid, '%s\n', 'radiobuttonSelectAlg1To5');
                    fprintf(fid, '%d\n', radiobuttonSelectAlg1To5);
                    fprintf(fid, '%s\n', 'radiobuttonSelectAlg4And5');
                    fprintf(fid, '%d\n', radiobuttonSelectAlg4And5);
                    fprintf(fid, '%s\n', 'radiobuttonSelectAlg4Only');
                    fprintf(fid, '%d\n', radiobuttonSelectAlg4Only);
                    fprintf(fid, '%s\n', '***type of encounters');
                    fprintf(fid, '%s\n', 'encounterType');
                    fprintf(fid, '%d\n', encounterTypeV);
                    fprintf(fid, '%s\n', encounterTypeS);
                    fprintf(fid, '%s\n', 'minEncounters');
                    fprintf(fid, '%d\n', minEncountersV);
                    fprintf(fid, '%s\n', 'maxEncounters');
                    fprintf(fid, '%d\n', maxEncountersV);
                    fprintf(fid, '%s\n', 'encounterProb');
                    fprintf(fid, '%d\n', encounterProbV);
                    fprintf(fid, '%s\n', '***weighting factors');
                    fprintf(fid, '%s\n', 'diversityWeight');
                    fprintf(fid, '%d\n', diversityWeightV);
                    fprintf(fid, '%s\n', 'commWeight');
                    fprintf(fid, '%d\n', commWeightV);
                    fprintf(fid, '%s\n', 'indWeight');
                    fprintf(fid, '%d\n', indWeightV);
                    fprintf(fid, '%s\n', 'effWeight');
                    fprintf(fid, '%d\n', effWeightV);
                    fprintf(fid, '%s\n', '***dynamic weighting factors');
                    fprintf(fid, '%s\n', 'checkboxDynDiv');
                    fprintf(fid, '%d\n', dynDivV);
                    fprintf(fid, '%s\n', 'checkboxDynCom');
                    fprintf(fid, '%d\n', dynComV);
                    fprintf(fid, '%s\n', 'checkboxDynInd');
                    fprintf(fid, '%d\n', dynIndV);
                    fprintf(fid, '%s\n', 'checkboxDynQual');
                    fprintf(fid, '%d\n', dynQualV);
                    fprintf(fid, '%s\n', 'editWeightFileDiv');
                    fprintf(fid, '%s\n', weightFileDiv);
                    fprintf(fid, '%s\n', 'editWeightFileCom');
                    fprintf(fid, '%s\n', weightFileCom);
                    fprintf(fid, '%s\n', 'editWeightFileInd');
                    fprintf(fid, '%s\n', weightFileInd);
                    fprintf(fid, '%s\n', 'editWeightFileQual');
                    fprintf(fid, '%s\n', weightFileQual);             
                    fprintf(fid, '%s\n', '***stopping the genotype update process');
                    fprintf(fid, '%s\n', 'updateSampleSize');
                    fprintf(fid, '%d\n', updateSampleSizeV);
                    fprintf(fid, '%s\n', 'alg4UpdateType');
                    fprintf(fid, '%d\n', alg4UpdateTypeV);
                    fprintf(fid, '%s\n', 'alg4Minstd');
                    fprintf(fid, '%d\n', alg4MinstdV);
                    fprintf(fid, '%s\n', 'alg4MaxStd');
                    fprintf(fid, '%d\n', alg4MaxStdV);
                    fprintf(fid, '%s\n', 'alg5UpdateType');
                    fprintf(fid, '%d\n', alg5UpdateTypeV);
                    fprintf(fid, '%s\n', 'alg5MinStd');
                    fprintf(fid, '%d\n', alg5MinStdV);
                    fprintf(fid, '%s\n', 'alg5MaxStd');
                    fprintf(fid, '%d\n', alg5MaxStdV);
                    fprintf(fid, '%s\n', 'alg4MinstdE');
                    fprintf(fid, '%s\n', editAlg4MinstdE);
                    fprintf(fid, '%s\n', 'alg4MaxStdE');
                    fprintf(fid, '%s\n', editAlg4MaxStdE);
                    fprintf(fid, '%s\n', 'alg5MinStdE');
                    fprintf(fid, '%s\n', editAlg5MinStdE);
                    fprintf(fid, '%s\n', 'alg5MaxStdE');
                    fprintf(fid, '%s\n', editAlg5MaxStdE);
                    fprintf(fid, '%s\n', '***dynamic desirability changes');
                    fprintf(fid, '%s\n', 'changeCommDes');
                    fprintf(fid, '%d\n', changeCommDesV);
                    fprintf(fid, '%s\n', 'commTime1');
                    fprintf(fid, '%d\n', commTime1V);
                    fprintf(fid, '%s\n', 'checkboxCommTime1');
                    fprintf(fid, '%d\n', checkboxCommTime1V);
                    fprintf(fid, '%s\n', 'commTime2');
                    fprintf(fid, '%d\n', commTime2V);
                    fprintf(fid, '%s\n', 'checkboxCommTime2');
                    fprintf(fid, '%d\n', checkboxCommTime2V);
                    fprintf(fid, '%s\n', 'changeIndDes');
                    fprintf(fid, '%d\n', changeIndDesV);
                    fprintf(fid, '%s\n', 'indTime1');
                    fprintf(fid, '%d\n', indTime1V);
                    fprintf(fid, '%s\n', 'checkboxIndTime1');
                    fprintf(fid, '%d\n', checkboxIndTime1V);
                    fprintf(fid, '%s\n', 'indTime2');
                    fprintf(fid, '%d\n', indTime2V);
                    fprintf(fid, '%s\n', 'checkboxIndTime2');
                    fprintf(fid, '%d\n', checkboxIndTime2V);
                    fprintf(fid, '%s\n', 'changeEffDes');
                    fprintf(fid, '%d\n', changeEffDesV);
                    fprintf(fid, '%s\n', 'effTime1');
                    fprintf(fid, '%d\n', effTime1V);
                    fprintf(fid, '%s\n', 'checkboxEffTime1');
                    fprintf(fid, '%d\n', checkboxEffTime1V);
                    fprintf(fid, '%s\n', 'effTime2');
                    fprintf(fid, '%d\n', effTime2V);
                    fprintf(fid, '%s\n', 'checkboxEffTime2');
                    fprintf(fid, '%d\n', checkboxEffTime2V);
                    fprintf(fid, '%s\n', '***dynamic desirability files');
                    fprintf(fid, '%s\n', 'checkComDes');
                    fprintf(fid, '%d\n', checkComDesV);
                    fprintf(fid, '%s\n', 'checkIndDes');
                    fprintf(fid, '%d\n', checkIndDesV);
                    fprintf(fid, '%s\n', 'checkQualDes');
                    fprintf(fid, '%d\n', checkQualDesV);
                    fprintf(fid, '%s\n', 'editDesComFilename');
                    fprintf(fid, '%s\n', editDesComFilenameS);
                    fprintf(fid, '%s\n', 'editDesIndFilename');
                    fprintf(fid, '%s\n', editDesIndFilenameS);
                    fprintf(fid, '%s\n', 'editDesQualFilename');
                    fprintf(fid, '%s\n', editDesQualFilenameS);
                    guidata(mainhandles.diversityModelParameters_FavScoreFig, handles); %save data
                    case 'Static Diversity' %added 31st March 2016
                        fprintf(fid, '%s\n', '***No parameters to save'); 
                    otherwise
                    fprintf(fid, '%s\n', '***Error: Invalid diversity algorithm!!!!');   
                end
            end
            function saveSettings1SimPerf(fid, mainhandles)
                performanceV = get(mainhandles.popupmenuNetworkPerformance, 'Value');
                performanceS = GUIClass.getPerfModelString(performanceV);
                %save to file
                fprintf(fid, '%s\n', '***************************');
                fprintf(fid, '%s\n', '***Diversity Performance***');
                fprintf(fid, '%s\n', '***************************');
                switch(performanceS)
                    case 'Nei Only'
                    fprintf(fid, '%s\n', '***No parameters to set yet');   
                    otherwise
                    fprintf(fid, '%s\n', '***Error: Invalid performance algorithm!!!!'); 
                end
            end
            function saveSettings1SimSaveParameters(fid,handles)
                %save to file
                fprintf(fid, '%s\n', '**************************************');
                fprintf(fid, '%s\n', '***Parameters to save write below  ***');
                 fprintf(fid, '%s\n','***Format [1 1 1 .... 1]  22 par.  ***');
                fprintf(fid, '%s\n', '**************************************');
                editSaveParameters =get(handles.editSaveParameters,'String');
                fprintf(fid, '%s\n', editSaveParameters);  
            end
            %added 20th April 2016
            function saveOutputsTempFile_simHeader(outputTempDir,simNumber,setOutputs,trueNumRuns,trueNumTimesteps)
                fidtempName = [outputTempDir 'outputsTemp_sim' num2str(simNumber) 'Header.bin'];
                fidtemp = fopen(fidtempName, 'w');
                %save all data
                GUIClass.saveOutputs1SimBinary_Header(fidtemp, setOutputs,trueNumRuns,trueNumTimesteps);
                %close file
                fclose(fidtemp);
            end
            function saveOutputs1SimBinary_Header(fid, outputs,trueNumRuns,trueNumTimesteps)               
                %write once
                fixLength = outputs.numTimeSteps; %block length of data
                numRuns = trueNumRuns; %true number of runs
                numTimeSteps = trueNumTimesteps; %true number of timesteps
                fwrite(fid,int32(fixLength),'int32');
                fwrite(fid,int16(outputs.numNodes),'int16');
                fwrite(fid,int8(numRuns),'int16');
                fwrite(fid,numTimeSteps,'double');
                fwrite(fid,int8(outputs.len_apps),'int8');
                fwrite(fid,int8(outputs.len_serv),'int8');
                fwrite(fid,int8(outputs.len_os),'int8');
                fwrite(fid,int8(outputs.len_dr),'int8');
                fwrite(fid,int8(outputs.CD_apps),'int8');
                fwrite(fid,int8(outputs.CD_serv),'int8');
                fwrite(fid,int8(outputs.CD_os),'int8');
                fwrite(fid,int8(outputs.CD_dr),'int8');
                fwrite(fid,int8(outputs.ChangeDesirabilitiesFlag),'int8');
                fwrite(fid,outputs.tChange1,'double');
                fwrite(fid,outputs.tChange2,'double');
                fwrite(fid,int8(outputs.CD_apps2),'int8');
                fwrite(fid,int8(outputs.CD_serv2),'int8');
                fwrite(fid,int8(outputs.CD_os2),'int8');
                fwrite(fid,int8(outputs.CD_dr2),'int8');
                fwrite(fid,int8(outputs.E_apps),'int8');
                fwrite(fid,int8(outputs.E_serv),'int8');
                fwrite(fid,int8(outputs.E_os),'int8');
                fwrite(fid,int8(outputs.E_dr),'int8');
                fwrite(fid,int8(outputs.ChangeEffFlag),'int8');
                fwrite(fid,outputs.tChangeEff1,'double');
                fwrite(fid,outputs.tChangeEff2,'double');
                fwrite(fid,int8(outputs.E_apps2),'int8');
                fwrite(fid,int8(outputs.E_serv2),'int8');
                fwrite(fid,int8(outputs.E_os2),'int8');
                fwrite(fid,int8(outputs.E_dr2),'int8');
                fwrite(fid,int8(outputs.Genotype_apps),'int8');
                fwrite(fid,int8(outputs.Genotype_serv),'int8');
                fwrite(fid,int8(outputs.Genotype_os),'int8');
                fwrite(fid,int8(outputs.Genotype_dr),'int8');
                fwrite(fid,outputs.textdata,'uchar');
                fwrite(fid,int8(outputs.ID_apps),'int8');
                fwrite(fid,int8(outputs.ID_serv),'int8');
                fwrite(fid,int8(outputs.ID_os),'int8');
                fwrite(fid,int8(outputs.ID_dr),'int8');
                fwrite(fid,int8(outputs.ChangeIndDesirabilitiesFlag),'int8');
                fwrite(fid,outputs.tChangeInd1,'double');
                fwrite(fid,outputs.tChangeInd2,'double');
                fwrite(fid,int8(outputs.ID_apps2),'int8');
                fwrite(fid,int8(outputs.ID_serv2),'int8');
                fwrite(fid,int8(outputs.ID_os2),'int8');
                fwrite(fid,int8(outputs.ID_dr2),'int8');
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                fwrite(fid,int8(outputs.minstd4Tolerance),'int8');
                fwrite(fid,int8(outputs.maxstd4Tolerance),'int8');
                fwrite(fid,int8(outputs.minstd5Tolerance),'int8');
                fwrite(fid,int8(outputs.maxstd5Tolerance),'int8');
                fwrite(fid,int8(outputs.turnOnSol1),'int8');
                fwrite(fid,int8(outputs.turnOnSol2),'int8');
                fwrite(fid,int8(outputs.turnOnSol3),'int8');
                fwrite(fid,int8(outputs.turnOnSol4),'int8');
                fwrite(fid,int8(outputs.turnOnSol5),'int8');
                fwrite(fid,int8(outputs.stdMargin),'int8');
                fwrite(fid,int8(outputs.singleStopMode),'int8');
                fwrite(fid,int8(outputs.restartMode),'int8');
                fwrite(fid,int8(outputs.ChangeOwnIndDesirabilitiesFlag),'int8');
                %enable properties added - sept 2015
                fwrite(fid,int8(outputs.datasetEnable),'int8');
                fwrite(fid,int8(outputs.datasetNumber),'int8');
                fwrite(fid,int8(outputs.datasetStaticEnable),'int8'); %added 31st March 2016
                fwrite(fid,int8(outputs.datasetStaticNumber),'int8'); %added 31st March 2016
                fwrite(fid,int8(outputs.networkModelEnable),'int8');
                fwrite(fid,int8(outputs.networkModelV),'int8');
                fwrite(fid,int8(outputs.attackModelEnable),'int8');
                fwrite(fid,int8(outputs.attackModelV),'int8');
                fwrite(fid,int8(outputs.exploitEnable),'int8');
                fwrite(fid,int8(outputs.exploitTypeV),'int8');
                fwrite(fid,int8(outputs.algModelEnable),'int8');
                fwrite(fid,int8(outputs.algModelV),'int8');
                fwrite(fid,int8(outputs.networkPerfEnable),'int8');
                fwrite(fid,int8(outputs.perfModelV),'int8');
                fwrite(fid,int8(outputs.editSaveParameters),'int8');
                %random waypoint animation
                 if and(outputs.networkModelEnable,strcmp(outputs.networkModelS,'Random Waypoint'))
                %x and y waypoints
                for nodeIndex = 1:outputs.numNodes
                fwrite(fid,outputs.vs_node(nodeIndex).v_x(1:outputs.numTimeSteps), 'double'); %structure
                fwrite(fid,outputs.vs_node(nodeIndex).v_y(1:outputs.numTimeSteps), 'double'); %structure
                end
                 end
                %if outputs.attackModelEnable == 1
                %    fwrite(fid,outputs.numStates,'double');
                %end
            end %func
            
            %run files***********
            %open run files
            function runFileHandles = saveOutputsTempFile_OpenRunFiles(outputsTempFileNames,simNumber,runNumber)
            %  filenames(i,j,1) %i numSims j numRuns
            numFiles = size(outputsTempFileNames,2);
            runFileHandles = zeros(1,numFiles);
            for fileNumber = 1:numFiles
            runFileHandles(fileNumber) = fopen(char(outputsTempFileNames(fileNumber)), 'w+'); %w+ reading and writing
            %runFileHandles(fileNumber) = fopen(char(outputsTempFileNames.runFiles(simNumber,runNumber,fileNumber)), 'w'); 
            end
            end  
            
            
            %save run outputs
            function saveOutputsTempFile_SaveOutputs1RunBinaryData(runFileHandles,runOutputs,simOutputs,setOutputs,handles,lengthCounter,fileSavePrecision)
            %fileSavePrecision 1 save data to file as 'double' precision or 2 'single' precision (fileSaveOption 2 and 3 only)  
            %  filenames(i,j,1) %i numSims j numRuns
            %UPTO HERE - write this function
            %simOutputs sizes
            %nodesDiversity4 = zeros(numRunsPerSim,numNodes,numTimeSteps+1); 
            %nodesMean4 = zeros(numRunsPerSim,numNodes,numTimeSteps+1);
            %nodesStd4 = zeros(numRunsPerSim,numNodes,numTimeSteps+1);
            %updateStopDisplay4 = ones(numRunsPerSim,numTimeSteps,numNodes);
            %minStd4Curve = zeros(numRunsPerSim,numNodes,1);
            %updateStopDisplay5 = ones(numRunsPerSim,numTimeSteps,numNodes);
            %nodesStd5 = zeros(numRunsPerSim,numNodes,numTimeSteps+1);
            %minStd5Curve = zeros(numRunsPerSim,numNodes,1);
            %lastStoppedStdMatrix = ones(numRunsPerSim,numTimeSteps,numNodes);
            %lastStoppedStdMatrix2 = ones(numRunsPerSim,numTimeSteps,numNodes);
            %Nei_indDynamic = zeros(numRunsPerSim,1,numTimeSteps);
            %numStates = StateMachineClass.getNumStates(ihandles);
            %ohandles.sumOfStates = zeros(ihandles.numRunsPerSim,numStates,ihandles.numTimeSteps);
            %ohandles.nodeStateAtTime = zeros(ihandles.numRunsPerSim,ihandles.numTimeSteps,ihandles.numNodes); %state of each node at time t
            %ohandles.allRunsNodeStateAtTime = zeros(ihandles.numRunsPerSim);
            %ohandles.allRunsNodeStateAtTime.nodeStateAtTimeStr = cell(ihandles.numTimeSteps,ihandles.numNodes);
            %ohandles.allRunsNodeStateAtTime.nodeStateAtTimeNumber = zeros(ihandles.numTimeSteps,ihandles.numNodes);
            %ohandles.beta = zeros(ihandles.numRunsPerSim,ihandles.numTimeSteps); %measures the instantaneous value of beta(infection rate)for every run
            %ohandles.gammaSIR = zeros(ihandles.numRunsPerSim,ihandles.numTimeSteps);
            %rhandles.nodeStateAtTimeStr = cell(ihandles.numTimeSteps,ihandles.numNodes); %state of each node at time t
            %rhandles.nodeStateAtTimeNumber = zeros(ihandles.numTimeSteps,ihandles.numNodes);
             %   SingleLocus_sol4 = zeros(numRunsPerSim,numTimeSteps,numLoci);
            %SingleLocus_sol5 = zeros(numRunsPerSim,numTimeSteps,numLoci);
            editSaveParameters = str2num(get(handles.editSaveParameters,'string'));
           if fileSavePrecision == 1
            if editSaveParameters(1)==1; fwrite(runFileHandles(1),simOutputs.Nei_sol4(1:lengthCounter),'double'); end;
            if editSaveParameters(2)==1; fwrite(runFileHandles(2),simOutputs.Nei_sol5(1:lengthCounter),'double'); end;
            if editSaveParameters(3)==1; fwrite(runFileHandles(3),runOutputs.nodesDiversity4(:,1:lengthCounter),'double'); end;
            if editSaveParameters(4)==1; fwrite(runFileHandles(4),runOutputs.nodesMean4(:,1:lengthCounter),'double'); end;
            if editSaveParameters(5)==1; fwrite(runFileHandles(5),runOutputs.nodesStd4(:,1:lengthCounter),'double'); end;
            if editSaveParameters(6)==1; fwrite(runFileHandles(6),runOutputs.updateStopDisplay4(1:lengthCounter,:),'double'); end;
            if editSaveParameters(8)==1; fwrite(runFileHandles(8),runOutputs.nodesStd5(:,1:lengthCounter),'double'); end;
            if editSaveParameters(9)==1; fwrite(runFileHandles(9),runOutputs.updateStopDisplay5(1:lengthCounter,:),'double');  end;
            if editSaveParameters(11)==1; fwrite(runFileHandles(11),runOutputs.lastStoppedStdMatrix(1:lengthCounter,:),'double'); end;
            if editSaveParameters(12)==1; fwrite(runFileHandles(12),runOutputs.lastStoppedStdMatrix2(1:lengthCounter,:),'double'); end;
            if editSaveParameters(13)==1; fwrite(runFileHandles(13),runOutputs.Nei_indDynamic(1,1:lengthCounter),'double'); end;
           elseif fileSavePrecision ==2
            if editSaveParameters(1)==1; fwrite(runFileHandles(1),single(simOutputs.Nei_sol4(1:lengthCounter)),'single'); end;
            if editSaveParameters(2)==1; fwrite(runFileHandles(2),single(simOutputs.Nei_sol5(1:lengthCounter)),'single'); end;
            if editSaveParameters(3)==1; fwrite(runFileHandles(3),single(runOutputs.nodesDiversity4(:,1:lengthCounter)),'single'); end;
            if editSaveParameters(4)==1; fwrite(runFileHandles(4),single(runOutputs.nodesMean4(:,1:lengthCounter)),'single'); end;
            if editSaveParameters(5)==1; fwrite(runFileHandles(5),single(runOutputs.nodesStd4(:,1:lengthCounter)),'single'); end;
            if editSaveParameters(6)==1; fwrite(runFileHandles(6),single(runOutputs.updateStopDisplay4(1:lengthCounter,:)),'single'); end;
            if editSaveParameters(8)==1; fwrite(runFileHandles(8),single(runOutputs.nodesStd5(:,1:lengthCounter)),'single'); end;
            if editSaveParameters(9)==1; fwrite(runFileHandles(9),single(runOutputs.updateStopDisplay5(1:lengthCounter,:)),'single');  end;
            if editSaveParameters(11)==1; fwrite(runFileHandles(11),single(runOutputs.lastStoppedStdMatrix(1:lengthCounter,:)),'single'); end;
            if editSaveParameters(12)==1; fwrite(runFileHandles(12),single(runOutputs.lastStoppedStdMatrix2(1:lengthCounter,:)),'single'); end;
            if editSaveParameters(13)==1; fwrite(runFileHandles(13),single(runOutputs.Nei_indDynamic(1,1:lengthCounter)),'single'); end;
           end
            %TO BE WRITTEN SEPERATELY 
            %fwrite(runFileHandles(7),runOutputs.minStd4Curve,'double');
            %fwrite(runFileHandles(10),runOutputs.minStd5Curve,'double');
            %attack model outputs
            if setOutputs.attackModelEnable == 1
                %disp('attack model save outputs to file')
                %TO BE WRITTEN SEPERATELY 
                fwrite(runFileHandles(14),runOutputs.numStates,'double'); %need to save this for reading editSaveParameters(15)
                 if editSaveParameters(15)==1; fwrite(runFileHandles(15),int32(runOutputs.sumOfStates(:,1:lengthCounter)),'int32'); end;
                %temp1.nodeStateAtTimeStr = outputs.allRunsNodeStateAtTime(runNumber).nodeStateAtTimeStr
                if editSaveParameters(16)==1; 
                for j = 1:lengthCounter
                   temp2 = char(runOutputs.nodeStateAtTimeStr(j,:)); %convert from cellstr to char for fwrite
                    fwrite(runFileHandles(16),temp2, 'char');%structure
                end
                end
            %temp1.nodeStateAtTimeNumber = outputs.allRunsNodeStateAtTime(runNumber).nodeStateAtTimeNumber;
             if editSaveParameters(17)==1; fwrite(runFileHandles(17),int8(runOutputs.nodeStateAtTimeNumber(1:lengthCounter,:)),'int8'); end;%structure   
            end
            
            if fileSavePrecision ==1
             if editSaveParameters(18)==1; 
            neiIndexSingleLocus_sol4(1:lengthCounter,:) = simOutputs.neiIndexSingleLocus_sol4(1,1:lengthCounter,:);
            fwrite(runFileHandles(18),neiIndexSingleLocus_sol4(1:lengthCounter,:),'double');
             end
             if editSaveParameters(19)==1; 
            neiIndexSingleLocus_sol5(1:lengthCounter,:) = simOutputs.neiIndexSingleLocus_sol5(1,1:lengthCounter,:);
            fwrite(runFileHandles(19),neiIndexSingleLocus_sol5(1:lengthCounter,:),'double');
             end
            elseif fileSavePrecision ==2
             if editSaveParameters(18)==1; 
            neiIndexSingleLocus_sol4(1:lengthCounter,:) = simOutputs.neiIndexSingleLocus_sol4(1,1:lengthCounter,:);
            fwrite(runFileHandles(18),single(neiIndexSingleLocus_sol4(1:lengthCounter,:)),'single');
             end
             if editSaveParameters(19)==1; 
            neiIndexSingleLocus_sol5(1:lengthCounter,:) = simOutputs.neiIndexSingleLocus_sol5(1,1:lengthCounter,:);
            fwrite(runFileHandles(19),single(neiIndexSingleLocus_sol5(1:lengthCounter,:)),'single');   
             end
            end
            if editSaveParameters(20)==1; 
            genoTemp4_1(:,:,:) = simOutputs.GenoSim_sol4(1,:,:,:);
            for nodeNumber = 1:setOutputs.numNodes
                genoTemp4(:,:) = genoTemp4_1(:,nodeNumber,:);
                fwrite(runFileHandles(20),int8(genoTemp4(1:lengthCounter,:)),'int8');
            end
            end
             if editSaveParameters(21)==1; 
            %GenoSim_sol5 = zeros(numRunsPerSim,numTimeSteps,numNodes,numVariants);
            genoTemp5_1(:,:,:) = simOutputs.GenoSim_sol5(1,:,:,:);
            for nodeNumber = 1:setOutputs.numNodes
                genoTemp5(:,:) = genoTemp5_1(:,nodeNumber,:);
                fwrite(runFileHandles(21),int8(genoTemp5(1:lengthCounter,:)),'int8');
            end
             end
             %test
            % fileSavePrecisionD=fileSavePrecision
            % editSaveParametersD=editSaveParameters
            % size(editSaveParameters)
            % size(runFileHandles)
            % lengthCounterd=lengthCounter
            % size(runOutputs.beta)
             %
            if fileSavePrecision ==1 
                if editSaveParameters(22)==1; fwrite(runFileHandles(22),runOutputs.beta(1:lengthCounter),'double'); end;
            elseif fileSavePrecision == 2
               if editSaveParameters(22)==1; fwrite(runFileHandles(22),single(runOutputs.beta(1:lengthCounter)),'single'); end   
            end
            end %func

            %close run files
            function saveOutputsTempFile_CloseRunFiles(runFileHandles)
            %fclose('all');
            for i = 1 : size(runFileHandles,2)
            fclose(runFileHandles(i));
            end
            end %func
            
            %save single values that are not every time step
            function saveOutputsTempFile_SaveOutputs1RunBinaryDataSingleValues(runFileHandles,runOutputs,ohandles,setOutputs,handles)
                editSaveParameters = str2num(get(handles.editSaveParameters,'string'))
                 %minStd4Curve = zeros(numRunsPerSim,numNodes,1);
                if editSaveParameters(7)==1; fwrite(runFileHandles(7),runOutputs.minStd4Curve,'double'); end;
                if editSaveParameters(10)==1; fwrite(runFileHandles(10),runOutputs.minStd5Curve,'double'); end;
                if setOutputs.attackModelEnable == 1
                    if editSaveParameters(14)==1; fwrite(runFileHandles(14),runOutputs.numStates,'double'); end;
                 end
            end
            
            %load temp files
            %Simulation header************
            function simOutputs = loadOutputsTempFile_simHeader(outputTempDir,simNumber)
                fidtempName = [outputTempDir 'outputsTemp_sim' num2str(simNumber) 'Header.bin'];
                fidtemp = fopen(fidtempName, 'r');
                 %load all sim data
                simOutputs = GUIClass.loadOutputs1SimBinary_Header(fidtemp);
                %close file
                fclose(fidtemp);
            end
            function outputs = loadOutputs1SimBinary_Header(fid)
                %read once
                outputs.fixLength = double(fread(fid,1,'int32'));
                outputs.numNodes = double(fread(fid,1,'int16'));
                outputs.numRuns = double(fread(fid,1,'int16')); %NEEDS TO BE INT 16
                outputs.numTimeSteps = fread(fid,1,'double');
                outputs.len_apps = double(fread(fid,1,'int8'));
                outputs.len_serv = double(fread(fid,1,'int8'));
                outputs.len_os = double(fread(fid,1,'int8'));
                outputs.len_dr = double(fread(fid,1,'int8'));
                outputs.CD_apps = double(fread(fid,[1,outputs.len_apps],'int8'));
                outputs.CD_serv = double(fread(fid,[1,outputs.len_serv],'int8'));
                outputs.CD_os = double(fread(fid,[1,outputs.len_os],'int8'));
                outputs.CD_dr = double(fread(fid,[1,outputs.len_dr],'int8'));              
                outputs.ChangeDesirabilitiesFlag = double(fread(fid,1,'int8'));
                outputs.tChange1 = fread(fid,1,'double');
                outputs.tChange2 = fread(fid,1,'double');
                outputs.CD_apps2 = double(fread(fid,[1,outputs.len_apps],'int8'));
                outputs.CD_serv2 = double(fread(fid,[1,outputs.len_serv],'int8'));
                outputs.CD_os2 = double(fread(fid,[1,outputs.len_os],'int8'));
                outputs.CD_dr2 = double(fread(fid,[1,outputs.len_dr],'int8'));
                outputs.E_apps = double(fread(fid,[1,outputs.len_apps],'int8'));
                outputs.E_serv = double(fread(fid,[1,outputs.len_serv],'int8'));
                outputs.E_os = double(fread(fid,[1,outputs.len_os],'int8'));
                outputs.E_dr = double(fread(fid,[1,outputs.len_dr],'int8'));      
                outputs.ChangeEffFlag = double(fread(fid,1,'int8'));
                outputs.tChangeEff1 = fread(fid,1,'double');
                outputs.tChangeEff2 = fread(fid,1,'double');
                outputs.E_apps2 = double(fread(fid,[1,outputs.len_apps],'int8'));
                outputs.E_serv2 = double(fread(fid,[1,outputs.len_serv],'int8'));
                outputs.E_os2 = double(fread(fid,[1,outputs.len_os],'int8'));
                outputs.E_dr2 = double(fread(fid,[1,outputs.len_dr],'int8'));                            
                outputs.Genotype_apps = double(fread(fid,[outputs.numNodes,outputs.len_apps],'int8'));
                outputs.Genotype_serv = double(fread(fid,[outputs.numNodes,outputs.len_serv],'int8'));
                outputs.Genotype_os = double(fread(fid,[outputs.numNodes,outputs.len_os],'int8'));
                outputs.Genotype_dr = double(fread(fid,[outputs.numNodes,outputs.len_dr],'int8'));
                rowstextdata = outputs.len_apps + outputs.len_serv + outputs.len_os + outputs.len_dr;
                outputs.textdata = fread(fid,[rowstextdata,2],'*char');
                outputs.ID_apps = double(fread(fid,[outputs.numNodes,outputs.len_apps],'int8'));
                outputs.ID_serv = double(fread(fid,[outputs.numNodes,outputs.len_serv],'int8'));
                outputs.ID_os = double(fread(fid,[outputs.numNodes,outputs.len_os],'int8'));
                outputs.ID_dr = double(fread(fid,[outputs.numNodes,outputs.len_dr],'int8'));
                outputs.ID = [outputs.ID_apps outputs.ID_serv outputs.ID_os outputs.ID_dr]';
                outputs.ChangeIndDesirabilitiesFlag = double(fread(fid,1,'int8'));
                outputs.tChangeInd1 = fread(fid,1,'double');
                outputs.tChangeInd2 = fread(fid,1,'double');
                outputs.ID_apps2 = double(fread(fid,[outputs.numNodes,outputs.len_apps],'int8'));
                outputs.ID_serv2 = double(fread(fid,[outputs.numNodes,outputs.len_serv],'int8'));
                outputs.ID_os2 = double(fread(fid,[outputs.numNodes,outputs.len_os],'int8'));
                outputs.ID_dr2 = double(fread(fid,[outputs.numNodes,outputs.len_dr],'int8'));
                outputs.ID2 = [outputs.ID_apps2 outputs.ID_serv2 outputs.ID_os2 outputs.ID_dr2]'; 
                outputs.minstd4Tolerance = double(fread(fid,1,'int8'));
                outputs.maxstd4Tolerance = double(fread(fid,1,'int8'));
                outputs.minstd5Tolerance = double(fread(fid,1,'int8'));
                outputs.maxstd5Tolerance = double(fread(fid,1,'int8'));
                outputs.turnOnSol1 = double(fread(fid,1,'int8'));
                outputs.turnOnSol2 = double(fread(fid,1,'int8'));
                outputs.turnOnSol3 = double(fread(fid,1,'int8'));
                outputs.turnOnSol4 = double(fread(fid,1,'int8'));
                outputs.turnOnSol5 = double(fread(fid,1,'int8'));
                outputs.stdMargin = double(fread(fid,1,'int8'));
                outputs.singleStopMode = double(fread(fid,1,'int8'));
                outputs.restartMode = double(fread(fid,1,'int8'));
                outputs.ChangeOwnIndDesirabilitiesFlag = double(fread(fid,1,'int8'));
                outputs.datasetEnable = double(fread(fid,1,'int8'));
                outputs.datasetNumber = double(fread(fid,1,'int8'));
                outputs.datasetStaticEnable = double(fread(fid,1,'int8'));
                outputs.datasetStaticNumber = double(fread(fid,1,'int8'));
                outputs.networkModelEnable = double(fread(fid,1,'int8'));
                outputs.networkModelV = double(fread(fid,1,'int8'));
                outputs.networkModelS = GUIClass.getNetworkModelString(outputs.networkModelV);
                outputs.attackModelEnable = double(fread(fid,1,'int8'));
                outputs.attackModelV = double(fread(fid,1,'int8'));
                outputs.attackModelS = GUIClass.getAttackModelString(outputs.attackModelV);
                outputs.exploitEnable= double(fread(fid,1,'int8'));
                outputs.exploitTypeV= double(fread(fid,1,'int8'));
                outputs.exploitTypeS = GUIClass.getExploitModelString(outputs.exploitTypeV);
                outputs.algModelEnable = double(fread(fid,1,'int8'));
                outputs.algModelV = double(fread(fid,1,'int8'));
                outputs.algModelS = GUIClass.getAlgModelString(outputs.algModelV);
                outputs.networkPerfEnable = double(fread(fid,1,'int8'));
                outputs.perfModelV = double(fread(fid,1,'int8'));
                outputs.perfModelS = GUIClass.getPerfModelString(outputs.perfModelV);
                outputs.editSaveParameters = double(fread(fid,22,'int8'));
                %random waypoint animation
                if and(outputs.networkModelEnable,strcmp(outputs.networkModelS,'Random Waypoint'))
                %x and y waypoints
                    for nodeIndex = 1:outputs.numNodes  
                        temp1.vs_node(nodeIndex).v_x = fread(fid,[outputs.numTimeSteps],'double');
                        temp1.vs_node(nodeIndex).v_y = fread(fid,[outputs.numTimeSteps],'double');
                        %get in correct orientation
                        outputs.vs_node(nodeIndex).v_x = temp1.vs_node(nodeIndex).v_x';
                        outputs.vs_node(nodeIndex).v_y = temp1.vs_node(nodeIndex).v_y'; 
                    end  
                end %if enable
            end %func
            %load 1 set of run files
           function [runOutputs runOutputs_single] = loadOutputsTempFile_oneRun(outputTempDir,simNumber,runNumber,simOutputs,fileSavePrecision)
                outputsTempFileNames = GUIClass.getTempRunFileNames(outputTempDir,simNumber,runNumber);
                numFiles = size(outputsTempFileNames,2);
                runFileHandles = zeros(1,numFiles);
                for fileNumber = 1:numFiles
                    runFileHandles(fileNumber) = fopen(char(outputsTempFileNames(fileNumber)), 'r'); %r reading only
                end
                runOutputs_single = GUIClass.loadOutputsTempFile_loadOutputs1RunBinaryDataSingleValues(runFileHandles,simOutputs);
                runOutputs = GUIClass.loadOutputsTempFile_LoadRunOutputs(runFileHandles,simOutputs,runOutputs_single,fileSavePrecision);
                GUIClass.saveOutputsTempFile_CloseRunFiles(runFileHandles);
           end
           %just loads sum of states data
            function [runOutputs] = loadOutputsTempFile_LoadOneRunSumOfStates(outputTempDir,simNumber,runNumber,simOutputs)
                outputsTempFileNames = GUIClass.getTempRunFileNames(outputTempDir,simNumber,runNumber);
                numFiles = size(outputsTempFileNames,2);
                numberOfSumFile = 14; %number of states file number
                numberOfStateFile = 15; %sum of states file number
                runFileHandles = zeros(1,numFiles);
                %******************************************* 
                %get number of states
                runFileHandles(numberOfSumFile) = fopen(char(outputsTempFileNames(numberOfSumFile)), 'r'); %r reading only
                runOutputs_single.numStates = fread(runFileHandles(14),[simOutputs.numNodes,1],'double'); %disp('no numStates data to load'); end;
                fclose(runFileHandles(numberOfSumFile));
                %*****************************************
                %get values of states
                runFileHandles(numberOfStateFile) = fopen(char(outputsTempFileNames(numberOfStateFile)), 'r'); %r reading only
                %runOutputs_single = GUIClass.loadOutputsTempFile_loadOutputs1RunBinaryDataSingleValues(runFileHandles,simOutputs);

             lengthCounter=1;
            %fixLengthReadFile = simOutputs.fixLength
            for t = 1 : simOutputs.numTimeSteps
                if or(lengthCounter == simOutputs.fixLength,t==simOutputs.numTimeSteps)
                    %attack model outputs
                    if simOutputs.attackModelEnable == 1
                        %disp('attack model load outputs from file')
                        %a = fread(fid,outputs.numRuns*outputs.numStates*outputs.numTimeSteps, '*double');
                        %outputs.sumOfStates = reshape(a,[outputs.numRuns,outputs.numStates,outputs.numTimeSteps]);
                        if simOutputs.editSaveParameters(15) == 1; %needs simOutputs.editSaveParameters(14) saved to work
                            runOutputs.sumOfStates(:,t-lengthCounter+1:t) = double(fread(runFileHandles(15),[runOutputs_single.numStates(1),lengthCounter],'int32'));
                        else runOutputs.sumOfStates(:,t-lengthCounter+1:t) = zeros(runOutputs_single.numStates,lengthCounter); 
                        end; %disp('no sumOfStates data to load'); end;
                     end
                    lengthCounter=1;
                else
                    lengthCounter=lengthCounter+1;
                end%if
            end%for
            fclose(runFileHandles(numberOfStateFile));
            %******************************************************
            end%func
           
            function runOutputs = loadOutputsTempFile_LoadRunOutputs(runFileHandles,simOutputs,runOutputs_single,fileSavePrecision)
            lengthCounter=1;
            %fixLengthReadFile = simOutputs.fixLength
            for t = 1 : simOutputs.numTimeSteps
                if or(lengthCounter == simOutputs.fixLength,t==simOutputs.numTimeSteps)
                    if fileSavePrecision == 1
                    if simOutputs.editSaveParameters(1) == 1; runOutputs.Nei_sol4(:,t-lengthCounter+1:t) = fread(runFileHandles(1),[1,lengthCounter],'double'); else runOutputs.Nei_sol4(:,t-lengthCounter+1:t) = zeros(1,lengthCounter); end; %disp('no Nei_sol4 data to load'); end;
                    if simOutputs.editSaveParameters(2) == 1; runOutputs.Nei_sol5(:,t-lengthCounter+1:t) = fread(runFileHandles(2),[1,lengthCounter],'double'); else runOutputs.Nei_sol5(:,t-lengthCounter+1:t) = zeros(1,lengthCounter); end; %disp('no Nei_sol5 data to load'); end;
                    %outputs.nodesDiversity4 = reshape(a,[outputs.numRuns,outputs.numNodes,outputs.numTimeSteps+1]);
                    if simOutputs.editSaveParameters(3) == 1; runOutputs.nodesDiversity4(:,t-lengthCounter+1:t) = fread(runFileHandles(3),[simOutputs.numNodes,lengthCounter],'double'); else runOutputs.nodesDiversity4(:,t-lengthCounter+1:t) = zeros(simOutputs.numNodes,lengthCounter); end; %disp('no nodesDiversity4 data to load'); end;
                    %a = fread(fid,outputs.numRuns*outputs.numNodes*(outputs.numTimeSteps+1), '*double');
                    %outputs.nodesMean4 = reshape(a,[outputs.numRuns,outputs.numNodes,outputs.numTimeSteps+1]);
                    if simOutputs.editSaveParameters(4) == 1; runOutputs.nodesMean4(:,t-lengthCounter+1:t) = fread(runFileHandles(4),[simOutputs.numNodes,lengthCounter],'double'); else runOutputs.nodesMean4(:,t-lengthCounter+1:t) = zeros(simOutputs.numNodes,lengthCounter); end; %disp('no nodesMean4 data to load'); end;
                    if simOutputs.editSaveParameters(5) == 1; runOutputs.nodesStd4(:,t-lengthCounter+1:t) = fread(runFileHandles(5),[simOutputs.numNodes,lengthCounter],'double'); else runOutputs.nodesStd4(:,t-lengthCounter+1:t) = zeros(simOutputs.numNodes,lengthCounter); end; %disp('no nodesStd4 data to load'); end;
                    %a = fread(fid,outputs.numRuns*outputs.numTimeSteps*outputs.numNodes, '*double');
                    %outputs.updateStopDisplay4 = reshape(a,[outputs.numRuns,outputs.numTimeSteps,outputs.numNodes]);
                    if simOutputs.editSaveParameters(6) == 1; runOutputs.updateStopDisplay4(t-lengthCounter+1:t,:) = fread(runFileHandles(6),[lengthCounter,simOutputs.numNodes],'double'); else runOutputs.updateStopDisplay4(t-lengthCounter+1:t,:) = zeros(lengthCounter,simOutputs.numNodes); end; %disp('no updateStopDisplay4 data to load'); end;
                    %a = fread(fid,outputs.numRuns*outputs.numNodes*1, '*double');
                    %outputs.minStd4Curve = reshape(a,[outputs.numRuns,outputs.numNodes,1]);
                    %READ SEPERATELYrunOutputs.minStd4Curve(:,1) = fread(runFileHandles(7),[numNodes,1],'double'); 
                    if simOutputs.editSaveParameters(8) == 1; runOutputs.nodesStd5(:,t-lengthCounter+1:t) = fread(runFileHandles(8),[simOutputs.numNodes,lengthCounter],'double'); else runOutputs.nodesStd5(:,t-lengthCounter+1:t) = zeros(simOutputs.numNodes,lengthCounter); end; %disp('no nodesStd5 data to load'); end;
                    if simOutputs.editSaveParameters(9) == 1; runOutputs.updateStopDisplay5(t-lengthCounter+1:t,:) = fread(runFileHandles(9),[lengthCounter,simOutputs.numNodes],'double'); else runOutputs.updateStopDisplay5(t-lengthCounter+1:t,:) = zeros(lengthCounter,simOutputs.numNodes); end; %disp('no updateStopDisplay5 data to load'); end;
                    %READ SEPERATELY runOutputs.minStd5Curve(:,1) = fread(runFileHandles(10),[numNodes,1],'double');  
                    %a = fread(fid,outputs.numRuns*outputs.numTimeSteps*outputs.numNodes, '*double');
                    %outputs.lastStoppedStdMatrix = reshape(a,[outputs.numRuns,outputs.numTimeSteps,outputs.numNodes]);
                    if simOutputs.editSaveParameters(11) == 1; runOutputs.lastStoppedStdMatrix(t-lengthCounter+1:t,:) = fread(runFileHandles(11),[lengthCounter,simOutputs.numNodes],'double'); else runOutputs.lastStoppedStdMatrix(t-lengthCounter+1:t,:) = zeros(lengthCounter,simOutputs.numNodes); end; %disp('no lastStoppedStdMatrix data to load'); end;
                    if simOutputs.editSaveParameters(12) == 1; unOutputs.lastStoppedStdMatrix2(t-lengthCounter+1:t,:) = fread(runFileHandles(12),[lengthCounter,simOutputs.numNodes],'double'); else runOutputs.lastStoppedStdMatrix2(t-lengthCounter+1:t,:) = zeros(lengthCounter,simOutputs.numNodes); end; %disp('no lastStoppedStdMatrix2 data to load'); end;
                    %a = fread(fid,outputs.numRuns*1*outputs.numTimeSteps, '*double');
                    %outputs.Nei_indDynamic = reshape(a,[outputs.numRuns,1,outputs.numTimeSteps]);
                    if simOutputs.editSaveParameters(13) == 1; runOutputs.Nei_indDynamic(1,t-lengthCounter+1:t) = fread(runFileHandles(13),[1,lengthCounter],'double'); else runOutputs.Nei_indDynamic(1,t-lengthCounter+1:t) = zeros(1,lengthCounter); end; %disp('no Nei_indDynamic data to load'); end;
                    elseif fileSavePrecision == 2
                    if simOutputs.editSaveParameters(1) == 1; runOutputs.Nei_sol4(:,t-lengthCounter+1:t) = double(fread(runFileHandles(1),[1,lengthCounter],'single')); else runOutputs.Nei_sol4(:,t-lengthCounter+1:t) = zeros(1,lengthCounter); end; %disp('no Nei_sol4 data to load'); end;
                    if simOutputs.editSaveParameters(2) == 1; runOutputs.Nei_sol5(:,t-lengthCounter+1:t) = double(fread(runFileHandles(2),[1,lengthCounter],'single')); else runOutputs.Nei_sol5(:,t-lengthCounter+1:t) = zeros(1,lengthCounter); end; %disp('no Nei_sol5 data to load'); end;
                    %outputs.nodesDiversity4 = reshape(a,[outputs.numRuns,outputs.numNodes,outputs.numTimeSteps+1]);
                    if simOutputs.editSaveParameters(3) == 1; runOutputs.nodesDiversity4(:,t-lengthCounter+1:t) = double(fread(runFileHandles(3),[simOutputs.numNodes,lengthCounter],'single')); else runOutputs.nodesDiversity4(:,t-lengthCounter+1:t) = zeros(simOutputs.numNodes,lengthCounter); end; %disp('no nodesDiversity4 data to load'); end;
                    %a = fread(fid,outputs.numRuns*outputs.numNodes*(outputs.numTimeSteps+1), '*double');
                    %outputs.nodesMean4 = reshape(a,[outputs.numRuns,outputs.numNodes,outputs.numTimeSteps+1]);
                    if simOutputs.editSaveParameters(4) == 1; runOutputs.nodesMean4(:,t-lengthCounter+1:t) = double(fread(runFileHandles(4),[simOutputs.numNodes,lengthCounter],'single')); else runOutputs.nodesMean4(:,t-lengthCounter+1:t) = zeros(simOutputs.numNodes,lengthCounter); end; %disp('no nodesMean4 data to load'); end;
                    if simOutputs.editSaveParameters(5) == 1; runOutputs.nodesStd4(:,t-lengthCounter+1:t) = double(fread(runFileHandles(5),[simOutputs.numNodes,lengthCounter],'single')); else runOutputs.nodesStd4(:,t-lengthCounter+1:t) = zeros(simOutputs.numNodes,lengthCounter); end; %disp('no nodesStd4 data to load'); end;
                    %a = fread(fid,outputs.numRuns*outputs.numTimeSteps*outputs.numNodes, '*double');
                    %outputs.updateStopDisplay4 = reshape(a,[outputs.numRuns,outputs.numTimeSteps,outputs.numNodes]);
                    if simOutputs.editSaveParameters(6) == 1; runOutputs.updateStopDisplay4(t-lengthCounter+1:t,:) = double(fread(runFileHandles(6),[lengthCounter,simOutputs.numNodes],'single')); else runOutputs.updateStopDisplay4(t-lengthCounter+1:t,:) = zeros(lengthCounter,simOutputs.numNodes); end; %disp('no updateStopDisplay4 data to load'); end;
                    %a = fread(fid,outputs.numRuns*outputs.numNodes*1, '*double');
                    %outputs.minStd4Curve = reshape(a,[outputs.numRuns,outputs.numNodes,1]);
                    %READ SEPERATELYrunOutputs.minStd4Curve(:,1) = fread(runFileHandles(7),[numNodes,1],'double'); 
                    if simOutputs.editSaveParameters(8) == 1; runOutputs.nodesStd5(:,t-lengthCounter+1:t) = double(fread(runFileHandles(8),[simOutputs.numNodes,lengthCounter],'single')); else runOutputs.nodesStd5(:,t-lengthCounter+1:t) = zeros(simOutputs.numNodes,lengthCounter); end; %disp('no nodesStd5 data to load'); end;
                    if simOutputs.editSaveParameters(9) == 1; runOutputs.updateStopDisplay5(t-lengthCounter+1:t,:) = double(fread(runFileHandles(9),[lengthCounter,simOutputs.numNodes],'single')); else runOutputs.updateStopDisplay5(t-lengthCounter+1:t,:) = zeros(lengthCounter,simOutputs.numNodes); end; %disp('no updateStopDisplay5 data to load'); end;
                    %READ SEPERATELY runOutputs.minStd5Curve(:,1) = fread(runFileHandles(10),[numNodes,1],'double');  
                    %a = fread(fid,outputs.numRuns*outputs.numTimeSteps*outputs.numNodes, '*double');
                    %outputs.lastStoppedStdMatrix = reshape(a,[outputs.numRuns,outputs.numTimeSteps,outputs.numNodes]);
                    if simOutputs.editSaveParameters(11) == 1; runOutputs.lastStoppedStdMatrix(t-lengthCounter+1:t,:) = double(fread(runFileHandles(11),[lengthCounter,simOutputs.numNodes],'single')); else runOutputs.lastStoppedStdMatrix(t-lengthCounter+1:t,:) = zeros(lengthCounter,simOutputs.numNodes); end; %disp('no lastStoppedStdMatrix data to load'); end;
                    if simOutputs.editSaveParameters(12) == 1; runOutputs.lastStoppedStdMatrix2(t-lengthCounter+1:t,:) = double(fread(runFileHandles(12),[lengthCounter,simOutputs.numNodes],'single')); else runOutputs.lastStoppedStdMatrix2(t-lengthCounter+1:t,:) = zeros(lengthCounter,simOutputs.numNodes); end; %disp('no lastStoppedStdMatrix2 data to load'); end;
                    %a = fread(fid,outputs.numRuns*1*outputs.numTimeSteps, '*double');
                    %outputs.Nei_indDynamic = reshape(a,[outputs.numRuns,1,outputs.numTimeSteps]);
                    if simOutputs.editSaveParameters(13) == 1; runOutputs.Nei_indDynamic(1,t-lengthCounter+1:t) = double(fread(runFileHandles(13),[1,lengthCounter],'single')); else runOutputs.Nei_indDynamic(1,t-lengthCounter+1:t) = zeros(1,lengthCounter); end; %disp('no Nei_indDynamic data to load'); end;    
                    end
                    %attack model outputs
                    if simOutputs.attackModelEnable == 1
                        %disp('attack model load outputs from file')
                        %a = fread(fid,outputs.numRuns*outputs.numStates*outputs.numTimeSteps, '*double');
                        %outputs.sumOfStates = reshape(a,[outputs.numRuns,outputs.numStates,outputs.numTimeSteps]);
                        if simOutputs.editSaveParameters(15) == 1; %needs simOutputs.editSaveParameters(14) saved to work
                            runOutputs.sumOfStates(:,t-lengthCounter+1:t) = double(fread(runFileHandles(15),[runOutputs_single.numStates(1),lengthCounter],'int32'));
                        else runOutputs.sumOfStates(:,t-lengthCounter+1:t) = zeros(runOutputs_single.numStates,lengthCounter); 
                        end; %disp('no sumOfStates data to load'); end;
                        %outputs.allRunsNodeStateAtTime
                        if simOutputs.editSaveParameters(16) == 1; 
                        for j = 1:lengthCounter
                            temp5 = fread(runFileHandles(16),[simOutputs.numNodes,1],'*char'); %structure;
                            temp3.nodeStateAtTimeStr(j,:) = cellstr(temp5);
                        end
                        runOutputs.nodeStateAtTimeStr(t-lengthCounter+1:t,:) = cellstr(temp3.nodeStateAtTimeStr(1:lengthCounter,:));
                        else runOutputs.nodeStateAtTimeStr(t-lengthCounter+1:t,:) = cellstr(num2str(zeros(1,lengthCounter))); %disp('no nodeStateAtTimeStr data to load');
                        end
                        if simOutputs.editSaveParameters(17) == 1;runOutputs.nodeStateAtTimeNumber(t-lengthCounter+1:t,:) = double(fread(runFileHandles(17),[lengthCounter,simOutputs.numNodes],'int8')) ; else runOutputs.nodeStateAtTimeNumber(t-lengthCounter+1:t,:)  = zeros(lengthCounter,simOutputs.numNodes); end; %disp('no nodeStateAtTimeStr data to load'); end;
                    end
                    %disp('load outputs file num loci set')
                    numLoci = 4;
                    if fileSavePrecision == 1
                    %SingleLocus_sol4 = zeros(numRunsPerSim,numTimeSteps,numLoci);
                    if simOutputs.editSaveParameters(18) == 1; runOutputs.SingleLocus_sol4(t-lengthCounter+1:t,:) = fread(runFileHandles(18),[lengthCounter,numLoci],'double'); else runOutputs.SingleLocus_sol4(t-lengthCounter+1:t,:) = zeros(lengthCounter,numLoci); end; %disp('no SingleLocus_sol4 data to load'); end;
                    if simOutputs.editSaveParameters(19) == 1; runOutputs.SingleLocus_sol5(t-lengthCounter+1:t,:) = fread(runFileHandles(19),[lengthCounter,numLoci],'double'); else runOutputs.SingleLocus_sol5(t-lengthCounter+1:t,:) = zeros(lengthCounter,numLoci); end; %disp('no SingleLocus_sol5 data to load'); end;   
                    elseif fileSavePrecision == 2
                    if simOutputs.editSaveParameters(18) == 1; runOutputs.SingleLocus_sol4(t-lengthCounter+1:t,:) = double(fread(runFileHandles(18),[lengthCounter,numLoci],'single')); else runOutputs.SingleLocus_sol4(t-lengthCounter+1:t,:) = zeros(lengthCounter,numLoci); end; %disp('no SingleLocus_sol4 data to load'); end;
                    if simOutputs.editSaveParameters(19) == 1; runOutputs.SingleLocus_sol5(t-lengthCounter+1:t,:) = double(fread(runFileHandles(19),[lengthCounter,numLoci],'single')); else runOutputs.SingleLocus_sol5(t-lengthCounter+1:t,:) = zeros(lengthCounter,numLoci); end; %disp('no SingleLocus_sol5 data to load'); end;  
                    end
                    %GenoSim_sol4 = zeros(numRunsPerSim,numTimeSteps,numNodes,numVariants); 
                    %GenoSim_sol5 = zeros(numRunsPerSim,numTimeSteps,numNodes,numVariants);
                    numVariants = simOutputs.len_apps + simOutputs.len_serv + simOutputs.len_os + simOutputs.len_dr ;
                    if simOutputs.editSaveParameters(20) == 1; 
                    for nodeNumber = 1:simOutputs.numNodes
                        genoTemp4 = double(fread(runFileHandles(20),[lengthCounter,numVariants],'int8'));
                        runOutputs.GenoSim_sol4(t-lengthCounter+1:t,nodeNumber,:) = genoTemp4;
                    end
                    else runOutputs.GenoSim_sol4(t-lengthCounter+1:t,:,:) = zeros(lengthCounter,simOutputs.numNodes,numVariants);
                    end
                    if simOutputs.editSaveParameters(21) == 1; 
                    for nodeNumber = 1:simOutputs.numNodes
                        genoTemp5 = double(fread(runFileHandles(21),[lengthCounter,numVariants],'int8'));
                        runOutputs.GenoSim_sol5(t-lengthCounter+1:t,nodeNumber,:) = genoTemp5;
                    end
                    else runOutputs.GenoSim_sol5(t-lengthCounter+1:t,:,:) = zeros(lengthCounter,simOutputs.numNodes,numVariants);
                    end
                    %ohandles.beta = zeros(ihandles.numRunsPerSim,ihandles.numTimeSteps);
                    if fileSavePrecision == 1
                        if simOutputs.editSaveParameters(22) == 1; runOutputs.beta(t-lengthCounter+1:t) = fread(runFileHandles(22),[lengthCounter],'double'); else runOutputs.beta(t-lengthCounter+1:t) = zeros(1,lengthCounter); end; %disp('no beta data to load'); end;
                    elseif fileSavePrecision == 2
                        if simOutputs.editSaveParameters(22) == 1; runOutputs.beta(t-lengthCounter+1:t) = double(fread(runFileHandles(22),[lengthCounter],'single')); else runOutputs.beta(t-lengthCounter+1:t) = zeros(1,lengthCounter); end; %disp('no beta data to load'); end;
                    end
                    lengthCounter=1;
                else
                    lengthCounter=lengthCounter+1;
                end%if
            end%for
            end%func
            %load single values that are not every time step
            function runOutputs_single = loadOutputsTempFile_loadOutputs1RunBinaryDataSingleValues(runFileHandles,setOutputs)
                 %minStd4Curve = zeros(numRunsPerSim,numNodes,1);
                if setOutputs.editSaveParameters(7) == 1; runOutputs_single.minStd4Curve = fread(runFileHandles(7),[setOutputs.numNodes,1],'double'); else runOutputs_single.minStd4Curve = zeros(setOutputs.numNodes,1); end; %disp('no minStd4Curve data to load'); end;
                if setOutputs.editSaveParameters(10) == 1; runOutputs_single.minStd5Curve = fread(runFileHandles(10),[setOutputs.numNodes,1],'double'); else runOutputs_single.minStd5Curve = zeros(setOutputs.numNodes,1); end; %disp('no minStd5Curve data to load'); end;
                if setOutputs.attackModelEnable == 1
                    if setOutputs.editSaveParameters(14) == 1; runOutputs_single.numStates = fread(runFileHandles(14),[setOutputs.numNodes,1],'double'); else runOutputs_single.numStates = zeros(setOutputs.numNodes,1); end; %disp('no numStates data to load'); end;
                 end
            end   
            function outputs = combineSimRunOutputs(simOutputs,runOutputs,runOutputs_single)
                outputs = simOutputs;
                outputs.Genotype = [simOutputs.Genotype_apps simOutputs.Genotype_serv simOutputs.Genotype_os simOutputs.Genotype_dr];
                %outputs.numRuns = 1; %only display one run
                outputs.minStd4Curve = runOutputs_single.minStd4Curve;
                outputs.minStd5Curve=runOutputs_single.minStd5Curve;
                outputs.numStates=runOutputs_single.numStates;
                outputs.Nei_sol4 = runOutputs.Nei_sol4;
                outputs.Nei_sol5 = runOutputs.Nei_sol5;
                outputs.Nei_sol1 = 0;
                outputs.Nei_sol2 = 0;
                outputs.Nei_sol3 = 0;
                outputs.nodesDiversity4 = runOutputs.nodesDiversity4;
                outputs.nodesMean4 = runOutputs.nodesMean4;
                outputs.nodesStd4 = runOutputs.nodesStd4;
                outputs.updateStopDisplay4 = runOutputs.updateStopDisplay4;
                outputs.nodesStd5 = runOutputs.nodesStd5;
                outputs.updateStopDisplay5 = runOutputs.updateStopDisplay5;
                outputs.lastStoppedStdMatrix = runOutputs.lastStoppedStdMatrix;
                outputs.lastStoppedStdMatrix2 = runOutputs.lastStoppedStdMatrix2;
                outputs.Nei_indDynamic = runOutputs.Nei_indDynamic;
                outputs.sumOfStates(1,:,:) = runOutputs.sumOfStates;            
                outputs.nodeStateAtTimeStr = runOutputs.nodeStateAtTimeStr;
                outputs.nodeStateAtTimeNumber = runOutputs.nodeStateAtTimeNumber;
                outputs.SingleLocus_sol4 = runOutputs.SingleLocus_sol4;
                outputs.SingleLocus_sol5 = runOutputs.SingleLocus_sol5;               
                outputs.GenoSim_sol4 = runOutputs.GenoSim_sol4;
                outputs.GenoSim_sol5 = runOutputs.GenoSim_sol5;
                outputs.beta =  runOutputs.beta;
            end
            
            
            
            
            
            
            
            
            
            
            
            
            
            
            
            
            %***************Load
            function loadSettings1SimMain(fid, handles)
                %main text
                mainheader1 = fgetl(fid);
                mainheader2 = fgetl(fid);
                mainheader3 = fgetl(fid);
                %Multiple simulations    
                subheader1 = fgetl(fid);
                text = fgetl(fid);
                numRunsV = fgetl(fid);
                text = fgetl(fid);
                numTimeStepsV = fgetl(fid);
                text = fgetl(fid);
                autoSaveOutV = fgetl(fid);
                text = fgetl(fid);
                autoSaveGrV = fgetl(fid);
                text = fgetl(fid);
                binmodeV = fgetl(fid);
                text = fgetl(fid);%added 27th April 2016
                fileSaveOptionV = fgetl(fid);%added 27th April 2016
                text = fgetl(fid);%added 27th April 2016
                fileSavePrecisionV = fgetl(fid);%added 27th April 2016
                text = fgetl(fid);%added 27th April 2016
                blockLengthV = fgetl(fid);%added 27th April 2016
                %simulation includes
                subheader2 = fgetl(fid);
                text = fgetl(fid);
                datasetEnableV = fgetl(fid);
                text = fgetl(fid); %added 31st March 2016
                datasetStaticEnableV = fgetl(fid); %added 31st March 2016
                text = fgetl(fid);
                networkModelEnableV = fgetl(fid);
                text = fgetl(fid);
                attackModelEnableV = fgetl(fid);
                text = fgetl(fid);
                exploitEnableV = fgetl(fid);
                text = fgetl(fid);
                algEnableV = fgetl(fid);
                text = fgetl(fid);
                networkPerfEnableV = fgetl(fid);
                %simulation selections    
                subheader3 = fgetl(fid);
                text = fgetl(fid);
                inputDatasetV = fgetl(fid);
                text = fgetl(fid);
                alignDataV = fgetl(fid);
                text = fgetl(fid); %added 31st March 2016
                inputStaticDatasetV = fgetl(fid); %added 31st March 2016
                text = fgetl(fid);
                networkModelV = fgetl(fid);
                networkModelS = fgetl(fid);
                text = fgetl(fid);
                attackModelV = fgetl(fid);
                attackModelS = fgetl(fid);
                text = fgetl(fid);
                exploitTypeV = fgetl(fid);
                exploitTypeS = fgetl(fid);
                text = fgetl(fid);
                algorithmV = fgetl(fid);
                algorithmS = fgetl(fid);
                text = fgetl(fid);
                networkPerformanceV = fgetl(fid);
                networkPerformanceS = fgetl(fid);
                %graphical output
                subheader4 = fgetl(fid);
                text = fgetl(fid);
                graphTypeV = fgetl(fid);
                graphTypeS = fgetl(fid);
                %save to handles
                set(handles.editNumRuns,'Value',str2num(numRunsV));
                set(handles.editNumRuns,'String',numRunsV);
                set(handles.editNumTimeSteps,'Value',str2num(numTimeStepsV));
                set(handles.editNumTimeSteps,'String',numTimeStepsV);
                set(handles.checkboxSaveOutput,'Value',str2num(autoSaveOutV));
                %set(handles.checkboxSaveOutput,'String',autoSaveOutV);
                set(handles.checkboxSaveGraphs,'Value',str2num(autoSaveGrV));
                %set(handles.checkboxSaveGraphs,'String',autoSaveGrV);
                set(handles.checkboxbinmode,'Value',str2num(binmodeV));
                %set(handles.checkboxbinmode,'String',binmodeV);
                set(handles.popupmenuFileSaveOption,'Value',str2num(fileSaveOptionV));%added 27th April 2016
                set(handles.popupmenuFileSavePrecision,'Value',str2num(fileSavePrecisionV));%added 27th April 2016
                set(handles.editBlockLength,'Value',str2num(blockLengthV)); %added 27th April 2016
                set(handles.editBlockLength,'String',blockLengthV);%added 27th April 2016
                %simulation includes
                set(handles.checkboxDatasetEnable,'Value',str2num(datasetEnableV));
                set(handles.checkboxStaticDatasetEnable,'Value',str2num(datasetStaticEnableV));%added 31st March 2016
                set(handles.checkboxNetworkModelEnable,'Value',str2num(networkModelEnableV));
                set(handles.checkboxAttackModelEnable,'Value',str2num(attackModelEnableV));
                set(handles.checkboxExploitEnable,'Value',str2num(exploitEnableV));
                set(handles.checkboxAlgEnable,'Value',str2num(algEnableV));
                set(handles.checkboxNetworkPerfEnable,'Value',str2num(networkPerfEnableV));
                %simulation selections
                set(handles.popupmenuInputDataset,'Value',str2num(inputDatasetV)); 
                set(handles.popupmenuStaticDataset,'Value',str2num(inputStaticDatasetV)); %added 31st March 2016
                set(handles.checkboxAlignData,'Value',str2num(alignDataV));
                set(handles.popupmenuNetworkModel,'Value',str2num(networkModelV));
                set(handles.popupmenuAttackModel,'Value',str2num(attackModelV));
                set(handles.popupmenuExploitType,'Value',str2num(exploitTypeV));
                set(handles.popupmenuAlgorithm,'Value',str2num(algorithmV));
                set(handles.popupmenuNetworkPerformance,'Value',str2num(networkPerformanceV));
                %graphical output
                set(handles.popupmenuGraphType,'Value',str2num(graphTypeV));          
            end
              function loadSettings1SimStatic(fid, mainhandles) %added 7th Jan 2017
                %static text
                networkheader1 = fgetl(fid);
                networkheader2 = fgetl(fid);
                networkheader3 = fgetl(fid);
                %Sub header   
                dataSetS = str2num(fgetl(fid));
                switch(dataSetS)
                    case 11 %random assignment
                        text = fgetl(fid);
                        editNumNodesV = fgetl(fid);
                        text = fgetl(fid);
                        for i = 1:4
                        editVarArrayV(i) = str2num(fgetl(fid));
                        end
                        handles = guidata(mainhandles.randomAssignmentParametersFig); %load data
                        set(handles.editNumNodes,'Value',str2num(editNumNodesV));
                        set(handles.editNumNodes,'String',editNumNodesV);
                        set(handles.editVarArray,'Value',editVarArrayV);
                        set(handles.editVarArray,'String',['[' num2str(editVarArrayV) ']']);
                        guidata(mainhandles.randomAssignmentParametersFig, handles); %save data
                end
              end
            function loadSettings1SimNetwork(fid, mainhandles)
                %network text
                networkheader1 = fgetl(fid);
                networkheader2 = fgetl(fid);
                networkheader3 = fgetl(fid);
                %Sub header   
                networkModelS = fgetl(fid);
                switch(networkModelS)
                    case 'Random Encounter'
                        text = fgetl(fid);
                        editRandomEncounterProbV = fgetl(fid);
                        handles = guidata(mainhandles.networkModelParameters_randomEncounterFig); %load data
                        set(handles.editRandomEncounterProb,'Value',str2num(editRandomEncounterProbV));
                        set(handles.editRandomEncounterProb,'String',editRandomEncounterProbV);
                        guidata(mainhandles.networkModelParameters_randomEncounterFig, handles); %save data
                    case 'Wireless Bluetooth'
                        text = fgetl(fid);
                        editDensityV = fgetl(fid);
                        text = fgetl(fid);
                        editAvVelocityV = fgetl(fid);
                        text = fgetl(fid);
                        popupmenuTxRadiusV = fgetl(fid);
                        text = fgetl(fid);
                        editTVirusV = fgetl(fid);
                        handles = guidata(mainhandles.networkModelParameters_wirelessBluetoothFig); %load data
                        set(handles.editDensity,'Value',str2num(editDensityV));
                        set(handles.editDensity,'String',editDensityV);
                        set(handles.editAvVelocity,'Value',str2num(editAvVelocityV));
                        set(handles.editAvVelocity,'String',editAvVelocityV);
                        set(handles.popupmenuTxRadius,'Value',str2num(popupmenuTxRadiusV));
                        set(handles.editTVirus,'Value',str2num(editTVirusV));
                        set(handles.editTVirus,'String',editTVirusV);
                        guidata(mainhandles.networkModelParameters_wirelessBluetoothFig, handles); %save data
                    case 'Random Waypoint'
                        text = fgetl(fid);
                        checkboxPlotWayPointsV = fgetl(fid);
                        text = fgetl(fid);
                        popupmenuRWPTxRadiusV = fgetl(fid);
                        popupmenuRWPTxRadiusS = fgetl(fid);
                        text = fgetl(fid);
                        popupmenuRWPEncounterChoiceV = fgetl(fid);
                        popupmenuRWPEncounterChoiceS = fgetl(fid);
                        text = fgetl(fid);
                        editRWPPosXminV = fgetl(fid);
                        text = fgetl(fid);
                        editRWPPosXmaxV = fgetl(fid);
                        text = fgetl(fid);
                        editRWPPosYminV = fgetl(fid);
                        text = fgetl(fid);
                        editRWPPosYmaxV = fgetl(fid);
                        text = fgetl(fid);
                        editRWPSpeedminV = fgetl(fid);
                        text = fgetl(fid);
                        editRWPSpeedmaxV = fgetl(fid);
                        text = fgetl(fid);
                        editRWPPauseminV = fgetl(fid);
                        text = fgetl(fid);
                        editRWPPausemaxV = fgetl(fid);
                        text = fgetl(fid);
                        editRWPWalkTimeminV = fgetl(fid);
                        text = fgetl(fid);
                        editRWPWalkTimemaxV = fgetl(fid);
                        text = fgetl(fid);
                        editRWPDirectionminV = fgetl(fid);
                        text = fgetl(fid);
                        editRWPDirectionmaxV = fgetl(fid);
                        handles = guidata(mainhandles.networkModelParameters_randomWaypointFig); %load data
                        set(handles.checkboxPlotWayPoints,'Value',str2num(checkboxPlotWayPointsV));
                        set(handles.popupmenuRWPTxRadius,'Value',str2num(popupmenuRWPTxRadiusV));
                        set(handles.popupmenuRWPEncounterChoice,'Value',str2num(popupmenuRWPEncounterChoiceV));
                        set(handles.editRWPPosXmin,'Value',str2num(editRWPPosXminV));
                        set(handles.editRWPPosXmax,'Value',str2num(editRWPPosXmaxV));
                        set(handles.editRWPPosYmin,'Value',str2num(editRWPPosYminV));
                        set(handles.editRWPPosYmax,'Value',str2num(editRWPPosYmaxV));
                        set(handles.editRWPSpeedmin,'Value',str2num(editRWPSpeedminV));
                        set(handles.editRWPSpeedmax,'Value',str2num(editRWPSpeedmaxV));
                        set(handles.editRWPPausemin,'Value',str2num(editRWPPauseminV));
                        set(handles.editRWPPausemax,'Value',str2num(editRWPPausemaxV));
                        set(handles.editRWPWalkTimemin,'Value',str2num(editRWPWalkTimeminV));
                        set(handles.editRWPWalkTimemax,'Value',str2num(editRWPWalkTimemaxV));
                        set(handles.editRWPDirectionmin,'Value',str2num(editRWPDirectionminV));
                        set(handles.editRWPDirectionmax,'Value',str2num(editRWPDirectionmaxV));
                        set(handles.editRWPPosXmin,'String',editRWPPosXminV);
                        set(handles.editRWPPosXmax,'String',editRWPPosXmaxV);
                        set(handles.editRWPPosYmin,'String',editRWPPosYminV);
                        set(handles.editRWPPosYmax,'String',editRWPPosYmaxV);
                        set(handles.editRWPSpeedmin,'String',editRWPSpeedminV);
                        set(handles.editRWPSpeedmax,'String',editRWPSpeedmaxV);
                        set(handles.editRWPPausemin,'String',editRWPPauseminV);
                        set(handles.editRWPPausemax,'String',editRWPPausemaxV);
                        set(handles.editRWPWalkTimemin,'String',editRWPWalkTimeminV);
                        set(handles.editRWPWalkTimemax,'String',editRWPWalkTimemaxV);
                        set(handles.editRWPDirectionmin,'String',editRWPDirectionminV);
                        set(handles.editRWPDirectionmax,'String',editRWPDirectionmaxV);
                        guidata(mainhandles.networkModelParameters_randomWaypointFig, handles); %save data
                    otherwise
                        fprintf(fid, '%s\n', '***Error: Invalid network model!!!!');
                end
            end
            function loadSettings1SimAttack(fid, mainhandles)
                %attack text
                attackheader1 = fgetl(fid);
                attackheader2 = fgetl(fid);
                attackheader3 = fgetl(fid);
                %sub header   
                attackModelS = fgetl(fid);
                switch(attackModelS)
                    case 'SI'
                    text = fgetl(fid);    
                    handles = guidata(mainhandles.attackModelParameters_SIFig); %load data
                    %no data to save
                    guidata(mainhandles.attackModelParameters_SIFig, handles); %save data
                    case 'SIR'
                    text = fgetl(fid);
                    editRecovProbV = fgetl(fid);
                    handles = guidata(mainhandles.attackModelParameters_SIRFig); %load data
                    set(handles.editRecovProb,'Value',str2num(editRecovProbV));
                    set(handles.editRecovProb,'String',editRecovProbV);
                    guidata(mainhandles.attackModelParameters_SIRFig, handles); %save data
                    case 'SEIR'
                   % text = fgetl(fid);
                   % popupmenuVirusTypeV = fgetl(fid);   
                   % text = fgetl(fid);
                   % editUserProbV = fgetl(fid);
                   % text = fgetl(fid);
                   % editLatTimeV = fgetl(fid);
                   % text = fgetl(fid);
                   % popupmenuLatUnitsV = fgetl(fid);
                    text = fgetl(fid);
                    editRecoveryProbV = fgetl(fid) ;     
                    handles = guidata(mainhandles.attackModelParameters_SEIRFig); %load data
                   % set(handles.popupmenuVirusType,'Value',str2num(popupmenuVirusTypeV));
                   % set(handles.editPei1,'Value',str2num(editUserProbV));
                   % set(handles.editPei1,'String',editUserProbV);
                   % set(handles.editPei2,'Value',str2num(editLatTimeV));
                   % set(handles.editPei2,'String',editLatTimeV);
                   % set(handles.popupmenuLatUnits,'Value',str2num(popupmenuLatUnitsV));
                    set(handles.editRecoveryProb,'Value',str2num(editRecoveryProbV));
                    set(handles.editRecoveryProb,'String',editRecoveryProbV);
                    guidata(mainhandles.attackModelParameters_SEIRFig, handles); %save data
                    case 'SEPTICOX'
                    %text = fgetl(fid);
                    %popupmenuVirusTypeV = fgetl(fid) ;
                    %text = fgetl(fid);
                    %editPei1V = fgetl(fid);
                    %text = fgetl(fid);
                    %editPei2V = fgetl(fid);
                    %text = fgetl(fid);
                    %popupmenuLUnitsV = fgetl(fid);
                    text = fgetl(fid);
                    editDevicesOfflineV = fgetl(fid);
                    text = fgetl(fid);
                    editPos1V = fgetl(fid);
                    text = fgetl(fid);
                    editPos2V = fgetl(fid);
                    text = fgetl(fid);
                    editPso1V = fgetl(fid);
                    text = fgetl(fid);
                    editPso2V = fgetl(fid);
                    text = fgetl(fid);
                    editPexV = fgetl(fid);
                    text = fgetl(fid);
                    editPxeV = fgetl(fid);
                    text = fgetl(fid);
                    editPto1V = fgetl(fid);
                    text = fgetl(fid);
                    editPto2V = fgetl(fid);
                    text = fgetl(fid);
                    editPspV = fgetl(fid);
                    text = fgetl(fid);
                    editPetV = fgetl(fid);
                    text = fgetl(fid);
                    editPitV = fgetl(fid);
                    text = fgetl(fid);
                    editPtsV = fgetl(fid);
                    text = fgetl(fid);
                    editPtpV = fgetl(fid);
                    text = fgetl(fid);
                    editPicV = fgetl(fid);
                    text = fgetl(fid);
                    editPciV = fgetl(fid);
                    text = fgetl(fid);
                    editFreqOnV = fgetl(fid);
                    text = fgetl(fid);
                    editPeakOnV = fgetl(fid);
                    text = fgetl(fid);
                    editSpreadOnV = fgetl(fid);
                    text = fgetl(fid);
                    editAmpOnV = fgetl(fid);
                    text = fgetl(fid);
                    editFreqOffV = fgetl(fid);
                    text = fgetl(fid);
                    editPeakOffV = fgetl(fid);
                    text = fgetl(fid);
                    editSpreadOffV = fgetl(fid);
                    text = fgetl(fid);
                    editAmpOffV = fgetl(fid);
                    text = fgetl(fid);
                    checkboxUsePeriodicV = fgetl(fid);
                    text = fgetl(fid);
                    checkboxPlotPeriodV = fgetl(fid);   
                    handles = guidata(mainhandles.attackModelParameters_SEPTICOXFig); %load data
                    %set(handles.popupmenuVirusType,'Value',str2num(popupmenuVirusTypeV));
                    %set(handles.editPei1,'Value',str2num(editPei1V));
                    %set(handles.editPei1,'String',editPei1V);
                    %set(handles.editPei2,'Value',str2num(editPei2V));
                    %set(handles.editPei2,'String',editPei2V);
                    %set(handles.popupmenuLUnits,'Value',str2num(popupmenuLUnitsV));
                    set(handles.editDevicesOffline,'Value',str2num(editDevicesOfflineV));
                    set(handles.editDevicesOffline,'String',editDevicesOfflineV);
                    set(handles.editPos1,'Value',str2num(editPos1V));
                    set(handles.editPos1,'String',editPos1V);
                    set(handles.editPos2,'Value',str2num(editPos2V));
                    set(handles.editPos2,'String',editPos2V);
                    set(handles.editPso1,'Value',str2num(editPso1V));
                    set(handles.editPso1,'String',editPso1V);
                    set(handles.editPso2,'Value',str2num(editPso2V));
                    set(handles.editPso2,'String',editPso2V);
                    set(handles.editPex,'Value',str2num(editPexV));
                    set(handles.editPex,'String',editPexV);
                    set(handles.editPxe,'Value',str2num(editPxeV));
                    set(handles.editPxe,'String',editPxeV);
                    set(handles.editPto1,'Value',str2num(editPto1V));
                    set(handles.editPto1,'String',editPto1V);
                    set(handles.editPto2,'Value',str2num(editPto2V));
                    set(handles.editPto2,'String',editPto2V);
                    set(handles.editPsp,'Value',str2num(editPspV));
                    set(handles.editPsp,'String',editPspV);
                    set(handles.editPet,'Value',str2num(editPetV));
                    set(handles.editPet,'String',editPetV);
                    set(handles.editPit,'Value',str2num(editPitV));
                    set(handles.editPit,'String',editPitV);
                    set(handles.editPts,'Value',str2num(editPtsV));
                    set(handles.editPts,'String',editPtsV);
                    set(handles.editPtp,'Value',str2num(editPtpV));
                    set(handles.editPtp,'String',editPtpV);
                    set(handles.editPic,'Value',str2num(editPicV));
                    set(handles.editPic,'String',editPicV);
                    set(handles.editPci,'Value',str2num(editPciV));
                    set(handles.editPci,'String',editPciV);
                    set(handles.editFreqOn,'Value',str2num(editFreqOnV));
                    set(handles.editFreqOn,'String',editFreqOnV);
                    set(handles.editPeakOn,'Value',str2num(editPeakOnV));
                    set(handles.editPeakOn,'String',editPeakOnV);
                    set(handles.editSpreadOn,'Value',str2num(editSpreadOnV));
                    set(handles.editSpreadOn,'String',editSpreadOnV);
                    set(handles.editAmpOn,'Value',str2num(editAmpOnV));
                    set(handles.editAmpOn,'String',editAmpOnV);
                    set(handles.editFreqOff,'Value',str2num(editFreqOffV));
                    set(handles.editFreqOff,'String',editFreqOffV);
                    set(handles.editPeakOff,'Value',str2num(editPeakOffV));
                    set(handles.editPeakOff,'String',editPeakOffV);
                    set(handles.editSpreadOff,'Value',str2num(editSpreadOffV));
                    set(handles.editSpreadOff,'String',editSpreadOffV);
                    set(handles.editAmpOff,'Value',str2num(editAmpOffV));
                    set(handles.editAmpOff,'String',editAmpOffV);
                    set(handles.checkboxUsePeriodic,'Value',str2num(checkboxUsePeriodicV));
                    set(handles.checkboxUsePeriodic,'String',checkboxUsePeriodicV);
                    set(handles.checkboxPlotPeriod,'Value',str2num(checkboxPlotPeriodV));
                    set(handles.checkboxPlotPeriod,'String',checkboxPlotPeriodV);
                    guidata(mainhandles.attackModelParameters_SEPTICOXFig, handles); %save data
                    otherwise
                        fprintf(fid, '%s\n', '***Error: Invalid attack model!!!!');
                end
            end
             function loadSettings1SimExploit(fid, mainhandles)
                %exploit text
                algheader1 = fgetl(fid);
                algheader2 = fgetl(fid);
                algheader3 = fgetl(fid);
                text = fgetl(fid);
                editExploitAppV = fgetl(fid);
                text = fgetl(fid);
                editExploitServV = fgetl(fid);
                text = fgetl(fid);
                editExploitOSV = fgetl(fid);
                text = fgetl(fid);
                editExploitDrV = fgetl(fid);
                text = fgetl(fid); %added 7th Jan 2017
                popupmenuMalwareTypeV = fgetl(fid); %added 7th Jan 2017
                text = fgetl(fid);
                popupmenuVirusTypeV = fgetl(fid);
                text = fgetl(fid);
                editPei1V = fgetl(fid);
                text = fgetl(fid);
                editPei2V = fgetl(fid);
                text = fgetl(fid);
                popupmenuLatUnitsV = fgetl(fid);
                text = fgetl(fid);
                editInfectionTimeV = fgetl(fid);
                text = fgetl(fid);
                editNumInitialInfectedV = fgetl(fid);
                text = fgetl(fid);
                popupmenuInfectTimeUnitsV = fgetl(fid);
                text = fgetl(fid);
                editTimeToTxVirusV = fgetl(fid);
                text = fgetl(fid);
                popupmenuTransmitTimeUnitsV = fgetl(fid);
                text = fgetl(fid);
                editTimeToTxGenotypeV = fgetl(fid);
                text = fgetl(fid);
                popupmenuTimeToTxGenotypeUnitsV = fgetl(fid);
                text = fgetl(fid);
                editTimeOutPeriodTxGenotypeV = fgetl(fid);
                text = fgetl(fid);
                popupmenuTimeOutPeriodTxGenotypeUnitsV = fgetl(fid);
                
                handles = guidata(mainhandles.editExploitFig); %load data
                set(handles.editExploitApp,'String',editExploitAppV);
                set(handles.editExploitApp,'Value',str2num(editExploitAppV));
                set(handles.editExploitServ,'String',editExploitServV);
                set(handles.editExploitServ,'Value',str2num(editExploitServV));
                set(handles.editExploitOS,'String',editExploitOSV);
                set(handles.editExploitOS,'Value',str2num(editExploitOSV));
                set(handles.editExploitDr,'String',editExploitDrV);
                set(handles.editExploitDr,'Value',str2num(editExploitDrV));
                set(handles.popupmenuMalwareType,'Value',str2num(popupmenuMalwareTypeV)); %added 7th Jan 2017
                set(handles.editPei1,'String',editPei1V);
                set(handles.editPei1,'Value',str2num(editPei1V));
                set(handles.editPei2,'String',editPei2V);
                set(handles.editPei2,'Value',str2num(editPei2V));
                set(handles.editInfectionTime,'String',editInfectionTimeV);
                set(handles.editInfectionTime,'Value',str2num(editInfectionTimeV));
                set(handles.editNumInitialInfected,'String',editNumInitialInfectedV);
                set(handles.editNumInitialInfected,'Value',str2num(editNumInitialInfectedV));
                set(handles.popupmenuVirusType,'Value',str2num(popupmenuVirusTypeV));
                set(handles.popupmenuLatUnits,'Value',str2num(popupmenuLatUnitsV));
                set(handles.popupmenuInfectTimeUnits,'Value',str2num(popupmenuInfectTimeUnitsV));
                set(handles.editTimeToTxVirus,'String',editTimeToTxVirusV);
                set(handles.editTimeToTxVirus,'Value',str2num(editTimeToTxVirusV));
                set(handles.popupmenuTransmitTimeUnits,'Value',str2num(popupmenuTransmitTimeUnitsV));
                set(handles.editTimeToTxGenotype,'String',editTimeToTxGenotypeV);
                set(handles.editTimeToTxGenotype,'Value',str2num(editTimeToTxGenotypeV));
                set(handles.popupmenuTransmitTimeUnits,'Value',str2num(popupmenuTransmitTimeUnitsV));
                set(handles.editTimeOutPeriodTxGenotype,'String',editTimeOutPeriodTxGenotypeV);
                set(handles.editTimeOutPeriodTxGenotype,'Value',str2num(editTimeOutPeriodTxGenotypeV));
                set(handles.popupmenuTimeOutPeriodTxGenotypeUnits,'Value',str2num(popupmenuTimeOutPeriodTxGenotypeUnitsV));                
                guidata(mainhandles.editExploitFig, handles); %save data
             end
            function loadSettings1SimAlg(fid, mainhandles)
                    %network text
                algheader1 = fgetl(fid);
                algheader2 = fgetl(fid);
                algheader3 = fgetl(fid);
                diversityAlgS = fgetl(fid);
                switch(diversityAlgS)
                    case 'Favourability Score'
                    %select algorithms    
                    subheader1 = fgetl(fid);
                    text = fgetl(fid);
                    radiobuttonSelectAlg1To5 = fgetl(fid);
                    text = fgetl(fid);
                    radiobuttonSelectAlg4And5 = fgetl(fid);
                    text = fgetl(fid);
                    radiobuttonSelectAlg4Only = fgetl(fid);
                    %type of encounters
                    subheader2 = fgetl(fid);
                    text = fgetl(fid);
                    encounterTypeV = fgetl(fid);
                    encounterTypeS = fgetl(fid);
                    text = fgetl(fid);
                    minEncountersV = fgetl(fid);
                    text = fgetl(fid);
                    maxEncountersV = fgetl(fid);
                    text = fgetl(fid);
                    encounterProbV = fgetl(fid);
                    %weighting factors
                    subheader2 = fgetl(fid);
                    text = fgetl(fid);
                    diversityWeightV = fgetl(fid);
                    text = fgetl(fid);
                    commWeightV = fgetl(fid);
                    text = fgetl(fid);
                    indWeightV = fgetl(fid);
                    text = fgetl(fid);
                    effWeightV = fgetl(fid);
                    %dynamic weighting factors
                    subheader3 = fgetl(fid);
                    text = fgetl(fid);
                    dynDivV = fgetl(fid);
                    text = fgetl(fid);
                    dynComV = fgetl(fid);
                    text = fgetl(fid);
                    dynIndV = fgetl(fid);
                    text = fgetl(fid);
                    dynQualV = fgetl(fid);
                    text = fgetl(fid);
                    weightFileDiv = fgetl(fid);
                    text = fgetl(fid);
                    weightFileCom = fgetl(fid);
                    text = fgetl(fid);
                    weightFileInd = fgetl(fid);
                    text = fgetl(fid);
                    weightFileQual = fgetl(fid);
                    %stopping the genotype update process
                    subheader3 = fgetl(fid);
                    text = fgetl(fid);
                    updateSampleSizeV = fgetl(fid);
                    text = fgetl(fid);
                    alg4UpdateTypeV = fgetl(fid);
                    text = fgetl(fid);
                    alg4MinstdV = fgetl(fid);
                    text = fgetl(fid);
                    alg4MaxStdV = fgetl(fid);
                    text = fgetl(fid);
                    alg5UpdateTypeV = fgetl(fid);
                    text = fgetl(fid);
                    alg5MinStdV = fgetl(fid);
                    text = fgetl(fid);
                    alg5MaxStdV = fgetl(fid);
                    text = fgetl(fid);
                    editAlg4MinstdE = fgetl(fid);
                    text = fgetl(fid);
                    editAlg4MaxStdE = fgetl(fid);
                    text = fgetl(fid);
                    editAlg5MinStdE = fgetl(fid);
                    text = fgetl(fid);
                    editAlg5MaxStdE = fgetl(fid);
                    %Dynamic desirability changes
                    subheader4 = fgetl(fid);
                    text = fgetl(fid);
                    changeCommDesV = fgetl(fid);
                    text = fgetl(fid);
                    commTime1V = fgetl(fid);
                    text = fgetl(fid);
                    checkboxCommTime1V = fgetl(fid);
                    text = fgetl(fid);
                    commTime2V = fgetl(fid);
                    text = fgetl(fid);
                    checkboxCommTime2V = fgetl(fid);
                    text = fgetl(fid);
                    changeIndDesV = fgetl(fid);
                    text = fgetl(fid);
                    indTime1V = fgetl(fid);
                    text = fgetl(fid);
                    checkboxIndTime1V = fgetl(fid);
                    text = fgetl(fid);
                    indTime2V = fgetl(fid);
                    text = fgetl(fid);
                    checkboxIndTime2V = fgetl(fid);
                    text = fgetl(fid);
                    changeEffDesV = fgetl(fid);
                    text = fgetl(fid);
                    effTime1V = fgetl(fid);
                    text = fgetl(fid);
                    checkboxEffTime1V = fgetl(fid);
                    text = fgetl(fid);
                    effTime2V = fgetl(fid);
                    text = fgetl(fid);
                    checkboxEffTime2V = fgetl(fid)  ;                 
                    %Dynamic desirability changes from file
                    subheader5 = fgetl(fid);
                    text = fgetl(fid);
                    checkComDesV = fgetl(fid);
                    text = fgetl(fid);
                    checkIndDesV = fgetl(fid);
                    text = fgetl(fid);
                    checkQualDesV = fgetl(fid);
                    text = fgetl(fid);
                    editDesComFilenameS = fgetl(fid);
                    text = fgetl(fid);
                    editDesIndFilenameS = fgetl(fid);
                    text = fgetl(fid);
                    editDesQualFilenameS = fgetl(fid);
                    handles = guidata(mainhandles.diversityModelParameters_FavScoreFig); %load data
                    %select algorithms
                    set(handles.radiobuttonSelectAlg1To5,'Value',str2num(radiobuttonSelectAlg1To5));
                    set(handles.radiobuttonSelectAlg4And5,'Value',str2num(radiobuttonSelectAlg4And5));
                    set(handles.radiobuttonSelectAlg4Only,'Value',str2num(radiobuttonSelectAlg4Only));
                    %type of encounters
                    set(handles.popupmenuEncounterType,'Value',str2num(encounterTypeV));
                    set(handles.editMinEncounters,'Value',str2num(minEncountersV));
                    set(handles.editMinEncounters,'String',minEncountersV);
                    set(handles.editMaxEncounters,'Value',str2num(maxEncountersV));
                    set(handles.editMaxEncounters,'String',maxEncountersV);
                    set(handles.editEncounterProb,'Value',str2num(encounterProbV));
                    set(handles.editEncounterProb,'String',encounterProbV);
                    %weighting factors
                    set(handles.editDiversityWeight,'Value',str2num(diversityWeightV));
                    set(handles.editDiversityWeight,'String',diversityWeightV);
                    set(handles.editCommWeight,'Value',str2num(commWeightV));
                    set(handles.editCommWeight,'String',commWeightV);
                    set(handles.editIndWeight,'Value',str2num(indWeightV));
                    set(handles.editIndWeight,'String',indWeightV);
                    set(handles.editEffWeight,'Value',str2num(effWeightV));  
                    set(handles.editEffWeight,'String',effWeightV); 
                    %dynamic weighting factors
                    set(handles.checkboxDynDiv,'Value',str2num(dynDivV));
                    set(handles.checkboxDynCom,'Value',str2num(dynComV));
                    set(handles.checkboxDynInd,'Value',str2num(dynIndV));
                    set(handles.checkboxDynQual,'Value',str2num(dynQualV));
                    set(handles.editWeightFileDiv,'String',weightFileDiv);
                    set(handles.editWeightFileCom,'String',weightFileCom);
                    set(handles.editWeightFileInd,'String',weightFileInd);
                    set(handles.editWeightFileQual,'String',weightFileQual);
                    %stopping the genotype update process
                    set(handles.editUpdateSampleSize,'Value',str2num(updateSampleSizeV));
                    set(handles.editUpdateSampleSize,'String',updateSampleSizeV);
                    set(handles.popupmenuAlg4UpdateType,'Value',str2num(alg4UpdateTypeV));
                    set(handles.editAlg4Minstd,'Value',str2num(alg4MinstdV));
                    set(handles.editAlg4Minstd,'String',alg4MinstdV);
                    set(handles.editAlg4MaxStd,'Value',str2num(alg4MaxStdV));
                    set(handles.editAlg4MaxStd,'String',alg4MaxStdV);
                    set(handles.popupmenuAlg5UpdateType,'Value',str2num(alg5UpdateTypeV));
                    set(handles.editAlg5MinStd,'Value',str2num(alg5MinStdV));
                    set(handles.editAlg5MinStd,'String',alg5MinStdV);
                    set(handles.editAlg5MaxStd,'Value',str2num(alg5MaxStdV));
                    set(handles.editAlg5MaxStd,'String',alg5MaxStdV);
                    set(handles.editAlg4Minstd, 'Enable',editAlg4MinstdE);
                    set(handles.editAlg4MaxStd, 'Enable',editAlg4MaxStdE);
                    set(handles.editAlg5MinStd, 'Enable',editAlg5MinStdE);
                    set(handles.editAlg5MaxStd, 'Enable',editAlg5MaxStdE);
                    %Dynamic desirability changes
                    set(handles.checkboxChangeCommDes,'Value',str2num(changeCommDesV));
                    set(handles.editCommTime1,'Value',str2num(commTime1V));
                    set(handles.editCommTime1,'String',commTime1V);
                    set(handles.checkboxCommTime1,'Value',str2num(checkboxCommTime1V));
                    set(handles.editCommTime2,'Value',str2num(commTime2V));
                    set(handles.editCommTime2,'String',commTime2V);
                    set(handles.checkboxCommTime2,'Value',str2num(checkboxCommTime2V));
                    set(handles.checkboxChangeIndDes,'Value',str2num(changeIndDesV));
                    set(handles.editIndTime1,'Value',str2num(indTime1V));
                    set(handles.editIndTime1,'String',indTime1V);
                    set(handles.checkboxIndTime1,'Value',str2num(checkboxIndTime1V));
                    set(handles.editIndTime2,'Value',str2num(indTime2V));
                    set(handles.editIndTime2,'String',indTime2V);
                    set(handles.checkboxIndTime2,'Value',str2num(checkboxIndTime2V));
                    set(handles.checkboxChangeEff,'Value',str2num(changeEffDesV));
                    set(handles.editEffTime1,'Value',str2num(effTime1V));
                    set(handles.editEffTime1,'String',effTime1V);
                    set(handles.checkboxEffTime1,'Value',str2num(checkboxEffTime1V));
                    set(handles.editEffTime2,'Value',str2num(effTime2V));
                    set(handles.editEffTime2,'String',effTime2V);
                    set(handles.checkboxEffTime2,'Value',str2num(checkboxEffTime2V));
                    %Dynamic desirability changes from file
                    set(handles.checkComDes,'Value',str2num(checkComDesV));
                    set(handles.checkIndDes,'Value',str2num(checkIndDesV));
                    set(handles.checkQualDes,'Value',str2num(checkQualDesV));
                    set(handles.editDesComFilename,'String',editDesComFilenameS);
                    set(handles.editDesIndFilename,'String',editDesIndFilenameS);
                    set(handles.editDesQualFilename,'String',editDesQualFilenameS);
                    guidata(mainhandles.diversityModelParameters_FavScoreFig, handles); %save data
                    case 'Static Diversity' %added 31st March 2016
                    %subheader1 = fgetl(fid)
                    text = fgetl(fid);
                    otherwise
                        fprintf(fid, '%s\n', '***Error: Invalid alg model!!!!');
                end  
            end
            function loadSettings1SimPerf(fid, mainhandles)
                %perf text
                perfheader1 = fgetl(fid);
                perfheader2 = fgetl(fid);
                perfheader3 = fgetl(fid); 
                performanceS = fgetl(fid);
            end %func
            function loadSettings1SimSaveParameters(fid, mainhandles)
                %perf text
                header = fgetl(fid);
                header = fgetl(fid);
                header = fgetl(fid) ;
                header = fgetl(fid) ;
                saveParametersS = fgetl(fid);
               set(mainhandles.editSaveParameters,'Value',str2num(saveParametersS));
               set(mainhandles.editSaveParameters,'String',saveParametersS);
               % guidata(mainhandles); %save data
            end %func
            %***********************************************************************************
            %Save, load, delete, and copy temporary output files
            %************************************************************************************
            function saveOutputsTempFile(outputsTempFileNames,simNumber,outputs,handles)
             binmode=get(handles.checkboxbinmode,'Value'); %this is set for each setting but should be applied to all sims really
             fileSavePrecision= get(handles.popupmenuFileSavePrecision, 'Value')% if 3 save as mat files
             
            %save temp output data
            %check if text or binary mode selected
            if and(binmode == 1, fileSavePrecision < 3)
                %save as binary
                fidtemp = fopen(char(outputsTempFileNames(simNumber)), 'w');
                %save all data
                saveOutputs1SimBinary(fidtemp, outputs);
                %close file
                fclose(fidtemp); 
            elseif and(binmode == 1, fileSavePrecision == 3) % NOT USED not sure if will use this now as will have to save seperate files if done in blocks
                save(char(outputsTempFileNames(simNumber)),'outputs') %added 29th April 2016
            else
                %save as text
                fidtemp = fopen(char(outputsTempFileNames(simNumber)), 'w');
                fprintf(fidtemp, '%s\n', '%Dynamic Diversity Algorithm');
                simText = ['%Matlab outputs for simulation ' num2str(simNumber)];
                fprintf(fidtemp, '%s\n', simText);
                %save all data
                saveOutputs1Sim(fidtemp, outputs);
                %close file
                fclose(fidtemp);
            end;              
            end
            function outputs = loadOutputsTempFile(outputsTempFileNames,simNumber,binmode)
            %load temp output data
            %check if text or binary mode selected
            fidtemp = fopen(char(outputsTempFileNames(simNumber)), 'r')
            %check to see if file exists
            if fidtemp == -1
                disp('sim file doesnt exist yet')
                %outputs = null Need a function here to write zeros to
                %outputs = GUIClass.defaultOutputs();
            else
                %load all data
                if binmode == 1
                    outputs = loadOutputs1SimBinary(fidtemp);
                else
                headertext1 = fgetl(fidtemp);
                headertext2 = fgetl(fidtemp);
                outputs = loadOutputs1Sim(fidtemp);
                end
            %close file
            fclose(fidtemp);
            end;              
            end
            function deleteAllOutputsTempFiles()
                %Delete all outputs temp files
                %get max possible number of temp files
                numFiles = GUIClass.getMaxNumSims();
                %generate filename list
                directorynameOut = GUIClass.getDirectoryNameOut();
                extension = 'bin';
                outputsTempFileNames = GUIClass.genOutputsTempFileNames(directorynameOut,numFiles,extension);
                %delete all temp files with bin extension
                for outputsSimNumber = 1 : numFiles
                    GUIClass.deleteOutputsTempFile(outputsTempFileNames,outputsSimNumber);
                end
                extension = 'txt';
                outputsTempFileNames = GUIClass.genOutputsTempFileNames(directorynameOut,numFiles,extension);
                %delete all temp files with txt extension
                for outputsSimNumber = 1 : numFiles
                    GUIClass.deleteOutputsTempFile(outputsTempFileNames,outputsSimNumber);
                end
            end
            function deleteOutputsTempFile(outputsTempFileNames,outputsSimNumber)
                %Delete a single settings temp file
                fid1=fopen(char(outputsTempFileNames(outputsSimNumber)),'r');
                if fid1 >= 3
                    fclose(fid1);
                    delete(char(outputsTempFileNames(outputsSimNumber)));
                end
            end
            %NO LONGER USED
            %function copyOutputs(outputsTempFileNames,editSaveSimAs,simNumber,extension)
            %    %copy temp outputs to sim outputs
            %    outputSimFileName = [editSaveSimAs '_sim' num2str(simNumber) '.' extension];
            %    copyfile(char(outputsTempFileNames(simNumber)),char(outputSimFileName));
            %end
            function copyAllOutputs(handles)
                %get file name for saving - save as multiple files
                directorynameOut = GUIClass.getDirectoryNameOut()
                extension = 'bin';
                outputsTempFileNames = GUIClass.genOutputsTempFileNames(directorynameOut,GUIClass.getMaxNumSims(),extension)
                editSaveSimAs = get(handles.editSaveSimAs,'String');
                numSims = get(handles.main.popupmenuNumSims,'Value');
                %for simNumber = 1: numSims
                 %   GUIClass.copyOutputs(outputsTempFileNames,editSaveSimAs,simNumber,extension);
                %end
                source = [directorynameOut '*.bin'] 
                destination = [editSaveSimAs '\']
                copyfile(source,destination);
            end
            function openOutputs(outputsTempFileNames,editOpenSimAs,simNumber,extension)
                %copy temp outputs to sim outputs
                outputSimFileName = [editOpenSimAs '_sim' num2str(simNumber) '.' extension];
                copyfile(char(outputSimFileName),char(outputsTempFileNames(simNumber)));
            end
            function openOutputSettings(settingsTempFileNames,editOpenSetAs,simNumber)
                %copy temp outputs to sim outputs
                outputSetFileName = [editOpenSetAs '_set' num2str(simNumber) '.txt'];
                copyfile(char(outputSetFileName),char(settingsTempFileNames(simNumber)));
            end
            function numFiles = countOutputFiles(editOpenSimAs,extension)
                countFiles = 0;
                for simNumber = 1:GUIClass.getMaxNumSims()
                    outputSimFileName = [editOpenSimAs '_sim' num2str(simNumber) '.' extension];
                    fidTemp = fopen(outputSimFileName, 'r');
                    if fidTemp < 0
                    %file doesn't exist
                    else
                    countFiles = countFiles+1;
                    fclose(fidTemp);
                    end;
                end; %end for loop i
                numFiles = countFiles;
            end
            function openAllOutputs(handles)
                %get file name for saving - save as multiple files
                directorynameOut = GUIClass.getDirectoryNameOut()
                directorynameSettings = GUIClass.getDirectoryNameSettings()
                extension = 'bin';
                binmode = 1;
                outputName = '\outputs';
                settingsName = '\settings';
                outputsTempFileNames = GUIClass.genOutputsTempFileNames(directorynameOut,GUIClass.getMaxNumSims(),extension)
                settingsTempFileNames = GUIClass.genSettingsTempFileNames(directorynameSettings,GUIClass.getMaxNumSims())
                editOpenSimDir = get(handles.editOpenSimAs,'String'); 
                editOpenSimAs = [editOpenSimDir outputName];
                editOpenSetAs = [editOpenSimDir settingsName];
                numFiles = GUIClass.countOutputFiles(editOpenSimAs,extension)
                for simNumber = 1:numFiles
                    GUIClass.openOutputs(outputsTempFileNames,editOpenSimAs,simNumber,extension);
                    GUIClass.openOutputSettings(settingsTempFileNames,editOpenSetAs,simNumber);
                end
                %set sim number and graphical display
                set(handles.main.popupmenuNumSims, 'Value', numFiles)
                %set list of available simulations
                popupmenuList = GUIClass.genSimList(numFiles);
                set(handles.main.popupmenuGraphicalOutput, 'String', popupmenuList);                
                set(handles.main.popupmenuGraphicalOutput, 'Value', numFiles)
                %load data and display
                outputs = GUIClass.loadOutputsTempFile(outputsTempFileNames,numFiles,binmode);
                
                %load temp settings for sim
                GUIClass.loadSettingsTempFileMultipleGUIs(settingsTempFileNames,numFiles,handles.main) ;
                %select sim settings on display
                GUIClass.selectSimSettings(handles.main,numFiles);
                
                displayGUIGraph(outputs,handles.main); 
                displayAttackGraph(outputs,handles.main);
            end
            %***********************************************************************************
            %Manual output files
            %***********************************************************************************
            function numFiles = countSettingsFiles(editLoadSettings)
                countFiles = 0;
                for simNumber = 1:GUIClass.getMaxNumSims()
                    settingsFileName = [editLoadSettings '_set' num2str(simNumber) '.txt'];
                    fidTemp = fopen(settingsFileName, 'r');
                    if fidTemp < 0
                    %file doesn't exist
                    else
                    countFiles = countFiles+1;
                    fclose(fidTemp);
                    end;
                end; %end for loop i
                numFiles = countFiles;
            end
            %***********************************************************************************
            %Auto save output files
            %************************************************************************************
            function autoSaveOutputs(directorynameOut,folderName,simNumber,outputsTempFileNames,settingsTempFileNames,outputsSimFileNames,settingsSimFileNames,directoryName,outputs,handles)
            %save data and settings in seperate files
            if get(handles.checkboxSaveOutput, 'Value')==1 || get(handles.checkboxSaveGraphs, 'Value')==1
                mkdir(directorynameOut,folderName);
            end;
            if get(handles.checkboxSaveOutput, 'Value')==1
                %save data as seperate simulation files
                copyfile(char(outputsTempFileNames(simNumber)),char(outputsSimFileNames(simNumber)));
                %save copy of settings files
                copyfile(char(settingsTempFileNames(simNumber)),char(settingsSimFileNames(simNumber)));  
            end
            if get(handles.checkboxSaveGraphs, 'Value')==1
                %create and save seperate graphs
                simText = ['sim' num2str(simNumber)];
                autosaveGraphs(directoryName, simText, outputs)
            end;
            end
            %added 2nd May 2016 to save and compress data files
            function autoSaveAndCompressOutputs(directorynameOut,folderName,simNumber,outputsTempFileNames,settingsTempFileNames,outputsSimFileNames,settingsSimFileNames,directoryName,outputs,handles)
            %save data and settings in seperate files
            if get(handles.checkboxSaveOutput, 'Value')==1 || get(handles.checkboxSaveGraphs, 'Value')==1
                mkdir(directorynameOut,folderName);
            end;
            if get(handles.checkboxSaveOutput, 'Value')==1
                %save and compress data
                copyfileNamesO = [directorynameOut '*.bin'];
                zipFileNameO = [directoryName 'outputs'];
                entrynamesO = zip(zipFileNameO,copyfileNamesO);
                %save and copress settings
                copyfileNamesS = ['settings\' 'settingsTemp*.txt'];
                zipFileNameS = [directoryName 'settings'];
                %entrynamesS = zip(zipFileNameS,copyfileNamesS) 
                entrynamesS = zip(zipFileNameS,settingsTempFileNames) ;
            end
            if get(handles.checkboxSaveGraphs, 'Value')==1
                %create and save seperate graphs
                %simText = ['sim' num2str(simNumber)];
                %autosaveGraphs(directoryName, simText, outputs)
                copyfileNamesG = {[directorynameOut '*.emf'],[directorynameOut '*.fig']};
                zipFileNameG = [directoryName 'graphs'];
                entrynamesG = zip(zipFileNameG,copyfileNamesG) ;
            end;
            end
            function autoSaveOutputs_settingsFiles(directorynameOut,folderName,settingsTempFileNames,settingsSimFileNames,simNumber,handles)
            %save data and settings in seperate files
            if get(handles.checkboxSaveOutput, 'Value')==1 || get(handles.checkboxSaveGraphs, 'Value')==1
                mkdir(directorynameOut,folderName);
            end;
            if get(handles.checkboxSaveOutput, 'Value')==1
                %save data as seperate simulation files
                %check for autosave outputs
                disp('copying settings file....') 
                copyfile(char(settingsTempFileNames(simNumber)),char(settingsSimFileNames(simNumber)));
            end
            end%func
            %seperate run files
            function autoSaveOutputs_sepRunFiles(directorynameOut,folderName,Tempfilenames_runFiles,simfilenames_runFiles,handles)
            %save data and settings in seperate files
            if get(handles.checkboxSaveOutput, 'Value')==1 || get(handles.checkboxSaveGraphs, 'Value')==1
                mkdir(directorynameOut,folderName);
            end;
            if get(handles.checkboxSaveOutput, 'Value')==1 %check for autosave outputs
                %save data as seperate run files
                disp('copying run files....') 
                numFiles = size(Tempfilenames_runFiles,2)
                for i = 1:numFiles
                    copyfile(char(Tempfilenames_runFiles(i)),char(simfilenames_runFiles(i)));
                end 
            end
            if get(handles.checkboxSaveGraphs, 'Value')==1
                %create and save seperate graphs
                %simText = ['sim' num2str(simNumber)];
                %autosaveGraphs(directoryName, simText, outputs)
            end;
            end
            %seperate header files
            function autoSaveOutputs_simHFiles(Tempfilenames_simHeaders,simfilenames_simHeaders,settingsTempFileNames,settingsSimFileNames,simNumber,handles)
            %save data and settings in seperate files
            if get(handles.checkboxSaveOutput, 'Value')==1
                %save data as seperate simulation files
                %check for autosave outputs
                disp('copying sim header and settings files....') 
                t = Tempfilenames_simHeaders
                h = simfilenames_simHeaders
                copyfile(char(Tempfilenames_simHeaders),char(simfilenames_simHeaders));
                copyfile(char(settingsTempFileNames(simNumber)),char(settingsSimFileNames(simNumber)));
            end

            end            
            
            

            
            %************************************************************************************
            % Default Outputs
            %************************************************************************************
            function outputs = defaultOutputs()
                %default outputs for either initialisation purposes or when
                %output files do not exist, the outputs will default to
                %these null values
                outputs.numNodes = 0;   outputs.numRuns = 0;   outputs.numTimeSteps = 0;
                outputs.len_apps = 0;   outputs.len_serv = 0;  outputs.len_os = 0;           outputs.len_dr = 0;
                outputs.CD_apps = 0;    outputs.CD_serv = 0;   outputs.CD_os = 0;            outputs.CD_dr = 0;
                outputs.ChangeDesirabilitiesFlag = 0;          outputs.tChange1 = 0;         outputs.tChange2 = 0;
                outputs.CD_apps2 = 0;   outputs.CD_serv2 = 0;  outputs.CD_os2 = 0;           outputs.CD_dr2 = 0;
                outputs.E_apps = 0;     outputs.E_serv = 0;    outputs.E_os = 0;             outputs.E_dr = 0;
                outputs.ChangeEffFlag = 0;                     outputs.tChangeEff1 = 0;      outputs.tChangeEff2 = 0;
                outputs.E_apps2 = 0;    outputs.E_serv2 = 0;   outputs.E_os2 = 0;            outputs.E_dr2 = 0;
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                outputs.Genotype_apps(1,1) = 0;                outputs.Genotype_serv(1,1) = 0;
                outputs.Genotype_os(1,1) = 0;                  outputs.Genotype_dr(1,1) = 0;
                outputs.Genotype = [outputs.Genotype_apps outputs.Genotype_serv outputs.Genotype_os outputs.Genotype_dr];
                outputs.textdata(1,1) = 0;
                outputs.ID_apps(1,1) = 0;                      outputs.ID_serv(1,1) = 0;
                outputs.ID_os(1,1) = 0;                        outputs.ID_dr(1,1) = 0;
                outputs.ID = [outputs.ID_apps outputs.ID_serv outputs.ID_os outputs.ID_dr]';
                outputs.ChangeIndDesirabilitiesFlag = 0;       outputs.tChangeInd1 = 0;      outputs.tChangeInd2 = 0;
                outputs.ID_apps2(1,1) = 0;                     outputs.ID_serv2(1,1) = 0;
                outputs.ID_os2(1,1) = 0;                       outputs.ID_dr2(1,1) = 0;
                outputs.ID2 = [outputs.ID_apps2 outputs.ID_serv2 outputs.ID_os2 outputs.ID_dr2]';
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                outputs.Nei_sol1=0;     outputs.Nei_sol2=0;    outputs.Nei_sol3=0;           outputs.Nei_sol4=0;          outputs.Nei_sol5=0;
                outputs.nodesDiversity4=0;                     outputs.nodesMean4=0;         outputs.nodesStd4=0;
                outputs.updateStopDisplay4(1,1) = 0;           outputs.minstd4Tolerance = 0; outputs.maxstd4Tolerance = 0;outputs.minStd4Curve =0;
                outputs.updateStopDisplay5(1,1) = 0;           outputs.nodesStd5=0;          outputs.minstd5Tolerance = 0;outputs.maxstd5Tolerance = 0;
                outputs.minStd5Curve =0;
                outputs.turnOnSol1 = 0;   outputs.turnOnSol2 = 0; outputs.turnOnSol3 = 0;    outputs.turnOnSol4 = 0;      outputs.turnOnSol5 = 0;
                outputs.lastStoppedStdMatrix=0;                outputs.lastStoppedStdMatrix2=0;
                outputs.stdMargin = 0;    outputs.Nei_indDynamic = 0;
                outputs.singleStopMode = 0;                    outputs.restartMode = 0;
                outputs.ChangeOwnIndDesirabilitiesFlag = 0;
            end
            %************************************************************************************
            %Default Handles for initialisation
            %************************************************************************************
            function defaultHandlesMain(handles)
                %initialisation parameters
                set(handles.editNumRuns,'Value',1);
                set(handles.editNumRuns,'String','1');
                set(handles.editNumTimeSteps,'Value',200);
                set(handles.editNumTimeSteps,'String','200');
                set(handles.checkboxSaveOutput,'Value',0);
                set(handles.checkboxSaveGraphs,'Value',0);
                set(handles.checkboxbinmode,'Value',1);
                set(handles.popupmenuFileSaveOption, 'Value',3);
                set(handles.popupmenuFileSavePrecision, 'Value',2);
                set(handles.editBlockLength, 'Value', 100);
                set(handles.editBlockLength, 'String', '100');
                %simulation includes
                set(handles.checkboxDatasetEnable,'Value',0);
                set(handles.checkboxStaticDatasetEnable,'Value',1);
                set(handles.checkboxNetworkModelEnable,'Value',1);
                set(handles.checkboxAttackModelEnable,'Value',1);
                set(handles.checkboxExploitEnable,'Value',1);
                set(handles.checkboxAlgEnable,'Value',1);
                set(handles.checkboxNetworkPerfEnable,'Value',1);
                %simulation selections
                set(handles.popupmenuInputDataset,'Value',1);   
                set(handles.checkboxAlignData,'Value',1);
                set(handles.popupmenuStaticDataset,'Value',11); %random assignment
                set(handles.popupmenuNetworkModel,'Value',1);
                set(handles.popupmenuAttackModel,'Value',1);
                set(handles.popupmenuExploitType,'Value',1);
                set(handles.popupmenuAlgorithm,'Value',1);
                set(handles.popupmenuNetworkPerformance,'Value',1);
                %graphical output
                set(handles.popupmenuGraphType,'Value',1);
            end
            function defaultHandlesRandomAssignmentParameters(handles)
                 %initialisation parameters for static diversity random
                 %assignment of variants
                set(handles.editNumNodes,'String','1000')
                set(handles.editNumNodes,'Value',1000)
                set(handles.editVarArray,'String','[4 4 4 4]')
                set(handles.editVarArray,'Value',[4 4 4 4])
            end
            function defaultHandlesAlg_FavScore(handles)
                %initialisation parameters
                set(handles.popupmenuSettings,'Value',1)
                set(handles.editOldSimSetting,'String','1')
                %weighting factors
                set(handles.editDiversityWeight,'String','1')
                set(handles.editDiversityWeight,'Value',1)
                set(handles.editCommWeight,'String','0')
                set(handles.editCommWeight,'Value',0)
                set(handles.editIndWeight,'String','0')
                set(handles.editIndWeight,'Value',0)
                set(handles.editEffWeight,'String','0')
                set(handles.editEffWeight,'Value',0)
                %select algorithms
                %type of encounters
                set(handles.editEncounterProb,'String','0.5')
                set(handles.editEncounterProb,'Value',0.5)
                set(handles.popupmenuEncounterType,'Value',1)
                set(handles.editMinEncounters,'String','5')
                set(handles.editMinEncounters,'Value',5)
                set(handles.editMaxEncounters,'String','10')
                set(handles.editMaxEncounters,'Value',10)
                %stopping the genotype update process
                set(handles.editUpdateSampleSize,'String','1')
                set(handles.editUpdateSampleSize,'Value',1)
                set(handles.popupmenuAlg4UpdateType,'Value',1);
                set(handles.popupmenuAlg5UpdateType,'Value',1);
                set(handles.editAlg4Minstd,'String','0')
                set(handles.editAlg4Minstd,'Value',0)
                set(handles.editAlg4MaxStd,'String','0')
                set(handles.editAlg4MaxStd,'Value', 0)
                set(handles.editAlg5MinStd,'String','0')
                set(handles.editAlg5MinStd,'Value', 0)
                set(handles.editAlg5MaxStd,'String','0')
                set(handles.editAlg5MaxStd,'Value',0)
                set(handles.editAlg4Minstd, 'Enable', 'off');
                set(handles.editAlg4MaxStd, 'Enable', 'off');
                set(handles.editAlg5MinStd, 'Enable', 'off');
                set(handles.editAlg5MaxStd, 'Enable', 'off');
                %desirability changes
                set(handles.editCommTime1,'String','0')
                set(handles.editCommTime1,'Value', 0)
                set(handles.editCommTime2,'String','0')
                set(handles.editCommTime2,'Value', 0)
                set(handles.editIndTime1,'String','0')
                set(handles.editIndTime1,'Value', 0)
                set(handles.editIndTime2,'String','0')
                set(handles.editIndTime2,'Value', 0)
                set(handles.editEffTime1,'String','0')
                set(handles.editEffTime1,'Value', 0)
                set(handles.editEffTime2,'String', '0')
                set(handles.editEffTime2,'Value', 0)
                %graph output
            end
            function defaultHandlesNetwork_randomEncounter(handles)
                %initialisation parameters
                set(handles.popupmenuSettings,'Value',1)
                set(handles.editOldSimSetting,'String','1')
                set(handles.editRandomEncounterProb,'Value',0.5);
                set(handles.editRandomEncounterProb,'String','0.5');
            end
            function defaultHandlesNetwork_wirelessBluetooth(handles)
                %initialisation parameters
                set(handles.popupmenuSettings,'Value',1)
                set(handles.editOldSimSetting,'String','1')
                set(handles.editDensity,'Value',0.005);
                set(handles.editDensity,'String','0.005');
                set(handles.editAvVelocity,'Value',10);
                set(handles.editAvVelocity,'String','10');
                set(handles.popupmenuTxRadius,'Value',2);
                set(handles.editTVirus,'Value',5);
                set(handles.editTVirus,'String','5');
            end
            function defaultHandlesNetwork_randomWaypoint(handles)
                %initialisation parameters
                set(handles.popupmenuSettings,'Value',1)
                set(handles.editOldSimSetting,'String','1')
                set(handles.popupmenuRWPTxRadius,'Value',2)
                set(handles.popupmenuRWPEncounterChoice,'Value',2)
                set(handles.editRWPPosXmin,'Value',10);
                set(handles.editRWPPosXmin,'String','10');
                set(handles.editRWPPosXmax,'Value',300);
                set(handles.editRWPPosXmax,'String','300');
                set(handles.editRWPPosYmin,'Value',10);
                set(handles.editRWPPosYmin,'String','10');
                set(handles.editRWPPosYmax,'Value',300);
                set(handles.editRWPPosYmax,'String','300');
                set(handles.editRWPSpeedmin,'Value',0.2);
                set(handles.editRWPSpeedmin,'String','0.2');
                set(handles.editRWPSpeedmax,'Value',2.2);
                set(handles.editRWPSpeedmax,'String','2.2');
                set(handles.editRWPPausemin,'Value',0);
                set(handles.editRWPPausemin,'String','0');
                set(handles.editRWPPausemax,'Value',1);
                set(handles.editRWPPausemax,'String','1');
                set(handles.editRWPWalkTimemin,'Value',2.00);
                set(handles.editRWPWalkTimemin,'String','2.00');
                set(handles.editRWPWalkTimemax,'Value',6.00);  
                set(handles.editRWPWalkTimemax,'String','6.00');
                set(handles.editRWPDirectionmin,'Value',-180);
                set(handles.editRWPDirectionmin,'String','-180');
                set(handles.editRWPDirectionmax,'Value',180); 
                set(handles.editRWPDirectionmax,'String',180);
            end
            function defaultHandlesAttack_SI(handles)
                %initialisation parameters
                set(handles.popupmenuSettings,'Value',1)
                set(handles.editOldSimSetting,'String','1')
            end
            function defaultHandlesAttack_SIR(handles)
                %initialisation parameters
                set(handles.popupmenuSettings,'Value',1)
                set(handles.editOldSimSetting,'String','1')
                set(handles.editRecovProb,'Value',0.0025);
                set(handles.editRecovProb,'String','0.0025');
                set(handles.editRecovTime,'Value',1);
                set(handles.editRecovTime,'String','1');
            end
            function defaultHandlesAttack_SEIR(handles)
               %initialisation parameters
                set(handles.popupmenuSettings,'Value',1)
                set(handles.editOldSimSetting,'String','1')
               %set(handles.popupmenuVirusType,'Value',1);
               %set(handles.editPei1,'Value',0.0025);
               %set(handles.editPei1,'String','0.0025');
               %set(handles.editPei2,'Value',100);
               %set(handles.editPei2,'String','100');
               %set(handles.popupmenuLatUnits,'Value',1);
               set(handles.editRecoveryProb,'Value',0.0025);
               set(handles.editRecoveryProb,'String','0.0025');
            end
            function defaultHandlesAttack_SEPTICOX(handles)
               %initialisation parameters
                set(handles.popupmenuSettings,'Value',1)
                set(handles.editOldSimSetting,'String','1')
               %data taken from the mini project paper - class 3 users
              [Pso1 Pso2 Pos1 Pos2 Pxe Pex Pei1 Pei2 Psp Pts Pet Pit Pic Pci Ptp Pto1 Pto2] = StateMachineClass.getSEPTICOXProbList_class3Users();   
              [periodOn meanOn stdOn ampOn periodOff meanOff stdOff ampOff] = StateMachineClass.getSEPTICOXProbList_PeriodicList();
               %set(handles.popupmenuVirusType,'Value',1);
               %set(handles.editPei1,'Value',Pei1);
               %set(handles.editPei1,'String',num2str(Pei1));
               %set(handles.editPei2,'Value',100);
               %set(handles.editPei2,'String',num2str(100));
               %set(handles.popupmenuLUnits,'Value',1);
               set(handles.editDevicesOffline,'Value',0);
               set(handles.editDevicesOffline,'String','0');
               set(handles.editPos1,'Value',Pos1);
               set(handles.editPos1,'String',num2str(Pos1));
               set(handles.editPos2,'Value',Pos2);
               set(handles.editPos2,'String',num2str(Pos2));
               set(handles.editPso1,'Value',Pso1);
               set(handles.editPso1,'String',num2str(Pso1));
               set(handles.editPso2,'Value',Pso2);
               set(handles.editPso2,'String',num2str(Pso2));
               set(handles.editPex,'Value',Pex);
               set(handles.editPex,'String',num2str(Pex));
               set(handles.editPxe,'Value',Pxe);
               set(handles.editPxe,'String',num2str(Pxe));
               set(handles.editPto1,'Value',Pto1);
               set(handles.editPto1,'String',num2str(Pto1));
               set(handles.editPto2,'Value',Pto2);
               set(handles.editPto2,'String',num2str(Pto2));
               set(handles.editPsp,'Value',Psp);
               set(handles.editPsp,'String',num2str(Psp));
               set(handles.editPet,'Value',Pet);
               set(handles.editPet,'String',num2str(Pet));
               set(handles.editPit,'Value',Pit);
               set(handles.editPit,'String',num2str(Pit));
               set(handles.editPts,'Value',Pts);
               set(handles.editPts,'String',num2str(Pts));
               set(handles.editPtp,'Value',Ptp);
               set(handles.editPtp,'String',num2str(Ptp));
               set(handles.editPic,'Value',Pic);
               set(handles.editPic,'String',num2str(Pic));
               set(handles.editPci,'Value',Pci);
               set(handles.editPci,'String',num2str(Pci));
               set(handles.editFreqOn,'Value',periodOn);
               set(handles.editFreqOn,'String',num2str(periodOn));
               set(handles.editPeakOn,'Value',meanOn);
               set(handles.editPeakOn,'String',num2str(meanOn));
               set(handles.editSpreadOn,'Value',stdOn);
               set(handles.editSpreadOn,'String',num2str(stdOn));
               set(handles.editAmpOn,'Value',ampOn);
               set(handles.editAmpOn,'String',num2str(ampOn));
               set(handles.editFreqOff,'Value',periodOff);
               set(handles.editFreqOff,'String',num2str(periodOff));
               set(handles.editPeakOff,'Value',meanOff);
               set(handles.editPeakOff,'String',num2str(meanOff));
               set(handles.editSpreadOff,'Value',stdOff);
               set(handles.editSpreadOff,'String',num2str(stdOff));
               set(handles.editAmpOff,'Value',ampOff);
               set(handles.editAmpOff,'String',num2str(ampOff));
               set(handles.checkboxUsePeriodic,'Value',0);
               set(handles.checkboxPlotPeriod,'Value',0);
            end
            function defaultHandlesEditExploit(handles)
               set(handles.editExploitApp,'Value',[0 0 0 0 0 0 0 0]);
               set(handles.editExploitApp,'String','[0 0 0 0 0 0 0 0]');
               set(handles.editExploitServ,'Value',[0 0 0 0 0 0 0 0]);
               set(handles.editExploitServ,'String','[0 0 0 0 0 0 0 0]');
               set(handles.editExploitOS,'Value',[1 0]);
               set(handles.editExploitOS,'String','[1 0]');
               set(handles.editExploitDr,'Value',[0 0 0 0 0 0 0 0]);
               set(handles.editExploitDr,'String','[0 0 0 0 0 0 0 0]');
               set(handles.popupmenuMalwareType,'Value',2);
               set(handles.editPei1,'Value',0.5);
               set(handles.editPei1,'String','0.5');
               set(handles.editPei2,'Value',10);
               set(handles.editPei2,'String','10');
               set(handles.editInfectionTime,'Value',1);
               set(handles.editInfectionTime,'String','1');
               set(handles.editTimeToTxVirus,'Value',10);
               set(handles.editTimeToTxVirus,'String','10');
               set(handles.editNumInitialInfected,'Value',1);
               set(handles.editNumInitialInfected,'String','1');
               set(handles.popupmenuVirusType,'Value',1);
               set(handles.popupmenuLatUnits,'Value',1);
               set(handles.popupmenuInfectTimeUnits,'Value',1);  
            end
            %************************************************************************************
            %Simulation controls
            %************************************************************************************
            function popupmenuList = genSimList(numSims)
            %create list of available simulations
            for i = 1 : numSims
                popupmenuListCell(i) = cellstr(int2str(i)); %create cell string
            end
            popupmenuList = char(popupmenuListCell); %convert to string array
            end
            %************************************************************************************
            %Retrive/convert GUI information
            %************************************************************************************
            function attackModelS = getAttackModelString(attackModelNumber)
                %convert attack model number to the correct string
                    attackModelString = {'SI' 'SIR' 'SEIR' 'SEPTICOX'};
                    attackModelS = char(attackModelString(attackModelNumber)');
            end
            function exploitModelS = getExploitModelString(exploitModelNumber)
                %convert attack model number to the correct string
                    exploitModelString = {'Exploit sent with genotype'};
                    exploitModelS = char(exploitModelString(exploitModelNumber)');
            end
            function networkModelS = getNetworkModelString(networkModelNumber)
                %convert network model number to the correct string
                    networkModelString = {'Random Encounter' 'Wireless Bluetooth' 'Random Waypoint'};
                    networkModelS = char(networkModelString(networkModelNumber)');
            end
            function algModelS = getAlgModelString(algModelNumber)
                %convert model number to the correct string
                    algModelString = {'Favourability Score' 'Static Diversity'};
                    algModelS = char(algModelString(algModelNumber)');
            end
            function perfModelS = getPerfModelString(perfModelNumber)
                %convert model number to the correct string
                    perfModelString = {'Nei Only'};
                    perfModelS = char(perfModelString(perfModelNumber)');
            end
            function graphS = getGraphString(graphTypeNumber)
                %convert model number to the correct string
                    graphString = {'Diversity Index' 'Diversity Index Log' 'Individual Diversities' ...
                        'Individual Diversities Log' 'Node 1 mean Alg 4' 'Node 1 standard deviation Alg 4' ...
                        'Node 1 stop/start update Alg 4' 'Node 1 stop/start update Alg 5'};
                    graphS = char(graphString(graphTypeNumber)');
            end
            function encounterS = getEncounterString(encounterTypeNumber)
                %convert model number to the correct string
                    encounterString = {'Fixed number of encounters' 'Stochastic Normal distribution' ...
                        'Stochastic Uniform distribution'};
                    encounterS = char(encounterString(encounterTypeNumber)');
            end
            function rangeTx = getTxRadius(popupmenuTxRadiusSelection)
                switch(popupmenuTxRadiusSelection) 
                    case 1
                        rangeTx = 100;
                    case 2
                        rangeTx = 10;
                    case 3
                        rangeTx = 1;
                end
            end
            function RWPmodeStr = getRWPmodeStr(RWPmodeValue)
                switch(RWPmodeValue)
                    case 1
                        RWPmodeStr = 'Nearest in range';
                    case 2
                        RWPmodeStr = 'Random in range';
                    case 3
                        RWPmodeStr = 'Available in range';
                    case 4
                        RWPmodeStr = 'Available in range with a virus tx time';
                    case 5
                        RWPmodeStr = 'Independent geno-exploit'
                end
            end
            function multiply = convertTimeUnits(TimeUnits)
                switch(TimeUnits) 
                    case 1
                        multiply = 1;
                    case 2
                        multiply = 60;
                    case 3
                        multiply = 60*60;
                end
            end
            function checkMainIncludes(handles, ihandles)               
                ihandles.datasetEnable = get(handles.checkboxDatasetEnable,'Value');
                ihandles.datasetNumber = get(handles.popupmenuInputDataset,'Value');
                ihandles.datasetStaticEnable = get(handles.checkboxStaticDatasetEnable,'Value'); %added 31st March 2016
                ihandles.datasetStaticNumber = get(handles.popupmenuStaticDataset,'Value'); %added 31st March 2016
                ihandles.networkModelEnable = get(handles.checkboxNetworkModelEnable,'Value');
                ihandles.networkModelV = get(handles.popupmenuNetworkModel,'Value');
                ihandles.networkModelS = GUIClass.getNetworkModelString(ihandles.networkModelV);
                ihandles.attackModelEnable = get(handles.checkboxAttackModelEnable,'Value');
                ihandles.attackModelV = get(handles.popupmenuAttackModel,'Value');
                ihandles.attackModelS = GUIClass.getAttackModelString(ihandles.attackModelV);
                ihandles.exploitEnable = get(handles.checkboxExploitEnable,'Value');
                ihandles.exploitTypeV = get(handles.popupmenuExploitType,'Value');
                ihandles.exploitTypeS = GUIClass.getExploitModelString(ihandles.exploitTypeV);
                ihandles.algModelEnable = get(handles.checkboxAlgEnable,'Value');
                ihandles.algModelV = get(handles.popupmenuAlgorithm,'Value');
                ihandles.algModelS = GUIClass.getAlgModelString(ihandles.algModelV);
                ihandles.networkPerfEnable = get(handles.checkboxNetworkPerfEnable,'Value');
                ihandles.perfModelV = get(handles.popupmenuNetworkPerformance,'Value');
                ihandles.perfModelS = GUIClass.getPerfModelString(ihandles.perfModelV);
                ihandles.editSaveParameters = get(handles.editSaveParameters,'Value');
            end
            function [trueNumRuns trueNumTimesteps]  = getSimInfo(handles,ihandles)
            fileSaveOption = get(handles.popupmenuFileSaveOption, 'Value')
            blocklength = get(handles.editBlockLength, 'Value')
            %fileSaveOption %1 current, 2 save run at end, 3 save run in blocks
            if fileSaveOption == 1 %save all runs in one go
                %number of simulations(number of simulation runs)
                ihandles.numRunsPerSim = get(handles.editNumRuns, 'Value');
                %number of runs per simulation(number of time steps per simulation run)
                ihandles.numTimeSteps = get(handles.editNumTimeSteps, 'Value');
                trueNumRuns = ihandles.numRunsPerSim
                trueNumTimesteps = ihandles.numTimeSteps
            elseif fileSaveOption == 2 %save whole run at end
                ihandles.numRunsPerSim = 1;
                ihandles.numTimeSteps = get(handles.editNumTimeSteps, 'Value');
                trueNumRuns = get(handles.editNumRuns, 'Value');
                trueNumTimesteps = ihandles.numTimeSteps
            elseif fileSaveOption == 3 %save run in blocks
                ihandles.numRunsPerSim = 1;
                ihandles.numTimeSteps = blocklength;
                trueNumRuns = get(handles.editNumRuns, 'Value');
                trueNumTimesteps = get(handles.editNumTimeSteps, 'Value')
            end
            end
            %*************************************************************************
            %Pop up menu settings change function
            %*************************************************************************
            function changePopupMenuSettings(hObject,handles)
                %handles can be passed handles or handles.main
               %get old value of selected button
                oldSelectionS=get(handles.editOldSimSetting,'String');
                settingsSimNumber = str2num(oldSelectionS);
                maxNumSims = GUIClass.getMaxNumSims();
                %get save settings info
                directorynameSettings = GUIClass.getDirectoryNameSettings();
                settingsTempFileNames = GUIClass.genSettingsTempFileNames(directorynameSettings,maxNumSims);   
            %retrieve GUI handles  
            diversityModelParameters_FavScoreFigData = guidata(handles.diversityModelParameters_FavScoreFig); %load sub gui data
            networkModelParameters_wirelessBluetoothFigData = guidata(handles.networkModelParameters_wirelessBluetoothFig); %load sub gui data
            networkModelParameters_randomWaypointFigData = guidata(handles.networkModelParameters_randomWaypointFig);
            attackModelParameters_SIFigData = guidata(handles.attackModelParameters_SIFig); %load data
            attackModelParameters_SIRFigData = guidata(handles.attackModelParameters_SIRFig); %load data
            attackModelParameters_SEIRFigData = guidata(handles.attackModelParameters_SEIRFig); %load data
            attackModelParameters_SEPTICOXFigData = guidata(handles.attackModelParameters_SEPTICOXFig); %load data
            editExploitFigData = guidata(handles.editExploitFig); %load data
            %save settings from other GUI's 
            %diversityModelParameters_FavScoreFigData = GUIClass.updateHandlesList(diversityModelParameters_FavScoreFigData,handles) %update handle list
            %GUIClass.saveSettingsTempFile(settingsTempFileNames,settingsSimNumber,diversityModelParameters_FavScoreFigData);
            GUIClass.saveSettingsTempFileMultipleGUIs(settingsTempFileNames,settingsSimNumber,handles)
                %get new value of selected button and save new value
                newSelectionV = get(hObject,'Value');
                set(handles.popupmenuSettings,'Value',newSelectionV);
                set(networkModelParameters_wirelessBluetoothFigData.popupmenuSettings, 'Value', newSelectionV); %Save new value in sub window
                set(networkModelParameters_randomWaypointFigData.popupmenuSettings, 'Value', newSelectionV); %Save new value in sub window
                set(diversityModelParameters_FavScoreFigData.popupmenuSettings, 'Value', newSelectionV); %Save new value in sub window
                set(attackModelParameters_SIFigData.popupmenuSettings, 'Value', newSelectionV); %Save new value in sub window
                set(attackModelParameters_SIRFigData.popupmenuSettings, 'Value', newSelectionV); %Save new value in sub window
                set(attackModelParameters_SEIRFigData.popupmenuSettings, 'Value', newSelectionV); %Save new value in sub window
                set(attackModelParameters_SEPTICOXFigData.popupmenuSettings, 'Value', newSelectionV); %Save new value in sub window
                 set(editExploitFigData.popupmenuSettings, 'Value', newSelectionV); %Save new value in sub window
            %load any previous settings recorded - NEED TO DO THIS FOR SUB GUI TOO
            %GUIClass.loadSettingsTempFileMultipleGUIs(settingsTempFileNames,newSelectionV,diversityModelParameters_FavScoreFigData)%load settings to sub gui
            GUIClass.loadSettingsTempFileMultipleGUIs(settingsTempFileNames,newSelectionV,handles)%load settings
                % Set text to current sim setting value and save changes
                set(handles.editOldSimSetting, 'String', num2str(newSelectionV)); %Save new value in main window
                set(networkModelParameters_wirelessBluetoothFigData.editOldSimSetting, 'String', num2str(newSelectionV));
                set(networkModelParameters_randomWaypointFigData.editOldSimSetting, 'String', num2str(newSelectionV));
                set(diversityModelParameters_FavScoreFigData.editOldSimSetting, 'String', num2str(newSelectionV)); %Save new value in other windows
                set(attackModelParameters_SIFigData.editOldSimSetting, 'String', num2str(newSelectionV));
                set(attackModelParameters_SIRFigData.editOldSimSetting, 'String', num2str(newSelectionV));
                set(attackModelParameters_SEIRFigData.editOldSimSetting, 'String', num2str(newSelectionV));
                set(attackModelParameters_SEPTICOXFigData.editOldSimSetting, 'String', num2str(newSelectionV));
                set(editExploitFigData.editOldSimSetting, 'String', num2str(newSelectionV));
                guidata(gcbo,handles);
                guidata(handles.networkModelParameters_wirelessBluetoothFig, networkModelParameters_wirelessBluetoothFigData); %save data %SUB GUI SAVE
                guidata(handles.networkModelParameters_randomWaypointFig, networkModelParameters_randomWaypointFigData); %save data %SUB GUI SAVE
                guidata(handles.diversityModelParameters_FavScoreFig, diversityModelParameters_FavScoreFigData); %save data %SUB GUI SAVE
                guidata(handles.attackModelParameters_SIFig, attackModelParameters_SIFigData);
                guidata(handles.attackModelParameters_SIRFig, attackModelParameters_SIRFigData);
                guidata(handles.attackModelParameters_SEIRFig, attackModelParameters_SEIRFigData);
                guidata(handles.attackModelParameters_SEPTICOXFig, attackModelParameters_SEPTICOXFigData);
                guidata(handles.editExploitFig, editExploitFigData);
            end
            %*************************************************************************************
            %Manipulate handles
            %*************************************************************************************
            function selectSimSettings(handles,simNumber)
                %select sim settings on display
                %load data
                diversityModelParameters_FavScoreFigData = guidata(handles.diversityModelParameters_FavScoreFig); %load sub gui data
                networkModelParameters_wirelessBluetoothFigData = guidata(handles.networkModelParameters_wirelessBluetoothFig); %load sub gui data
                networkModelParameters_randomWaypointFigData = guidata(handles.networkModelParameters_randomWaypointFig);
                attackModelParameters_SIFigData = guidata(handles.attackModelParameters_SIFig); %load data
                attackModelParameters_SIRFigData = guidata(handles.attackModelParameters_SIRFig); %load data
                attackModelParameters_SEIRFigData = guidata(handles.attackModelParameters_SEIRFig); %load data
                attackModelParameters_SEPTICOXFigData = guidata(handles.attackModelParameters_SEPTICOXFig); %load data
                %change data
                set(handles.popupmenuSettings,'Value', simNumber);
                set(handles.editOldSimSetting,'String', num2str(simNumber));
                set(networkModelParameters_wirelessBluetoothFigData.popupmenuSettings,'Value', simNumber);
                set(networkModelParameters_wirelessBluetoothFigData.editOldSimSetting,'String', num2str(simNumber));
                set(networkModelParameters_randomWaypointFigData.popupmenuSettings,'Value', simNumber);
                set(networkModelParameters_randomWaypointFigData.editOldSimSetting,'String', num2str(simNumber));
                set(diversityModelParameters_FavScoreFigData.popupmenuSettings,'Value', simNumber);
                set(diversityModelParameters_FavScoreFigData.editOldSimSetting,'String', num2str(simNumber));
                set(attackModelParameters_SIFigData.popupmenuSettings,'Value', simNumber);
                set(attackModelParameters_SIFigData.editOldSimSetting,'String', num2str(simNumber));
                set(attackModelParameters_SIRFigData.popupmenuSettings,'Value', simNumber);
                set(attackModelParameters_SIRFigData.editOldSimSetting,'String', num2str(simNumber));
                set(attackModelParameters_SEIRFigData.popupmenuSettings,'Value', simNumber);
                set(attackModelParameters_SEIRFigData.editOldSimSetting,'String', num2str(simNumber));
                set(attackModelParameters_SEPTICOXFigData.popupmenuSettings,'Value', simNumber);
                set(attackModelParameters_SEPTICOXFigData.editOldSimSetting,'String', num2str(simNumber));
                %save data
                %guidata(gcbo,handles); %commented out as not sure if
                %needed couldn't get batch mode to run 8th April 2016
                guidata(handles.networkModelParameters_wirelessBluetoothFig, networkModelParameters_wirelessBluetoothFigData); 
                guidata(handles.networkModelParameters_randomWaypointFig, networkModelParameters_randomWaypointFigData); 
                guidata(handles.diversityModelParameters_FavScoreFig, diversityModelParameters_FavScoreFigData); 
                guidata(handles.attackModelParameters_SIFig, attackModelParameters_SIFigData);
                guidata(handles.attackModelParameters_SIRFig, attackModelParameters_SIRFigData);
                guidata(handles.attackModelParameters_SEIRFig, attackModelParameters_SEIRFigData);
                guidata(handles.attackModelParameters_SEPTICOXFig, attackModelParameters_SEPTICOXFigData);
            end
            function diversityModelParameters_FavScoreFigData = updateHandlesList(diversityModelParameters_FavScoreFigDataOld,handles)
                diversityModelParameters_FavScoreFigData = diversityModelParameters_FavScoreFigDataOld;
                diversityModelParameters_FavScoreFigData.editNumRuns = handles.editNumRuns;
                diversityModelParameters_FavScoreFigData.editNumTimeSteps = handles.editNumTimeSteps;
                diversityModelParameters_FavScoreFigData.popupmenuInputDataset = handles.popupmenuInputDataset;
                diversityModelParameters_FavScoreFigData.checkboxSaveOutput = handles.checkboxSaveOutput;
                diversityModelParameters_FavScoreFigData.checkboxbinmode = handles.checkboxbinmode;
                diversityModelParameters_FavScoreFigData.checkboxSaveOutput = handles.checkboxSaveOutput;
                diversityModelParameters_FavScoreFigData.checkboxSaveGraphs = handles.checkboxSaveGraphs;
                diversityModelParameters_FavScoreFigData.checkboxAlignData = handles.checkboxAlignData;
                diversityModelParameters_FavScoreFigData.popupmenuGraphType = handles.popupmenuGraphType;
                diversityModelParameters_FavScoreFigData.popupmenuGraphicalOutput = handles.popupmenuGraphicalOutput;
                diversityModelParameters_FavScoreFigData.popupmenuNumSims = handles.popupmenuNumSims;
                diversityModelParameters_FavScoreFigData.editSimProgress = handles.editSimProgress;
                diversityModelParameters_FavScoreFigData.axesGraph = handles.axesGraph;
            end
            %update time progress on GUI display
            function UpdateProgress(t,psim,currentSimNumber,handles,ihandles)
                if t == fix(ihandles.numTimeSteps/2)
                    numTotSims = num2str(get(handles.popupmenuNumSims, 'Value'))%total number of simulations
                    prog=num2str(((psim-1)*(100/ihandles.numRunsPerSim)) + (50/ihandles.numRunsPerSim));
                    progressStr = [prog '%  ' num2str(currentSimNumber) ' of ' numTotSims]
                    set(handles.editSimProgress, 'String' ,progressStr);
                    pause(0.001)%time to update screen
                end
                if t == ihandles.numTimeSteps
                    numTotSims = num2str(get(handles.popupmenuNumSims, 'Value'))%total number of simulations
                    prog=num2str(((psim-1)*(100/ihandles.numRunsPerSim)) + (100/ihandles.numRunsPerSim));
                    progressStr = [prog '%  ' num2str(currentSimNumber) ' of ' numTotSims]
                    set(handles.editSimProgress, 'String' ,progressStr);
                    pause(0.001)%time to update screen
                end
            end %func
            %displays time status on Matlab window
            function  simTimeStatus(t, numRuns)
                if t == 1
                    disp ('t at 1')
                    disp (clock)
                end
                if t == 1000
                    disp ('t at 1000')
                    disp (clock)
                end
                if t == 2500
                    disp ('t at 2500')
                    disp (clock)
                end
                if t == 5000
                    disp ('t at 2500')
                    disp (clock)
                end 
                if t == 10000
                    disp ('t at 10000')
                    disp (clock)
                end
                if t == 15000
                    disp ('t at 15000')
                    disp (clock)
                end
                if t == 20000
                    disp ('t at 20000')
                    disp (clock)
                end
                if t == 25000
                    disp ('t at 25000')
                    disp (clock)
                end
                if t == numRuns
                    disp ('t at end sim')
                    disp (clock)
                end
            end %func
            %*************************************************
            % Assign main gui window inputs as outputs
            %************************************************
            function MainWindowAssignOutputs(ihandles,ohandles)
                %from ihandles
                ohandles.numNodes=ihandles.numNodes;
                ohandles.numRunsPerSim=ihandles.numRunsPerSim;
                ohandles.numTimeSteps=ihandles.numTimeSteps;
                %model enables
                ohandles.datasetEnable =ihandles.datasetEnable;
                ohandles.datasetNumber =ihandles.datasetNumber;
                ohandles.datasetStaticEnable =ihandles.datasetStaticEnable;%added 31st March 2016
                ohandles.datasetStaticNumber =ihandles.datasetStaticNumber;  %added 31st March 2016              
                ohandles.networkModelEnable =ihandles.networkModelEnable; 
                ohandles.networkModelS =ihandles.networkModelS;
                ohandles.networkModelV =ihandles.networkModelV;
                ohandles.attackModelEnable =ihandles.attackModelEnable;
                ohandles.attackModelS =ihandles.attackModelS;
                ohandles.attackModelV =ihandles.attackModelV;
                ohandles.exploitEnable=ihandles.exploitEnable;
                ohandles.exploitTypeS=ihandles.exploitTypeS;
                ohandles.exploitTypeV=ihandles.exploitTypeV;
                ohandles.algModelEnable =ihandles.algModelEnable;
                ohandles.algModelS =ihandles.algModelS;
                ohandles.algModelV =ihandles.algModelV;
                ohandles.networkPerfEnable =ihandles.networkPerfEnable;
                ohandles.perfModelS =ihandles.perfModelS;
                ohandles.perfModelV =ihandles.perfModelV;
            end
    end
    
end

