%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Jennifer Jackson 2017                                                        %
% A Biodiversity Approach to Cyber Security                                    %
% PhD Complexity Science                                                       %
% University of Warwick                                                        %
% http://wrap.warwick.ac.uk/98458                                              %
% This software is distributed under the  Creative Commons Attribution License %
% https://creativecommons.org/licenses/by/4.0/                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%function outputs = DynAlgStochastic(handles,currentSimNumber,networkModelEnable,probOfEncounter)
function SingleRun(handles,ihandles,rhandles,ohandles,currentSimNumber,runNumber,trueNumTimesteps,trueNumRuns,directorynameOut)
    lengthCounter=1; %init counter to save data in blocks added 20th April 2016
    fileSaveOption = get(handles.popupmenuFileSaveOption, 'Value');
    fileSavePrecision = get(handles.popupmenuFileSavePrecision, 'Value');
    %file save option check
    if fileSaveOption > 1
        falseRunNumber = 1; %save one run at a time, and data in blocks, so fool into storing only 1 array
    else
        falseRunNumber = runNumber %wait until end of sim before saving out data, store all arrays in matrix
    end
    
    %directorynameOut output directory for temp files
    %initialise run variables
    NodeEncounterClass.nodeEncounterRunInit(ihandles,rhandles)
    NodeEncounterClass.RWPRunInit(ihandles,rhandles)
    StateMachineClass.attackModelRunInit(ihandles,rhandles)
    StateMachineClass.exploitRunInit(ihandles,rhandles)
    DynAlgClass.AlgModelFavScoreRunInit(handles,ihandles,rhandles)%****O.K
    StaticDiversityClass.StaticRunInit(ihandles,rhandles,ohandles)%**********added
    
     %save outputs - open run files added 21st April 2016
 if fileSaveOption == 3 %save data in blocks    
      filenames = GUIClass.getTempRunFileNames(directorynameOut,currentSimNumber+1,runNumber);
      runFileHandles = GUIClass.saveOutputsTempFile_OpenRunFiles(filenames,currentSimNumber+1,runNumber);
 end %type
    
   for t = 1:trueNumTimesteps %ihandles.numTimeSteps
     %********************************************************************
     % Simulation progress
     %*******************************************************************
     %Display time in Matlab window
     %GUIClass.simTimeStatus(t, ihandles.numTimeSteps)
     GUIClass.simTimeStatus(t, trueNumTimesteps)
     %update time progress
     inputsP.numTimeSteps = trueNumTimesteps;
     inputsP.numRunsPerSim = trueNumRuns;
     %time progress needs to have the true number of timesteps and number of
     %runs to be able to indicate progress
     GUIClass.UpdateProgress(t,runNumber,currentSimNumber,handles,inputsP);%GUIClass.UpdateProgress(t,runNumber,currentSimNumber,handles,ihandles)
     %*********************************************************************
     % Start virus outbreak at set time if enabled
     %*********************************************************************
     StateMachineClass.startExploitInfection(t,ihandles,rhandles) %OK not affected
     %*********************************************************************
     % Node encounter function
     %*********************************************************************
     NodeEncounterClass.nodeEncounter(lengthCounter,ihandles,rhandles) %NodeEncounterClass.nodeEncounter(t,ihandles,rhandles) %used for Random Encounter and wireless bluetooth
     NodeEncounterClass.RWPNodeEncounterTloop(t,lengthCounter,ihandles,rhandles) %added lengthCounter 22nd April 2016
     %*********************************************************************
     % Run attack Model if enabled
     %*********************************************************************
     StateMachineClass.attackModelRunTloop(t,lengthCounter,ihandles,rhandles)%will need t checking
     %*********************************************************************
     %run Diversity algorithm
     %*********************************************************************
     %AlgModelFavScoreTloop(t,handles,ihandles,rhandles,ohandles,falseRunNumber)%need t checking updated 20th April 2016
     AlgModelFavScoreTloop(t,lengthCounter,handles,ihandles,rhandles,ohandles,falseRunNumber)%need t checking updated 20th April 2016
     %increment buffer pointer and check sample size - used for measuring
     %rolling mean and std of diversity independent of saving block length
     %StaticDiversityClass.StaticDiversityTLoop(t,handles,ihandles,rhandles,ohandles,falseRunNumber)%need t checking updated 20th April 2016
      StaticDiversityClass.StaticDiversityTLoop(lengthCounter,handles,ihandles,rhandles,ohandles,falseRunNumber)%need t checking updated 20th April 2016
     %save Outputs - save data in blocks
     if or(lengthCounter == ihandles.numTimeSteps,t==trueNumTimesteps) % in this case ihandles.numTimeSteps is the blocklength
        if fileSaveOption == 3 
         %currentSimNumberD=currentSimNumber+1
         %runNumberD=runNumber   
        GUIClass.saveOutputsTempFile_SaveOutputs1RunBinaryData(runFileHandles,rhandles,ohandles,ihandles,handles,lengthCounter,fileSavePrecision);
        
        end %type
        lengthCounter=1;
        %resetParameters
        %runOutputs = runInit(fixLength);
     else
        lengthCounter=lengthCounter+1;
    end
     
    end; %end t
    
    
    %save outputs and close files     
    if fileSaveOption == 3 
        GUIClass.saveOutputsTempFile_SaveOutputs1RunBinaryDataSingleValues(runFileHandles,rhandles,ohandles,ihandles,handles);
        GUIClass.saveOutputsTempFile_CloseRunFiles(runFileHandles);
    end
    %assign outputs
    GUIClass.MainWindowAssignOutputs(ihandles,ohandles);
    DynAlgClass.AlgModelFavScoreAssignOutputs(falseRunNumber,ihandles,rhandles,ohandles); %updated 20th April 2016
    StateMachineClass.AttackModelAssignOutputs(falseRunNumber,ihandles,rhandles,ohandles); %updated 20th April 2016
    NodeEncounterClass.RWPAssignOutputs(ihandles,ohandles);
    
    
    %plot
    PlotWayPointsRWPAttack=0;
    NodeEncounterClass.RWP_animateAttack(ihandles,rhandles,PlotWayPointsRWPAttack);
    
    %display final genotype values
        %genotype inputs
        Genotype_apps = ihandles.Genotype_apps;
        Genotype_serv = ihandles.Genotype_serv;
        Genotype_os = ihandles.Genotype_os;
        Genotype_dr = ihandles.Genotype_dr;
        
        %GenotypeSol4=rhandles.GenotypeSol4
        GenotypeSol4_apps=rhandles.GenotypeSol4_apps;
        GenotypeSol4_serv=rhandles.GenotypeSol4_serv;
        GenotypeSol4_os=rhandles.GenotypeSol4_os;
        GenotypeSol4_dr=rhandles.GenotypeSol4_dr;
        
        %GenotypeSol5=rhandles.GenotypeSol5
        GenotypeSol5_apps=rhandles.GenotypeSol5_apps;
        GenotypeSol5_serv=rhandles.GenotypeSol5_serv;
        GenotypeSol5_os=rhandles.GenotypeSol5_os;
        GenotypeSol5_dr=rhandles.GenotypeSol5_dr;
end

          