%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Jennifer Jackson 2017                                                        %
% A Biodiversity Approach to Cyber Security                                    %
% PhD Complexity Science                                                       %
% University of Warwick                                                        %
% http://wrap.warwick.ac.uk/98458                                              %
% This software is distributed under the  Creative Commons Attribution License %
% https://creativecommons.org/licenses/by/4.0/                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
classdef StateMachineClass
    %List of functions to support state machines for the SI, SIR and
    %SEPTICOX models
    %functions listed here:
    %***********************************************
    %SI State machine specific functions:
    %SIStateMachine = genSIStateMachine(numNodes)
    %SINumStates = getSINumStates()
    %SIStateString = getSIStateString(stateNumber)
    %SIStateNumber = getSIStateNumber(stateString)
    %nextStateNumber = SIGetNextState(SIStateMachine, infectionDetected)
    %***********************************************
    %SIR State machine specific functions:
    %SIRStateMachine = genSIRStateMachine(numNodes)
    %SIRNumStates = getSIRNumStates()
    %SIRStateString = getSIRStateString(stateNumber)
    %SIRRecoveryProb = getSIRRecoveryProb()
    %SIRStateNumber = getSIRStateNumber(stateString)
    %nextStateNumber = SIRGetNextState(SIRStateMachine, infectionDetected)
    %**************************************************
    %SEIR State machine specific functions:
    %SEIRStateMachine = genSEIRStateMachine(numNodes)
    %SEIRNumStates = getSEIRNumStates()
    %SEIRStateString = getSEIRStateString(stateNumber)
    %SEIRRecoveryProb = getSEIRRecoveryProb()
    %SEIRStateNumber = getSEIRStateNumber(stateString)
    %[nextStateNumber saveInfectionExploitData updateOwnExploitData resetOwnExploitData NewNextState_Eentry NewNextState_Eprob NewNext_LatencyCounter] = SEIRGetNextState(SEIRStateMachine, currentE_entryState, currentE_entryProb, current_LatencyCounter, infectionDetected, txTypeDetectedValue, txProbDetectedValue)
    %*****************************************************
    %SEPTICOX State machine specific functions:
    %SEPTICOXStateMachine = genSEPTICOXStateMachine(numNodes)
    %SEPTICOXNumStates = getSEPTICOXNumStates()
    %SEPTICOXStateString = getSEPTICOXStateString(stateNumber)
    %[Pso1 Pso2 Pos1 Pos2 Pxe Pex Pei1 Pei2 Psp Pts Pet Pit Pic Pci Ptp Pto1 Pto2] = getSEPTICOXProbList()
    %[Pso1 Pso2 Pos1 Pos2 Pxe Pex Pei1 Pei2 Psp Pts Pet Pit Pic Pci Ptp Pto1 Pto2] = getSEPTICOXProbList_class1Users()
    %[Pso1 Pso2 Pos1 Pos2 Pxe Pex Pei1 Pei2 Psp Pts Pet Pit Pic Pci Ptp Pto1 Pto2] = getSEPTICOXProbList_class2Users()
    %[Pso1 Pso2 Pos1 Pos2 Pxe Pex Pei1 Pei2 Psp Pts Pet Pit Pic Pci Ptp Pto1 Pto2] = getSEPTICOXProbList_class3Users()
    %SEPTICOXStateNumber = getSEPTICOXStateNumber(stateString)
    %nextStateNumber = SEPTICOXGetNextState(SEPTICOXStateMachine, infectionTransmitted)
    %*********************************************************
    %General state machine functions
    %StateMachine = genStateMachine(numNodes,numStates)
    %currentStateNumber = getCurrentStateNumberAllNodes(StateMachine)
    %currentStateString = getCurrentStateStringAllNodes(StateMachine)
    %newStateMachine = changeState(nextStateNumber, oldStateMachine)
    %**************************************************************
    %Functions to support exploit data
    %numTxTypes = getNumTxTypes()
    %numDamageTypes = getNumDamageTypes()
    %txTypeString = getTxTypeString(txNumber)
    %txTypeLatency = getTxTypeLatency()
    %txTypeUserProbability = getTxTypeUserProbability()
    %damageTypeString = getDamageTypeString(damageNumber)
    %exploitData = genExploitData(numNodes,sw_len)
    %[infectionDetected, infectionExploitData] = checkExploitRoute(randomEncounter,encounterYes,exploitData,genotype)
    %[infectionTransmitted,infectionExploitData,latencyExploitData,txTypeLatencyCounter,latencyFlag] = checkExploitTxType(infectionDetected,infectionExploitData,latencyExploitData,txTypeLatencyCounter,latencyFlag)
    %txTypeDetectedValue = checkExploitTxTypeValue(exploitData)
    %damageTypeDetectedValue= checkDamageTypeValue(exploitData)
    %updatedExploitData = updateOwnExploitData(exploitData,infectionTransmitted,infectionExploitData) 
    %
    % 
    
    properties
    end
    
    methods
    end
    %static methods that can be called without instantiating a class
    methods (Static)
        %******************************************
        %functions to support an SI state machine
        %******************************************
        function SIStateMachine = genSIStateMachine(numNodes)
            SIStateMachine = StateMachineClass.genStateMachine(numNodes,StateMachineClass.getSINumStates());
        end
        function SINumStates = getSINumStates()
            %SI state machine has two states (S) and (I)
            SINumStates=2;
        end
        function SIStateString = getSIStateString(stateNumber)
            %SI state machine has two states (S) and (I)
            SIString = {'S' 'I'};
            repmat(SIString,size(stateNumber,1),1);
            SIStateString = SIString(stateNumber)';
        end
        function SIStateNumber = getSIStateNumber(stateString)
            SIString = {'S' 'I'};
            SIString = repmat(SIString,size(stateString,1),1);
            SStringNumber = strcmp(SIString(:,1),stateString);
            IStringNumber = strcmp(SIString(:,2),stateString).*2;
            SIStateNumber = SStringNumber+IStringNumber;
        end
        function [nextStateNumber saveInfectionExploitData updateOwnExploitData]  = SIGetNextState(SIStateMachine, infectionDetected)
            currentStateNumber = StateMachineClass.getCurrentStateNumberAllNodes(SIStateMachine);
            %decisions for changing state
            %check if in state 'S' (1) AND there is an exploit match ->change to state 'I' (2)
            %instruct to save infectionExploit data
            changeState = and((currentStateNumber==StateMachineClass.getSIStateNumber('S')),(infectionDetected==1));%mask of nodes to change
            nextState_I = changeState.*StateMachineClass.getSIStateNumber('I'); %change for string conversion, here and line above   %change state
            NOTchangeState = (changeState-1).*(-1);%inv of change state
            saveInfectionExploitData = changeState;
            %check if in state 'I' (2) 
            %instruct to update own exploit data
            updateOwnExploitData = (currentStateNumber==StateMachineClass.getSIStateNumber('I'));
            %rest stay as they are
            nextState_temp2 = NOTchangeState.*currentStateNumber;
            nextStateNumber=nextState_I+nextState_temp2;
        end
        %******************************************
        %functions to support an SIR state machine
        %******************************************
        function SIRStateMachine = genSIRStateMachine(numNodes)
            %SIR state machine has three states (S)(I)(R)
            SIRStateMachine = StateMachineClass.genStateMachine(numNodes,StateMachineClass.getSIRNumStates());
        end
        function SIRNumStates = getSIRNumStates()
            %SIR state machine has three states (S)(I)(R)
            SIRNumStates=3;
        end
       function SIRStateString = getSIRStateString(stateNumber)
            %SI state machine has THREE states
            SIRString = {'S' 'I' 'R'};
            repmat(SIRString,size(stateNumber,1),1);
            SIRStateString = SIRString(stateNumber)';
       end
       function SIRRecoveryProb = getSIRRecoveryProb(t,testMode,ihandles)
           %define the probability of recovery, fixed no. here but could be
           %a distribution
           if testMode == 1 
           SIRRecoveryProb = 0.00025;
           else
               %if t >20000 %set time to start recovery NEEDS ADDING AS A PARAMETER
               if t >ihandles.SIRrecovTime %set time to start recovery
                SIRRecoveryProb = ihandles.SIRrecovProb;
               else
                   SIRRecoveryProb = 0;
               end
           end
       end
       function SIRStateNumber = getSIRStateNumber(stateString)
            SIRString = {'S' 'I' 'R'};
            SIRString = repmat(SIRString,size(stateString,1),1);
            SStringNumber = strcmp(SIRString(:,1),stateString);
            IStringNumber = strcmp(SIRString(:,2),stateString).*2;
            RStringNumber = strcmp(SIRString(:,3),stateString).*3;
            SIRStateNumber = SStringNumber+IStringNumber+RStringNumber;
       end
       function [nextStateNumber saveInfectionExploitData updateOwnExploitData resetOwnExploitData] = SIRGetNextState(t,testMode,ihandles,SIRStateMachine, infectionDetected,updateGenotypeYesIn)
           updateGenotypeYesEnable = 0; %set to 1 if want to reset infection to susceptible if model a VM reset when updating. SHOULD BE SET AS AN INPUT PARAMETER
           tD = t;
           updateGenotypeYes = updateGenotypeYesIn.*updateGenotypeYesEnable;
            numNodes = size(infectionDetected,1);
            currentStateNumber = StateMachineClass.getCurrentStateNumberAllNodes(SIRStateMachine);
            %***************************************************************************
            %decisions for changing state
            %***************************************************************************
            %check if in state 'S' (1) AND there is an exploit match ->change to state 'I'(2) and flag to save infection exploitData
            changeState_I = and((currentStateNumber==StateMachineClass.getSIRStateNumber('S')),(infectionDetected==1));%mask of nodes to change
            nextState_I = changeState_I.*StateMachineClass.getSIRStateNumber('I'); %change for string conversion, here and line above   %change state
            saveInfectionExploitData = changeState_I;
            %********************************************************************************
            %check if in state 'I' (2) -> flag to update own exploit data
            updateOwnExploitData = (currentStateNumber==StateMachineClass.getSIRStateNumber('I'));
            %decide who can recover
            randomNumber = rand(numNodes,1); %choose a random number upto 1
            SIRRecoveryProb = zeros(numNodes,1);
            SIRRecoveryProb(:) = StateMachineClass.getSIRRecoveryProb(t,testMode,ihandles); %get the probability of recovery
            recoveryYes = (randomNumber < SIRRecoveryProb); %decide who can recover
            %check if in state 'I' (2) AND ready to recover ->change to state 'R' (3)
            changeState_R = and((currentStateNumber==StateMachineClass.getSIRStateNumber('I')),(recoveryYes==1));%mask of nodes to change
            nextState_R = changeState_R.*StateMachineClass.getSIRStateNumber('R') ;
            %check if in state 'I' (2) AND NOT ready to recover, AND updateGenotypeYes ->change back to state 'S' (1)
            changeState_S = and((updateGenotypeYes==1),and((currentStateNumber==StateMachineClass.getSIRStateNumber('I')),(recoveryYes==0)));%mask of nodes to change
            nextState_S = changeState_S.*StateMachineClass.getSIRStateNumber('S') ;
            
            %rest stay as they are
            NOTchangeState = (changeState_I+changeState_R+changeState_S-1).*(-1);%inv of change state
            nextState_NoChange = NOTchangeState.*currentStateNumber;
            nextStateNumber=nextState_I + nextState_R + nextState_S + nextState_NoChange;
            %*********************************************************************************
            %check if in state 'R' (3) OR going from state I back to state S -> flag to reset own exploit data
            resetOwnExploitData = or(currentStateNumber==StateMachineClass.getSIRStateNumber('S'),currentStateNumber==StateMachineClass.getSIRStateNumber('R'));
            %********************************************************************************
       end
        %******************************************
        %functions to support an SEIR state machine
        %******************************************
        function SEIRStateMachine = genSEIRStateMachine(numNodes)
            %SEIR state machine has four states (S)(E)(I)(R)
            SEIRStateMachine = StateMachineClass.genStateMachine(numNodes,StateMachineClass.getSEIRNumStates());
        end
        function SEIRNumStates = getSEIRNumStates()
            %SIR state machine has four states (S)(E)(I)(R)
            SEIRNumStates=4;
        end
       function SEIRStateString = getSEIRStateString(stateNumber)
            %SI state machine has FIVE states
            SEIRString = {'S' 'E' 'I' 'R'};
            repmat(SEIRString,size(stateNumber,1),1);
            SEIRStateString = SEIRString(stateNumber)';
       end
       function SEIRRecoveryProb = getSEIRRecoveryProb(testMode,ihandles)
           %define the probability of recovery, fixed no. here but could be
           %a distribution
           if testMode == 1
           SEIRRecoveryProb = 0.00025;
           else
           SEIRRecoveryProb = ihandles.SEIRrecovProb;
           end
       end
       function SEIRStateNumber = getSEIRStateNumber(stateString)
            SEIRString = {'S' 'E' 'I' 'R'};
            SEIRString = repmat(SEIRString,size(stateString,1),1);
            SStringNumber = strcmp(SEIRString(:,1),stateString);
            EStringNumber = strcmp(SEIRString(:,2),stateString).*2;
            IStringNumber = strcmp(SEIRString(:,3),stateString).*3;
            RStringNumber = strcmp(SEIRString(:,4),stateString).*4;
            SEIRStateNumber = SStringNumber+EStringNumber+IStringNumber+RStringNumber;
       end
       function [nextStateNumber saveInfectionExploitData updateOwnExploitData resetOwnExploitData NewNextState_Eentry NewNextState_Eprob NewNext_LatencyCounter] = SEIRGetNextState(testMode,ihandles,SEIRStateMachine, currentE_entryState, currentE_entryProb, current_LatencyCounter, infectionDetected, txTypeDetectedValue, txProbDetectedValue)
           numNodes = size(infectionDetected,1);
           currentStateNumber = StateMachineClass.getCurrentStateNumberAllNodes(SEIRStateMachine);
            
            %***************************************************************************
            %decisions for changing state
            %***************************************************************************
            %state S
            %***************************************************************************
            %check if in state 'S' (1) AND there is an exploit match AND Auto->change to state 'I' (3)
            changeState_Ia = and((currentStateNumber==StateMachineClass.getSEIRStateNumber('S')),and((infectionDetected==1),(txTypeDetectedValue==StateMachineClass.getTxTypeNumber('Automated'))));%mask of nodes to change
            nextState_Ia = changeState_Ia.*StateMachineClass.getSEIRStateNumber('I'); %change state
            %********************************************************************************
            %check if in state 'S' (1) AND there is an exploit match AND (Latency OR User)->change to state 'E' (2)
            changeState_E = and((currentStateNumber==StateMachineClass.getSEIRStateNumber('S')),and((infectionDetected==1), (or((txTypeDetectedValue==StateMachineClass.getTxTypeNumber('Latency')),(txTypeDetectedValue==StateMachineClass.getTxTypeNumber('User'))))));%mask of nodes to change
            nextState_E = changeState_E.*StateMachineClass.getSEIRStateNumber('E'); %change for string conversion, here and line above   %change state
            nextState_Eentry = changeState_E.*txTypeDetectedValue;
            nextState_Eprob = changeState_E.*txProbDetectedValue;
            %************************************************************************************
            %check if in state 'S' (1) AND there is an exploit match,regardless of the tx type, then we need to save the infection exploit data for use later
            saveInfectionExploitData = and(currentStateNumber==StateMachineClass.getSEIRStateNumber('S'),infectionDetected==1);
            %********************************************************************************
            %state E
            %********************************************************************************
           %user
            %check if in state 'E' (2) AND currentE_entryState==user(3) MOVE WITH PROB ->change to state 'I' (3)   
            randomNumber = rand(numNodes,1); %choose a random number upto 1
            %currentE_entryProbD = currentE_entryProb
            %currentE_entryStateD = currentE_entryState
            userProbDetected = (randomNumber < currentE_entryProb); %decide who can recover
            changeState_Iu = and((currentStateNumber==StateMachineClass.getSEIRStateNumber('E')),and((userProbDetected==1),currentE_entryState==StateMachineClass.getTxTypeNumber('User')));%mask of nodes to change
            nextState_Iu = changeState_Iu*StateMachineClass.getSEIRStateNumber('I');  %change state
           %***********************************************************************************
           %latency 
           %check if in state 'E' (2) AND currentE_entryState==latency(2)-> INCREMENT LATENCY COUNTER 
            incrementLatencyCounter = and(currentStateNumber==StateMachineClass.getSEIRStateNumber('E'),currentE_entryState==StateMachineClass.getTxTypeNumber('Latency'));%mask of nodes to change
            %currentE_entryStateD=currentE_entryState %testing only
            %current_LatencyCounterD =current_LatencyCounter %testing only
            %currentE_entryProbD = currentE_entryProb %testing only
            next_LatencyCounter = current_LatencyCounter + 1.* incrementLatencyCounter; %add 1 to counter if conditions met
            latDelayEnded = (next_LatencyCounter == currentE_entryProb); %flag if reached count value
            resetCounter = (latDelayEnded-1).*-1; % conditions to reset counter(if reached count value, create mask of zeros to reset)
            NewNext_LatencyCounter = next_LatencyCounter.*resetCounter; %reset latency counter or leave as is.
            %************************************************
           %check if in state 'E' (2) AND currentE_entryState==latency(2)AND NOT REACHED LATENCY-> stay in state E (2)
            changeState_EE = and(currentStateNumber==StateMachineClass.getSEIRStateNumber('E'),and(resetCounter,currentE_entryState==StateMachineClass.getTxTypeNumber('Latency')));
            nextState_EE = changeState_EE.*StateMachineClass.getSEIRStateNumber('E'); %stay in state E if not reached count value
           %***************************************************
            %check if in state 'E' (2) AND currentE_entryState==latency(2) AND REACHED LATENCY -> change to state 'I' (3)
            changeState_Il = and((currentStateNumber==StateMachineClass.getSEIRStateNumber('E')),and(latDelayEnded,currentE_entryState==StateMachineClass.getTxTypeNumber('Latency')));%mask of nodes to change
            nextState_Il = changeState_Il*StateMachineClass.getSEIRStateNumber('I'); %change for string conversion, here and line above   %change state
            %*********************************************************************************
            %state I
            %*********************************************************************************
            %check if in state 'I' (2) -> flag to update own exploit data
            updateOwnExploitData = (currentStateNumber==StateMachineClass.getSEIRStateNumber('I'));
            %***********************************************************************************
            %decide who can recover
            randomNumber = rand(numNodes,1); %choose a random number upto 1
            SEIRRecoveryProb = zeros(numNodes,1);
            SEIRRecoveryProb(:) = StateMachineClass.getSEIRRecoveryProb(testMode,ihandles); %get the probability of recovery
            recoveryYes = (randomNumber < SEIRRecoveryProb); %decide who can recover
            %******************************************************************************
            %check if in state 'I' (3) AND ready to recover ->change to state 'R' (4)
            changeState_R = and((currentStateNumber==StateMachineClass.getSEIRStateNumber('I')),recoveryYes);%mask of nodes to change
            nextState_R = changeState_R.*StateMachineClass.getSEIRStateNumber('R');
            %********************************************************************************
            %state R
            %**********************************************************************************
            %check if in state 'R' (4) -> flag to reset own exploit data
            resetOwnExploitData = (currentStateNumber==StateMachineClass.getSEIRStateNumber('R'));
            %*********************************************************************************
            %unchanged - rest stay as they are
            %*********************************************************************************
            NOTchangeState = (changeState_Ia+changeState_E+changeState_EE+changeState_Iu+changeState_Il+changeState_R-1).*(-1);%inv of change state
            nextState_NoChange = NOTchangeState.*currentStateNumber;
            nextStateNumber=nextState_Ia + nextState_Iu + nextState_Il + nextState_E + nextState_EE + nextState_R + nextState_NoChange;
            
            %only time want to change E_Entry state is when goes from S to E state
            NOTchangeTxState=((changeState_E-1).*-1);
            nextState_Eentry_NoChange = NOTchangeTxState.*currentE_entryState;
            NewNextState_Eentry = nextState_Eentry + nextState_Eentry_NoChange;
            
            %only time want to change E_Entry prob is when goes from S to E state
            NOTchangeProbState=((changeState_E-1).*-1);
            nextState_Eprob_NoChange = NOTchangeProbState.*currentE_entryProb;
            NewNextState_Eprob = nextState_Eprob + nextState_Eprob_NoChange;
        end
        %******************************************
        %functions to support an SEPTICOX state machine
        %******************************************
        function SEPTICOXStateMachine = genSEPTICOXStateMachine(numNodes)
            %SEPTICOX state machine has EIGHT states 
            SEPTICOXStateMachine = StateMachineClass.genStateMachine(numNodes,StateMachineClass.getSEPTICOXNumStates());
        end
        function SEPTICOXNumStates = getSEPTICOXNumStates()
            %SIR state machine  has EIGHT states
            SEPTICOXNumStates=8;
        end
        function SEPTICOXStateString = getSEPTICOXStateString(stateNumber)
            %SI state machine has THREE states
            SEPTICOXString = {'S' 'E' 'P' 'T' 'I' 'C' 'O' 'X'};
            repmat(SEPTICOXString,size(stateNumber,1),1);
            SEPTICOXStateString = SEPTICOXString(stateNumber)';
        end
        function [Pso1 Pso2 Pos1 Pos2 Pxe Pex Pei1 Pei2 Psp Pts Pet Pit Pic Pci Ptp Pto1 Pto2] = getSEPTICOXProbList(testMode,ihandles)
           %define the probabilities here but could be a distribution
           if testMode == 1
          [Pso1 Pso2 Pos1 Pos2 Pxe Pex Pei1 Pei2 Psp Pts Pet Pit Pic Pci Ptp Pto1 Pto2] = StateMachineClass.getSEPTICOXProbList_class3Users();
          
          %SI testing
          %set some probs to zero
          %Pso1=0; 
          Pso2=0;
          %Pos1=0;
          Pos2=0; 
          Psp=0;
          Pic=0;
          Pci=0;
          %Pit=0;
          
          %SIR testing - additional probs set
          Pit = 0.00025;
          Ptp = 0;
          Pts = 0;
          Pto1 = 0;
          Pto2 = 0;
          %Pex = 0.5;
          %Pxe=0.5;
          Pet=0;
           else
          [Pso1 Pso2 Pos1 Pos2 Pxe Pex Pei1 Pei2 Psp Pts Pet Pit Pic Pci Ptp Pto1 Pto2] = StateMachineClass.getSEPTICOXProbList_fromihandles(ihandles);     
           end
          
        end
        function [Pso1 Pso2 Pos1 Pos2 Pxe Pex Pei1 Pei2 Psp Pts Pet Pit Pic Pci Ptp Pto1 Pto2] = getSEPTICOXProbList_class1Users()
           %data taken from the mini project paper - class 1 users  
           Pso1=0.5*1/(60*60*24*1); %0.5* 1/(60sec x60min x24hrs x1days) 50% every evening
           Pso2=0.75*1/(60*60*24*3); %0.75 * 1/(60sec x60min x24hrs x3days)75% of those 1 in 1000 switch to invisible after 3 days(off-line)
           Pos1=1/(60*60*24*1); %1/(60sec x60min x24hrs x1days)all turn on Every morning
           Pos2=(1/1000)*(1/(60*60*24*1));%1/1000 * 1/(60sec x60min x24hrs x1days) 1 in 1000 after 1 day
           Psp=0;  %0 patch not installed
           Pxe=1/(60*60*24*1);  %1/(60sec x60min x24hrs x1days) turn on every morning     
           Pex=0.5*1/(60*60*24*1);  %0.5 * 1/(60sec x60min x24hrs x1days) 50% every evening
           Pei1=(0.5*100*0.0035*1)/(60*60*24*1);%0.5*100*0.0035*1/(60sec x60min x24hrs x1days)user click onto a virus 0.5x100 links x0.35% after 1 day
           Pei2=0; %NA - prob 0
           Pet= 1/(60*60*24*5);%1/(60sec x60min x24hrs x5days)after 5 days
           Pit=1/(60*60*24*3);%1/(60sec x60min x24hrs x3days)treatment after 3 days
           Pic=0.5*1/(60*60*3);%0.5* 1/(60sec x60min x3hrs) 50% after 3 hours
           Pci=1/(60*60*24*1); %turn on after 1 day
           Pts=1/(60*60*12); %1/(60sec x60min x12hrs) become susceptible after 12 hours
           Ptp=0;%none install patches
           Pto1=0;%none turn off device
           Pto2=0;%none switch to invisible       
        end
       function [Pso1 Pso2 Pos1 Pos2 Pxe Pex Pei1 Pei2 Psp Pts Pet Pit Pic Pci Ptp Pto1 Pto2] = getSEPTICOXProbList_class2Users()
           %data taken from the mini project paper - class 2 users  
           Pso1=0.19*1/(60*60*24*1); %0.19* 1/(60sec x60min x24hrs x1days) 19% every evening
           Pso2=0.75*1/(60*60*24*3); %0.75 * 1/(60sec x60min x24hrs x3days)75% of those 1 in 1000 switch to invisible after 3 days(off-line)
           Pos1=1/(60*60*24*1); %1/(60sec x60min x24hrs x1days)all turn on Every morning
           Pos2=0; %NA
           Psp=0;  %0 patch not installed
           Pxe=1/(60*60*24*1);  %1/(60sec x60min x24hrs x1days) turn on every morning     
           Pex=0.19*1/(60*60*24*1);  %0.19 * 1/(60sec x60min x24hrs x1days) 19% every evening
           Pei1=(0.5*100*0.0035*1)/(60*60*24*1);%0.5*100*0.0035*1/(60sec x60min x24hrs x1days)user click onto a virus 0.5x100 links x0.35% after 1 day
           Pei2=0; %NA - prob 0
           Pet= 1/(60*60*24*5);%1/(60sec x60min x24hrs x5days)after 5 days
           Pit=1/(60*60*24*3);%1/(60sec x60min x24hrs x3days)treatment after 3 days
           Pic=0.5*1/(60*60*3);%0.5* 1/(60sec x60min x3hrs) 50% after 3 hours
           Pci=1/(60*60*24*1); %turn on after 1 day
           Pts=1/(60*60*12); %1/(60sec x60min x12hrs) become susceptible after 12 hours
           Ptp=0;%none install patches
           Pto1=0;%none turn off device
           Pto2=0;%none switch to invisible       
       end
       function [Pso1 Pso2 Pos1 Pos2 Pxe Pex Pei1 Pei2 Psp Pts Pet Pit Pic Pci Ptp Pto1 Pto2] = getSEPTICOXProbList_class3Users()
           %data taken from the mini project paper - class 3 users  
           Pso1=0.5*1/(60*60*24*1); %0.5* 1/(60sec x60min x24hrs x1days) 50% every evening
           Pso2=0.75*1/(60*60*24*3); %0.75 * 1/(60sec x60min x24hrs x3days)75% of those 1 in 1000 switch
           Pos1=1/(60*60*24*1); %1/(60sec x60min x24hrs x1days)all turn on Every morning
           Pos2=(1/1000)*(1/(60*60*24*1));%1/1000 * 1/(60sec x60min x24hrs x1days) 1 in 1000 after 1 day
           Psp=1/(60*60*24*5);  %1/(60*60*24*5) patch installed after 5 days
           Pxe=1/(60*60*24*1);  %1/(60sec x60min x24hrs x1days) turn on every morning     
           Pex=0.50*1/(60*60*24*1);  %0.50 * 1/(60sec x60min x24hrs x1days) 50% every evening
           Pei1=(0.0016*100*0.0035*1)/(60*60*24*1);%0.5*100*0.0035*1/(60sec x60min x24hrs x1days)user click onto a virus 0.5x100 links x0.35% after 1 day
           Pei2=0; %NA - prob 0
           Pet= 1/(60*60*24*3);%1/(60sec x60min x24hrs x3days)after 3 days
           Pit=1/(60*60*2);%1/(60sec x60min x2)treatment after 2 hours
           Pic=0.5*1/(60*60*3);%0.5* 1/(60sec x60min x3hrs) 50% after 3 hours
           Pci=1/(60*60*24*1); %turn on after 1 day
           Pts=0.25*1/(60*60*12); %0.25*1/(60sec x60min x12hrs) become susceptible after 12 hours
           Ptp=0.5*1/(60*60*12);%0.5*1/(60sec x60min x12hrs) 50% install patches after 12 hours
           Pto1=0.1*1/(60*60*12);%0.1*1/(60sec x60min x12hrs) 10% turn off device after 12 hours
           Pto2=0.15*1/(60*60*12);%15% switch to invisible after 12 hours     
       end
       function [Pso1 Pso2 Pos1 Pos2 Pxe Pex Pei1 Pei2 Psp Pts Pet Pit Pic Pci Ptp Pto1 Pto2] = getSEPTICOXProbList_fromihandles(ihandles)
        Pso1 = ihandles.Pso1;
        Pso2 = ihandles.Pso2;
        Pos1 = ihandles.Pos1;
        Pos2 = ihandles.Pos2;
        Pxe = ihandles.Pxe;
        Pex = ihandles.Pex;
        Pei1 = ihandles.Pei1;
        Pei2 = ihandles.Pei2;
        Psp = ihandles.Psp;
        Pts = ihandles.Pts;
        Pet = ihandles.Pet;
        Pit = ihandles.Pit;
        Pic = ihandles.Pic;
        Pci = ihandles.Pci;
        Ptp = ihandles.Ptp;
        Pto1 = ihandles.Pto1;
        Pto2 = ihandles.Pto2;
       end
       function [periodOn meanOn stdOn ampOn periodOff meanOff stdOff ampOff plotPeriod] = getSEPTICOXProbList_PeriodicList()
            periodOn = 24; %24 hour period
            meanOn =  9; %peak time 9am
            stdOn =  3; %peak spread of 3 hours  - divide by 3 std deviations 99.5%
            ampOn= 1; %amplitude of 1;
            periodOff = 24;%24 hour period
            meanOff =  20;%peak time 8pm (20hours)
            stdOff =  3; %peak spread of 3 hours - divide by 3 std deviations 99.5%
            ampOff = 1;
            plotPeriod = 0;
       end
       function [periodOn meanOn stdOn ampOn periodOff meanOff stdOff ampOff plotPeriod] = getSEPTICOXProbList_PeriodicListFromihandles(ihandles)
            periodOn = ihandles.FreqOn;
            meanOn = ihandles.PeakOn; 
            stdOn = ihandles.SpreadOn;
            ampOn= ihandles.AmpOn;
            periodOff = ihandles.FreqOff;
            meanOff = ihandles.PeakOff;
            stdOff =  ihandles.SpreadOff;
            ampOff = ihandles.AmpOff;
            plotPeriod = ihandles.PlotPeriod;
       end
       function [Pso1t Pos1t Pxet Pext] = getSEPTICOXProbList_Periodic(testMode,ihandles,t,deltaT,Pos1,Pso1,Pxe,Pex)
           %inputs
           %t:current time t in simulation
           %deltaT timesteps of t, seconds(1), minutes(1*60) hours(60*60) etc
           %original prob values not affected by time
           if testMode == 1
               [periodOn meanOn stdOn ampOn periodOff meanOff stdOff ampOff plotPeriod] = StateMachineClass.getSEPTICOXProbList_PeriodicList();
           else
               [periodOn meanOn stdOn ampOn periodOff meanOff stdOff ampOff plotPeriod] = StateMachineClass.getSEPTICOXProbList_PeriodicListFromihandles(ihandles);
           end
           
           %plotPeriod=0; %dont plot graphs
           simTime = 24*60*60; %sim for 24 hours %86400 numsteps needed for 1 cycle if deltaT is 1 second
           %deltaT = 1*60*60; %deltaT of 1 minute
           numSteps = fix(simTime/deltaT); %number of time steps per cycle
           %Original prob values - to be read in
           %Pos1(1)=1
           %Pso1(1)=1
           %Pxe(1)=1
           %Pex(1)=1
           %check for point in cycle to return probability distribution value
           divDiv = t/numSteps;
           remDiv = rem(t,numSteps);
           %if and(remDiv==0,divDiv==1)
           if remDiv==0
               indexProb = numSteps;
           else
               indexProb = remDiv;
           end    

           %***************
           %Set up periodic parameters, currently set but could be read from file
           %**************
            %period parameters of distribution - turn on devices
            period1 = periodOn*60*60; %24 hour period
            mean1 =  meanOn*60*60; %peak time 9am
            std1 =  (stdOn*60*60/3); %peak spread of 3 hours  - divide by 3 std deviations 99.5%
            amp1 = ampOn; %amplitude of 1;
            dayOnUnscale = normpdf([0:deltaT:period1-deltaT],mean1,std1);
            dayOn = amp1 .* (dayOnUnscale ./ max(dayOnUnscale));
            dayOn = (dayOn > 1) + ((dayOn <= 1) .* dayOn); %clip as necessary
            Period1On = repmat(dayOn,1,ceil(numSteps / length(dayOn)));
            Period1On = Period1On(1:numSteps); %only keep numSteps of it

            %switch off period
            period2 = periodOff*60*60;%24 hour period
            mean2 =  meanOff*60*60;%peak time 8pm (20hours)
            std2 =  (stdOff*60*60/3); %peak spread of 3 hours - divide by 3 std deviations 99.5%
            amp2 = ampOff;

            dayOffUnscale = normpdf([0:deltaT:period2-deltaT],mean2,std2);
            dayOff = amp2 .* (dayOffUnscale ./ max(dayOffUnscale));
            dayOff = (dayOff > 1) + ((dayOff <= 1) .* dayOff); %clip as necessary
            Period1Off = repmat(dayOff,1,ceil(numSteps / length(dayOff)));
            Period1Off = Period1Off(1:numSteps); %only keep numSteps of it
            %Pos1 periodic rate - switching on susceptible devices
            Pos1t_gr1 = Period1On.*Pos1(1); %multiply by on rate
            %Pso1 periodic rate - switching off susceptible devices 
            Pso1t_gr1 = Period1Off.*Pso1(1);%multiply by off rate
            %Pxe periodic rate - switching on exposed devices
            Pxet_gr1 = Period1On.*Pxe(1); %multiply by on rate
            %Pex periodic rate - switching off exposed devices 
            Pext_gr1 = Period1Off.*Pex(1);%multiply by off rate
            
            %return single probability values
            Pso1t = Pso1t_gr1(indexProb);
            Pos1t = Pos1t_gr1(indexProb);
            Pxet = Pxet_gr1(indexProb);
            Pext = Pext_gr1(indexProb);

            %plot graph
            if plotPeriod == 1
            handleCurrent = gcf;
            figure(15);
            cla(15);
            %if time step of 1 second need at least 86340 numSteps to plot
            tarr = [0:deltaT:simTime-deltaT] ./ 3600;
            plot(tarr,Pos1t_gr1(1:length(tarr)),'b');
            hold on;
            plot(tarr,Pso1t_gr1(1:length(tarr)),'g');
            legend('Pos1 on rate','Pso1 off rate');
            title('CLASS 1: Pos1 on rate and Pso1 off rate');
            figure(16);
            cla(16);
            plot(tarr,Pxet_gr1(1:length(tarr)),'b');
            hold on;
            plot(tarr,Pext_gr1(1:length(tarr)),'g');
            legend('Pxe on rate','Pex off rate');
            title('CLASS 1: Pxe on rate and Pex off rate');
            figure(handleCurrent);
            end
       end
       function SEPTICOXStateNumber = getSEPTICOXStateNumber(stateString)
            SEPTICOXString = {'S' 'E' 'P' 'T' 'I' 'C' 'O' 'X'};
            SEPTICOXString = repmat(SEPTICOXString,size(stateString,1),1);
            SStringNumber = strcmp(SEPTICOXString(:,1),stateString);
            EStringNumber = strcmp(SEPTICOXString(:,2),stateString).*2;
            PStringNumber = strcmp(SEPTICOXString(:,3),stateString).*3;
            TStringNumber = strcmp(SEPTICOXString(:,4),stateString).*4;
            IStringNumber = strcmp(SEPTICOXString(:,5),stateString).*5;
            CStringNumber = strcmp(SEPTICOXString(:,6),stateString).*6;
            OStringNumber = strcmp(SEPTICOXString(:,7),stateString).*7;
            XStringNumber = strcmp(SEPTICOXString(:,8),stateString).*8;
            SEPTICOXStateNumber = SStringNumber+EStringNumber+PStringNumber+TStringNumber+IStringNumber+CStringNumber+OStringNumber+XStringNumber;
       end
       function [nextStateNumber saveInfectionExploitData updateOwnExploitData resetOwnExploitData deviceOffline NewNextState_Eentry NewNextState_Eprob NewNext_LatencyCounter] = SEPTICOXGetNextState(testMode,ihandles,SEPTICOXStateMachine, t, deltaT, periodicProbE, latCountOption, currentE_entryState, currentE_entryProb, current_LatencyCounter, infectionDetected, txTypeDetectedValue, txProbDetectedValue)
           numNodes = size(infectionDetected,1);
           currentStateNumber = StateMachineClass.getCurrentStateNumberAllNodes(SEPTICOXStateMachine);
           %latency counter option:
           %latCountOption = 1 %stop latency counter if device goes offline
           %latCountOption = 2 %continue latency counter if device goes offline
           %**********************************************************************************
            %retrive state transition probabilities (some time based when included in function)
            %*********************************************************************************
            [Pso1 Pso2 Pos1 Pos2 Pxe Pex Pei1 Pei2 Psp Pts Pet Pit Pic Pci Ptp Pto1 Pto2] = StateMachineClass.getSEPTICOXProbList(testMode,ihandles);
            %Pei1 and Pei2 not used as this info is carried through by the exploit data information 
            %deltaT = 1 %1 time step = 1 second, 1*60 time step = 1 minute
            %check if are using periodic probability distributions for some state transitions
            if periodicProbE == 1
            [Pso1 Pos1 Pxe Pex] = StateMachineClass.getSEPTICOXProbList_Periodic(testMode,ihandles,t,deltaT,Pos1,Pso1,Pxe,Pex);
            end
            %********************************************
            %calculate all the fixed probability conditions
            %************************************************
           %prob condition for going offline to state 'O' with Prob Pso1(t)+ Pso2
            randomNumber = rand(numNodes,1); %choose a random number upto 1
            deviceOfflineDetected_O = (randomNumber < (Pso1 + Pso2)); %device off-line to state 'O'
            NOTdeviceOfflineDetected_O = not(deviceOfflineDetected_O); %device NOT off-line to state 'O'
           %prob condition for going offline to state 'X' with Prob Pex(t)
            randomNumber = rand(numNodes,1); %choose a random number upto 1
            deviceOfflineDetected_X = (randomNumber < Pex); %device off-line to state 'X'
            NOTdeviceOfflineDetected_X = not(deviceOfflineDetected_X); %device NOT off-line to state 'X'
           %prob condition for applying patch to state 'P' from 'S' - prevention of infection with prob Psp
            randomNumber = rand(numNodes,1); %choose a random number upto 1
            patchDetected_Ps = (randomNumber < Psp); %patch applied to device to stop infection
            NOTpatchDetected_Ps = not(patchDetected_Ps);
           %prob condition for applying treatment to state 'T' from 'E' with Prob Pet
            randomNumber = rand(numNodes,1); %choose a random number upto 1
            treatmentDetected_Te = (randomNumber < Pet); %treatment applied to device to prevent infection
            NOTtreatmentDetected_Te = not(treatmentDetected_Te);
           %prob condition for applying treatment to state 'T' from 'I' with Prob Pit
            randomNumber = rand(numNodes,1); %choose a random number upto 1
            treatmentDetected_Ti = (randomNumber < Pit); %treatment applied to device to prevent infection
           %prob condition for containing infection to state 'C' from 'I' with Prob Pic
            randomNumber = rand(numNodes,1); %choose a random number upto 1
            deviceContainedDetected = (randomNumber < Pic); %containment of device to prevent infection
            NOTdeviceContainedDetected = not(deviceContainedDetected);
           %prob condition for applying a patch for preventing infection to state 'P' from 'T' with Prob Ptp
            randomNumber = rand(numNodes,1); %choose a random number upto 1
            patchDetected_Pt = (randomNumber < Ptp); %containment of device to prevent infection
            NOTpatchDetected_Pt = not(patchDetected_Pt);
           %prob condition for going to state 'O' from 'T' with Prob Pto1+Pto2
            randomNumber = rand(numNodes,1); %choose a random number upto 1
            deviceOfflineDetected_Ot = (randomNumber < Pto1+Pto2); 
            NOTdeviceOfflineDetected_Ot = not(deviceOfflineDetected_Ot);
           %prob condition for going to state 'S' from 'T' with Prob Pts
            randomNumber = rand(numNodes,1); %choose a random number upto 1
            susceptibleDetected_S = (randomNumber < Pts); 
           %prob condition for going to state 'I' from 'C' with Prob Pci
            randomNumber = rand(numNodes,1); %choose a random number upto 1
            unContainedDetected_I = (randomNumber < Pci); 
           %prob condition for going to state 'S' from 'O' with Prob Pos1(t)+Pos2
            randomNumber = rand(numNodes,1); %choose a random number upto 1
            susceptibleDetected_So = (randomNumber < Pos1+Pos2);
           %prob condition for going to state 'E' from 'X' with Prob Pxe
            randomNumber = rand(numNodes,1); %choose a random number upto 1
            exposedDetected_Ex = (randomNumber < Pxe);
            %*************************************************
            %calculate all the variable probability conditions
            %*************************************************
            %Prob condition for a user virus
            randomNumber = rand(numNodes,1); %choose a random number upto 1
            %currentE_entryProbD = currentE_entryProb
            %currentE_entryStateD = currentE_entryState
            userProbDetected = (randomNumber < currentE_entryProb);
            %******************************************************
            %latency counter
            %******************************************************
            if latCountOption == 1
                %only count whilst in E state
                %check if in state 'E' (2) currentE_entryState==latency(2)-> INCREMENT LATENCY COUNTER 
                incrementLatencyCounter = and(currentStateNumber==StateMachineClass.getSEPTICOXStateNumber('E'),currentE_entryState==StateMachineClass.getTxTypeNumber('Latency'));%mask of nodes to change
            else
                %count in E state and X state when offline
                %check if in state 'E' or 'X' state and currentE_entryState==latency(2)-> INCREMENT LATENCY COUNTER 
                incrementLatencyCounter = and(or(currentStateNumber==StateMachineClass.getSEPTICOXStateNumber('E'),currentStateNumber==StateMachineClass.getSEPTICOXStateNumber('X')),currentE_entryState==StateMachineClass.getTxTypeNumber('Latency'));%mask of nodes to change
            end
            %currentE_entryStateD=currentE_entryState %testing only
            %current_LatencyCounterD =current_LatencyCounter %testing only
            %currentE_entryProbD = currentE_entryProb %testing only
            next_LatencyCounter = current_LatencyCounter + 1.* incrementLatencyCounter; %add 1 to counter if conditions met
            latDelayEnded_temp = (next_LatencyCounter >= currentE_entryProb); %flag if reached count value or more than if still counting whilst off-line
            latDelayEnded = and(currentStateNumber==StateMachineClass.getSEPTICOXStateNumber('E'),latDelayEnded_temp);
            %resetCounter = not(latDelayEnded); % conditions to reset counter(if reached count value in E state, create mask of zeros to reset)
            resetCounter = not(or(currentStateNumber==StateMachineClass.getSEPTICOXStateNumber('S'),currentStateNumber==StateMachineClass.getSEPTICOXStateNumber('I'))); % co
            NewNext_LatencyCounter = next_LatencyCounter.*resetCounter; %reset latency counter or leave as is.
           
            %***************************************************************************
            %decisions for changing state
            %***************************************************************************
            %state S
            %***************************************************************************
            %1.check if in state 'S' (1) AND user has decided to turn the device off-line ->change to state 'O'
            changeState_O = and(currentStateNumber==StateMachineClass.getSEPTICOXStateNumber('S'),deviceOfflineDetected_O);%mask of nodes to change
            nextState_O = changeState_O*StateMachineClass.getSEPTICOXStateNumber('O');  %change state
            %********************************************************************************
            %2.check if in state 'S' AND device not off-line AND patch has been applied to device -> change to state prevented 'P'
            changeState_Ps = and(currentStateNumber==StateMachineClass.getSEPTICOXStateNumber('S'),and(NOTdeviceOfflineDetected_O,patchDetected_Ps));%mask of nodes to change
            nextState_Ps = changeState_Ps*StateMachineClass.getSEPTICOXStateNumber('P');  %change state
            %*****************************************************************************************
            %3.check if in state 'S' (1) AND device not off-line AND there is no patch AND there is an exploit match AND Auto->change to state 'I' (3)
            changeState_Ia =  and((currentStateNumber==StateMachineClass.getSEPTICOXStateNumber('S')),and((NOTpatchDetected_Ps),and((NOTdeviceOfflineDetected_O),and((infectionDetected),(txTypeDetectedValue==StateMachineClass.getTxTypeNumber('Automated'))))));%mask of nodes to change
            nextState_Ia = changeState_Ia.*StateMachineClass.getSEPTICOXStateNumber('I'); %change state
            %********************************************************************************
            %4.check if in state 'S' (1) AND device not off-line AND there is no patch AND an exploit match AND (Latency OR User)->change to state 'E' (2)
            changeState_E = and((currentStateNumber==StateMachineClass.getSEPTICOXStateNumber('S')),and((NOTpatchDetected_Ps),and((NOTdeviceOfflineDetected_O),and((infectionDetected), (or((txTypeDetectedValue==StateMachineClass.getTxTypeNumber('Latency')),(txTypeDetectedValue==StateMachineClass.getTxTypeNumber('User'))))))));%mask of nodes to change
            nextState_E = changeState_E.*StateMachineClass.getSEPTICOXStateNumber('E'); %change for string conversion, here and line above   %change state
            %set internal E_entry states
            nextState_Eentry = changeState_E.*txTypeDetectedValue;
            nextState_Eprob = changeState_E.*txProbDetectedValue;
            %************************************************************************************
            %S state Flag
            %check if in state 'S' (1) AND device not off-line AND there is an exploit match,regardless of the tx type, then we need to save the infection exploit data for use later
            saveInfectionExploitData = and(currentStateNumber==StateMachineClass.getSEPTICOXStateNumber('S'),and(NOTdeviceOfflineDetected_O,infectionDetected));
            %********************************************************************************
            %state E
            %********************************************************************************
            %1.check if in state 'E' AND user has decided to turn the device off-line -> change state to 'X'
            changeState_X = and(currentStateNumber==StateMachineClass.getSEPTICOXStateNumber('E'),deviceOfflineDetected_X);%mask of nodes to change
            nextState_X = changeState_X*StateMachineClass.getSEPTICOXStateNumber('X');  %change state
            %**********************************************************************************
            %2.check if in state 'E' AND device not going off-line AND treatment is to be applied -> change to state 'T'
            changeState_Te = and((currentStateNumber==StateMachineClass.getSEPTICOXStateNumber('E')),and((NOTdeviceOfflineDetected_X),treatmentDetected_Te));%mask of nodes to change
            nextState_Te = changeState_Te*StateMachineClass.getSEPTICOXStateNumber('T'); 
            %********************************************************************************
           %user
            %3.check if in state 'E' (2) AND device not going off-line AND not going for treatment AND user prob detected AND currentE_entryState==user(3) MOVE WITH PROB ->change to state 'I' (3)    
            changeState_Iu = and((currentStateNumber==StateMachineClass.getSEPTICOXStateNumber('E')),and((NOTtreatmentDetected_Te),and((NOTdeviceOfflineDetected_X),and((userProbDetected),currentE_entryState==StateMachineClass.getTxTypeNumber('User')))));%mask of nodes to change
            nextState_Iu = changeState_Iu*StateMachineClass.getSEPTICOXStateNumber('I');  %change state
            %***********************************************************************************
           %latency
            %4.check if in state 'E' (2) AND device not going off-line AND not going for treatment AND currentE_entryState==latency(2) AND REACHED LATENCY -> change to state 'I' (3)
            changeState_Il = and((currentStateNumber==StateMachineClass.getSEPTICOXStateNumber('E')),and((NOTtreatmentDetected_Te),and((NOTdeviceOfflineDetected_X),and(latDelayEnded,currentE_entryState==StateMachineClass.getTxTypeNumber('Latency')))));%mask of nodes to change
            nextState_Il = changeState_Il*StateMachineClass.getSEPTICOXStateNumber('I');    %change state
            %*********************************************************************************
            
            %************************************************
           %check if in state 'E' (2) AND device not going off-line AND currentE_entryState==latency(2)AND NOT REACHED LATENCY-> stay in state E (2)
            %changeState_EE = and(currentStateNumber==StateMachineClass.getSEPTICOXStateNumber('E'),and(NOTdeviceOfflineDetected_X,and(resetCounter,currentE_entryState==StateMachineClass.getTxTypeNumber('Latency'))));
            %nextState_EE = changeState_EE.*StateMachineClass.getSEPTICOXStateNumber('E'); %stay in state E if not reached count value
           %***************************************************

            %state I
            %***********************************************************************************
            %1.check in state 'I' AND device going to contain infection by turning off / going off-line -> change to state 'C'
            changeState_C = and((currentStateNumber==StateMachineClass.getSEPTICOXStateNumber('I')),(deviceContainedDetected));%mask of nodes to change
            nextState_C = changeState_C*StateMachineClass.getSEPTICOXStateNumber('C'); 
            %***********************************************************************************
            %2.check in state 'I' AND not going to contain off-line / turn off AND treatment is to be applied -> change to state 'T'
            changeState_Ti = and((currentStateNumber==StateMachineClass.getSEPTICOXStateNumber('I')),and((NOTdeviceContainedDetected),treatmentDetected_Ti));%mask of nodes to change
            nextState_Ti = changeState_Ti*StateMachineClass.getSEPTICOXStateNumber('T'); 
            %********************************************************************************
            %state T
            %**********************************************************************************
            %1.check if in state 'T' AND patch to be applied to prevent infection -> change to state 'P'
            changeState_Pt = and((currentStateNumber==StateMachineClass.getSEPTICOXStateNumber('T')),(patchDetected_Pt));%mask of nodes to change
            nextState_Pt = changeState_Pt*StateMachineClass.getSEPTICOXStateNumber('P'); 
            %*********************************************************************************
            %2.check if in state 'T' AND NO patch to be applied AND going to susceptible off-line -> change to state 'O'
            changeState_Ot = and((currentStateNumber==StateMachineClass.getSEPTICOXStateNumber('T')),and((NOTpatchDetected_Pt),deviceOfflineDetected_Ot));%mask of nodes to change
            nextState_Ot = changeState_Ot*StateMachineClass.getSEPTICOXStateNumber('O'); 
            %**********************************************************************************
            %3.check if in state 'T' AND NO patch to be applied AND NOT going to susceptible off-line AND going to susceptible -> change to state 'S'
            changeState_St = and((currentStateNumber==StateMachineClass.getSEPTICOXStateNumber('T')),and((susceptibleDetected_S),and((NOTpatchDetected_Pt),NOTdeviceOfflineDetected_Ot)));%mask of nodes to change
            nextState_St = changeState_St*StateMachineClass.getSEPTICOXStateNumber('S'); 
            %***********************************************************************************
            %state C
            %***********************************************************************************
            %1.check if in state 'C' and device turns back on-line -> change to state 'I'
            changeState_Ic = and((currentStateNumber==StateMachineClass.getSEPTICOXStateNumber('C')),(unContainedDetected_I));%mask of nodes to change
            nextState_Ic = changeState_Ic*StateMachineClass.getSEPTICOXStateNumber('I'); 
            %**********************************************************************************
            %state O
            %**********************************************************************************
            %1.check if in state 'O' and device turns back on-line -> change to state 'S'
            changeState_So = and((currentStateNumber==StateMachineClass.getSEPTICOXStateNumber('O')),(susceptibleDetected_So));%mask of nodes to change
            nextState_So = changeState_So*StateMachineClass.getSEPTICOXStateNumber('S'); 
            %************************************************************************************
            %state X           
            %*********************************************************************************
            %1.check if in state 'X' and device turns back on-line -> change to state 'E'
            changeState_Ex = and((currentStateNumber==StateMachineClass.getSEPTICOXStateNumber('X')),(exposedDetected_Ex));%mask of nodes to change
            nextState_Ex = changeState_Ex*StateMachineClass.getSEPTICOXStateNumber('E');
            %**********************************************************************************
            %unchanged - rest stay as they are
            %*********************************************************************************
            NOTchangeState = not(changeState_Ia+changeState_O+changeState_Ps+changeState_E+changeState_X+changeState_Iu+changeState_Il+changeState_Te+changeState_C+changeState_Ti+changeState_Pt+changeState_Ot+changeState_St+changeState_Ic+changeState_So+changeState_Ex); %inv of change state
            nextState_NoChange = NOTchangeState.*currentStateNumber;
            nextStateNumber=nextState_Ia + nextState_O + nextState_Ps + nextState_Iu + nextState_Il + nextState_E + nextState_X + nextState_Te + nextState_C + nextState_Ti + nextState_Pt + nextState_Ot + nextState_St + nextState_Ic + nextState_So + nextState_Ex + nextState_NoChange;
            
            %only time want to change E_Entry state is when goes from S to E state
            NOTchangeTxState=((changeState_E-1).*-1);
            nextState_Eentry_NoChange = NOTchangeTxState.*currentE_entryState;
            NewNextState_Eentry = nextState_Eentry + nextState_Eentry_NoChange;
            
            %only time want to change E_Entry prob is when goes from S to E state
            NOTchangeProbState=((changeState_E-1).*-1);
            nextState_Eprob_NoChange = NOTchangeProbState.*currentE_entryProb;
            NewNextState_Eprob = nextState_Eprob + nextState_Eprob_NoChange;

            %************************************************************************
            %Other flags
            %***********************************************************************
            %check if in state 'I' -> flag to update own exploit data
            updateOwnExploitData = (currentStateNumber==StateMachineClass.getSEPTICOXStateNumber('I'));
            %***********************************************************************
            %reset exploit data flag
            %check if in state 'C' -> flag to reset own exploit data
            resetOwnExploitData_C = (currentStateNumber==StateMachineClass.getSEPTICOXStateNumber('C'));
            %check if in state 'T'  -> flag to reset own exploit data
            resetOwnExploitData_T = (currentStateNumber==StateMachineClass.getSEPTICOXStateNumber('T'));
           
            resetOwnExploitData = resetOwnExploitData_C + resetOwnExploitData_T; %could or this too and take out of code above?
             %**********************************************************************************
            %device off-line flag
            deviceOffline = or(currentStateNumber==StateMachineClass.getSEPTICOXStateNumber('O'),or(currentStateNumber==StateMachineClass.getSEPTICOXStateNumber('X'),currentStateNumber==StateMachineClass.getSEPTICOXStateNumber('C')));
        end
       
        %******************************************
        %General state machine functions
        %******************************************
        %generate a state machine for every node
        function StateMachine = genStateMachine(numNodes,numStates)
            StateMachine = zeros(numNodes,numStates);
            %set initial state to first state
            StateMachine(:,1) = 1;
        end
        function currentStateNumber = getCurrentStateNumberAllNodes(StateMachine)
            %find index of '1'
            [stateValue stateIndex] = sort(StateMachine,2,'descend');
            currentStateNumber = stateIndex(:,1);
        end
        function currentStateString = getCurrentStateStringAllNodes(StateMachine)
            SINumStates = StateMachineClass.getSINumStates();
            SIRNumStates = StateMachineClass.getSIRNumStates();
            SEIRNumStates = StateMachineClass.getSEIRNumStates();
            SEPTICOXNumStates = StateMachineClass.getSEPTICOXNumStates();
            stateMachineNumStates = size(StateMachine,2);
            %find current state number
            [stateValue stateIndex] = sort(StateMachine,2,'descend');
            currentStateNumber = stateIndex(:,1);
            %determine type of state machine and get the string value
            if stateMachineNumStates == SINumStates
                currentStateString = StateMachineClass.getSIStateString(currentStateNumber);
            elseif stateMachineNumStates == SIRNumStates
                currentStateString = StateMachineClass.getSIRStateString(currentStateNumber);
            elseif stateMachineNumStates == SEIRNumStates
                currentStateString = StateMachineClass.getSEIRStateString(currentStateNumber);
            elseif stateMachineNumStates == SEPTICOXNumStates
                currentStateString = StateMachineClass.getSEPTICOXStateString(currentStateNumber);
            end
        end
        function numStates = getNumStates(ihandles)
            if strcmp(ihandles.attackModelS,'SI')
                numStates = 2;
            elseif strcmp(ihandles.attackModelS,'SIR')
                numStates = 3;
            elseif strcmp(ihandles.attackModelS,'SEIR')
                numStates = 4;
            elseif strcmp(ihandles.attackModelS,'SEPTICOX')
                numStates = 8;
            end   
        end
        function getNextState(t,ihandles,rhandles,infectionDetected,txTypeDetectedValue,txProbDetectedValue)
            SINumStates = StateMachineClass.getSINumStates();
            SIRNumStates = StateMachineClass.getSIRNumStates();
            SEIRNumStates = StateMachineClass.getSEIRNumStates();
            SEPTICOXNumStates = StateMachineClass.getSEPTICOXNumStates();
            stateMachineNumStates = size(rhandles.StateMachine,2);
             %Decide on the next state
             testMode = 0; %not running in test mode
            if stateMachineNumStates == SINumStates
                [rhandles.nextStateNumber rhandles.saveInfectionExploitDataEnable rhandles.updateOwnExploitDataEnable] = StateMachineClass.SIGetNextState(rhandles.StateMachine, infectionDetected);
            elseif stateMachineNumStates == SIRNumStates
                [rhandles.nextStateNumber rhandles.saveInfectionExploitDataEnable rhandles.updateOwnExploitDataEnable rhandles.resetOwnExploitDataEnable] = StateMachineClass.SIRGetNextState(t,testMode,ihandles,rhandles.StateMachine, infectionDetected, rhandles.updateGenotypeYes);
            elseif stateMachineNumStates == SEIRNumStates
                [rhandles.nextStateNumber rhandles.saveInfectionExploitDataEnable rhandles.updateOwnExploitDataEnable rhandles.resetOwnExploitDataEnable rhandles.NewNextState_Eentry rhandles.NewNextState_Eprob rhandles.NewNext_LatencyCounter] = StateMachineClass.SEIRGetNextState(testMode,ihandles,rhandles.StateMachine, rhandles.currentE_entryState, rhandles.currentE_entryProb, rhandles.current_LatencyCounter, infectionDetected, txTypeDetectedValue, txProbDetectedValue);
            elseif stateMachineNumStates == SEPTICOXNumStates
                [rhandles.nextStateNumber rhandles.saveInfectionExploitDataEnable rhandles.updateOwnExploitDataEnable rhandles.resetOwnExploitDataEnable rhandles.deviceOffline rhandles.NewNextState_Eentry rhandles.NewNextState_Eprob rhandles.NewNext_LatencyCounter] = StateMachineClass.SEPTICOXGetNextState(testMode,ihandles,rhandles.StateMachine, t, rhandles.deltaT, ihandles.UsePeriodic, rhandles.latCountOption, rhandles.currentE_entryState, rhandles.currentE_entryProb, rhandles.current_LatencyCounter, infectionDetected, txTypeDetectedValue, txProbDetectedValue);
            end
         end
        function newStateMachine = changeState(nextStateNumber, oldStateMachine)
            %convert nextStateNumber into a state machine
            newStateMachine_temp = zeros(size(oldStateMachine));
            %newStateMachine_temp(:,nextStateNumber(:))=1
            for i = 1:size(oldStateMachine,1)
                newStateMachine_temp(i,nextStateNumber(i)) = 1;
            end
            newStateMachine = newStateMachine_temp;
        end
        function StateNumber = getStateNumber(stateString,attackModelS)
            switch(attackModelS)
                case 'SI'
                    StateNumber = StateMachineClass.getSIStateNumber(stateString)
                case 'SIR'
                    StateNumber = StateMachineClass.getSIRStateNumber(stateString)
                case 'SEIR'
                    StateNumber = StateMachineClass.getSEIRStateNumber(stateString)
                case 'SEPTICOX'
                    StateNumber = StateMachineClass.getSEPTICOXStateNumber(stateString)
            end
        end
        function sumAllStates(t,rhandles)
            %keeps tally of how many nodes in which state
            for i = 1:rhandles.numStates 
            rhandles.sumOfStates(i,t) = sum(rhandles.StateMachine(:,i),1);
            end
        end
        function calcBeta(t,lengthCounter,rhandles)
            %THIS FUNCTION DOESN'T WORK PROPAERLY AS DOESN'T TAKE INTO
            %ACCOUNT TRUE NUMBER SUSCEPTIBLE WHEN EXPLOIT SET
            %measures instantaneous beta(infection rate) for SI and SIR state machines
            S = 1; %state 1
            I = 2; %state 2
            if t > 1 % needs to be greater than 1 else no difference to find
            %rhandles.numStates 
            %timeBeta = t
            %SNew = rhandles.sumOfStates(S,t) ;
            %SOld = rhandles.sumOfStates(S,t-1);
            %IOld = rhandles.sumOfStates(I,t-1);
            %numNodes = sum(rhandles.sumOfStates(:,1));
            %rhandles.beta(t) = ((SNew - SOld)/(IOld * SOld/numNodes))*-1;
            %%beta = rhandles.beta(t)
            %calc beta
            rhandles.S_buffer(2) = rhandles.S_buffer(1); %assign old S
            rhandles.S_buffer(1) = rhandles.sumOfStates(S,lengthCounter);%assign new SNew = rhandles.sumOfStates(S,t)
            rhandles.I_buffer(2) =  rhandles.I_buffer(1); %assign old I
            rhandles.I_buffer(1) = rhandles.sumOfStates(I,lengthCounter);%assign new I
            numNodes = sum(rhandles.sumOfStates(:,1));
            rhandles.beta(lengthCounter) =  ((rhandles.S_buffer(1)-rhandles.S_buffer(2))/(rhandles.I_buffer(2)*rhandles.S_buffer(2)/numNodes))*-1;
            end
        end
        function calcGammaSIR(t,rhandles)
            %measures instantaneous gamma(recovery rate) for an SIR state machine
            S = 1; %state 1
            I = 2; %state 2
            R = 3; %state 3
            if t > 1 % needs to be greater than 1 else no difference to find
            %rhandles.numStates 
            %timeBeta = t
            RNew = rhandles.sumOfStates(R,t) ;
            ROld = rhandles.sumOfStates(R,t-1);
            IOld = rhandles.sumOfStates(I,t-1);
            rhandles.gammaSIR(t) = (RNew - ROld)/IOld;
            %beta = rhandles.beta(t)
            end
        end
        %*****************************************************************************************
        %Functions to support exploit data
        %*****************************************************************************************
        function numTxTypes = getNumTxTypes()
            %transmission types: automated, latency, user
            numTxTypes = 3;
        end
        function numTxProbs = getNumTxProbs()
            %transmission types: automated, latency, user
            numTxProbs = 3;
        end
        function numDamageTypes = getNumDamageTypes()
            %damage types: shut down, gather data, delete files, inhibit speed
            %get some realistic damage types
            numDamageTypes = 4;
        end
        function txTypeString = getTxTypeString(txNumber)
            %tx type has three types
            txTypeString = {'Automated' 'Latency' 'User'};
            txTypeString = txTypeString(txNumber)';
        end
        function malwareTypeString = getMalwareTypeString(malNumber)
            %tx type has three types
            malwareTypeString = {'AND' 'OR'};
            malwareTypeString = malwareTypeString(malNumber)';
        end
        function txTypeProbString = getTxProbString(txNumber)
            %tx prob has three types
            txTypeProbString = {'AutomatedProb' 'LatencyDelay' 'UserProb'};
            txTypeProbString = txTypeProbString(txNumber)';
        end
        function txTypeNumber = getTxTypeNumber(txString)
            txTypeString = {'Automated' 'Latency' 'User'};
            txTypeString = repmat(txTypeString,size(txString,1),1);
            AStringNumber = strcmp(txTypeString(:,1),txString);
            LStringNumber = strcmp(txTypeString(:,2),txString).*2;
            UStringNumber = strcmp(txTypeString(:,3),txString).*3;
            txTypeNumber = AStringNumber+LStringNumber+UStringNumber;
        end
        function txProbNumber = getTxProbNumber(txProbString)
            txTypeProbString = {'AutomatedProb' 'LatencyDelay' 'UserProb'};
            txTypeProbString = repmat(txTypeProbString,size(txProbString,1),1);
            AStringNumber = strcmp(txTypeProbString(:,1),txProbString);
            LStringNumber = strcmp(txTypeProbString(:,2),txProbString).*2;
            UStringNumber = strcmp(txTypeProbString(:,3),txProbString).*3;
            txProbNumber = AStringNumber+LStringNumber+UStringNumber;
       end
        function txTypeLatency = getTxTypeLatency()
            %latency time
            txTypeLatency = 500; %could be passed through handles
        end
        function txTypeUserProbability = getTxTypeUserProbability()
            %probablility of user transmission
            txTypeUserProbability = 0.025; %could be passed through handles
        end
        function damageTypeString = getDamageTypeString(damageNumber)
            %tx type has three types
            damageTypeString = {'Shutdown' 'GatherData' 'DeleteFiles' 'InhibitSpeed'};
            damageTypeString = damageTypeString(damageNumber)';
        end
       
        function exploitData = genExploitData(numNodes,sw_len)
            %generate the exploit data as a structure of 4 fields
            exploitRoute = zeros(numNodes,sw_len);
            exploitTxType = zeros(numNodes,StateMachineClass.getNumTxTypes());
            exploitTxProb = zeros(numNodes,StateMachineClass.getNumTxProbs());
            exploitDamageType = zeros(numNodes,StateMachineClass.getNumDamageTypes());
            malwareType = zeros(numNodes,1); %1 AND 0 OR
            data.exploitRoute=exploitRoute;
            data.exploitTxType=exploitTxType;
            data.exploitTxProb=exploitTxProb;
            data.exploitDamageType=exploitDamageType;
            data.malwareType = malwareType;
            exploitData=data;  
        end
        function [infectionDetected, infectionExploitData] = checkExploitRoute(ihandles,randomEncounter,encounterYes,exploitData,genotype)
            %disp('checkExploitRoute')
            %test
            %randomEncounter
            %encounterYes
            %genotype
            
            
            %get length of each locus
            ihandleslen_apps=ihandles.len_apps;
            ihandleslen_serv=ihandles.len_serv;
            ihandleslen_os=ihandles.len_os;
            ihandleslen_dr=ihandles.len_dr;
            %encounterYesD = encounterYes %enable an encounter based upon a probability distribution - display for testing
            %randomEncounterD=randomEncounter %encounter another node at random(but not itself) - display for testing
            encounterYesRepRoute = repmat(encounterYes,1,size(exploitData.exploitRoute,2));%extend encounterYes to length of software
            encounterYesRepTx = repmat(encounterYes,1,size(exploitData.exploitTxType,2));
            encounterYesRepProb = repmat(encounterYes,1,size(exploitData.exploitTxProb,2));
            encounterYesRepDam = repmat(encounterYes,1,size(exploitData.exploitDamageType,2));
            encounterYesRepMal = repmat(encounterYes,1,size(exploitData.malwareType,2));
            %exploit data
            %exploitRouteBefore = exploitData.exploitRoute %display for test
            %exploitTxTypeBefore = exploitData.exploitTxType %display for test
            %exploitDamageTypeBefore = exploitData.exploitDamageType; %display for test
            %get the exploit data associated with the encountered node
            encounteredNodeExploitRoute = exploitData.exploitRoute(randomEncounter,:);
            encounteredNodeExploitTxType = exploitData.exploitTxType(randomEncounter,:);
            encounteredNodeExploitTxProb = exploitData.exploitTxProb(randomEncounter,:);
            encounteredNodeExploitDamageType = exploitData.exploitDamageType(randomEncounter,:);
            encounteredNodeExploitMalwareType = exploitData.malwareType(randomEncounter,:);
            %only enable those that have actually had an encounter in this time step
            encounteredNodeExploitRouteEnabled = encounteredNodeExploitRoute.*encounterYesRepRoute;
            encounteredNodeExploitTxTypeEnabled = encounteredNodeExploitTxType.*encounterYesRepTx;
            encounteredNodeExploitTxProbEnabled = encounteredNodeExploitTxProb.*encounterYesRepProb;
            encounteredNodeExploitDamageTypeEnabled = encounteredNodeExploitDamageType.*encounterYesRepDam;  
            encounteredNodeExploitMalwareTypeEnabled = encounteredNodeExploitMalwareType.*encounterYesRepMal ;
            %check to see if an exploit exists that can infect the current
            %genotype, returns a 1 if an infection is detected.
            %genotypeD = genotype %test display
                   %detect=and(encounteredNodeExploitRouteEnabled,genotype)
            %sum along rows to see if virus can cause infection
                    %infectionDetected = sum(detect,2)
                    %infectionDetected = (infectionDetected>0) %to allow for the case of multiple exploit routes
            %**************************************
            %Added 5th Jan 2017 AND / OR detection
            [ startP endP numLoci] = StaticDiversityClass.getStartAndEndLoci([ihandleslen_apps ihandleslen_serv ihandleslen_os ihandleslen_dr]);
            %split out genotype for logical matching across loci
            genotypeApps = genotype(:,startP(1):endP(1));
            genotypeServ = genotype(:,startP(2):endP(2));
            genotypeOs = genotype(:,startP(3):endP(3));
            genotypeDr = genotype(:,startP(4):endP(4));
            %split out exploit route for logical matching across loci
            exploitApps = encounteredNodeExploitRouteEnabled(:,startP(1):endP(1));
            exploitServ = encounteredNodeExploitRouteEnabled(:,startP(2):endP(2));
            exploitOs = encounteredNodeExploitRouteEnabled(:,startP(3):endP(3));
            exploitDr = encounteredNodeExploitRouteEnabled(:,startP(4):endP(4));
            %check for match at each locus
            detectApps=and(exploitApps,genotypeApps);
            infectionDetectedApps = sum(detectApps,2); infectionDetectedApps = (infectionDetectedApps>0);%to allow for the case of multiple exploit routes
            detectServ=and(exploitServ,genotypeServ);
            infectionDetectedServ = sum(detectServ,2); infectionDetectedServ = (infectionDetectedServ>0);
            detectOs=and(exploitOs,genotypeOs);
            infectionDetectedOs = sum(detectOs,2); infectionDetectedOs = (infectionDetectedOs>0);
            detectDr=and(exploitDr,genotypeDr);
            infectionDetectedDr = sum(detectDr,2); infectionDetectedDr = (infectionDetectedDr>0);
            %check for AND / OR malware type
            %1 if AND type, 2 if OR type
            if max(encounteredNodeExploitMalwareTypeEnabled) == 1 
                %AND genetic matching
                %disp('AND')
                infectionDetected=and(and(and(infectionDetectedApps,infectionDetectedServ),infectionDetectedOs),infectionDetectedDr);
            else
                %OR AND genetic matching
                %disp('OR')
                infectionDetected=or(or(or(infectionDetectedApps,infectionDetectedServ),infectionDetectedOs),infectionDetectedDr);
            end
            %**************************************
            
            
            
            
            %only enable those data that will actually cause an infection -
            %this step may not be necessary - depends if next step uses the
            %'infectionDetected' as an enable
            infectionDetectedRepRoute = repmat(infectionDetected,1,size(exploitData.exploitRoute,2));%extend to length of software
            infectionDetectedRepTx = repmat(infectionDetected,1,size(exploitData.exploitTxType,2));
            infectionDetectedRepProb = repmat(infectionDetected,1,size(exploitData.exploitTxProb,2));
            infectionDetectedRepDam = repmat(infectionDetected,1,size(exploitData.exploitDamageType,2));
            infectionDetectedRepMal = repmat(infectionDetected,1,size(exploitData.malwareType,2));
            infectionExploitData.exploitRoute = encounteredNodeExploitRouteEnabled.*infectionDetectedRepRoute;
            infectionExploitData.exploitTxType = encounteredNodeExploitTxTypeEnabled.*infectionDetectedRepTx;
            infectionExploitData.exploitTxProb = encounteredNodeExploitTxProbEnabled.*infectionDetectedRepProb;
            infectionExploitData.exploitDamageType = encounteredNodeExploitDamageTypeEnabled.*infectionDetectedRepDam;   
            infectionExploitData.malwareType = encounteredNodeExploitMalwareTypeEnabled.*infectionDetectedRepMal; 
            %infectionExploitDataExploitRoute=infectionExploitData.exploitRoute %test Display
            %infectionExploitDataExploitTxType=infectionExploitData.exploitTxType %test Display
            %infectionExploitDataExploitTxProb=infectionExploitData.exploitTxProb %test Display
            %infectionExploitDataExploitDamageType=infectionExploitData.exploitDamageType; %test Display
        end
        function [infectionTransmitted,infectionExploitData,latencyExploitData,txTypeLatencyCounter,latencyFlag] = checkExploitTxType(infectionDetected,infectionExploitData,latencyExploitData,txTypeLatencyCounter,latencyFlag)
        %only takes into account 1 exploit, because when it goes to I state
        %it stays there, would need a seperate state machine for each exploit type
        %auto - happen straight away
        %latency - after set time
        %user - based on a probability
        infectionExploitDataIncommingExploitRouteD = infectionExploitData.exploitRoute;
        
        exploitTxType=infectionExploitData.exploitTxType;
        txTypeDetectedValue = StateMachineClass.checkExploitTxTypeValue(infectionExploitData);
        txEnable = zeros(size(infectionDetected));
        %decision here about three types
        %1.automated
        txEnableAuto = (txTypeDetectedValue ==1);

        %2.Latency
        latTx = (txTypeDetectedValue ==2);
        latflgZero = (latencyFlag==0);
        %latencyFlag_temp = and(latTx,latflgZero) %only set if not set already
        %txTypeLatencyCounter=StateMachineClass.getTxTypeLatency().*latencyFlag_temp
        %infectionExploitDataexploitRouteD = infectionExploitData.exploitRoute
        %latencyExploitRoute = infectionExploitData.exploitRoute.*repmat(latencyFlag_temp,1,size(infectionExploitData.exploitRoute,2))
        
        %if latency type detected and latency flag not set, then need to
        %set it and start the latency counter
        for i =1:size(txEnableAuto,1) %for all nodes - dont know how to do without loop
            if and((txTypeDetectedValue(i) ==2),(latencyFlag(i)==0))
                latencyFlag(i) =1;
                txTypeLatencyCounter(i)=StateMachineClass.getTxTypeLatency();
                latencyExploitData.exploitRoute(i,:) = infectionExploitData.exploitRoute(i,:); %remember data for later
                latencyExploitData.exploitTxType(i,:) = infectionExploitData.exploitTxType(i,:);
                latencyExploitData.exploitDamageType(i,:) = infectionExploitData.exploitDamageType(i,:);
            end %if
        end %for
        %latencyFlag_tempD=latencyFlag
        %txTypeLatencyCounterD=txTypeLatencyCounter
        %latencyExploitDataexploitRouteD = latencyExploitData.exploitRoute
        
        %count down to zero if counter greater than 1
        txTypeLatencyCounter=txTypeLatencyCounter-1.*(txTypeLatencyCounter >0) ;
        %if counter now zero and latency flag set, enable the latency exploit to go through
        txEnableLat = and((txTypeLatencyCounter ==0),(latencyFlag==1)); 
        %assign saved exploit data to current infection data
        for i =1:size(txEnableAuto,1) %for all nodes - dont know how to do without loop
            if txEnableLat(i) ==1
                infectionExploitData.exploitRoute(i,:) = latencyExploitData.exploitRoute(i,:);
                infectionExploitData.exploitTxType(i,:) = latencyExploitData.exploitTxType(i,:);
                infectionExploitData.exploitDamageType(i,:) = latencyExploitData.exploitDamageType(i,:);
            end
        end
        
        %3.User
        userTx = (txTypeDetectedValue ==3);
        txRandomNumber = rand(size(userTx,1),1);
        txEnableUser = txRandomNumber < StateMachineClass.getTxTypeUserProbability();
        txEnableUser = txEnableUser.*userTx;
         
        infectionTransmitted=txEnableAuto+txEnableLat+txEnableUser;
        infectionExploitDataOutgoingExploitRoute = infectionExploitData.exploitRoute;
        
        end
        function txProbDetectedValue = checkExploitTxProbValue(infectionExploitData)
            txProbDetectedValue = infectionExploitData.exploitTxType .* infectionExploitData.exploitTxProb;
            txProbDetectedValue = sum(txProbDetectedValue,2);
        end
        function txTypeDetectedValue = checkExploitTxTypeValue(exploitData)
            %check to see what tx type it is
            if sum(sum(exploitData.exploitTxType,2))<size(exploitData.exploitTxType,1)
                %disp('not all transmission types are set')
            end
            [txValue txIndex] = sort(exploitData.exploitTxType,2,'descend');
            txTypeDetectedValue = txIndex(:,1);
            txTypeDetectedValue = txTypeDetectedValue.*sum(exploitData.exploitTxType,2);
        end
        function damageTypeDetectedValue= checkDamageTypeValue(exploitData)
            %check to see 
            if sum(sum(exploitData.exploitDamageType,2))<size(exploitData.exploitDamageType,1)
                %disp('not all transmission types are set')
            end
            [damValue damIndex] = sort(exploitData.exploitDamageType,2,'descend');
            damageTypeDetectedValue = damIndex(:,1); 
            damageTypeDetectedValue = damageTypeDetectedValue.*sum(exploitData.exploitDamageType,2);
        end
        function updatedExploitData = updateOwnExploitData(exploitData,infectionTransmitted,infectionExploitData)
              %infectionTransmittedD=infectionTransmitted %testing
              %exploitDataexploitRoute=exploitData.exploitRoute%testing
              %exploitDataexploitTxType=exploitData.exploitTxType%testing
              %exploitDataexploitTxProb=exploitData.exploitTxProb%testing
              %exploitDataexploitDamageType=exploitData.exploitDamageType%testing

              %infectionExploitDataexploitRoute=infectionExploitData.exploitRoute%testing
              %infectionExploitDataexploitTxType=infectionExploitData.exploitTxType%testing
              %infectionExploitDataexploitTxProb=infectionExploitData.exploitTxProb%testing
              %infectionExploitDataexploitDamageType=infectionExploitData.exploitDamageType%testing
              
              
              infTxRepRoute = repmat(infectionTransmitted,1,size(infectionExploitData.exploitRoute,2));
              infTxRepTx = repmat(infectionTransmitted,1,size(infectionExploitData.exploitTxType,2));
              infTxRepProb = repmat(infectionTransmitted,1,size(infectionExploitData.exploitTxProb,2));
              infTxRepDam = repmat(infectionTransmitted,1,size(infectionExploitData.exploitDamageType,2));
              inv_infTxRepRoute = (infTxRepRoute-1).*(-1);
              inv_infTxRepTx =(infTxRepTx-1).*(-1);
              inv_infTxRepProb =(infTxRepProb-1).*(-1);
              inv_infTxRepDam =(infTxRepDam-1).*(-1);

              newDataRoute = infectionExploitData.exploitRoute.*infTxRepRoute;
              newDataTx = infectionExploitData.exploitTxType.*infTxRepTx;
              newDataProb = infectionExploitData.exploitTxProb.*infTxRepProb;
              newDataDam = infectionExploitData.exploitDamageType.*infTxRepDam;
              oldDataRoute = exploitData.exploitRoute.*inv_infTxRepRoute;
              oldDataTx = exploitData.exploitTxType.*inv_infTxRepTx;
              oldDataProb = exploitData.exploitTxProb.*inv_infTxRepProb;
              oldDataDam = exploitData.exploitDamageType.*inv_infTxRepDam;
              
              %updatedExploitDatexploitRoute = newDataRoute + oldDataRoute%testing
              %updatedExploitDatexploitTxType = newDataTx+ oldDataTx%testing
              %updatedExploitDataexploitTxProb = newDataProb+ oldDataProb%testing
              %updatedExploitDatexploitDamageType = newDataDam + oldDataDam%testing
              
              
              updatedExploitData.exploitRoute = newDataRoute + oldDataRoute;
              updatedExploitData.exploitTxType = newDataTx+ oldDataTx;
              updatedExploitData.exploitTxProb = newDataProb+ oldDataProb;
              updatedExploitData.exploitDamageType = newDataDam + oldDataDam;
        end
        function savedInfectionExploitData = saveInfectionExploitData(infectionExploitData, saveInfectionExploitData, savedExploitData)
                %save new
              infTxRepRoute = repmat(saveInfectionExploitData,1,size(infectionExploitData.exploitRoute,2));
              infTxRepTx = repmat(saveInfectionExploitData,1,size(infectionExploitData.exploitTxType,2));
              infTxRepProb = repmat(saveInfectionExploitData,1,size(infectionExploitData.exploitTxProb,2));
              infTxRepDam = repmat(saveInfectionExploitData,1,size(infectionExploitData.exploitDamageType,2));
              infTxRepMal = repmat(saveInfectionExploitData,1,size(infectionExploitData.malwareType,2));
              
              ToSaveRoute = infectionExploitData.exploitRoute.*infTxRepRoute;
              ToSaveTx = infectionExploitData.exploitTxType.*infTxRepTx;
              ToSaveProb = infectionExploitData.exploitTxProb.*infTxRepProb;
              ToSaveDam = infectionExploitData.exploitDamageType.*infTxRepDam;
              ToSaveMal = infectionExploitData.malwareType.*infTxRepMal;
              
              %keep old
              inv_infTxRepRoute = (infTxRepRoute-1).*(-1);
              inv_infTxRepTx =(infTxRepTx-1).*(-1);
              inv_infTxRepProb =(infTxRepProb-1).*(-1);
              inv_infTxRepDam =(infTxRepDam-1).*(-1);
              inv_infTxRepMal =(infTxRepMal-1).*(-1);
            
              OldRouteToKeep = savedExploitData.exploitRoute.*inv_infTxRepRoute;
              OldTxToKeep = savedExploitData.exploitTxType.*inv_infTxRepTx;
              OldProbToKeep = savedExploitData.exploitTxProb.*inv_infTxRepProb;
              OldDamToKeep = savedExploitData.exploitDamageType.*inv_infTxRepDam;
              OldMalToKeep = savedExploitData.malwareType.*inv_infTxRepMal;
              
              %Saved
              savedRoute = ToSaveRoute + OldRouteToKeep;
              savedTx = ToSaveTx + OldTxToKeep;
              savedProb = ToSaveProb + OldProbToKeep;
              savedDam = ToSaveDam + OldDamToKeep;
              savedMal = ToSaveMal + OldMalToKeep;

              savedInfectionExploitData.exploitRoute=savedRoute;
              savedInfectionExploitData.exploitTxType=savedTx;
              savedInfectionExploitData.exploitTxProb=savedProb;
              savedInfectionExploitData.exploitDamageType=savedDam;
              savedInfectionExploitData.malwareType=savedMal;
        end
                function ownExploitDataNew = updateResetOwnExploitData(infectionExploitData, saveInfectionExploitData, savedExploitData, resetExploitData)
              %disp('in updateOwnExploitData function')
              %savedExploitDataIn = infectionExploitData
              %ownExploitDataIn = savedExploitData
              %updateOwnEnableIn = saveInfectionExploitData
              %resetExploitDataIn = resetExploitData
              
              %update own exploit data with saved data
              infTxRepRoute = repmat(saveInfectionExploitData,1,size(infectionExploitData.exploitRoute,2));
              infTxRepTx = repmat(saveInfectionExploitData,1,size(infectionExploitData.exploitTxType,2));
              infTxRepProb = repmat(saveInfectionExploitData,1,size(infectionExploitData.exploitTxProb,2));
              infTxRepDam = repmat(saveInfectionExploitData,1,size(infectionExploitData.exploitDamageType,2));
              infTxRepMal = repmat(saveInfectionExploitData,1,size(infectionExploitData.malwareType,2));
              
              ToSaveRoute = infectionExploitData.exploitRoute.*infTxRepRoute;
              ToSaveTx = infectionExploitData.exploitTxType.*infTxRepTx;
              ToSaveProb = infectionExploitData.exploitTxProb.*infTxRepProb;
              ToSaveDam = infectionExploitData.exploitDamageType.*infTxRepDam;
              ToSaveMal = infectionExploitData.malwareType.*infTxRepMal;
              
              %keep old
              %inv_infTxRepRoute = (infTxRepRoute-1).*(-1);
              %inv_infTxRepTx =(infTxRepTx-1).*(-1);
              %inv_infTxRepProb =(infTxRepProb-1).*(-1);
              %inv_infTxRepDam =(infTxRepDam-1).*(-1);
            
              %OldRouteToKeep = savedExploitData.exploitRoute.*inv_infTxRepRoute;
              %OldTxToKeep = savedExploitData.exploitTxType.*inv_infTxRepTx;
              %OldProbToKeep = savedExploitData.exploitTxProb.*inv_infTxRepProb;
              %OldDamToKeep = savedExploitData.exploitDamageType.*inv_infTxRepDam;
            
              %Saved
              %savedRoute = ToSaveRoute + OldRouteToKeep;
              %savedTx = ToSaveTx + OldTxToKeep;
              %savedProb = ToSaveProb + OldProbToKeep;
              %savedDam = ToSaveDam + OldDamToKeep;

              ownExploitDataNew.exploitRoute=ToSaveRoute;
              ownExploitDataNew.exploitTxType=ToSaveTx;
              ownExploitDataNew.exploitTxProb=ToSaveProb;
              ownExploitDataNew.exploitDamageType=ToSaveDam;
              ownExploitDataNew.malwareType=ToSaveMal;
        end
        function displayExploitData(exploitData)
            %function for test purposes to display the export data
            %testing
            exploitRouteDisplay = exploitData.exploitRoute
            exploitTxProbDisplay = exploitData.exploitTxProb
            exploitTxTypeDisplay = exploitData.exploitTxType
            exploitDamageTypeDisplay = exploitData.exploitDamageType
            exploitMalwareTypeDisplay = exploitData.malwareType
        end
        function startExploitInfection(t,ihandles,rhandles)
            %check if exploit is enabled
            if ihandles.exploitEnable == 1
            infectionTime = ihandles.infectionTime*GUIClass.convertTimeUnits(ihandles.infectTimeUnits);       
            if t == infectionTime
                %check how many nodes to infect
              %  for i = 1:ihandles.numInitialInfected
                    %set exploit data
                    %check virus type to set
                    if ihandles.virusType ==1
                        rhandles.exploitData.exploitTxType(1:ihandles.numInitialInfected,1)=1; %auto
                        rhandles.exploitData.exploitTxProb(1:ihandles.numInitialInfected,1)=1; %autoProb = 1
                    elseif ihandles.virusType ==2
                        rhandles.exploitData.exploitTxType(1:ihandles.numInitialInfected,2)=1; %latency
                        rhandles.exploitData.exploitTxProb(1:ihandles.numInitialInfected,2)=ihandles.Pei2*GUIClass.convertTimeUnits(ihandles.latUnits); %latency time
                    elseif ihandles.virusType ==3
                        rhandles.exploitData.exploitTxType(1:ihandles.numInitialInfected,3)=1; %user
                        rhandles.exploitData.exploitTxProb(1:ihandles.numInitialInfected,3)=ihandles.Pei1; %userProb = 0.025
                    end
                    %check malware type to set
                    rhandles.exploitData.malwareType(1:ihandles.numInitialInfected,:)=ihandles.malwareType; %1 AND, 0 OR
                    %set exploit routes
                    extendedExploitRoute = repmat(ihandles.exploitRoute,ihandles.numInitialInfected,1); %extend to avoid a for loop
                    rhandles.exploitData.exploitRoute(1:ihandles.numInitialInfected,:)=extendedExploitRoute; %Need to set this to infect
                    %set saved data
                    rhandles.savedExploitData.exploitRoute(1:ihandles.numInitialInfected,:)= extendedExploitRoute;
                    rhandles.savedExploitData.exploitTxType(1:ihandles.numInitialInfected,:)= rhandles.exploitData.exploitTxType(1:ihandles.numInitialInfected,:);
                    rhandles.savedExploitData.exploitTxProb(1:ihandles.numInitialInfected,:)= rhandles.exploitData.exploitTxProb(1:ihandles.numInitialInfected,:);
                    rhandles.savedExploitData.malwareType(1:ihandles.numInitialInfected,:)= rhandles.exploitData.malwareType(1:ihandles.numInitialInfected,:);
                    %check which state machine using so change to correct state
                    infectedStateNumber = StateMachineClass.getStateNumber('I',ihandles.attackModelS);
                    susceptibleStateNumber = 1 ;%always 1 in all cases
                    rhandles.StateMachine(1:ihandles.numInitialInfected,infectedStateNumber)=1;
                    rhandles.StateMachine(1:ihandles.numInitialInfected,susceptibleStateNumber)=0;
             %   end %for loop
            end %t
            end %exploit enable
            %DISPLAY TEST TO CHECK
            %rhandlesexploitDataexploitRoute=rhandles.exploitData.exploitRoute
            %rhandlesexploitDataexploitTxType=rhandles.exploitData.exploitTxType
            %rhandlesexploitDataexploitTxProb=rhandles.exploitData.exploitTxProb
            %rhandlesexploitDatamalwareType= rhandles.exploitData.malwareType
        end %func
        %*****************************
        %Simulation initialisations
        %*****************************
        function exploitSimInit(mainhandles, ihandles)
            %function for initialising exploit for a simulation
            handles = guidata(mainhandles.editExploitFig); %load data
            ihandles.exploitApp = get(handles.editExploitApp, 'Value');
            ihandles.exploitServ = get(handles.editExploitServ, 'Value');
            ihandles.exploitOS = get(handles.editExploitOS, 'Value');
            ihandles.exploitDr = get(handles.editExploitDr, 'Value');
            ihandles.exploitRoute = [ihandles.exploitApp ihandles.exploitServ ihandles.exploitOS ihandles.exploitDr];
            ihandles.malwareType = get(handles.popupmenuMalwareType, 'Value');
            ihandles.virusType = get(handles.popupmenuVirusType, 'Value');
            ihandles.Pei1 = get(handles.editPei1, 'Value');
            ihandles.Pei2 = get(handles.editPei2, 'Value');
            ihandles.latUnits = get(handles.popupmenuLatUnits, 'Value');
            ihandles.infectionTime = get(handles.editInfectionTime, 'Value');
            ihandles.infectTimeUnits = get(handles.popupmenuInfectTimeUnits, 'Value');
            ihandles.timeToTxVirus = get(handles.editTimeToTxVirus, 'Value');
            ihandles.timeToTxVirusUnits = get(handles.popupmenuTransmitTimeUnits, 'Value');
            ihandles.numInitialInfected = get(handles.editNumInitialInfected, 'Value');
            ihandles.timeToTxGenotype = get(handles.editTimeToTxGenotype, 'Value');
            ihandles.timeToTxGenotypeUnits = get(handles.popupmenuTimeToTxGenotypeUnits, 'Value');
            ihandles.timeOutPeriodTxGenotype = get(handles.editTimeOutPeriodTxGenotype, 'Value');
            ihandles.timeOutPeriodTxGenotypeUnits = get(handles.popupmenuTimeOutPeriodTxGenotypeUnits, 'Value');
        end
        function attackModelSISimInit(mainhandles, ihandles)
            %function for initialising SI for a simulation
            %handles = guidata(mainhandles.attackModelParameters_SIFig); %load data
            %no parameters to load
        end
        function attackModelSIRSimInit(mainhandles, ihandles)
            %function for initialising SIR for a simulation
            handles = guidata(mainhandles.attackModelParameters_SIRFig); %load data
            ihandles.SIRrecovProb = get(handles.editRecovProb, 'Value');
            ihandles.SIRrecovTime = get(handles.editRecovTime, 'Value');
        end
        function attackModelSEIRSimInit(mainhandles, ihandles)
            %function for initialising SEIR for a simulation
            handles = guidata(mainhandles.attackModelParameters_SEIRFig); %load data
            ihandles.SEIRrecovProb = get(handles.editRecoveryProb, 'Value');
        end
        function attackModelSEPTICOXSimInit(mainhandles, ihandles)
            %function for initialising SEPTICOX for a simulation
            handles = guidata(mainhandles.attackModelParameters_SEPTICOXFig); %load data
            ihandles.DevicesOffline = get(handles.editDevicesOffline, 'Value');
            ihandles.Pos1 = get(handles.editPos1, 'Value');
            ihandles.Pos2 = get(handles.editPos2, 'Value');
            ihandles.Pso1 = get(handles.editPso1, 'Value');
            ihandles.Pso2 = get(handles.editPso2, 'Value');
            ihandles.Pex = get(handles.editPex, 'Value');
            ihandles.Pxe = get(handles.editPxe, 'Value');
            ihandles.Pto1 = get(handles.editPto1, 'Value');
            ihandles.Pto2 = get(handles.editPto2, 'Value');
            ihandles.Psp = get(handles.editPsp, 'Value');
            ihandles.Pet = get(handles.editPet, 'Value');
            ihandles.Pit = get(handles.editPit, 'Value');
            ihandles.Pts = get(handles.editPts, 'Value');
            ihandles.Ptp = get(handles.editPtp, 'Value');
            ihandles.Pic = get(handles.editPic, 'Value');
            ihandles.Pci = get(handles.editPci, 'Value');
            ihandles.FreqOn = get(handles.editFreqOn, 'Value');
            ihandles.PeakOn = get(handles.editPeakOn, 'Value');
            ihandles.SpreadOn = get(handles.editSpreadOn, 'Value');
            ihandles.AmpOn = get(handles.editAmpOn, 'Value');
            ihandles.FreqOff = get(handles.editFreqOff, 'Value');
            ihandles.PeakOff = get(handles.editPeakOff, 'Value');
            ihandles.SpreadOff = get(handles.editSpreadOff, 'Value');
            ihandles.AmpOff = get(handles.editAmpOff, 'Value');
            ihandles.UsePeriodic = get(handles.checkboxUsePeriodic, 'Value');
            ihandles.PlotPeriod = get(handles.checkboxPlotPeriod, 'Value');
        end
        function attackModelSimInit(handles,ihandles,ohandles) 
            %init inputs
            StateMachineClass.exploitSimInit(handles, ihandles)
            StateMachineClass.attackModelSISimInit(handles, ihandles)
            StateMachineClass.attackModelSIRSimInit(handles, ihandles)
            StateMachineClass.attackModelSEIRSimInit(handles, ihandles)
            StateMachineClass.attackModelSEPTICOXSimInit(handles, ihandles)
            %init outputs
            numStates = StateMachineClass.getNumStates(ihandles);
            ohandles.sumOfStates = zeros(ihandles.numRunsPerSim,numStates,ihandles.numTimeSteps);
            %ohandles.nodeStateAtTime = zeros(ihandles.numRunsPerSim,ihandles.numTimeSteps,ihandles.numNodes); %state of each node at time t
            %AHJ ohandles.allRunsNodeStateAtTime = zeros(ihandles.numRunsPerSim);
            ohandles.allRunsNodeStateAtTime.nodeStateAtTimeStr = cell(ihandles.numTimeSteps,ihandles.numNodes);
            ohandles.allRunsNodeStateAtTime.nodeStateAtTimeNumber = zeros(ihandles.numTimeSteps,ihandles.numNodes);
            ohandles.beta = zeros(ihandles.numRunsPerSim,ihandles.numTimeSteps); %measures the instantaneous value of beta(infection rate)for every run
            ohandles.gammaSIR = zeros(ihandles.numRunsPerSim,ihandles.numTimeSteps);
        end
        %***********************************
        %Run initialisations
        %***********************************
        function exploitRunInit(ihandles,rhandles)
            if ihandles.exploitEnable == 1
            rhandles.exploitData = StateMachineClass.genExploitData(ihandles.numNodes,ihandles.len_sw);
            rhandles.savedExploitData = StateMachineClass.genExploitData(ihandles.numNodes,ihandles.len_sw);
            end
        end
        %
        function attackModelRunInit(ihandles,rhandles)
        %set state machine run variables
        if ihandles.attackModelEnable == 1
            switch (ihandles.attackModelS)
                case 'SI'
                    rhandles.StateMachine = StateMachineClass.genSIStateMachine(ihandles.numNodes);
                    rhandles.numStates = 2; 
                case 'SIR'
                    rhandles.StateMachine = StateMachineClass.genSIRStateMachine(ihandles.numNodes);
                    rhandles.numStates = 3;
                case 'SEIR'
                    rhandles.StateMachine = StateMachineClass.genSEIRStateMachine(ihandles.numNodes);
                    rhandles.numStates = 4;
                case 'SEPTICOX'
                    rhandles.StateMachine = StateMachineClass.genSEPTICOXStateMachine(ihandles.numNodes);
                    rhandles.numStates = 8;
            end %switch
            rhandles.sumOfStates = zeros(rhandles.numStates,ihandles.numTimeSteps); %keeps tally of how many in each state over time
            rhandles.beta = zeros(1,ihandles.numTimeSteps); %measures the instantaneous value of beta(infection rate)
            rhandles.S_buffer = zeros(1,2); %buffers S for 2 time steps, used in beta calc
            rhandles.I_buffer = zeros(1,2);  %buffers I for 2 time steps, used in beta calc
            rhandles.gammaSIR = zeros(1,ihandles.numTimeSteps); %measures the instantaneous value of beta(infection rate)
            rhandles.nodeStateAtTimeStr = cell(ihandles.numTimeSteps,ihandles.numNodes); %state of each node at time t
            rhandles.nodeStateAtTimeNumber = zeros(ihandles.numTimeSteps,ihandles.numNodes); %state of each node at time t
            rhandles.nextStateNumber = zeros(ihandles.numNodes,1);
            rhandles.saveInfectionExploitDataEnable = zeros(ihandles.numNodes,1);
            rhandles.updateOwnExploitDataEnable = zeros(ihandles.numNodes,1);
            rhandles.resetOwnExploitDataEnable = zeros(ihandles.numNodes,1);
            rhandles.deviceOffline = zeros(ihandles.numNodes,1);
            rhandles.NewNextState_Eentry = zeros(ihandles.numNodes,1);
            rhandles.NewNextState_Eprob = zeros(ihandles.numNodes,1);
            rhandles.NewNext_LatencyCounter = zeros(ihandles.numNodes,1);
            rhandles.deltaT = zeros(1,1);                   %time interval - set to 1 means 1 second
            rhandles.latCountOption = zeros(1,1);
            rhandles.currentE_entryState = zeros(ihandles.numNodes,1);
            rhandles.currentE_entryProb = zeros(ihandles.numNodes,1);
            rhandles.current_LatencyCounter = zeros(ihandles.numNodes,1);
        end %if
        end %func

        %**********************************************************************************
        % Attack model main program function
        %**********************************************************************************
        function attackModelRunTloop(t,lengthCounter,ihandles,rhandles)
            if ihandles.attackModelEnable == 1
            %check incomming exploit data against genotype
            %check what mode running in - if RWP selected AND in 'Available
            %in range with a virus tx time' then used the virustx flag as
            %the exploit encounter, otherwise use the encounterYes flag as
            %normal
            %disp('now in attackModelRunTloop')
            if and(ihandles.networkModelEnable,strcmp(ihandles.RWPmode,'Available in range with a virus tx time'))
                %disp('Available in range with a virus tx time, change encounterYes to virustx')
                encounterYes = rhandles.RWPNodeTagged(rhandles.RWPvirusTransmitted,:)'
            elseif and(ihandles.networkModelEnable,strcmp(ihandles.RWPmode,'Independent geno-exploit'))
                %disp('Independent geno-exploit change encounterYes for attack model to txExploitEncounterYes')
                encounterYes = rhandles.txExploitEncounterYes';
                %encounterYesD = encounterYes'; %testing
                %randomEncounterD = rhandles.randomEncounter' %testing
            else
                %disp('encounterYes as normal')
                encounterYes = rhandles.encounterYes;
            end
            
            [infectionDetected, infectionExploitData] = StateMachineClass.checkExploitRoute(ihandles,rhandles.randomEncounter,encounterYes,rhandles.exploitData,rhandles.GenotypeSol4);


            
            %check against transmission type
            txTypeDetectedValue = StateMachineClass.checkExploitTxTypeValue(infectionExploitData);
            %check against transmission Prob
            txProbDetectedValue = StateMachineClass.checkExploitTxProbValue(infectionExploitData);     
            %TESTING check current state of nodes for testing
            currentStateString = StateMachineClass.getCurrentStateStringAllNodes(rhandles.StateMachine);
            %Decide on the next state
            StateMachineClass.getNextState(t,ihandles,rhandles,infectionDetected,txTypeDetectedValue,txProbDetectedValue);
            %save infection data if required when moving out of susceptible state
            rhandles.savedExploitData = StateMachineClass.saveInfectionExploitData(infectionExploitData, rhandles.saveInfectionExploitDataEnable, rhandles.savedExploitData);
            
            %TESTING - display export data before
            %StateMachineClass.displayExploitData(exploitData);
            %update own exploit data if required when in the infected state so that can infect others
            rhandles.exploitData = StateMachineClass.updateResetOwnExploitData(rhandles.savedExploitData,rhandles.updateOwnExploitDataEnable,rhandles.exploitData, rhandles.resetOwnExploitDataEnable);

            %TESTING - display export data after
            %StateMachineClass.displayExploitData(rhandles.savedExploitData);
            %StateMachineClass.displayExploitData(rhandles.exploitData);
            
            %Change the state
            rhandles.StateMachine = StateMachineClass.changeState(rhandles.nextStateNumber, rhandles.StateMachine);
            %TESTING check current state of nodes for testing
            stateStringAfter = StateMachineClass.getCurrentStateStringAllNodes(rhandles.StateMachine);
            %sum states for output
            %StateMachineClass.sumAllStates(t,rhandles);
            StateMachineClass.sumAllStates(lengthCounter,rhandles); %updated 21st April 2016
            %measure beta infection rate
            StateMachineClass.calcBeta(t,lengthCounter,rhandles); %updated 21st April 2016
            %measure gamma SIR recovery rate - take out for now as need SIR
            %StateMachineClass.calcGammaSIR(t,rhandles);
            %sizerhandlesnodeStateAtTime = size(rhandles.nodeStateAtTimeStr)
            %sizecurrentstatestring = size(stateStringAfter)
            stateStringAfterT = StateMachineClass.getCurrentStateStringAllNodes(rhandles.StateMachine);
            stateNumberAfterT = StateMachineClass.getCurrentStateNumberAllNodes(rhandles.StateMachine);
            at = stateStringAfterT';
            bt = stateNumberAfterT';
            %rhandles.nodeStateAtTimeStr(t,:) = at;
            %rhandles.nodeStateAtTimeNumber(t,:) = bt;
            rhandles.nodeStateAtTimeStr(lengthCounter,:) = at; %changed 21st April 2016
            rhandles.nodeStateAtTimeNumber(lengthCounter,:) = bt; %changed 21st April 2016
            %Str_rhandlesnodeStateAtTimeDisplay=rhandles.nodeStateAtTimeStr(t,:)
            %Number_rhandlesnodeStateAtTimeDisplay=rhandles.nodeStateAtTimeNumber(t,:)
            end
        end
        %********************************************
        % Attack Model - assign outputs
        %********************************************
        function AttackModelAssignOutputs(runNumber,ihandles,rhandles,ohandles)
            if ihandles.attackModelEnable == 1
            %from rhandles
            ohandles.sumOfStates(runNumber,:,:)=rhandles.sumOfStates;
            ohandles.allRunsNodeStateAtTime(runNumber).nodeStateAtTimeStr=rhandles.nodeStateAtTimeStr;
            ohandles.allRunsNodeStateAtTime(runNumber).nodeStateAtTimeNumber=rhandles.nodeStateAtTimeNumber;
            ohandles.numStates = rhandles.numStates;
            ohandles.beta(runNumber,:) = rhandles.beta;
            ohandles.gammaSIR(runNumber,:) = rhandles.gammaSIR;
            
            %TEST
           % ohandlesallRunsNodeStateAtTimenodeStateAtTimeStr=ohandles.allRunsNodeStateAtTime(runNumber).nodeStateAtTimeStr
          %  ohandlesallRunsNodeStateAtTimenodeStateAtTimeNumber=ohandles.allRunsNodeStateAtTime(runNumber).nodeStateAtTimeNumber
            end
        end
        
    end %end static methods
    
end

