%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Jennifer Jackson 2017                                                        %
% A Biodiversity Approach to Cyber Security                                    %
% PhD Complexity Science                                                       %
% University of Warwick                                                        %
% http://wrap.warwick.ac.uk/98458                                              %
% This software is distributed under the  Creative Commons Attribution License %
% https://creativecommons.org/licenses/by/4.0/                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [ output_args ] = testRandomGenotypes( )
clear all;
%IMPORTANT NOTE: monteCarloSIRN2 changed from SEpidemic=numNodes - 60 input to SEpidemic=60;
%*************************************
% TESTING AGENT BASED STOCHASIC MODELS
%*************************************
outputDirectory = 'agentSIRGraphs\'
close all;

doBGTests = zeros(1,10) %tests for background
doTests = zeros(1,10) %tests with no genotypes
doSIRNTests = zeros(1,30) %tests with genotypes
%*****************
% BACKGROUND TESTS
%****************
%outputDirectory = 'E:\Users\Jen\Documents\ComplexityScience\PhD\simulations\Matlab\AgentModels\BGGraphs\'
%doBGTests(1) = 1 %contact rate averages out over time, instantaneous contact is probablistic
%doBGTests(2) = 1 %SI beta tests beta(contact rate set) v's measured beta
%doBGTests(3) = 1 %beta error test
%doBGTests(4) = 1 %exponential pdf distribution
%***************

%***************
% SIR BASIC TESTS - no genotypes
%**************
%outputDirectory = 'agentSIRGraphs\' 
%uses monteCarloAgentSIR - basic SIR model with no genotypes
%doTests(1) = 1 %background model SI single beta with graphs- compare to deterministic
%doTests(2) = 1 %background model SIR - compare to SIR deterministic
%doTests(3) = 1 %background model SIR single run graphs - compare to SIR deterministic
%%%%doTests(4) = 1 %background model SIR vary beta, 0.45, 0.50, 0.55, gamma==0.50, compare against deterministic tests
%doTests(5) = 1 %background model SIR vary beta = [0.50 0.45 0.40 0.35 0.30 0.25 0.20 0.15 0.10], fix gamma = 0.3, minor/major separation = 900

%***************
% SIRN TESTS - with genotypes
%**************
%outputDirectory = 'E:\Users\Jen\Documents\ComplexityScience\PhD\simulations\Matlab\AgentModels\randomGenotypesGraphs\'
%uses monteCarloSIRN   - SIR model with dynamic and static genotypes
%doSIRNTests(1) = 1 %SI single beta with graphs- compare to deterministic and basic SIR doTest(1) above
%doSIRNTests(2) = 1 %SI Increase variants upto 8, fix exploit - single locus
%doSIRNTests(3) = 1 %SI fix variants at 8 and increase number of exploits - single locus
%doSIRNTests(4) = 1 %SI AND CASE Increase variants upto 8, fix exploit - 2 loci
%doSIRNTests(5) = 1 %SI AND CASE fix variants at 8 and increase number of exploits - 2 loci
%doSIRNTests(6) = 1 %SI OR CASE Increase variants upto 8, fix exploit - 2 loci
%doSIRNTests(7) = 1 %SI OR CASE fix variants at 8 and increase number of exploits - 2 loci
%uses monteCarloSIRN2   - SIR model with dynamic and static genotypes
%doSIRNTests(8) = 1 %SIR Increase variants upto 8, fix exploit - single locus
%doSIRNTests(9) = 1 %SIR fix variants at 8 and increase number of exploits - single locus
%doSIRNTests(10) = 1 %SIR AND CASE Increase variants upto 8, fix exploit - 2 loci
%doSIRNTests(11) = 1 %SIR AND CASE fix variants at 8 and increase number of exploits - 2 loci
%doSIRNTests(12) = 1 %SIR OR CASE Increase variants upto 8, fix exploit - 2 loci
%doSIRNTests(13) = 1 %SIR OR CASE fix variants at 8 and increase number of exploits - 2 loci
doSIRNTests(14) = 1 %SIR OR CASE Increase variants upto 30, fix exploit 1 - 4 loci
%doSIRNTests(15) = 1 %SIR AND CASE Increase variants upto 8, fix exploit 1 to 3 - 4 loci
%************************
%SUB FUNCTIONS
%************************
%monteCarloSIR
%monteCarloSIRN   - SIR model with dynamic and static genotypes
%betaErrorTest
%monteCarloAgentSIR - basic SIR model with no genotypes


%*************************************************************************************************
% TESTING






%***************
%SIR TEST 1 compare SI agent, stochastic, deterministic
%*********************
if doTests(1) == 1  
outputDirectory = 'agentSIRGraphs\'    
%SI single beta with graphs - compare to deterministic
%numRuns = 500,numNodes =100, beta=0.5, numTimesteps=60, plotGraphs=0, plotGraphsSIRN = 0, gamma=0,    throwAway = 'nonEpidemics', SEpidemic=99;
%numRuns = 10000,numNodes =1000, beta=0.05, numTimesteps=1000, plotGraphs=0, plotGraphsSIRN = 0, gamma=0,    throwAway = 'nonEpidemics', SEpidemic=99;
%****************************************************
dtt = 0.01
timeRun = 80
numRuns = 1000,
numNodes =1000, 
beta=0.3,
gamma = 0,
maxRunPlot=50
%stochastic SI
numsteps = floor(timeRun/dtt); %7000
time = 0:dtt:numsteps*dtt-dtt;
for i = 1:numRuns
 [ Ss(i,:) Is(i,:) Rs(i,:) ] = stochasticSIR(numNodes, beta, numsteps, dtt, gamma);
end

 figI = figure
 for i = 1:maxRunPlot
 plot(time,Is(i,:),'k')
 hold on
 end
 xlabel('Time')
 ylabel('Infected')
 
  figR = figure
 for i = 1:maxRunPlot
 plot(time,Rs(i,:),'k')
 hold on
 end
 xlabel('Time')
 ylabel('Recovered')
 
meanIs=mean(Is);
meanSs=mean(Ss);
sizemeanIs = size(meanIs)


%agent SI
 numTimesteps=timeRun, plotGraphs=0, plotGraphsSIRN = 0, gamma=0,    throwAway = 'nonEpidemics', SEpidemic=99;

beta = beta*dtt, gamma = gamma*dtt, numTimesteps = floor(numTimesteps/dtt), timeScale = 0:dtt:timeRun-dtt %set all with same dtt

[fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetanew fighistGammanew figInfectedRun1 figBetaRun1  figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
    betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
    betameanmean gammameanmean betameanmean2 betameanmeanNew betameanmeanNew2 gammameanmeanNew Smean Imean Rmean Smedian Imedian Rmedian S I R SmeanN ImeanN RmeanN Snewendmean Snewendstd Rnewendmean Rnewendstd Rnew_nonendmean Rnew_nonendstd propSimsNonEp propSimsNonEp_theoretical] = monteCarloAgentSIR(numRuns,numNodes, beta, numTimesteps, plotGraphs, plotGraphsSIRN, gamma,throwAway,SEpidemic);


sfigSI=figure;
for  i = 1:maxRunPlot
    i
    plot(S(i,:),'k')
    hold on
end
xlabel('Timesteps')
ylabel('Number of susceptible individuals S(t)')
title('a) susceptibles - 50 runs')

ifigSI=figure;
for  i = 1:maxRunPlot
    plot(I(i,:),'k')
    hold on
end
xlabel('Timesteps')
ylabel('Number of infected individuals I(t)')
title('b) infected - 50 runs')


%Deterministic SI
N=numNodes %100
S = N %total number of susceptible nodes in network
R=0, I=1, drawPlots=0,
%beta = 0.003, numTimesteps = 8000
[DRo StD ItD RtD] = SIRDeterministic(beta, gamma, S, I, R, numTimesteps, drawPlots);
t2 = 1:1:numTimesteps;
ItD2 = ItD(1:1:end);

 step = numTimesteps / length(ItD);
 ts = [0:step:numTimesteps-step];
 size(ts)
 size(ItD)


agentSImeanfig = figure
  %plot(Smean,'k--', 'Linewidth',2)%agent
   hold on
%   size(timeD)
   size(Imean)
   %plot(ts,ItD,'k.-', 'Linewidth',1)%deterministic
  %plot(t2,ItD2,'k.-', 'Linewidth',1)%deterministic with no dtt
  %plot(timeD,ItD,'k.-', 'Linewidth',1)%deterministic with dtt set
   plot(timeScale,ItD,'k', 'Linewidth',2)%deterministic with dtt set
   %plot(timeD,Imean,'k', 'Linewidth',1)%agent
   plot(timeScale,Imean,'k', 'Linewidth',1)%agent
 %plot(time,meanIs,'k--', 'Linewidth',1) %stochastic
 plot(timeScale,meanIs,'k--', 'Linewidth',1) %stochastic
 xlabel('Time')
 %ylabel('Infected')
 text=['c) SI mean of ' num2str(numRuns) ' runs, betamean: ',num2str(betameanmean)];
title(text)
legend('Deterministic I(t)','Agent I(t)',  'Stochastic I(t)')

%agentSImeanfig = figure
%plot(Smean,'k--')
%hold on
%plot(Imean,'k', 'Linewidth',2)
%plot(ts,ItD,'k.-')
%text=['c) SI mean of ' num2str(numRuns) ' runs, betamean: ',num2str(betameanmean)];
%title(text)
%legend('S(t)', 'I(t)', 'Deterministic I(t)')


betameanmean
gammameanmean
betameanmeanNew
gammameanmeanNew
betameanmean2
betameanmeanNew2

betastd
gammastd 
betastd2 
betastdNew 
betastdNew2 
gammastdNew

peakTimemean 
peakTimestd 
peakValuemean 
peakValuestd

saveas(fighistBeta,[outputDirectory 'Test1fighistBeta.fig']);
saveas(fighistBeta,[outputDirectory 'Test1fighistBeta.emf']);
saveas(fighistGamma,[outputDirectory 'Test1fighistGamma.fig']);
saveas(fighistGamma,[outputDirectory 'Test1fighistGamma.emf']);

saveas(fighistRecovered,[outputDirectory 'Test1fighistRecovered.fig']);
saveas(fighistRecovered,[outputDirectory 'Test1fighistRecovered.emf']);
saveas(fighistSusceptible,[outputDirectory 'Test1fighistSusceptible.fig']);
saveas(fighistSusceptible,[outputDirectory 'Test1fighistSusceptible.emf']);
saveas(fighistpeakValue,[outputDirectory 'Test1fighistpeakValue.fig']);
saveas(fighistpeakValue,[outputDirectory 'Test1fighistpeakValue.emf']);
saveas(fighistpeakTime,[outputDirectory 'Test1fighistpeakTime.fig']);
saveas(fighistpeakTime,[outputDirectory 'Test1fighistpeakTime.emf']);

saveas(sfigSI,[outputDirectory 'Test1sfigSI.fig']);
saveas(sfigSI,[outputDirectory 'Test1sfigSI.emf']);
saveas(ifigSI,[outputDirectory 'Test1ifigSI.fig']);
saveas(ifigSI,[outputDirectory 'Test1ifigSI.emf']);
saveas(agentSImeanfig,[outputDirectory 'Test1agentSImeanfig.fig']);
saveas(agentSImeanfig,[outputDirectory 'Test1agentSImeanfig.emf']);
saveas(figBetaRun1,[outputDirectory 'Test1figBetaRun1SI.fig']);
saveas(figBetaRun1,[outputDirectory 'Test1figBetaRun1SI.emf']);
close all;


  plot(timeScale,ItD,'k', 'Linewidth',2)%deterministic with dtt set
   plot(timeScale,Imean,'k', 'Linewidth',1)%agent
 plot(timeScale,meanIs,'k--', 'Linewidth',1) %stochastic

 timeScale1=timeScale;
 ItD1=ItD%deterministic
 Imean1=Imean %agent
meanIs1=meanIs %stochastic
 
%**************************************************************************************
%dtt set to 1
dtt = 1
timeRun = 80
numRuns = 1000,
numNodes =1000, 
beta=0.3,
gamma = 0,
maxRunPlot=50
%stochastic SI
numsteps = floor(timeRun/dtt); %7000
time = 0:dtt:numsteps*dtt-dtt;
for i = 1:numRuns
 [ Ss2(i,:) Is2(i,:) Rs2(i,:) ] = stochasticSIR(numNodes, beta, numsteps, dtt, gamma);
end

 figI = figure
 for i = 1:maxRunPlot
 plot(time,Is2(i,:),'k')
 hold on
 end
 xlabel('Time')
 ylabel('Infected')
 
  figR = figure
 for i = 1:maxRunPlot
 plot(time,Rs2(i,:),'k')
 hold on
 end
 xlabel('Time')
 ylabel('Recovered')
 
meanIs=mean(Is2);
meanSs=mean(Ss2);
sizemeanIs = size(meanIs)


%agent SI
 numTimesteps=timeRun, plotGraphs=0, plotGraphsSIRN = 0, gamma=0,    throwAway = 'nonEpidemics', SEpidemic=99;

beta = beta*dtt, gamma = gamma*dtt, numTimesteps = floor(numTimesteps/dtt), timeScale = 0:dtt:timeRun-dtt %set all with same dtt

[fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetanew fighistGammanew figInfectedRun1 figBetaRun1  figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
    betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
    betameanmean gammameanmean betameanmean2 betameanmeanNew betameanmeanNew2 gammameanmeanNew Smean2 Imean2 Rmean2 Smedian2 Imedian2 Rmedian2 S2 I2 R2 SmeanN ImeanN RmeanN Snewendmean Snewendstd Rnewendmean Rnewendstd Rnew_nonendmean Rnew_nonendstd propSimsNonEp propSimsNonEp_theoretical] = monteCarloAgentSIR(numRuns,numNodes, beta, numTimesteps, plotGraphs, plotGraphsSIRN, gamma,throwAway,SEpidemic);


sfigSI=figure;
for  i = 1:maxRunPlot
    i
    plot(S2(i,:),'k')
    hold on
end
xlabel('Timesteps')
ylabel('Number of susceptible individuals S(t)')
title('a) susceptibles - 50 runs')

ifigSI=figure;
for  i = 1:maxRunPlot
    plot(I2(i,:),'k')
    hold on
end
xlabel('Timesteps')
ylabel('Number of infected individuals I(t)')
title('b) infected - 50 runs')


%Deterministic SI
N=numNodes %100
S = N %total number of susceptible nodes in network
R=0, I=1, drawPlots=0,
%beta = 0.003, numTimesteps = 8000
[DRo StD ItD RtD] = SIRDeterministic(beta, gamma, S, I, R, numTimesteps, drawPlots);
t2 = 1:1:numTimesteps;
ItD2 = ItD(1:1:end);

 step = numTimesteps / length(ItD);
 ts = [0:step:numTimesteps-step];
 size(ts)
 size(ItD)


agentSImeanfig = figure
  %plot(Smean,'k--', 'Linewidth',2)%agent
   hold on
%   size(timeD)
   size(Imean2)
   %plot(ts,ItD,'k.-', 'Linewidth',1)%deterministic
  %plot(t2,ItD2,'k.-', 'Linewidth',1)%deterministic with no dtt
  %plot(timeD,ItD,'k.-', 'Linewidth',1)%deterministic with dtt set
   plot(timeScale,ItD,'k', 'Linewidth',2)%deterministic with dtt set
   %plot(timeD,Imean,'k', 'Linewidth',1)%agent
   plot(timeScale,Imean2,'k', 'Linewidth',1)%agent
 %plot(time,meanIs,'k--', 'Linewidth',1) %stochastic
 plot(timeScale,meanIs,'k--', 'Linewidth',1) %stochastic
 xlabel('Time')
 %ylabel('Infected')
 text=['c) SI mean of ' num2str(numRuns) ' runs, betamean: ',num2str(betameanmean)];
title(text)
legend('Deterministic I(t)','Agent I(t)',  'Stochastic I(t)')



betameanmean
gammameanmean
betameanmeanNew
gammameanmeanNew
betameanmean2
betameanmeanNew2

betastd
gammastd 
betastd2 
betastdNew 
betastdNew2 
gammastdNew

peakTimemean 
peakTimestd 
peakValuemean 
peakValuestd

saveas(fighistBeta,[outputDirectory 'Test1bfighistBeta.fig']);
saveas(fighistBeta,[outputDirectory 'Test1bfighistBeta.emf']);
saveas(fighistGamma,[outputDirectory 'Test1bfighistGamma.fig']);
saveas(fighistGamma,[outputDirectory 'Test1bfighistGamma.emf']);

saveas(fighistRecovered,[outputDirectory 'Test1bfighistRecovered.fig']);
saveas(fighistRecovered,[outputDirectory 'Test1bfighistRecovered.emf']);
saveas(fighistSusceptible,[outputDirectory 'Test1bfighistSusceptible.fig']);
saveas(fighistSusceptible,[outputDirectory 'Test1bfighistSusceptible.emf']);
saveas(fighistpeakValue,[outputDirectory 'Test1bfighistpeakValue.fig']);
saveas(fighistpeakValue,[outputDirectory 'Test1bfighistpeakValue.emf']);
saveas(fighistpeakTime,[outputDirectory 'Test1bfighistpeakTime.fig']);
saveas(fighistpeakTime,[outputDirectory 'Test1bfighistpeakTime.emf']);

saveas(sfigSI,[outputDirectory 'Test1bsfigSI.fig']);
saveas(sfigSI,[outputDirectory 'Test1bsfigSI.emf']);
saveas(ifigSI,[outputDirectory 'Test1bifigSI.fig']);
saveas(ifigSI,[outputDirectory 'Test1bifigSI.emf']);
saveas(agentSImeanfig,[outputDirectory 'Test1bagentSImeanfig.fig']);
saveas(agentSImeanfig,[outputDirectory 'Test1bagentSImeanfig.emf']);
saveas(figBetaRun1,[outputDirectory 'Test1bfigBetaRun1SI.fig']);
saveas(figBetaRun1,[outputDirectory 'Test1bfigBetaRun1SI.emf']);
close all;


agentSImeanfigAll = figure
   hold on
   %dtt 0.01
   dec=200
 plot(timeScale1(1:dec:end),ItD1(1:dec:end),'k.-', 'Linewidth',1)%deterministic with dtt set
    hold on
   plot(timeScale1(1:170:end),Imean1(1:170:end),'ko-', 'Linewidth',1)%agent
 plot(timeScale1(1:dec:end),meanIs1(1:dec:end),'k+-', 'Linewidth',1) %stochastic
 %dtt 1
 dec=2
   plot(timeScale(1:dec:end),ItD(1:dec:end),'k.-', 'Linewidth',2)%deterministic with dtt set
   plot(timeScale(1:dec:end),Imean2(1:dec:end),'ko-', 'Linewidth',2)%agent
 plot(timeScale(1:dec:end),meanIs(1:dec:end),'k+-', 'Linewidth',2) %stochastic
ylabel('Infected I(t)')
 xlabel('Time')
 text=['c) SI mean of ' num2str(numRuns) ' runs, betamean: ',num2str(betameanmean)];
title(text)
%legend('Deterministic I(t)','Agent I(t)',  'Stochastic I(t)')
legend('dtt=0.01 Deterministic I(t)','dtt=0.01 Agent I(t)',  'dtt=0.01 Stochastic I(t)','dtt=1 Deterministic I(t)','dtt=1 Agent I(t)',  'dtt=1 Stochastic I(t)')
axis([10 40 0 1000])
saveas(agentSImeanfigAll,[outputDirectory 'Test1bagentSImeanfigAll.fig']);
saveas(agentSImeanfigAll,[outputDirectory 'Test1bagentSImeanfigAll.emf']);

close all;
end


%***************
%SIR TEST 2
%*********************
%SIR - compare to SIR deterministic
if doTests(2) == 1   
        throwAway = 'nonEpidemics';
%numRuns = 10000,numNodes =1000, beta=0.05, numTimesteps=1000, plotGraphs=0, plotGraphsSIRN = 0, gamma=0.015, SEpidemic=900;
numRuns = 1000,numNodes =1000, beta=0.3, numTimesteps=200, plotGraphs=0, plotGraphsSIRN = 0, gamma=0.15, SEpidemic=900;

solveR = (solve('1-x = exp(-1.0*(0.3/0.15)*x)', 'Real', true))
meanRTheory =  solveR*numNodes
coeffVar = 1
z = solveR(1)
rzero = beta/gamma
top = z*(1-z)*(1+(coeffVar*rzero^2*(1-z)))
topsqrt = sqrt(top)
bot = 1-((1-z)*rzero)
standDev = topsqrt/bot
stdTheory = double(standDev*sqrt(numNodes))

%agent
%numRuns = 10000,numNodes =1000, beta=0.15, numTimesteps=500, plotGraphs=0, plotGraphsSIRN = 0, gamma=0.1, SEpidemic=700;
%meanRTheory =  solve('1-x = exp(-1.0*(0.15/0.1)*x)', 'Real', true)
[fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetanew fighistGammanew figInfectedRun1 figBetaRun1  figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
    betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
    betameanmean gammameanmean betameanmean2 betameanmeanNew betameanmeanNew2 gammameanmeanNew Smean Imean Rmean Smedian Imedian Rmedian S I R SmeanN ImeanN RmeanN Snewendmean Snewendstd Rnewendmean Rnewendstd Rnew_nonendmean Rnew_nonendstd propSimsNonEp propSimsNonEp_theoretical] = monteCarloAgentSIR(numRuns,numNodes, beta, numTimesteps, plotGraphs, plotGraphsSIRN, gamma,throwAway,SEpidemic);

%Deterministic SIR
drawPlots=0,
[DRo StD ItD RtD] = SIRDeterministic(beta, gamma, numNodes, 1, 0, numTimesteps, drawPlots);
dec = 3
t2 = 1:dec:numTimesteps;
StD2 = StD(1:dec:end);
ItD2 = ItD(1:dec:end);
RtD2 = RtD(1:dec:end);
%stochastic SIR
dtt=0.01
numsteps = floor(numTimesteps/dtt); %7000
for i = 1:numRuns
 [ Ss(i,:) Is(i,:) Rs(i,:) ] = stochasticSIR(numNodes, beta, numsteps, dtt, gamma);
end
timeS = 0:dtt:numsteps*dtt-dtt;


agentSIRColourmeanfig = figure

for i =1:numRuns
    plot(S(i,:),'b')
    hold on
    plot(I(i,:),'r')
    plot(R(i,:),'g')
end


maxRunPlot=50
sfigSIR=figure;
for  i = 1:maxRunPlot
    plot(S(i,:),'k')
    hold on
end
xlabel('Timesteps')
ylabel('Number of susceptible individuals S(t)')
title('susceptibles - 50 runs')

ifigSIR=figure;
for  i = 1:maxRunPlot
    plot(I(i,:),'k')
    hold on
end
xlabel('Timesteps')
ylabel('Number of infected individuals I(t)')
title('infected - 50 runs')

rfigSIR=figure;
for  i = 1:maxRunPlot
    plot(R(i,:),'k')
    hold on
end
xlabel('Timesteps')
ylabel('Number of recovered individuals R(t)')
title('recovered - 50 runs')

agentSIRmeanfig = figure;
plot(Smean,'k--')
hold on
plot(Imean,'k','Linewidth',2)
plot(Rmean,'k')
xlabel('Timesteps')
ylabel('Number of individuals')
text=['Agent SIR -with all runs: ' num2str(numRuns) ', betamean: ',num2str(betameanmean)]
title(text)
legend('S(t)', 'I(t)','R(t)')

agentSIRmedianfig = figure;
plot(Smedian,'k--')
hold on
plot(Imedian,'k','Linewidth',2)
plot(Rmedian,'k')
xlabel('Timesteps')
ylabel('Number of individuals')
text=['Agent SIR -with all runs, median']
title(text)
legend('S(t)', 'I(t)','R(t)')

agentSIRmeanNewfig = figure;
plot(SmeanN,'k--')
hold on
plot(ImeanN,'k','Linewidth',2)
plot(RmeanN,'k')
xlabel('Timesteps')
ylabel('Number of individuals')
text=['Agent SIR -without non epidemics, betamean: ', num2str(betameanmeanNew)]
title(text)
legend('S(t)', 'I(t)','R(t)')


%graphs with comparison to other models

agentSIRmeanfigAll = figure;
%agent
%plot(Smean,'k--')
hold on
%plot(Rmean,'k')
%deterministic
%plot(t2,StD2,'k+-')
plot(t2,ItD2,'k.-')
plot(Imean,'k','Linewidth',2)
%plot(t2,RtD2,'k+-')
%stochastic
%plot(timeS,mean(Ss),'k.-')
meanIs = mean(Is);
dec=200
plot(timeS(1:dec:end),meanIs(1:dec:end),'k+-')
%plot(timeS,mean(Rs),'k.-')
xlabel('Timesteps')
ylabel('Number of individuals')
text=['Agent SIR -with all runs: ' num2str(numRuns) ', betamean: ',num2str(betameanmean)]
title(text)
%legend('S(t)', 'I(t)','R(t)')
legend('Deterministic', 'Agent','Stochastic')
axis([0 120 0 200])

agentSIRmedianfigAll = figure;
%plot(Smedian,'k--')
hold on
%plot(Rmedian,'k')
%deterministic
%plot(t2,StD2,'k+-')
plot(t2,ItD2,'k.-')
plot(Imedian,'k','Linewidth',2)
%plot(t2,RtD2,'k+-')
%stochastic
%plot(timeS,mean(Ss),'k.-')
meanIs = mean(Is);
dec=200
plot(timeS(1:dec:end),meanIs(1:dec:end),'k+-')
%plot(timeS,mean(Rs),'k.-')
xlabel('Timesteps')
ylabel('Number of individuals')
text=['Agent SIR -with all runs, median']
title(text)
%legend('S(t)', 'I(t)','R(t)')
legend('Deterministic', 'Agent','Stochastic')
axis([0 120 0 1000])

agentSIRmeanNewfigAll = figure;
hold on
%agent
plot(SmeanN,'k--')
plot(ImeanN,'k','Linewidth',2)
plot(RmeanN,'k')
%deterministic
plot(t2,StD2,'k.-')
plot(t2,ItD2,'k.-')
plot(t2,RtD2,'k.-')
%stochastic
%plot(timeS,mean(Ss),'b--')
%plot(timeS,mean(Is),'b','Linewidth',2)
%plot(timeS,mean(Rs),'b')
xlabel('Timesteps')
ylabel('Number of individuals')
text=['Agent SIR -without non epidemics, betamean: ', num2str(betameanmeanNew)]
title(text)
legend('S(t)', 'I(t)','R(t)', 'Deterministic')
%legend('Deterministic', 'Agent')
axis([0 120 0 1000])

agentSIRmeanNewfigjustIAll = figure;
hold on
%deterministic
plot(t2,ItD2,'k.-')
%agent
plot(ImeanN,'k','Linewidth',2)
xlabel('Timesteps')
ylabel('Number of individuals')
text=['Agent SIR -without non epidemics, betamean: ', num2str(betameanmeanNew)]
title(text)
%legend('S(t)', 'I(t)','R(t)', 'Deterministic')
legend('Deterministic', 'Agent')
axis([0 120 0 200])



%get distribution of S at end of sim
send = S(:,numTimesteps);
shistSIRfig = figure;
hist(send,50)
xlabel('Final value of S')
ylabel('Number of runs')
title('Histogram of S(t=end)')
h = findobj(gca,'Type','patch');
set(h,'FaceColor','k')

betameanmean
gammameanmean
betameanmeanNew
gammameanmeanNew
betameanmean2
betameanmeanNew2

betastd 
gammastd
betastd2
betastdNew
betastdNew2
gammastdNew

Rnewendmean%epidemics only
Rnewendstd
%theoryMean = 583
%theorystd = 53
%Snewendmean%epidemics only
%Snewendstd
%Rnew_nonendmean %non epidemics only
%Rnew_nonendstd

 propSimsNonEp % proportion of sims with non-epidemics
 propSimsNonEp_theoretical %theoretical value


saveas(fighistRecovered,[outputDirectory 'Test2fighistRecovered.fig']);
saveas(fighistRecovered,[outputDirectory 'Test2fighistRecovered.emf']);
saveas(fighistSusceptible,[outputDirectory 'Test2fighistSusceptible.fig']);
saveas(fighistSusceptible,[outputDirectory 'Test2fighistSusceptible.emf']);
saveas(fighistpeakValue,[outputDirectory 'Test2fighistpeakValue.fig']);
saveas(fighistpeakValue,[outputDirectory 'Test2fighistpeakValue.emf']);
saveas(fighistpeakTime,[outputDirectory 'Test2fighistpeakTime.fig']);
saveas(fighistpeakTime,[outputDirectory 'Test2fighistpeakTime.emf']);

saveas(fighistBeta,[outputDirectory 'Test2fighistBeta.fig']);
saveas(fighistBeta,[outputDirectory 'Test2fighistBeta.emf']);
saveas(fighistGamma,[outputDirectory 'Test2fighistGamma.fig']);
saveas(fighistGamma,[outputDirectory 'Test2fighistGamma.emf']);

saveas(fighistBetanew,[outputDirectory 'Test2fighistBetanew.fig']); %non epidemics removed
saveas(fighistBetanew,[outputDirectory 'Test2fighistBetanew.emf']); %non epidemics removed
saveas(fighistGammanew,[outputDirectory 'Test2fighistGammanew.fig']); %non epidemics removed
saveas(fighistGammanew,[outputDirectory 'Test2fighistGammanew.emf']); %non epidemics removed

saveas(agentSIRColourmeanfig,[outputDirectory 'Test2agentSIRColourmeanfig.fig']);
saveas(agentSIRColourmeanfig,[outputDirectory 'Test2agentSIRColourmeanfig.emf']);
saveas(sfigSIR,[outputDirectory 'Test2sfigSIR.fig']);
saveas(sfigSIR,[outputDirectory 'Test2sfigSIR.emf']);
saveas(ifigSIR,[outputDirectory 'Test2ifigSIR.fig']);
saveas(ifigSIR,[outputDirectory 'Test2ifigSIR.emf']);
saveas(rfigSIR,[outputDirectory 'Test2rfigSIR.fig']);
saveas(rfigSIR,[outputDirectory 'Test2rfigSIR.emf']);
saveas(agentSIRmeanfig,[outputDirectory 'Test2agentSIRmeanfig.fig']);
saveas(agentSIRmeanfig,[outputDirectory 'Test2agentSIRmeanfig.emf']);
saveas(agentSIRmedianfig,[outputDirectory 'Test2agentSIRmedianfig.fig']);
saveas(agentSIRmedianfig,[outputDirectory 'Test2agentSIRmedianfig.emf']);
saveas(agentSIRmeanNewfig,[outputDirectory 'Test2agentSIRmeanNewfig.fig']);
saveas(agentSIRmeanNewfig,[outputDirectory 'Test2agentSIRmeanNewfig.emf']);

saveas(agentSIRmeanfigAll,[outputDirectory 'Test2agentSIRmeanfigAll.fig']);
saveas(agentSIRmeanfigAll,[outputDirectory 'Test2agentSIRmeanfigAll.emf']);
saveas(agentSIRmedianfigAll,[outputDirectory 'Test2agentSIRmedianfigAll.fig']);
saveas(agentSIRmedianfigAll,[outputDirectory 'Test2agentSIRmedianfigAll.emf']);
saveas(agentSIRmeanNewfigAll,[outputDirectory 'Test2agentSIRmeanNewfigAll.fig']);
saveas(agentSIRmeanNewfigAll,[outputDirectory 'Test2agentSIRmeanNewfigAll.emf']);
saveas(agentSIRmeanNewfigjustIAll,[outputDirectory 'Test2agentSIRmeanNewfigjustIAll.fig']);
saveas(agentSIRmeanNewfigjustIAll,[outputDirectory 'Test2agentSIRmeanNewfigjustIAll.emf']);

%saveas(figBetaRun1,[outputDirectory 'Test2figBetaRun1SIR.fig']);
%saveas(figBetaRun1,[outputDirectory 'Test2figBetaRun1SIR.emf']);
%saveas(shistSIRfig,[outputDirectory 'Test2shistSIRfig.fig']);
%saveas(shistSIRfig,[outputDirectory 'Test2shistSIRfig.emf']);


close all;
end %SIR TEST 2

%***************
%SIR TEST 3
%*****************
%single run
if doTests(3) == 1
    throwAway = 'nonEpidemics';
numRuns = 1,numNodes =100, beta=0.5, numTimesteps=80, plotGraphs=0, plotGraphsSIRN = 1, gamma=0.15, SEpidemic=70;
[fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetaNew fighistGammaNew figInfectedRun1 figBetaRun1  figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2... 
    betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
    betameanmean gammameanmean betameanmean2 betameanmeanNew betameanmeanNew2 gammameanmeanNew Smean Imean Rmean Smedian Imedian Rmedian S I R SmeanN ImeanN Snewendmean Snewendstd Rnewendmean Rnewendstd Rnew_nonendmean Rnew_nonendstd propSimsNonEp propSimsNonEp_theoretical] = monteCarloAgentSIR(numRuns,numNodes, beta, numTimesteps, plotGraphs, plotGraphsSIRN,gamma,throwAway,SEpidemic);

agentSIRColourmeanfig=figure
for i =1:numRuns
    plot(S(i,:),'b')
    hold on
    plot(I(i,:),'r')
    plot(R(i,:),'g')
end

sfigSIR=figure;
for  i = 1:numRuns
    plot(S(i,:),'k')
    hold on
end
xlabel('Timesteps')
ylabel('Number of susceptible individuals S(t)')

ifigSIR=figure;
for  i = 1:numRuns
    plot(I(i,:),'k')
    hold on
end
xlabel('Timesteps')
ylabel('Number of infected individuals I(t)')

rfigSIR=figure;
for  i = 1:numRuns
    plot(R(i,:),'k')
    hold on
end
xlabel('Timesteps')
ylabel('Number of recovered individuals R(t)')

betameanmean
gammameanmean
betameanmeanNew
gammameanmeanNew
betameanmean2
betameanmeanNew2


saveas(agentSIRColourmeanfig,[outputDirectory 'Test3agentSIRColourmeanfig.fig']);
saveas(agentSIRColourmeanfig,[outputDirectory 'Test3agentSIRColourmeanfig.emf']);
saveas(sfigSIR,[outputDirectory 'Test3sfigSIR.fig']);
saveas(sfigSIR,[outputDirectory 'Test3sfigSIR.emf']);
saveas(ifigSIR,[outputDirectory 'Test3ifigSIR.fig']);
saveas(ifigSIR,[outputDirectory 'Test3ifigSIR.emf']);
saveas(rfigSIR,[outputDirectory 'Test3rfigSIR.fig']);
saveas(rfigSIR,[outputDirectory 'Test3rfigSIR.emf']);
saveas(figBetaRun1,[outputDirectory 'Test3figBetaRun1SIR.fig']);
saveas(figBetaRun1,[outputDirectory 'Test3figBetaRun1SIR.emf']);
saveas(figGammaRun1,[outputDirectory 'Test3figGammaRun1.fig']);
saveas(figGammaRun1,[outputDirectory 'Test3figGammaRun1.emf']);
saveas(figRoRun1,[outputDirectory 'Test3figRoRun1.fig']);
saveas(figRoRun1,[outputDirectory 'Test3figRoRun1.emf']);
saveas(figBetaGamma1,[outputDirectory 'Test3figBetaGamma1.fig']);
saveas(figBetaGamma1,[outputDirectory 'Test3figBetaGamma1.emf']);
saveas(figBetaGamma2,[outputDirectory 'Test3figBetaGamma2.fig']);
saveas(figBetaGamma2,[outputDirectory 'Test3figBetaGamma2.emf']);
close all;
end %SIR TEST 3

%***************
%SIR TEST 4
%*****************
if doTests(4) == 1
    %vary beta, 0.45, 0.50, 0.55, gamma==0.50, compare against
    %deterministic tests
    
    beta = [0.045, 0.05, 0.055]
    for i = 1:3
        if i == 1;     throwAway = 'nonEpidemics'; end; %betameanmeanNew and mean SIR graph
        if i == 2;     throwAway = 'nonEpidemics'; end; %betameanmeanNew and mean SIR graph
        if i == 3;     throwAway = 'nonEpidemics'; end; %betameanmeanNew and mean SIR graph
    %numRuns = 100,numNodes =100, numTimesteps=60, plotGraphs=0, plotGraphsSIRN = 0, gamma=0.15, SEpidemic=99 %99
    numRuns = 10000,numNodes =1000, numTimesteps=1000, plotGraphs=0, plotGraphsSIRN = 0, gamma=0.05, SEpidemic=999 %99
    
    
   [fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetaNew fighistGammaNew figInfectedRun1 figBetaRun1  figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
       betastd(i) gammastd(i) betastd2(i) betastdNew(i) betastdNew2(i) gammastdNew(i) peakTimemean(i) peakTimestd(i) peakValuemean(i) peakValuestd(i)...
       betameanmean(i,:) gammameanmean(i,:) betameanmean2(i,:) betameanmeanNew(i,:) betameanmeanNew2(i,:) gammameanmeanNew(i,:) Smean(i,:) Imean(i,:) Rmean(i,:) Smedian Imedian(i,:) Rmedian S I R SmeanN(i,:) ImeanN(i,:) RmeanN(i,:) Snewendmean(i) Snewendstd(i) Rnewendmean(i) Rnewendstd(i) Rnew_nonendmean(i) Rnew_nonendstd(i) propSimsNonEp(i) propSimsNonEp_theoretical(i)] = monteCarloAgentSIR(numRuns,numNodes, beta(i), numTimesteps, plotGraphs, plotGraphsSIRN, gamma,throwAway,SEpidemic);
    %Histograms
    saveas(fighistRecovered,[outputDirectory 'Test4fighistRecovered' num2str(i) '.fig']);
    saveas(fighistRecovered,[outputDirectory 'Test4fighistRecovered' num2str(i) '.emf']);
    saveas(fighistSusceptible,[outputDirectory 'Test4fighistSusceptible' num2str(i) '.fig']);
    saveas(fighistSusceptible,[outputDirectory 'Test4fighistSusceptible' num2str(i) '.emf']);
    saveas(fighistpeakValue,[outputDirectory 'Test4fighistpeakValue' num2str(i) '.fig']);
    saveas(fighistpeakValue,[outputDirectory 'Test4fighistpeakValue' num2str(i) '.emf']);
    saveas(fighistpeakTime,[outputDirectory 'Test4fighistpeakTime' num2str(i) '.fig']);
    saveas(fighistpeakTime,[outputDirectory 'Test4fighistpeakTime' num2str(i) '.emf']);
    saveas(fighistBeta,[outputDirectory 'Test4histBeta' num2str(i) '.fig']);
    saveas(fighistBeta,[outputDirectory 'Test4histBeta' num2str(i) '.emf']);
    saveas(fighistGamma,[outputDirectory 'Test4histGamma' num2str(i) '.fig']);
    saveas(fighistGamma,[outputDirectory 'Test4histGamma' num2str(i) '.emf']);
    saveas(fighistBetaNew,[outputDirectory 'Test4histBetaNew' num2str(i) '.fig']);
    saveas(fighistBetaNew,[outputDirectory 'Test4histBetaNew' num2str(i) '.emf']);
    saveas(fighistGammaNew,[outputDirectory 'Test4histGammaNew' num2str(i) '.fig']);
    saveas(fighistGammaNew,[outputDirectory 'Test4histGammaNew' num2str(i) '.emf']);
   
    %get distribution of S at end of sim
    send = S(:,numTimesteps);
    shistSIRfig = figure;
    hist(send,50)
    xlabel('Final value of S')
    ylabel('Number of runs')
    title('Histogram of S(t=end)')
    h = findobj(gca,'Type','patch');
    set(h,'FaceColor','k')
    saveas(shistSIRfig,[outputDirectory 'Test4histS' num2str(i) '.fig']);
    saveas(shistSIRfig,[outputDirectory 'Test4histS' num2str(i) '.emf']);
    
    agentSIRColourmeanfig=figure
    for k =1:numRuns
        plot(S(k,:),'b')
        hold on
        plot(I(k,:),'r')
        plot(R(k,:),'g')
    end
    saveas(agentSIRColourmeanfig,[outputDirectory 'Test4colmean' num2str(i) '.fig']);
    saveas(agentSIRColourmeanfig,[outputDirectory 'Test4colmean' num2str(i) '.emf']); 
    %get beta and gamma from mean of all data
    for j = 1 : (numTimesteps) -1
        Ns = Smean(i,j+1)+Imean(i,j+1)+Rmean(i,j+1); %only count those actually in susceptible part of network at time t
        %measure beta
        B(i,j)=(-(Smean(i,j+1) - Smean(i,j))/(Imean(i,j)*Smean(i,j)/Ns)); %measured beta
        %measure gamma
        Y(i,j) = ((Rmean(i,j+1)-Rmean(i,j))/Imean(i,j));
        B2(i,j) = ((Ns.*(Imean(i,j+1)+(Y(i,j)-1).*Imean(i,j)))/(Imean(i,j)*Smean(i,j))); %measured beta
        %measure Ro
        Ro(i,j) = (B(i,j)*Smean(i,j))/(Ns*Y(i,j));
    end
        %get beta and gamma from mean of only data wanted
    for j = 1 : (numTimesteps) -1
        Ns = SmeanN(i,j+1)+ImeanN(i,j+1)+RmeanN(i,j+1); %only count those actually in susceptible part of network at time t
        %measure beta
        BN(i,j)=(-(SmeanN(i,j+1) - SmeanN(i,j))/(ImeanN(i,j)*SmeanN(i,j)/Ns)); %measured beta
        %measure gamma
        YN(i,j) = ((RmeanN(i,j+1)-RmeanN(i,j))/ImeanN(i,j));
        B2N(i,j) = ((Ns.*(ImeanN(i,j+1)+(YN(i,j)-1).*ImeanN(i,j)))/(ImeanN(i,j)*SmeanN(i,j))); %measured beta
        %measure Ro
        RoN(i,j) = (BN(i,j)*SmeanN(i,j))/(Ns*YN(i,j));
    end
    end %i loop

figmean = figure
plot(Imean(1,:),'k--')
hold on
plot(Imean(2,:),'k','Linewidth',2)
plot(Imean(3,:),'k.-')
legend('av. contact rate = 0.045','av. contact rate = 0.050','av. contact rate = 0.055')
xlabel('Timesteps')
ylabel('Number of individuals infected I(t)')
title('mean')

figmeannone = figure
plot(ImeanN(1,:),'k--')
hold on
plot(ImeanN(2,:),'k','Linewidth',2)
plot(ImeanN(3,:),'k.-')
legend('av. contact rate = 0.045','av. contact rate = 0.050','av. contact rate = 0.055')
xlabel('Timesteps')
ylabel('Number of individuals infected I(t)')
title('mean without non-epidemics')


figmedian = figure
plot(Imedian(1,:),'k--')
hold on
plot(Imedian(2,:),'k','Linewidth',2)
plot(Imedian(3,:),'k.-')
legend('av. contact rate = 0.045','av. contact rate = 0.050','av. contact rate = 0.055')
xlabel('Timesteps')
ylabel('Number of individuals infected I(t)')
title('median')

betameanmean
gammameanmean

betameanmean2
betameanmeanNew 
betameanmeanNew2
gammameanmeanNew

Rnewendmean%epidemics only
Rnewendstd

 propSimsNonEp % proportion of sims with non-epidemics
 propSimsNonEp_theoretical %theoretical value
 
     meanRTheory1 =  (solve('1-x = exp(-1.0*(0.045/0.05)*x)', 'Real', true))*numNodes
    meanRTheory2 =  (solve('1-x = exp(-1.0*(0.05/0.05)*x)', 'Real', true))*numNodes
    meanRTheory3 =  (solve('1-x = exp(-1.0*(0.055/0.05)*x)', 'Real', true))*numNodes

%BETA AND GAMMA FROM MEAN OF DATA
B;
 %sort beta to get rid of nan and inf values
sortedbeta = sort(B);
nanMaskb= isnan(sortedbeta);
infMaskb = isinf(sortedbeta);
[startvb startib] = min(infMaskb,[],2);
[cb indexNanb] = max(nanMaskb,[],2);
if indexNanb(1) == 1; indexNanb(1) = (numTimesteps)-1; end;
if indexNanb(2) == 1; indexNanb(2) = (numTimesteps)-1; end;
if indexNanb(3) == 1; indexNanb(3) = (numTimesteps)-1; end;
meanBetaFromMeanData(1) = mean(sortedbeta(1,startib(1):indexNanb(1)-1));
meanBetaFromMeanData(2) = mean(sortedbeta(2,startib(2):indexNanb(2)-1));
meanBetaFromMeanData(3) = mean(sortedbeta(3,startib(3):indexNanb(3)-1))
Y;
 %sort gamma to get rid of nan and inf values
sortedgamma = sort(Y);
nanMaskg= isnan(sortedgamma);
infMaskg = isinf(sortedgamma);
[startvg startig] = min(infMaskg,[],2);
[cg indexNang] = max(nanMaskg,[],2);
if indexNang == 1; indexNang(1) = (numTimesteps)-1; end;
if indexNang == 2; indexNang(2) = (numTimesteps)-1; end;
if indexNang == 3; indexNang(3) = (numTimesteps)-1; end;
meanGammaFromMeanData(1) = mean(sortedgamma(1,startig(1):indexNang(1)-1));
meanGammaFromMeanData(2) = mean(sortedgamma(2,startig(2):indexNang(2)-1));
meanGammaFromMeanData(3) = mean(sortedgamma(3,startig(3):indexNang(3)-1))

BN;
 %sort beta with removed runs to get rid of nan and inf values
sortedbeta = sort(BN);
nanMaskb= isnan(sortedbeta);
infMaskb = isinf(sortedbeta);
[startvb startib] = min(infMaskb,[],2);
[cb indexNanb] = max(nanMaskb,[],2);
if indexNanb(1) == 1; indexNanb(1) = (numTimesteps)-1; end;
if indexNanb(2) == 1; indexNanb(2) = (numTimesteps)-1; end;
if indexNanb(3) == 1; indexNanb(3) = (numTimesteps)-1; end;
%sortedbeta(startib(1):indexNanb(1)-1);
meanBetaFromMeanDataN(1) = mean(sortedbeta(1,startib(1):indexNanb(1)-1));
meanBetaFromMeanDataN(2) = mean(sortedbeta(2,startib(2):indexNanb(2)-1));
meanBetaFromMeanDataN(3) = mean(sortedbeta(3,startib(3):indexNanb(3)-1))
YN;
 %sort gamma with removed runs to get rid of nan and inf values
sortedgamma = sort(YN);
nanMaskg= isnan(sortedgamma);
infMaskg = isinf(sortedgamma);
[startvg startig] = min(infMaskg,[],2);
[cg indexNang] = max(nanMaskg,[],2);
if indexNang(1) == 1; indexNang(1) = (numTimesteps)-1; end;
if indexNang(2) == 1; indexNang(2) = (numTimesteps)-1; end;
if indexNang(3) == 1; indexNang(3) = (numTimesteps)-1; end;
meanGammaFromMeanDataN(1) = mean(sortedgamma(1,startig(1):indexNang(1)-1));
meanGammaFromMeanDataN(2) = mean(sortedgamma(2,startig(2):indexNang(2)-1));
meanGammaFromMeanDataN(3) = mean(sortedgamma(3,startig(3):indexNang(3)-1))



R = betameanmean./gammameanmean
figro = figure
plot(betameanmean,R,'k')
hold on
plot(betameanmean,[1 1 1],'k--')


saveas(figmean,[outputDirectory 'Test4figmean.fig']);
saveas(figmean,[outputDirectory 'Test4figmean.emf']);
saveas(figmeannone,[outputDirectory 'Test4figmeannone.fig']);
saveas(figmeannone,[outputDirectory 'Test4figmeannone.emf']);
saveas(figmedian,[outputDirectory 'Test4figmedian.fig']);
saveas(figmedian,[outputDirectory 'Test4figmedian.emf']);
saveas(figro,[outputDirectory 'Test4figro.fig']);
saveas(figro,[outputDirectory 'Test4figro.emf']);
saveas(figInfectedRun1,[outputDirectory 'Test4figInfectedRun1.fig']);
saveas(figInfectedRun1,[outputDirectory 'Test4figInfectedRun1.emf']);
saveas(figBetaRun1,[outputDirectory 'Test4figBetaRun1.fig']);
saveas(figBetaRun1,[outputDirectory 'Test4figBetaRun1.emf']);
saveas(figGammaRun1,[outputDirectory 'Test4figGammaRun1.fig']);
saveas(figGammaRun1,[outputDirectory 'Test4figGammaRun1.emf']);
saveas(figRoRun1,[outputDirectory 'Test4figRoRun1.fig']);
saveas(figRoRun1,[outputDirectory 'Test4figRoRun1.emf']);
close all;
end % SIR TEST 4





%***************
%SIR TEST 5
%*****************
if doTests(5) == 1
    %vary beta, 0.45, 0.50, 0.55, gamma==0.50, compare against
    %deterministic tests
%    beta = 0.5
%gamma = [0.05 0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8]; %1/48*60*60 %2 days
%SEpidemic=[950 950 950 950 950 999 999 999 999] %99

    beta = [0.50 0.45 0.40 0.35 0.30 0.25 0.20 0.15 0.10]
gamma = [0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3]; %1/48*60*60 %2 days
SEpidemic=[900 900 900 900 900 900 900 900 900] %99

%    beta = [0.050 0.045 0.040 0.035 0.030 0.025 0.020 0.015 0.010]
%gamma = [0.03 0.03 0.03 0.03 0.03 0.03 0.03 0.03 0.03]; %1/48*60*60 %2 days
%SEpidemic=[950 950 950 950 999 999 999 999 999] %99

throwAway = 'nonEpidemics'%betameanmeanNew and mean SIR graph
    for i = 1:9

    %numRuns = 100,numNodes =100, numTimesteps=60, plotGraphs=0, plotGraphsSIRN = 0, gamma=0.15, SEpidemic=99 %99
 %   numRuns = 10000,numNodes =1000, numTimesteps=1000, plotGraphs=0, plotGraphsSIRN = 0;
       numRuns = 1000,numNodes =1000, numTimesteps=200, plotGraphs=0, plotGraphsSIRN = 0;
   %agent SIR 
   [fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetaNew fighistGammaNew figInfectedRun1 figBetaRun1  figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
       betastd(i) gammastd(i) betastd2(i) betastdNew(i) betastdNew2(i) gammastdNew(i) peakTimemean(i) peakTimestd(i) peakValuemean(i) peakValuestd(i)...
       betameanmean(i,:) gammameanmean(i,:) betameanmean2(i,:) betameanmeanNew(i,:) betameanmeanNew2(i,:) gammameanmeanNew(i,:) Smean(i,:) Imean(i,:) Rmean(i,:) Smedian Imedian(i,:) Rmedian S I R SmeanN(i,:) ImeanN(i,:) RmeanN(i,:) Snewendmean(i) Snewendstd(i) Rnewendmean(i) Rnewendstd(i) Rnew_nonendmean(i) Rnew_nonendstd(i) propSimsNonEp(i) propSimsNonEp_theoretical(i)] = monteCarloAgentSIR(numRuns,numNodes, beta(i), numTimesteps, plotGraphs, plotGraphsSIRN, gamma(i),throwAway,SEpidemic(i));
    %Histograms
    saveas(fighistRecovered,[outputDirectory 'Test5fighistRecovered' num2str(i) '.fig']);
    saveas(fighistRecovered,[outputDirectory 'Test5fighistRecovered' num2str(i) '.emf']);
    saveas(fighistSusceptible,[outputDirectory 'Test5fighistSusceptible' num2str(i) '.fig']);
    saveas(fighistSusceptible,[outputDirectory 'Test5fighistSusceptible' num2str(i) '.emf']);
    saveas(fighistpeakValue,[outputDirectory 'Test5fighistpeakValue' num2str(i) '.fig']);
    saveas(fighistpeakValue,[outputDirectory 'Test5fighistpeakValue' num2str(i) '.emf']);
    saveas(fighistpeakTime,[outputDirectory 'Test5fighistpeakTime' num2str(i) '.fig']);
    saveas(fighistpeakTime,[outputDirectory 'Test5fighistpeakTime' num2str(i) '.emf']);
    saveas(fighistBeta,[outputDirectory 'Test5histBeta' num2str(i) '.fig']);
    saveas(fighistBeta,[outputDirectory 'Test5histBeta' num2str(i) '.emf']);
    saveas(fighistGamma,[outputDirectory 'Test5histGamma' num2str(i) '.fig']);
    saveas(fighistGamma,[outputDirectory 'Test5histGamma' num2str(i) '.emf']);
    saveas(fighistBetaNew,[outputDirectory 'Test5histBetaNew' num2str(i) '.fig']);
    saveas(fighistBetaNew,[outputDirectory 'Test5histBetaNew' num2str(i) '.emf']);
    saveas(fighistGammaNew,[outputDirectory 'Test5histGammaNew' num2str(i) '.fig']);
    saveas(fighistGammaNew,[outputDirectory 'Test5histGammaNew' num2str(i) '.emf']);
   
    
    %Deterministic SIR
drawPlots=0,
[DRo StD ItD RtD] = SIRDeterministic(beta(i), gamma(i), numNodes, 1, 0, numTimesteps, drawPlots);
dec = 5
t2 = 1:dec:numTimesteps;
ItD2(i,:) = ItD(1:dec:end);

    
    
    %get distribution of S at end of sim
    send = S(:,numTimesteps);
    shistSIRfig = figure;
    hist(send,50)
    xlabel('Final value of S')
    ylabel('Number of runs')
    title('Histogram of S(t=end)')
    h = findobj(gca,'Type','patch');
    set(h,'FaceColor','k')
    saveas(shistSIRfig,[outputDirectory 'Test5histS' num2str(i) '.fig']);
    saveas(shistSIRfig,[outputDirectory 'Test5histS' num2str(i) '.emf']);
    
    agentSIRColourmeanfig=figure
    for k =1:numRuns
        plot(S(k,:),'b')
        hold on
        plot(I(k,:),'r')
        plot(R(k,:),'g')
    end
    saveas(agentSIRColourmeanfig,[outputDirectory 'Test5colmean' num2str(i) '.fig']);
    saveas(agentSIRColourmeanfig,[outputDirectory 'Test5colmean' num2str(i) '.emf']); 
    %get beta and gamma from mean of all data
    for j = 1 : (numTimesteps) -1
        Ns = Smean(i,j+1)+Imean(i,j+1)+Rmean(i,j+1); %only count those actually in susceptible part of network at time t
        %measure beta
        B(i,j)=(-(Smean(i,j+1) - Smean(i,j))/(Imean(i,j)*Smean(i,j)/Ns)); %measured beta
        %measure gamma
        Y(i,j) = ((Rmean(i,j+1)-Rmean(i,j))/Imean(i,j));
        B2(i,j) = ((Ns.*(Imean(i,j+1)+(Y(i,j)-1).*Imean(i,j)))/(Imean(i,j)*Smean(i,j))); %measured beta
        %measure Ro
        Ro(i,j) = (B(i,j)*Smean(i,j))/(Ns*Y(i,j));
    end
        %get beta and gamma from mean of only data wanted
    for j = 1 : (numTimesteps) -1
        Ns = SmeanN(i,j+1)+ImeanN(i,j+1)+RmeanN(i,j+1); %only count those actually in susceptible part of network at time t
        %measure beta
        BN(i,j)=(-(SmeanN(i,j+1) - SmeanN(i,j))/(ImeanN(i,j)*SmeanN(i,j)/Ns)); %measured beta
        %measure gamma
        YN(i,j) = ((RmeanN(i,j+1)-RmeanN(i,j))/ImeanN(i,j));
        B2N(i,j) = ((Ns.*(ImeanN(i,j+1)+(YN(i,j)-1).*ImeanN(i,j)))/(ImeanN(i,j)*SmeanN(i,j))); %measured beta
        %measure Ro
        RoN(i,j) = (BN(i,j)*SmeanN(i,j))/(Ns*YN(i,j));
    end
    end %i loop

figmean = figure
%agent
dec = 3
time = 1:dec:numTimesteps
plot(time,Imean(1,1:dec:numTimesteps),'k+-')
hold on
plot(time,Imean(2,1:dec:numTimesteps),'ko-')
plot(time,Imean(3,1:dec:numTimesteps),'k*-')
plot(Imean(4,:),'k','Linewidth',2)
plot(Imean(5,:),'k--')
plot(Imean(6,:),'k-.')
plot(Imean(7,:),'k')
%deterministic
%plot(t2,ItD2(1,:),'k.-')
%plot(t2,ItD2(2,:),'k.-')
%plot(t2,ItD2(3,:),'k.-')
%plot(t2,ItD2(4,:),'k.-')
plot(t2,ItD2(5,:),'k.-')
plot(t2,ItD2(6,:),'k.-')
plot(t2,ItD2(7,:),'k.-')
%plot(Imean(8,:),'m')
%plot(Imean(9,:),'y')
%legend('0.50', '0.45', '0.40', '0.35', '0.30', '0.25', '0.20', '0.15', '0.10')
legend('0.50', '0.45', '0.40', '0.35', '0.30', '0.25', '0.20','Deterministic <=0.3')
xlabel('Timesteps')
ylabel('Number of individuals infected I(t)')
title('Mean of all runs')
axis([0 100 0 60])

figmeannone = figure
%agent
plot(time,ImeanN(1,1:dec:numTimesteps),'k+-')
hold on
plot(time,ImeanN(2,1:dec:numTimesteps),'ko-')
plot(time,ImeanN(3,1:dec:numTimesteps),'k*-')
plot(ImeanN(4,:),'k','Linewidth',2)
plot(ImeanN(5,:),'k--')
plot(ImeanN(6,:),'k-.')
plot(ImeanN(7,:),'k')
%plot(ImeanN(8,:),'m')
%plot(ImeanN(9,:),'y')
%deterministic
%plot(t2,ItD2(1,:),'k.-')
%plot(t2,ItD2(2,:),'k.-')
%plot(t2,ItD2(3,:),'k.-')
%plot(t2,ItD2(4,:),'k.-')
%plot(t2,ItD2(5,:),'k.-')
%plot(t2,ItD2(6,:),'k.-')
%plot(t2,ItD2(7,:),'k.-')
%legend('0.50', '0.45', '0.40', '0.35', '0.30', '0.25', '0.20', '0.15', '0.10')
legend('0.50', '0.45', '0.40', '0.35', '0.30', '0.25', '0.20')
xlabel('Timesteps')
ylabel('Number of individuals infected I(t)')
title('Mean of major outbreaks')
axis([0 100 0 90])

figmedian = figure
plot(Imedian(1,:),'k--')
hold on
plot(Imedian(2,:),'k','Linewidth',2)
plot(Imedian(3,:),'k')
plot(Imedian(4,:),'b')
plot(Imedian(5,:),'g')
plot(Imedian(6,:),'r')
plot(Imedian(7,:),'c')
plot(Imedian(8,:),'m')
plot(Imedian(9,:),'y')
legend('0.50', '0.45', '0.40', '0.35', '0.30', '0.25', '0.20', '0.15', '0.10')
xlabel('Timesteps')
ylabel('Number of individuals infected I(t)')
title('median')

betameanmean
gammameanmean

betameanmean2
betameanmeanNew 
betameanmeanNew2
gammameanmeanNew

Rnewendmean%epidemics only
Rnewendstd

 propSimsNonEp % proportion of sims with non-epidemics
 propSimsNonEp_theoretical %theoretical value
 %vary gamma
     %meanRTheory1 =  (solve('1-x = exp(-1.0*(0.5/0.05)*x)', 'Real', true))*numNodes
    %meanRTheory2 =  (solve('1-x = exp(-1.0*(0.5/0.1)*x)', 'Real', true))*numNodes
    %meanRTheory3 =  (solve('1-x = exp(-1.0*(0.5/0.2)*x)', 'Real', true))*numNodes
    %meanRTheory4 =  (solve('1-x = exp(-1.0*(0.5/0.3)*x)', 'Real', true))*numNodes
    %meanRTheory5 =  (solve('1-x = exp(-1.0*(0.5/0.4)*x)', 'Real', true))*numNodes
    %meanRTheory6 =  (solve('1-x = exp(-1.0*(0.5/0.5)*x)', 'Real', true))*numNodes
    %meanRTheory7 =  (solve('1-x = exp(-1.0*(0.5/0.6)*x)', 'Real', true))*numNodes
    %meanRTheory8 =  (solve('1-x = exp(-1.0*(0.5/0.7)*x)', 'Real', true))*numNodes
    %meanRTheory9 =  (solve('1-x = exp(-1.0*(0.5/0.8)*x)', 'Real', true))*numNodes
    
    %vary beta
    meanRTheory1 =  (solve('1-x = exp(-1.0*(0.50/0.3)*x)', 'Real', true))*numNodes
    meanRTheory2 =  (solve('1-x = exp(-1.0*(0.45/0.3)*x)', 'Real', true))*numNodes
    meanRTheory3 =  (solve('1-x = exp(-1.0*(0.40/0.3)*x)', 'Real', true))*numNodes
    meanRTheory4 =  (solve('1-x = exp(-1.0*(0.35/0.3)*x)', 'Real', true))*numNodes
    meanRTheory5 =  (solve('1-x = exp(-1.0*(0.30/0.3)*x)', 'Real', true))*numNodes
    meanRTheory6 =  (solve('1-x = exp(-1.0*(0.25/0.3)*x)', 'Real', true))*numNodes
    meanRTheory7 =  (solve('1-x = exp(-1.0*(0.20/0.3)*x)', 'Real', true))*numNodes
    meanRTheory8 =  (solve('1-x = exp(-1.0*(0.15/0.3)*x)', 'Real', true))*numNodes
    meanRTheory9 =  (solve('1-x = exp(-1.0*(0.10/0.3)*x)', 'Real', true))*numNodes

%BETA AND GAMMA FROM MEAN OF DATA
B;
 %sort beta to get rid of nan and inf values
sortedbeta = sort(B);
nanMaskb= isnan(sortedbeta);
infMaskb = isinf(sortedbeta);
[startvb startib] = min(infMaskb,[],2);
[cb indexNanb] = max(nanMaskb,[],2);
if indexNanb(1) == 1; indexNanb(1) = (numTimesteps)-1; end;
if indexNanb(2) == 1; indexNanb(2) = (numTimesteps)-1; end;
if indexNanb(3) == 1; indexNanb(3) = (numTimesteps)-1; end;
if indexNanb(4) == 1; indexNanb(4) = (numTimesteps)-1; end;
if indexNanb(5) == 1; indexNanb(5) = (numTimesteps)-1; end;
if indexNanb(6) == 1; indexNanb(6) = (numTimesteps)-1; end;
if indexNanb(7) == 1; indexNanb(7) = (numTimesteps)-1; end;
if indexNanb(8) == 1; indexNanb(8) = (numTimesteps)-1; end;
if indexNanb(9) == 1; indexNanb(9) = (numTimesteps)-1; end;

meanBetaFromMeanData(1) = mean(sortedbeta(1,startib(1):indexNanb(1)-1));
meanBetaFromMeanData(2) = mean(sortedbeta(2,startib(2):indexNanb(2)-1));
meanBetaFromMeanData(3) = mean(sortedbeta(3,startib(3):indexNanb(3)-1));
meanBetaFromMeanData(4) = mean(sortedbeta(4,startib(4):indexNanb(4)-1));
meanBetaFromMeanData(5) = mean(sortedbeta(5,startib(5):indexNanb(5)-1));
meanBetaFromMeanData(6) = mean(sortedbeta(6,startib(6):indexNanb(6)-1));
meanBetaFromMeanData(7) = mean(sortedbeta(7,startib(7):indexNanb(7)-1));
meanBetaFromMeanData(8) = mean(sortedbeta(8,startib(8):indexNanb(8)-1));
meanBetaFromMeanData(9) = mean(sortedbeta(9,startib(9):indexNanb(9)-1))
Y;
 %sort gamma to get rid of nan and inf values
sortedgamma = sort(Y);
nanMaskg= isnan(sortedgamma);
infMaskg = isinf(sortedgamma);
[startvg startig] = min(infMaskg,[],2);
[cg indexNang] = max(nanMaskg,[],2);
if indexNang == 1; indexNang(1) = (numTimesteps)-1; end;
if indexNang == 2; indexNang(2) = (numTimesteps)-1; end;
if indexNang == 3; indexNang(3) = (numTimesteps)-1; end;
if indexNang == 4; indexNang(4) = (numTimesteps)-1; end;
if indexNang == 5; indexNang(5) = (numTimesteps)-1; end;
if indexNang == 6; indexNang(6) = (numTimesteps)-1; end;
if indexNang == 7; indexNang(7) = (numTimesteps)-1; end;
if indexNang == 8; indexNang(8) = (numTimesteps)-1; end;
if indexNang == 9; indexNang(9) = (numTimesteps)-1; end;
meanGammaFromMeanData(1) = mean(sortedgamma(1,startig(1):indexNang(1)-1));
meanGammaFromMeanData(2) = mean(sortedgamma(2,startig(2):indexNang(2)-1));
meanGammaFromMeanData(3) = mean(sortedgamma(3,startig(3):indexNang(3)-1));
meanGammaFromMeanData(4) = mean(sortedgamma(4,startig(4):indexNang(4)-1));
meanGammaFromMeanData(5) = mean(sortedgamma(5,startig(5):indexNang(5)-1));
meanGammaFromMeanData(6) = mean(sortedgamma(6,startig(6):indexNang(6)-1));
meanGammaFromMeanData(7) = mean(sortedgamma(7,startig(7):indexNang(7)-1));
meanGammaFromMeanData(8) = mean(sortedgamma(8,startig(8):indexNang(8)-1));
meanGammaFromMeanData(9) = mean(sortedgamma(9,startig(9):indexNang(9)-1))
BN;
 %sort beta with removed runs to get rid of nan and inf values
sortedbeta = sort(BN);
nanMaskb= isnan(sortedbeta);
infMaskb = isinf(sortedbeta);
[startvb startib] = min(infMaskb,[],2);
[cb indexNanb] = max(nanMaskb,[],2);
if indexNanb(1) == 1; indexNanb(1) = (numTimesteps)-1; end;
if indexNanb(2) == 1; indexNanb(2) = (numTimesteps)-1; end;
if indexNanb(3) == 1; indexNanb(3) = (numTimesteps)-1; end;
if indexNanb(4) == 1; indexNanb(4) = (numTimesteps)-1; end;
if indexNanb(5) == 1; indexNanb(5) = (numTimesteps)-1; end;
if indexNanb(6) == 1; indexNanb(6) = (numTimesteps)-1; end;
if indexNanb(7) == 1; indexNanb(7) = (numTimesteps)-1; end;
if indexNanb(8) == 1; indexNanb(8) = (numTimesteps)-1; end;
if indexNanb(9) == 1; indexNanb(9) = (numTimesteps)-1; end;
%sortedbeta(startib(1):indexNanb(1)-1);
meanBetaFromMeanDataN(1) = mean(sortedbeta(1,startib(1):indexNanb(1)-1));
meanBetaFromMeanDataN(2) = mean(sortedbeta(2,startib(2):indexNanb(2)-1));
meanBetaFromMeanDataN(3) = mean(sortedbeta(3,startib(3):indexNanb(3)-1));
meanBetaFromMeanDataN(4) = mean(sortedbeta(4,startib(4):indexNanb(4)-1));
meanBetaFromMeanDataN(5) = mean(sortedbeta(5,startib(5):indexNanb(5)-1));
meanBetaFromMeanDataN(6) = mean(sortedbeta(6,startib(6):indexNanb(6)-1));
meanBetaFromMeanDataN(7) = mean(sortedbeta(7,startib(7):indexNanb(7)-1));
meanBetaFromMeanDataN(8) = mean(sortedbeta(8,startib(8):indexNanb(8)-1));
meanBetaFromMeanDataN(9) = mean(sortedbeta(9,startib(9):indexNanb(9)-1))
YN;
 %sort gamma with removed runs to get rid of nan and inf values
sortedgamma = sort(YN);
nanMaskg= isnan(sortedgamma);
infMaskg = isinf(sortedgamma);
[startvg startig] = min(infMaskg,[],2);
[cg indexNang] = max(nanMaskg,[],2);
if indexNang(1) == 1; indexNang(1) = (numTimesteps)-1; end;
if indexNang(2) == 1; indexNang(2) = (numTimesteps)-1; end;
if indexNang(3) == 1; indexNang(3) = (numTimesteps)-1; end;
if indexNang(4) == 1; indexNang(4) = (numTimesteps)-1; end;
if indexNang(5) == 1; indexNang(5) = (numTimesteps)-1; end;
if indexNang(6) == 1; indexNang(6) = (numTimesteps)-1; end;
if indexNang(7) == 1; indexNang(7) = (numTimesteps)-1; end;
if indexNang(8) == 1; indexNang(8) = (numTimesteps)-1; end;
if indexNang(9) == 1; indexNang(9) = (numTimesteps)-1; end;
meanGammaFromMeanDataN(1) = mean(sortedgamma(1,startig(1):indexNang(1)-1));
meanGammaFromMeanDataN(2) = mean(sortedgamma(2,startig(2):indexNang(2)-1));
meanGammaFromMeanDataN(3) = mean(sortedgamma(3,startig(3):indexNang(3)-1));
meanGammaFromMeanDataN(4) = mean(sortedgamma(4,startig(4):indexNang(4)-1));
meanGammaFromMeanDataN(5) = mean(sortedgamma(5,startig(5):indexNang(5)-1));
meanGammaFromMeanDataN(6) = mean(sortedgamma(6,startig(6):indexNang(6)-1));
meanGammaFromMeanDataN(7) = mean(sortedgamma(7,startig(7):indexNang(7)-1));
meanGammaFromMeanDataN(8) = mean(sortedgamma(8,startig(8):indexNang(8)-1));
meanGammaFromMeanDataN(9) = mean(sortedgamma(9,startig(9):indexNang(9)-1))

R = betameanmean./gammameanmean
RN = betameanmeanNew./gammameanmeanNew
RTheoretical = beta./gamma

figro = figure
%plot(beta,R,'k')
hold on
plot(beta,RN,'k')
plot(beta,RTheoretical,'k.-')
plot(beta,[1 1 1 1 1 1 1 1 1],'k--')
xlabel('beta')
ylabel('ro')
%legend('R','RNew','Rtheoretical') 
legend('Ro measured','Ro theoretical') 
axis([0.1 0.5 0 2])

figbeta = figure
plot(betameanmean,'k')
hold on
plot(betameanmeanNew,'k')
plot(beta,'k.-')
xlabel('sim')
ylabel('betamean')
legend('betameanmean','betameanmeanNew','betaTheory') 

figgamma = figure
plot(gammameanmean,'k')
hold on
plot(gammameanmeanNew,'g')
plot(gamma,'r')
xlabel('sim')
ylabel('gammamean')
legend('gammameanmean','gammameanmeanNew','gammaTheory') 

saveas(figbeta,[outputDirectory 'Test5figbeta.fig']);
saveas(figbeta,[outputDirectory 'Test5figbeta.emf']);
saveas(figgamma,[outputDirectory 'Test5figgamma.fig']);
saveas(figgamma,[outputDirectory 'Test5figgamma.emf']);

saveas(figmean,[outputDirectory 'Test5figmean.fig']);
saveas(figmean,[outputDirectory 'Test5figmean.emf']);
saveas(figmeannone,[outputDirectory 'Test5figmeannone.fig']);
saveas(figmeannone,[outputDirectory 'Test5figmeannone.emf']);
saveas(figmedian,[outputDirectory 'Test5figmedian.fig']);
saveas(figmedian,[outputDirectory 'Test5figmedian.emf']);
saveas(figro,[outputDirectory 'Test5figro.fig']);
saveas(figro,[outputDirectory 'Test5figro.emf']);
saveas(figInfectedRun1,[outputDirectory 'Test5figInfectedRun1.fig']);
saveas(figInfectedRun1,[outputDirectory 'Test5figInfectedRun1.emf']);
saveas(figBetaRun1,[outputDirectory 'Test5figBetaRun1.fig']);
saveas(figBetaRun1,[outputDirectory 'Test5figBetaRun1.emf']);
saveas(figGammaRun1,[outputDirectory 'Test5figGammaRun1.fig']);
saveas(figGammaRun1,[outputDirectory 'Test5figGammaRun1.emf']);
saveas(figRoRun1,[outputDirectory 'Test5figRoRun1.fig']);
saveas(figRoRun1,[outputDirectory 'Test5figRoRun1.emf']);
close all;
end % SIR TEST 5






%***************
% SIRN TESTS - with genotypes
%**************
%************************************
% SIRN TEST 1
%*****************
if doSIRNTests(1) == 1
% compare against doTest(1) and deterministic
outputDirectory = 'randomGenotypesGraphs\'
mode = 'static'
numRuns = 1000; numNodes = 1000; betas = 0.3; numGenotypes =  1;  numTimesteps = 80; plotGraphs = 0; gamma = 0; numExploits=1; numLoci=1; casetype = 'AND';
[betameanmean(1) betameanmean2(1) Smean(1,:) Imean(1,:) Rmean(1,:) NSmean(1,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
%

%Deterministic SI
N=numNodes %100
S = N %total number of susceptible nodes in network
R=0, I=1, drawPlots=0,
[DRo StD ItD RtD] = SIRDeterministic(betas, gamma, S, I, R, numTimesteps, drawPlots);
t2 = 1:1:numTimesteps;
ItD2 = ItD(1:1:end);
StD2 = StD(1:1:end);
 
figSImean = figure
 for i = 1:1
     %agent
     %plot(Smean(i,:),'k--')
     hold on
     plot(Imean(i,:),'ko-', 'Linewidth', 2)
     %deterministic
     %plot(t2,StD2,'ko-')
     plot(t2,ItD2,'k.-', 'Linewidth', 2)
 end
 ylabel('Number of individuals')
 xlabel('Timesteps')
 %title('Increasing variants1')
 legend('Agent I(t)','Deterministic I(t)')
 saveas(figSImean,[outputDirectory 'Test1figSImean.fig']);
saveas(figSImean,[outputDirectory 'Test1figSImean.emf']);
 
end %SIRN TEST 1

%***************
% SIRN TEST 2
% SI - single locus, increase number of variants, 1 exploit - takes 45mins
%***************
if doSIRNTests(2) == 1
outputDirectory = 'randomGenotypesGraphs\'
%Increase variants upto 8, fix exploit
%***************************************
%SIRN testing
%***************************************
mode = 'static'
%monte carlo random genotype SIR model
numRuns = 200
%1 genotype
numNodes = 100; betas = 0.02; numGenotypes =  1;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=1; numLoci=1; casetype = 'AND';
[betameanmean(1) betameanmean2(1) Smean(1,:) Imean(1,:) Rmean(1,:) NSmean(1,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
%2 genotype
numNodes = 100; betas = 0.02; numGenotypes =  2;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=1; numLoci=1; casetype = 'AND';
[betameanmean(2) betameanmean2(2) Smean(2,:) Imean(2,:) Rmean(2,:) NSmean(2,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
%3 genotype
numNodes = 100; betas = 0.02; numGenotypes =  3;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=1; numLoci=1; casetype = 'AND';
[betameanmean(3) betameanmean2(3) Smean(3,:) Imean(3,:) Rmean(3,:) NSmean(3,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
%4 genotype
numNodes = 100; betas = 0.02; numGenotypes =  4;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=1; numLoci=1; casetype = 'AND';
[betameanmean(4) betameanmean2(4) Smean(4,:) Imean(4,:) Rmean(4,:) NSmean(4,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
%5 genotype
numNodes = 100; betas = 0.02; numGenotypes =  5;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=1; numLoci=1; casetype = 'AND';
[betameanmean(5) betameanmean2(5) Smean(5,:) Imean(5,:) Rmean(5,:) NSmean(5,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
%6 genotype
numNodes = 100; betas = 0.02; numGenotypes =  6;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=1; numLoci=1; casetype = 'AND';
[betameanmean(6) betameanmean2(6) Smean(6,:) Imean(6,:) Rmean(6,:) NSmean(6,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
%7 genotype
numNodes = 100; betas = 0.02; numGenotypes =  7;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=1; numLoci=1; casetype = 'AND';
[betameanmean(7) betameanmean2(7) Smean(7,:) Imean(7,:) Rmean(7,:) NSmean(7,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
%8 genotype
numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=1; numLoci=1; casetype = 'AND';
[betameanmean(8) betameanmean2(8) Smean(8,:) Imean(8,:) Rmean(8,:) NSmean(8,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);

%Deterministic SI model
N=100; Vmax = 1:8; V=1%fixed exploited
Sprop = V./Vmax; S = N.*Sprop; B = 0.02; beta = B.*Sprop; gamma = 0;
for i = 1:8
    [Ro(i) St(i,:) It(i,:) Rt(i,:)] = SIRDeterministic(beta(i), gamma, S(i), 1, 0, numTimesteps, 0); 
end
dec = 50; t2 = 1:dec:numTimesteps; 


betameanmeanD=betameanmean
 figTest9beta0_02 = figure
 plot(betameanmean,'k')
 hold on
 plot(beta,'k.-')
 ylabel('beta')
 xlabel('Number of variants')
 %title('b) SI-Increasing variants on 1 locus - ')
 title('b) SI-Increasing variants: Infection rate')
 legend('Agent(measured)', 'Deterministic')
 
 figTest9inf0_02 = figure
 for i = 1:8
     plot(Imean(i,:),'k')
     hold on
     plot(t2,It(i,1:dec:end),'k.-')
 end
 ylabel('Number of nodes infected - It')
 xlabel('Timesteps')
 title('a) SI-Increasing variants: Nodes infected')
 legend('Agent', 'Deterministic')
 axis([0 1000 0 100])
 
 %number of nodes infected
 figTest2FinalInfected = figure
 agentImax = max(Imean,[],2);
 plot(agentImax,'k')
 hold on
 plot(S,'k.-')
 ylabel('Final number infected')
 xlabel('Number of variants')
 title('c) SI-Increasing variants: Final size')
 legend('Agent', 'Deterministic')
 
saveas(figTest9beta0_02,[outputDirectory 'Test2beta0_02.fig']);
saveas(figTest9beta0_02,[outputDirectory 'Test2beta0_02.emf']);
saveas(figTest9inf0_02,[outputDirectory 'Test2inf0_02.fig']);
saveas(figTest9inf0_02,[outputDirectory 'Test2inf0_02.emf']); 
saveas(figTest2FinalInfected,[outputDirectory 'Test2FinalInfected.fig']);
saveas(figTest2FinalInfected,[outputDirectory 'Test2FinalInfected.emf']); 

 %close all;
end %SIRN TEST 2

%***************
% SIRN TEST 3
% SI - fix variants at 8 and increase number of exploits - takes 45 mins
%***************
if doSIRNTests(3) == 1
outputDirectory = 'randomGenotypesGraphs\'
%monte carlo random genotype SIR model
mode = 'static'
numRuns = 500;
%1 genotype
%agent model
numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=1; numLoci=1; casetype = 'AND';
[betameanmean(1) betameanmean2(1) Smean(1,:) Imean(1,:) Rmean(1,:) NSmean(1,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
%2 genotype
numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=2; numLoci=1; casetype = 'AND';
[betameanmean(2) betameanmean2(2) Smean(2,:) Imean(2,:) Rmean(2,:) NSmean(2,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
%3 genotype
numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=3; numLoci=1; casetype = 'AND';
[betameanmean(3) betameanmean2(3) Smean(3,:) Imean(3,:) Rmean(3,:) NSmean(3,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
%4 genotype
numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=4; numLoci=1; casetype = 'AND';
[betameanmean(4) betameanmean2(4) Smean(4,:) Imean(4,:) Rmean(4,:) NSmean(4,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
%5 genotype
numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=5; numLoci=1; casetype = 'AND';
[betameanmean(5) betameanmean2(5) Smean(5,:) Imean(5,:) Rmean(5,:) NSmean(5,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
%6 genotype
numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=6; numLoci=1; casetype = 'AND';
[betameanmean(6) betameanmean2(6) Smean(6,:) Imean(6,:) Rmean(6,:) NSmean(6,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
%7 genotype
numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=7; numLoci=1; casetype = 'AND';
[betameanmean(7) betameanmean2(7) Smean(7,:) Imean(7,:) Rmean(7,:) NSmean(7,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
%8 genotype
numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=8; numLoci=1; casetype = 'AND';
[betameanmean(8) betameanmean2(8) Smean(8,:) Imean(8,:) Rmean(8,:) NSmean(8,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
 
%Deterministic SI model
N=100; Vmax = 8; V=1:Vmax %vary number exploited
Sprop = V./Vmax; S = N.*Sprop; B = 0.02; beta = B.*Sprop; gamma = 0; 
for i = 1:8
    [Ro(i) St(i,:) It(i,:) Rt(i,:)] = SIRDeterministic(beta(i), gamma, S(i), 1, 0, numTimesteps, 0); 
end
dec = 50; t2 = 1:dec:numTimesteps;


betameanmeanD=betameanmean
  figTest10beta0_02 = figure
 plot(betameanmean,'k')
 hold on
 plot(beta,'k.-')
 ylabel('beta')
 xlabel('Number of exploits')
 title('b) SI-Increasing exploits: Infection rate')
 legend('Agent(measured)', 'Deterministic')
 
  figTest10inf0_02 = figure
 for i = 1:8
     plot(Imean(i,:),'k')
     hold on
     plot(t2,It(i,1:dec:end),'k.-')
 end
 ylabel('Number of nodes infected - It')
 xlabel('Timesteps')
 title('a) SI-Increasing exploits: Nodes infected')
 legend('Agent', 'Deterministic')
 axis([0 1000 0 100])
 
 %number of nodes infected
 figTest3FinalInfected = figure
 agentImax = max(Imean,[],2);
 plot(agentImax,'k')
 hold on
 plot(S,'k.-')
 ylabel('Final number infected')
 xlabel('Number of exploits')
 title('c) SI-Increasing exploits: Final size')
 legend('Agent', 'Deterministic')
 
saveas(figTest10beta0_02,[outputDirectory 'Test3beta0_02.fig']);
saveas(figTest10beta0_02,[outputDirectory 'Test3beta0_02.emf']);
saveas(figTest10inf0_02,[outputDirectory 'Test3inf0_02.fig']);
saveas(figTest10inf0_02,[outputDirectory 'Test3inf0_02.emf']); 
saveas(figTest3FinalInfected,[outputDirectory 'Test3FinalInfected.fig']);
saveas(figTest3FinalInfected,[outputDirectory 'Test3FinalInfected.emf']); 

close all;
end %SIRN TEST 3

%***************
% SIRN TEST 4
% SI - 2 loci AND fix exploit at 1 and increase variants - takes 45 mins
%***************
if doSIRNTests(4) == 1
outputDirectory = 'randomGenotypesGraphs\'
%monte carlo random genotype SIRN model
mode = 'static'
numRuns = 200;
%1 genotype
numNodes = 400; betas = 0.02; numGenotypes =  1;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=1; numLoci=2; casetype = 'AND';
[betameanmean(1) betameanmean2(1) Smean(1,:) Imean(1,:) Rmean(1,:) NSmean(1,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
%random g
%2 genotype
numNodes = 400; betas = 0.02; numGenotypes =  2;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=1; numLoci=2; casetype = 'AND';
[betameanmean(2) betameanmean2(2) Smean(2,:) Imean(2,:) Rmean(2,:) NSmean(2,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
%3 genotype
numNodes = 400; betas = 0.02; numGenotypes =  3;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=1; numLoci=2; casetype = 'AND';
[betameanmean(3) betameanmean2(3) Smean(3,:) Imean(3,:) Rmean(3,:) NSmean(3,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
%4 genotype
numNodes = 400; betas = 0.02; numGenotypes =  4;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=1; numLoci=2; casetype = 'AND';
[betameanmean(4) betameanmean2(4) Smean(4,:) Imean(4,:) Rmean(4,:) NSmean(4,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
%5 genotype
numNodes = 400; betas = 0.02; numGenotypes =  5;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=1; numLoci=2; casetype = 'AND';
[betameanmean(5) betameanmean2(5) Smean(5,:) Imean(5,:) Rmean(5,:) NSmean(5,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
%6 genotype
numNodes = 400; betas = 0.02; numGenotypes =  6;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=1; numLoci=2; casetype = 'AND';
[betameanmean(6) betameanmean2(6) Smean(6,:) Imean(6,:) Rmean(6,:) NSmean(6,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
%7 genotype
numNodes = 400; betas = 0.02; numGenotypes =  7;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=1; numLoci=2; casetype = 'AND';
[betameanmean(7) betameanmean2(7) Smean(7,:) Imean(7,:) Rmean(7,:) NSmean(7,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
%8 genotype
numNodes = 400; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=1; numLoci=2; casetype = 'AND';
[betameanmean(8) betameanmean2(8) Smean(8,:) Imean(8,:) Rmean(8,:) NSmean(8,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
  
    
%Deterministic - 2 loci
N=400; Vmax = 1:8; G = Vmax.^2; %number genotypes
%both loci have the same number of variants and both have 1 exploit
Sprop = 1./G; S = N.*Sprop; B = 0.02; beta = B.*Sprop; gamma = 0;
for i = 1:8
    [Ro(i) St(i,:) It(i,:) Rt(i,:)] = SIRDeterministic(beta(i), gamma, S(i), 1, 0, numTimesteps, 0); 
end
dec = 50; t2 = 1:dec:numTimesteps;

betameanmeanD=betameanmean
 figTest9beta0_02 = figure
 plot(betameanmean,'k')
 hold on
 plot(beta,'k.-')
 ylabel('beta')
 xlabel('Number of variants')
 %title('b) SI-Increasing variants on 1 locus - ')
 title('b) SI-Increasing variants: Infection rate')
 legend('Agent(measured)', 'Deterministic')
 
 figTest9inf0_02 = figure
 for i = 1:8
     plot(Imean(i,:),'k')
     hold on
     plot(t2,It(i,1:dec:end),'k.-')
 end
 ylabel('Number of nodes infected - It')
 xlabel('Timesteps')
 title('a) SI-Increasing variants: Nodes infected')
 legend('Agent', 'Deterministic')
 axis([0 1000 0 400])
 
 %number of nodes infected
 figTest2FinalInfected = figure
 agentImax = max(Imean,[],2);
 plot(agentImax,'k')
 hold on
 plot(S,'k.-')
 ylabel('Final number infected')
 xlabel('Number of variants')
 title('c) SI-Increasing variants: Final size')
 legend('Agent', 'Deterministic')
 
saveas(figTest9beta0_02,[outputDirectory 'Test4beta0_02.fig']);
saveas(figTest9beta0_02,[outputDirectory 'Test4beta0_02.emf']);
saveas(figTest9inf0_02,[outputDirectory 'Test4inf0_02.fig']);
saveas(figTest9inf0_02,[outputDirectory 'Test4inf0_02.emf']); 
saveas(figTest2FinalInfected,[outputDirectory 'Test4FinalInfected.fig']);
saveas(figTest2FinalInfected,[outputDirectory 'Test4FinalInfected.emf']); 

close all;
end %SIRN Test4

%***************
% SIRN TEST 5
% SI - 2 loci AND fix variants at 8 and increase exploits - takes 45 mins
%***************
if doSIRNTests(5) == 1
outputDirectory = 'randomGenotypesGraphs\'
%monte carlo random genotype SIRN model
mode = 'static'
numRuns = 200;
%1 genotype
numNodes = 400; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=1; numLoci=2; casetype = 'AND';
[betameanmean(1) betameanmean2(1) Smean(1,:) Imean(1,:) Rmean(1,:) NSmean(1,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
%random g
%2 genotype
numNodes = 400; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=2; numLoci=2; casetype = 'AND';
[betameanmean(2) betameanmean2(2) Smean(2,:) Imean(2,:) Rmean(2,:) NSmean(2,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
%3 genotype
numNodes = 400; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=3; numLoci=2; casetype = 'AND';
[betameanmean(3) betameanmean2(3) Smean(3,:) Imean(3,:) Rmean(3,:) NSmean(3,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
%4 genotype
numNodes = 400; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=4; numLoci=2; casetype = 'AND';
[betameanmean(4) betameanmean2(4) Smean(4,:) Imean(4,:) Rmean(4,:) NSmean(4,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
%5 genotype
numNodes = 400; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=5; numLoci=2; casetype = 'AND';
[betameanmean(5) betameanmean2(5) Smean(5,:) Imean(5,:) Rmean(5,:) NSmean(5,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
%6 genotype
numNodes = 400; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=6; numLoci=2; casetype = 'AND';
[betameanmean(6) betameanmean2(6) Smean(6,:) Imean(6,:) Rmean(6,:) NSmean(6,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
%7 genotype
numNodes = 400; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=7; numLoci=2; casetype = 'AND';
[betameanmean(7) betameanmean2(7) Smean(7,:) Imean(7,:) Rmean(7,:) NSmean(7,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
%8 genotype
numNodes = 400; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=8; numLoci=2; casetype = 'AND';
[betameanmean(8) betameanmean2(8) Smean(8,:) Imean(8,:) Rmean(8,:) NSmean(8,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
  
    
%Deterministic - 2 loci
susProp2lociAND =[0.0156,0.0625,0.1406,0.2500,0.3906,0.5625,0.7656,1.0000]
N=400; Vmax = 8
S = N.*susProp2lociAND; B = 0.02; beta = B.*susProp2lociAND; gamma = 0; %1/48*60*60 %2 days
for i = 1:8
    [Ro(i) St(i,:) It(i,:) Rt(i,:)] = SIRDeterministic(beta(i), gamma, S(i), 1, 0, numTimesteps, 0); 
end
dec = 50; t2 = 1:dec:numTimesteps;

betameanmeanD=betameanmean
 figTest9beta0_02 = figure
 plot(betameanmean,'k')
 hold on
 plot(beta,'k.-')
 ylabel('beta')
 xlabel('Number of exploits')
 %title('b) SI-Increasing variants on 1 locus - ')
 title('b) SI-Increasing exploits: Infection rate')
 legend('Agent(measured)', 'Deterministic')
 
 figTest9inf0_02 = figure
 for i = 1:8
     plot(Imean(i,:),'k')
     hold on
     plot(t2,It(i,1:dec:end),'k.-')
 end
 ylabel('Number of nodes infected - It')
 xlabel('Timesteps')
 title('a) SI-Increasing exploits: Nodes infected')
 legend('Agent', 'Deterministic')
 %axis([0 1000 0 400])
 
 %number of nodes infected
 figTest2FinalInfected = figure
 agentImax = max(Imean,[],2);
 plot(agentImax,'k')
 hold on
 plot(S,'k.-')
 ylabel('Final number infected')
 xlabel('Number of exploits')
 title('c) SI-Increasing exploits: Final size')
 legend('Agent', 'Deterministic')
 
saveas(figTest9beta0_02,[outputDirectory 'Test5beta0_02.fig']);
saveas(figTest9beta0_02,[outputDirectory 'Test5beta0_02.emf']);
saveas(figTest9inf0_02,[outputDirectory 'Test5inf0_02.fig']);
saveas(figTest9inf0_02,[outputDirectory 'Test5inf0_02.emf']); 
saveas(figTest2FinalInfected,[outputDirectory 'Test5FinalInfected.fig']);
saveas(figTest2FinalInfected,[outputDirectory 'Test5FinalInfected.emf']); 

close all;
end %SIRN Test5

%***************
% SIRN TEST 6
% SI - 2 loci OR fix exploit at 1 and increase variants - takes 45 mins
%***************
if doSIRNTests(6) == 1
outputDirectory = 'randomGenotypesGraphs\'
%monte carlo random genotype SIRN model
mode = 'static'
numRuns = 500;
%1 genotype
numNodes = 100; betas = 0.02; numGenotypes =  1;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=1; numLoci=2; casetype = 'OR';
[betameanmean(1) betameanmean2(1) Smean(1,:) Imean(1,:) Rmean(1,:) NSmean(1,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
%random g
%2 genotype
numNodes = 100; betas = 0.02; numGenotypes =  2;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=1; numLoci=2; casetype = 'OR';
[betameanmean(2) betameanmean2(2) Smean(2,:) Imean(2,:) Rmean(2,:) NSmean(2,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
%3 genotype
numNodes = 100; betas = 0.02; numGenotypes =  3;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=1; numLoci=2; casetype = 'OR';
[betameanmean(3) betameanmean2(3) Smean(3,:) Imean(3,:) Rmean(3,:) NSmean(3,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
%4 genotype
numNodes = 100; betas = 0.02; numGenotypes =  4;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=1; numLoci=2; casetype = 'OR';
[betameanmean(4) betameanmean2(4) Smean(4,:) Imean(4,:) Rmean(4,:) NSmean(4,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
%5 genotype
numNodes = 100; betas = 0.02; numGenotypes =  5;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=1; numLoci=2; casetype = 'OR';
[betameanmean(5) betameanmean2(5) Smean(5,:) Imean(5,:) Rmean(5,:) NSmean(5,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
%6 genotype
numNodes = 100; betas = 0.02; numGenotypes =  6;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=1; numLoci=2; casetype = 'OR';
[betameanmean(6) betameanmean2(6) Smean(6,:) Imean(6,:) Rmean(6,:) NSmean(6,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
%7 genotype
numNodes = 100; betas = 0.02; numGenotypes =  7;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=1; numLoci=2; casetype = 'OR';
[betameanmean(7) betameanmean2(7) Smean(7,:) Imean(7,:) Rmean(7,:) NSmean(7,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
%8 genotype
numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=1; numLoci=2; casetype = 'OR';
[betameanmean(8) betameanmean2(8) Smean(8,:) Imean(8,:) Rmean(8,:) NSmean(8,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
  
    
%Deterministic - 2 loci
N=100; Vmax = 1:8;  G = Vmax.^2; %number genotypes
%for OR case
Sprop = 2./Vmax - 1./G %both loci have the same number of variants and both have 1 exploit
S = N.*Sprop; B = 0.02; beta = B.*Sprop; gamma = 0; %1/48*60*60 %2 days
for i = 1:8
    [Ro(i) St(i,:) It(i,:) Rt(i,:)] = SIRDeterministic(beta(i), gamma, S(i), 1, 0, numTimesteps, 0); 
end
dec = 50; t2 = 1:dec:numTimesteps;

betameanmeanD=betameanmean
 figTest9beta0_02 = figure
 plot(betameanmean,'k')
 hold on
 plot(beta,'k.-')
 ylabel('beta')
 xlabel('Number of variants')
 %title('b) SI-Increasing variants on 1 locus - ')
 title('b) SI-Increasing variants: Infection rate')
 legend('Agent(measured)', 'Deterministic')
 
 figTest9inf0_02 = figure
 for i = 1:8
     plot(Imean(i,:),'k')
     hold on
     plot(t2,It(i,1:dec:end),'k.-')
 end
 ylabel('Number of nodes infected - It')
 xlabel('Timesteps')
 title('a) SI-Increasing variants: Nodes infected')
 legend('Agent', 'Deterministic')
 axis([0 1000 0 100])
 
 %number of nodes infected
 figTest2FinalInfected = figure
 agentImax = max(Imean,[],2);
 plot(agentImax,'k')
 hold on
 plot(S,'k.-')
 ylabel('Final number infected')
 xlabel('Number of variants')
 title('c) SI-Increasing variants: Final size')
 legend('Agent', 'Deterministic')
 
saveas(figTest9beta0_02,[outputDirectory 'Test6beta0_02.fig']);
saveas(figTest9beta0_02,[outputDirectory 'Test6beta0_02.emf']);
saveas(figTest9inf0_02,[outputDirectory 'Test6inf0_02.fig']);
saveas(figTest9inf0_02,[outputDirectory 'Test6inf0_02.emf']); 
saveas(figTest2FinalInfected,[outputDirectory 'Test6FinalInfected.fig']);
saveas(figTest2FinalInfected,[outputDirectory 'Test6FinalInfected.emf']); 

close all;
end %SIRN Test6

%***************
% SIRN TEST 7
% SI - 2 loci OR fix variants and increase exploits - takes 45 mins
%***************
if doSIRNTests(7) == 1
outputDirectory = 'randomGenotypesGraphs\'
%monte carlo random genotype SIRN model
mode = 'static'
numRuns = 500;
%1 genotype
numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=1; numLoci=2; casetype = 'OR';
[betameanmean(1) betameanmean2(1) Smean(1,:) Imean(1,:) Rmean(1,:) NSmean(1,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
%random g
%2 genotype
numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=2; numLoci=2; casetype = 'OR';
[betameanmean(2) betameanmean2(2) Smean(2,:) Imean(2,:) Rmean(2,:) NSmean(2,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
%3 genotype
numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=3; numLoci=2; casetype = 'OR';
[betameanmean(3) betameanmean2(3) Smean(3,:) Imean(3,:) Rmean(3,:) NSmean(3,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
%4 genotype
numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=4; numLoci=2; casetype = 'OR';
[betameanmean(4) betameanmean2(4) Smean(4,:) Imean(4,:) Rmean(4,:) NSmean(4,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
%5 genotype
numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=5; numLoci=2; casetype = 'OR';
[betameanmean(5) betameanmean2(5) Smean(5,:) Imean(5,:) Rmean(5,:) NSmean(5,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
%6 genotype
numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=6; numLoci=2; casetype = 'OR';
[betameanmean(6) betameanmean2(6) Smean(6,:) Imean(6,:) Rmean(6,:) NSmean(6,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
%7 genotype
numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=7; numLoci=2; casetype = 'OR';
[betameanmean(7) betameanmean2(7) Smean(7,:) Imean(7,:) Rmean(7,:) NSmean(7,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
%8 genotype
numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=8; numLoci=2; casetype = 'OR';
[betameanmean(8) betameanmean2(8) Smean(8,:) Imean(8,:) Rmean(8,:) NSmean(8,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
  
%Deterministic - 2 loci
susProp2loci = [0.2344,0.4375,0.6094,0.7500,0.8594,0.9375,0.9844,1.0000]%susceptibilityTests.m
N=100; S = N.*susProp2loci; B = 0.02; beta = B.*susProp2loci; gamma = 0;
for i = 1:8
    [Ro(i) St(i,:) It(i,:) Rt(i,:)] = SIRDeterministic(beta(i), gamma, S(i), 1, 0, numTimesteps, 0); 
end
dec = 50; t2 = 1:dec:numTimesteps;

betameanmeanD=betameanmean
 figTest9beta0_02 = figure
 plot(betameanmean,'k')
 hold on
 plot(beta,'k.-')
 ylabel('beta')
 xlabel('Number of variants')
 %title('b) SI-Increasing variants on 1 locus - ')
 title('b) SI-Increasing variants: Infection rate')
 legend('Agent(measured)', 'Deterministic')
 
 figTest9inf0_02 = figure
 for i = 1:8
     plot(Imean(i,:),'k')
     hold on
     plot(t2,It(i,1:dec:end),'k.-')
 end
 ylabel('Number of nodes infected - It')
 xlabel('Timesteps')
 title('a) SI-Increasing variants: Nodes infected')
 legend('Agent', 'Deterministic')
 axis([0 1000 0 100])
 
 %number of nodes infected
 figTest2FinalInfected = figure
 agentImax = max(Imean,[],2);
 plot(agentImax,'k')
 hold on
 plot(S,'k.-')
 ylabel('Final number infected')
 xlabel('Number of variants')
 title('c) SI-Increasing variants: Final size')
 legend('Agent', 'Deterministic')
 
saveas(figTest9beta0_02,[outputDirectory 'Test7beta0_02.fig']);
saveas(figTest9beta0_02,[outputDirectory 'Test7beta0_02.emf']);
saveas(figTest9inf0_02,[outputDirectory 'Test7inf0_02.fig']);
saveas(figTest9inf0_02,[outputDirectory 'Test7inf0_02.emf']); 
saveas(figTest2FinalInfected,[outputDirectory 'Test7FinalInfected.fig']);
saveas(figTest2FinalInfected,[outputDirectory 'Test7FinalInfected.emf']); 

close all;
end %SIRN Test7


%*******************8
%SIR tests
%doSIRNTests(8) = 1 %SIR Increase variants upto 8, fix exploit - single locus
%doSIRNTests(9) = 1 %SIR fix variants at 8 and increase number of exploits - single locus
%doSIRNTests(10) = 1 %SIR AND CASE Increase variants upto 8, fix exploit - 2 loci
%doSIRNTests(11) = 1 %SIR AND CASE fix variants at 8 and increase number of exploits - 2 loci
%doSIRNTests(12) = 1 %SIR OR CASE Increase variants upto 8, fix exploit - 2 loci
%doSIRNTests(13) = 1 %SIR OR CASE fix variants at 8 and increase number of exploits - 2 loci

%doSIRNTests(8) = 1 %SIR Increase variants upto 8, fix exploit - single locus
if doSIRNTests(8) == 1
outputDirectory = 'randomGenotypesGraphs\'
%Increase variants upto 8, fix exploit
%***************************************
%SIRN testing
%***************************************
mode = 'static'
%monte carlo random genotype SIR model
numRuns = 200
%1 genotype
numNodes = 400; betas = 0.02; numGenotypes =  1;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1; numLoci=1; casetype = 'AND'; throwAway = 'nonEpidemics', SEpidemic=400-100;
%[betameanmean(1) betameanmean2(1) Smean(1,:) Imean(1,:) Rmean(1,:) NSmean(1,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
[fighistRecovered(1) fighistSusceptible(1) fighistpeakValue(1) fighistpeakTime(1) fighistBeta(1) fighistGamma(1) fighistBetanew(1) fighistGammanew(1) figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean(1) gammameanmean betameanmean2(1) betameanmeanNew(1) betameanmeanNew2 gammameanmeanNew(1) Smean(1,:) Imean(1,:) Rmean(1,:) NSmean(1,:) Smedian Imedian Rmedian S I R NS SmeanN ImeanN1 RmeanN1 Snewendmean Snewendstd Rnewendmean(1) Rnewendstd(1) Rnew_nonendmean Rnew_nonendstd propSimsNonEp(1) propSimsNonEp_theoretical(1)...
            ] = monteCarloSIRN2(numRuns,numNodes, betas, numGenotypes, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,mode, numLoci, casetype);
   size(ImeanN1)
FinalSize(1) = max(RmeanN1,[],2) %mean final size

%2 genotype
numNodes = 400; betas = 0.02; numGenotypes =  2;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1; numLoci=1; casetype = 'AND';
%[betameanmean(2) betameanmean2(2) Smean(2,:) Imean(2,:) Rmean(2,:) NSmean(2,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
[fighistRecovered(2) fighistSusceptible(2) fighistpeakValue(2) fighistpeakTime(2) fighistBeta(2) fighistGamma(2) fighistBetanew(2) fighistGammanew(2) figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean(2) gammameanmean betameanmean2(2) betameanmeanNew(2) betameanmeanNew2 gammameanmeanNew(2) Smean(2,:) Imean(2,:) Rmean(2,:) NSmean(2,:) Smedian Imedian Rmedian S I R NS SmeanN ImeanN2 RmeanN2 Snewendmean Snewendstd Rnewendmean(2) Rnewendstd(2) Rnew_nonendmean Rnew_nonendstd propSimsNonEp(2) propSimsNonEp_theoretical(2)...
            ] = monteCarloSIRN2(numRuns,numNodes, betas, numGenotypes, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,mode, numLoci, casetype);
   size(ImeanN2)
FinalSize(2) = max(RmeanN2,[],2) %mean final size

%3 genotype
numNodes = 400; betas = 0.02; numGenotypes =  3;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1; numLoci=1; casetype = 'AND';
%[betameanmean(3) betameanmean2(3) Smean(3,:) Imean(3,:) Rmean(3,:) NSmean(3,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
[fighistRecovered(3) fighistSusceptible(3) fighistpeakValue(3) fighistpeakTime(3) fighistBeta(3) fighistGamma(3) fighistBetanew(3) fighistGammanew(3) figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean(3) gammameanmean betameanmean2(3) betameanmeanNew(3) betameanmeanNew2 gammameanmeanNew(3) Smean(3,:) Imean(3,:) Rmean(3,:) NSmean(3,:) Smedian Imedian Rmedian S I R NS SmeanN ImeanN3 RmeanN3 Snewendmean Snewendstd Rnewendmean(3) Rnewendstd(3) Rnew_nonendmean Rnew_nonendstd propSimsNonEp(3) propSimsNonEp_theoretical(3)...
            ] = monteCarloSIRN2(numRuns,numNodes, betas, numGenotypes, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,mode, numLoci, casetype);

   size(ImeanN3)
FinalSize(3) = max(RmeanN3,[],2) %mean final size
%4 genotype
numNodes = 400; betas = 0.02; numGenotypes =  4;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1; numLoci=1; casetype = 'AND';
%[betameanmean(4) betameanmean2(4) Smean(4,:) Imean(4,:) Rmean(4,:) NSmean(4,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
[fighistRecovered(4) fighistSusceptible(4) fighistpeakValue(4) fighistpeakTime(4) fighistBeta(4) fighistGamma(4) fighistBetanew(4) fighistGammanew(4) figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean(4) gammameanmean betameanmean2(4) betameanmeanNew(4) betameanmeanNew2 gammameanmeanNew(4) Smean(4,:) Imean(4,:) Rmean(4,:) NSmean(4,:) Smedian Imedian Rmedian S I R NS SmeanN ImeanN4 RmeanN4 Snewendmean Snewendstd Rnewendmean(4) Rnewendstd(4) Rnew_nonendmean Rnew_nonendstd propSimsNonEp(4) propSimsNonEp_theoretical(4)...
            ] = monteCarloSIRN2(numRuns,numNodes, betas, numGenotypes, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,mode, numLoci, casetype);

   size(ImeanN4)
FinalSize(4) = max(RmeanN4,[],2) %mean final size
    %5 genotype
numNodes = 400; betas = 0.02; numGenotypes =  5;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1; numLoci=1; casetype = 'AND';
%[betameanmean(5) betameanmean2(5) Smean(5,:) Imean(5,:) Rmean(5,:) NSmean(5,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
[fighistRecovered(5) fighistSusceptible(5) fighistpeakValue(5) fighistpeakTime(5) fighistBeta(5) fighistGamma(5) fighistBetanew(5) fighistGammanew(5) figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean(5) gammameanmean betameanmean2(5) betameanmeanNew(5) betameanmeanNew2 gammameanmeanNew(5) Smean(5,:) Imean(5,:) Rmean(5,:) NSmean(5,:) Smedian Imedian Rmedian S I R NS SmeanN ImeanN5 RmeanN5 Snewendmean Snewendstd Rnewendmean(5) Rnewendstd(5) Rnew_nonendmean Rnew_nonendstd propSimsNonEp(5) propSimsNonEp_theoretical(5)...
            ] = monteCarloSIRN2(numRuns,numNodes, betas, numGenotypes, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,mode, numLoci, casetype);

   size(ImeanN5)
FinalSize(5) = max(RmeanN5,[],2) %mean final size
  %6 genotype
numNodes = 400; betas = 0.02; numGenotypes =  6;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1; numLoci=1; casetype = 'AND';
%[betameanmean(6) betameanmean2(6) Smean(6,:) Imean(6,:) Rmean(6,:) NSmean(6,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
[fighistRecovered(6) fighistSusceptible(6) fighistpeakValue(6) fighistpeakTime(6) fighistBeta(6) fighistGamma(6) fighistBetanew(6) fighistGammanew(6) figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean(6) gammameanmean betameanmean2(6) betameanmeanNew(6) betameanmeanNew2 gammameanmeanNew(6) Smean(6,:) Imean(6,:) Rmean(6,:) NSmean(6,:) Smedian Imedian Rmedian S I R NS SmeanN ImeanN6 RmeanN6 Snewendmean Snewendstd Rnewendmean(6) Rnewendstd(6) Rnew_nonendmean Rnew_nonendstd propSimsNonEp(6) propSimsNonEp_theoretical(6)...
            ] = monteCarloSIRN2(numRuns,numNodes, betas, numGenotypes, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,mode, numLoci, casetype);


   size(ImeanN6)
FinalSize(6) = max(RmeanN6,[],2) %mean final size
        %7 genotype
numNodes = 400; betas = 0.02; numGenotypes =  7;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1; numLoci=1; casetype = 'AND';
%[betameanmean(7) betameanmean2(7) Smean(7,:) Imean(7,:) Rmean(7,:) NSmean(7,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
[fighistRecovered(7) fighistSusceptible(7) fighistpeakValue(7) fighistpeakTime(7) fighistBeta(7) fighistGamma(7) fighistBetanew(7) fighistGammanew(7) figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2 ...
            betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean(7) gammameanmean betameanmean2(7) betameanmeanNew(7) betameanmeanNew2 gammameanmeanNew(7) Smean(7,:) Imean(7,:) Rmean(7,:) NSmean(7,:) Smedian Imedian Rmedian S I R NS SmeanN ImeanN7 RmeanN7 Snewendmean Snewendstd Rnewendmean(7) Rnewendstd(7) Rnew_nonendmean Rnew_nonendstd propSimsNonEp(7) propSimsNonEp_theoretical(7)...
            ] = monteCarloSIRN2(numRuns,numNodes, betas, numGenotypes, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,mode, numLoci, casetype);

   size(ImeanN7)
FinalSize(7) = max(RmeanN7,[],2) %mean final size
        %8 genotype
numNodes = 400; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1; numLoci=1; casetype = 'AND';
%[betameanmean(8) betameanmean2(8) Smean(8,:) Imean(8,:) Rmean(8,:) NSmean(8,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
[fighistRecovered(8) fighistSusceptible(8) fighistpeakValue(8) fighistpeakTime(8) fighistBeta(8) fighistGamma(8) fighistBetanew(8) fighistGammanew(8) figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean(8) gammameanmean betameanmean2(8) betameanmeanNew(8) betameanmeanNew2 gammameanmeanNew(8) Smean(8,:) Imean(8,:) Rmean(8,:) NSmean(8,:) Smedian Imedian Rmedian S I R NS SmeanN ImeanN8 RmeanN8 Snewendmean Snewendstd Rnewendmean(8) Rnewendstd(8) Rnew_nonendmean Rnew_nonendstd propSimsNonEp(8) propSimsNonEp_theoretical(8)...
            ] = monteCarloSIRN2(numRuns,numNodes, betas, numGenotypes, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,mode, numLoci, casetype);

   size(ImeanN8)
FinalSize(8) = max(RmeanN8,[],2) %mean final size
    %Histograms
 for i = 1:8
    saveas(fighistRecovered(i),[outputDirectory 'Test8fighistRecovered' num2str(i) '.fig']);
    saveas(fighistRecovered(i),[outputDirectory 'Test8fighistRecovered' num2str(i) '.emf']);
    saveas(fighistSusceptible(i),[outputDirectory 'Test8fighistSusceptible' num2str(i) '.fig']);
    saveas(fighistSusceptible(i),[outputDirectory 'Test8fighistSusceptible' num2str(i) '.emf']);
    saveas(fighistpeakValue(i),[outputDirectory 'Test8fighistpeakValue' num2str(i) '.fig']);
    saveas(fighistpeakValue(i),[outputDirectory 'Test8fighistpeakValue' num2str(i) '.emf']);
    saveas(fighistpeakTime(i),[outputDirectory 'Test8fighistpeakTime' num2str(i) '.fig']);
    saveas(fighistpeakTime(i),[outputDirectory 'Test8fighistpeakTime' num2str(i) '.emf']);
    saveas(fighistBeta(i),[outputDirectory 'Test8histBeta' num2str(i) '.fig']);
    saveas(fighistBeta(i),[outputDirectory 'Test8histBeta' num2str(i) '.emf']);
    saveas(fighistGamma(i),[outputDirectory 'Test8histGamma' num2str(i) '.fig']);
    saveas(fighistGamma(i),[outputDirectory 'Test8histGamma' num2str(i) '.emf']);
    saveas(fighistBetanew(i),[outputDirectory 'Test8histBetaNew' num2str(i) '.fig']);
    saveas(fighistBetanew(i),[outputDirectory 'Test8histBetaNew' num2str(i) '.emf']);
    saveas(fighistGammanew(i),[outputDirectory 'Test8histGammaNew' num2str(i) '.fig']);
    saveas(fighistGammanew(i),[outputDirectory 'Test8histGammaNew' num2str(i) '.emf']);
 end
 close all;
%Deterministic SI model
N=400; Vmax = 1:8; V=1%fixed exploited
Sprop = V./Vmax; S = N.*Sprop; B = 0.02; beta = B.*Sprop; gamma = 0.01;
for i = 1:8
    [Ro(i) St(i,:) It(i,:) Rt(i,:)] = SIRDeterministic(beta(i), gamma, S(i), 1, 0, numTimesteps, 0); 
end
dec = 50; t2 = 1:dec:numTimesteps; 
%stochastic final size approximation
betaSolve =[0.0200    0.0100    0.0067    0.0050    0.0040   0.0033    0.0029    0.0025] %calculated from above
    meanRTheory1 =  (solve('1-x = exp(-1.0*(0.02/0.01)*x)', 'Real', true))*numNodes
    meanRTheory2 =  (solve('1-x = exp(-1.0*(0.01/0.01)*x)', 'Real', true))*numNodes
    meanRTheory3 =  (solve('1-x = exp(-1.0*(0.0067/0.01)*x)', 'Real', true))*numNodes
    meanRTheory4 =  (solve('1-x = exp(-1.0*(0.0050/0.01)*x)', 'Real', true))*numNodes
    meanRTheory5 =  (solve('1-x = exp(-1.0*(0.0040/0.01)*x)', 'Real', true))*numNodes
    meanRTheory6 =  (solve('1-x = exp(-1.0*(0.0033/0.01)*x)', 'Real', true))*numNodes
    meanRTheory7 =  (solve('1-x = exp(-1.0*(0.0029/0.01)*x)', 'Real', true))*numNodes
    meanRTheory8 =  (solve('1-x = exp(-1.0*(0.0025/0.01)*x)', 'Real', true))*numNodes
FinalSizeS = [max(double(meanRTheory1)) max(double(meanRTheory2)) max(double(meanRTheory3)) max(double(meanRTheory4)) max(double(meanRTheory5)) max(double(meanRTheory6)) max(double(meanRTheory7)) max(double(meanRTheory8))];
%FinalSizeSMask = FinalSizeS>0; %if less than zero, then zero it as only valid if positive
%FinalSizeS = FinalSizeS.*FinalSizeSMask
betameanmeanD=betameanmean
 figTest9beta0_02 = figure
 plot(betameanmeanNew,'ko-')
 hold on
 plot(beta,'k.-')
 ylabel('beta')
 xlabel('Number of variants')
 %title('b) SI-Increasing variants on 1 locus - ')
 title('b) SI-Increasing variants: Infection rate')
 legend('Agent(measured)', 'Deterministic')
 
   figTest9gamma0_02 = figure
 plot(gammameanmeanNew,'ko-')
 hold on
 plot([0.01 0.01 0.01 0.01 0.01 0.01 0.01 0.01],'k.-')
 ylabel('gama')
 xlabel('Number of exploits')
 %title('b) SI-Increasing variants on 1 locus - ')
 title(' SI-Increasing exploits: Recovery rate')
 legend('Agent(measured)', 'Deterministic')
 axis([1 8 0 0.025])
 
 figTest9inf0_02 = figure
 for i = 1:8
     plot(Imean(i,:),'k')
     hold on
     plot(t2,It(i,1:dec:end),'k.-')
 end
 ylabel('Number of nodes infected - It')
 xlabel('Timesteps')
 title('a) SI-Increasing variants: Nodes infected')
 legend('Agent', 'Deterministic')
 %axis([0 1000 0 400])
 
  figmajor = figure
 for i = 1:8
     plot(ImeanN1,'k')
     hold on
     plot(t2,It(i,1:dec:end),'k.-')
     plot(ImeanN2,'k')
     plot(ImeanN3,'k')
     plot(ImeanN4,'k')
     plot(ImeanN5,'k')
     plot(ImeanN6,'k')
     plot(ImeanN7,'k')
 end
xlabel('Timesteps')
ylabel('Number of individuals infected I(t)')
title('Mean of major outbreaks')
 legend('Agent', 'Deterministic')
%axis([0 100 0 90])
 
 
 %number of nodes infected
 figTest2FinalInfected = figure
 %agentImax = max(Imean,[],2);
 %plot(agentImax,'k')
 for j = 1:8
 if isnan(FinalSizeS(j)); h(j) = 0; else h(j) = FinalSizeS(j); end;
 if isnan(FinalSize(j)); q(j) = 0; else q(j) = FinalSize(j); end;
 end
 FinalSize=q
 FinalSizeS=h
 plot(FinalSize,'ko-')
 hold on
 plot(S,'k.-')
 plot(FinalSizeS,'k+-')
 plot([2 2],[0 400],'k--')
 ylabel('Final number infected')
 xlabel('Number of variants')
 title('c) SI-Increasing variants: Final size')
 legend('Agent', 'Deterministic', 'Stochastic Approximation')
 
 
saveas(figTest9beta0_02,[outputDirectory 'Test8beta0_02.fig']);
saveas(figTest9beta0_02,[outputDirectory 'Test8beta0_02.emf']);
saveas(figTest9gamma0_02,[outputDirectory 'Test8gama0_02.fig']);
saveas(figTest9gamma0_02,[outputDirectory 'Test8gama0_02.emf']);
saveas(figTest9inf0_02,[outputDirectory 'Test8meanall.fig']);
saveas(figTest9inf0_02,[outputDirectory 'Test8meanall.emf']); 
saveas(figmajor,[outputDirectory 'Test8meanmajor.fig']);
saveas(figmajor,[outputDirectory 'Test8meanmajor.emf']);
saveas(figTest2FinalInfected,[outputDirectory 'Test8FinalInfected.fig']);
saveas(figTest2FinalInfected,[outputDirectory 'Test8FinalInfected.emf']);
%close all


betameanmean
gammameanmean

betameanmean2
betameanmeanNew 
betameanmeanNew2
gammameanmeanNew

Rnewendmean%epidemics only
Rnewendstd

 propSimsNonEp % proportion of sims with non-epidemics
 propSimsNonEp_theoretical %theoretical value
    
R = betameanmean./gammameanmean
RN = betameanmeanNew./gammameanmeanNew
RTheoretical = beta./gamma

figro = figure
%plot(beta,R,'k')
hold on
plot(RN,'ko-')
plot(RTheoretical,'k.-')
plot([1 1 1 1 1 1 1 1],'k--')
plot([2 2],[0 2],'k--')
xlabel('Number of variants')
ylabel('ro')
%legend('R','RNew','Rtheoretical') 
legend('Ro Agent(measured)','Ro Deterministic') 
%axis([0.1 0.5 0 2])
saveas(figro,[outputDirectory 'Test8figro.fig']);
saveas(figro,[outputDirectory 'Test8figro.emf']);
%close all
end  %SIRN 8

%doSIRNTests(9) = 1 %SIR fix variants at 8 and increase number of exploits - single locus
if doSIRNTests(9) == 1
outputDirectory = 'randomGenotypesGraphs\'
%Increase variants upto 8, fix exploit
%***************************************
%SIRN testing
%***************************************
mode = 'static'
%monte carlo random genotype SIR model
numRuns = 200
%1 genotype
numNodes = 400; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1; numLoci=1; casetype = 'AND'; throwAway = 'nonEpidemics', SEpidemic=400-110;
%[betameanmean(1) betameanmean2(1) Smean(1,:) Imean(1,:) Rmean(1,:) NSmean(1,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
[fighistRecovered(1) fighistSusceptible(1) fighistpeakValue(1) fighistpeakTime(1) fighistBeta(1) fighistGamma(1) fighistBetanew(1) fighistGammanew(1) figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean(1) gammameanmean betameanmean2(1) betameanmeanNew(1) betameanmeanNew2 gammameanmeanNew(1) Smean(1,:) Imean(1,:) Rmean(1,:) NSmean(1,:) Smedian Imedian Rmedian S I R NS SmeanN ImeanN1 RmeanN1 Snewendmean Snewendstd Rnewendmean(1) Rnewendstd(1) Rnew_nonendmean Rnew_nonendstd propSimsNonEp(1) propSimsNonEp_theoretical(1)...
            ] = monteCarloSIRN2(numRuns,numNodes, betas, numGenotypes, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,mode, numLoci, casetype);
   size(ImeanN1)
FinalSize(1) = max(RmeanN1,[],2) %mean final size
%2 genotype
numNodes = 400; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=2; numLoci=1; casetype = 'AND';
%[betameanmean(2) betameanmean2(2) Smean(2,:) Imean(2,:) Rmean(2,:) NSmean(2,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
[fighistRecovered(2) fighistSusceptible(2) fighistpeakValue(2) fighistpeakTime(2) fighistBeta(2) fighistGamma(2) fighistBetanew(2) fighistGammanew(2) figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean(2) gammameanmean betameanmean2(2) betameanmeanNew(2) betameanmeanNew2 gammameanmeanNew(2) Smean(2,:) Imean(2,:) Rmean(2,:) NSmean(2,:) Smedian Imedian Rmedian S I R NS SmeanN ImeanN2 RmeanN2 Snewendmean Snewendstd Rnewendmean(2) Rnewendstd(2) Rnew_nonendmean Rnew_nonendstd propSimsNonEp(2) propSimsNonEp_theoretical(2)...
            ] = monteCarloSIRN2(numRuns,numNodes, betas, numGenotypes, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,mode, numLoci, casetype);
   size(ImeanN2)
FinalSize(2) = max(RmeanN2,[],2) %mean final size
%3 genotype
numNodes = 400; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=3; numLoci=1; casetype = 'AND';
%[betameanmean(3) betameanmean2(3) Smean(3,:) Imean(3,:) Rmean(3,:) NSmean(3,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
[fighistRecovered(3) fighistSusceptible(3) fighistpeakValue(3) fighistpeakTime(3) fighistBeta(3) fighistGamma(3) fighistBetanew(3) fighistGammanew(3) figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean(3) gammameanmean betameanmean2(3) betameanmeanNew(3) betameanmeanNew2 gammameanmeanNew(3) Smean(3,:) Imean(3,:) Rmean(3,:) NSmean(3,:) Smedian Imedian Rmedian S I R NS SmeanN ImeanN3 RmeanN3 Snewendmean Snewendstd Rnewendmean(3) Rnewendstd(3) Rnew_nonendmean Rnew_nonendstd propSimsNonEp(3) propSimsNonEp_theoretical(3)...
            ] = monteCarloSIRN2(numRuns,numNodes, betas, numGenotypes, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,mode, numLoci, casetype);

   size(ImeanN3)
FinalSize(3) = max(RmeanN3,[],2) %mean final size
%4 genotype
numNodes = 400; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=4; numLoci=1; casetype = 'AND';
%[betameanmean(4) betameanmean2(4) Smean(4,:) Imean(4,:) Rmean(4,:) NSmean(4,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
[fighistRecovered(4) fighistSusceptible(4) fighistpeakValue(4) fighistpeakTime(4) fighistBeta(4) fighistGamma(4) fighistBetanew(4) fighistGammanew(4) figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean(4) gammameanmean betameanmean2(4) betameanmeanNew(4) betameanmeanNew2 gammameanmeanNew(4) Smean(4,:) Imean(4,:) Rmean(4,:) NSmean(4,:) Smedian Imedian Rmedian S I R NS SmeanN ImeanN4 RmeanN4 Snewendmean Snewendstd Rnewendmean(4) Rnewendstd(4) Rnew_nonendmean Rnew_nonendstd propSimsNonEp(4) propSimsNonEp_theoretical(4)...
            ] = monteCarloSIRN2(numRuns,numNodes, betas, numGenotypes, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,mode, numLoci, casetype);

   size(ImeanN4)
FinalSize(4) = max(RmeanN4,[],2) %mean final size
    %5 genotype
numNodes = 400; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=5; numLoci=1; casetype = 'AND';
%[betameanmean(5) betameanmean2(5) Smean(5,:) Imean(5,:) Rmean(5,:) NSmean(5,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
[fighistRecovered(5) fighistSusceptible(5) fighistpeakValue(5) fighistpeakTime(5) fighistBeta(5) fighistGamma(5) fighistBetanew(5) fighistGammanew(5) figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean(5) gammameanmean betameanmean2(5) betameanmeanNew(5) betameanmeanNew2 gammameanmeanNew(5) Smean(5,:) Imean(5,:) Rmean(5,:) NSmean(5,:) Smedian Imedian Rmedian S I R NS SmeanN ImeanN5 RmeanN5 Snewendmean Snewendstd Rnewendmean(5) Rnewendstd(5) Rnew_nonendmean Rnew_nonendstd propSimsNonEp(5) propSimsNonEp_theoretical(5)...
            ] = monteCarloSIRN2(numRuns,numNodes, betas, numGenotypes, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,mode, numLoci, casetype);

   size(ImeanN5)
FinalSize(5) = max(RmeanN5,[],2) %mean final size
  %6 genotype
numNodes = 400; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=6; numLoci=1; casetype = 'AND';
%[betameanmean(6) betameanmean2(6) Smean(6,:) Imean(6,:) Rmean(6,:) NSmean(6,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
[fighistRecovered(6) fighistSusceptible(6) fighistpeakValue(6) fighistpeakTime(6) fighistBeta(6) fighistGamma(6) fighistBetanew(6) fighistGammanew(6) figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean(6) gammameanmean betameanmean2(6) betameanmeanNew(6) betameanmeanNew2 gammameanmeanNew(6) Smean(6,:) Imean(6,:) Rmean(6,:) NSmean(6,:) Smedian Imedian Rmedian S I R NS SmeanN ImeanN6 RmeanN6 Snewendmean Snewendstd Rnewendmean(6) Rnewendstd(6) Rnew_nonendmean Rnew_nonendstd propSimsNonEp(6) propSimsNonEp_theoretical(6)...
            ] = monteCarloSIRN2(numRuns,numNodes, betas, numGenotypes, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,mode, numLoci, casetype);


   size(ImeanN6)
FinalSize(6) = max(RmeanN6,[],2) %mean final size
        %7 genotype
numNodes = 400; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=7; numLoci=1; casetype = 'AND';
%[betameanmean(7) betameanmean2(7) Smean(7,:) Imean(7,:) Rmean(7,:) NSmean(7,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
[fighistRecovered(7) fighistSusceptible(7) fighistpeakValue(7) fighistpeakTime(7) fighistBeta(7) fighistGamma(7) fighistBetanew(7) fighistGammanew(7) figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2 ...
            betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean(7) gammameanmean betameanmean2(7) betameanmeanNew(7) betameanmeanNew2 gammameanmeanNew(7) Smean(7,:) Imean(7,:) Rmean(7,:) NSmean(7,:) Smedian Imedian Rmedian S I R NS SmeanN ImeanN7 RmeanN7 Snewendmean Snewendstd Rnewendmean(7) Rnewendstd(7) Rnew_nonendmean Rnew_nonendstd propSimsNonEp(7) propSimsNonEp_theoretical(7)...
            ] = monteCarloSIRN2(numRuns,numNodes, betas, numGenotypes, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,mode, numLoci, casetype);

   size(ImeanN7)
FinalSize(7) = max(RmeanN7,[],2) %mean final size
        %8 genotype
numNodes = 400; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=8; numLoci=1; casetype = 'AND';
%[betameanmean(8) betameanmean2(8) Smean(8,:) Imean(8,:) Rmean(8,:) NSmean(8,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
[fighistRecovered(8) fighistSusceptible(8) fighistpeakValue(8) fighistpeakTime(8) fighistBeta(8) fighistGamma(8) fighistBetanew(8) fighistGammanew(8) figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean(8) gammameanmean betameanmean2(8) betameanmeanNew(8) betameanmeanNew2 gammameanmeanNew(8) Smean(8,:) Imean(8,:) Rmean(8,:) NSmean(8,:) Smedian Imedian Rmedian S I R NS SmeanN ImeanN8 RmeanN8 Snewendmean Snewendstd Rnewendmean(8) Rnewendstd(8) Rnew_nonendmean Rnew_nonendstd propSimsNonEp(8) propSimsNonEp_theoretical(8)...
            ] = monteCarloSIRN2(numRuns,numNodes, betas, numGenotypes, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,mode, numLoci, casetype);

   size(ImeanN8)
FinalSize(8) = max(RmeanN8,[],2) %mean final size
    %Histograms
 for i = 1:8
    saveas(fighistRecovered(i),[outputDirectory 'Test9fighistRecovered' num2str(i) '.fig']);
    saveas(fighistRecovered(i),[outputDirectory 'Test9fighistRecovered' num2str(i) '.emf']);
    saveas(fighistSusceptible(i),[outputDirectory 'Test9fighistSusceptible' num2str(i) '.fig']);
    saveas(fighistSusceptible(i),[outputDirectory 'Test9fighistSusceptible' num2str(i) '.emf']);
    saveas(fighistpeakValue(i),[outputDirectory 'Test9fighistpeakValue' num2str(i) '.fig']);
    saveas(fighistpeakValue(i),[outputDirectory 'Test9fighistpeakValue' num2str(i) '.emf']);
    saveas(fighistpeakTime(i),[outputDirectory 'Test9fighistpeakTime' num2str(i) '.fig']);
    saveas(fighistpeakTime(i),[outputDirectory 'Test9fighistpeakTime' num2str(i) '.emf']);
    saveas(fighistBeta(i),[outputDirectory 'Test9histBeta' num2str(i) '.fig']);
    saveas(fighistBeta(i),[outputDirectory 'Test9histBeta' num2str(i) '.emf']);
    saveas(fighistGamma(i),[outputDirectory 'Test9histGamma' num2str(i) '.fig']);
    saveas(fighistGamma(i),[outputDirectory 'Test9histGamma' num2str(i) '.emf']);
    saveas(fighistBetanew(i),[outputDirectory 'Test9histBetaNew' num2str(i) '.fig']);
    saveas(fighistBetanew(i),[outputDirectory 'Test9histBetaNew' num2str(i) '.emf']);
    saveas(fighistGammanew(i),[outputDirectory 'Test9histGammaNew' num2str(i) '.fig']);
    saveas(fighistGammanew(i),[outputDirectory 'Test9histGammaNew' num2str(i) '.emf']);
 end
 close all;
%Deterministic SI model
N=400; Vmax = 8; V=1:Vmax %vary number exploited
Sprop = V./Vmax; S = N.*Sprop; B = 0.02; beta = B.*Sprop; gamma = 0.01;
for i = 1:8
    [Ro(i) St(i,:) It(i,:) Rt(i,:)] = SIRDeterministic(beta(i), gamma, S(i), 1, 0, numTimesteps, 0); 
end
dec = 50; t2 = 1:dec:numTimesteps;

%stochastic final size approximation
betaSolve =[0.0025    0.0050    0.0075    0.0100    0.0125   0.0150    0.0175    0.0200]%calculated from above
    meanRTheory1 =  (solve('1-x = exp(-1.0*(0.0025/0.01)*x)', 'Real', true))*numNodes
    meanRTheory2 =  (solve('1-x = exp(-1.0*(0.0050/0.01)*x)', 'Real', true))*numNodes
    meanRTheory3 =  (solve('1-x = exp(-1.0*(0.0075/0.01)*x)', 'Real', true))*numNodes
    meanRTheory4 =  (solve('1-x = exp(-1.0*(0.01/0.01)*x)', 'Real', true))*numNodes
    meanRTheory5 =  (solve('1-x = exp(-1.0*(0.0125/0.01)*x)', 'Real', true))*numNodes
    meanRTheory6 =  (solve('1-x = exp(-1.0*(0.0150/0.01)*x)', 'Real', true))*numNodes
    meanRTheory7 =  (solve('1-x = exp(-1.0*(0.0175/0.01)*x)', 'Real', true))*numNodes
    meanRTheory8 =  (solve('1-x = exp(-1.0*(0.02/0.01)*x)', 'Real', true))*numNodes
FinalSizeS = [max(double(meanRTheory1)) max(double(meanRTheory2)) max(double(meanRTheory3)) max(double(meanRTheory4)) max(double(meanRTheory5)) max(double(meanRTheory6)) max(double(meanRTheory7)) max(double(meanRTheory8))];
%FinalSizeSMask = FinalSizeS>0; %if less than zero, then zero it as only valid if positive
%FinalSizeS = FinalSizeS.*FinalSizeSMask
betameanmeanD=betameanmean
 figTest9beta0_02 = figure
 plot(betameanmeanNew,'ko-')
 hold on
 plot(beta,'k.-')
 ylabel('beta')
 xlabel('Number of exploits')
 %title('b) SI-Increasing variants on 1 locus - ')
 title('b) SI-Increasing exploits: Infection rate')
 legend('Agent(measured)', 'Deterministic')
 
   figTest9gamma0_02 = figure
 plot(gammameanmeanNew,'ko-')
 hold on
 plot([0.01 0.01 0.01 0.01 0.01 0.01 0.01 0.01],'k.-')
 ylabel('gama')
 xlabel('Number of exploits')
 %title('b) SI-Increasing variants on 1 locus - ')
 title(' SI-Increasing exploits: Recovery rate')
 legend('Agent(measured)', 'Deterministic')
 axis([1 8 0 0.025])
 
 figTest9inf0_02 = figure
 for i = 1:8
     plot(Imean(i,:),'k')
     hold on
     plot(t2,It(i,1:dec:end),'k.-')
 end
 ylabel('Number of nodes infected - It')
 xlabel('Timesteps')
 title('a) SI-Increasing exploits: Nodes infected')
 legend('Agent', 'Deterministic')
 %axis([0 1000 0 400])
 
  figmajor = figure
 for i = 1:8
     plot(ImeanN1,'k')
     hold on
     plot(t2,It(i,1:dec:end),'k.-')
     plot(ImeanN2,'k')
     plot(ImeanN3,'k')
     plot(ImeanN4,'k')
     plot(ImeanN5,'k')
     plot(ImeanN6,'k')
     plot(ImeanN7,'k')
 end
xlabel('Timesteps')
ylabel('Number of individuals infected I(t)')
title('Mean of major outbreaks')
 legend('Agent', 'Deterministic')
%axis([0 100 0 90])
 
 
 %number of nodes infected
 figTest2FinalInfected = figure
 %agentImax = max(Imean,[],2);
 %plot(agentImax,'k')
 for j = 1:8
 if isnan(FinalSizeS(j)); h(j) = 0; else h(j) = FinalSizeS(j); end;
 if isnan(FinalSize(j)); q(j) = 0; else q(j) = FinalSize(j); end;
 end
 FinalSize=q
 FinalSizeS=h
 plot(FinalSize,'ko-')
 hold on
 plot(S,'k.-')
 plot(FinalSizeS,'k+-')
 plot([5 5],[0 400],'k--')
 ylabel('Final number infected')
 xlabel('Number of exploits')
 title('c) SI-Increasing exploits: Final size')
 legend('Agent', 'Deterministic', 'Stochastic Approximation')
 
 
saveas(figTest9beta0_02,[outputDirectory 'Test9beta0_02.fig']);
saveas(figTest9beta0_02,[outputDirectory 'Test9beta0_02.emf']);
saveas(figTest9gamma0_02,[outputDirectory 'Test9gama0_02.fig']);
saveas(figTest9gamma0_02,[outputDirectory 'Test9gama0_02.emf']);
saveas(figTest9inf0_02,[outputDirectory 'Test9meanall.fig']);
saveas(figTest9inf0_02,[outputDirectory 'Test9meanall.emf']); 
saveas(figmajor,[outputDirectory 'Test9meanmajor.fig']);
saveas(figmajor,[outputDirectory 'Test9meanmajor.emf']);
saveas(figTest2FinalInfected,[outputDirectory 'Test9FinalInfected.fig']);
saveas(figTest2FinalInfected,[outputDirectory 'Test9FinalInfected.emf']);
%close all


betameanmean
gammameanmean

betameanmean2
betameanmeanNew 
betameanmeanNew2
gammameanmeanNew

Rnewendmean%epidemics only
Rnewendstd

 propSimsNonEp % proportion of sims with non-epidemics
 propSimsNonEp_theoretical %theoretical value
    
R = betameanmean./gammameanmean
RN = betameanmeanNew./gammameanmeanNew
RTheoretical = beta./gamma

figro = figure
%plot(beta,R,'k')
hold on
plot(RN,'ko-')
plot(RTheoretical,'k.-')
plot([1 1 1 1 1 1 1 1],'k--')
plot([5 5],[0 2],'k--')
xlabel('Number of exploits')
ylabel('ro')
%legend('R','RNew','Rtheoretical') 
legend('Ro Agent(measured)','Ro Deterministic') 
%axis([0.1 0.5 0 2])
saveas(figro,[outputDirectory 'Test9figro.fig']);
saveas(figro,[outputDirectory 'Test9figro.emf']);
%close all
end  %SIRN 9


% SIRN TEST 10
% SIR - 2 loci AND fix exploit at 1 and increase variants - takes 45 mins
%***************
if doSIRNTests(10) == 1
outputDirectory = 'randomGenotypesGraphs\'
%Increase variants upto 8, fix exploit
%***************************************
%SIRN testing
%***************************************
mode = 'static'
%monte carlo random genotype SIR model
numRuns = 200
%1 genotype
numNodes = 400; betas = 0.02; numGenotypes =  1;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1; numLoci=2; casetype = 'AND'; throwAway = 'nonEpidemics', SEpidemic=400-60;
%[betameanmean(1) betameanmean2(1) Smean(1,:) Imean(1,:) Rmean(1,:) NSmean(1,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
[fighistRecovered(1) fighistSusceptible(1) fighistpeakValue(1) fighistpeakTime(1) fighistBeta(1) fighistGamma(1) fighistBetanew(1) fighistGammanew(1) figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean(1) gammameanmean betameanmean2(1) betameanmeanNew(1) betameanmeanNew2 gammameanmeanNew(1) Smean(1,:) Imean(1,:) Rmean(1,:) NSmean(1,:) Smedian Imedian Rmedian S I R NS SmeanN ImeanN1 RmeanN1 Snewendmean Snewendstd Rnewendmean(1) Rnewendstd(1) Rnew_nonendmean Rnew_nonendstd propSimsNonEp(1) propSimsNonEp_theoretical(1)...
            ] = monteCarloSIRN2(numRuns,numNodes, betas, numGenotypes, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,mode, numLoci, casetype);
   size(ImeanN1)
FinalSize(1) = max(RmeanN1,[],2) %mean final size
%2 genotype
numNodes = 400; betas = 0.02; numGenotypes =  2;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1; numLoci=2; casetype = 'AND';
%[betameanmean(2) betameanmean2(2) Smean(2,:) Imean(2,:) Rmean(2,:) NSmean(2,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
[fighistRecovered(2) fighistSusceptible(2) fighistpeakValue(2) fighistpeakTime(2) fighistBeta(2) fighistGamma(2) fighistBetanew(2) fighistGammanew(2) figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean(2) gammameanmean betameanmean2(2) betameanmeanNew(2) betameanmeanNew2 gammameanmeanNew(2) Smean(2,:) Imean(2,:) Rmean(2,:) NSmean(2,:) Smedian Imedian Rmedian S I R NS SmeanN ImeanN2 RmeanN2 Snewendmean Snewendstd Rnewendmean(2) Rnewendstd(2) Rnew_nonendmean Rnew_nonendstd propSimsNonEp(2) propSimsNonEp_theoretical(2)...
            ] = monteCarloSIRN2(numRuns,numNodes, betas, numGenotypes, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,mode, numLoci, casetype);
   size(ImeanN2)
FinalSize(2) = max(RmeanN2,[],2) %mean final size
%3 genotype
numNodes = 400; betas = 0.02; numGenotypes =  3;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1; numLoci=2; casetype = 'AND';
%[betameanmean(3) betameanmean2(3) Smean(3,:) Imean(3,:) Rmean(3,:) NSmean(3,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
[fighistRecovered(3) fighistSusceptible(3) fighistpeakValue(3) fighistpeakTime(3) fighistBeta(3) fighistGamma(3) fighistBetanew(3) fighistGammanew(3) figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean(3) gammameanmean betameanmean2(3) betameanmeanNew(3) betameanmeanNew2 gammameanmeanNew(3) Smean(3,:) Imean(3,:) Rmean(3,:) NSmean(3,:) Smedian Imedian Rmedian S I R NS SmeanN ImeanN3 RmeanN3 Snewendmean Snewendstd Rnewendmean(3) Rnewendstd(3) Rnew_nonendmean Rnew_nonendstd propSimsNonEp(3) propSimsNonEp_theoretical(3)...
            ] = monteCarloSIRN2(numRuns,numNodes, betas, numGenotypes, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,mode, numLoci, casetype);

   size(ImeanN3)
FinalSize(3) = max(RmeanN3,[],2) %mean final size
%4 genotype
numNodes = 400; betas = 0.02; numGenotypes =  4;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1; numLoci=2; casetype = 'AND';
%[betameanmean(4) betameanmean2(4) Smean(4,:) Imean(4,:) Rmean(4,:) NSmean(4,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
[fighistRecovered(4) fighistSusceptible(4) fighistpeakValue(4) fighistpeakTime(4) fighistBeta(4) fighistGamma(4) fighistBetanew(4) fighistGammanew(4) figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean(4) gammameanmean betameanmean2(4) betameanmeanNew(4) betameanmeanNew2 gammameanmeanNew(4) Smean(4,:) Imean(4,:) Rmean(4,:) NSmean(4,:) Smedian Imedian Rmedian S I R NS SmeanN ImeanN4 RmeanN4 Snewendmean Snewendstd Rnewendmean(4) Rnewendstd(4) Rnew_nonendmean Rnew_nonendstd propSimsNonEp(4) propSimsNonEp_theoretical(4)...
            ] = monteCarloSIRN2(numRuns,numNodes, betas, numGenotypes, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,mode, numLoci, casetype);

   size(ImeanN4)
FinalSize(4) = max(RmeanN4,[],2) %mean final size
    %5 genotype
numNodes = 400; betas = 0.02; numGenotypes =  5;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1; numLoci=2; casetype = 'AND';
%[betameanmean(5) betameanmean2(5) Smean(5,:) Imean(5,:) Rmean(5,:) NSmean(5,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
[fighistRecovered(5) fighistSusceptible(5) fighistpeakValue(5) fighistpeakTime(5) fighistBeta(5) fighistGamma(5) fighistBetanew(5) fighistGammanew(5) figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean(5) gammameanmean betameanmean2(5) betameanmeanNew(5) betameanmeanNew2 gammameanmeanNew(5) Smean(5,:) Imean(5,:) Rmean(5,:) NSmean(5,:) Smedian Imedian Rmedian S I R NS SmeanN ImeanN5 RmeanN5 Snewendmean Snewendstd Rnewendmean(5) Rnewendstd(5) Rnew_nonendmean Rnew_nonendstd propSimsNonEp(5) propSimsNonEp_theoretical(5)...
            ] = monteCarloSIRN2(numRuns,numNodes, betas, numGenotypes, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,mode, numLoci, casetype);

   size(ImeanN5)
FinalSize(5) = max(RmeanN5,[],2) %mean final size
  %6 genotype
numNodes = 400; betas = 0.02; numGenotypes =  6;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1; numLoci=1; casetype = 'AND';
%[betameanmean(6) betameanmean2(6) Smean(6,:) Imean(6,:) Rmean(6,:) NSmean(6,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
[fighistRecovered(6) fighistSusceptible(6) fighistpeakValue(6) fighistpeakTime(6) fighistBeta(6) fighistGamma(6) fighistBetanew(6) fighistGammanew(6) figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean(6) gammameanmean betameanmean2(6) betameanmeanNew(6) betameanmeanNew2 gammameanmeanNew(6) Smean(6,:) Imean(6,:) Rmean(6,:) NSmean(6,:) Smedian Imedian Rmedian S I R NS SmeanN ImeanN6 RmeanN6 Snewendmean Snewendstd Rnewendmean(6) Rnewendstd(6) Rnew_nonendmean Rnew_nonendstd propSimsNonEp(6) propSimsNonEp_theoretical(6)...
            ] = monteCarloSIRN2(numRuns,numNodes, betas, numGenotypes, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,mode, numLoci, casetype);


   size(ImeanN6)
FinalSize(6) = max(RmeanN6,[],2) %mean final size
        %7 genotype
numNodes = 400; betas = 0.02; numGenotypes =  7;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1; numLoci=2; casetype = 'AND';
%[betameanmean(7) betameanmean2(7) Smean(7,:) Imean(7,:) Rmean(7,:) NSmean(7,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
[fighistRecovered(7) fighistSusceptible(7) fighistpeakValue(7) fighistpeakTime(7) fighistBeta(7) fighistGamma(7) fighistBetanew(7) fighistGammanew(7) figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2 ...
            betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean(7) gammameanmean betameanmean2(7) betameanmeanNew(7) betameanmeanNew2 gammameanmeanNew(7) Smean(7,:) Imean(7,:) Rmean(7,:) NSmean(7,:) Smedian Imedian Rmedian S I R NS SmeanN ImeanN7 RmeanN7 Snewendmean Snewendstd Rnewendmean(7) Rnewendstd(7) Rnew_nonendmean Rnew_nonendstd propSimsNonEp(7) propSimsNonEp_theoretical(7)...
            ] = monteCarloSIRN2(numRuns,numNodes, betas, numGenotypes, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,mode, numLoci, casetype);

   size(ImeanN7)
FinalSize(7) = max(RmeanN7,[],2) %mean final size
        %8 genotype
numNodes = 400; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1; numLoci=2; casetype = 'AND';
%[betameanmean(8) betameanmean2(8) Smean(8,:) Imean(8,:) Rmean(8,:) NSmean(8,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
[fighistRecovered(8) fighistSusceptible(8) fighistpeakValue(8) fighistpeakTime(8) fighistBeta(8) fighistGamma(8) fighistBetanew(8) fighistGammanew(8) figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean(8) gammameanmean betameanmean2(8) betameanmeanNew(8) betameanmeanNew2 gammameanmeanNew(8) Smean(8,:) Imean(8,:) Rmean(8,:) NSmean(8,:) Smedian Imedian Rmedian S I R NS SmeanN ImeanN8 RmeanN8 Snewendmean Snewendstd Rnewendmean(8) Rnewendstd(8) Rnew_nonendmean Rnew_nonendstd propSimsNonEp(8) propSimsNonEp_theoretical(8)...
            ] = monteCarloSIRN2(numRuns,numNodes, betas, numGenotypes, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,mode, numLoci, casetype);

   size(ImeanN8)
FinalSize(8) = max(RmeanN8,[],2) %mean final size
    %Histograms
 for i = 1:8
    saveas(fighistRecovered(i),[outputDirectory 'Test10fighistRecovered' num2str(i) '.fig']);
    saveas(fighistRecovered(i),[outputDirectory 'Test10fighistRecovered' num2str(i) '.emf']);
    saveas(fighistSusceptible(i),[outputDirectory 'Test10fighistSusceptible' num2str(i) '.fig']);
    saveas(fighistSusceptible(i),[outputDirectory 'Test10fighistSusceptible' num2str(i) '.emf']);
    saveas(fighistpeakValue(i),[outputDirectory 'Test10fighistpeakValue' num2str(i) '.fig']);
    saveas(fighistpeakValue(i),[outputDirectory 'Test10fighistpeakValue' num2str(i) '.emf']);
    saveas(fighistpeakTime(i),[outputDirectory 'Test10fighistpeakTime' num2str(i) '.fig']);
    saveas(fighistpeakTime(i),[outputDirectory 'Test10fighistpeakTime' num2str(i) '.emf']);
    saveas(fighistBeta(i),[outputDirectory 'Test10histBeta' num2str(i) '.fig']);
    saveas(fighistBeta(i),[outputDirectory 'Test10histBeta' num2str(i) '.emf']);
    saveas(fighistGamma(i),[outputDirectory 'Test10histGamma' num2str(i) '.fig']);
    saveas(fighistGamma(i),[outputDirectory 'Test10histGamma' num2str(i) '.emf']);
    saveas(fighistBetanew(i),[outputDirectory 'Test10histBetaNew' num2str(i) '.fig']);
    saveas(fighistBetanew(i),[outputDirectory 'Test10histBetaNew' num2str(i) '.emf']);
    saveas(fighistGammanew(i),[outputDirectory 'Test10histGammaNew' num2str(i) '.fig']);
    saveas(fighistGammanew(i),[outputDirectory 'Test10histGammaNew' num2str(i) '.emf']);
 end
 close all;
%Deterministic SI model
%Deterministic - 2 loci
N=400; Vmax = 1:8; G = Vmax.^2; %number genotypes
%both loci have the same number of variants and both have 1 exploit
Sprop = 1./G; S = N.*Sprop; B = 0.02; beta = B.*Sprop; gamma = 0.01;
for i = 1:8
    [Ro(i) St(i,:) It(i,:) Rt(i,:)] = SIRDeterministic(beta(i), gamma, S(i), 1, 0, numTimesteps, 0); 
end
dec = 50; t2 = 1:dec:numTimesteps;

%stochastic final size approximation
betaSolve =[0.0200    0.0050    0.0022    0.0013    0.0008   0.0006    0.0004    0.0003]%calculated from above

    meanRTheory1 =  (solve('1-x = exp(-1.0*(0.020/0.01)*x)', 'Real', true))*numNodes
    meanRTheory2 =  (solve('1-x = exp(-1.0*(0.0050/0.01)*x)', 'Real', true))*numNodes
    meanRTheory3 =  (solve('1-x = exp(-1.0*(0.0022/0.01)*x)', 'Real', true))*numNodes
    meanRTheory4 =  (solve('1-x = exp(-1.0*(0.0013/0.01)*x)', 'Real', true))*numNodes
    meanRTheory5 =  (solve('1-x = exp(-1.0*(0.0008/0.01)*x)', 'Real', true))*numNodes
    meanRTheory6 =  (solve('1-x = exp(-1.0*(0.0006/0.01)*x)', 'Real', true))*numNodes
    meanRTheory7 =  (solve('1-x = exp(-1.0*(0.0004/0.01)*x)', 'Real', true))*numNodes
    meanRTheory8 =  (solve('1-x = exp(-1.0*(0.0003/0.01)*x)', 'Real', true))*numNodes
FinalSizeS = [max(double(meanRTheory1)) max(double(meanRTheory2)) max(double(meanRTheory3)) max(double(meanRTheory4)) max(double(meanRTheory5)) max(double(meanRTheory6)) max(double(meanRTheory7)) max(double(meanRTheory8))];
%FinalSizeSMask = FinalSizeS>0; %if less than zero, then zero it as only valid if positive
%FinalSizeS = FinalSizeS.*FinalSizeSMask
betameanmeanD=betameanmean
 figTest9beta0_02 = figure
 plot(betameanmeanNew,'ko-')
 hold on
 plot(beta,'k.-')
 ylabel('beta')
 xlabel('Number of variants')
 %title('b) SI-Increasing variants on 1 locus - ')
 title('b) SI-Increasing variants: Infection rate')
 legend('Agent(measured)', 'Deterministic')
 
  figTest9gamma0_02 = figure
 plot(gammameanmeanNew,'ko-')
 hold on
 plot([0.01 0.01 0.01 0.01 0.01 0.01 0.01 0.01],'k.-')
 ylabel('gama')
 xlabel('Number of exploits')
 %title('b) SI-Increasing variants on 1 locus - ')
 title(' SI-Increasing exploits: Recovery rate')
 legend('Agent(measured)', 'Deterministic')
 axis([1 8 0 0.025])
 
 figTest9inf0_02 = figure
 for i = 1:8
     plot(Imean(i,:),'k')
     hold on
     plot(t2,It(i,1:dec:end),'k.-')
 end
 ylabel('Number of nodes infected - It')
 xlabel('Timesteps')
 title('a) SI-Increasing variants: Nodes infected')
 legend('Agent', 'Deterministic')
 %axis([0 1000 0 400])
 
  figmajor = figure
 for i = 1:8
     plot(ImeanN1,'k')
     hold on
     plot(t2,It(i,1:dec:end),'k.-')
     plot(ImeanN2,'k')
     plot(ImeanN3,'k')
     plot(ImeanN4,'k')
     plot(ImeanN5,'k')
     plot(ImeanN6,'k')
     plot(ImeanN7,'k')
 end
xlabel('Timesteps')
ylabel('Number of individuals infected I(t)')
title('Mean of major outbreaks')
 legend('Agent', 'Deterministic')
%axis([0 100 0 90])
 
 
 %number of nodes infected
 figTest2FinalInfected = figure
 %agentImax = max(Imean,[],2);
 %plot(agentImax,'k')
 for j = 1:8
 if isnan(FinalSizeS(j)); h(j) = 0; else h(j) = FinalSizeS(j); end;
 if isnan(FinalSize(j)); q(j) = 0; else q(j) = FinalSize(j); end;
 end
 FinalSize=q
 FinalSizeS=h
 plot(FinalSize,'ko-')
 hold on
 plot(S,'k.-')
 plot(FinalSizeS,'k+-')
 plot([2 2],[0 400],'k--')
 ylabel('Final number infected')
 xlabel('Number of variants')
 title('c) SI-Increasing variants: Final size')
 legend('Agent', 'Deterministic', 'Stochastic Approximation')
 
 
saveas(figTest9beta0_02,[outputDirectory 'Test10beta0_02.fig']);
saveas(figTest9beta0_02,[outputDirectory 'Test10beta0_02.emf']);
saveas(figTest9gamma0_02,[outputDirectory 'Test10gama0_02.fig']);
saveas(figTest9gamma0_02,[outputDirectory 'Test10gama0_02.emf']);
saveas(figTest9inf0_02,[outputDirectory 'Test10meanall.fig']);
saveas(figTest9inf0_02,[outputDirectory 'Test10meanall.emf']); 
saveas(figmajor,[outputDirectory 'Test10meanmajor.fig']);
saveas(figmajor,[outputDirectory 'Test10meanmajor.emf']);
saveas(figTest2FinalInfected,[outputDirectory 'Test10FinalInfected.fig']);
saveas(figTest2FinalInfected,[outputDirectory 'Test10FinalInfected.emf']);
%close all


betameanmean
gammameanmean

betameanmean2
betameanmeanNew 
betameanmeanNew2
gammameanmeanNew

Rnewendmean%epidemics only
Rnewendstd

 propSimsNonEp % proportion of sims with non-epidemics
 propSimsNonEp_theoretical %theoretical value
    
R = betameanmean./gammameanmean
RN = betameanmeanNew./gammameanmeanNew
RTheoretical = beta./gamma

figro = figure
%plot(beta,R,'k')
hold on
plot(RN,'ko-')
plot(RTheoretical,'k.-')
plot([1 1 1 1 1 1 1 1],'k--')
plot([2 2],[0 2],'k--')
xlabel('Number of variants')
ylabel('ro')
%legend('R','RNew','Rtheoretical') 
legend('Ro Agent(measured)','Ro Deterministic') 
%axis([0.1 0.5 0 2])
saveas(figro,[outputDirectory 'Test10figro.fig']);
saveas(figro,[outputDirectory 'Test10figro.emf']);
%close all
%close all;
end %SIRN Test10

%doSIRNTests(11) = 1 %SIR AND increase number of exploits - two locus
if doSIRNTests(11) == 1
outputDirectory = 'randomGenotypesGraphs\'
%Increase variants upto 8, fix exploit
%***************************************
%SIRN testing
%***************************************
mode = 'static'
%monte carlo random genotype SIR model
numRuns = 200
%1 genotype
numNodes = 400; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1; numLoci=2; casetype = 'AND'; throwAway = 'nonEpidemics', SEpidemic=400-60;
%[betameanmean(1) betameanmean2(1) Smean(1,:) Imean(1,:) Rmean(1,:) NSmean(1,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
[fighistRecovered(1) fighistSusceptible(1) fighistpeakValue(1) fighistpeakTime(1) fighistBeta(1) fighistGamma(1) fighistBetanew(1) fighistGammanew(1) figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean(1) gammameanmean betameanmean2(1) betameanmeanNew(1) betameanmeanNew2 gammameanmeanNew(1) Smean(1,:) Imean(1,:) Rmean(1,:) NSmean(1,:) Smedian Imedian Rmedian S I R NS SmeanN ImeanN1 RmeanN1 Snewendmean Snewendstd Rnewendmean(1) Rnewendstd(1) Rnew_nonendmean Rnew_nonendstd propSimsNonEp(1) propSimsNonEp_theoretical(1)...
            ] = monteCarloSIRN2(numRuns,numNodes, betas, numGenotypes, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,mode, numLoci, casetype);
   size(ImeanN1)
FinalSize(1) = max(RmeanN1,[],2) %mean final size

%2 genotype
numNodes = 400; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=2; numLoci=2; casetype = 'AND';
%[betameanmean(2) betameanmean2(2) Smean(2,:) Imean(2,:) Rmean(2,:) NSmean(2,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
[fighistRecovered(2) fighistSusceptible(2) fighistpeakValue(2) fighistpeakTime(2) fighistBeta(2) fighistGamma(2) fighistBetanew(2) fighistGammanew(2) figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean(2) gammameanmean betameanmean2(2) betameanmeanNew(2) betameanmeanNew2 gammameanmeanNew(2) Smean(2,:) Imean(2,:) Rmean(2,:) NSmean(2,:) Smedian Imedian Rmedian S I R NS SmeanN ImeanN2 RmeanN2 Snewendmean Snewendstd Rnewendmean(2) Rnewendstd(2) Rnew_nonendmean Rnew_nonendstd propSimsNonEp(2) propSimsNonEp_theoretical(2)...
            ] = monteCarloSIRN2(numRuns,numNodes, betas, numGenotypes, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,mode, numLoci, casetype);
   size(ImeanN2)
FinalSize(2) = max(RmeanN2,[],2) %mean final size
%3 genotype
numNodes = 400; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=3; numLoci=2; casetype = 'AND';
%[betameanmean(3) betameanmean2(3) Smean(3,:) Imean(3,:) Rmean(3,:) NSmean(3,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
[fighistRecovered(3) fighistSusceptible(3) fighistpeakValue(3) fighistpeakTime(3) fighistBeta(3) fighistGamma(3) fighistBetanew(3) fighistGammanew(3) figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean(3) gammameanmean betameanmean2(3) betameanmeanNew(3) betameanmeanNew2 gammameanmeanNew(3) Smean(3,:) Imean(3,:) Rmean(3,:) NSmean(3,:) Smedian Imedian Rmedian S I R NS SmeanN ImeanN3 RmeanN3 Snewendmean Snewendstd Rnewendmean(3) Rnewendstd(3) Rnew_nonendmean Rnew_nonendstd propSimsNonEp(3) propSimsNonEp_theoretical(3)...
            ] = monteCarloSIRN2(numRuns,numNodes, betas, numGenotypes, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,mode, numLoci, casetype);

   size(ImeanN3)
FinalSize(3) = max(RmeanN3,[],2) %mean final size
%4 genotype
numNodes = 400; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=4; numLoci=2; casetype = 'AND';
%[betameanmean(4) betameanmean2(4) Smean(4,:) Imean(4,:) Rmean(4,:) NSmean(4,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
[fighistRecovered(4) fighistSusceptible(4) fighistpeakValue(4) fighistpeakTime(4) fighistBeta(4) fighistGamma(4) fighistBetanew(4) fighistGammanew(4) figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean(4) gammameanmean betameanmean2(4) betameanmeanNew(4) betameanmeanNew2 gammameanmeanNew(4) Smean(4,:) Imean(4,:) Rmean(4,:) NSmean(4,:) Smedian Imedian Rmedian S I R NS SmeanN ImeanN4 RmeanN4 Snewendmean Snewendstd Rnewendmean(4) Rnewendstd(4) Rnew_nonendmean Rnew_nonendstd propSimsNonEp(4) propSimsNonEp_theoretical(4)...
            ] = monteCarloSIRN2(numRuns,numNodes, betas, numGenotypes, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,mode, numLoci, casetype);

   size(ImeanN4)
FinalSize(4) = max(RmeanN4,[],2) %mean final size
    %5 genotype
numNodes = 400; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=5; numLoci=2; casetype = 'AND';
%[betameanmean(5) betameanmean2(5) Smean(5,:) Imean(5,:) Rmean(5,:) NSmean(5,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
[fighistRecovered(5) fighistSusceptible(5) fighistpeakValue(5) fighistpeakTime(5) fighistBeta(5) fighistGamma(5) fighistBetanew(5) fighistGammanew(5) figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean(5) gammameanmean betameanmean2(5) betameanmeanNew(5) betameanmeanNew2 gammameanmeanNew(5) Smean(5,:) Imean(5,:) Rmean(5,:) NSmean(5,:) Smedian Imedian Rmedian S I R NS SmeanN ImeanN5 RmeanN5 Snewendmean Snewendstd Rnewendmean(5) Rnewendstd(5) Rnew_nonendmean Rnew_nonendstd propSimsNonEp(5) propSimsNonEp_theoretical(5)...
            ] = monteCarloSIRN2(numRuns,numNodes, betas, numGenotypes, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,mode, numLoci, casetype);

   size(ImeanN5)
FinalSize(5) = max(RmeanN5,[],2) %mean final size
  %6 genotype
numNodes = 400; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=6; numLoci=2; casetype = 'AND';
%[betameanmean(6) betameanmean2(6) Smean(6,:) Imean(6,:) Rmean(6,:) NSmean(6,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
[fighistRecovered(6) fighistSusceptible(6) fighistpeakValue(6) fighistpeakTime(6) fighistBeta(6) fighistGamma(6) fighistBetanew(6) fighistGammanew(6) figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean(6) gammameanmean betameanmean2(6) betameanmeanNew(6) betameanmeanNew2 gammameanmeanNew(6) Smean(6,:) Imean(6,:) Rmean(6,:) NSmean(6,:) Smedian Imedian Rmedian S I R NS SmeanN ImeanN6 RmeanN6 Snewendmean Snewendstd Rnewendmean(6) Rnewendstd(6) Rnew_nonendmean Rnew_nonendstd propSimsNonEp(6) propSimsNonEp_theoretical(6)...
            ] = monteCarloSIRN2(numRuns,numNodes, betas, numGenotypes, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,mode, numLoci, casetype);


   size(ImeanN6)
FinalSize(6) = max(RmeanN6,[],2) %mean final size
        %7 genotype
numNodes = 400; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=7; numLoci=2; casetype = 'AND';
%[betameanmean(7) betameanmean2(7) Smean(7,:) Imean(7,:) Rmean(7,:) NSmean(7,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
[fighistRecovered(7) fighistSusceptible(7) fighistpeakValue(7) fighistpeakTime(7) fighistBeta(7) fighistGamma(7) fighistBetanew(7) fighistGammanew(7) figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2 ...
            betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean(7) gammameanmean betameanmean2(7) betameanmeanNew(7) betameanmeanNew2 gammameanmeanNew(7) Smean(7,:) Imean(7,:) Rmean(7,:) NSmean(7,:) Smedian Imedian Rmedian S I R NS SmeanN ImeanN7 RmeanN7 Snewendmean Snewendstd Rnewendmean(7) Rnewendstd(7) Rnew_nonendmean Rnew_nonendstd propSimsNonEp(7) propSimsNonEp_theoretical(7)...
            ] = monteCarloSIRN2(numRuns,numNodes, betas, numGenotypes, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,mode, numLoci, casetype);

   size(ImeanN7)
FinalSize(7) = max(RmeanN7,[],2) %mean final size
        %8 genotype
numNodes = 400; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=8; numLoci=2; casetype = 'AND';
%[betameanmean(8) betameanmean2(8) Smean(8,:) Imean(8,:) Rmean(8,:) NSmean(8,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
[fighistRecovered(8) fighistSusceptible(8) fighistpeakValue(8) fighistpeakTime(8) fighistBeta(8) fighistGamma(8) fighistBetanew(8) fighistGammanew(8) figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean(8) gammameanmean betameanmean2(8) betameanmeanNew(8) betameanmeanNew2 gammameanmeanNew(8) Smean(8,:) Imean(8,:) Rmean(8,:) NSmean(8,:) Smedian Imedian Rmedian S I R NS SmeanN ImeanN8 RmeanN8 Snewendmean Snewendstd Rnewendmean(8) Rnewendstd(8) Rnew_nonendmean Rnew_nonendstd propSimsNonEp(8) propSimsNonEp_theoretical(8)...
            ] = monteCarloSIRN2(numRuns,numNodes, betas, numGenotypes, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,mode, numLoci, casetype);

   size(ImeanN8)
FinalSize(8) = max(RmeanN8,[],2) %mean final size
    %Histograms
 for i = 1:8
    saveas(fighistRecovered(i),[outputDirectory 'Test11fighistRecovered' num2str(i) '.fig']);
    saveas(fighistRecovered(i),[outputDirectory 'Test11fighistRecovered' num2str(i) '.emf']);
    saveas(fighistSusceptible(i),[outputDirectory 'Test11fighistSusceptible' num2str(i) '.fig']);
    saveas(fighistSusceptible(i),[outputDirectory 'Test11fighistSusceptible' num2str(i) '.emf']);
    saveas(fighistpeakValue(i),[outputDirectory 'Test11fighistpeakValue' num2str(i) '.fig']);
    saveas(fighistpeakValue(i),[outputDirectory 'Test11fighistpeakValue' num2str(i) '.emf']);
    saveas(fighistpeakTime(i),[outputDirectory 'Test11fighistpeakTime' num2str(i) '.fig']);
    saveas(fighistpeakTime(i),[outputDirectory 'Test11fighistpeakTime' num2str(i) '.emf']);
    saveas(fighistBeta(i),[outputDirectory 'Test11histBeta' num2str(i) '.fig']);
    saveas(fighistBeta(i),[outputDirectory 'Test11histBeta' num2str(i) '.emf']);
    saveas(fighistGamma(i),[outputDirectory 'Test11histGamma' num2str(i) '.fig']);
    saveas(fighistGamma(i),[outputDirectory 'Test11histGamma' num2str(i) '.emf']);
    saveas(fighistBetanew(i),[outputDirectory 'Test11histBetaNew' num2str(i) '.fig']);
    saveas(fighistBetanew(i),[outputDirectory 'Test11histBetaNew' num2str(i) '.emf']);
    saveas(fighistGammanew(i),[outputDirectory 'Test11histGammaNew' num2str(i) '.fig']);
    saveas(fighistGammanew(i),[outputDirectory 'Test11histGammaNew' num2str(i) '.emf']);
 end
 close all;
%Deterministic - 2 loci
susProp2lociAND =[0.0156,0.0625,0.1406,0.2500,0.3906,0.5625,0.7656,1.0000]
N=400; Vmax = 8
S = N.*susProp2lociAND; B = 0.02; beta = B.*susProp2lociAND; gamma = 0.01; %1/48*60*60 %2 days
for i = 1:8
    [Ro(i) St(i,:) It(i,:) Rt(i,:)] = SIRDeterministic(beta(i), gamma, S(i), 1, 0, numTimesteps, 0); 
end
dec = 50; t2 = 1:dec:numTimesteps;

%stochastic final size approximation
betaSolve =[0.0003    0.0013    0.0028    0.0050    0.0078   0.0113    0.0153    0.0200]%calculated from above

    meanRTheory1 =  (solve('1-x = exp(-1.0*(0.0003/0.01)*x)', 'Real', true))*numNodes
    meanRTheory2 =  (solve('1-x = exp(-1.0*(0.0013/0.01)*x)', 'Real', true))*numNodes
    meanRTheory3 =  (solve('1-x = exp(-1.0*(0.0028/0.01)*x)', 'Real', true))*numNodes
    meanRTheory4 =  (solve('1-x = exp(-1.0*(0.0050/0.01)*x)', 'Real', true))*numNodes
    meanRTheory5 =  (solve('1-x = exp(-1.0*(0.0078/0.01)*x)', 'Real', true))*numNodes
    meanRTheory6 =  (solve('1-x = exp(-1.0*(0.0113/0.01)*x)', 'Real', true))*numNodes
    meanRTheory7 =  (solve('1-x = exp(-1.0*(0.0153/0.01)*x)', 'Real', true))*numNodes
    meanRTheory8 =  (solve('1-x = exp(-1.0*(0.02/0.01)*x)', 'Real', true))*numNodes
FinalSizeS = [max(double(meanRTheory1)) max(double(meanRTheory2)) max(double(meanRTheory3)) max(double(meanRTheory4)) max(double(meanRTheory5)) max(double(meanRTheory6)) max(double(meanRTheory7)) max(double(meanRTheory8))];
%FinalSizeSMask = FinalSizeS>0; %if less than zero, then zero it as only valid if positive
%FinalSizeS = FinalSizeS.*FinalSizeSMask
betameanmeanD=betameanmean
 figTest9beta0_02 = figure
 plot(betameanmeanNew,'ko-')
 hold on
 plot(beta,'k.-')
 ylabel('beta')
 xlabel('Number of exploits')
 %title('b) SI-Increasing variants on 1 locus - ')
 title('b) SI-Increasing exploits: Infection rate')
 legend('Agent(measured)', 'Deterministic')

 
 figTest9gamma0_02 = figure
 plot(gammameanmeanNew,'ko-')
 hold on
 plot([0.01 0.01 0.01 0.01 0.01 0.01 0.01 0.01],'k.-')
 ylabel('gama')
 xlabel('Number of exploits')
 %title('b) SI-Increasing variants on 1 locus - ')
 title(' SI-Increasing exploits: Recovery rate')
 legend('Agent(measured)', 'Deterministic')
axis([1 8 0 0.025])
  
 figTest9inf0_02 = figure
 for i = 1:8
     plot(Imean(i,:),'k')
     hold on
     plot(t2,It(i,1:dec:end),'k.-')
 end
 ylabel('Number of nodes infected - It')
 xlabel('Timesteps')
 title('a) SI-Increasing exploits: Nodes infected')
 legend('Agent', 'Deterministic')
 %axis([0 1000 0 400])
 
  figmajor = figure
 for i = 1:8
     plot(ImeanN1,'k')
     hold on
     plot(t2,It(i,1:dec:end),'k.-')
     plot(ImeanN2,'k')
     plot(ImeanN3,'k')
     plot(ImeanN4,'k')
     plot(ImeanN5,'k')
     plot(ImeanN6,'k')
     plot(ImeanN7,'k')
 end
xlabel('Timesteps')
ylabel('Number of individuals infected I(t)')
title('Mean of major outbreaks')
 legend('Agent', 'Deterministic')
%axis([0 100 0 90])
 
 
 %number of nodes infected
 figTest2FinalInfected = figure
 %agentImax = max(Imean,[],2);
 %plot(agentImax,'k')
 for j = 1:8
 if isnan(FinalSizeS(j)); h(j) = 0; else h(j) = FinalSizeS(j); end;
 if isnan(FinalSize(j)); q(j) = 0; else q(j) = FinalSize(j); end;
 end
 FinalSize=q
 FinalSizeS=h
 plot(FinalSize,'ko-')
 hold on
 plot(S,'k.-')
 plot(FinalSizeS,'k+-')
 plot([6 6],[0 400],'k--')
 ylabel('Final number infected')
 xlabel('Number of exploits')
 title('c) SI-Increasing exploits: Final size')
 legend('Agent', 'Deterministic', 'Stochastic Approximation')
 
 
saveas(figTest9beta0_02,[outputDirectory 'Test11beta0_02.fig']);
saveas(figTest9beta0_02,[outputDirectory 'Test11beta0_02.emf']);
saveas(figTest9gamma0_02,[outputDirectory 'Test11gama0_02.fig']);
saveas(figTest9gamma0_02,[outputDirectory 'Test11gama0_02.emf']);
saveas(figTest9inf0_02,[outputDirectory 'Test11meanall.fig']);
saveas(figTest9inf0_02,[outputDirectory 'Test11meanall.emf']); 
saveas(figmajor,[outputDirectory 'Test11meanmajor.fig']);
saveas(figmajor,[outputDirectory 'Test11meanmajor.emf']);
saveas(figTest2FinalInfected,[outputDirectory 'Test11FinalInfected.fig']);
saveas(figTest2FinalInfected,[outputDirectory 'Test11FinalInfected.emf']);
%close all


betameanmean
gammameanmean

betameanmean2
betameanmeanNew 
betameanmeanNew2
gammameanmeanNew

Rnewendmean%epidemics only
Rnewendstd

 propSimsNonEp % proportion of sims with non-epidemics
 propSimsNonEp_theoretical %theoretical value
    
R = betameanmean./gammameanmean
RN = betameanmeanNew./gammameanmeanNew
RTheoretical = beta./gamma

figro = figure
%plot(beta,R,'k')
hold on
plot(RN,'ko-')
plot(RTheoretical,'k.-')
plot([1 1 1 1 1 1 1 1],'k--')
plot([6 6],[0 2],'k--')
xlabel('Number of exploits')
ylabel('ro')
%legend('R','RNew','Rtheoretical') 
legend('Ro Agent(measured)','Ro Deterministic') 
%axis([0.1 0.5 0 2])
saveas(figro,[outputDirectory 'Test11figro.fig']);
saveas(figro,[outputDirectory 'Test11figro.emf']);
%close all
end  %SIRN 11

% SIRN TEST 12
% SIR - 2 loci OR fix exploit at 1 and increase variants - takes 45 mins
%***************
if doSIRNTests(12) == 1
outputDirectory = 'randomGenotypesGraphs\'
%Increase variants upto 8, fix exploit
%***************************************
%SIRN testing
%***************************************
mode = 'static'
%monte carlo random genotype SIR model
numRuns = 200
%1 genotype
numNodes = 400; betas = 0.02; numGenotypes =  1;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1; numLoci=2; casetype = 'OR'; throwAway = 'nonEpidemics', SEpidemic=400-60;
%[betameanmean(1) betameanmean2(1) Smean(1,:) Imean(1,:) Rmean(1,:) NSmean(1,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
[fighistRecovered(1) fighistSusceptible(1) fighistpeakValue(1) fighistpeakTime(1) fighistBeta(1) fighistGamma(1) fighistBetanew(1) fighistGammanew(1) figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean(1) gammameanmean betameanmean2(1) betameanmeanNew(1) betameanmeanNew2 gammameanmeanNew(1) Smean(1,:) Imean(1,:) Rmean(1,:) NSmean(1,:) Smedian Imedian Rmedian S I R NS SmeanN ImeanN1 RmeanN1 Snewendmean Snewendstd Rnewendmean(1) Rnewendstd(1) Rnew_nonendmean Rnew_nonendstd propSimsNonEp(1) propSimsNonEp_theoretical(1)...
            ] = monteCarloSIRN2(numRuns,numNodes, betas, numGenotypes, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,mode, numLoci, casetype);
   size(ImeanN1)
FinalSize(1) = max(RmeanN1,[],2) %mean final size
    %resilience
     maska = RmeanN1 >= FinalSize(1)-1; %when within 1 of the final value
    [C Index] = max(maska);
    MLa(1) = Index; %T1
%2 genotype
numNodes = 400; betas = 0.02; numGenotypes =  2;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1; numLoci=2; casetype = 'OR'; throwAway = 'nonEpidemics', SEpidemic=400-60;
%[betameanmean(2) betameanmean2(2) Smean(2,:) Imean(2,:) Rmean(2,:) NSmean(2,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
[fighistRecovered(2) fighistSusceptible(2) fighistpeakValue(2) fighistpeakTime(2) fighistBeta(2) fighistGamma(2) fighistBetanew(2) fighistGammanew(2) figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean(2) gammameanmean betameanmean2(2) betameanmeanNew(2) betameanmeanNew2 gammameanmeanNew(2) Smean(2,:) Imean(2,:) Rmean(2,:) NSmean(2,:) Smedian Imedian Rmedian S I R NS SmeanN ImeanN2 RmeanN2 Snewendmean Snewendstd Rnewendmean(2) Rnewendstd(2) Rnew_nonendmean Rnew_nonendstd propSimsNonEp(2) propSimsNonEp_theoretical(2)...
            ] = monteCarloSIRN2(numRuns,numNodes, betas, numGenotypes, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,mode, numLoci, casetype);
   size(ImeanN2)
FinalSize(2) = max(RmeanN2,[],2) %mean final size
    %resilience
     maska = RmeanN2 >= FinalSize(2)-1; %when within 1 of the final value
    [C Index] = max(maska);
    MLa(2) = Index; %T1
%3 genotype
numNodes = 400; betas = 0.02; numGenotypes =  3;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1; numLoci=2; casetype = 'OR'; throwAway = 'nonEpidemics', SEpidemic=400-60;
%[betameanmean(3) betameanmean2(3) Smean(3,:) Imean(3,:) Rmean(3,:) NSmean(3,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
[fighistRecovered(3) fighistSusceptible(3) fighistpeakValue(3) fighistpeakTime(3) fighistBeta(3) fighistGamma(3) fighistBetanew(3) fighistGammanew(3) figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean(3) gammameanmean betameanmean2(3) betameanmeanNew(3) betameanmeanNew2 gammameanmeanNew(3) Smean(3,:) Imean(3,:) Rmean(3,:) NSmean(3,:) Smedian Imedian Rmedian S I R NS SmeanN ImeanN3 RmeanN3 Snewendmean Snewendstd Rnewendmean(3) Rnewendstd(3) Rnew_nonendmean Rnew_nonendstd propSimsNonEp(3) propSimsNonEp_theoretical(3)...
            ] = monteCarloSIRN2(numRuns,numNodes, betas, numGenotypes, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,mode, numLoci, casetype);

   size(ImeanN3)
FinalSize(3) = max(RmeanN3,[],2) %mean final size
    %resilience
     maska = RmeanN3 >= FinalSize(3)-1; %when within 1 of the final value
    [C Index] = max(maska);
    MLa(3) = Index; %T1
%4 genotype
numNodes = 400; betas = 0.02; numGenotypes =  4;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1; numLoci=2; casetype = 'OR';  throwAway = 'nonEpidemics', SEpidemic=400-100;
%[betameanmean(4) betameanmean2(4) Smean(4,:) Imean(4,:) Rmean(4,:) NSmean(4,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
[fighistRecovered(4) fighistSusceptible(4) fighistpeakValue(4) fighistpeakTime(4) fighistBeta(4) fighistGamma(4) fighistBetanew(4) fighistGammanew(4) figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean(4) gammameanmean betameanmean2(4) betameanmeanNew(4) betameanmeanNew2 gammameanmeanNew(4) Smean(4,:) Imean(4,:) Rmean(4,:) NSmean(4,:) Smedian Imedian Rmedian S I R NS SmeanN ImeanN4 RmeanN4 Snewendmean Snewendstd Rnewendmean(4) Rnewendstd(4) Rnew_nonendmean Rnew_nonendstd propSimsNonEp(4) propSimsNonEp_theoretical(4)...
            ] = monteCarloSIRN2(numRuns,numNodes, betas, numGenotypes, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,mode, numLoci, casetype);

   size(ImeanN4)
FinalSize(4) = max(RmeanN4,[],2) %mean final size
    %resilience
     maska = RmeanN4 >= FinalSize(4)-1; %when within 1 of the final value
    [C Index] = max(maska);
    MLa(4) = Index; %T1
    %5 genotype
numNodes = 400; betas = 0.02; numGenotypes =  5;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1; numLoci=2; casetype = 'OR';
%[betameanmean(5) betameanmean2(5) Smean(5,:) Imean(5,:) Rmean(5,:) NSmean(5,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
[fighistRecovered(5) fighistSusceptible(5) fighistpeakValue(5) fighistpeakTime(5) fighistBeta(5) fighistGamma(5) fighistBetanew(5) fighistGammanew(5) figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean(5) gammameanmean betameanmean2(5) betameanmeanNew(5) betameanmeanNew2 gammameanmeanNew(5) Smean(5,:) Imean(5,:) Rmean(5,:) NSmean(5,:) Smedian Imedian Rmedian S I R NS SmeanN ImeanN5 RmeanN5 Snewendmean Snewendstd Rnewendmean(5) Rnewendstd(5) Rnew_nonendmean Rnew_nonendstd propSimsNonEp(5) propSimsNonEp_theoretical(5)...
            ] = monteCarloSIRN2(numRuns,numNodes, betas, numGenotypes, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,mode, numLoci, casetype);

   size(ImeanN5)
FinalSize(5) = max(RmeanN5,[],2) %mean final size
    %resilience
     maska = RmeanN5 >= FinalSize(5)-1; %when within 1 of the final value
    [C Index] = max(maska);
    MLa(5) = Index; %T1
  %6 genotype
numNodes = 400; betas = 0.02; numGenotypes =  6;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1; numLoci=1; casetype = 'OR';
%[betameanmean(6) betameanmean2(6) Smean(6,:) Imean(6,:) Rmean(6,:) NSmean(6,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
[fighistRecovered(6) fighistSusceptible(6) fighistpeakValue(6) fighistpeakTime(6) fighistBeta(6) fighistGamma(6) fighistBetanew(6) fighistGammanew(6) figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean(6) gammameanmean betameanmean2(6) betameanmeanNew(6) betameanmeanNew2 gammameanmeanNew(6) Smean(6,:) Imean(6,:) Rmean(6,:) NSmean(6,:) Smedian Imedian Rmedian S I R NS SmeanN ImeanN6 RmeanN6 Snewendmean Snewendstd Rnewendmean(6) Rnewendstd(6) Rnew_nonendmean Rnew_nonendstd propSimsNonEp(6) propSimsNonEp_theoretical(6)...
            ] = monteCarloSIRN2(numRuns,numNodes, betas, numGenotypes, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,mode, numLoci, casetype);


   size(ImeanN6)
FinalSize(6) = max(RmeanN6,[],2) %mean final size
    %resilience
     maska = RmeanN6 >= FinalSize(6)-1; %when within 1 of the final value
    [C Index] = max(maska);
    MLa(6) = Index; %T1
        %7 genotype
numNodes = 400; betas = 0.02; numGenotypes =  7;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1; numLoci=2; casetype = 'OR';
%[betameanmean(7) betameanmean2(7) Smean(7,:) Imean(7,:) Rmean(7,:) NSmean(7,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
[fighistRecovered(7) fighistSusceptible(7) fighistpeakValue(7) fighistpeakTime(7) fighistBeta(7) fighistGamma(7) fighistBetanew(7) fighistGammanew(7) figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2 ...
            betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean(7) gammameanmean betameanmean2(7) betameanmeanNew(7) betameanmeanNew2 gammameanmeanNew(7) Smean(7,:) Imean(7,:) Rmean(7,:) NSmean(7,:) Smedian Imedian Rmedian S I R NS SmeanN ImeanN7 RmeanN7 Snewendmean Snewendstd Rnewendmean(7) Rnewendstd(7) Rnew_nonendmean Rnew_nonendstd propSimsNonEp(7) propSimsNonEp_theoretical(7)...
            ] = monteCarloSIRN2(numRuns,numNodes, betas, numGenotypes, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,mode, numLoci, casetype);

   size(ImeanN7)
FinalSize(7) = max(RmeanN7,[],2) %mean final size
    %resilience
     maska = RmeanN7 >= FinalSize(7)-1; %when within 1 of the final value
    [C Index] = max(maska);
    MLa(7) = Index; %T1
        %8 genotype
numNodes = 400; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1; numLoci=2; casetype = 'AND';
%[betameanmean(8) betameanmean2(8) Smean(8,:) Imean(8,:) Rmean(8,:) NSmean(8,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
[fighistRecovered(8) fighistSusceptible(8) fighistpeakValue(8) fighistpeakTime(8) fighistBeta(8) fighistGamma(8) fighistBetanew(8) fighistGammanew(8) figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean(8) gammameanmean betameanmean2(8) betameanmeanNew(8) betameanmeanNew2 gammameanmeanNew(8) Smean(8,:) Imean(8,:) Rmean(8,:) NSmean(8,:) Smedian Imedian Rmedian S I R NS SmeanN ImeanN8 RmeanN8 Snewendmean Snewendstd Rnewendmean(8) Rnewendstd(8) Rnew_nonendmean Rnew_nonendstd propSimsNonEp(8) propSimsNonEp_theoretical(8)...
            ] = monteCarloSIRN2(numRuns,numNodes, betas, numGenotypes, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,mode, numLoci, casetype);

   size(ImeanN8)
FinalSize(8) = max(RmeanN8,[],2) %mean final size
    %resilience
     maska = RmeanN8 >= FinalSize(8)-1; %when within 1 of the final value
    [C Index] = max(maska);
    MLa(8) = Index; %T1
    %Histograms
 for i = 1:8
    saveas(fighistRecovered(i),[outputDirectory 'Test12fighistRecovered' num2str(i) '.fig']);
    saveas(fighistRecovered(i),[outputDirectory 'Test12fighistRecovered' num2str(i) '.emf']);
    saveas(fighistSusceptible(i),[outputDirectory 'Test12fighistSusceptible' num2str(i) '.fig']);
    saveas(fighistSusceptible(i),[outputDirectory 'Test12fighistSusceptible' num2str(i) '.emf']);
    saveas(fighistpeakValue(i),[outputDirectory 'Test12fighistpeakValue' num2str(i) '.fig']);
    saveas(fighistpeakValue(i),[outputDirectory 'Test12fighistpeakValue' num2str(i) '.emf']);
    saveas(fighistpeakTime(i),[outputDirectory 'Test12fighistpeakTime' num2str(i) '.fig']);
    saveas(fighistpeakTime(i),[outputDirectory 'Test12fighistpeakTime' num2str(i) '.emf']);
    saveas(fighistBeta(i),[outputDirectory 'Test12histBeta' num2str(i) '.fig']);
    saveas(fighistBeta(i),[outputDirectory 'Test12histBeta' num2str(i) '.emf']);
    saveas(fighistGamma(i),[outputDirectory 'Test12histGamma' num2str(i) '.fig']);
    saveas(fighistGamma(i),[outputDirectory 'Test12histGamma' num2str(i) '.emf']);
    saveas(fighistBetanew(i),[outputDirectory 'Test12histBetaNew' num2str(i) '.fig']);
    saveas(fighistBetanew(i),[outputDirectory 'Test12histBetaNew' num2str(i) '.emf']);
    saveas(fighistGammanew(i),[outputDirectory 'Test12histGammaNew' num2str(i) '.fig']);
    saveas(fighistGammanew(i),[outputDirectory 'Test12histGammaNew' num2str(i) '.emf']);
 end
 close all;
%Deterministic - 2 loci
N=400; Vmax = 1:8;  G = Vmax.^2; %number genotypes
%for OR case
Sprop = 2./Vmax - 1./G %both loci have the same number of variants and both have 1 exploit
S = N.*Sprop; B = 0.02; beta = B.*Sprop; gamma = 0.01; %1/48*60*60 %2 days
for i = 1:8
    [Ro(i) St(i,:) It(i,:) Rt(i,:)] = SIRDeterministic(beta(i), gamma, S(i), 1, 0, numTimesteps, 0); 
    FinalSized(i) = Rt(i,end);
     
    %resilience
     maskd = Rt(i,:) >= Rt(i,end)-1; %when within 1 of the final value
    [C Index] = max(maskd);
    MLd(i) = Index; %T1
end

dec = 50; t2 = 1:dec:numTimesteps;

%stochastic final size approximation
betaSolve =[0.0200    0.0150    0.0111    0.0088    0.0072   0.0061    0.0053    0.0047]%calculated from above

    meanRTheory1 =  (solve('1-x = exp(-1.0*(0.020/0.01)*x)', 'Real', true))*numNodes
    meanRTheory2 =  (solve('1-x = exp(-1.0*(0.0150/0.01)*x)', 'Real', true))*numNodes
    meanRTheory3 =  (solve('1-x = exp(-1.0*(0.0111/0.01)*x)', 'Real', true))*numNodes
    meanRTheory4 =  (solve('1-x = exp(-1.0*(0.0088/0.01)*x)', 'Real', true))*numNodes
    meanRTheory5 =  (solve('1-x = exp(-1.0*(0.0072/0.01)*x)', 'Real', true))*numNodes
    meanRTheory6 =  (solve('1-x = exp(-1.0*(0.0061/0.01)*x)', 'Real', true))*numNodes
    meanRTheory7 =  (solve('1-x = exp(-1.0*(0.0053/0.01)*x)', 'Real', true))*numNodes
    meanRTheory8 =  (solve('1-x = exp(-1.0*(0.0047/0.01)*x)', 'Real', true))*numNodes
FinalSizeS = [max(double(meanRTheory1)) max(double(meanRTheory2)) max(double(meanRTheory3)) max(double(meanRTheory4)) max(double(meanRTheory5)) max(double(meanRTheory6)) max(double(meanRTheory7)) max(double(meanRTheory8))];
%FinalSizeSMask = FinalSizeS>0; %if less than zero, then zero it as only valid if positive
%FinalSizeS = FinalSizeS.*FinalSizeSMask
betameanmeanD=betameanmean
 figTest9beta0_02 = figure
 plot(betameanmeanNew,'ko-')
 hold on
 plot(beta,'k.-')
 ylabel('beta')
 xlabel('Number of variants')
 %title('b) SI-Increasing variants on 1 locus - ')
 title('b) SI-Increasing variants: Infection rate')
 legend('Agent(measured)', 'Deterministic')
 
  figTest9gamma0_02 = figure
 plot(gammameanmeanNew,'ko-')
 hold on
 plot([0.01 0.01 0.01 0.01 0.01 0.01 0.01 0.01],'k.-')
 ylabel('gama')
 xlabel('Number of variants')
 %title('b) SI-Increasing variants on 1 locus - ')
 title(' SI-Increasing variants: Recovery rate')
 legend('Agent(measured)', 'Deterministic')
 axis([1 8 0 0.025])
 
 figTest9inf0_02 = figure
 for i = 1:8
     plot(Imean(i,:),'k')
     hold on
     plot(t2,It(i,1:dec:end),'k.-')
 end
 ylabel('Number of nodes infected - It')
 xlabel('Timesteps')
 title('a) SI-Increasing variants: Nodes infected')
 legend('Agent', 'Deterministic')
 %axis([0 1000 0 400])
 
  figmajor = figure
 for i = 1:8
     plot(ImeanN1,'k')
     hold on
     plot(t2,It(i,1:dec:end),'k.-')
     plot(ImeanN2,'k')
     plot(ImeanN3,'k')
     plot(ImeanN4,'k')
     plot(ImeanN5,'k')
     plot(ImeanN6,'k')
     plot(ImeanN7,'k')
 end
xlabel('Timesteps')
ylabel('Number of individuals infected I(t)')
title('Mean of major outbreaks')
 legend('Agent', 'Deterministic')
%axis([0 100 0 90])
 
 
 %number of nodes infected
 figTest2FinalInfected = figure
 %agentImax = max(Imean,[],2);
 %plot(agentImax,'k')
 for j = 1:8
 if isnan(FinalSizeS(j)); h(j) = 0; else h(j) = FinalSizeS(j); end;
 if isnan(FinalSize(j)); q(j) = 0; else q(j) = FinalSize(j); end;
 end
 FinalSize=q
 FinalSizeS=h
 plot(FinalSize,'ko-')
 hold on
 %plot(S,'k.-')
 plot(FinalSized,'k.-')
 plot(FinalSizeS,'k+-')
 plot([4 4],[0 400],'k--')
 ylabel('Final number infected')
 xlabel('Number of variants')
 title('c) SI-Increasing variants: Final size')
 legend('Agent', 'Deterministic', 'Stochastic Approximation')
 
  %RESISTANCE
 figTest2Resistance = figure
 %agentImax = max(Imean,[],2);
 %plot(agentImax,'k')
 for j = 1:8
 if isnan(FinalSizeS(j)); h(j) = 0; else h(j) = FinalSizeS(j); end;
 if isnan(FinalSize(j)); q(j) = 0; else q(j) = FinalSize(j); end;
 end
 FinalSize=q
 FinalSizeS=h
 plot(1-FinalSize./N,'ko-')
 hold on
 %plot(S,'k.-')
 plot(1-FinalSized./N,'k.-')
 plot(1-FinalSizeS./N,'k+-')
 plot([4 4],[0 1],'k--')
 ylabel('Resistance M_R')
 xlabel('Variant Richness')
 title('c) SI-Increasing variants: Resistance')
 legend('measured Agent', ' measured Deterministic', 'Stochastic Approximation')
 
 %RESILIENCE
 figTest2Resilience = figure
  plot(1./MLa,'ko-') %agent
 hold on
 plot(1./MLd,'k.-') %deterministic
 hold on
  ylabel('Resilience M_L')
 xlabel('Variant Richness')
 title('c) SI-Increasing variants: Resilience')
 legend('measured Agent', ' measured Deterministic', 'Stochastic Approximation')
 
saveas(figTest9beta0_02,[outputDirectory 'Test12beta0_02.fig']);
saveas(figTest9beta0_02,[outputDirectory 'Test12beta0_02.emf']);
saveas(figTest9gamma0_02,[outputDirectory 'Test12gama0_02.fig']);
saveas(figTest9gamma0_02,[outputDirectory 'Test12gama0_02.emf']);
saveas(figTest9inf0_02,[outputDirectory 'Test12meanall.fig']);
saveas(figTest9inf0_02,[outputDirectory 'Test12meanall.emf']); 
saveas(figmajor,[outputDirectory 'Test12meanmajor.fig']);
saveas(figmajor,[outputDirectory 'Test12meanmajor.emf']);
saveas(figTest2FinalInfected,[outputDirectory 'Test12FinalInfected.fig']);
saveas(figTest2FinalInfected,[outputDirectory 'Test12FinalInfected.emf']);
saveas(figTest2Resistance,[outputDirectory 'Test12Resistance.fig']);
saveas(figTest2Resistance,[outputDirectory 'Test12Resistance.emf']);
saveas(figTest2Resilience,[outputDirectory 'Test12Resilience.fig']);
saveas(figTest2Resilience,[outputDirectory 'Test12Resilience.emf']);
%close all


betameanmean
gammameanmean

betameanmean2
betameanmeanNew 
betameanmeanNew2
gammameanmeanNew

Rnewendmean%epidemics only
Rnewendstd

 propSimsNonEp % proportion of sims with non-epidemics
 propSimsNonEp_theoretical %theoretical value
    
R = betameanmean./gammameanmean
RN = betameanmeanNew./gammameanmeanNew
RTheoretical = beta./gamma

figro = figure
%plot(beta,R,'k')
hold on
plot(RN,'ko-')
plot(RTheoretical,'k.-')
plot([1 1 1 1 1 1 1 1],'k--')
plot([4 4],[0 2],'k--')
xlabel('Number of variants')
ylabel('ro')
%legend('R','RNew','Rtheoretical') 
legend('Ro Agent(measured)','Ro Deterministic') 
%axis([0.1 0.5 0 2])
saveas(figro,[outputDirectory 'Test12figro.fig']);
saveas(figro,[outputDirectory 'Test12figro.emf']);
%close all
%close all;
end %SIRN Test12


%doSIRNTests(11) = 1 %SIR OR increase number of exploits - two locus
if doSIRNTests(13) == 1
outputDirectory = 'randomGenotypesGraphs\'
%Increase variants upto 8, fix exploit
%***************************************
%SIRN testing
%***************************************
mode = 'static'
%monte carlo random genotype SIR model
numRuns = 200
%1 genotype
numNodes = 400; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1; numLoci=2; casetype = 'OR'; throwAway = 'nonEpidemics', SEpidemic=400-110;
%[betameanmean(1) betameanmean2(1) Smean(1,:) Imean(1,:) Rmean(1,:) NSmean(1,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
[fighistRecovered(1) fighistSusceptible(1) fighistpeakValue(1) fighistpeakTime(1) fighistBeta(1) fighistGamma(1) fighistBetanew(1) fighistGammanew(1) figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean(1) gammameanmean betameanmean2(1) betameanmeanNew(1) betameanmeanNew2 gammameanmeanNew(1) Smean(1,:) Imean(1,:) Rmean(1,:) NSmean(1,:) Smedian Imedian Rmedian S I R NS SmeanN ImeanN1 RmeanN1 Snewendmean Snewendstd Rnewendmean(1) Rnewendstd(1) Rnew_nonendmean Rnew_nonendstd propSimsNonEp(1) propSimsNonEp_theoretical(1)...
            ] = monteCarloSIRN2(numRuns,numNodes, betas, numGenotypes, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,mode, numLoci, casetype);
   size(ImeanN1)
FinalSize(1) = max(RmeanN1,[],2) %mean final size
    %resilience
     maska = RmeanN1 >= FinalSize(1)-1; %when within 1 of the final value
    [C Index] = max(maska);
    MLa(1) = Index; %T1

%2 genotype
numNodes = 400; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=2; numLoci=2; casetype = 'OR'; throwAway = 'nonEpidemics', SEpidemic=400-110; %can see only 1 distribution
%[betameanmean(2) betameanmean2(2) Smean(2,:) Imean(2,:) Rmean(2,:) NSmean(2,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
[fighistRecovered(2) fighistSusceptible(2) fighistpeakValue(2) fighistpeakTime(2) fighistBeta(2) fighistGamma(2) fighistBetanew(2) fighistGammanew(2) figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean(2) gammameanmean betameanmean2(2) betameanmeanNew(2) betameanmeanNew2 gammameanmeanNew(2) Smean(2,:) Imean(2,:) Rmean(2,:) NSmean(2,:) Smedian Imedian Rmedian S I R NS SmeanN ImeanN2 RmeanN2 Snewendmean Snewendstd Rnewendmean(2) Rnewendstd(2) Rnew_nonendmean Rnew_nonendstd propSimsNonEp(2) propSimsNonEp_theoretical(2)...
            ] = monteCarloSIRN2(numRuns,numNodes, betas, numGenotypes, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,mode, numLoci, casetype);
   size(ImeanN2)
FinalSize(2) = max(RmeanN2,[],2) %mean final size
    %resilience
     maska = RmeanN2 >= FinalSize(2)-1; %when within 1 of the final value
    [C Index] = max(maska);
    MLa(2) = Index; %T1
%3 genotype
numNodes = 400; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=3; numLoci=2; casetype = 'OR'; throwAway = 'nonEpidemics', SEpidemic=400-60; %two distributions
%[betameanmean(3) betameanmean2(3) Smean(3,:) Imean(3,:) Rmean(3,:) NSmean(3,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
[fighistRecovered(3) fighistSusceptible(3) fighistpeakValue(3) fighistpeakTime(3) fighistBeta(3) fighistGamma(3) fighistBetanew(3) fighistGammanew(3) figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean(3) gammameanmean betameanmean2(3) betameanmeanNew(3) betameanmeanNew2 gammameanmeanNew(3) Smean(3,:) Imean(3,:) Rmean(3,:) NSmean(3,:) Smedian Imedian Rmedian S I R NS SmeanN ImeanN3 RmeanN3 Snewendmean Snewendstd Rnewendmean(3) Rnewendstd(3) Rnew_nonendmean Rnew_nonendstd propSimsNonEp(3) propSimsNonEp_theoretical(3)...
            ] = monteCarloSIRN2(numRuns,numNodes, betas, numGenotypes, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,mode, numLoci, casetype);

   size(ImeanN3)
FinalSize(3) = max(RmeanN3,[],2) %mean final size
    %resilience
     maska = RmeanN3 >= FinalSize(3)-1; %when within 1 of the final value
    [C Index] = max(maska);
    MLa(3) = Index; %T1
%4 genotype
numNodes = 400; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=4; numLoci=2; casetype = 'OR';
%[betameanmean(4) betameanmean2(4) Smean(4,:) Imean(4,:) Rmean(4,:) NSmean(4,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
[fighistRecovered(4) fighistSusceptible(4) fighistpeakValue(4) fighistpeakTime(4) fighistBeta(4) fighistGamma(4) fighistBetanew(4) fighistGammanew(4) figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean(4) gammameanmean betameanmean2(4) betameanmeanNew(4) betameanmeanNew2 gammameanmeanNew(4) Smean(4,:) Imean(4,:) Rmean(4,:) NSmean(4,:) Smedian Imedian Rmedian S I R NS SmeanN ImeanN4 RmeanN4 Snewendmean Snewendstd Rnewendmean(4) Rnewendstd(4) Rnew_nonendmean Rnew_nonendstd propSimsNonEp(4) propSimsNonEp_theoretical(4)...
            ] = monteCarloSIRN2(numRuns,numNodes, betas, numGenotypes, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,mode, numLoci, casetype);

   size(ImeanN4)
FinalSize(4) = max(RmeanN4,[],2) %mean final size
    %resilience
     maska = RmeanN4 >= FinalSize(4)-1; %when within 1 of the final value
    [C Index] = max(maska);
    MLa(4) = Index; %T1
    %5 genotype
numNodes = 400; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=5; numLoci=2; casetype = 'OR';
%[betameanmean(5) betameanmean2(5) Smean(5,:) Imean(5,:) Rmean(5,:) NSmean(5,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
[fighistRecovered(5) fighistSusceptible(5) fighistpeakValue(5) fighistpeakTime(5) fighistBeta(5) fighistGamma(5) fighistBetanew(5) fighistGammanew(5) figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean(5) gammameanmean betameanmean2(5) betameanmeanNew(5) betameanmeanNew2 gammameanmeanNew(5) Smean(5,:) Imean(5,:) Rmean(5,:) NSmean(5,:) Smedian Imedian Rmedian S I R NS SmeanN ImeanN5 RmeanN5 Snewendmean Snewendstd Rnewendmean(5) Rnewendstd(5) Rnew_nonendmean Rnew_nonendstd propSimsNonEp(5) propSimsNonEp_theoretical(5)...
            ] = monteCarloSIRN2(numRuns,numNodes, betas, numGenotypes, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,mode, numLoci, casetype);

   size(ImeanN5)
FinalSize(5) = max(RmeanN5,[],2) %mean final size
    %resilience
     maska = RmeanN5 >= FinalSize(5)-1; %when within 1 of the final value
    [C Index] = max(maska);
    MLa(5) = Index; %T1
  %6 genotype
numNodes = 400; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=6; numLoci=2; casetype = 'OR';
%[betameanmean(6) betameanmean2(6) Smean(6,:) Imean(6,:) Rmean(6,:) NSmean(6,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
[fighistRecovered(6) fighistSusceptible(6) fighistpeakValue(6) fighistpeakTime(6) fighistBeta(6) fighistGamma(6) fighistBetanew(6) fighistGammanew(6) figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean(6) gammameanmean betameanmean2(6) betameanmeanNew(6) betameanmeanNew2 gammameanmeanNew(6) Smean(6,:) Imean(6,:) Rmean(6,:) NSmean(6,:) Smedian Imedian Rmedian S I R NS SmeanN ImeanN6 RmeanN6 Snewendmean Snewendstd Rnewendmean(6) Rnewendstd(6) Rnew_nonendmean Rnew_nonendstd propSimsNonEp(6) propSimsNonEp_theoretical(6)...
            ] = monteCarloSIRN2(numRuns,numNodes, betas, numGenotypes, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,mode, numLoci, casetype);


   size(ImeanN6)
FinalSize(6) = max(RmeanN6,[],2) %mean final size
    %resilience
     maska = RmeanN6 >= FinalSize(6)-1; %when within 1 of the final value
    [C Index] = max(maska);
    MLa(6) = Index; %T1
        %7 genotype
numNodes = 400; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=7; numLoci=2; casetype = 'OR';
%[betameanmean(7) betameanmean2(7) Smean(7,:) Imean(7,:) Rmean(7,:) NSmean(7,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
[fighistRecovered(7) fighistSusceptible(7) fighistpeakValue(7) fighistpeakTime(7) fighistBeta(7) fighistGamma(7) fighistBetanew(7) fighistGammanew(7) figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2 ...
            betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean(7) gammameanmean betameanmean2(7) betameanmeanNew(7) betameanmeanNew2 gammameanmeanNew(7) Smean(7,:) Imean(7,:) Rmean(7,:) NSmean(7,:) Smedian Imedian Rmedian S I R NS SmeanN ImeanN7 RmeanN7 Snewendmean Snewendstd Rnewendmean(7) Rnewendstd(7) Rnew_nonendmean Rnew_nonendstd propSimsNonEp(7) propSimsNonEp_theoretical(7)...
            ] = monteCarloSIRN2(numRuns,numNodes, betas, numGenotypes, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,mode, numLoci, casetype);

   size(ImeanN7)
FinalSize(7) = max(RmeanN7,[],2) %mean final size
    %resilience
     maska = RmeanN7 >= FinalSize(7)-1; %when within 1 of the final value
    [C Index] = max(maska);
    MLa(7) = Index; %T1
        %8 genotype
numNodes = 400; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=8; numLoci=2; casetype = 'OR';
%[betameanmean(8) betameanmean2(8) Smean(8,:) Imean(8,:) Rmean(8,:) NSmean(8,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci, casetype);
[fighistRecovered(8) fighistSusceptible(8) fighistpeakValue(8) fighistpeakTime(8) fighistBeta(8) fighistGamma(8) fighistBetanew(8) fighistGammanew(8) figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean(8) gammameanmean betameanmean2(8) betameanmeanNew(8) betameanmeanNew2 gammameanmeanNew(8) Smean(8,:) Imean(8,:) Rmean(8,:) NSmean(8,:) Smedian Imedian Rmedian S I R NS SmeanN ImeanN8 RmeanN8 Snewendmean Snewendstd Rnewendmean(8) Rnewendstd(8) Rnew_nonendmean Rnew_nonendstd propSimsNonEp(8) propSimsNonEp_theoretical(8)...
            ] = monteCarloSIRN2(numRuns,numNodes, betas, numGenotypes, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,mode, numLoci, casetype);

   size(ImeanN8)
FinalSize(8) = max(RmeanN8,[],2) %mean final size
    %resilience
     maska = RmeanN8 >= FinalSize(8)-1; %when within 1 of the final value
    [C Index] = max(maska);
    MLa(8) = Index; %T1
    %Histograms
 for i = 1:8
    saveas(fighistRecovered(i),[outputDirectory 'Test13fighistRecovered' num2str(i) '.fig']);
    saveas(fighistRecovered(i),[outputDirectory 'Test13fighistRecovered' num2str(i) '.emf']);
    saveas(fighistSusceptible(i),[outputDirectory 'Test13fighistSusceptible' num2str(i) '.fig']);
    saveas(fighistSusceptible(i),[outputDirectory 'Test13fighistSusceptible' num2str(i) '.emf']);
    saveas(fighistpeakValue(i),[outputDirectory 'Test13fighistpeakValue' num2str(i) '.fig']);
    saveas(fighistpeakValue(i),[outputDirectory 'Test13fighistpeakValue' num2str(i) '.emf']);
    saveas(fighistpeakTime(i),[outputDirectory 'Test13fighistpeakTime' num2str(i) '.fig']);
    saveas(fighistpeakTime(i),[outputDirectory 'Test13fighistpeakTime' num2str(i) '.emf']);
    saveas(fighistBeta(i),[outputDirectory 'Test13histBeta' num2str(i) '.fig']);
    saveas(fighistBeta(i),[outputDirectory 'Test13histBeta' num2str(i) '.emf']);
    saveas(fighistGamma(i),[outputDirectory 'Test13histGamma' num2str(i) '.fig']);
    saveas(fighistGamma(i),[outputDirectory 'Test13histGamma' num2str(i) '.emf']);
    saveas(fighistBetanew(i),[outputDirectory 'Test13histBetaNew' num2str(i) '.fig']);
    saveas(fighistBetanew(i),[outputDirectory 'Test13histBetaNew' num2str(i) '.emf']);
    saveas(fighistGammanew(i),[outputDirectory 'Test13histGammaNew' num2str(i) '.fig']);
    saveas(fighistGammanew(i),[outputDirectory 'Test13histGammaNew' num2str(i) '.emf']);
 end
 close all;
%Deterministic - 2 loci
susProp2loci = [0.2344,0.4375,0.6094,0.7500,0.8594,0.9375,0.9844,1.0000]%susceptibilityTests.m
N=400; S = N.*susProp2loci; B = 0.02; beta = B.*susProp2loci; gamma = 0.01;
for i = 1:8
    [Ro(i) St(i,:) It(i,:) Rt(i,:)] = SIRDeterministic(beta(i), gamma, S(i), 1, 0, numTimesteps, 0);
     FinalSized(i) = Rt(i,end);
          
    %resilience
     maskd = Rt(i,:) >= Rt(i,end)-1; %when within 1 of the final value
    [C Index] = max(maskd);
    MLd(i) = Index; %T1
end
dec = 50; t2 = 1:dec:numTimesteps;


%stochastic final size approximation
betaSolve =[0.0047    0.0088    0.0122    0.0150    0.0172   0.0187    0.0197    0.0200]%calculated from above

    meanRTheory1 =  (solve('1-x = exp(-1.0*(0.0047/0.01)*x)', 'Real', true))*numNodes
    meanRTheory2 =  (solve('1-x = exp(-1.0*(0.0088/0.01)*x)', 'Real', true))*numNodes
    meanRTheory3 =  (solve('1-x = exp(-1.0*(0.0122/0.01)*x)', 'Real', true))*numNodes
    meanRTheory4 =  (solve('1-x = exp(-1.0*(0.0150/0.01)*x)', 'Real', true))*numNodes
    meanRTheory5 =  (solve('1-x = exp(-1.0*(0.0172/0.01)*x)', 'Real', true))*numNodes
    meanRTheory6 =  (solve('1-x = exp(-1.0*(0.0187/0.01)*x)', 'Real', true))*numNodes
    meanRTheory7 =  (solve('1-x = exp(-1.0*(0.0197/0.01)*x)', 'Real', true))*numNodes
    meanRTheory8 =  (solve('1-x = exp(-1.0*(0.02/0.01)*x)', 'Real', true))*numNodes
FinalSizeS = [max(double(meanRTheory1)) max(double(meanRTheory2)) max(double(meanRTheory3)) max(double(meanRTheory4)) max(double(meanRTheory5)) max(double(meanRTheory6)) max(double(meanRTheory7)) max(double(meanRTheory8))];
%FinalSizeSMask = FinalSizeS>0; %if less than zero, then zero it as only valid if positive
%FinalSizeS = FinalSizeS.*FinalSizeSMask
betameanmeanD=betameanmean
 figTest9beta0_02 = figure
 plot(betameanmeanNew,'ko-')
 hold on
 plot(beta,'k.-')
 ylabel('beta')
 xlabel('Number of exploits')
 %title('b) SI-Increasing variants on 1 locus - ')
 title('b) SI-Increasing exploits: Infection rate')
 legend('Agent(measured)', 'Deterministic')

 
 figTest9gamma0_02 = figure
 plot(gammameanmeanNew,'ko-')
 hold on
 plot([0.01 0.01 0.01 0.01 0.01 0.01 0.01 0.01],'k.-')
 ylabel('gama')
 xlabel('Number of exploits')
 %title('b) SI-Increasing variants on 1 locus - ')
 title(' SI-Increasing exploits: Recovery rate')
 legend('Agent(measured)', 'Deterministic')
axis([1 8 0 0.025])
  
 figTest9inf0_02 = figure
 for i = 1:8
     plot(Imean(i,:),'k')
     hold on
     plot(t2,It(i,1:dec:end),'k.-')
 end
 ylabel('Number of nodes infected - It')
 xlabel('Timesteps')
 title('a) SI-Increasing exploits: Nodes infected')
 legend('Agent', 'Deterministic')
 %axis([0 1000 0 400])
 
  figmajor = figure
 for i = 1:8
     plot(ImeanN1,'k')
     hold on
     plot(t2,It(i,1:dec:end),'k.-')
     plot(ImeanN2,'k')
     plot(ImeanN3,'k')
     plot(ImeanN4,'k')
     plot(ImeanN5,'k')
     plot(ImeanN6,'k')
     plot(ImeanN7,'k')
 end
xlabel('Timesteps')
ylabel('Number of individuals infected I(t)')
title('Mean of major outbreaks')
 legend('Agent', 'Deterministic')
%axis([0 100 0 90])
 
 
 %number of nodes infected
 figTest2FinalInfected = figure
 %agentImax = max(Imean,[],2);
 %plot(agentImax,'k')
 for j = 1:8
 if isnan(FinalSizeS(j)); h(j) = 0; else h(j) = FinalSizeS(j); end;
 if isnan(FinalSize(j)); q(j) = 0; else q(j) = FinalSize(j); end;
 end
 FinalSize=q
 FinalSizeS=h
 plot(FinalSize,'ko-')
 hold on
 %plot(S,'k.-')
 plot(FinalSized,'k.-')
 plot(FinalSizeS,'k+-')
 plot([3 3],[0 400],'k--')
 ylabel('Final number infected')
 xlabel('Number of exploits')
 title('c) SI-Increasing exploits: Final size')
 legend('Agent', 'Deterministic', 'Stochastic Approximation')
 
  %RESISTANCE
 figTest2Resistance = figure
 %agentImax = max(Imean,[],2);
 %plot(agentImax,'k')
 for j = 1:8
 if isnan(FinalSizeS(j)); h(j) = 0; else h(j) = FinalSizeS(j); end;
 if isnan(FinalSize(j)); q(j) = 0; else q(j) = FinalSize(j); end;
 end
 FinalSize=q
 FinalSizeS=h
 plot(1-FinalSize./N,'ko-')
 hold on
 %plot(S,'k.-')
 plot(1-FinalSized./N,'k.-')
 plot(1-FinalSizeS./N,'k+-')
 plot([4 4],[0 1],'k--')
 ylabel('Resistance M_R')
 xlabel('Variant Richness')
 title('c) SI-Increasing variants: Resistance')
 legend('measured Agent', ' measured Deterministic', 'Stochastic Approximation')
 
 %RESILIENCE
 figTest2Resilience = figure
  plot(1./MLa,'ko-') %agent
 hold on
 plot(1./MLd,'k.-') %deterministic
 hold on
  ylabel('Resilience M_L')
 xlabel('Variant Richness')
 title('c) SI-Increasing variants: Resilience')
 legend('measured Agent', ' measured Deterministic', 'Stochastic Approximation')
 
 
saveas(figTest9beta0_02,[outputDirectory 'Test13beta0_02.fig']);
saveas(figTest9beta0_02,[outputDirectory 'Test13beta0_02.emf']);
saveas(figTest9gamma0_02,[outputDirectory 'Test13gama0_02.fig']);
saveas(figTest9gamma0_02,[outputDirectory 'Test13gama0_02.emf']);
saveas(figTest9inf0_02,[outputDirectory 'Test13meanall.fig']);
saveas(figTest9inf0_02,[outputDirectory 'Test13meanall.emf']); 
saveas(figmajor,[outputDirectory 'Test13meanmajor.fig']);
saveas(figmajor,[outputDirectory 'Test13meanmajor.emf']);
saveas(figTest2FinalInfected,[outputDirectory 'Test13FinalInfected.fig']);
saveas(figTest2FinalInfected,[outputDirectory 'Test13FinalInfected.emf']);
saveas(figTest2Resistance,[outputDirectory 'Test13Resistance.fig']);
saveas(figTest2Resistance,[outputDirectory 'Test13Resistance.emf']);
saveas(figTest2Resilience,[outputDirectory 'Test12Resilience.fig']);
saveas(figTest2Resilience,[outputDirectory 'Test12Resilience.emf']);
%close all


betameanmean
gammameanmean

betameanmean2
betameanmeanNew 
betameanmeanNew2
gammameanmeanNew

Rnewendmean%epidemics only
Rnewendstd

 propSimsNonEp % proportion of sims with non-epidemics
 propSimsNonEp_theoretical %theoretical value
    
R = betameanmean./gammameanmean
RN = betameanmeanNew./gammameanmeanNew
RTheoretical = beta./gamma

figro = figure
%plot(beta,R,'k')
hold on
plot(RN,'ko-')
plot(RTheoretical,'k.-')
plot([1 1 1 1 1 1 1 1],'k--')
plot([3 3],[0 2],'k--')
xlabel('Number of exploits')
ylabel('ro')
%legend('R','RNew','Rtheoretical') 
legend('Ro Agent(measured)','Ro Deterministic') 
%axis([0.1 0.5 0 2])
saveas(figro,[outputDirectory 'Test13figro.fig']);
saveas(figro,[outputDirectory 'Test13figro.emf']);
%close all
end  %SIRN 13

% SIRN TEST 14
% SIR - 4 loci OR fix exploit at 1 and increase variants - takes 8 hours
%***************
if doSIRNTests(14) == 1
outputDirectory = 'randomGenotypesGraphs\'
%Increase variants upto 25, fix exploit
%***************************************
%SIRN testing
%***************************************
%Deterministic - 4 loci OR
tic
numNodes = 1000;
VRmin = 1 %25
VRmax = 30 %25
N=numNodes; Vmax = 1:VRmax;  G = Vmax.^2; %number genotypes
%for OR case
Sprop = SPExact4LociOR( 1, Vmax )
mode = 'static'
%monte carlo random genotype SIR model
numRuns = 100 %100 %200

for z= VRmin:VRmax
 betan = 0.02; variantRichness =  z;  numTimesteps = 7000; plotGraphs = 0; gamma = 0.004; numExploits=1; numLoci=4; casetype = 'OR'; throwAway = 'nonEpidemics', cutoff = gamma.*numNodes./(betan.*Sprop); SEpidemic=numNodes - 60; %cutoff;
%agent
 [fighistRecovered(z) fighistSusceptible(z) fighistpeakValue(z) fighistpeakTime(z) fighistBeta(z) fighistGamma(z) fighistBetanew(z) fighistGammanew(z) figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean(z) gammameanmean betameanmean2(z) betameanmeanNew(z) betameanmeanNew2 gammameanmeanNew(z) Smean(z,:) Imean(z,:) Rmean(z,:) NSmean(z,:) Smedian Imedian Rmedian S I R NS SmeanN ImeanN1(z,:) RmeanN1 Snewendmean Snewendstd Rnewendmean(z) Rnewendstd(z) Rnew_nonendmean Rnew_nonendstd propSimsNonEp(z) propSimsNonEp_theoretical(z)...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,mode, numLoci, casetype);
   size(ImeanN1)
FinalSize(z) = max(RmeanN1,[],2) %mean final size
    %resilience of major only
     maska = RmeanN1 >= FinalSize(z)-1; %when within 1 of the final value
    [C Index] = max(maska);
    MLa(z) = Index; %T1
    RmeanN1
    size(RmeanN1)
    size(Index)
        %resilience of all
     maska = Rmean(z,:) >= FinalSize(z)-1; %when within 1 of the final value
    [C Index] = max(maska);
        size(Rmean(z,:))
    size(Index)
    MLa_all(z) = Index; %T1
        saveas(fighistRecovered(z),[outputDirectory 'Test14fighistRecovered' num2str(z) '.fig']);
    saveas(fighistRecovered(z),[outputDirectory 'Test14fighistRecovered' num2str(z) '.emf']);
    saveas(fighistSusceptible(z),[outputDirectory 'Test14fighistSusceptible' num2str(z) '.fig']);
    saveas(fighistSusceptible(z),[outputDirectory 'Test14fighistSusceptible' num2str(z) '.emf']);
    saveas(fighistpeakValue(z),[outputDirectory 'Test14fighistpeakValue' num2str(z) '.fig']);
    saveas(fighistpeakValue(z),[outputDirectory 'Test14fighistpeakValue' num2str(z) '.emf']);
    saveas(fighistpeakTime(z),[outputDirectory 'Test14fighistpeakTime' num2str(z) '.fig']);
    saveas(fighistpeakTime(z),[outputDirectory 'Test14fighistpeakTime' num2str(z) '.emf']);
    saveas(fighistBeta(z),[outputDirectory 'Test14histBeta' num2str(z) '.fig']);
    saveas(fighistBeta(z),[outputDirectory 'Test14histBeta' num2str(z) '.emf']);
    saveas(fighistGamma(z),[outputDirectory 'Test14histGamma' num2str(z) '.fig']);
    saveas(fighistGamma(z),[outputDirectory 'Test14histGamma' num2str(z) '.emf']);
    saveas(fighistBetanew(z),[outputDirectory 'Test14histBetaNew' num2str(z) '.fig']);
    saveas(fighistBetanew(z),[outputDirectory 'Test14histBetaNew' num2str(z) '.emf']);
    saveas(fighistGammanew(z),[outputDirectory 'Test14histGammaNew' num2str(z) '.fig']);
    saveas(fighistGammanew(z),[outputDirectory 'Test14histGammaNew' num2str(z) '.emf']);
    close(fighistRecovered(z));
    close(fighistSusceptible(z));
    close(fighistpeakValue(z));
    close(fighistpeakTime(z));
    close(fighistBeta(z));
    close(fighistGamma(z));
    close(fighistBetanew(z));
    close(fighistGammanew(z));
    
    %stochastic
    [SfighistRecovered(z) SfighistSusceptible(z) SfighistpeakValue(z) SfighistpeakTime(z) SfighistBeta(z) SfighistGamma(z) SfighistBetanew(z) SfighistGammanew(z) SfigInfectedRun1 SfigBetaRun1 SfigGammaRun1 SfigRoRun1 SfigBetaGamma1 SfigBetaGamma2...
            Sbetastd Sgammastd Sbetastd2 SbetastdNew SbetastdNew2 SgammastdNew SpeakTimemean SpeakTimestd SpeakValuemean SpeakValuestd...
            Sbetameanmean(z) Sgammameanmean Sbetameanmean2(z) SbetameanmeanNew(z) SbetameanmeanNew2 SgammameanmeanNew(z) SSmean(z,:) SImean(z,:) SRmean(z,:) SNSmean(z,:) SSmedian SImedian SRmedian SS SI SR SNS SSmeanN SImeanN1(z,:) SRmeanN1 SSnewendmean SSnewendstd SRnewendmean(z) SRnewendstd(z) SRnew_nonendmean SRnew_nonendstd SpropSimsNonEp(z) SpropSimsNonEp_theoretical(z)...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,mode, numLoci, casetype);
     size(SImeanN1)
SFinalSize(z) = max(SRmeanN1,[],2) %mean final size
    %resilience of major only
     Smaska = SRmeanN1 >= SFinalSize(z)-1; %when within 1 of the final value
    [SC SIndex] = max(Smaska);
    SMLa(z) = SIndex; %T1
        %resilience of all
     Smaska = SRmean(z,:) >= SFinalSize(z)-1; %when within 1 of the final value
    [SC SIndex] = max(Smaska);
    SMLa_all(z) = SIndex; %T1
        saveas(SfighistRecovered(z),[outputDirectory 'Test14StochfighistRecovered' num2str(z) '.fig']);
    saveas(SfighistRecovered(z),[outputDirectory 'Test14StochfighistRecovered' num2str(z) '.emf']);
    saveas(SfighistSusceptible(z),[outputDirectory 'Test14StochfighistSusceptible' num2str(z) '.fig']);
    saveas(SfighistSusceptible(z),[outputDirectory 'Test14StochfighistSusceptible' num2str(z) '.emf']);
    saveas(SfighistpeakValue(z),[outputDirectory 'Test14StochfighistpeakValue' num2str(z) '.fig']);
    saveas(SfighistpeakValue(z),[outputDirectory 'Test14StochfighistpeakValue' num2str(z) '.emf']);
    saveas(SfighistpeakTime(z),[outputDirectory 'Test14StochfighistpeakTime' num2str(z) '.fig']);
    saveas(SfighistpeakTime(z),[outputDirectory 'Test14StochfighistpeakTime' num2str(z) '.emf']);
    saveas(SfighistBeta(z),[outputDirectory 'Test14StochhistBeta' num2str(z) '.fig']);
    saveas(SfighistBeta(z),[outputDirectory 'Test14StochhistBeta' num2str(z) '.emf']);
    saveas(SfighistGamma(z),[outputDirectory 'Test14StochhistGamma' num2str(z) '.fig']);
    saveas(SfighistGamma(z),[outputDirectory 'Test14StochhistGamma' num2str(z) '.emf']);
    saveas(SfighistBetanew(z),[outputDirectory 'Test14StochhistBetaNew' num2str(z) '.fig']);
    saveas(SfighistBetanew(z),[outputDirectory 'Test14StochhistBetaNew' num2str(z) '.emf']);
    saveas(SfighistGammanew(z),[outputDirectory 'Test14StochhistGammaNew' num2str(z) '.fig']);
    saveas(SfighistGammanew(z),[outputDirectory 'Test14StochhistGammaNew' num2str(z) '.emf']);
    close(SfighistRecovered(z));
    close(SfighistSusceptible(z));
    close(SfighistpeakValue(z));
    close(SfighistpeakTime(z));
    close(SfighistBeta(z));
    close(SfighistGamma(z));
    close(SfighistBetanew(z));
    close(SfighistGammanew(z));
    text = ['done ' num2str(z) ' out of ' num2str(VRmax)];
    disp(text)
end    
    
    
    %Histograms
% for i = 1:VRmax
%    saveas(fighistRecovered(i),[outputDirectory 'Test14fighistRecovered' num2str(i) '.fig']);
%    saveas(fighistRecovered(i),[outputDirectory 'Test14fighistRecovered' num2str(i) '.emf']);
%    saveas(fighistSusceptible(i),[outputDirectory 'Test14fighistSusceptible' num2str(i) '.fig']);
%    saveas(fighistSusceptible(i),[outputDirectory 'Test14fighistSusceptible' num2str(i) '.emf']);
%    saveas(fighistpeakValue(i),[outputDirectory 'Test14fighistpeakValue' num2str(i) '.fig']);
%    saveas(fighistpeakValue(i),[outputDirectory 'Test14fighistpeakValue' num2str(i) '.emf']);
%    saveas(fighistpeakTime(i),[outputDirectory 'Test14fighistpeakTime' num2str(i) '.fig']);
%    saveas(fighistpeakTime(i),[outputDirectory 'Test14fighistpeakTime' num2str(i) '.emf']);
%    saveas(fighistBeta(i),[outputDirectory 'Test14histBeta' num2str(i) '.fig']);
%    saveas(fighistBeta(i),[outputDirectory 'Test14histBeta' num2str(i) '.emf']);
%    saveas(fighistGamma(i),[outputDirectory 'Test14histGamma' num2str(i) '.fig']);
%    saveas(fighistGamma(i),[outputDirectory 'Test14histGamma' num2str(i) '.emf']);
%    saveas(fighistBetanew(i),[outputDirectory 'Test14histBetaNew' num2str(i) '.fig']);
%    saveas(fighistBetanew(i),[outputDirectory 'Test14histBetaNew' num2str(i) '.emf']);
%    saveas(fighistGammanew(i),[outputDirectory 'Test14histGammaNew' num2str(i) '.fig']);
%    saveas(fighistGammanew(i),[outputDirectory 'Test14histGammaNew' num2str(i) '.emf']);
% end
% close all;
%Deterministic - 4 loci OR
N=numNodes; Vmax = 1:VRmax;  G = Vmax.^2; %number genotypes
%for OR case
Sprop = SPExact4LociOR( 1, Vmax )
%Sprop = 2./Vmax - 1./G %both loci have the same number of variants and both have 1 exploit
S = N.*Sprop; B = 0.02; beta = B.*Sprop; gamma = 0.004; %1/48*60*60 %2 days
for i = VRmin:VRmax
    [Ro(i) St(i,:) It(i,:) Rt(i,:)] = SIRDeterministic(beta(i), gamma, S(i), 1, 0, numTimesteps, 0); 
    FinalSized(i) = Rt(i,end);
     
    %resilience
     maskd = Rt(i,:) >= Rt(i,end)-1; %when within 1 of the final value
    [C Index] = max(maskd);
    MLd(i) = Index; %T1
end

dec = 50; t2 = 1:dec:numTimesteps;

%stochastic final size approximation

for h = VRmin:VRmax
  text = ['1-x = exp(-1.0*(' num2str(Sprop(h)*betan/gamma) ')*x)'] 
meanRTheory =  (solve(text, 'Real', true))*Sprop(h)*numNodes
FinalSizeS(h) = max(double(meanRTheory))
end

 
 figTest9inf0_02 = figure
 for i = VRmin:VRmax
     plot(Imean(i,:),'k')
     hold on
     plot(t2,It(i,1:dec:end),'k.-')
      plot(SImean(i,:),'k--')
 end
 ylabel('Number of nodes infected - It')
 xlabel('Timesteps')
 title('a) SI-Increasing variants: Nodes infected')
 legend('Agent', 'Deterministic', 'Stochastic')
 %axis([0 1000 0 400])
 
  figmajor = figure
 for i = VRmin:VRmax
     plot(ImeanN1(1,:),'k')
     hold on
     plot(t2,It(i,1:dec:end),'k.-')
     plot(SImeanN1(i,:),'k--')
     plot(ImeanN1(i,:),'k')
 end
xlabel('Timesteps')
ylabel('Number of individuals infected I(t)')
title('Mean of major outbreaks')
 legend('Agent', 'Deterministic', 'Stochastic')
%axis([0 100 0 90])
 
 
 %number of nodes infected
 figTest2FinalInfected = figure
 %agentImax = max(Imean,[],2);
 %plot(agentImax,'k')
 for j = VRmin:VRmax
 if isnan(FinalSizeS(j)); h(j) = 0; else h(j) = FinalSizeS(j); end;
 if isnan(FinalSize(j)); q(j) = 0; else q(j) = FinalSize(j); end;
 end
 FinalSize=q
 FinalSizeS=h
 plot(FinalSize,'ko-')
 hold on
 %plot(S,'k.-')
 plot(FinalSized,'k.-')
 plot(FinalSizeS,'k+-')
plot([18.615 18.615],[0 1],'k--')
 ylabel('Final number infected')
 xlabel('Number of variants')
 title('c) SI-Increasing variants: Final size')
 legend('Agent', 'Deterministic', 'Stochastic Approximation')
 
  %RESISTANCE
 figTest2Resistance = figure
 %agentImax = max(Imean,[],2);
 %plot(agentImax,'k')
 for j = VRmin:VRmax
 %if isnan(FinalSizeS(j)); h(j) = 0; else h(j) = FinalSizeS(j); end; %STOCHASTIC APPROX
 %if isnan(FinalSize(j)); q(j) = 0; else q(j) = FinalSize(j); end; %AGENT
 if isnan(SFinalSize(j)); w(j) = 0; else w(j) = SFinalSize(j); end; %STOCHASTIC MEASURED
 end
 FinalSize=q
 FinalSizeS=h
 SFinalSize=w
 plot(1-FinalSize./N,'ko-')
 hold on
 %plot(S,'k.-')
 plot(1-FinalSized./N,'k.-')
 plot(1-FinalSizeS./N,'k+-')
  plot(1-SFinalSize./N,'ks-')
plot([18.615 18.615],[0 1],'k--')
 ylabel('Resistance M_R')
 xlabel('Variant Richness')
 title('c) SI-Increasing variants: Resistance')
 legend('measured Agent', ' measured Deterministic', 'approximated Stochastic','measured Stochastic')
 
 %RESILIENCE major outbreaks only
 figTest2Resilience = figure
  plot(1./MLa,'ko-') %agent measured
 hold on
 plot(1./MLd,'k.-') %deterministic
  plot(1./SMLa,'ks-') %stochastic measured
 hold on
  ylabel('Resilience M_L')
 xlabel('Variant Richness')
 title('c) SI-Increasing variants: Resilience major')
 legend('measured Agent', ' measured Deterministic', 'measured Stochastic')
 plot([18.615 18.615],[0 1],'k--')

  %RESILIENCE all
 figTest2Resilience2 = figure
  plot(1./MLa_all,'ko-') %agent measured
 hold on
 plot(1./MLd,'k.-') %deterministic
  plot(1./SMLa_all,'ks-') %stochastic measured
 hold on
   plot(1./MLa,'ro-') %agent major only
    plot(1./SMLa,'rs-') %stochastic major only
  ylabel('Resilience M_L')
 xlabel('Variant Richness')
 title('c) SI-Increasing variants: Resilience all')
 legend('measured Agent - all', ' measured Deterministic', 'measured Stochastic - all','measured Agent - major','measured Stochastic - major')
 plot([18.615 18.615],[0 1],'k--')
 
saveas(figTest9inf0_02,[outputDirectory 'Test14meanall.fig']);
saveas(figTest9inf0_02,[outputDirectory 'Test14meanall.emf']); 
saveas(figmajor,[outputDirectory 'Test14meanmajor.fig']);
saveas(figmajor,[outputDirectory 'Test14meanmajor.emf']);
saveas(figTest2FinalInfected,[outputDirectory 'Test14FinalInfected.fig']);
saveas(figTest2FinalInfected,[outputDirectory 'Test14FinalInfected.emf']);
saveas(figTest2Resistance,[outputDirectory 'Test14Resistance.fig']);
saveas(figTest2Resistance,[outputDirectory 'Test14Resistance.emf']);
saveas(figTest2Resilience,[outputDirectory 'Test14Resilience.fig']);
saveas(figTest2Resilience,[outputDirectory 'Test14Resilience.emf']);
saveas(figTest2Resilience2,[outputDirectory 'Test14ResilienceAll.fig']);
saveas(figTest2Resilience2,[outputDirectory 'Test14ResilienceAll.emf']);
%close all


betameanmean
gammameanmean

betameanmean2
betameanmeanNew 
betameanmeanNew2
gammameanmeanNew

Rnewendmean%epidemics only
Rnewendstd

 propSimsNonEp % proportion of sims with non-epidemics
 propSimsNonEp_theoretical %theoretical value
    
R = betameanmean./gammameanmean
RN = betameanmeanNew./gammameanmeanNew
SRN = SbetameanmeanNew./SgammameanmeanNew
RTheoretical = beta./gamma

figro = figure
%plot(beta,R,'k')
hold on
plot(RN,'ko-')
plot(RTheoretical,'k.-')
plot(RN,'ks-') %stochastic measured
plot([1 1 1 1 1 1 1 1],'k--')
plot([18.615 18.615],[0 1],'k--')
xlabel('Number of variants')
ylabel('ro')
%legend('R','RNew','Rtheoretical') 
legend('Ro Agent(measured)','Ro Deterministic','Ro Stochastic(measured)') 
%axis([0.1 0.5 0 2])
saveas(figro,[outputDirectory 'Test14figro.fig']);
saveas(figro,[outputDirectory 'Test14figro.emf']);
%close all
%close all;
toc
   %SAVE FOR LATER
Resistancea_1EXP=1-FinalSize./N
Resistanced_1EXP=1-FinalSized./N
Resistancesapp_1EXP=1-FinalSizeS./N
Resistances_1EXP=1-SFinalSize./N
Resiliencea_1EXP=1./MLa
Resilienced_1EXP=1./MLd
Resiliences_1EXP=1./SMLa

Resistancea_1EXP =[...
    0.0072    0.0724    0.2147    0.3429    0.4450    0.5295    0.6062 ...
    0.6611    0.7033    0.7619    0.7943    0.8384    0.8655    0.8909 ...
    0.8875    0.8905    0.9069    0.9167    0.9015    0.9397    0.9529 ...
    0.9183    0.9405    1.0000    1.0000    1.0000    1.0000    1.0000 ...
    1.0000    1.0000];


Resistanced_1EXP =[...
    0.0069    0.0714    0.2131    0.3417    0.4465    0.5319    0.6027 ...
    0.6625    0.7139    0.7587    0.7982    0.8336    0.8656    0.8952 ...
    0.9228    0.9475    0.9668    0.9793    0.9864    0.9904    0.9927 ...
    0.9941    0.9950    0.9957    0.9961    0.9965    0.9968    0.9970 ...
    0.9972    0.9973];


Resistancesapp_1EXP =[...
    0.0070    0.0715    0.2132    0.3419    0.4467    0.5322    0.6030 ...
    0.6629    0.7144    0.7593    0.7991    0.8347    0.8667    0.8959 ...
    0.9227    0.9473    0.9701    0.9913    1.0000    1.0000    1.0000 ...
    1.0000    1.0000    1.0000    1.0000    1.0000    1.0000    1.0000...
    1.0000    1.0000];


Resistances_1EXP =[...
    0.0071    0.0710    0.2129    0.3397    0.4427    0.5307    0.6012 ...
    0.6688    0.7142    0.7603    0.7964    0.8384    0.8646    0.8770 ...
    0.8950    0.9094    0.9134    0.9370    0.9210    0.9526    0.9120 ...
    0.9506    1.0000    1.0000    1.0000    1.0000    1.0000    1.0000 ...
    1.0000    1.0000];

Resiliencea_1EXP =[...
    0.0004    0.0004    0.0004    0.0004    0.0003    0.0003    0.0003 ...
    0.0002    0.0003    0.0002    0.0002    0.0002    0.0002    0.0002 ...
    0.0002    0.0002    0.0003    0.0002    0.0002    1.0000    1.0000 ...
    0.0004    1.0000    1.0000    1.0000    1.0000    1.0000    1.0000 ...
    1.0000    1.0000];

Resilienced_1EXP =[...
    0.0004    0.0004    0.0004    0.0004    0.0004    0.0003    0.0003 ...
    0.0003    0.0002    0.0002    0.0002    0.0002    0.0002    0.0002 ...
    0.0001    0.0001    0.0002    0.0002    0.0002    0.0002    0.0003 ...
    0.0004    0.0005    0.0006    0.0008    0.0009    0.0011    0.0012 ...
    0.0014    0.0015]

Resiliences_1EXP =[...
    0.0004    0.0004    0.0004    0.0004    0.0004    0.0003    0.0003 ...
    0.0002    0.0002    0.0002    0.0002    0.0002    0.0002    0.0002 ...
    0.0002    0.0002    0.0002    0.0003    0.0002    1.0000    0.0002 ...
    1.0000    1.0000    1.0000    1.0000    1.0000    1.0000    1.0000 ...
    1.0000    1.0000];

end %SIRN Test14


% SIRN TEST 15
% SIR - 4 loci AND fix exploit at 1 and increase variants - takes ?? mins
%***************
if doSIRNTests(15) == 1
outputDirectory = 'randomGenotypesGraphs\'
%Increase variants upto 5, fix exploit
%***************************************
%SIRN testing
%***************************************
%   1 EXPLOIT
%   *******************************************************************************************************
%Deterministic - 4 loci AND
tic
numNodes = 1000;
VRmin = 1 %25
VRmax = 8 %25
N=numNodes; Vmax = 1:VRmax;  G = Vmax.^2; %number genotypes
%for AND case
Sprop = sp4LociAND( 1, Vmax )
mode = 'static'
%monte carlo random genotype SIR model
numRuns = 20 %100 %200

for z= VRmin:VRmax
 betan = 0.02; variantRichness =  z;  numTimesteps = 7000; plotGraphs = 0; gamma = 0.004; numExploits=1; numLoci=4; casetype = 'AND'; throwAway = 'nonEpidemics', cutoff = gamma.*numNodes./(betan.*Sprop); SEpidemic=numNodes - 60; %cutoff;
%agent
 [fighistRecovered(z) fighistSusceptible(z) fighistpeakValue(z) fighistpeakTime(z) fighistBeta(z) fighistGamma(z) fighistBetanew(z) fighistGammanew(z) figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean(z) gammameanmean betameanmean2(z) betameanmeanNew(z) betameanmeanNew2 gammameanmeanNew(z) Smean(z,:) Imean(z,:) Rmean(z,:) NSmean(z,:) Smedian Imedian Rmedian S I R NS SmeanN ImeanN1(z,:) RmeanN1 Snewendmean Snewendstd Rnewendmean(z) Rnewendstd(z) Rnew_nonendmean Rnew_nonendstd propSimsNonEp(z) propSimsNonEp_theoretical(z)...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,mode, numLoci, casetype);
   size(ImeanN1)
FinalSize(z) = max(RmeanN1,[],2) %mean final size
    %resilience of major only
     maska = RmeanN1 >= FinalSize(z)-1; %when within 1 of the final value
    [C Index] = max(maska);
    MLa(z) = Index; %T1
    RmeanN1
    size(RmeanN1)
    size(Index)
        %resilience of all
     maska = Rmean(z,:) >= FinalSize(z)-1; %when within 1 of the final value
    [C Index] = max(maska);
        size(Rmean(z,:))
    size(Index)
    MLa_all(z) = Index; %T1
        saveas(fighistRecovered(z),[outputDirectory 'Test15_1EXPfighistRecovered' num2str(z) '.fig']);
    saveas(fighistRecovered(z),[outputDirectory 'Test15_1EXPfighistRecovered' num2str(z) '.emf']);
    saveas(fighistSusceptible(z),[outputDirectory 'Test15_1EXPfighistSusceptible' num2str(z) '.fig']);
    saveas(fighistSusceptible(z),[outputDirectory 'Test15_1EXPfighistSusceptible' num2str(z) '.emf']);
    saveas(fighistpeakValue(z),[outputDirectory 'Test15_1EXPfighistpeakValue' num2str(z) '.fig']);
    saveas(fighistpeakValue(z),[outputDirectory 'Test15_1EXPfighistpeakValue' num2str(z) '.emf']);
    saveas(fighistpeakTime(z),[outputDirectory 'Test15_1EXPfighistpeakTime' num2str(z) '.fig']);
    saveas(fighistpeakTime(z),[outputDirectory 'Test15_1EXPfighistpeakTime' num2str(z) '.emf']);
    saveas(fighistBeta(z),[outputDirectory 'Test15_1EXPhistBeta' num2str(z) '.fig']);
    saveas(fighistBeta(z),[outputDirectory 'Test15_1EXPhistBeta' num2str(z) '.emf']);
    saveas(fighistGamma(z),[outputDirectory 'Test15_1EXPhistGamma' num2str(z) '.fig']);
    saveas(fighistGamma(z),[outputDirectory 'Test15_1EXPhistGamma' num2str(z) '.emf']);
    saveas(fighistBetanew(z),[outputDirectory 'Test15_1EXPhistBetaNew' num2str(z) '.fig']);
    saveas(fighistBetanew(z),[outputDirectory 'Test15_1EXPhistBetaNew' num2str(z) '.emf']);
    saveas(fighistGammanew(z),[outputDirectory 'Test15_1EXPhistGammaNew' num2str(z) '.fig']);
    saveas(fighistGammanew(z),[outputDirectory 'Test15_1EXPhistGammaNew' num2str(z) '.emf']);
    close(fighistRecovered(z));
    close(fighistSusceptible(z));
    close(fighistpeakValue(z));
    close(fighistpeakTime(z));
    close(fighistBeta(z));
    close(fighistGamma(z));
    close(fighistBetanew(z));
    close(fighistGammanew(z));
    
    %stochastic
    [SfighistRecovered(z) SfighistSusceptible(z) SfighistpeakValue(z) SfighistpeakTime(z) SfighistBeta(z) SfighistGamma(z) SfighistBetanew(z) SfighistGammanew(z) SfigInfectedRun1 SfigBetaRun1 SfigGammaRun1 SfigRoRun1 SfigBetaGamma1 SfigBetaGamma2...
            Sbetastd Sgammastd Sbetastd2 SbetastdNew SbetastdNew2 SgammastdNew SpeakTimemean SpeakTimestd SpeakValuemean SpeakValuestd...
            Sbetameanmean(z) Sgammameanmean Sbetameanmean2(z) SbetameanmeanNew(z) SbetameanmeanNew2 SgammameanmeanNew(z) SSmean(z,:) SImean(z,:) SRmean(z,:) SNSmean(z,:) SSmedian SImedian SRmedian SS SI SR SNS SSmeanN SImeanN1(z,:) SRmeanN1 SSnewendmean SSnewendstd SRnewendmean(z) SRnewendstd(z) SRnew_nonendmean SRnew_nonendstd SpropSimsNonEp(z) SpropSimsNonEp_theoretical(z)...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,mode, numLoci, casetype);
     size(SImeanN1)
SFinalSize(z) = max(SRmeanN1,[],2) %mean final size
    %resilience of major only
     Smaska = SRmeanN1 >= SFinalSize(z)-1; %when within 1 of the final value
    [SC SIndex] = max(Smaska);
    SMLa(z) = SIndex; %T1
        %resilience of all
     Smaska = SRmean(z,:) >= SFinalSize(z)-1; %when within 1 of the final value
    [SC SIndex] = max(Smaska);
    SMLa_all(z) = SIndex; %T1
        saveas(SfighistRecovered(z),[outputDirectory 'Test15_1EXPStochfighistRecovered' num2str(z) '.fig']);
    saveas(SfighistRecovered(z),[outputDirectory 'Test15_1EXPStochfighistRecovered' num2str(z) '.emf']);
    saveas(SfighistSusceptible(z),[outputDirectory 'Test15_1EXPStochfighistSusceptible' num2str(z) '.fig']);
    saveas(SfighistSusceptible(z),[outputDirectory 'Test15_1EXPStochfighistSusceptible' num2str(z) '.emf']);
    saveas(SfighistpeakValue(z),[outputDirectory 'Test15_1EXPStochfighistpeakValue' num2str(z) '.fig']);
    saveas(SfighistpeakValue(z),[outputDirectory 'Test15_1EXPStochfighistpeakValue' num2str(z) '.emf']);
    saveas(SfighistpeakTime(z),[outputDirectory 'Test15_1EXPStochfighistpeakTime' num2str(z) '.fig']);
    saveas(SfighistpeakTime(z),[outputDirectory 'Test15_1EXPStochfighistpeakTime' num2str(z) '.emf']);
    saveas(SfighistBeta(z),[outputDirectory 'Test15_1EXPStochhistBeta' num2str(z) '.fig']);
    saveas(SfighistBeta(z),[outputDirectory 'Test15_1EXPStochhistBeta' num2str(z) '.emf']);
    saveas(SfighistGamma(z),[outputDirectory 'Test15_1EXPStochhistGamma' num2str(z) '.fig']);
    saveas(SfighistGamma(z),[outputDirectory 'Test15_1EXPStochhistGamma' num2str(z) '.emf']);
    saveas(SfighistBetanew(z),[outputDirectory 'Test15_1EXPStochhistBetaNew' num2str(z) '.fig']);
    saveas(SfighistBetanew(z),[outputDirectory 'Test15_1EXPStochhistBetaNew' num2str(z) '.emf']);
    saveas(SfighistGammanew(z),[outputDirectory 'Test15_1EXPStochhistGammaNew' num2str(z) '.fig']);
    saveas(SfighistGammanew(z),[outputDirectory 'Test15_1EXPStochhistGammaNew' num2str(z) '.emf']);
    close(SfighistRecovered(z));
    close(SfighistSusceptible(z));
    close(SfighistpeakValue(z));
    close(SfighistpeakTime(z));
    close(SfighistBeta(z));
    close(SfighistGamma(z));
    close(SfighistBetanew(z));
    close(SfighistGammanew(z));
    text = ['done ' num2str(z) ' out of ' num2str(VRmax)];
    disp(text)
end    
    
   
%Deterministic - 4 loci AND
N=numNodes; Vmax = 1:VRmax;  G = Vmax.^2; %number genotypes
%for AND case
Sprop = sp4LociAND( 1, Vmax )
%Sprop = 2./Vmax - 1./G %both loci have the same number of variants and both have 1 exploit
S = N.*Sprop; B = 0.02; beta = B.*Sprop; gamma = 0.004; %1/48*60*60 %2 days
for i = VRmin:VRmax
    [Ro(i) St(i,:) It(i,:) Rt(i,:)] = SIRDeterministic(beta(i), gamma, S(i), 1, 0, numTimesteps, 0); 
    FinalSized(i) = Rt(i,end);
     
    %resilience
     maskd = Rt(i,:) >= Rt(i,end)-1; %when within 1 of the final value
    [C Index] = max(maskd);
    MLd(i) = Index; %T1
end

dec = 50; t2 = 1:dec:numTimesteps;

%stochastic final size approximation

for h = VRmin:VRmax
  text = ['1-x = exp(-1.0*(' num2str(Sprop(h)*betan/gamma) ')*x)'] 
meanRTheory =  (solve(text, 'Real', true))*Sprop(h)*numNodes
FinalSizeS(h) = max(double(meanRTheory))
end

 
 figTest9inf0_02 = figure
 for i = VRmin:VRmax
     plot(Imean(i,:),'k')
     hold on
     plot(t2,It(i,1:dec:end),'k.-')
      plot(SImean(i,:),'k--')
 end
 ylabel('Number of nodes infected - It')
 xlabel('Timesteps')
 title('a) SI-Increasing variants: Nodes infected')
 legend('Agent', 'Deterministic', 'Stochastic')
 %axis([0 1000 0 400])
 
  figmajor = figure
 for i = VRmin:VRmax
     plot(ImeanN1(1,:),'k')
     hold on
     plot(t2,It(i,1:dec:end),'k.-')
     plot(SImeanN1(i,:),'k--')
     plot(ImeanN1(i,:),'k')
 end
xlabel('Timesteps')
ylabel('Number of individuals infected I(t)')
title('Mean of major outbreaks')
 legend('Agent', 'Deterministic', 'Stochastic')
%axis([0 100 0 90])
 
 
 %number of nodes infected
 figTest2FinalInfected = figure
 %agentImax = max(Imean,[],2);
 %plot(agentImax,'k')
 for j = VRmin:VRmax
 if isnan(FinalSizeS(j)); h(j) = 0; else h(j) = FinalSizeS(j); end;
 if isnan(FinalSize(j)); q(j) = 0; else q(j) = FinalSize(j); end;
 end
 FinalSize=q
 FinalSizeS=h
 plot(FinalSize,'ko-')
 hold on
 %plot(S,'k.-')
 plot(FinalSized,'k.-')
 plot(FinalSizeS,'k+-')
plot([1.8533 1.8533],[0 1],'k--')
 ylabel('Final number infected')
 xlabel('Number of variants')
 title('c) SI-Increasing variants: Final size')
 legend('Agent', 'Deterministic', 'Stochastic Approximation')
 
  %RESISTANCE
 figTest2Resistance = figure
 %agentImax = max(Imean,[],2);
 %plot(agentImax,'k')
 for j = VRmin:VRmax
 %if isnan(FinalSizeS(j)); h(j) = 0; else h(j) = FinalSizeS(j); end; %STOCHASTIC APPROX
 %if isnan(FinalSize(j)); q(j) = 0; else q(j) = FinalSize(j); end; %AGENT
 if isnan(SFinalSize(j)); w(j) = 0; else w(j) = SFinalSize(j); end; %STOCHASTIC MEASURED
 end
 FinalSize=q
 FinalSizeS=h
 SFinalSize=w
 plot(1-FinalSize./N,'ko-')
 hold on
 %plot(S,'k.-')
 plot(1-FinalSized./N,'k.-')
 plot(1-FinalSizeS./N,'k+-')
  plot(1-SFinalSize./N,'ks-')
 plot([1.8533 1.8533],[0 1],'k--')
 ylabel('Resistance M_R')
 xlabel('Variant Richness')
 title('c) SI-Increasing variants: Resistance')
 legend('measured Agent', ' measured Deterministic', 'approximated Stochastic','measured Stochastic')
 axis([1 8 0 1.1])
 %RESILIENCE major outbreaks only
 figTest2Resilience = figure
  plot(1./MLa,'ko-') %agent measured
 hold on
 plot(1./MLd,'k.-') %deterministic
  plot(1./SMLa,'ks-') %stochastic measured
 hold on
  ylabel('Resilience M_L')
 xlabel('Variant Richness')
 title('c) SI-Increasing variants: Resilience major')
 legend('measured Agent', ' measured Deterministic', 'measured Stochastic')
  plot([1.8533 1.8533],[0 1],'k--')
   axis([1 8 0 1.1])
  
   %SAVE FOR LATER
Resistancea_1EXP=1-FinalSize./N;
Resistanced_1EXP=1-FinalSized./N;
Resistancesapp_1EXP=1-FinalSizeS./N;
Resistances_1EXP=1-SFinalSize./N;
Resiliencea_1EXP=1./MLa;
Resilienced_1EXP=1./MLd;
Resiliences_1EXP=1./SMLa;

  %RESILIENCE all
 figTest2Resilience2 = figure
  plot(1./MLa_all,'ko-') %agent measured
 hold on
 plot(1./MLd,'k.-') %deterministic
  plot(1./SMLa_all,'ks-') %stochastic measured
 hold on
   plot(1./MLa,'ro-') %agent major only
    plot(1./SMLa,'rs-') %stochastic major only
  ylabel('Resilience M_L')
 xlabel('Variant Richness')
 title('c) SI-Increasing variants: Resilience all')
 legend('measured Agent - all', ' measured Deterministic', 'measured Stochastic - all','measured Agent - major','measured Stochastic - major')
  plot([1.8533 1.8533],[0 1],'k--')
   axis([1 8 0 1.1])
 
saveas(figTest9inf0_02,[outputDirectory 'Test15_1EXPmeanall.fig']);
saveas(figTest9inf0_02,[outputDirectory 'Test15_1EXPmeanall.emf']); 
saveas(figmajor,[outputDirectory 'Test15_1EXPmeanmajor.fig']);
saveas(figmajor,[outputDirectory 'Test15_1EXPmeanmajor.emf']);
saveas(figTest2FinalInfected,[outputDirectory 'Test15_1EXPFinalInfected.fig']);
saveas(figTest2FinalInfected,[outputDirectory 'Test15_1EXPFinalInfected.emf']);
saveas(figTest2Resistance,[outputDirectory 'Test15_1EXPResistance.fig']);
saveas(figTest2Resistance,[outputDirectory 'Test15_1EXPResistance.emf']);
saveas(figTest2Resilience,[outputDirectory 'Test15_1EXPResilience.fig']);
saveas(figTest2Resilience,[outputDirectory 'Test15_1EXPResilience.emf']);
saveas(figTest2Resilience2,[outputDirectory 'Test15_1EXPResilienceAll.fig']);
saveas(figTest2Resilience2,[outputDirectory 'Test15_1EXPResilienceAll.emf']);
%close all


betameanmean
gammameanmean

betameanmean2
betameanmeanNew 
betameanmeanNew2
gammameanmeanNew

Rnewendmean%epidemics only
Rnewendstd

 propSimsNonEp % proportion of sims with non-epidemics
 propSimsNonEp_theoretical %theoretical value
    
R = betameanmean./gammameanmean
RN = betameanmeanNew./gammameanmeanNew
SRN = SbetameanmeanNew./SgammameanmeanNew
RTheoretical = beta./gamma

figro = figure
%plot(beta,R,'k')
hold on
plot(RN,'ko-')
plot(RTheoretical,'k.-')
plot(RN,'ks-') %stochastic measured
plot([1 1 1 1 1 1 1 1],'k--')
%plot([19 19],[0 2],'k--')
xlabel('Number of variants')
ylabel('ro')
%legend('R','RNew','Rtheoretical') 
legend('Ro Agent(measured)','Ro Deterministic','Ro Stochastic(measured)') 
%axis([0.1 0.5 0 2])
saveas(figro,[outputDirectory 'Test15_1EXPfigro.fig']);
saveas(figro,[outputDirectory 'Test15_1EXPfigro.emf']);
%close all
%close all;
% 2 EXPLOITS
% *************************************************************************************************
%Deterministic - 4 loci AND
tic
numNodes = 1000;
VRmin = 1 %25
VRmax = 8 %25
N=numNodes; Vmax = 1:VRmax;  G = Vmax.^2; %number genotypes
%for AND case
Sprop = sp4LociAND( 2, Vmax )
mode = 'static'
%monte carlo random genotype SIR model
numRuns = 20 %100 %200

for z= VRmin:VRmax
 betan = 0.02; variantRichness =  z;  numTimesteps = 7000; plotGraphs = 0; gamma = 0.004; numExploits=2; numLoci=4; casetype = 'AND'; throwAway = 'nonEpidemics', cutoff = gamma.*numNodes./(betan.*Sprop); SEpidemic=numNodes - 60; %cutoff;
%agent
 [fighistRecovered(z) fighistSusceptible(z) fighistpeakValue(z) fighistpeakTime(z) fighistBeta(z) fighistGamma(z) fighistBetanew(z) fighistGammanew(z) figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean(z) gammameanmean betameanmean2(z) betameanmeanNew(z) betameanmeanNew2 gammameanmeanNew(z) Smean(z,:) Imean(z,:) Rmean(z,:) NSmean(z,:) Smedian Imedian Rmedian S I R NS SmeanN ImeanN1(z,:) RmeanN1 Snewendmean Snewendstd Rnewendmean(z) Rnewendstd(z) Rnew_nonendmean Rnew_nonendstd propSimsNonEp(z) propSimsNonEp_theoretical(z)...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,mode, numLoci, casetype);
   size(ImeanN1)
FinalSize(z) = max(RmeanN1,[],2) %mean final size
    %resilience of major only
     maska = RmeanN1 >= FinalSize(z)-1; %when within 1 of the final value
    [C Index] = max(maska);
    MLa(z) = Index; %T1
    RmeanN1
    size(RmeanN1)
    size(Index)
        %resilience of all
     maska = Rmean(z,:) >= FinalSize(z)-1; %when within 1 of the final value
    [C Index] = max(maska);
        size(Rmean(z,:))
    size(Index)
    MLa_all(z) = Index; %T1
        saveas(fighistRecovered(z),[outputDirectory 'Test15_2EXPfighistRecovered' num2str(z) '.fig']);
    saveas(fighistRecovered(z),[outputDirectory 'Test15_2EXPfighistRecovered' num2str(z) '.emf']);
    saveas(fighistSusceptible(z),[outputDirectory 'Test15_2EXPfighistSusceptible' num2str(z) '.fig']);
    saveas(fighistSusceptible(z),[outputDirectory 'Test15_2EXPfighistSusceptible' num2str(z) '.emf']);
    saveas(fighistpeakValue(z),[outputDirectory 'Test15_2EXPfighistpeakValue' num2str(z) '.fig']);
    saveas(fighistpeakValue(z),[outputDirectory 'Test15_2EXPfighistpeakValue' num2str(z) '.emf']);
    saveas(fighistpeakTime(z),[outputDirectory 'Test15_2EXPfighistpeakTime' num2str(z) '.fig']);
    saveas(fighistpeakTime(z),[outputDirectory 'Test15_2EXPfighistpeakTime' num2str(z) '.emf']);
    saveas(fighistBeta(z),[outputDirectory 'Test15_2EXPhistBeta' num2str(z) '.fig']);
    saveas(fighistBeta(z),[outputDirectory 'Test15_2EXPhistBeta' num2str(z) '.emf']);
    saveas(fighistGamma(z),[outputDirectory 'Test15_2EXPhistGamma' num2str(z) '.fig']);
    saveas(fighistGamma(z),[outputDirectory 'Test15_2EXPhistGamma' num2str(z) '.emf']);
    saveas(fighistBetanew(z),[outputDirectory 'Test15_2EXPhistBetaNew' num2str(z) '.fig']);
    saveas(fighistBetanew(z),[outputDirectory 'Test15_2EXPhistBetaNew' num2str(z) '.emf']);
    saveas(fighistGammanew(z),[outputDirectory 'Test15_2EXPhistGammaNew' num2str(z) '.fig']);
    saveas(fighistGammanew(z),[outputDirectory 'Test15_2EXPhistGammaNew' num2str(z) '.emf']);
    close(fighistRecovered(z));
    close(fighistSusceptible(z));
    close(fighistpeakValue(z));
    close(fighistpeakTime(z));
    close(fighistBeta(z));
    close(fighistGamma(z));
    close(fighistBetanew(z));
    close(fighistGammanew(z));
    
    %stochastic
    [SfighistRecovered(z) SfighistSusceptible(z) SfighistpeakValue(z) SfighistpeakTime(z) SfighistBeta(z) SfighistGamma(z) SfighistBetanew(z) SfighistGammanew(z) SfigInfectedRun1 SfigBetaRun1 SfigGammaRun1 SfigRoRun1 SfigBetaGamma1 SfigBetaGamma2...
            Sbetastd Sgammastd Sbetastd2 SbetastdNew SbetastdNew2 SgammastdNew SpeakTimemean SpeakTimestd SpeakValuemean SpeakValuestd...
            Sbetameanmean(z) Sgammameanmean Sbetameanmean2(z) SbetameanmeanNew(z) SbetameanmeanNew2 SgammameanmeanNew(z) SSmean(z,:) SImean(z,:) SRmean(z,:) SNSmean(z,:) SSmedian SImedian SRmedian SS SI SR SNS SSmeanN SImeanN1(z,:) SRmeanN1 SSnewendmean SSnewendstd SRnewendmean(z) SRnewendstd(z) SRnew_nonendmean SRnew_nonendstd SpropSimsNonEp(z) SpropSimsNonEp_theoretical(z)...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,mode, numLoci, casetype);
     size(SImeanN1)
SFinalSize(z) = max(SRmeanN1,[],2) %mean final size
    %resilience of major only
     Smaska = SRmeanN1 >= SFinalSize(z)-1; %when within 1 of the final value
    [SC SIndex] = max(Smaska);
    SMLa(z) = SIndex; %T1
        %resilience of all
     Smaska = SRmean(z,:) >= SFinalSize(z)-1; %when within 1 of the final value
    [SC SIndex] = max(Smaska);
    SMLa_all(z) = SIndex; %T1
        saveas(SfighistRecovered(z),[outputDirectory 'Test15_2EXPStochfighistRecovered' num2str(z) '.fig']);
    saveas(SfighistRecovered(z),[outputDirectory 'Test15_2EXPStochfighistRecovered' num2str(z) '.emf']);
    saveas(SfighistSusceptible(z),[outputDirectory 'Test15_2EXPStochfighistSusceptible' num2str(z) '.fig']);
    saveas(SfighistSusceptible(z),[outputDirectory 'Test15_2EXPStochfighistSusceptible' num2str(z) '.emf']);
    saveas(SfighistpeakValue(z),[outputDirectory 'Test15_2EXPStochfighistpeakValue' num2str(z) '.fig']);
    saveas(SfighistpeakValue(z),[outputDirectory 'Test15_2EXPStochfighistpeakValue' num2str(z) '.emf']);
    saveas(SfighistpeakTime(z),[outputDirectory 'Test15_2EXPStochfighistpeakTime' num2str(z) '.fig']);
    saveas(SfighistpeakTime(z),[outputDirectory 'Test15_2EXPStochfighistpeakTime' num2str(z) '.emf']);
    saveas(SfighistBeta(z),[outputDirectory 'Test15_2EXPStochhistBeta' num2str(z) '.fig']);
    saveas(SfighistBeta(z),[outputDirectory 'Test15_2EXPStochhistBeta' num2str(z) '.emf']);
    saveas(SfighistGamma(z),[outputDirectory 'Test15_2EXPStochhistGamma' num2str(z) '.fig']);
    saveas(SfighistGamma(z),[outputDirectory 'Test15_2EXPStochhistGamma' num2str(z) '.emf']);
    saveas(SfighistBetanew(z),[outputDirectory 'Test15_2EXPStochhistBetaNew' num2str(z) '.fig']);
    saveas(SfighistBetanew(z),[outputDirectory 'Test15_2EXPStochhistBetaNew' num2str(z) '.emf']);
    saveas(SfighistGammanew(z),[outputDirectory 'Test15_2EXPStochhistGammaNew' num2str(z) '.fig']);
    saveas(SfighistGammanew(z),[outputDirectory 'Test15_2EXPStochhistGammaNew' num2str(z) '.emf']);
    close(SfighistRecovered(z));
    close(SfighistSusceptible(z));
    close(SfighistpeakValue(z));
    close(SfighistpeakTime(z));
    close(SfighistBeta(z));
    close(SfighistGamma(z));
    close(SfighistBetanew(z));
    close(SfighistGammanew(z));
    text = ['done ' num2str(z) ' out of ' num2str(VRmax)];
    disp(text)
end    
    
   
%Deterministic - 4 loci AND
N=numNodes; Vmax = 1:VRmax;  G = Vmax.^2; %number genotypes
%for AND case
Sprop = sp4LociAND( 2, Vmax )
%Sprop = 2./Vmax - 1./G %both loci have the same number of variants and both have 1 exploit
S = N.*Sprop; B = 0.02; beta = B.*Sprop; gamma = 0.004; %1/48*60*60 %2 days
for i = VRmin:VRmax
    [Ro(i) St(i,:) It(i,:) Rt(i,:)] = SIRDeterministic(beta(i), gamma, S(i), 1, 0, numTimesteps, 0); 
    FinalSized(i) = Rt(i,end);
     
    %resilience
     maskd = Rt(i,:) >= Rt(i,end)-1; %when within 1 of the final value
    [C Index] = max(maskd);
    MLd(i) = Index; %T1
end

dec = 50; t2 = 1:dec:numTimesteps;

%stochastic final size approximation

for h = VRmin:VRmax
  text = ['1-x = exp(-1.0*(' num2str(Sprop(h)*betan/gamma) ')*x)'] 
meanRTheory =  (solve(text, 'Real', true))*Sprop(h)*numNodes
FinalSizeS(h) = max(double(meanRTheory))
end

 
 figTest9inf0_02 = figure
 for i = VRmin:VRmax
     plot(Imean(i,:),'k')
     hold on
     plot(t2,It(i,1:dec:end),'k.-')
      plot(SImean(i,:),'k--')
 end
 ylabel('Number of nodes infected - It')
 xlabel('Timesteps')
 title('a) SI-Increasing variants: Nodes infected')
 legend('Agent', 'Deterministic', 'Stochastic')
 %axis([0 1000 0 400])
 
  figmajor = figure
 for i = VRmin:VRmax
     plot(ImeanN1(1,:),'k')
     hold on
     plot(t2,It(i,1:dec:end),'k.-')
     plot(SImeanN1(i,:),'k--')
     plot(ImeanN1(i,:),'k')
 end
xlabel('Timesteps')
ylabel('Number of individuals infected I(t)')
title('Mean of major outbreaks')
 legend('Agent', 'Deterministic', 'Stochastic')
%axis([0 100 0 90])
 
 
 %number of nodes infected
 figTest2FinalInfected = figure
 %agentImax = max(Imean,[],2);
 %plot(agentImax,'k')
 for j = VRmin:VRmax
 if isnan(FinalSizeS(j)); h(j) = 0; else h(j) = FinalSizeS(j); end;
 if isnan(FinalSize(j)); q(j) = 0; else q(j) = FinalSize(j); end;
 end
 FinalSize=q
 FinalSizeS=h
 plot(FinalSize,'ko-')
 hold on
 %plot(S,'k.-')
 plot(FinalSized,'k.-')
 plot(FinalSizeS,'k+-')
%plot([1.8533 1.8533],[0 1],'k--')
 ylabel('Final number infected')
 xlabel('Number of variants')
 title('c) SI-Increasing variants: Final size')
 legend('Agent', 'Deterministic', 'Stochastic Approximation')
 
  %RESISTANCE
 figTest2Resistance = figure
 %agentImax = max(Imean,[],2);
 %plot(agentImax,'k')
 for j = VRmin:VRmax
 %if isnan(FinalSizeS(j)); h(j) = 0; else h(j) = FinalSizeS(j); end; %STOCHASTIC APPROX
 %if isnan(FinalSize(j)); q(j) = 0; else q(j) = FinalSize(j); end; %AGENT
 if isnan(SFinalSize(j)); w(j) = 0; else w(j) = SFinalSize(j); end; %STOCHASTIC MEASURED
 end
 FinalSize=q
 FinalSizeS=h
 SFinalSize=w
 
      %SAVE FOR LATER
Resistancea_2EXP=1-FinalSize./N;
Resistanced_2EXP=1-FinalSized./N;
Resistancesapp_2EXP=1-FinalSizeS./N;
Resistances_2EXP=1-SFinalSize./N;
Resiliencea_2EXP=1./MLa;
Resilienced_2EXP=1./MLd;
Resiliences_2EXP=1./SMLa;


 plot(2:VRmax,Resistancea_2EXP(2:end),'ko-')
 hold on
 %plot(S,'k.-')
 plot(2:VRmax,Resistanced_2EXP(2:end),'k.-')
 plot(2:VRmax,Resistancesapp_2EXP(2:end),'k+-')
  plot(2:VRmax,Resistances_2EXP(2:end),'ks-')
 %plot([1.8533 1.8533],[0 1],'k--')
 ylabel('Resistance M_R')
 xlabel('Variant Richness')
 title('c) SI-Increasing variants: Resistance')
 legend('measured Agent', ' measured Deterministic', 'approximated Stochastic','measured Stochastic')
  axis([1 8 0 1.1])
 
 %RESILIENCE major outbreaks only
 figTest2Resilience = figure
  plot(2:VRmax,Resiliencea_2EXP(2:end),'ko-') %agent measured
 hold on
 plot(2:VRmax,Resilienced_2EXP(2:end),'k.-') %deterministic
  plot(2:VRmax,Resiliences_2EXP(2:end),'ks-') %stochastic measured
 hold on
  ylabel('Resilience M_L')
 xlabel('Variant Richness')
 title('c) SI-Increasing variants: Resilience major')
 legend('measured Agent', ' measured Deterministic', 'measured Stochastic')
  %plot([1.8533 1.8533],[0 1],'k--')
 axis([1 8 0 1.1])

  
  %RESILIENCE all
 figTest2Resilience2 = figure
  plot(1./MLa_all,'ko-') %agent measured
 hold on
 plot(1./MLd,'k.-') %deterministic
  plot(1./SMLa_all,'ks-') %stochastic measured
 hold on
   plot(1./MLa,'ro-') %agent major only
    plot(1./SMLa,'rs-') %stochastic major only
  ylabel('Resilience M_L')
 xlabel('Variant Richness')
 title('c) SI-Increasing variants: Resilience all')
 legend('measured Agent - all', ' measured Deterministic', 'measured Stochastic - all','measured Agent - major','measured Stochastic - major')
  %plot([1.8533 1.8533],[0 1],'k--')
  axis([1 8 0 1.1])
saveas(figTest9inf0_02,[outputDirectory 'Test15_2EXPmeanall.fig']);
saveas(figTest9inf0_02,[outputDirectory 'Test15_2EXPmeanall.emf']); 
saveas(figmajor,[outputDirectory 'Test15_2EXPmeanmajor.fig']);
saveas(figmajor,[outputDirectory 'Test15_2EXPmeanmajor.emf']);
saveas(figTest2FinalInfected,[outputDirectory 'Test15_2EXPFinalInfected.fig']);
saveas(figTest2FinalInfected,[outputDirectory 'Test15_2EXPFinalInfected.emf']);
saveas(figTest2Resistance,[outputDirectory 'Test15_2EXPResistance.fig']);
saveas(figTest2Resistance,[outputDirectory 'Test15_2EXPResistance.emf']);
saveas(figTest2Resilience,[outputDirectory 'Test15_2EXPResilience.fig']);
saveas(figTest2Resilience,[outputDirectory 'Test15_2EXPResilience.emf']);
saveas(figTest2Resilience2,[outputDirectory 'Test15_2EXPResilienceAll.fig']);
saveas(figTest2Resilience2,[outputDirectory 'Test15_2EXPResilienceAll.emf']);
%close all


betameanmean
gammameanmean

betameanmean2
betameanmeanNew 
betameanmeanNew2
gammameanmeanNew

Rnewendmean%epidemics only
Rnewendstd

 propSimsNonEp % proportion of sims with non-epidemics
 propSimsNonEp_theoretical %theoretical value
    
R = betameanmean./gammameanmean
RN = betameanmeanNew./gammameanmeanNew
SRN = SbetameanmeanNew./SgammameanmeanNew
RTheoretical = beta./gamma

figro = figure
%plot(beta,R,'k')
hold on
plot(RN,'ko-')
plot(RTheoretical,'k.-')
plot(RN,'ks-') %stochastic measured
plot([1 1 1 1 1 1 1 1],'k--')
%plot([19 19],[0 2],'k--')
xlabel('Number of variants')
ylabel('ro')
%legend('R','RNew','Rtheoretical') 
legend('Ro Agent(measured)','Ro Deterministic','Ro Stochastic(measured)') 
%axis([0.1 0.5 0 2])
saveas(figro,[outputDirectory 'Test15_2EXPfigro.fig']);
saveas(figro,[outputDirectory 'Test15_2EXPfigro.emf']);
%close all
%close all;

% 3 EXPLOITS *************************************************************
%Deterministic - 4 loci AND
tic
numNodes = 1000;
VRmin = 1 %25
VRmax = 8 %25
N=numNodes; Vmax = 1:VRmax;  G = Vmax.^2; %number genotypes
%for AND case
Sprop = sp4LociAND( 3, Vmax )
mode = 'static'
%monte carlo random genotype SIR model
numRuns = 20 %100 %200

for z= VRmin:VRmax
 betan = 0.02; variantRichness =  z;  numTimesteps = 7000; plotGraphs = 0; gamma = 0.004; numExploits=3; numLoci=4; casetype = 'AND'; throwAway = 'nonEpidemics', cutoff = gamma.*numNodes./(betan.*Sprop); SEpidemic=numNodes - 60; %cutoff;
%agent
 [fighistRecovered(z) fighistSusceptible(z) fighistpeakValue(z) fighistpeakTime(z) fighistBeta(z) fighistGamma(z) fighistBetanew(z) fighistGammanew(z) figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean(z) gammameanmean betameanmean2(z) betameanmeanNew(z) betameanmeanNew2 gammameanmeanNew(z) Smean(z,:) Imean(z,:) Rmean(z,:) NSmean(z,:) Smedian Imedian Rmedian S I R NS SmeanN ImeanN1(z,:) RmeanN1 Snewendmean Snewendstd Rnewendmean(z) Rnewendstd(z) Rnew_nonendmean Rnew_nonendstd propSimsNonEp(z) propSimsNonEp_theoretical(z)...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,mode, numLoci, casetype);
   size(ImeanN1)
FinalSize(z) = max(RmeanN1,[],2) %mean final size
    %resilience of major only
     maska = RmeanN1 >= FinalSize(z)-1; %when within 1 of the final value
    [C Index] = max(maska);
    MLa(z) = Index; %T1
    RmeanN1
    size(RmeanN1)
    size(Index)
        %resilience of all
     maska = Rmean(z,:) >= FinalSize(z)-1; %when within 1 of the final value
    [C Index] = max(maska);
        size(Rmean(z,:))
    size(Index)
    MLa_all(z) = Index; %T1
        saveas(fighistRecovered(z),[outputDirectory 'Test15_3EXPfighistRecovered' num2str(z) '.fig']);
    saveas(fighistRecovered(z),[outputDirectory 'Test15_3EXPfighistRecovered' num2str(z) '.emf']);
    saveas(fighistSusceptible(z),[outputDirectory 'Test15_3EXPfighistSusceptible' num2str(z) '.fig']);
    saveas(fighistSusceptible(z),[outputDirectory 'Test15_3EXPfighistSusceptible' num2str(z) '.emf']);
    saveas(fighistpeakValue(z),[outputDirectory 'Test15_3EXPfighistpeakValue' num2str(z) '.fig']);
    saveas(fighistpeakValue(z),[outputDirectory 'Test15_3EXPfighistpeakValue' num2str(z) '.emf']);
    saveas(fighistpeakTime(z),[outputDirectory 'Test15_3EXPfighistpeakTime' num2str(z) '.fig']);
    saveas(fighistpeakTime(z),[outputDirectory 'Test15_3EXPfighistpeakTime' num2str(z) '.emf']);
    saveas(fighistBeta(z),[outputDirectory 'Test15_3EXPhistBeta' num2str(z) '.fig']);
    saveas(fighistBeta(z),[outputDirectory 'Test15_3EXPhistBeta' num2str(z) '.emf']);
    saveas(fighistGamma(z),[outputDirectory 'Test15_3EXPhistGamma' num2str(z) '.fig']);
    saveas(fighistGamma(z),[outputDirectory 'Test15_3EXPhistGamma' num2str(z) '.emf']);
    saveas(fighistBetanew(z),[outputDirectory 'Test15_3EXPhistBetaNew' num2str(z) '.fig']);
    saveas(fighistBetanew(z),[outputDirectory 'Test15_3EXPhistBetaNew' num2str(z) '.emf']);
    saveas(fighistGammanew(z),[outputDirectory 'Test15_3EXPhistGammaNew' num2str(z) '.fig']);
    saveas(fighistGammanew(z),[outputDirectory 'Test15_3EXPhistGammaNew' num2str(z) '.emf']);
    close(fighistRecovered(z));
    close(fighistSusceptible(z));
    close(fighistpeakValue(z));
    close(fighistpeakTime(z));
    close(fighistBeta(z));
    close(fighistGamma(z));
    close(fighistBetanew(z));
    close(fighistGammanew(z));
    
    %stochastic
    [SfighistRecovered(z) SfighistSusceptible(z) SfighistpeakValue(z) SfighistpeakTime(z) SfighistBeta(z) SfighistGamma(z) SfighistBetanew(z) SfighistGammanew(z) SfigInfectedRun1 SfigBetaRun1 SfigGammaRun1 SfigRoRun1 SfigBetaGamma1 SfigBetaGamma2...
            Sbetastd Sgammastd Sbetastd2 SbetastdNew SbetastdNew2 SgammastdNew SpeakTimemean SpeakTimestd SpeakValuemean SpeakValuestd...
            Sbetameanmean(z) Sgammameanmean Sbetameanmean2(z) SbetameanmeanNew(z) SbetameanmeanNew2 SgammameanmeanNew(z) SSmean(z,:) SImean(z,:) SRmean(z,:) SNSmean(z,:) SSmedian SImedian SRmedian SS SI SR SNS SSmeanN SImeanN1(z,:) SRmeanN1 SSnewendmean SSnewendstd SRnewendmean(z) SRnewendstd(z) SRnew_nonendmean SRnew_nonendstd SpropSimsNonEp(z) SpropSimsNonEp_theoretical(z)...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,mode, numLoci, casetype);
     size(SImeanN1)
SFinalSize(z) = max(SRmeanN1,[],2) %mean final size
    %resilience of major only
     Smaska = SRmeanN1 >= SFinalSize(z)-1; %when within 1 of the final value
    [SC SIndex] = max(Smaska);
    SMLa(z) = SIndex; %T1
        %resilience of all
     Smaska = SRmean(z,:) >= SFinalSize(z)-1; %when within 1 of the final value
    [SC SIndex] = max(Smaska);
    SMLa_all(z) = SIndex; %T1
        saveas(SfighistRecovered(z),[outputDirectory 'Test15_3EXPStochfighistRecovered' num2str(z) '.fig']);
    saveas(SfighistRecovered(z),[outputDirectory 'Test15_3EXPStochfighistRecovered' num2str(z) '.emf']);
    saveas(SfighistSusceptible(z),[outputDirectory 'Test15_3EXPStochfighistSusceptible' num2str(z) '.fig']);
    saveas(SfighistSusceptible(z),[outputDirectory 'Test15_3EXPStochfighistSusceptible' num2str(z) '.emf']);
    saveas(SfighistpeakValue(z),[outputDirectory 'Test15_3EXPStochfighistpeakValue' num2str(z) '.fig']);
    saveas(SfighistpeakValue(z),[outputDirectory 'Test15_3EXPStochfighistpeakValue' num2str(z) '.emf']);
    saveas(SfighistpeakTime(z),[outputDirectory 'Test15_3EXPStochfighistpeakTime' num2str(z) '.fig']);
    saveas(SfighistpeakTime(z),[outputDirectory 'Test15_3EXPStochfighistpeakTime' num2str(z) '.emf']);
    saveas(SfighistBeta(z),[outputDirectory 'Test15_3EXPStochhistBeta' num2str(z) '.fig']);
    saveas(SfighistBeta(z),[outputDirectory 'Test15_3EXPStochhistBeta' num2str(z) '.emf']);
    saveas(SfighistGamma(z),[outputDirectory 'Test15_3EXPStochhistGamma' num2str(z) '.fig']);
    saveas(SfighistGamma(z),[outputDirectory 'Test15_3EXPStochhistGamma' num2str(z) '.emf']);
    saveas(SfighistBetanew(z),[outputDirectory 'Test15_3EXPStochhistBetaNew' num2str(z) '.fig']);
    saveas(SfighistBetanew(z),[outputDirectory 'Test15_3EXPStochhistBetaNew' num2str(z) '.emf']);
    saveas(SfighistGammanew(z),[outputDirectory 'Test15_3EXPStochhistGammaNew' num2str(z) '.fig']);
    saveas(SfighistGammanew(z),[outputDirectory 'Test15_3EXPStochhistGammaNew' num2str(z) '.emf']);
    close(SfighistRecovered(z));
    close(SfighistSusceptible(z));
    close(SfighistpeakValue(z));
    close(SfighistpeakTime(z));
    close(SfighistBeta(z));
    close(SfighistGamma(z));
    close(SfighistBetanew(z));
    close(SfighistGammanew(z));
    text = ['done ' num2str(z) ' out of ' num2str(VRmax)];
    disp(text)
end    
    
   
%Deterministic - 4 loci AND
N=numNodes; Vmax = 1:VRmax;  G = Vmax.^2; %number genotypes
%for AND case
Sprop = sp4LociAND( 3, Vmax )
%Sprop = 2./Vmax - 1./G %both loci have the same number of variants and both have 1 exploit
S = N.*Sprop; B = 0.02; beta = B.*Sprop; gamma = 0.004; %1/48*60*60 %2 days
for i = VRmin:VRmax
    [Ro(i) St(i,:) It(i,:) Rt(i,:)] = SIRDeterministic(beta(i), gamma, S(i), 1, 0, numTimesteps, 0); 
    FinalSized(i) = Rt(i,end);
     
    %resilience
     maskd = Rt(i,:) >= Rt(i,end)-1; %when within 1 of the final value
    [C Index] = max(maskd);
    MLd(i) = Index; %T1
end

dec = 50; t2 = 1:dec:numTimesteps;

%stochastic final size approximation

for h = VRmin:VRmax
  text = ['1-x = exp(-1.0*(' num2str(Sprop(h)*betan/gamma) ')*x)'] 
meanRTheory =  (solve(text, 'Real', true))*Sprop(h)*numNodes
FinalSizeS(h) = max(double(meanRTheory))
end

 
 figTest9inf0_02 = figure
 for i = VRmin:VRmax
     plot(Imean(i,:),'k')
     hold on
     plot(t2,It(i,1:dec:end),'k.-')
      plot(SImean(i,:),'k--')
 end
 ylabel('Number of nodes infected - It')
 xlabel('Timesteps')
 title('a) SI-Increasing variants: Nodes infected')
 legend('Agent', 'Deterministic', 'Stochastic')
 %axis([0 1000 0 400])
 
  figmajor = figure
 for i = VRmin:VRmax
     plot(ImeanN1(1,:),'k')
     hold on
     plot(t2,It(i,1:dec:end),'k.-')
     plot(SImeanN1(i,:),'k--')
     plot(ImeanN1(i,:),'k')
 end
xlabel('Timesteps')
ylabel('Number of individuals infected I(t)')
title('Mean of major outbreaks')
 legend('Agent', 'Deterministic', 'Stochastic')
%axis([0 100 0 90])
 
 
 %number of nodes infected
 figTest2FinalInfected = figure
 %agentImax = max(Imean,[],2);
 %plot(agentImax,'k')
 for j = VRmin:VRmax
 if isnan(FinalSizeS(j)); h(j) = 0; else h(j) = FinalSizeS(j); end;
 if isnan(FinalSize(j)); q(j) = 0; else q(j) = FinalSize(j); end;
 end
 FinalSize=q
 FinalSizeS=h
 plot(FinalSize,'ko-')
 hold on
 %plot(S,'k.-')
 plot(FinalSized,'k.-')
 plot(FinalSizeS,'k+-')
%plot([1.8533 1.8533],[0 1],'k--')
 ylabel('Final number infected')
 xlabel('Number of variants')
 title('c) SI-Increasing variants: Final size')
 legend('Agent', 'Deterministic', 'Stochastic Approximation')
 
  %RESISTANCE
 figTest2Resistance = figure
 %agentImax = max(Imean,[],2);
 %plot(agentImax,'k')
 for j = VRmin:VRmax
 %if isnan(FinalSizeS(j)); h(j) = 0; else h(j) = FinalSizeS(j); end; %STOCHASTIC APPROX
 %if isnan(FinalSize(j)); q(j) = 0; else q(j) = FinalSize(j); end; %AGENT
 if isnan(SFinalSize(j)); w(j) = 0; else w(j) = SFinalSize(j); end; %STOCHASTIC MEASURED
 end
 FinalSize=q
 FinalSizeS=h
 SFinalSize=w
 
      %SAVE FOR LATER
Resistancea_3EXP=1-FinalSize./N;
Resistanced_3EXP=1-FinalSized./N;
Resistancesapp_3EXP=1-FinalSizeS./N;
Resistances_3EXP=1-SFinalSize./N;
Resiliencea_3EXP=1./MLa;
Resilienced_3EXP=1./MLd;
Resiliences_3EXP=1./SMLa;
 
 plot(3:VRmax,Resistancea_3EXP(3:end),'ko-')
 hold on
 %plot(S,'k.-')
 plot(3:VRmax,Resistanced_3EXP(3:end),'k.-')
 plot(3:VRmax,Resistancesapp_3EXP(3:end),'k+-')
  plot(3:VRmax,Resistances_3EXP(3:end),'ks-')
 plot([1.8533 1.8533],[0 1],'k--')
 ylabel('Resistance M_R')
 xlabel('Variant Richness')
 title('c) SI-Increasing variants: Resistance')
 legend('measured Agent', ' measured Deterministic', 'approximated Stochastic','measured Stochastic')
  axis([1 8 0 1.1])
  
 %RESILIENCE major outbreaks only
 figTest2Resilience = figure
  plot(3:VRmax,Resiliencea_3EXP(3:end),'ko-') %agent measured
 hold on
 plot(3:VRmax,Resilienced_3EXP(3:end),'k.-') %deterministic
  plot(3:VRmax,Resiliences_3EXP(3:end),'ks-') %stochastic measured
 hold on
  ylabel('Resilience M_L')
 xlabel('Variant Richness')
 title('c) SI-Increasing variants: Resilience major')
 legend('measured Agent', ' measured Deterministic', 'measured Stochastic')
  %plot([1.8533 1.8533],[0 1],'k--')
   axis([1 8 0 1.1])


  %RESILIENCE all
 figTest2Resilience2 = figure
  plot(1./MLa_all,'ko-') %agent measured
 hold on
 plot(1./MLd,'k.-') %deterministic
  plot(1./SMLa_all,'ks-') %stochastic measured
 hold on
   plot(1./MLa,'ro-') %agent major only
    plot(1./SMLa,'rs-') %stochastic major only
  ylabel('Resilience M_L')
 xlabel('Variant Richness')
 title('c) SI-Increasing variants: Resilience all')
 legend('measured Agent - all', ' measured Deterministic', 'measured Stochastic - all','measured Agent - major','measured Stochastic - major')
  %plot([1.8533 1.8533],[0 1],'k--')
  axis([1 8 0 1.1])
  
saveas(figTest9inf0_02,[outputDirectory 'Test15_3EXPmeanall.fig']);
saveas(figTest9inf0_02,[outputDirectory 'Test15_3EXPmeanall.emf']); 
saveas(figmajor,[outputDirectory 'Test15_3EXPmeanmajor.fig']);
saveas(figmajor,[outputDirectory 'Test15_3EXPmeanmajor.emf']);
saveas(figTest2FinalInfected,[outputDirectory 'Test15_3EXPFinalInfected.fig']);
saveas(figTest2FinalInfected,[outputDirectory 'Test15_3EXPFinalInfected.emf']);
saveas(figTest2Resistance,[outputDirectory 'Test15_3EXPResistance.fig']);
saveas(figTest2Resistance,[outputDirectory 'Test15_3EXPResistance.emf']);
saveas(figTest2Resilience,[outputDirectory 'Test15_3EXPResilience.fig']);
saveas(figTest2Resilience,[outputDirectory 'Test15_3EXPResilience.emf']);
saveas(figTest2Resilience2,[outputDirectory 'Test15_3EXPResilienceAll.fig']);
saveas(figTest2Resilience2,[outputDirectory 'Test15_3EXPResilienceAll.emf']);
%close all


betameanmean
gammameanmean

betameanmean2
betameanmeanNew 
betameanmeanNew2
gammameanmeanNew

Rnewendmean%epidemics only
Rnewendstd

 propSimsNonEp % proportion of sims with non-epidemics
 propSimsNonEp_theoretical %theoretical value
    
R = betameanmean./gammameanmean
RN = betameanmeanNew./gammameanmeanNew
SRN = SbetameanmeanNew./SgammameanmeanNew
RTheoretical = beta./gamma

figro = figure
%plot(beta,R,'k')
hold on
plot(RN,'ko-')
plot(RTheoretical,'k.-')
plot(RN,'ks-') %stochastic measured
plot([1 1 1 1 1 1 1 1],'k--')
%plot([19 19],[0 2],'k--')
xlabel('Number of variants')
ylabel('ro')
%legend('R','RNew','Rtheoretical') 
legend('Ro Agent(measured)','Ro Deterministic','Ro Stochastic(measured)') 
%axis([0.1 0.5 0 2])
saveas(figro,[outputDirectory 'Test15_3EXPfigro.fig']);
saveas(figro,[outputDirectory 'Test15_3EXPfigro.emf']);
%close all
%close all;
%COMBINED EXPLOIT GRAPHS

  %RESISTANCE
 figTest2ResistanceAllExploits = figure

  plot(Resistancea_1EXP,'ko-','linewidth',2)
 hold on
 plot(Resistanced_1EXP,'k.-','linewidth',2)
 plot(Resistancesapp_1EXP,'k+-','linewidth',2)
plot(Resistances_1EXP,'ks-','linewidth',2)

  plot(2:VRmax,Resistancea_2EXP(2:end),'ko-')
 plot(2:VRmax,Resistanced_2EXP(2:end),'k.-')
 plot(2:VRmax,Resistancesapp_2EXP(2:end),'k+-')
plot(2:VRmax,Resistances_2EXP(2:end),'ks-')

 plot(3:VRmax,Resistancea_3EXP(3:end),'ko-')
 plot(3:VRmax,Resistanced_3EXP(3:end),'k.-')
 plot(3:VRmax,Resistancesapp_3EXP(3:end),'k+-')
plot(3:VRmax,Resistances_3EXP(3:end),'ks-')

 plot([1.8533 1.8533],[0 1],'k--')
 ylabel('Resistance M_R')
 xlabel('Variant Richness')
 title('c) SI-Increasing variants: Resistance')
 legend('measured Agent', ' measured Deterministic', 'approximated Stochastic','measured Stochastic')
 axis([1 8 0 1.1])
 
 %RESILIENCE major outbreaks only
 figTest2ResilienceAllExploits = figure
 plot(Resiliencea_1EXP,'ko-','linewidth',2) %agent
 hold on
 plot(Resilienced_1EXP,'k.-','linewidth',2) %deterministic
plot(Resiliences_1EXP,'ks-','linewidth',2) %stochastic measured

 plot(2:VRmax,Resiliencea_2EXP(2:end),'ko-') %agent
 plot(2:VRmax,Resilienced_2EXP(2:end),'k.-') %deterministic
plot(2:VRmax,Resiliences_2EXP(2:end),'ks-') %stochastic measured

 plot(3:VRmax,Resiliencea_3EXP(3:end),'ko-') %agent
 plot(3:VRmax,Resilienced_3EXP(3:end),'k.-') %deterministic
plot(3:VRmax,Resiliences_3EXP(3:end),'ks-') %stochastic measured
  ylabel('Resilience M_L')
 xlabel('Variant Richness')
 title('c) SI-Increasing variants: Resilience major')
 legend('measured Agent', ' measured Deterministic', 'measured Stochastic')
  plot([1.8533 1.8533],[0 1],'k--')
 axis([1 8 0 1.1])
  
  saveas(figTest2ResistanceAllExploits,[outputDirectory 'Test15_ALLEXPResistenceAll.fig']);
saveas(figTest2ResistanceAllExploits,[outputDirectory 'Test15_ALLEXPResistenceAll.emf']);
saveas(figTest2ResilienceAllExploits,[outputDirectory 'Test15_ALLEXPResilienceAll.fig']);
saveas(figTest2ResilienceAllExploits,[outputDirectory 'Test15_ALLEXPResilienceAll.emf']);

toc

%saved data
Resistancea_1EXP
Resistanced_1EXP
Resistancesapp_1EXP
Resistances_1EXP

Resistancea_2EXP
Resistanced_2EXP
Resistancesapp_2EXP
Resistances_2EXP

Resistancea_3EXP
Resistanced_3EXP
Resistancesapp_3EXP
Resistances_3EXP

Resiliencea_1EXP
Resilienced_1EXP
Resiliences_1EXP

Resiliencea_2EXP
Resilienced_2EXP
Resiliences_2EXP

Resiliencea_3EXP
Resilienced_3EXP
Resiliences_3EXP

end %SIRN Test15



%**************************************************************************
%STUFF TO PROCESS SIRN TEST 8
%****************
if doSIRNTests(19) == 1
 %*************************************
%R0 and gamma tests
%*************************************
%*************************************
%SIR version TEST11
%*************************************
%monte carlo random genotype SIR model
%1 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  1;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1;
[betameanmean(1) Smean(1,:) Imean(1,:) Rmean(1,:) NSmean(1,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%2 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  2;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1;
[betameanmean(2) Smean(2,:) Imean(2,:) Rmean(2,:) NSmean(2,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%3 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  3;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1;
[betameanmean(3) Smean(3,:) Imean(3,:) Rmean(3,:) NSmean(3,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%4 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  4;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1;
[betameanmean(4) Smean(4,:) Imean(4,:) Rmean(4,:) NSmean(4,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%5 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  5;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1;
[betameanmean(5) Smean(5,:) Imean(5,:) Rmean(5,:) NSmean(5,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%6 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  6;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1;
[betameanmean(6) Smean(6,:) Imean(6,:) Rmean(6,:) NSmean(6,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%7 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  7;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1;
[betameanmean(7) Smean(7,:) Imean(7,:) Rmean(7,:) NSmean(7,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%8 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1;
[betameanmean(8) Smean(8,:) Imean(8,:) Rmean(8,:) NSmean(8,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
betameanmeanD=betameanmean
 figTest11beta0_02 = figure
 plot(betameanmean,'k')
 ylabel('Measured beta')
 xlabel('Number of variants')
 title('Increasing variants')
 
 figTest11inf0_02 = figure
 for i = 1:8
     plot(Imean(i,:),'k')
     hold on
 end
 ylabel('Number of nodes infected - It')
 xlabel('Timesteps')
 title('Increasing variants')
 
 r0 = betameanmean./gamma
 r1 = 1
r1rep = repmat(r1,size(r0));

 figTest11r00_02 = figure
 plot(r0,'k')
 hold on
 plot(r1rep, 'k--')
 ylabel('r0')
 xlabel('Number of variants')
 title('Increasing variants')
 
 saveas(figTest11beta0_02,[outputDirectory 'figTest11beta0_02.fig']);
saveas(figTest11beta0_02,[outputDirectory 'figTest11beta0_02.emf']);
saveas(figTest11inf0_02,[outputDirectory 'figTest11inf0_02.fig']);
saveas(figTest11inf0_02,[outputDirectory 'figTest11inf0_02.emf']); 
saveas(figTest11r00_02,[outputDirectory 'figTest11r00_02.fig']);
saveas(figTest11r00_02,[outputDirectory 'figTest11r00_02.emf']); 
 
%*************************************
%Exploit tests TEST 12
%*************************************
%*************************************
%SIR version
%*************************************
%monte carlo random genotype SIR model
%1 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1;
[betameanmean(1) Smean(1,:) Imean(1,:) Rmean(1,:) NSmean(1,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%2 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=2;
[betameanmean(2) Smean(2,:) Imean(2,:) Rmean(2,:) NSmean(2,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%3 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=3;
[betameanmean(3) Smean(3,:) Imean(3,:) Rmean(3,:) NSmean(3,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%4 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=4;
[betameanmean(4) Smean(4,:) Imean(4,:) Rmean(4,:) NSmean(4,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%5 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=5;
[betameanmean(5) Smean(5,:) Imean(5,:) Rmean(5,:) NSmean(5,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%6 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=6;
[betameanmean(6) Smean(6,:) Imean(6,:) Rmean(6,:) NSmean(6,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%7 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=7;
[betameanmean(7) Smean(7,:) Imean(7,:) Rmean(7,:) NSmean(7,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%8 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=8;
[betameanmean(8) Smean(8,:) Imean(8,:) Rmean(8,:) NSmean(8,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
 
betameanmeanD=betameanmean
 figTest12beta0_02 = figure
 plot(betameanmean,'k')
 ylabel('Measured beta')
 xlabel('Number of exploits')
 title('Increasing exploits')

 figTest12inf0_02 = figure
 for i = 1:8
     plot(Imean(i,:),'k')
     hold on
 end
 ylabel('Number of nodes infected - It')
 xlabel('Timesteps')
 title('Increasing exploits')
 
  r0 = betameanmean./gamma
 r1 = 1
r1rep = repmat(r1,size(r0));
 figTest12r00_02 = figure
 plot(r0,'k')
 hold on
 plot(r1rep, 'k--')
 ylabel('r0')
 xlabel('Number of exploits')
 title('Increasing exploits')
 
saveas(figTest12beta0_02,[outputDirectory 'figTest12beta0_02.fig']);
saveas(figTest12beta0_02,[outputDirectory 'figTest12beta0_02.emf']);
saveas(figTest12inf0_02,[outputDirectory 'figTest12inf0_02.fig']);
saveas(figTest12inf0_02,[outputDirectory 'figTest12inf0_02.emf']); 
saveas(figTest12r00_02,[outputDirectory 'figTest12r00_02.fig']);
saveas(figTest12r00_02,[outputDirectory 'figTest12r00_02.emf']); 

mode = 'dynamic'
%*************************************
%Beta tests
%*************************************
%*************************************
%SIR version - TEST13
%*************************************
%monte carlo random genotype SIR model
%1 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  1;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=1;
[betameanmean(1) Smean(1,:) Imean(1,:) Rmean(1,:) NSmean(1,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%2 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  2;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=1;
[betameanmean(2) Smean(2,:) Imean(2,:) Rmean(2,:) NSmean(2,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%3 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  3;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=1;
[betameanmean(3) Smean(3,:) Imean(3,:) Rmean(3,:) NSmean(3,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%4 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  4;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=1;
[betameanmean(4) Smean(4,:) Imean(4,:) Rmean(4,:) NSmean(4,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%5 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  5;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=1;
[betameanmean(5) Smean(5,:) Imean(5,:) Rmean(5,:) NSmean(5,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%6 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  6;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=1;
[betameanmean(6) Smean(6,:) Imean(6,:) Rmean(6,:) NSmean(6,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%7 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  7;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=1;
[betameanmean(7) Smean(7,:) Imean(7,:) Rmean(7,:) NSmean(7,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%8 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=1;
[betameanmean(8) Smean(8,:) Imean(8,:) Rmean(8,:) NSmean(8,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
betameanmeanD=betameanmean
 figTest13beta0_02 = figure
 plot(betameanmean,'k')
 ylabel('Measured beta')
 xlabel('Number of variants')
 title('Increasing variants')
 
 figTest13inf0_02 = figure
 for i = 1:8
     plot(Imean(i,:),'k')
     hold on
 end
 ylabel('Number of nodes infected - It')
 xlabel('Timesteps')
 title('Increasing variants')
 
saveas(figTest13beta0_02,[outputDirectory 'figTest13beta0_02.fig']);
saveas(figTest13beta0_02,[outputDirectory 'figTest13beta0_02.emf']);
saveas(figTest13inf0_02,[outputDirectory 'figTest13inf0_02.fig']);
saveas(figTest13inf0_02,[outputDirectory 'figTest13inf0_02.emf']); 
 
%*************************************
%Exploit tests TEST14
%*************************************
%*************************************
%SIR version
%*************************************
%monte carlo random genotype SIR model
%1 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=1;
[betameanmean(1) Smean(1,:) Imean(1,:) Rmean(1,:) NSmean(1,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%2 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=2;
[betameanmean(2) Smean(2,:) Imean(2,:) Rmean(2,:) NSmean(2,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%3 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=3;
[betameanmean(3) Smean(3,:) Imean(3,:) Rmean(3,:) NSmean(3,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%4 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=4;
[betameanmean(4) Smean(4,:) Imean(4,:) Rmean(4,:) NSmean(4,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%5 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=5;
[betameanmean(5) Smean(5,:) Imean(5,:) Rmean(5,:) NSmean(5,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%6 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=6;
[betameanmean(6) Smean(6,:) Imean(6,:) Rmean(6,:) NSmean(6,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%7 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=7;
[betameanmean(7) Smean(7,:) Imean(7,:) Rmean(7,:) NSmean(7,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%8 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=8;
[betameanmean(8) Smean(8,:) Imean(8,:) Rmean(8,:) NSmean(8,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
 
betameanmeanD=betameanmean
  figTest14beta0_02 = figure
 plot(betameanmean,'k')
 ylabel('Measured beta')
 xlabel('Number of exploits')
 title('Increasing exploits')

  figTest14inf0_02 = figure
 for i = 1:8
     plot(Imean(i,:),'k')
     hold on
 end
 ylabel('Number of nodes infected - It')
 xlabel('Timesteps')
 title('Increasing exploits')
 
saveas(figTest14beta0_02,[outputDirectory 'figTest14beta0_02.fig']);
saveas(figTest14beta0_02,[outputDirectory 'figTest14beta0_02.emf']);
saveas(figTest14inf0_02,[outputDirectory 'figTest14inf0_02.fig']);
saveas(figTest14inf0_02,[outputDirectory 'figTest14inf0_02.emf']); 

 %*************************************
%R0 and gamma tests
%*************************************
%*************************************
%SIR version TEST15
%*************************************
%monte carlo random genotype SIR model
%1 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  1;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1;
[betameanmean(1) Smean(1,:) Imean(1,:) Rmean(1,:) NSmean(1,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%2 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  2;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1;
[betameanmean(2) Smean(2,:) Imean(2,:) Rmean(2,:) NSmean(2,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%3 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  3;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1;
[betameanmean(3) Smean(3,:) Imean(3,:) Rmean(3,:) NSmean(3,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%4 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  4;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1;
[betameanmean(4) Smean(4,:) Imean(4,:) Rmean(4,:) NSmean(4,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%5 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  5;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1;
[betameanmean(5) Smean(5,:) Imean(5,:) Rmean(5,:) NSmean(5,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%6 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  6;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1;
[betameanmean(6) Smean(6,:) Imean(6,:) Rmean(6,:) NSmean(6,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%7 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  7;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1;
[betameanmean(7) Smean(7,:) Imean(7,:) Rmean(7,:) NSmean(7,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%8 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1;
[betameanmean(8) Smean(8,:) Imean(8,:) Rmean(8,:) NSmean(8,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
betameanmeanD=betameanmean
 figTest15beta0_02 = figure
 plot(betameanmean,'k')
 ylabel('Measured beta')
 xlabel('Number of variants')
 title('Increasing variants')
 
 figTest15inf0_02 = figure
 for i = 1:8
     plot(Imean(i,:),'k')
     hold on
 end
 ylabel('Number of nodes infected - It')
 xlabel('Timesteps')
 title('Increasing variants')
 
 r0 = betameanmean./gamma
 r1 = 1
r1rep = repmat(r1,size(r0));

 figTest15r00_02 = figure
 plot(r0,'k')
 hold on
 plot(r1rep, 'k--')
 ylabel('r0')
 xlabel('Number of variants')
 title('Increasing variants')
 
 saveas(figTest15beta0_02,[outputDirectory 'figTest15beta0_02.fig']);
saveas(figTest15beta0_02,[outputDirectory 'figTest15beta0_02.emf']);
saveas(figTest15inf0_02,[outputDirectory 'figTest15inf0_02.fig']);
saveas(figTest15inf0_02,[outputDirectory 'figTest15inf0_02.emf']); 
saveas(figTest15r00_02,[outputDirectory 'figTest15r00_02.fig']);
saveas(figTest15r00_02,[outputDirectory 'figTest15r00_02.emf']); 
 
%*************************************
%Exploit tests TEST 16
%*************************************
%*************************************
%SIR version
%*************************************
%monte carlo random genotype SIR model
%1 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1;
[betameanmean(1) Smean(1,:) Imean(1,:) Rmean(1,:) NSmean(1,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%2 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=2;
[betameanmean(2) Smean(2,:) Imean(2,:) Rmean(2,:) NSmean(2,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%3 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=3;
[betameanmean(3) Smean(3,:) Imean(3,:) Rmean(3,:) NSmean(3,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%4 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=4;
[betameanmean(4) Smean(4,:) Imean(4,:) Rmean(4,:) NSmean(4,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%5 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=5;
[betameanmean(5) Smean(5,:) Imean(5,:) Rmean(5,:) NSmean(5,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%6 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=6;
[betameanmean(6) Smean(6,:) Imean(6,:) Rmean(6,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%7 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=7;
[betameanmean(7) Smean(7,:) Imean(7,:) Rmean(7,:) NSmean(7,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%8 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=8;
[betameanmean(8) Smean(8,:) Imean(8,:) Rmean(8,:) NSmean(8,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
 
betameanmeanD=betameanmean
 figTest16beta0_02 = figure
 plot(betameanmean,'k')
 ylabel('Measured beta')
 xlabel('Number of exploits')
 title('Increasing exploits')

 figTest16inf0_02 = figure
 for i = 1:8
     plot(Imean(i,:),'k')
     hold on
 end
 ylabel('Number of nodes infected - It')
 xlabel('Timesteps')
 title('Increasing exploits')
 
  r0 = betameanmean./gamma
 r1 = 1
r1rep = repmat(r1,size(r0));
 figTest16r00_02 = figure
 plot(r0,'k')
 hold on
 plot(r1rep, 'k--')
 ylabel('r0')
 xlabel('Number of exploits')
 title('Increasing exploits')
 
saveas(figTest16beta0_02,[outputDirectory 'figTest16beta0_02.fig']);
saveas(figTest16beta0_02,[outputDirectory 'figTest16beta0_02.emf']);
saveas(figTest16inf0_02,[outputDirectory 'figTest16inf0_02.fig']);
saveas(figTest16inf0_02,[outputDirectory 'figTest16inf0_02.emf']); 
saveas(figTest16r00_02,[outputDirectory 'figTest16r00_02.fig']);
saveas(figTest16r00_02,[outputDirectory 'figTest16r00_02.emf']); 



%*************************STOP HERE

mode = 'static'
%*************************************
%Beta tests
%*************************************
%*************************************
%SIR version - TEST5
%*************************************
%monte carlo random genotype SIR model
%1 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  1;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=1;
[betameanmean(1) Smean(1,:) Imean(1,:) Rmean(1,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%2 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  2;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=1;
[betameanmean(2) Smean(2,:) Imean(2,:) Rmean(2,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%3 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  3;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=1;
[betameanmean(3) Smean(3,:) Imean(3,:) Rmean(3,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%4 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  4;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=1;
[betameanmean(4) Smean(4,:) Imean(4,:) Rmean(4,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%5 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  5;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=1;
[betameanmean(5) Smean(5,:) Imean(5,:) Rmean(5,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%6 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  6;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=1;
[betameanmean(6) Smean(6,:) Imean(6,:) Rmean(6,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%7 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  7;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=1;
[betameanmean(7) Smean(7,:) Imean(7,:) Rmean(7,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%8 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=1;
[betameanmean(8) Smean(8,:) Imean(8,:) Rmean(8,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
betameanmeanD=betameanmean
 figTest5beta0_02 = figure
 plot(betameanmean,'k')
 ylabel('Measured beta')
 xlabel('Number of variants')
 title('Increasing variants')
 
 figTest5inf0_02 = figure
 for i = 1:8
     plot(Imean(i,:),'k')
     hold on
 end
 ylabel('Number of nodes infected - It')
 xlabel('Timesteps')
 title('Increasing variants')
 
saveas(figTest5beta0_02,[outputDirectory 'figTest5beta0_02.fig']);
saveas(figTest5beta0_02,[outputDirectory 'figTest5beta0_02.emf']);
saveas(figTest5inf0_02,[outputDirectory 'figTest5inf0_02.fig']);
saveas(figTest5inf0_02,[outputDirectory 'figTest5inf0_02.emf']); 
 
%*************************************
%Exploit tests TEST6
%*************************************
%*************************************
%SIR version
%*************************************
%monte carlo random genotype SIR model
%1 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=1;
[betameanmean(1) Smean(1,:) Imean(1,:) Rmean(1,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%2 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=2;
[betameanmean(2) Smean(2,:) Imean(2,:) Rmean(2,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%3 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=3;
[betameanmean(3) Smean(3,:) Imean(3,:) Rmean(3,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%4 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=4;
[betameanmean(4) Smean(4,:) Imean(4,:) Rmean(4,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%5 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=5;
[betameanmean(5) Smean(5,:) Imean(5,:) Rmean(5,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%6 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=6;
[betameanmean(6) Smean(6,:) Imean(6,:) Rmean(6,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%7 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=7;
[betameanmean(7) Smean(7,:) Imean(7,:) Rmean(7,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%8 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=8;
[betameanmean(8) Smean(8,:) Imean(8,:) Rmean(8,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
 
betameanmeanD=betameanmean
  figTest6beta0_02 = figure
 plot(betameanmean,'k')
 ylabel('Measured beta')
 xlabel('Number of exploits')
 title('Increasing exploits')

  figTest6inf0_02 = figure
 for i = 1:8
     plot(Imean(i,:),'k')
     hold on
 end
 ylabel('Number of nodes infected - It')
 xlabel('Timesteps')
 title('Increasing exploits')
 
saveas(figTest6beta0_02,[outputDirectory 'figTest6beta0_02.fig']);
saveas(figTest6beta0_02,[outputDirectory 'figTest6beta0_02.emf']);
saveas(figTest6inf0_02,[outputDirectory 'figTest6inf0_02.fig']);
saveas(figTest6inf0_02,[outputDirectory 'figTest6inf0_02.emf']); 

 %*************************************
%R0 and gamma tests
%*************************************
%*************************************
%SIR version TEST7
%*************************************
%monte carlo random genotype SIR model
%1 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  1;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1;
[betameanmean(1) Smean(1,:) Imean(1,:) Rmean(1,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%2 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  2;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1;
[betameanmean(2) Smean(2,:) Imean(2,:) Rmean(2,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%3 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  3;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1;
[betameanmean(3) Smean(3,:) Imean(3,:) Rmean(3,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%4 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  4;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1;
[betameanmean(4) Smean(4,:) Imean(4,:) Rmean(4,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%5 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  5;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1;
[betameanmean(5) Smean(5,:) Imean(5,:) Rmean(5,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%6 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  6;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1;
[betameanmean(6) Smean(6,:) Imean(6,:) Rmean(6,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%7 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  7;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1;
[betameanmean(7) Smean(7,:) Imean(7,:) Rmean(7,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%8 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1;
[betameanmean(8) Smean(8,:) Imean(8,:) Rmean(8,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
betameanmeanD=betameanmean
 figTest7beta0_02 = figure
 plot(betameanmean,'k')
 ylabel('Measured beta')
 xlabel('Number of variants')
 title('Increasing variants')
 
 figTest7inf0_02 = figure
 for i = 1:8
     plot(Imean(i,:),'k')
     hold on
 end
 ylabel('Number of nodes infected - It')
 xlabel('Timesteps')
 title('Increasing variants')
 
 r0 = betameanmean./gamma
 r1 = 1
r1rep = repmat(r1,size(r0));

 figTest7r00_02 = figure
 plot(r0,'k')
 hold on
 plot(r1rep, 'k--')
 ylabel('r0')
 xlabel('Number of variants')
 title('Increasing variants')
 
 saveas(figTest7beta0_02,[outputDirectory 'figTest7beta0_02.fig']);
saveas(figTest7beta0_02,[outputDirectory 'figTest7beta0_02.emf']);
saveas(figTest7inf0_02,[outputDirectory 'figTest7inf0_02.fig']);
saveas(figTest7inf0_02,[outputDirectory 'figTest7inf0_02.emf']); 
saveas(figTest7r00_02,[outputDirectory 'figTest7r00_02.fig']);
saveas(figTest7r00_02,[outputDirectory 'figTest7r00_02.emf']); 
 
%*************************************
%Exploit tests TEST 8
%*************************************
%*************************************
%SIR version
%*************************************
%monte carlo random genotype SIR model
%1 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1;
[betameanmean(1) Smean(1,:) Imean(1,:) Rmean(1,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%2 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=2;
[betameanmean(2) Smean(2,:) Imean(2,:) Rmean(2,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%3 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=3;
[betameanmean(3) Smean(3,:) Imean(3,:) Rmean(3,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%4 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=4;
[betameanmean(4) Smean(4,:) Imean(4,:) Rmean(4,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%5 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=5;
[betameanmean(5) Smean(5,:) Imean(5,:) Rmean(5,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%6 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=6;
[betameanmean(6) Smean(6,:) Imean(6,:) Rmean(6,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%7 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=7;
[betameanmean(7) Smean(7,:) Imean(7,:) Rmean(7,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%8 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=8;
[betameanmean(8) Smean(8,:) Imean(8,:) Rmean(8,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
 
betameanmeanD=betameanmean
 figTest8beta0_02 = figure
 plot(betameanmean,'k')
 ylabel('Measured beta')
 xlabel('Number of exploits')
 title('Increasing exploits')

 figTest8inf0_02 = figure
 for i = 1:8
     plot(Imean(i,:),'k')
     hold on
 end
 ylabel('Number of nodes infected - It')
 xlabel('Timesteps')
 title('Increasing exploits')
 
  r0 = betameanmean./gamma
 r1 = 1
r1rep = repmat(r1,size(r0));
 figTest8r00_02 = figure
 plot(r0,'k')
 hold on
 plot(r1rep, 'k--')
 ylabel('r0')
 xlabel('Number of exploits')
 title('Increasing exploits')
 
saveas(figTest8beta0_02,[outputDirectory 'figTest8beta0_02.fig']);
saveas(figTest8beta0_02,[outputDirectory 'figTest8beta0_02.emf']);
saveas(figTest8inf0_02,[outputDirectory 'figTest8inf0_02.fig']);
saveas(figTest8inf0_02,[outputDirectory 'figTest8inf0_02.emf']); 
saveas(figTest8r00_02,[outputDirectory 'figTest8r00_02.fig']);
saveas(figTest8r00_02,[outputDirectory 'figTest8r00_02.emf']); 
toc
return
mode = 'dynamic'
%*************************************
%Beta tests
%*************************************
%*************************************
%SIR version - TEST1
%*************************************
%monte carlo random genotype SIR model
%1 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  1;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=1;
[betameanmean(1) Smean(1,:) Imean(1,:) Rmean(1,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%2 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  2;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=1;
[betameanmean(2) Smean(2,:) Imean(2,:) Rmean(2,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%3 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  3;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=1;
[betameanmean(3) Smean(3,:) Imean(3,:) Rmean(3,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%4 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  4;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=1;
[betameanmean(4) Smean(4,:) Imean(4,:) Rmean(4,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%5 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  5;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=1;
[betameanmean(5) Smean(5,:) Imean(5,:) Rmean(5,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%6 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  6;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=1;
[betameanmean(6) Smean(6,:) Imean(6,:) Rmean(6,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%7 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  7;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=1;
[betameanmean(7) Smean(7,:) Imean(7,:) Rmean(7,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%8 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=1;
[betameanmean(8) Smean(8,:) Imean(8,:) Rmean(8,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
betameanmeanD=betameanmean
 figTest1beta0_02 = figure
 plot(betameanmean,'k')
 ylabel('Measured beta')
 xlabel('Number of variants')
 title('Increasing variants')
 
 figTest1inf0_02 = figure
 for i = 1:8
     plot(Imean(i,:),'k')
     hold on
 end
 ylabel('Number of nodes infected - It')
 xlabel('Timesteps')
 title('Increasing variants')
 
saveas(figTest1beta0_02,[outputDirectory 'figTest1beta0_02.fig']);
saveas(figTest1beta0_02,[outputDirectory 'figTest1beta0_02.emf']);
saveas(figTest1inf0_02,[outputDirectory 'figTest1inf0_02.fig']);
saveas(figTest1inf0_02,[outputDirectory 'figTest1inf0_02.emf']); 
 
%*************************************
%Exploit tests TEST2
%*************************************
%*************************************
%SIR version
%*************************************
%monte carlo random genotype SIR model
%1 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=1;
[betameanmean(1) Smean(1,:) Imean(1,:) Rmean(1,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%2 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=2;
[betameanmean(2) Smean(2,:) Imean(2,:) Rmean(2,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%3 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=3;
[betameanmean(3) Smean(3,:) Imean(3,:) Rmean(3,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%4 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=4;
[betameanmean(4) Smean(4,:) Imean(4,:) Rmean(4,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%5 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=5;
[betameanmean(5) Smean(5,:) Imean(5,:) Rmean(5,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%6 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=6;
[betameanmean(6) Smean(6,:) Imean(6,:) Rmean(6,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%7 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=7;
[betameanmean(7) Smean(7,:) Imean(7,:) Rmean(7,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%8 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0; numExploits=8;
[betameanmean(8) Smean(8,:) Imean(8,:) Rmean(8,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
 
betameanmeanD=betameanmean
  figTest2beta0_02 = figure
 plot(betameanmean,'k')
 ylabel('Measured beta')
 xlabel('Number of exploits')
 title('Increasing exploits')

  figTest2inf0_02 = figure
 for i = 1:8
     plot(Imean(i,:),'k')
     hold on
 end
 ylabel('Number of nodes infected - It')
 xlabel('Timesteps')
 title('Increasing exploits')
 
saveas(figTest2beta0_02,[outputDirectory 'figTest2beta0_02.fig']);
saveas(figTest2beta0_02,[outputDirectory 'figTest2beta0_02.emf']);
saveas(figTest2inf0_02,[outputDirectory 'figTest2inf0_02.fig']);
saveas(figTest2inf0_02,[outputDirectory 'figTest2inf0_02.emf']); 

 %*************************************
%R0 and gamma tests
%*************************************
%*************************************
%SIR version TEST3
%*************************************
%monte carlo random genotype SIR model
%1 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  1;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1;
[betameanmean(1) Smean(1,:) Imean(1,:) Rmean(1,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%2 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  2;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1;
[betameanmean(2) Smean(2,:) Imean(2,:) Rmean(2,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%3 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  3;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1;
[betameanmean(3) Smean(3,:) Imean(3,:) Rmean(3,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%4 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  4;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1;
[betameanmean(4) Smean(4,:) Imean(4,:) Rmean(4,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%5 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  5;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1;
[betameanmean(5) Smean(5,:) Imean(5,:) Rmean(5,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%6 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  6;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1;
[betameanmean(6) Smean(6,:) Imean(6,:) Rmean(6,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%7 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  7;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1;
[betameanmean(7) Smean(7,:) Imean(7,:) Rmean(7,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%8 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1;
[betameanmean(8) Smean(8,:) Imean(8,:) Rmean(8,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
betameanmeanD=betameanmean
 figTest3beta0_02 = figure
 plot(betameanmean,'k')
 ylabel('Measured beta')
 xlabel('Number of variants')
 title('Increasing variants')
 
 figTest3inf0_02 = figure
 for i = 1:8
     plot(Imean(i,:),'k')
     hold on
 end
 ylabel('Number of nodes infected - It')
 xlabel('Timesteps')
 title('Increasing variants')
 
 r0 = betameanmean./gamma
 r1 = 1
r1rep = repmat(r1,size(r0));

 figTest3r00_02 = figure
 plot(r0,'k')
 hold on
 plot(r1rep, 'k--')
 ylabel('r0')
 xlabel('Number of variants')
 title('Increasing variants')
 
 saveas(figTest3beta0_02,[outputDirectory 'figTest3beta0_02.fig']);
saveas(figTest3beta0_02,[outputDirectory 'figTest3beta0_02.emf']);
saveas(figTest3inf0_02,[outputDirectory 'figTest3inf0_02.fig']);
saveas(figTest3inf0_02,[outputDirectory 'figTest3inf0_02.emf']); 
saveas(figTest3r00_02,[outputDirectory 'figTest3r00_02.fig']);
saveas(figTest3r00_02,[outputDirectory 'figTest3r00_02.emf']); 
 
%*************************************
%Exploit tests TEST 4
%*************************************
%*************************************
%SIR version
%*************************************
%monte carlo random genotype SIR model
%1 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=1;
[betameanmean(1) Smean(1,:) Imean(1,:) Rmean(1,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%2 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=2;
[betameanmean(2) Smean(2,:) Imean(2,:) Rmean(2,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%3 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=3;
[betameanmean(3) Smean(3,:) Imean(3,:) Rmean(3,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%4 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=4;
[betameanmean(4) Smean(4,:) Imean(4,:) Rmean(4,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%5 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=5;
[betameanmean(5) Smean(5,:) Imean(5,:) Rmean(5,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%6 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=6;
[betameanmean(6) Smean(6,:) Imean(6,:) Rmean(6,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%7 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=7;
[betameanmean(7) Smean(7,:) Imean(7,:) Rmean(7,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%8 genotype
numRuns = 500; numNodes = 100; betas = 0.02; numGenotypes =  8;  numTimesteps = 4000; plotGraphs = 0; gamma = 0.01; numExploits=8;
[betameanmean(8) Smean(8,:) Imean(8,:) Rmean(8,:)] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
 
betameanmeanD=betameanmean
 figTest4beta0_02 = figure
 plot(betameanmean,'k')
 ylabel('Measured beta')
 xlabel('Number of exploits')
 title('Increasing exploits')

 figTest4inf0_02 = figure
 for i = 1:8
     plot(Imean(i,:),'k')
     hold on
 end
 ylabel('Number of nodes infected - It')
 xlabel('Timesteps')
 title('Increasing exploits')
 
  r0 = betameanmean./gamma
 r1 = 1
r1rep = repmat(r1,size(r0));
 figTest4r00_02 = figure
 plot(r0,'k')
 hold on
 plot(r1rep, 'k--')
 ylabel('r0')
 xlabel('Number of exploits')
 title('Increasing exploits')
 
saveas(figTest4beta0_02,[outputDirectory 'figTest4beta0_02.fig']);
saveas(figTest4beta0_02,[outputDirectory 'figTest4beta0_02.emf']);
saveas(figTest4inf0_02,[outputDirectory 'figTest4inf0_02.fig']);
saveas(figTest4inf0_02,[outputDirectory 'figTest4inf0_02.emf']); 
saveas(figTest4r00_02,[outputDirectory 'figTest4r00_02.fig']);
saveas(figTest4r00_02,[outputDirectory 'figTest4r00_02.emf']); 




return

%**********************************
%COMPARISON TESTS
%COMPARE SI WITH SIR WHEN GAMMA = 0
%**********************************
%*************************************
%SI version
%*************************************
%monte carlo random genotype SI model

numRuns = 500

numNodes = 400
betas = 0.5; %0.02
numGenotypes =  4; %8
numTimesteps = 150; %7000
plotGraphs = 0
%figure
for i = 1:numRuns
[numSusceptibles numInfected] = randomGenotypes(numNodes, betas, numGenotypes, numTimesteps, plotGraphs);
%plot(numSusceptibles,'b')
%hold on
%plot(numInfected, 'r')
for j = 1 : numTimesteps -1
B(j)=-(numSusceptibles(j+1) - numSusceptibles(j))/(numInfected(j)*numSusceptibles(j)/numNodes); %measured beta
end
%sort beta to get rid of nan values
sortedbeta = sort(B);
nanMask = isnan(sortedbeta);
[c indexNan] = max(nanMask);
if indexNan == 1; indexNan = numTimesteps-1; end;
betamean(i) = mean(sortedbeta(1:indexNan-1));

%beta(i,:) = B;
S(i,:) = numSusceptibles;
I(i,:) = numInfected;

%figure
%plot(beta(i,:))
%title('beta 1 run')
end
%betamean = mean(beta(:,2:23),2) %needs doing better
Smean = mean(S);
Imean = mean(I);
figure
plot(Smean,'b')
hold on
plot(Imean,'r')
title('SI - Compare to SIR')

%sortedbetamean = sort(betamean)
%nanMask = isnan(sortedbetamean)
%[c indexNan] = max(nanMask)
%betameanmean = mean(sortedbetamean(1:indexNan-1))
betameanmean = mean(betamean)

%*************************************
%SIR version
%*************************************
%monte carlo random genotype SIR model

numRuns = 500

numNodes = 400
betas = 0.5; %0.02
numGenotypes =  4; %8
numTimesteps = 150; %7000
plotGraphs = 0
gamma = 0
numExploits=1
%figure
mode = 'dynamic'
for i = 1:numRuns

[numSusceptibles numInfected numRecovered] = randomGenotypesSIR(numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%plot(numSusceptibles,'b')
%hold on
%plot(numInfected, 'r')
for j = 1 : numTimesteps -1
B(j)=-(numSusceptibles(j+1) - numSusceptibles(j))/(numInfected(j)*numSusceptibles(j)/numNodes); %measured beta
end
%sort beta to get rid of nan values
sortedbeta = sort(B);
nanMask = isnan(sortedbeta);
[c indexNan] = max(nanMask);
if indexNan == 1; indexNan = numTimesteps-1; end;
betamean(i) = mean(sortedbeta(1:indexNan-1));

%beta(i,:) = B;
S(i,:) = numSusceptibles;
I(i,:) = numInfected;
R(i,:) = numRecovered;
%figure
%plot(beta(i,:))
%title('beta 1 run')
end
%betamean = mean(beta(:,2:23),2) %needs doing better
Smean = mean(S);
Imean = mean(I);
Rmean = mean(R);
figure
plot(Smean,'b')
hold on
plot(Imean,'r')
plot(Rmean,'g')
title('SIR -compare to SI')

%sortedbetamean = sort(betamean)
%nanMask = isnan(sortedbetamean)
%[c indexNan] = max(nanMask)
%betameanmean = mean(sortedbetamean(1:indexNan-1))
betameanmean = mean(betamean)
%*********************************************************************************
end %SIRN TESTS 8

%*****************
% BACKGROUND TESTS
%****************
% BG TEST 1
%****************
%contact rate averages out over time
%instantaneous contact is probablistic
if doBGTests(1) ==1
outputDirectory = 'BGGraphs\'
numRuns = 100
numNodes = 15
Betat = 0.5
betasrep = repmat(Betat,1,numNodes); 
for i = 1:numRuns
randomNumberBeta = rand(1,numNodes);
encounterYes(i,:) = randomNumberBeta < betasrep;
runningAvcontactRateEachNode(i,:) = sum(encounterYes,1)./i;
end
figAvContact = figure
plot(runningAvcontactRateEachNode(:,1),'k--','Linewidth', 2)
hold on
plot(runningAvcontactRateEachNode(:,2),'k')
hold on
plot(runningAvcontactRateEachNode(:,3),'k.-') %plot each node average contact rate
xlabel('Time')
ylabel('Contact rate')
saveas(figAvContact,[outputDirectory 'TestBG1figAvContact.fig']);
saveas(figAvContact,[outputDirectory 'TestBG1afigAvContact.emf']);

%gamma rate of each node
numRuns = 200
numNodes = 50000
gamma = 0.05
gammarep = repmat(gamma,1,numNodes);

for i = 1:numRuns
randomNumberGamma = rand(1,numNodes);
gammaYes(i,:) = randomNumberGamma < gammarep;
%runningAvcontactRateEachNode(i,:) = sum(encounterYes,1)./i;
end
[v index] = max(gammaYes);
rateEachNode = 1./index;
recoveryFig = figure
hist(rateEachNode,50)
xlabel('Rate of recovery')
ylabel('Number of nodes')
h = findobj(gca,'Type','patch');
set(h,'FaceColor','k')
saveas(recoveryFig,[outputDirectory 'TestBG1brecoveryFig.fig']);
saveas(recoveryFig,[outputDirectory 'TestBG1brecoveryFig.emf']);
end %BG TEST 1

%***************
% BG TEST 2
%***************
if doBGTests(2) ==1
outputDirectory = 'BGGraphs\'
%SI beta tests beta(contact rate set) v's measured beta
beta = 0.01:0.05:1
steps=60:50:1010
numTimesteps  = sort(steps,'descend')
for i = 1:10
numRuns = 200,numNodes =100,  plotGraphs=0, plotGraphsSIRN = 0, gamma=0,    throwAway = 'nonEpidemics', SEpidemic=99;
[fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetanew fighistGammanew figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2 betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd betameanmean(i) gammameanmean betameanmean2 betameanmeanNew betameanmeanNew2 gammameanmeanNew Smean Imean Rmean Smedian Imedian Rmedian S I R SmeanN ImeanN RmeanN Snewendmean Snewendstd Rnewendmean Rnewendstd Rnew_nonendmean Rnew_nonendstd propSimsNonEp propSimsNonEp_theoretical] = monteCarloAgentSIR(numRuns,numNodes, beta(i), numTimesteps(i), plotGraphs, plotGraphsSIRN, gamma,throwAway,SEpidemic);
close all
end

betameanmean    
figcontactBeta = figure
plot(beta(1:10),betameanmean(1:10),'k')
hold on
plot(beta(1:10),beta(1:10),'k.-')
xlabel('Contact Rate')
ylabel('Measured beta')
legend('Measured beta', 'Theoretical beta')
saveas(figcontactBeta,[outputDirectory 'TestBG2figcontactBeta.fig']);
saveas(figcontactBeta,[outputDirectory 'TestBG2figcontactBeta.emf']);
close all;
end %BG TEST 2

%***************
% BG TEST 3
%***************
if doBGTests(3) ==1
    %beta error test
outputDirectory = 'BGGraphs\'
%check betas vs beta measured
numRuns = 500;numNodes = 100
%[error1] = betaErrorTest(numRuns,numNodes)
numRuns = 500;numNodes = 1000
%[error2] = betaErrorTest(numRuns,numNodes)
numRuns = 500;numNodes = 10000
%[error3] = betaErrorTest(numRuns,numNodes)

numRuns = 1000;numNodes = 100
%[error4] = betaErrorTest(numRuns,numNodes)
numRuns = 1000;numNodes = 1000
%[error5] = betaErrorTest(numRuns,numNodes)
numRuns = 1000;numNodes = 10000
%[error6] = betaErrorTest(numRuns,numNodes)

numRuns = 10000;numNodes = 100
%[error7] = betaErrorTest(numRuns,numNodes)
numRuns = 10000;numNodes = 1000
%[error8] = betaErrorTest(numRuns,numNodes)
numRuns = 10000;numNodes = 10000
%[error9] = betaErrorTest(numRuns,numNodes)

%computed from above - takes 8 hours to run

betameanmean_1 =[0.0023,0.0076,0.0169,0.0248,0.0339,0.0444,0.0533,0.0630,0.0739,0.0834,0.1993,0.3107,0.4090,0.5151,0.6101,0.7182,0.8258,0.9220,1.0227]
betameanmean2_1 =[0.0023,0.0076,0.0169,0.0248,0.0339,0.0444,0.0533,0.0630,0.0739,0.0834,0.1993,0.3107,0.4090,0.5151,0.6101,0.7182,0.8258,0.9220,1.0227]
error1 =[0.0077,0.0124,0.0131,0.0152,0.0161,0.0156,0.0167,0.0170,0.0161,0.0166,0.0007,0.0107,0.0090,0.0151,0.0101,0.0182,0.0258,0.0220,0.0227]

betameanmean_2 =[0.0025,0.0081,0.0169,0.0241,0.0322,0.0447,0.0557,0.0637,0.0723,0.0849,0.1823,0.3026,0.4040,0.5034,0.6041,0.7058,0.8093,0.9032,1.0026]
betameanmean2_2 =[0.0025,0.0081,0.0169,0.0241,0.0322,0.0447,0.0557,0.0637,0.0723,0.0849,0.1823,0.3026,0.4040,0.5034,0.6041,0.7058,0.8093,0.9032,1.0026]
error2 =[0.0075,0.0119,0.0131,0.0159,0.0178,0.0153,0.0143,0.0163,0.0177,0.0151,0.0177,0.0026,0.0040,0.0034,0.0041,0.0058,0.0093,0.0032,0.0026]

betameanmean_3 =[0.0024,0.0090,0.0163,0.0252,0.0336,0.0435,0.0536,0.0645,0.0727,0.0847,0.1842,0.2859,0.4016,0.4986,0.6049,0.7067,0.8041,0.9073,0.9990]
betameanmean2_3 =[0.0024,0.0090,0.0163,0.0252,0.0336,0.0435,0.0536,0.0645,0.0727,0.0847,0.1842,0.2859,0.4016,0.4986,0.6049,0.7067,0.8041,0.9073,0.9990]    
error3 =[0.0076,0.0110,0.0137,0.0148,0.0164,0.0165,0.0164,0.0155,0.0173,0.0153,0.0158,0.0141,0.0016,0.0014,0.0049,0.0067,0.0041,0.0073,0.0010]

betameanmean_4 =[0.0025,0.0079,0.0156,0.0248,0.0335,0.0435,0.0536,0.0627,0.0740,0.0840,0.2007,0.3077,0.4082,0.5085,0.6130,0.7147,0.8187,0.9176,1.0089]
betameanmean2_4 =[0.0025,0.0079,0.0156,0.0248,0.0335,0.0435,0.0536,0.0627,0.0740,0.0840,0.2007,0.3077,0.4082,0.5085,0.6130,0.7147,0.8187,0.9176,1.0089]  
error4 =[0.0075,0.0121,0.0144,0.0152,0.0165,0.0165,0.0164,0.0173,0.0160,0.0160,0.0007,0.0077,0.0082,0.0085,0.0130,0.0147,0.0187,0.0176,0.0089]

betameanmean_5 =[0.0022,0.0081,0.0164,0.0255,0.0342,0.0448,0.0530,0.0634,0.0737,0.0832,0.1835,0.3029,0.4030,0.5069,0.6046,0.7090,0.8071,0.9096,1.0166]
betameanmean2_5 =[0.0022,0.0081,0.0164,0.0255,0.0342,0.0448,0.0530,0.0634,0.0737,0.0832,0.1835,0.3029,0.4030,0.5069,0.6046,0.7090,0.8071,0.9096,1.0166] 
error5 =[0.0078,0.0119,0.0136,0.0145,0.0158,0.0152,0.0170,0.0166,0.0163,0.0168,0.0165,0.0029,0.0030,0.0069,0.0046,0.0090,0.0071,0.0096,0.0166]

betameanmean_6 =[0.0026,0.0083,0.0157,0.0248,0.0343,0.0430,0.0531,0.0650,0.0737,0.0833,0.1835,0.2884,0.4021,0.5043,0.6024,0.7037,0.8025,0.9070,1.0081]
betameanmean2_6 =[0.0026,0.0083,0.0157,0.0248,0.0343,0.0430,0.0531,0.0650,0.0737,0.0833,0.1835,0.2884,0.4021,0.5043,0.6024,0.7037,0.8025,0.9070,1.0081]    
error6 =[0.0074,0.0117,0.0143,0.0152,0.0157,0.0170,0.0169,0.0150,0.0163,0.0167,0.0165,0.0116,0.0021,0.0043,0.0024,0.0037,0.0025,0.0070,0.0081]

betameanmean_7 =[0.0025,0.0083,0.0159,0.0246,0.0342,0.0437,0.0536,0.0634,0.0737,0.0837,0.2009,0.3082,0.4097,0.5121,0.6135,0.7150,0.8169,0.9204,1.0172]
betameanmean2_7 =[0.0025,0.0083,0.0159,0.0246,0.0342,0.0437,0.0536,0.0634,0.0737,0.0837,0.2009,0.3082,0.4097,0.5121,0.6135,0.7150,0.8169,0.9204,1.0172]
error7 =[0.0075,0.0117,0.0141,0.0154,0.0158,0.0163,0.0164,0.0166,0.0163,0.0163,0.0009,0.0082,0.0097,0.0121,0.0135,0.0150,0.0169,0.0204,0.0172]

betameanmean_8 =[0.0024,0.0081,0.0161,0.0247,0.0341,0.0436,0.0536,0.0636,0.0735,0.0838,0.1842,0.3015,0.4044,0.5054,0.6054,0.7065,0.8094,0.9099,1.0099]
betameanmean2_8 =[0.0024,0.0081,0.0161,0.0247,0.0341,0.0436,0.0536,0.0636,0.0735,0.0838,0.1842,0.3015,0.4044,0.5054,0.6054,0.7065,0.8094,0.9099,1.0099]   
error8 =[0.0076,0.0119,0.0139,0.0153,0.0159,0.0164,0.0164,0.0164,0.0165,0.0162,0.0158,0.0015,0.0044,0.0054,0.0054,0.0065,0.0094,0.0099,0.0099]

betameanmean_9 =[0.0024,0.0082,0.0159,0.0250,0.0337,0.0437,0.0535,0.0633,0.0738,0.0838,0.1838,0.2864,0.4031,0.5031,0.6042,0.7045,0.8042,0.9067,1.0060]
betameanmean2_9 =[0.0024,0.0082,0.0159,0.0250,0.0337,0.0437,0.0535,0.0633,0.0738,0.0838,0.1838,0.2864,0.4031,0.5031,0.6042,0.7045,0.8042,0.9067,1.0060]   
error9 =[0.0076,0.0118,0.0141,0.0150,0.0163,0.0163,0.0165,0.0167,0.0162,0.0162,0.0162,0.0136,0.0031,0.0031,0.0042,0.0045,0.0042,0.0067,0.0060]

betas = [0.01 0.02 0.03 0.04 0.05 0.06 0.07 0.08 0.09 0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1]
%plot error
betaErrorfig = figure
plot(betas,error1,'b')
hold on
plot(betas,error2,'g')
plot(betas,error3,'r')
plot(betas,error4,'c')
plot(betas,error5,'m')
plot(betas,error6,'b--')
plot(betas,error7,'g--')
plot(betas,error8,'r--')
plot(betas,error9,'c--')
ylabel('error of Measured beta mean')
xlabel('Betas')
legend('1','2','3','4','5','6','7','8','9')

betafig = figure
plot(betas,betameanmean_1,'b')
hold on
plot(betas,betameanmean_2,'g')
plot(betas,betameanmean_3,'r')
plot(betas,betameanmean_4,'c')
plot(betas,betameanmean_5,'m')
plot(betas,betameanmean_6,'b--')
plot(betas,betameanmean_7,'g--')
plot(betas,betameanmean_8,'r--')
plot(betas,betameanmean_9,'c--')
plot(betas,betas,'r', 'Linewidth',2)
ylabel('Measured beta mean')
xlabel('Betas')
legend('1','2','3','4','5','6','7','8','9')

saveas(betaErrorfig,[outputDirectory 'TestBG3betaErrorfig.fig']);
saveas(betaErrorfig,[outputDirectory 'TestBG3betaErrorfig.emf']);
saveas(betafig,[outputDirectory 'TestBG3betafig.fig']);
saveas(betafig,[outputDirectory 'TestBG3betafig.emf']);
close all;
end %BG TEST 3
if doBGTests(4) ==1
    %exponential distribution
outputDirectory = 'BGGraphs\'

t = 1: 15
beta = 0.3
meanTime = 1/0.3
y = beta*exp(-1*beta*t)
expFig = figure
plot(t,y,'k')
hold on
plot([meanTime meanTime],[0 0.2],'k--')
xlabel('Time')
ylabel('probability (t)')

saveas(expFig,[outputDirectory 'TestBG4expFig.fig']);
saveas(expFig,[outputDirectory 'TestBG4expFig.emf']);

end %BG TEST 4
end
%******************************************************************
%SUB FUNCTIONS
%******************************************************************
    function [betameanmean Smean Imean Rmean] = monteCarloSIR(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode)
        for i = 1:numRuns
            [numSusceptibles numInfected numRecovered] = randomGenotypesSIR(numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
            %plot(numSusceptibles,'b')
            %hold on
            %plot(numInfected, 'r')
                for j = 1 : numTimesteps -1
                B(j)=-(numSusceptibles(j+1) - numSusceptibles(j))/(numInfected(j)*numSusceptibles(j)/numNodes); %measured beta
                end
            %sort beta to get rid of nan values
            sortedbeta = sort(B);
            nanMask = isnan(sortedbeta);
            [c indexNan] = max(nanMask);
            if indexNan == 1; indexNan = numTimesteps-1; end;
            betamean(i) = mean(sortedbeta(1:indexNan-1));

            %beta(i,:) = B;
            S(i,:) = numSusceptibles;
            I(i,:) = numInfected;
            R(i,:) = numRecovered;
            %figure
            %plot(beta(i,:))
            %title('beta 1 run')
         end
            %betamean = mean(beta(:,2:23),2) %needs doing better
            Smean = mean(S);
            Imean = mean(I);
            Rmean = mean(R);
            %sizeImean = size(Imean)
            %figure
            %plot(Smean,'b')
            %hold on
            %plot(Imean,'r')
            %plot(Rmean,'g')
            %title('SIR -compare to SI')

            %sortedbetamean = sort(betamean)
            %nanMask = isnan(sortedbetamean)
            %[c indexNan] = max(nanMask)
            %betameanmean = mean(sortedbetamean(1:indexNan-1))
            betameanmean = mean(betamean)   
    end
    function [betameanmean betameanmean2 Smean Imean Rmean NSmean] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode, numLoci,casetype)
        plotGraphsSIRN=0
        for i = 1:numRuns          
             [numSusceptibles numInfected numRecovered numNotSusceptibles] = randomGenotypesSIRN(numNodes, betas, numGenotypes, numTimesteps, plotGraphsSIRN, gamma, numExploits, mode, numLoci,casetype);

            %plot(numSusceptibles,'b')
            %hold on
            %plot(numInfected, 'r')
                for j = 1 : numTimesteps -1
                    Ns = numSusceptibles(j+1)+numInfected(j+1)+numRecovered(j+1); %only count those actually in susceptible part of network at time t
                %B(j)=-(numSusceptibles(j+1) - numSusceptibles(j))/(numInfected(j)*numSusceptibles(j)/numNodes); %measured beta
                 B(j)=-(numSusceptibles(j+1) - numSusceptibles(j))/(numInfected(j)*numSusceptibles(j)/Ns); %measured beta
                 Y(j) = (numRecovered(j+1)-numRecovered(j))/numInfected(j);
                 B2(j) = (Ns.*(numInfected(j+1)+(Y(j)-1).*numInfected(j)))/(numInfected(j)*numSusceptibles(j)); %measured beta
                 %measure Ro
                 Ro(j) = (B(j)*numSusceptibles(j))/(Ns*Y(j));
                 B3(j) = betas*Ns/numNodes;
                end
                %BD = B
                %B2D=B2
            %sort beta to get rid of nan values
            sortedbeta = sort(B);
            nanMask = isnan(sortedbeta);
            infMask = isinf(sortedbeta);
            [startv starti] = min(infMask);
            [c indexNan] = max(nanMask);
            if indexNan == 1; indexNan = numTimesteps-1; end;
            betamean(i) = mean(sortedbeta(starti:indexNan-1));
            %sort beta2 to get rid of nan values
            sortedbeta2 = sort(B2);
            nanMask2= isnan(sortedbeta2);
            infMask2 = isinf(sortedbeta2);
            [startv2 starti2] = min(infMask2);
            [c2 indexNan2] = max(nanMask2);
            if indexNan2 == 1; indexNan2 = numTimesteps-1; end;
            betamean2(i) = mean(sortedbeta2(starti2:indexNan2-1));


            %beta(i,:) = B;
            S(i,:) = numSusceptibles;
            I(i,:) = numInfected;
            R(i,:) = numRecovered;
            NS(i,:) = numNotSusceptibles;
            if plotGraphsSIRN ==1
            figure
            plot(B)
            title('beta 1 run')
            figure
            plot(B2)
            title('beta2 1 run')
            figure
            plot(B3)
            title('beta3 1 run')
            end
         end
            %betamean = mean(beta(:,2:23),2) %needs doing better
            Smean = mean(S);
            Imean = mean(I);
            Rmean = mean(R);
            NSmean = mean(NS);
            %sizeImean = size(Imean)
            if plotGraphs==1
            figure
            plot(Smean,'b')
            hold on
            plot(Imean,'r')
            plot(Rmean,'g')
            title('SIR -compare to SI')
            end%plot
               figure
            for j =1:numRuns
                plot(S(j,:),'b')
                hold on
                plot(I(j,:),'r')
                plot(R(j,:),'g')
            end
            title('All runs')
            
            %Histograms
                histscale = [0 1000 0 1000]
    histSize = 100;
               %distribution of R at end of simulation
    rend = R(:,numTimesteps);
    fighistRecovered = figure
    hist(rend,histSize)
    xlabel('Final size (Recovered)')
    ylabel('Number of runs')
    title('Histogram of Final recovered size')
    h = findobj(gca,'Type','patch');
    set(h,'FaceColor','k')
    axis(histscale)
   %get distribution of S at end of simulation
    send = S(:,numTimesteps);
    fighistSusceptible = figure;
    hist(send,histSize)
    xlabel('Final value of S')
    ylabel('Number of runs')
    title('Histogram of S(t=end)')
    h = findobj(gca,'Type','patch');
    set(h,'FaceColor','k')
    axis(histscale)
    
            %sortedbetamean = sort(betamean)
            %nanMask = isnan(sortedbetamean)
            %[c indexNan] = max(nanMask)
            %betameanmean = mean(sortedbetamean(1:indexNan-1))
            betameanmean = mean(betamean)   
             betameanmean2 = mean(betamean2)  
    end    
    function [error] = betaErrorTest(numRuns,numNodes)
    mode = 'static'
%0.01
%numRuns = 500; numNodes = 1000; 
betas = 0.01; numGenotypes =  1;  numTimesteps = 60; plotGraphs = 0; gamma = 0; numExploits=1;
[betameanmean(1) betameanmean2(1) Smean(1,:) Imean(1,:) Rmean(1,:) NSmean(1,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%0.02
 betas = 0.02; numGenotypes =  1;  numTimesteps = 60; plotGraphs = 0; gamma = 0; numExploits=1;
[betameanmean(2) betameanmean2(2) Smean(2,:) Imean(2,:) Rmean(2,:) NSmean(2,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%0.03
betas = 0.03; numGenotypes =  1;  numTimesteps = 60; plotGraphs = 0; gamma = 0; numExploits=1;
[betameanmean(3) betameanmean2(3) Smean(3,:) Imean(3,:) Rmean(3,:) NSmean(3,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%0.04
 betas = 0.04; numGenotypes =  1;  numTimesteps = 60; plotGraphs = 0; gamma = 0; numExploits=1;
[betameanmean(4) betameanmean2(4) Smean(4,:) Imean(4,:) Rmean(4,:) NSmean(4,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%0.05
 betas = 0.05; numGenotypes =  1;  numTimesteps = 60; plotGraphs = 0; gamma = 0; numExploits=1;
[betameanmean(5) betameanmean2(5) Smean(5,:) Imean(5,:) Rmean(5,:) NSmean(5,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%0.06
 betas = 0.06; numGenotypes =  1;  numTimesteps = 60; plotGraphs = 0; gamma = 0; numExploits=1;
[betameanmean(6) betameanmean2(6) Smean(6,:) Imean(6,:) Rmean(6,:) NSmean(6,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%0.07
 betas = 0.07; numGenotypes =  1;  numTimesteps = 60; plotGraphs = 0; gamma = 0; numExploits=1;
[betameanmean(7) betameanmean2(7) Smean(7,:) Imean(7,:) Rmean(7,:) NSmean(7,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%0.08
 betas = 0.08; numGenotypes =  1;  numTimesteps = 60; plotGraphs = 0; gamma = 0; numExploits=1;
[betameanmean(8) betameanmean2(8) Smean(8,:) Imean(8,:) Rmean(8,:) NSmean(8,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%0.09
 betas = 0.09; numGenotypes =  1;  numTimesteps = 60; plotGraphs = 0; gamma = 0; numExploits=1;
[betameanmean(9) betameanmean2(9) Smean(9,:) Imean(9,:) Rmean(9,:) NSmean(9,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%0.1
 betas = 0.1; numGenotypes =  1;  numTimesteps = 60; plotGraphs = 0; gamma = 0; numExploits=1;
[betameanmean(10) betameanmean2(10) Smean(10,:) Imean(10,:) Rmean(10,:) NSmean(10,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);

%0.2
 betas = 0.2; numGenotypes =  1;  numTimesteps = 60; plotGraphs = 0; gamma = 0; numExploits=1;
[betameanmean(11) betameanmean2(11) Smean(11,:) Imean(11,:) Rmean(11,:) NSmean(11,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%0.3
 betas = 0.3; numGenotypes =  1;  numTimesteps = 60; plotGraphs = 0; gamma = 0; numExploits=1;
[betameanmean(12) betameanmean2(12) Smean(12,:) Imean(12,:) Rmean(12,:) NSmean(12,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%0.4
 betas = 0.4; numGenotypes =  1;  numTimesteps = 60; plotGraphs = 0; gamma = 0; numExploits=1;
[betameanmean(13) betameanmean2(13) Smean(13,:) Imean(13,:) Rmean(13,:) NSmean(13,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%0.5
 betas = 0.5; numGenotypes =  1;  numTimesteps = 60; plotGraphs = 0; gamma = 0; numExploits=1;
[betameanmean(14) betameanmean2(14) Smean(14,:) Imean(14,:) Rmean(14,:) NSmean(14,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%0.6
 betas = 0.6; numGenotypes =  1;  numTimesteps = 60; plotGraphs = 0; gamma = 0; numExploits=1;
[betameanmean(15) betameanmean2(15) Smean(15,:) Imean(15,:) Rmean(15,:) NSmean(15,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%0.7
 betas = 0.7; numGenotypes =  1;  numTimesteps = 60; plotGraphs = 0; gamma = 0; numExploits=1;
[betameanmean(16) betameanmean2(16) Smean(16,:) Imean(16,:) Rmean(16,:) NSmean(16,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%0.8
 betas = 0.8; numGenotypes =  1;  numTimesteps = 60; plotGraphs = 0; gamma = 0; numExploits=1;
[betameanmean(17) betameanmean2(17) Smean(17,:) Imean(17,:) Rmean(17,:) NSmean(17,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%0.9
 betas = 0.9; numGenotypes =  1;  numTimesteps = 60; plotGraphs = 0; gamma = 0; numExploits=1;
[betameanmean(18) betameanmean2(18) Smean(18,:) Imean(18,:) Rmean(18,:) NSmean(18,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);
%1
 betas = 1; numGenotypes =  1;  numTimesteps = 60; plotGraphs = 0; gamma = 0; numExploits=1;
[betameanmean(19) betameanmean2(19) Smean(19,:) Imean(19,:) Rmean(19,:) NSmean(19,:)] = monteCarloSIRN(numRuns, numNodes, betas, numGenotypes, numTimesteps, plotGraphs, gamma,numExploits,mode);

betas = [0.01 0.02 0.03 0.04 0.05 0.06 0.07 0.08 0.09 0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1]
%figure
%plot(betas,betameanmean,'k')
%hold on
%plot(betas,betas,'r')
%ylabel('Measured beta mean')
%xlabel('Betas')

error = abs(betas-betameanmean)
  end
        function [fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetanew fighistGammanew figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2 ...
            betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean gammameanmean betameanmean2 betameanmeanNew betameanmeanNew2 gammameanmeanNew Smean Imean Rmean Smedian Imedian Rmedian S I R SmeanN ImeanN RmeanN Snewendmean Snewendstd Rnewendmean Rnewendstd Rnew_nonendmean Rnew_nonendstd propSimsNonEp propSimsNonEp_theoretical...
            ] = monteCarloAgentSIR(numRuns,numNodes, beta, numTimesteps, plotGraphs, plotGraphsSIRN,gamma,throwAway,SEpidemic)
        
        deltat=1 
             
        %plotGraphsSIRN = 1
            %figure
            as=1; ai=1; ar=1; sm=0; rm=0;
            for i = 1:numRuns 
                    [numSusceptibles numInfected numRecovered] = agentSIR(numNodes, beta, numTimesteps, 0, gamma);
                for j = 1 : (numTimesteps/deltat) -1
                    Ns = numSusceptibles(j+1)+numInfected(j+1)+numRecovered(j+1); %only count those actually in susceptible part of network at time t
                    %measure beta
                    B(j)=(-(numSusceptibles(j+1) - numSusceptibles(j))/(numInfected(j)*numSusceptibles(j)/Ns))./deltat; %measured beta
                    %B2(j) = (Ns.*(numInfected(j+1)+(gamma-1).*numInfected(j)))/(numInfected(j)*numSusceptibles(j)); %measured beta
                    %measure gamma
                    Y(j) = ((numRecovered(j+1)-numRecovered(j))/numInfected(j))./deltat;
                    B2(j) = ((Ns.*(numInfected(j+1)+(Y(j)-1).*numInfected(j)))/(numInfected(j)*numSusceptibles(j)))./deltat; %measured beta
                    %measure Ro
                    Ro(j) = (B(j)*numSusceptibles(j))/(Ns*Y(j));
                end
                Bsave(i,:) = B;
                Ysave(i,:) = Y;
                %sort beta to get rid of nan values
                sortedbeta = sort(B);
                nanMask = isnan(sortedbeta);
                infMask = isinf(sortedbeta);
                [startv starti] = min(infMask);
                [c indexNan] = max(nanMask);
                if indexNan == 1; indexNan = (numTimesteps/deltat)-1; end;
                %BD = B %test only
                %sortedbetaDisplayPre = sortedbeta %test only
                %sortedbetaDisplay = sortedbeta(starti:indexNan-1)%test only
                %sortedbeta(indexNan)=0;
                %betamean(i) = mean(sortedbeta(starti:indexNan));
                betamean(i) = mean(sortedbeta(starti:indexNan-1));
                %sort beta2 to get rid of nan values
                sortedbeta2 = sort(B2);
                nanMask2= isnan(sortedbeta2);
                infMask2 = isinf(sortedbeta2);
                [startv2 starti2] = min(infMask2);
                [c2 indexNan2] = max(nanMask2);
                if indexNan2 == 1; indexNan2 = (numTimesteps/deltat)-1; end;
                betamean2(i) = mean(sortedbeta2(starti2:indexNan2-1));
                
                 %sort gamma to get rid of nan and inf values
                sortedgamma = sort(Y);
                nanMaskg= isnan(sortedgamma);
                infMaskg = isinf(sortedgamma);
                [startvg startig] = min(infMaskg);
                [cg indexNang] = max(nanMaskg);
                if indexNang == 1; indexNang = (numTimesteps/deltat)-1; end;
                gammamean(i) = mean(sortedgamma(startig:indexNang-1));

               % plot(numSusceptibles,'b')
               % hold on
               % plot(numInfected,'r')
               % hold on
               % plot(numRecovered,'g')
               % hold on
                
                S(i,:) = numSusceptibles;
                I(i,:) = numInfected;
                R(i,:) = numRecovered;
                 if strcmp(throwAway,'nonEpidemics') 
                 if min(numSusceptibles)>= SEpidemic; sm(as) = i; as = as+ 1; end %case of non epidemics to throw away
                 end
                 if strcmp(throwAway,'Epidemics')
                 if min(numSusceptibles) < SEpidemic; sm(as) = i; as = as+ 1; end %case of epidemics to throw away
                 end
                 %if max(numInfected)<= 1; im(ai) = i; ai = ai+ 1;end
                 %if max(numRecovered)<= 1; rm(ar) = i; ar = ar+ 1;end
                 
                 if min(numSusceptibles) < SEpidemic; rm(ar) = i; ar = ar+ 1; end %case of epidemics to throw away

            %title('S I R new - without non epidemics')
                %if plotGraphsSIRN ==1
                if i == 1
                    figBetaRun1 = figure
                    plot(B,'k')
                    BoneRun = B;
                    title('beta 1 run')
                    xlabel('Timesteps')
                    ylabel('Measured beta')
                    figInfectedRun1 = figure
                    plot(numInfected,'k')
                    title('infected 1 run')
                    xlabel('Timesteps')
                    ylabel('I(t)')
                    %figure
                    %plot(B2,'k')
                    %title('beta2 1 run')
                    figGammaRun1 = figure
                    plot(Y,'k')
                    YoneRun = Y;
                    xlabel('Timesteps')
                    ylabel('Measured gamma')
                    title('gamma 1 run')
                    figRoRun1 = figure
                    plot(Ro,'k', 'Linewidth', 2)
                    hold on
                    r1 = 1;
                    r1rep = repmat(r1,size(Ro,2));
                    plot(numInfected./5,'k')
                    plot(r1rep,'k--')
                    xlabel('Timesteps')
                    ylabel('Measured Re')
                    title('Re 1 run')
                    legend('Re','infected/5','Re=1')
                    figBetaGamma1 = figure
                    plot(B./Y,'k')
                    title('beta/gamma')
                    figBetaGamma2 = figure
                    plot(B>Y,'k')
                    title('beta>gamma')
                end
            end
            %betamean = mean(beta(:,2:23),2) %needs doing better
            Smean = mean(S);
            Imean = mean(I);
            Rmean = mean(R);
            Smedian = median(S);
            Imedian = median(I);
            Rmedian = median(R);
            
            
            %get rid of non epidemic runs in data
            sm;
            len = size(sm,2);
            lenS = size(S,1);
            if and(len>=1,sm>0)           
            Snew((1:sm(1)-1),:) = S(1:sm(1)-1,:);
            Inew((1:sm(1)-1),:) = I(1:sm(1)-1,:);
            Rnew((1:sm(1)-1),:) = R(1:sm(1)-1,:);
            betameannew((1:sm(1)-1)) = betamean(1:sm(1)-1);
            gammameannew((1:sm(1)-1)) = gammamean(1:sm(1)-1);
            betameannew2((1:sm(1)-1)) = betamean2(1:sm(1)-1);
            BOnly(1:sm(1)-1,:) = Bsave(1:sm(1)-1,:);
            Yonly(1:sm(1)-1,:) = Ysave(1:sm(1)-1,:);
            for j = 2:len
            Snew(sm(j-1)-(j-2):sm(j)-j,:) = S(sm(j-1)+1:sm(j)-1,:);
            Inew(sm(j-1)-(j-2):sm(j)-j,:) = I(sm(j-1)+1:sm(j)-1,:);
            Rnew(sm(j-1)-(j-2):sm(j)-j,:) = R(sm(j-1)+1:sm(j)-1,:);
            betameannew(sm(j-1)-(j-2):sm(j)-j) = betamean(sm(j-1)+1:sm(j)-1);
            gammameannew(sm(j-1)-(j-2):sm(j)-j) = gammamean(sm(j-1)+1:sm(j)-1);
            betameannew2(sm(j-1)-(j-2):sm(j)-j) = betamean2(sm(j-1)+1:sm(j)-1);
            BOnly((sm(j-1)-(j-2):sm(j)-j),:) = Bsave((sm(j-1)+1:sm(j)-1),:);
            YOnly((sm(j-1)-(j-2):sm(j)-j),:) = Ysave((sm(j-1)+1:sm(j)-1),:);
            end
            Snew(sm(j)-(j-1):lenS-j,:) = S(sm(j)+1:lenS,:);
            Inew(sm(j)-(j-1):lenS-j,:) = I(sm(j)+1:lenS,:);
            Rnew(sm(j)-(j-1):lenS-j,:) = R(sm(j)+1:lenS,:);
            betameannew(sm(j)-(j-1):lenS-j) = betamean(sm(j)+1:lenS);
            gammameannew(sm(j)-(j-1):lenS-j) = gammamean(sm(j)+1:lenS);
            betameannew2(sm(j)-(j-1):lenS-j) = betamean2(sm(j)+1:lenS);
            BOnly(sm(j)-(j-1):lenS-j,:) = Bsave(sm(j)+1:lenS,:);
            YOnly(sm(j)-(j-1):lenS-j,:) = Ysave(sm(j)+1:lenS,:);
            else
                Snew = S;
                Inew = I;
                Rnew = R;
                %betameannew=betamean;
                %gammameannew=gammamean;
                %betameannew2=betamean2;
                betameannew=betamean.*0.0;
                gammameannew=gammamean.*0.0;
                betameannew2=betamean2.*0.0;
            end 
            
           
            
            
            %Measure Final value size info - non epidemics
            lenr = size(rm,2);
            lenS = size(S,1);
            if and(lenr>=1,rm>0)           
            Rnew_non((1:rm(1)-1),:) = R(1:rm(1)-1,:);
            for j = 2:lenr
            Rnew_non(rm(j-1)-(j-2):rm(j)-j,:) = R(rm(j-1)+1:rm(j)-1,:);
            end
            Rnew_non(rm(j)-(j-1):lenS-j,:) = R(rm(j)+1:lenS,:);
            else
                Rnew_non = R;
            end          
            Rnew_nonMean = mean(Rnew_non);
            
            if plotGraphs==1
            figure
            for i =1:size(Snew,1)
                plot(Snew(i,:),'b')
                hold on
                plot(Inew(i,:),'r')
                plot(Rnew(i,:),'g')
            end
            title('S I R new - without non epidemics')
            
            figure
            for j =1:len
                sm(j)
                plot(S(sm(j),:),'b')
                hold on
                plot(I(sm(j),:),'r')
                plot(R(sm(j),:),'g')
            end
            title('sm only')
             end
            SmeanN = mean(Snew);
            ImeanN = mean(Inew);
            RmeanN = mean(Rnew);
            

            %sizeImean = size(Imean)
            if plotGraphs==1
            figure
            plot(Smean,'b')
            hold on
            plot(Imean,'r')
            plot(Rmean,'g')
            title('SIR -compare to SI')
            end
            
            %sortedbetamean = sort(betamean)
            %nanMask = isnan(sortedbetamean)
            %[c indexNan] = max(nanMask)
            %betameanmean = mean(sortedbetamean(1:indexNan-1))
            betameanmean = mean(betamean);
            betastd = std(betamean); %std of beta, not std of beta exponential!
            gammameanmean = mean(gammamean);
            gammastd = std(gammamean);
             betameanmean2 = mean(betamean2);  
             betastd2 = std(betamean2); 
             betameanmeanNew = mean(betameannew);%without non epidemics
             betastdNew = std(betameannew);
             betameanmeanNew2 = mean(betameannew2);%without non epidemics
             betastdNew2 = std(betameannew2);
             gammameanmeanNew = mean(gammameannew);%without non epidemics
             gammastdNew = std(gammameannew);
             %Snewend = Snew(:,numTimesteps);
             if size(Snew) < 1; Snewend = nan; else Snewend = Snew(:,end); end;
             Snewendmean = mean(Snewend); %mean of final value of S with epidemics
             Snewendstd = std(Snewend);
             %Rnewend = Rnew(:,numTimesteps);
             if size(Rnew) < 1; Rnewend = nan; else Rnewend = Rnew(:,end); end;
             Rnewendmean = mean(Rnewend);%mean of final value of R with epidemics
             Rnewendstd = std(Rnewend);
             if size(Rnew_non,2) > 0; Rnew_nonend = Rnew_non(:,numTimesteps); else Rnew_nonend = 0; end
             Rnew_nonendmean = mean(Rnew_nonend);%mean of final value of R with no epidemics
             Rnew_nonendstd = std(Rnew_nonend);
             %proportion of simulations resulting in minor outbreak(non-epidemics)
             numberNonEp= size(Rnew_nonend,1)
             propSimsNonEp = numberNonEp/numRuns
             %theoretical gamma/beta = 1/R0
             propSimsNonEp_theoretical = gamma/beta;
             %theoretical value of mean of final value of R with epidemics
             %numNodes *z, where z is +ve solution of 1-z = e(-Roz)
             

    %**********************
    %DISTRIBUTION GRAPHS
    %**********************
    histscale = [0 1000 0 1000]
    histSize = 100;
   %distribution of beta         
    fighistBeta=figure;
    hist(betamean,histSize)
    xlabel('Average value of beta for one run')
    ylabel('Number of runs')
    title('Histogram of Beta')
    h = findobj(gca,'Type','patch');
    set(h,'FaceColor','k')
    
   %distribution of gamma
    fighistGamma=figure;
    hist(gammamean,histSize)
    xlabel('Average value of gamma for one run')
    ylabel('Number of runs')
    title('Histogram of Gamma')
    h = findobj(gca,'Type','patch');
    set(h,'FaceColor','k')
    
   %distribution of beta with runs removed
    fighistBetanew=figure;
    hist(betameannew,histSize)
    xlabel('Average value of beta for one run')
    ylabel('Number of runs')
    title('Histogram of Beta')
    h = findobj(gca,'Type','patch');
    set(h,'FaceColor','k')
   %distribution of gamma with runs removed
    fighistGammanew=figure;
    hist(gammameannew,histSize)
    xlabel('Average value of gamma for one run')
    ylabel('Number of runs')
    title('Histogram of Gamma')
    h = findobj(gca,'Type','patch');
    set(h,'FaceColor','k')
   %distribution of R at end of simulation
    rend = R(:,numTimesteps);
    fighistRecovered = figure
    hist(rend,histSize)
    xlabel('Final size (Recovered)')
    ylabel('Number of runs')
    title('Histogram of Final recovered size')
    h = findobj(gca,'Type','patch');
    set(h,'FaceColor','k')
    axis(histscale)
   %get distribution of S at end of simulation
    send = S(:,numTimesteps);
    fighistSusceptible = figure;
    hist(send,histSize)
    xlabel('Final value of S')
    ylabel('Number of runs')
    title('Histogram of S(t=end)')
    h = findobj(gca,'Type','patch');
    set(h,'FaceColor','k')
    axis(histscale)
   %peak value and time of peak infection
    [peakValue peakTime] = max(I,[],2);
   %distribution of peak value infection 
    fighistpeakValue = figure
    hist(peakValue,histSize)
    xlabel('peak value of infectives')
    ylabel('Number of runs')
    title('Histogram of peak value of infectives')
    h = findobj(gca,'Type','patch');
    set(h,'FaceColor','k')
  %distribution of peak time infection
    fighistpeakTime = figure
    hist(peakTime,histSize)
    xlabel('peak time of infectives')
    ylabel('Number of runs')
    title('Histogram of peak time of infectives')
    h = findobj(gca,'Type','patch');
    set(h,'FaceColor','k')
    
    peakTimemean = mean(peakTime);
    peakTimestd = std(peakTime);
    peakValuemean = mean(peakValue);
    peakValuestd = std(peakValue);
    
    
     mask = gammamean<1; %elliminate all those with a gamma value of 1
     sumMask = sum(mask,2);%number to average over
     sumMaskRep = repmat(sumMask,1,size(gammamean,2));
      avGamma = sum(gammamean.*mask./sumMaskRep)
             
%             BOnlyD=BOnly(1,1:10)
%             YOnlyD=YOnly(1,1:10)
%             BOnlyD=BOnly(2,1:10)
%             YOnlyD=YOnly(2,1:10)
%             BOnlyD=BOnly(3,1:10)
%             YOnlyD=YOnly(3,1:10)
        end
  
   

