%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Jennifer Jackson 2017                                                        %
% A Biodiversity Approach to Cyber Security                                    %
% PhD Complexity Science                                                       %
% University of Warwick                                                        %
% http://wrap.warwick.ac.uk/98458                                              %
% This software is distributed under the  Creative Commons Attribution License %
% https://creativecommons.org/licenses/by/4.0/                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [ output_args ] = testRandomGenotypesRWP( input_args )
%NOTES: when blacklisting one variant VR needs to be at least 3, since FS
%algorithm sets FS score of blacklist to zero but if there is only one left
%to choose from will set this to zero also because considers it used to
%much already and then assigns the blacklisted one as a default measure 
%(as its the first in the list) as it doesn't know which one to pick.
%Notes always set to stationary rwp even if dynamic as changed code
clear all;
%*************************************
% TESTING AGENT BASED MODEL in RWP mode
%*************************************
close all;

%new rwp box constraints
rwph.nodesln  = 0;     % Number of limited nodes (absolute) - always first
rwph.xminln   = 0;     % Minimum x-axis location limited nodes (du)
rwph.xmaxln   = 0;    % Maximum x-axis location limited nodes (du)
rwph.yminln   = 0;     % Minimum y-axis location limited nodes (du)
rwph.ymaxln   = 0;    % Maximum y-axis location limited nodes (du)

%new parameters
modePar.blklistStartTime = 1
%vmupdate mode
modePar.vm = 'no' %yes or no
modePar.desirabilities = 'no' %include desirability constraints
modePar.filter = 'no' %include sw filtering

doSIRNTests = zeros(1,400); %tests with genotypes
doSIRNTests(300)=1 %testing rwp
%doSIRNTests(301)=1 %Dynamic Diversity reaching highest diversity level time graphs
%doSIRNTests(302)=1 %Dynamic Diversity reaching highest diversity level time graphs data:AgentModels\randomGenotypesGraphs\rwp\Test301\nearestInRange
%doSIRNTests(303)=1 %Dynamic Diversity - time to reach max div vs time to tx genotype, Random Available in range
%doSIRNTests(304)=1 %Dynamic Diversity - time to reach max div vs time to tx genotype, Random in range
%doSIRNTests(305)=1 %Dynamic Diversity - time to reach max div vs time to tx genotype, Nearest in range
%doSIRNTests(306)=1 %using data from 303 to 305
%doSIRNTests(307)=1 %Dynamic Diversity - with constraints, Random in range
%doSIRNTests(308)=1 %combining constraint data from 307 above, Random in range
%doSIRNTests(309)=1 %peak infection and time to transmit the virus
%doSIRNTests(310)=1; %diversity under geographical constraints, random in range test graphs
%*********************
%SI testing of comparative constraints, tests 311 to 317
%NEED TO SUBTRACT VIRUS START TIME FROM PEAK INFECTION TIME
%VR 1 to 10, Range 1 to 10
%FS/RV box constraint
%FS/RV random constraint
%FS/RV no constraint
%Outputs meanI, peak time, resistance, distance spread from origin(valid box constraint only)
%doSIRNTests(311)=1; %stationary nodes -  1 locus
%doSIRNTests(312)=1; %stationary nodes -  4 loci AND
%doSIRNTests(313)=1; %stationary nodes -  4 loci OR
%doSIRNTests(314)=1; %moving nodes -  1 locus
%doSIRNTests(315)=1; %moving nodes -  4 loci AND
%doSIRNTests(316)=1; %moving nodes -  4 loci OR
%doSIRNTests(317)=1; %combine data into graphs


%combine data into graphs
if doSIRNTests(317) == 1
    outputDirectory = 'Graphs\'
    %stationary nodes graphs
    statdistfig = figure
    statresistfig = figure
    statTfig = figure
    %moving nodes graphs
    movdistfig = figure
    movresistfig = figure
    movTfig = figure
     data.logDir = sprintf('%s\\outputdata', pwd)      
    fileNumber = [311 312 313 314 315 316]
    VR1=[1 2 3 4 5 6 8 10 12 14 16]; %311
    VR2 = [1 2 3 4 5 6 7]; %312
    VR3 = [1 2 4 6 8 10 12 14 16 18 20 25 30 35]; %313
    VR4=[1 2 3 4 5 6 8 10 12 14 16]; %314
    VR5 = [1 2 3 4 5 6 7]; %315
    VR6 = [1 2 4 6 8 10 12 14 16 18 20 25 30 35]; %316 
    %extra 1 locus data 311 314
    VR7 = [20 25 35];
    %extra AND data 312 315
    VR8 = [10 15 25 35];
    numFiles = size(fileNumber,2)
    for i = 1: numFiles
        data.m = fileNumber(i) %data file number 
        data.numTimesteps=350
        if i==1; VR=VR1; end; if i==2; VR=VR2; end; if i==3; VR=VR3; end;
        if i==4; VR=VR4; end; if i==5; VR=VR5; end; if i==6; VR=VR6; end;
        [meanmdist T1 Resistance FinalSize]= getconstraintsdata(data, VR);
        output = plotConstraintsData(data, meanmdist, T1, Resistance, FinalSize, VR);
        %*********************************
        %ADD EXTRA DATA TO ARRAYS
        %extra data 1 locus
        if i==1; VRX=VR7; data.m = 3111; end; if i==4; VRX=VR7; data.m = 3141;end;
        %extra data AND
        if i==2; VRX=VR8; data.m = 3121; end; if i==5; VRX=VR8; data.m = 3151;end;
        if i ==1 || i==2 || i==4 || i==5
            [meanmdistX T1X ResistanceX FinalSizeX]= getconstraintsdata(data, VRX);
            outputX = plotConstraintsData(data, meanmdistX, T1X, ResistanceX, FinalSizeX, VRX);
            %ADD EXTRA DATA TO ARRAYS
            VR=[VR VRX];
            output.FSmean1=[output.FSmean(1,:) outputX.FSmean(1,:)];
            output.FSmean2=[output.FSmean(2,:) outputX.FSmean(2,:)];
            output.FSmean3=[output.FSmean(3,:) outputX.FSmean(3,:)];
            output.FSmeanRes1=[output.FSmeanRes(1,:) outputX.FSmeanRes(1,:)];
            output.FSmeanRes2=[output.FSmeanRes(2,:) outputX.FSmeanRes(2,:)];
            output.FSmeanRes3=[output.FSmeanRes(3,:) outputX.FSmeanRes(3,:)];
            output.FSmeanT11=[output.FSmeanT1(1,:) outputX.FSmeanT1(1,:)];
            output.FSmeanT12=[output.FSmeanT1(2,:) outputX.FSmeanT1(2,:)];
            output.FSmeanT13=[output.FSmeanT1(3,:) outputX.FSmeanT1(3,:)];
        else
            output.FSmean1=[output.FSmean(1,:)];
            output.FSmean2=[output.FSmean(2,:)];
            output.FSmean3=[output.FSmean(3,:)];
            output.FSmeanRes1=[output.FSmeanRes(1,:)];
            output.FSmeanRes2=[output.FSmeanRes(2,:)];
            output.FSmeanRes3=[output.FSmeanRes(3,:)];
            output.FSmeanT11=[output.FSmeanT1(1,:)];
            output.FSmeanT12=[output.FSmeanT1(2,:)];
            output.FSmeanT13=[output.FSmeanT1(3,:)];    
        end
        %***********************************
        %stationary nodes
        if i ==1 %1 locus
        figure(statdistfig); plot(VR,output.FSmean3,'b+-','linewidth',2); hold on; %1.no constraints
        figure(statdistfig); plot(VR,output.FSmean2,'k+-','linewidth',2); hold on; %1.random constraints
        figure(statdistfig); plot(VR,output.FSmean1,'r+-','linewidth',2); hold on; %1.box constraints
        figure(statresistfig); plot(VR,output.FSmeanRes3,'b+-','linewidth',2); hold on; %1.no constraints
        figure(statresistfig); plot(VR,output.FSmeanRes2,'k+-','linewidth',2); hold on; %1.random constraints
        figure(statresistfig); plot(VR,output.FSmeanRes1,'r+-','linewidth',2); hold on; %1.box constraints
        figure(statTfig); plot(VR,output.FSmeanT13,'b+-','linewidth',2); hold on; %1.no constraints 10m
        figure(statTfig); plot(VR,output.FSmeanT12,'k+-','linewidth',2); hold on; %1.random constraints 10m
        figure(statTfig); plot(VR,output.FSmeanT11,'r+-','linewidth',2); hold on; %1.box constraints 10m
        end
        if i ==2 %4 loci AND
        figure(statdistfig); plot(VR,output.FSmean3,'bo-','linewidth',2); hold on; %1.no constraints
        figure(statdistfig); plot(VR,output.FSmean2,'ko-','linewidth',2); hold on; %1.random constraints
        figure(statdistfig); plot(VR,output.FSmean1,'ro-','linewidth',2); hold on; %1.box constraints
        figure(statresistfig); plot(VR,output.FSmeanRes3,'bo-','linewidth',2); hold on; %1.no constraints
        figure(statresistfig); plot(VR,output.FSmeanRes2,'ko-','linewidth',2); hold on; %1.random constraints
        figure(statresistfig); plot(VR,output.FSmeanRes1,'ro-','linewidth',2); hold on; %1.box constraints
        figure(statTfig); plot(VR,output.FSmeanT13,'bo-','linewidth',2); hold on; %1.no constraints 10m
        figure(statTfig); plot(VR,output.FSmeanT12,'ko-','linewidth',2); hold on; %1.random constraints 10m
        figure(statTfig); plot(VR,output.FSmeanT11,'ro-','linewidth',2); hold on; %1.box constraints 10m
        end
        if i ==3 %4 loci OR
        figure(statdistfig); plot(VR,output.FSmean3,'bs-','linewidth',2); hold on; %1.no constraints
        figure(statdistfig); plot(VR,output.FSmean2,'ks-','linewidth',2); hold on; %1.random constraints
        figure(statdistfig); plot(VR,output.FSmean1,'rs-','linewidth',2); hold on; %1.box constraints
        figure(statresistfig); plot(VR,output.FSmeanRes3,'bs-','linewidth',2); hold on; %1.no constraints
        figure(statresistfig); plot(VR,output.FSmeanRes2,'ks-','linewidth',2); hold on; %1.random constraints
        figure(statresistfig); plot(VR,output.FSmeanRes1,'rs-','linewidth',2); hold on; %1.box constraints
        figure(statTfig); plot(VR,output.FSmeanT13,'bs-','linewidth',2); hold on; %1.no constraints 10m
        figure(statTfig); plot(VR,output.FSmeanT12,'ks-','linewidth',2); hold on; %1.random constraints 10m
        figure(statTfig); plot(VR,output.FSmeanT11,'rs-','linewidth',2); hold on; %1.box constraints 10m
        end
        %moving nodes graphs
        if i ==4 %1locus
        figure(movdistfig); plot(VR,output.FSmean3,'b+-','linewidth',2); hold on; %1.no constraints
        figure(movdistfig); plot(VR,output.FSmean2,'k+-','linewidth',2); hold on; %1.random constraints
        figure(movdistfig); plot(VR,output.FSmean1,'r+-','linewidth',2); hold on; %1.box constraints
        figure(movresistfig); plot(VR,output.FSmeanRes3,'b+-','linewidth',2); hold on; %1.no constraints
        figure(movresistfig); plot(VR,output.FSmeanRes2,'k+-','linewidth',2); hold on; %1.random constraints
        figure(movresistfig); plot(VR,output.FSmeanRes1,'r+-','linewidth',2); hold on; %1.box constraints
        figure(movTfig); plot(VR,output.FSmeanT13,'b+-','linewidth',2); hold on; %1.no constraints 10m
        figure(movTfig); plot(VR,output.FSmeanT12,'k+-','linewidth',2); hold on; %1.random constraints 10m
        figure(movTfig); plot(VR,output.FSmeanT11,'r+-','linewidth',2); hold on; %1.box constraints 10m
        end
        if i ==5 %4 loci AND
        figure(movdistfig); plot(VR,output.FSmean3,'bo-','linewidth',2); hold on; %1.no constraints
        figure(movdistfig); plot(VR,output.FSmean2,'ko-','linewidth',2); hold on; %1.random constraints
        figure(movdistfig); plot(VR,output.FSmean1,'ro-','linewidth',2); hold on; %1.box constraints
        figure(movresistfig); plot(VR,output.FSmeanRes3,'bo-','linewidth',2); hold on; %1.no constraints
        figure(movresistfig); plot(VR,output.FSmeanRes2,'ko-','linewidth',2); hold on; %1.random constraints
        figure(movresistfig); plot(VR,output.FSmeanRes1,'ro-','linewidth',2); hold on; %1.box constraints
        figure(movTfig); plot(VR,output.FSmeanT13,'bo-','linewidth',2); hold on; %1.no constraints 10m
        figure(movTfig); plot(VR,output.FSmeanT12,'ko-','linewidth',2); hold on; %1.random constraints 10m
        figure(movTfig); plot(VR,output.FSmeanT11,'ro-','linewidth',2); hold on; %1.box constraints 10m
        end
        if i ==6 %4 loci OR
        figure(movdistfig); plot(VR,output.FSmean3,'bs-','linewidth',2); hold on; %1.no constraints
        figure(movdistfig); plot(VR,output.FSmean2,'ks-','linewidth',2); hold on; %1.random constraints
        figure(movdistfig); plot(VR,output.FSmean1,'rs-','linewidth',2); hold on; %1.box constraints
        figure(movresistfig); plot(VR,output.FSmeanRes3,'bs-','linewidth',2); hold on; %1.no constraints
        figure(movresistfig); plot(VR,output.FSmeanRes2,'ks-','linewidth',2); hold on; %1.random constraints
        figure(movresistfig); plot(VR,output.FSmeanRes1,'rs-','linewidth',2); hold on; %1.box constraints
        figure(movTfig); plot(VR,output.FSmeanT13,'bs-','linewidth',2); hold on; %1.no constraints 10m
        figure(movTfig); plot(VR,output.FSmeanT12,'ks-','linewidth',2); hold on; %1.random constraints 10m
        figure(movTfig); plot(VR,output.FSmeanT11,'rs-','linewidth',2); hold on; %1.box constraints 10m
        end
    end
    %moving nodes graphs label
    figure(movdistfig); title('moving nodes'); xlabel('Variant Richness'); ylabel('Mean distance malware travelled from source'); axis([0 35 0 45]);
    legend('1.No constraints - one locus','2.Random constraints - one locus','3.Room constraints - one locus','1.No constraints - 4 loci AND','2.Random constraints - 4 loci AND','3.Room constraints - 4 loci AND','1.No constraints - 4 loci OR','2.Random constraints - 4 loci OR','3.Room constraints - 4 loci OR');
    figure(movresistfig); title('moving nodes'); xlabel('Variant Richness'); ylabel('Resistance'); legend('1.No constraints','2.Random constraints','3.Room constraints');  axis([0 35 0 1]);
    legend('1.No constraints - one locus','2.Random constraints - one locus','3.Room constraints - one locus','1.No constraints - 4 loci AND','2.Random constraints - 4 loci AND','3.Room constraints - 4 loci AND','1.No constraints - 4 loci OR','2.Random constraints - 4 loci OR','3.Room constraints - 4 loci OR');
    figure(movTfig); title('moving nodes 10m'); xlabel('Variant Richness'); ylabel('Peak infection time'); legend('1.No constraints','2.Random constraints','3.Room constraints'); axis([0 35 0 140]);
    legend('1.No constraints - one locus','2.Random constraints - one locus','3.Room constraints - one locus','1.No constraints - 4 loci AND','2.Random constraints - 4 loci AND','3.Room constraints - 4 loci AND','1.No constraints - 4 loci OR','2.Random constraints - 4 loci OR','3.Room constraints - 4 loci OR');
    saveas(movdistfig,[outputDirectory 'movdistfig.fig']);
    saveas(movdistfig,[outputDirectory 'movdistfig.emf']);
    saveas(movresistfig,[outputDirectory 'movresistfig.fig']);
    saveas(movresistfig,[outputDirectory 'movresistfig.emf']);
    saveas(movTfig,[outputDirectory 'movTfig.fig']);
    saveas(movTfig,[outputDirectory 'movTfig.emf']);
    
    %stationary nodes graphs label
    figure(statdistfig); title('stationary nodes (10m)'); xlabel('Variant Richness'); ylabel('Mean distance malware travelled from source'); legend('1.No constraints','2.Random constraints','3.Room constraints'); axis([0 35 0 45]);
    legend('1.No constraints - one locus','2.Random constraints - one locus','3.Room constraints - one locus','1.No constraints - 4 loci AND','2.Random constraints - 4 loci AND','3.Room constraints - 4 loci AND','1.No constraints - 4 loci OR','2.Random constraints - 4 loci OR','3.Room constraints - 4 loci OR');
    figure(statresistfig); title('stationary nodes'); xlabel('Variant Richness'); ylabel('Resistance'); legend('1.No constraints','2.Random constraints','3.Room constraints');  axis([0 35 0 1]);
    legend('1.No constraints - one locus','2.Random constraints - one locus','3.Room constraints - one locus','1.No constraints - 4 loci AND','2.Random constraints - 4 loci AND','3.Room constraints - 4 loci AND','1.No constraints - 4 loci OR','2.Random constraints - 4 loci OR','3.Room constraints - 4 loci OR');
    figure(statTfig); title('stationary nodes 10m'); xlabel('Variant Richness'); ylabel('Peak infection time'); legend('1.No constraints','2.Random constraints','3.Room constraints'); axis([0 35 0 140]);
    legend('1.No constraints - one locus','2.Random constraints - one locus','3.Room constraints - one locus','1.No constraints - 4 loci AND','2.Random constraints - 4 loci AND','3.Room constraints - 4 loci AND','1.No constraints - 4 loci OR','2.Random constraints - 4 loci OR','3.Room constraints - 4 loci OR');
    saveas(statdistfig,[outputDirectory 'statdistfig.fig']);
    saveas(statdistfig,[outputDirectory 'statdistfig.emf']);
    saveas(statresistfig,[outputDirectory 'statresistfig.fig']);
    saveas(statresistfig,[outputDirectory 'statresistfig.emf']);
    saveas(statTfig,[outputDirectory 'statTfig.fig']);
    saveas(statTfig,[outputDirectory 'statTfig.emf']);    
end %317



%moving nodes -  4 loci OR
if doSIRNTests(316) == 1
data.m = 316 %data file number

%1 locus model
%outputDirectory = 'Graphs\'
outputDirectory = 'Graphs\'
tic

modePar.netStr = 'rwp'; %'rwp' random waypoint 're' random encounter
%rwp par
modePar.genotypeTimeOutPeriod = 10; %rwp
modePar.xminln   = 1;     % Minimum x-axis location limited nodes (du)
modePar.xmaxln   = 16;    % Maximum x-axis location limited nodes (du)
modePar.yminln   = 1;     % Minimum y-axis location limited nodes (du)
modePar.ymaxln   = 16;    % Maximum y-axis location limited nodes (du)
modePar.xmin     = 1;     % Minimum x-axis location (du)
modePar.xmax     = 31;    % Maximum x-axis location (du)
modePar.ymin     = 1;     % Minimum y-axis location (du)
modePar.ymax     = 31;    % Maximum y-axis location (du)
modePar.smin     = 1.4;  % Minimum speed (du / tu)
modePar.smax     = 1.4;  % Maximum speed (du / tu)
modePar.pmin     = 0;     % Minimum time for pause (integer tu)
modePar.pmax     = 0;     % Maximum time for pause (integer tu)
%modePar.dinr     = 3; %7    % Radial distance for in-range (du)
modePar.dexceed  = 1000;  % Value larger than any possible distance (du)
modePar.tinrg    = 1;     % Time in-range for genotype transfer (tu)
modePar.tinrv    = 1;     % Time in-range for virus transfer (tu)
% Mode
% rwph.mode = rwph.MODE_RIR;  1    % Select Random in range mode
% rwph.mode = rwph.MODE_NIR;  2    % Select Nearest in range mode
modePar.moderwp = 1; %MODE_AIR;  % 4    % Select (Random) Available in range mode
%rwp constrain
modePar.xboxc=0.5
modePar.yboxc=0.5
modePar.rwpcontrainplotGraph=0 %draw init constraints graph
% Log vector
modePar.logena = [1 2];
% Log directory for rwp
modePar.logDir = sprintf('%s\\log', pwd);
%data logging of output
data.logDir = sprintf('%s\\outputdata', pwd)
s = sprintf('%s\\data%i.txt', data.logDir,data.m)
data.logOutput(data.m) = fopen(s, 'wt');
%numEncountersBetweenUpdates = [2 5 10 20 50]
numEncountersBetweenUpdates = [1]
modePar.bufferlength = 10 %length of genotype buffer for favscore algorithm
modePar.genoInit = 'mindiv'  %initial conditions of genotypes maxdiv (maximum diversity - random variant chosen) mindiv (minimum diversity - all nodes same)
%blacklist mode
modePar.blklist = 'no'; %yes or no
modePar.numToBlklist = 1
modePar.blklistrate = 0.004 %0.001
%update mode
modePar.update = 'yes' %stop updating mode
modePar.bufferlengthNei=10 %buffer for buffering local nei diversity,
modePar.stdNeiUpdateThreshold = 0.15
%recovery
recovery = 0; 0.0005 %0.001

%main par
numRuns = 30 %20 %100 %200
numNodes = 100;
numLoci=4; 
casetype = 'OR';
modePar.virusStartTime = 140;
numTimestepsVR = [350];
range = [1 3 5 7 10];
VR= [1 2 4 6 8 10 12 14 16 18 20 25 30 35]; %316
modePar.statnodesrwp = 'stationary'; %'dynamic' or 'stationary' nodes, if dynamic box constraints applied at point of virus start
Imean = zeros(1,numTimestepsVR(1)); %init if runs all tests

%**********************
%1. rwp box constraints
modePar.numToConstrain = 0;%fix(numNodes*1/4); %constain a number of nodes to 1 variant
modePar.areaConstrainrwp='yes'
modePar.nodesln  = fix(numNodes/4);     % Number of limited nodes (absolute) - always first
%**********************
mode = 'Dynamic-favScore' %mode - can be 'static' or 'dynamic' 'Dynamic-random' 'Dynamic-favScore'
%monte carlo random genotype SIR model
%set up diversity mode parameters
modePar.modeStr = mode
modePar.stopUpdatingTime = 200000 %time at which to stop updating the genotypes
for j = 1:size(VR,2)%do for all VR
    variantRichness =  VR(j)
    numTimesteps = numTimestepsVR(1);
for z= 1:size(range,2)
    modePar.dinr = range(z);
    modePar.numEncountersBetweenUpdates = numEncountersBetweenUpdates(1);
 betan = 0.02;  plotGraphs = 0; gamma = recovery; numExploits=1; throwAway = 'nonEpidemics', SEpidemic=60; %cutoff;
 %agent
 [figMeanNei figNeiOneRun fighistSpRun fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetanew fighistGammanew figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            meanNeiIndexDynamic betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean gammameanmean betameanmean2 betameanmeanNew betameanmeanNew2 gammameanmeanNew Smean Imean Rmean Bmean NSmean Smedian Imedian Rmedian S I R NS SmeanN ImeanN1 RmeanN1 BmeanN1 Snewendmean Snewendstd Rnewendmean Rnewendstd Rnew_nonendmean Rnew_nonendstd propSimsNonEp propSimsNonEp_theoretical...
            numRunsMinor measuredProbMinor numRunsMajor measuredProbMajor meanmdist meannumInfectedVM...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,modePar, numLoci, casetype);
   close all; 
   
    %peak time
    FinalSize = max(Imean,[],2); %mean final size SI
    maska = Imean >= FinalSize-1; %when within 1 of the final value
    [C Index] = max(maska);
    MLa(j,z) = Index; %T1
    %resistance
    Resistance = (numNodes-FinalSize)./numNodes;
   
%data logging
 fprintf(data.logOutput(data.m), '%f,', Imean);     % infection
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', meanmdist); %distance spread
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Index);     %T1
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Resistance); %resistance
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', FinalSize); %final size
fprintf(data.logOutput(data.m), '\n'); %new line
end %range
end %VR

mode = 'Dynamic-random' %mode - can be 'static' or 'dynamic' 'Dynamic-random' 'Dynamic-favScore'
%monte carlo random genotype SIR model
%set up diversity mode parameters
modePar.modeStr = mode
modePar.stopUpdatingTime = modePar.virusStartTime %time at which to stop updating the genotypes
for j = 1:size(VR,2)%do for all VR
    variantRichness =  VR(j)
    numTimesteps = numTimestepsVR(1);
for z= 1:size(range,2)
    modePar.dinr = range(z);
    modePar.numEncountersBetweenUpdates = numEncountersBetweenUpdates(1);
 betan = 0.02;  plotGraphs = 0; gamma = recovery; numExploits=1; throwAway = 'nonEpidemics', SEpidemic=60; %cutoff;
 %agent
 [figMeanNei figNeiOneRun fighistSpRun fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetanew fighistGammanew figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            meanNeiIndexDynamic betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean gammameanmean betameanmean2 betameanmeanNew betameanmeanNew2 gammameanmeanNew Smean Imean Rmean Bmean NSmean Smedian Imedian Rmedian S I R NS SmeanN ImeanN1 RmeanN1 BmeanN1 Snewendmean Snewendstd Rnewendmean Rnewendstd Rnew_nonendmean Rnew_nonendstd propSimsNonEp propSimsNonEp_theoretical...
            numRunsMinor measuredProbMinor numRunsMajor measuredProbMajor meanmdist meannumInfectedVM...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,modePar, numLoci, casetype);
   close all; 
   
    %peak time
    FinalSize = max(Imean,[],2); %mean final size SI
    maska = Imean >= FinalSize-1; %when within 1 of the final value
    [C Index] = max(maska);
    MLa(j,z) = Index; %T1
    %resistance
    Resistance = (numNodes-FinalSize)./numNodes;
   
%data logging
 fprintf(data.logOutput(data.m), '%f,', Imean);     % infection
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', meanmdist); %distance spread
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Index);     %T1
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Resistance); %resistance
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', FinalSize); %final size
fprintf(data.logOutput(data.m), '\n'); %new line
end %range
end %VR

%**********************
%2. random constraints
modePar.numToConstrain = fix(numNodes*1/4); %constain a number of nodes to 1 variant
modePar.areaConstrainrwp='no'
modePar.nodesln  = 0;     % Number of limited nodes (absolute) - always first
%**********************
mode = 'Dynamic-favScore' %mode - can be 'static' or 'dynamic' 'Dynamic-random' 'Dynamic-favScore'
%monte carlo random genotype SIR model
%set up diversity mode parameters
modePar.modeStr = mode
modePar.stopUpdatingTime = 200000 %time at which to stop updating the genotypes
for j = 1:size(VR,2)%do for all VR
    variantRichness =  VR(j)
    numTimesteps = numTimestepsVR(1);
for z= 1:size(range,2)
    modePar.dinr = range(z);
    modePar.numEncountersBetweenUpdates = numEncountersBetweenUpdates(1);
 betan = 0.02;  plotGraphs = 0; gamma = recovery; numExploits=1; throwAway = 'nonEpidemics', SEpidemic=60; %cutoff;
 %agent
 [figMeanNei figNeiOneRun fighistSpRun fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetanew fighistGammanew figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            meanNeiIndexDynamic betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean gammameanmean betameanmean2 betameanmeanNew betameanmeanNew2 gammameanmeanNew Smean Imean Rmean Bmean NSmean Smedian Imedian Rmedian S I R NS SmeanN ImeanN1 RmeanN1 BmeanN1 Snewendmean Snewendstd Rnewendmean Rnewendstd Rnew_nonendmean Rnew_nonendstd propSimsNonEp propSimsNonEp_theoretical...
            numRunsMinor measuredProbMinor numRunsMajor measuredProbMajor meanmdist meannumInfectedVM...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,modePar, numLoci, casetype);
   close all; 
   
    %peak time
    FinalSize = max(Imean,[],2); %mean final size SI
    maska = Imean >= FinalSize-1; %when within 1 of the final value
    [C Index] = max(maska);
    MLa(j,z) = Index; %T1
    %resistance
    Resistance = (numNodes-FinalSize)./numNodes;
   
%data logging
 fprintf(data.logOutput(data.m), '%f,', Imean);     % infection
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', meanmdist); %distance spread
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Index);     %T1
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Resistance); %resistance
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', FinalSize); %final size
fprintf(data.logOutput(data.m), '\n'); %new line
end %range
end %VR

mode = 'Dynamic-random' %mode - can be 'static' or 'dynamic' 'Dynamic-random' 'Dynamic-favScore'
%monte carlo random genotype SIR model
%set up diversity mode parameters
modePar.modeStr = mode
modePar.stopUpdatingTime = modePar.virusStartTime %time at which to stop updating the genotypes
for j = 1:size(VR,2)%do for all VR
    variantRichness =  VR(j)
    numTimesteps = numTimestepsVR(1);
for z= 1:size(range,2)
    modePar.dinr = range(z);
    modePar.numEncountersBetweenUpdates = numEncountersBetweenUpdates(1);
 betan = 0.02;  plotGraphs = 0; gamma = recovery; numExploits=1; throwAway = 'nonEpidemics', SEpidemic=60; %cutoff;
 %agent
 [figMeanNei figNeiOneRun fighistSpRun fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetanew fighistGammanew figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            meanNeiIndexDynamic betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean gammameanmean betameanmean2 betameanmeanNew betameanmeanNew2 gammameanmeanNew Smean Imean Rmean Bmean NSmean Smedian Imedian Rmedian S I R NS SmeanN ImeanN1 RmeanN1 BmeanN1 Snewendmean Snewendstd Rnewendmean Rnewendstd Rnew_nonendmean Rnew_nonendstd propSimsNonEp propSimsNonEp_theoretical...
            numRunsMinor measuredProbMinor numRunsMajor measuredProbMajor meanmdist meannumInfectedVM...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,modePar, numLoci, casetype);
   close all; 
   
    %peak time
    FinalSize = max(Imean,[],2); %mean final size SI
    maska = Imean >= FinalSize-1; %when within 1 of the final value
    [C Index] = max(maska);
    MLa(j,z) = Index; %T1
    %resistance
    Resistance = (numNodes-FinalSize)./numNodes;
   
%data logging
 fprintf(data.logOutput(data.m), '%f,', Imean);     % infection
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', meanmdist); %distance spread
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Index);     %T1
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Resistance); %resistance
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', FinalSize); %final size
fprintf(data.logOutput(data.m), '\n'); %new line
end %range
end %VR

%**********************
%3. no constraints
modePar.numToConstrain = 0; %fix(numNodes*1/4); %constain a number of nodes to 1 variant
modePar.areaConstrainrwp='no'
modePar.nodesln  = 0;     % Number of limited nodes (absolute) - always first
%**********************
mode = 'Dynamic-favScore' %mode - can be 'static' or 'dynamic' 'Dynamic-random' 'Dynamic-favScore'
%monte carlo random genotype SIR model
%set up diversity mode parameters
modePar.modeStr = mode
modePar.stopUpdatingTime = 200000 %time at which to stop updating the genotypes
for j = 1:size(VR,2)%do for all VR
    variantRichness =  VR(j)
    numTimesteps = numTimestepsVR(1);
for z= 1:size(range,2)
    modePar.dinr = range(z);
    modePar.numEncountersBetweenUpdates = numEncountersBetweenUpdates(1);
 betan = 0.02;  plotGraphs = 0; gamma = recovery; numExploits=1; throwAway = 'nonEpidemics', SEpidemic=60; %cutoff;
 %agent
 [figMeanNei figNeiOneRun fighistSpRun fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetanew fighistGammanew figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            meanNeiIndexDynamic betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean gammameanmean betameanmean2 betameanmeanNew betameanmeanNew2 gammameanmeanNew Smean Imean Rmean Bmean NSmean Smedian Imedian Rmedian S I R NS SmeanN ImeanN1 RmeanN1 BmeanN1 Snewendmean Snewendstd Rnewendmean Rnewendstd Rnew_nonendmean Rnew_nonendstd propSimsNonEp propSimsNonEp_theoretical...
            numRunsMinor measuredProbMinor numRunsMajor measuredProbMajor meanmdist meannumInfectedVM...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,modePar, numLoci, casetype);
   close all; 
   
    %peak time
    FinalSize = max(Imean,[],2); %mean final size SI
    maska = Imean >= FinalSize-1; %when within 1 of the final value
    [C Index] = max(maska);
    MLa(j,z) = Index; %T1
    %resistance
    Resistance = (numNodes-FinalSize)./numNodes;
   
%data logging
 fprintf(data.logOutput(data.m), '%f,', Imean);     % infection
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', meanmdist); %distance spread
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Index);     %T1
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Resistance); %resistance
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', FinalSize); %final size
fprintf(data.logOutput(data.m), '\n'); %new line
end %range
end %VR

mode = 'Dynamic-random' %mode - can be 'static' or 'dynamic' 'Dynamic-random' 'Dynamic-favScore'
%monte carlo random genotype SIR model
%set up diversity mode parameters
modePar.modeStr = mode
modePar.stopUpdatingTime = modePar.virusStartTime %time at which to stop updating the genotypes
for j = 1:size(VR,2)%do for all VR
    variantRichness =  VR(j)
    numTimesteps = numTimestepsVR(1);
for z= 1:size(range,2)
    modePar.dinr = range(z);
    modePar.numEncountersBetweenUpdates = numEncountersBetweenUpdates(1);
 betan = 0.02;  plotGraphs = 0; gamma = recovery; numExploits=1; throwAway = 'nonEpidemics', SEpidemic=60; %cutoff;
 %agent
 [figMeanNei figNeiOneRun fighistSpRun fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetanew fighistGammanew figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            meanNeiIndexDynamic betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean gammameanmean betameanmean2 betameanmeanNew betameanmeanNew2 gammameanmeanNew Smean Imean Rmean Bmean NSmean Smedian Imedian Rmedian S I R NS SmeanN ImeanN1 RmeanN1 BmeanN1 Snewendmean Snewendstd Rnewendmean Rnewendstd Rnew_nonendmean Rnew_nonendstd propSimsNonEp propSimsNonEp_theoretical...
            numRunsMinor measuredProbMinor numRunsMajor measuredProbMajor meanmdist meannumInfectedVM...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,modePar, numLoci, casetype);
   close all; 
   
    %peak time
    FinalSize = max(Imean,[],2); %mean final size SI
    maska = Imean >= FinalSize-1; %when within 1 of the final value
    [C Index] = max(maska);
    MLa(j,z) = Index; %T1
    %resistance
    Resistance = (numNodes-FinalSize)./numNodes;
   
%data logging
 fprintf(data.logOutput(data.m), '%f,', Imean);     % infection
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', meanmdist); %distance spread
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Index);     %T1
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Resistance); %resistance
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', FinalSize); %final size
fprintf(data.logOutput(data.m), '\n'); %new line
end %range
end %VR



%close data logger
fclose(data.logOutput(data.m));

toc
end%316


%dynamic nodes -  4 loci AND
if doSIRNTests(315) == 1
data.m = 315 %data file number
%1 locus model
%outputDirectory = 'Graphs\'
outputDirectory = 'Graphs\'
tic
modePar.netStr = 'rwp'; %'rwp' random waypoint 're' random encounter
%rwp par
modePar.genotypeTimeOutPeriod = 10; %rwp
modePar.xminln   = 1;     % Minimum x-axis location limited nodes (du)
modePar.xmaxln   = 16;    % Maximum x-axis location limited nodes (du)
modePar.yminln   = 1;     % Minimum y-axis location limited nodes (du)
modePar.ymaxln   = 16;    % Maximum y-axis location limited nodes (du)
modePar.xmin     = 1;     % Minimum x-axis location (du)
modePar.xmax     = 31;    % Maximum x-axis location (du)
modePar.ymin     = 1;     % Minimum y-axis location (du)
modePar.ymax     = 31;    % Maximum y-axis location (du)
modePar.smin     = 1.4;  % Minimum speed (du / tu)
modePar.smax     = 1.4;  % Maximum speed (du / tu)
modePar.pmin     = 0;     % Minimum time for pause (integer tu)
modePar.pmax     = 0;     % Maximum time for pause (integer tu)
%modePar.dinr     = 3; %7    % Radial distance for in-range (du)
modePar.dexceed  = 1000;  % Value larger than any possible distance (du)
modePar.tinrg    = 1;     % Time in-range for genotype transfer (tu)
modePar.tinrv    = 1;     % Time in-range for virus transfer (tu)
% Mode
% rwph.mode = rwph.MODE_RIR;  1    % Select Random in range mode
% rwph.mode = rwph.MODE_NIR;  2    % Select Nearest in range mode
modePar.moderwp = 1; %MODE_AIR;  % 4    % Select (Random) Available in range mode
%rwp constrain
modePar.xboxc=0.5
modePar.yboxc=0.5
modePar.rwpcontrainplotGraph=0 %draw init constraints graph
% Log vector
modePar.logena = [1 2];
% Log directory for rwp
modePar.logDir = sprintf('%s\\log', pwd);
%data logging of output
data.logDir = sprintf('%s\\outputdata', pwd)
s = sprintf('%s\\data%i.txt', data.logDir,data.m)
data.logOutput(data.m) = fopen(s, 'wt');
%numEncountersBetweenUpdates = [2 5 10 20 50]
numEncountersBetweenUpdates = [1]
modePar.bufferlength = 10 %length of genotype buffer for favscore algorithm
modePar.genoInit = 'mindiv'  %initial conditions of genotypes maxdiv (maximum diversity - random variant chosen) mindiv (minimum diversity - all nodes same)
%blacklist mode
modePar.blklist = 'no'; %yes or no
modePar.numToBlklist = 1
modePar.blklistrate = 0.004 %0.001
%update mode
modePar.update = 'yes' %stop updating mode
modePar.bufferlengthNei=10 %buffer for buffering local nei diversity,
modePar.stdNeiUpdateThreshold = 0.15
%recovery
recovery = 0; 0.0005 %0.001

%main par
numRuns = 30 %20 %100 %200
numNodes = 100;
numLoci=4; 
casetype = 'AND';
modePar.virusStartTime = 140;
numTimestepsVR = [350];
range = [1 3 5 7 10];
 VR=[1 2 3 4 5 6 7]; %315
modePar.statnodesrwp = 'stationary'; %'dynamic' or 'stationary' nodes, if dynamic box constraints applied at point of virus start
Imean = zeros(1,numTimestepsVR(1)); %init if runs all tests
%**********************
%1. rwp box constraints
modePar.numToConstrain = 0;%fix(numNodes*1/4); %constain a number of nodes to 1 variant
modePar.areaConstrainrwp='yes'
modePar.nodesln  = fix(numNodes/4);     % Number of limited nodes (absolute) - always first
%**********************
mode = 'Dynamic-favScore' %mode - can be 'static' or 'dynamic' 'Dynamic-random' 'Dynamic-favScore'
%monte carlo random genotype SIR model
%set up diversity mode parameters
modePar.modeStr = mode
modePar.stopUpdatingTime = 200000 %time at which to stop updating the genotypes
for j = 1:size(VR,2)%do for all VR
    variantRichness =  VR(j)
    numTimesteps = numTimestepsVR(1);
for z= 1:size(range,2)
    modePar.dinr = range(z);
    modePar.numEncountersBetweenUpdates = numEncountersBetweenUpdates(1);
 betan = 0.02;  plotGraphs = 0; gamma = recovery; numExploits=1; throwAway = 'nonEpidemics', SEpidemic=60; %cutoff;
 %agent
 [figMeanNei figNeiOneRun fighistSpRun fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetanew fighistGammanew figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            meanNeiIndexDynamic betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean gammameanmean betameanmean2 betameanmeanNew betameanmeanNew2 gammameanmeanNew Smean Imean Rmean Bmean NSmean Smedian Imedian Rmedian S I R NS SmeanN ImeanN1 RmeanN1 BmeanN1 Snewendmean Snewendstd Rnewendmean Rnewendstd Rnew_nonendmean Rnew_nonendstd propSimsNonEp propSimsNonEp_theoretical...
            numRunsMinor measuredProbMinor numRunsMajor measuredProbMajor meanmdist meannumInfectedVM...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,modePar, numLoci, casetype);
   close all; 
   
    %peak time
    FinalSize = max(Imean,[],2); %mean final size SI
    maska = Imean >= FinalSize-1; %when within 1 of the final value
    [C Index] = max(maska);
    MLa(j,z) = Index; %T1
    %resistance
    Resistance = (numNodes-FinalSize)./numNodes;
   
%data logging
 fprintf(data.logOutput(data.m), '%f,', Imean);     % infection
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', meanmdist); %distance spread
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Index);     %T1
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Resistance); %resistance
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', FinalSize); %final size
fprintf(data.logOutput(data.m), '\n'); %new line
end %range
end %VR

mode = 'Dynamic-random' %mode - can be 'static' or 'dynamic' 'Dynamic-random' 'Dynamic-favScore'
%monte carlo random genotype SIR model
%set up diversity mode parameters
modePar.modeStr = mode
modePar.stopUpdatingTime = modePar.virusStartTime %time at which to stop updating the genotypes
for j = 1:size(VR,2)%do for all VR
    variantRichness =  VR(j)
    numTimesteps = numTimestepsVR(1);
for z= 1:size(range,2)
    modePar.dinr = range(z);
    modePar.numEncountersBetweenUpdates = numEncountersBetweenUpdates(1);
 betan = 0.02;  plotGraphs = 0; gamma = recovery; numExploits=1; throwAway = 'nonEpidemics', SEpidemic=60; %cutoff;
 %agent
 [figMeanNei figNeiOneRun fighistSpRun fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetanew fighistGammanew figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            meanNeiIndexDynamic betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean gammameanmean betameanmean2 betameanmeanNew betameanmeanNew2 gammameanmeanNew Smean Imean Rmean Bmean NSmean Smedian Imedian Rmedian S I R NS SmeanN ImeanN1 RmeanN1 BmeanN1 Snewendmean Snewendstd Rnewendmean Rnewendstd Rnew_nonendmean Rnew_nonendstd propSimsNonEp propSimsNonEp_theoretical...
            numRunsMinor measuredProbMinor numRunsMajor measuredProbMajor meanmdist meannumInfectedVM...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,modePar, numLoci, casetype);
   close all; 
   
    %peak time
    FinalSize = max(Imean,[],2); %mean final size SI
    maska = Imean >= FinalSize-1; %when within 1 of the final value
    [C Index] = max(maska);
    MLa(j,z) = Index; %T1
    %resistance
    Resistance = (numNodes-FinalSize)./numNodes;
   
%data logging
 fprintf(data.logOutput(data.m), '%f,', Imean);     % infection
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', meanmdist); %distance spread
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Index);     %T1
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Resistance); %resistance
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', FinalSize); %final size
fprintf(data.logOutput(data.m), '\n'); %new line
end %range
end %VR

%**********************
%2. random constraints
modePar.numToConstrain = fix(numNodes*1/4); %constain a number of nodes to 1 variant
modePar.areaConstrainrwp='no'
modePar.nodesln  = 0;     % Number of limited nodes (absolute) - always first
%**********************
mode = 'Dynamic-favScore' %mode - can be 'static' or 'dynamic' 'Dynamic-random' 'Dynamic-favScore'
%monte carlo random genotype SIR model
%set up diversity mode parameters
modePar.modeStr = mode
modePar.stopUpdatingTime = 200000 %time at which to stop updating the genotypes
for j = 1:size(VR,2)%do for all VR
    variantRichness =  VR(j)
    numTimesteps = numTimestepsVR(1);
for z= 1:size(range,2)
    modePar.dinr = range(z);
    modePar.numEncountersBetweenUpdates = numEncountersBetweenUpdates(1);
 betan = 0.02;  plotGraphs = 0; gamma = recovery; numExploits=1; throwAway = 'nonEpidemics', SEpidemic=60; %cutoff;
 %agent
 [figMeanNei figNeiOneRun fighistSpRun fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetanew fighistGammanew figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            meanNeiIndexDynamic betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean gammameanmean betameanmean2 betameanmeanNew betameanmeanNew2 gammameanmeanNew Smean Imean Rmean Bmean NSmean Smedian Imedian Rmedian S I R NS SmeanN ImeanN1 RmeanN1 BmeanN1 Snewendmean Snewendstd Rnewendmean Rnewendstd Rnew_nonendmean Rnew_nonendstd propSimsNonEp propSimsNonEp_theoretical...
            numRunsMinor measuredProbMinor numRunsMajor measuredProbMajor meanmdist meannumInfectedVM...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,modePar, numLoci, casetype);
   close all; 
   
    %peak time
    FinalSize = max(Imean,[],2); %mean final size SI
    maska = Imean >= FinalSize-1; %when within 1 of the final value
    [C Index] = max(maska);
    MLa(j,z) = Index; %T1
    %resistance
    Resistance = (numNodes-FinalSize)./numNodes;
   
%data logging
 fprintf(data.logOutput(data.m), '%f,', Imean);     % infection
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', meanmdist); %distance spread
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Index);     %T1
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Resistance); %resistance
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', FinalSize); %final size
fprintf(data.logOutput(data.m), '\n'); %new line
end %range
end %VR

mode = 'Dynamic-random' %mode - can be 'static' or 'dynamic' 'Dynamic-random' 'Dynamic-favScore'
%monte carlo random genotype SIR model
%set up diversity mode parameters
modePar.modeStr = mode
modePar.stopUpdatingTime = modePar.virusStartTime %time at which to stop updating the genotypes
for j = 1:size(VR,2)%do for all VR
    variantRichness =  VR(j)
    numTimesteps = numTimestepsVR(1);
for z= 1:size(range,2)
    modePar.dinr = range(z);
    modePar.numEncountersBetweenUpdates = numEncountersBetweenUpdates(1);
 betan = 0.02;  plotGraphs = 0; gamma = recovery; numExploits=1; throwAway = 'nonEpidemics', SEpidemic=60; %cutoff;
 %agent
 [figMeanNei figNeiOneRun fighistSpRun fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetanew fighistGammanew figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            meanNeiIndexDynamic betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean gammameanmean betameanmean2 betameanmeanNew betameanmeanNew2 gammameanmeanNew Smean Imean Rmean Bmean NSmean Smedian Imedian Rmedian S I R NS SmeanN ImeanN1 RmeanN1 BmeanN1 Snewendmean Snewendstd Rnewendmean Rnewendstd Rnew_nonendmean Rnew_nonendstd propSimsNonEp propSimsNonEp_theoretical...
            numRunsMinor measuredProbMinor numRunsMajor measuredProbMajor meanmdist meannumInfectedVM...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,modePar, numLoci, casetype);
   close all; 
   
    %peak time
    FinalSize = max(Imean,[],2); %mean final size SI
    maska = Imean >= FinalSize-1; %when within 1 of the final value
    [C Index] = max(maska);
    MLa(j,z) = Index; %T1
    %resistance
    Resistance = (numNodes-FinalSize)./numNodes;
   
%data logging
 fprintf(data.logOutput(data.m), '%f,', Imean);     % infection
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', meanmdist); %distance spread
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Index);     %T1
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Resistance); %resistance
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', FinalSize); %final size
fprintf(data.logOutput(data.m), '\n'); %new line
end %range
end %VR

%**********************
%3. no constraints
modePar.numToConstrain = 0; %fix(numNodes*1/4); %constain a number of nodes to 1 variant
modePar.areaConstrainrwp='no'
modePar.nodesln  = 0;     % Number of limited nodes (absolute) - always first
%**********************
mode = 'Dynamic-favScore' %mode - can be 'static' or 'dynamic' 'Dynamic-random' 'Dynamic-favScore'
%monte carlo random genotype SIR model
%set up diversity mode parameters
modePar.modeStr = mode
modePar.stopUpdatingTime = 200000 %time at which to stop updating the genotypes
for j = 1:size(VR,2)%do for all VR
    variantRichness =  VR(j)
    numTimesteps = numTimestepsVR(1);
for z= 1:size(range,2)
    modePar.dinr = range(z);
    modePar.numEncountersBetweenUpdates = numEncountersBetweenUpdates(1);
 betan = 0.02;  plotGraphs = 0; gamma = recovery; numExploits=1; throwAway = 'nonEpidemics', SEpidemic=60; %cutoff;
 %agent
 [figMeanNei figNeiOneRun fighistSpRun fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetanew fighistGammanew figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            meanNeiIndexDynamic betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean gammameanmean betameanmean2 betameanmeanNew betameanmeanNew2 gammameanmeanNew Smean Imean Rmean Bmean NSmean Smedian Imedian Rmedian S I R NS SmeanN ImeanN1 RmeanN1 BmeanN1 Snewendmean Snewendstd Rnewendmean Rnewendstd Rnew_nonendmean Rnew_nonendstd propSimsNonEp propSimsNonEp_theoretical...
            numRunsMinor measuredProbMinor numRunsMajor measuredProbMajor meanmdist meannumInfectedVM...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,modePar, numLoci, casetype);
   close all; 
   
    %peak time
    FinalSize = max(Imean,[],2); %mean final size SI
    maska = Imean >= FinalSize-1; %when within 1 of the final value
    [C Index] = max(maska);
    MLa(j,z) = Index; %T1
    %resistance
    Resistance = (numNodes-FinalSize)./numNodes;
   
%data logging
 fprintf(data.logOutput(data.m), '%f,', Imean);     % infection
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', meanmdist); %distance spread
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Index);     %T1
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Resistance); %resistance
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', FinalSize); %final size
fprintf(data.logOutput(data.m), '\n'); %new line
end %range
end %VR

mode = 'Dynamic-random' %mode - can be 'static' or 'dynamic' 'Dynamic-random' 'Dynamic-favScore'
%monte carlo random genotype SIR model
%set up diversity mode parameters
modePar.modeStr = mode
modePar.stopUpdatingTime = modePar.virusStartTime %time at which to stop updating the genotypes
for j = 1:size(VR,2)%do for all VR
    variantRichness =  VR(j)
    numTimesteps = numTimestepsVR(1);
for z= 1:size(range,2)
    modePar.dinr = range(z);
    modePar.numEncountersBetweenUpdates = numEncountersBetweenUpdates(1);
 betan = 0.02;  plotGraphs = 0; gamma = recovery; numExploits=1; throwAway = 'nonEpidemics', SEpidemic=60; %cutoff;
 %agent
 [figMeanNei figNeiOneRun fighistSpRun fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetanew fighistGammanew figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            meanNeiIndexDynamic betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean gammameanmean betameanmean2 betameanmeanNew betameanmeanNew2 gammameanmeanNew Smean Imean Rmean Bmean NSmean Smedian Imedian Rmedian S I R NS SmeanN ImeanN1 RmeanN1 BmeanN1 Snewendmean Snewendstd Rnewendmean Rnewendstd Rnew_nonendmean Rnew_nonendstd propSimsNonEp propSimsNonEp_theoretical...
            numRunsMinor measuredProbMinor numRunsMajor measuredProbMajor meanmdist meannumInfectedVM...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,modePar, numLoci, casetype);
   close all; 
   
    %peak time
    FinalSize = max(Imean,[],2); %mean final size SI
    maska = Imean >= FinalSize-1; %when within 1 of the final value
    [C Index] = max(maska);
    MLa(j,z) = Index; %T1
    %resistance
    Resistance = (numNodes-FinalSize)./numNodes;
   
%data logging
 fprintf(data.logOutput(data.m), '%f,', Imean);     % infection
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', meanmdist); %distance spread
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Index);     %T1
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Resistance); %resistance
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', FinalSize); %final size
fprintf(data.logOutput(data.m), '\n'); %new line
end %range
end %VR

%close data logger
fclose(data.logOutput(data.m));

toc
end%315


%dynamic nodes -  1 locus
if doSIRNTests(314) == 1
data.m = 314 %data file number
%1 locus model
%outputDirectory = 'Graphs\'
outputDirectory = 'Graphs\'
tic
modePar.netStr = 'rwp'; %'rwp' random waypoint 're' random encounter
%rwp par
modePar.genotypeTimeOutPeriod = 10; %rwp
modePar.xminln   = 1;     % Minimum x-axis location limited nodes (du)
modePar.xmaxln   = 16;    % Maximum x-axis location limited nodes (du)
modePar.yminln   = 1;     % Minimum y-axis location limited nodes (du)
modePar.ymaxln   = 16;    % Maximum y-axis location limited nodes (du)
modePar.xmin     = 1;     % Minimum x-axis location (du)
modePar.xmax     = 31;    % Maximum x-axis location (du)
modePar.ymin     = 1;     % Minimum y-axis location (du)
modePar.ymax     = 31;    % Maximum y-axis location (du)
modePar.smin     = 1.4;  % Minimum speed (du / tu)
modePar.smax     = 1.4;  % Maximum speed (du / tu)
modePar.pmin     = 0;     % Minimum time for pause (integer tu)
modePar.pmax     = 0;     % Maximum time for pause (integer tu)
%modePar.dinr     = 3; %7    % Radial distance for in-range (du)
modePar.dexceed  = 1000;  % Value larger than any possible distance (du)
modePar.tinrg    = 1;     % Time in-range for genotype transfer (tu)
modePar.tinrv    = 1;     % Time in-range for virus transfer (tu)
% Mode
% rwph.mode = rwph.MODE_RIR;  1    % Select Random in range mode
% rwph.mode = rwph.MODE_NIR;  2    % Select Nearest in range mode
modePar.moderwp = 1; %MODE_AIR;  % 4    % Select (Random) Available in range mode
%rwp constrain
modePar.xboxc=0.5
modePar.yboxc=0.5
modePar.rwpcontrainplotGraph=0 %draw init constraints graph
% Log vector
modePar.logena = [1 2];
% Log directory for rwp
modePar.logDir = sprintf('%s\\log', pwd);
%data logging of output
data.logDir = sprintf('%s\\outputdata', pwd)
s = sprintf('%s\\data%i.txt', data.logDir,data.m)
data.logOutput(data.m) = fopen(s, 'wt');
%numEncountersBetweenUpdates = [2 5 10 20 50]
numEncountersBetweenUpdates = [1]
modePar.bufferlength = 10 %length of genotype buffer for favscore algorithm
modePar.genoInit = 'mindiv'  %initial conditions of genotypes maxdiv (maximum diversity - random variant chosen) mindiv (minimum diversity - all nodes same)
%blacklist mode
modePar.blklist = 'no'; %yes or no
modePar.numToBlklist = 1
modePar.blklistrate = 0.004 %0.001
%update mode
modePar.update = 'yes' %stop updating mode
modePar.bufferlengthNei=10 %buffer for buffering local nei diversity,
modePar.stdNeiUpdateThreshold = 0.15
%recovery
recovery = 0; 0.0005 %0.001

%main par
numRuns = 30 %20 %100 %200
numNodes = 100;
numLoci=1; 
casetype = 'OR';
modePar.virusStartTime = 140;
numTimestepsVR = [350];
range = [1 3 5 7 10];
   VR=[1 2 3 4 5 6 8 10 12 14 16]; %314
modePar.statnodesrwp = 'stationary'; %'dynamic' or 'stationary' nodes, if dynamic box constraints applied at point of virus start
Imean = zeros(1,numTimestepsVR(1)); %init if runs all tests
%**********************
%1. rwp box constraints
modePar.numToConstrain = 0;%fix(numNodes*1/4); %constain a number of nodes to 1 variant
modePar.areaConstrainrwp='yes'
modePar.nodesln  = fix(numNodes/4);     % Number of limited nodes (absolute) - always first
%**********************
mode = 'Dynamic-favScore' %mode - can be 'static' or 'dynamic' 'Dynamic-random' 'Dynamic-favScore'
%monte carlo random genotype SIR model
%set up diversity mode parameters
modePar.modeStr = mode
modePar.stopUpdatingTime = 200000 %time at which to stop updating the genotypes
for j = 1:size(VR,2)%do for all VR
    variantRichness =  VR(j)
    numTimesteps = numTimestepsVR(1);
for z= 1:size(range,2)
    modePar.dinr = range(z);
    modePar.numEncountersBetweenUpdates = numEncountersBetweenUpdates(1);
 betan = 0.02;  plotGraphs = 0; gamma = recovery; numExploits=1; throwAway = 'nonEpidemics', SEpidemic=60; %cutoff;
 %agent
 [figMeanNei figNeiOneRun fighistSpRun fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetanew fighistGammanew figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            meanNeiIndexDynamic betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean gammameanmean betameanmean2 betameanmeanNew betameanmeanNew2 gammameanmeanNew Smean Imean Rmean Bmean NSmean Smedian Imedian Rmedian S I R NS SmeanN ImeanN1 RmeanN1 BmeanN1 Snewendmean Snewendstd Rnewendmean Rnewendstd Rnew_nonendmean Rnew_nonendstd propSimsNonEp propSimsNonEp_theoretical...
            numRunsMinor measuredProbMinor numRunsMajor measuredProbMajor meanmdist meannumInfectedVM...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,modePar, numLoci, casetype);
   close all; 
   
    %peak time
    FinalSize = max(Imean,[],2); %mean final size SI
    maska = Imean >= FinalSize-1; %when within 1 of the final value
    [C Index] = max(maska);
    MLa(j,z) = Index; %T1
    %resistance
    Resistance = (numNodes-FinalSize)./numNodes;
   
%data logging
 fprintf(data.logOutput(data.m), '%f,', Imean);     % infection
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', meanmdist); %distance spread
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Index);     %T1
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Resistance); %resistance
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', FinalSize); %final size
fprintf(data.logOutput(data.m), '\n'); %new line
end %range
end %VR

mode = 'Dynamic-random' %mode - can be 'static' or 'dynamic' 'Dynamic-random' 'Dynamic-favScore'
%monte carlo random genotype SIR model
%set up diversity mode parameters
modePar.modeStr = mode
modePar.stopUpdatingTime = modePar.virusStartTime %time at which to stop updating the genotypes
for j = 1:size(VR,2)%do for all VR
    variantRichness =  VR(j)
    numTimesteps = numTimestepsVR(1);
for z= 1:size(range,2)
    modePar.dinr = range(z);
    modePar.numEncountersBetweenUpdates = numEncountersBetweenUpdates(1);
 betan = 0.02;  plotGraphs = 0; gamma = recovery; numExploits=1;throwAway = 'nonEpidemics', SEpidemic=60; %cutoff;
 %agent
 [figMeanNei figNeiOneRun fighistSpRun fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetanew fighistGammanew figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            meanNeiIndexDynamic betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean gammameanmean betameanmean2 betameanmeanNew betameanmeanNew2 gammameanmeanNew Smean Imean Rmean Bmean NSmean Smedian Imedian Rmedian S I R NS SmeanN ImeanN1 RmeanN1 BmeanN1 Snewendmean Snewendstd Rnewendmean Rnewendstd Rnew_nonendmean Rnew_nonendstd propSimsNonEp propSimsNonEp_theoretical...
            numRunsMinor measuredProbMinor numRunsMajor measuredProbMajor meanmdist meannumInfectedVM...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,modePar, numLoci, casetype);
   close all; 
   
    %peak time
    FinalSize = max(Imean,[],2); %mean final size SI
    maska = Imean >= FinalSize-1; %when within 1 of the final value
    [C Index] = max(maska);
    MLa(j,z) = Index; %T1
    %resistance
    Resistance = (numNodes-FinalSize)./numNodes;
   
%data logging
 fprintf(data.logOutput(data.m), '%f,', Imean);     % infection
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', meanmdist); %distance spread
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Index);     %T1
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Resistance); %resistance
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', FinalSize); %final size
fprintf(data.logOutput(data.m), '\n'); %new line
end %range
end %VR

%**********************
%2. random constraints
modePar.numToConstrain = fix(numNodes*1/4); %constain a number of nodes to 1 variant
modePar.areaConstrainrwp='no'
modePar.nodesln  = 0;     % Number of limited nodes (absolute) - always first
%**********************
mode = 'Dynamic-favScore' %mode - can be 'static' or 'dynamic' 'Dynamic-random' 'Dynamic-favScore'
%monte carlo random genotype SIR model
%set up diversity mode parameters
modePar.modeStr = mode
modePar.stopUpdatingTime = 200000 %time at which to stop updating the genotypes
for j = 1:size(VR,2)%do for all VR
    variantRichness =  VR(j)
    numTimesteps = numTimestepsVR(1);
for z= 1:size(range,2)
    modePar.dinr = range(z);
    modePar.numEncountersBetweenUpdates = numEncountersBetweenUpdates(1);
 betan = 0.02;  plotGraphs = 0; gamma = recovery; numExploits=1; throwAway = 'nonEpidemics', SEpidemic=60; %cutoff;
 %agent
 [figMeanNei figNeiOneRun fighistSpRun fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetanew fighistGammanew figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            meanNeiIndexDynamic betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean gammameanmean betameanmean2 betameanmeanNew betameanmeanNew2 gammameanmeanNew Smean Imean Rmean Bmean NSmean Smedian Imedian Rmedian S I R NS SmeanN ImeanN1 RmeanN1 BmeanN1 Snewendmean Snewendstd Rnewendmean Rnewendstd Rnew_nonendmean Rnew_nonendstd propSimsNonEp propSimsNonEp_theoretical...
            numRunsMinor measuredProbMinor numRunsMajor measuredProbMajor meanmdist meannumInfectedVM...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,modePar, numLoci, casetype);
   close all; 
   
    %peak time
    FinalSize = max(Imean,[],2); %mean final size SI
    maska = Imean >= FinalSize-1; %when within 1 of the final value
    [C Index] = max(maska);
    MLa(j,z) = Index; %T1
    %resistance
    Resistance = (numNodes-FinalSize)./numNodes;
   
%data logging
 fprintf(data.logOutput(data.m), '%f,', Imean);     % infection
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', meanmdist); %distance spread
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Index);     %T1
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Resistance); %resistance
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', FinalSize); %final size
fprintf(data.logOutput(data.m), '\n'); %new line
end %range
end %VR

mode = 'Dynamic-random' %mode - can be 'static' or 'dynamic' 'Dynamic-random' 'Dynamic-favScore'
%monte carlo random genotype SIR model
%set up diversity mode parameters
modePar.modeStr = mode
modePar.stopUpdatingTime = modePar.virusStartTime %time at which to stop updating the genotypes
for j = 1:size(VR,2)%do for all VR
    variantRichness =  VR(j)
    numTimesteps = numTimestepsVR(1);
for z= 1:size(range,2)
    modePar.dinr = range(z);
    modePar.numEncountersBetweenUpdates = numEncountersBetweenUpdates(1);
 betan = 0.02;  plotGraphs = 0; gamma = recovery; numExploits=1; throwAway = 'nonEpidemics', SEpidemic=60; %cutoff;
 %agent
 [figMeanNei figNeiOneRun fighistSpRun fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetanew fighistGammanew figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            meanNeiIndexDynamic betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean gammameanmean betameanmean2 betameanmeanNew betameanmeanNew2 gammameanmeanNew Smean Imean Rmean Bmean NSmean Smedian Imedian Rmedian S I R NS SmeanN ImeanN1 RmeanN1 BmeanN1 Snewendmean Snewendstd Rnewendmean Rnewendstd Rnew_nonendmean Rnew_nonendstd propSimsNonEp propSimsNonEp_theoretical...
            numRunsMinor measuredProbMinor numRunsMajor measuredProbMajor meanmdist meannumInfectedVM...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,modePar, numLoci, casetype);
   close all; 
   
    %peak time
    FinalSize = max(Imean,[],2); %mean final size SI
    maska = Imean >= FinalSize-1; %when within 1 of the final value
    [C Index] = max(maska);
    MLa(j,z) = Index; %T1
    %resistance
    Resistance = (numNodes-FinalSize)./numNodes;
   
%data logging
 fprintf(data.logOutput(data.m), '%f,', Imean);     % infection
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', meanmdist); %distance spread
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Index);     %T1
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Resistance); %resistance
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', FinalSize); %final size
fprintf(data.logOutput(data.m), '\n'); %new line
end %range
end %VR

%**********************
%3. no constraints
modePar.numToConstrain = 0; %fix(numNodes*1/4); %constain a number of nodes to 1 variant
modePar.areaConstrainrwp='no'
modePar.nodesln  = 0;     % Number of limited nodes (absolute) - always first
%**********************
mode = 'Dynamic-favScore' %mode - can be 'static' or 'dynamic' 'Dynamic-random' 'Dynamic-favScore'
%monte carlo random genotype SIR model
%set up diversity mode parameters
modePar.modeStr = mode
modePar.stopUpdatingTime = 200000 %time at which to stop updating the genotypes
for j = 1:size(VR,2)%do for all VR
    variantRichness =  VR(j)
    numTimesteps = numTimestepsVR(1);
for z= 1:size(range,2)
    modePar.dinr = range(z);
    modePar.numEncountersBetweenUpdates = numEncountersBetweenUpdates(1);
 betan = 0.02;  plotGraphs = 0; gamma = recovery; numExploits=1; throwAway = 'nonEpidemics', SEpidemic=60; %cutoff;
 %agent
 [figMeanNei figNeiOneRun fighistSpRun fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetanew fighistGammanew figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            meanNeiIndexDynamic betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean gammameanmean betameanmean2 betameanmeanNew betameanmeanNew2 gammameanmeanNew Smean Imean Rmean Bmean NSmean Smedian Imedian Rmedian S I R NS SmeanN ImeanN1 RmeanN1 BmeanN1 Snewendmean Snewendstd Rnewendmean Rnewendstd Rnew_nonendmean Rnew_nonendstd propSimsNonEp propSimsNonEp_theoretical...
            numRunsMinor measuredProbMinor numRunsMajor measuredProbMajor meanmdist meannumInfectedVM...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,modePar, numLoci, casetype);
   close all; 
   
    %peak time
    FinalSize = max(Imean,[],2); %mean final size SI
    maska = Imean >= FinalSize-1; %when within 1 of the final value
    [C Index] = max(maska);
    MLa(j,z) = Index; %T1
    %resistance
    Resistance = (numNodes-FinalSize)./numNodes;
   
%data logging
 fprintf(data.logOutput(data.m), '%f,', Imean);     % infection
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', meanmdist); %distance spread
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Index);     %T1
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Resistance); %resistance
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', FinalSize); %final size
fprintf(data.logOutput(data.m), '\n'); %new line
end %range
end %VR

mode = 'Dynamic-random' %mode - can be 'static' or 'dynamic' 'Dynamic-random' 'Dynamic-favScore'
%monte carlo random genotype SIR model
%set up diversity mode parameters
modePar.modeStr = mode
modePar.stopUpdatingTime = modePar.virusStartTime %time at which to stop updating the genotypes
for j = 1:size(VR,2)%do for all VR
    variantRichness =  VR(j)
    numTimesteps = numTimestepsVR(1);
for z= 1:size(range,2)
    modePar.dinr = range(z);
    modePar.numEncountersBetweenUpdates = numEncountersBetweenUpdates(1);
 betan = 0.02;  plotGraphs = 0; gamma = recovery; numExploits=1; throwAway = 'nonEpidemics', SEpidemic=60; %cutoff;
 %agent
 [figMeanNei figNeiOneRun fighistSpRun fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetanew fighistGammanew figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            meanNeiIndexDynamic betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean gammameanmean betameanmean2 betameanmeanNew betameanmeanNew2 gammameanmeanNew Smean Imean Rmean Bmean NSmean Smedian Imedian Rmedian S I R NS SmeanN ImeanN1 RmeanN1 BmeanN1 Snewendmean Snewendstd Rnewendmean Rnewendstd Rnew_nonendmean Rnew_nonendstd propSimsNonEp propSimsNonEp_theoretical...
            numRunsMinor measuredProbMinor numRunsMajor measuredProbMajor meanmdist meannumInfectedVM...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,modePar, numLoci, casetype);
   close all; 
   
    %peak time
    FinalSize = max(Imean,[],2); %mean final size SI
    maska = Imean >= FinalSize-1; %when within 1 of the final value
    [C Index] = max(maska);
    MLa(j,z) = Index; %T1
    %resistance
    Resistance = (numNodes-FinalSize)./numNodes;
   
%data logging
 fprintf(data.logOutput(data.m), '%f,', Imean);     % infection
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', meanmdist); %distance spread
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Index);     %T1
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Resistance); %resistance
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', FinalSize); %final size
fprintf(data.logOutput(data.m), '\n'); %new line
end %range
end %VR

%close data logger
fclose(data.logOutput(data.m));

toc
end%314
%****************************
%stationary nodes -  4 loci OR
if doSIRNTests(313) == 1
data.m = 313 %data file number
%1 locus model
%outputDirectory = 'Graphs\'
outputDirectory = 'Graphs\'
tic
modePar.netStr = 'rwp'; %'rwp' random waypoint 're' random encounter
%rwp par
modePar.genotypeTimeOutPeriod = 10; %rwp
modePar.xminln   = 1;     % Minimum x-axis location limited nodes (du)
modePar.xmaxln   = 16;    % Maximum x-axis location limited nodes (du)
modePar.yminln   = 1;     % Minimum y-axis location limited nodes (du)
modePar.ymaxln   = 16;    % Maximum y-axis location limited nodes (du)
modePar.xmin     = 1;     % Minimum x-axis location (du)
modePar.xmax     = 31;    % Maximum x-axis location (du)
modePar.ymin     = 1;     % Minimum y-axis location (du)
modePar.ymax     = 31;    % Maximum y-axis location (du)
modePar.smin     = 0;  % Minimum speed (du / tu)
modePar.smax     = 0;  % Maximum speed (du / tu)
modePar.pmin     = 0;     % Minimum time for pause (integer tu)
modePar.pmax     = 0;     % Maximum time for pause (integer tu)
%modePar.dinr     = 3; %7    % Radial distance for in-range (du)
modePar.dexceed  = 1000;  % Value larger than any possible distance (du)
modePar.tinrg    = 1;     % Time in-range for genotype transfer (tu)
modePar.tinrv    = 1;     % Time in-range for virus transfer (tu)
% Mode
% rwph.mode = rwph.MODE_RIR;  1    % Select Random in range mode
% rwph.mode = rwph.MODE_NIR;  2    % Select Nearest in range mode
modePar.moderwp = 1; %MODE_AIR;  % 4    % Select (Random) Available in range mode
%rwp constrain
modePar.xboxc=0.5
modePar.yboxc=0.5
modePar.rwpcontrainplotGraph=0 %draw init constraints graph
% Log vector
modePar.logena = [1 2];
% Log directory for rwp
modePar.logDir = sprintf('%s\\log', pwd);
%data logging of output
data.logDir = sprintf('%s\\outputdata', pwd)
s = sprintf('%s\\data%i.txt', data.logDir,data.m)
data.logOutput(data.m) = fopen(s, 'wt');
%numEncountersBetweenUpdates = [2 5 10 20 50]
numEncountersBetweenUpdates = [1]
modePar.bufferlength = 10 %length of genotype buffer for favscore algorithm
modePar.genoInit = 'mindiv'  %initial conditions of genotypes maxdiv (maximum diversity - random variant chosen) mindiv (minimum diversity - all nodes same)
%blacklist mode
modePar.blklist = 'no'; %yes or no
modePar.numToBlklist = 1
modePar.blklistrate = 0.004 %0.001
%update mode
modePar.update = 'yes' %stop updating mode
modePar.bufferlengthNei=10 %buffer for buffering local nei diversity,
modePar.stdNeiUpdateThreshold = 0.15
%recovery
recovery = 0; 0.0005 %0.001

%main par
numRuns = 30 %20 %100 %200
numNodes = 100;
numLoci=4; 
casetype = 'OR';
modePar.virusStartTime = 140;
numTimestepsVR = [350];
range = [1 3 5 7 10];
VR = [1 2 4 6 8 10 12 14 16 18 20 25 30 35];
modePar.statnodesrwp = 'stationary'; %'dynamic' or 'stationary' nodes, if dynamic box constraints applied at point of virus start
Imean = zeros(1,numTimestepsVR(1)); %init if runs all tests
%**********************
%1. rwp box constraints
modePar.numToConstrain = 0;%fix(numNodes*1/4); %constain a number of nodes to 1 variant
modePar.areaConstrainrwp='yes'
modePar.nodesln  = fix(numNodes/4);     % Number of limited nodes (absolute) - always first
%**********************
mode = 'Dynamic-favScore' %mode - can be 'static' or 'dynamic' 'Dynamic-random' 'Dynamic-favScore'
%monte carlo random genotype SIR model
%set up diversity mode parameters
modePar.modeStr = mode
modePar.stopUpdatingTime = 200000 %time at which to stop updating the genotypes
for j = 1:size(VR,2)%do for all VR
    variantRichness =  VR(j)
    numTimesteps = numTimestepsVR(1);
for z= 1:size(range,2)
    modePar.dinr = range(z);
    modePar.numEncountersBetweenUpdates = numEncountersBetweenUpdates(1);
 betan = 0.02;  plotGraphs = 0; gamma = recovery; numExploits=1; throwAway = 'nonEpidemics', SEpidemic=60; %cutoff;
 %agent
 [figMeanNei figNeiOneRun fighistSpRun fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetanew fighistGammanew figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            meanNeiIndexDynamic betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean gammameanmean betameanmean2 betameanmeanNew betameanmeanNew2 gammameanmeanNew Smean Imean Rmean Bmean NSmean Smedian Imedian Rmedian S I R NS SmeanN ImeanN1 RmeanN1 BmeanN1 Snewendmean Snewendstd Rnewendmean Rnewendstd Rnew_nonendmean Rnew_nonendstd propSimsNonEp propSimsNonEp_theoretical...
            numRunsMinor measuredProbMinor numRunsMajor measuredProbMajor meanmdist meannumInfectedVM...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,modePar, numLoci, casetype);
   close all; 
   
    %peak time
    FinalSize = max(Imean,[],2); %mean final size SI
    maska = Imean >= FinalSize-1; %when within 1 of the final value
    [C Index] = max(maska);
    MLa(j,z) = Index; %T1
    %resistance
    Resistance = (numNodes-FinalSize)./numNodes;
   
%data logging
 fprintf(data.logOutput(data.m), '%f,', Imean);     % infection
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', meanmdist); %distance spread
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Index);     %T1
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Resistance); %resistance
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', FinalSize); %final size
fprintf(data.logOutput(data.m), '\n'); %new line
end %range
end %VR

mode = 'Dynamic-random' %mode - can be 'static' or 'dynamic' 'Dynamic-random' 'Dynamic-favScore'
%monte carlo random genotype SIR model
%set up diversity mode parameters
modePar.modeStr = mode
modePar.stopUpdatingTime = modePar.virusStartTime %time at which to stop updating the genotypes
for j = 1:size(VR,2)%do for all VR
    variantRichness =  VR(j)
    numTimesteps = numTimestepsVR(1);
for z= 1:size(range,2)
    modePar.dinr = range(z);
    modePar.numEncountersBetweenUpdates = numEncountersBetweenUpdates(1);
 betan = 0.02;  plotGraphs = 0; gamma = recovery; numExploits=1; throwAway = 'nonEpidemics', SEpidemic=60; %cutoff;
 %agent
 [figMeanNei figNeiOneRun fighistSpRun fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetanew fighistGammanew figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            meanNeiIndexDynamic betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean gammameanmean betameanmean2 betameanmeanNew betameanmeanNew2 gammameanmeanNew Smean Imean Rmean Bmean NSmean Smedian Imedian Rmedian S I R NS SmeanN ImeanN1 RmeanN1 BmeanN1 Snewendmean Snewendstd Rnewendmean Rnewendstd Rnew_nonendmean Rnew_nonendstd propSimsNonEp propSimsNonEp_theoretical...
            numRunsMinor measuredProbMinor numRunsMajor measuredProbMajor meanmdist meannumInfectedVM...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,modePar, numLoci, casetype);
   close all; 
   
    %peak time
    FinalSize = max(Imean,[],2); %mean final size SI
    maska = Imean >= FinalSize-1; %when within 1 of the final value
    [C Index] = max(maska);
    MLa(j,z) = Index; %T1
    %resistance
    Resistance = (numNodes-FinalSize)./numNodes;
   
%data logging
 fprintf(data.logOutput(data.m), '%f,', Imean);     % infection
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', meanmdist); %distance spread
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Index);     %T1
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Resistance); %resistance
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', FinalSize); %final size
fprintf(data.logOutput(data.m), '\n'); %new line
end %range
end %VR

%**********************
%2. random constraints
modePar.numToConstrain = fix(numNodes*1/4); %constain a number of nodes to 1 variant
modePar.areaConstrainrwp='no'
modePar.nodesln  = 0;     % Number of limited nodes (absolute) - always first
%**********************
mode = 'Dynamic-favScore' %mode - can be 'static' or 'dynamic' 'Dynamic-random' 'Dynamic-favScore'
%monte carlo random genotype SIR model
%set up diversity mode parameters
modePar.modeStr = mode
modePar.stopUpdatingTime = 200000 %time at which to stop updating the genotypes
for j = 1:size(VR,2)%do for all VR
    variantRichness =  VR(j)
    numTimesteps = numTimestepsVR(1);
for z= 1:size(range,2)
    modePar.dinr = range(z);
    modePar.numEncountersBetweenUpdates = numEncountersBetweenUpdates(1);
 betan = 0.02;  plotGraphs = 0; gamma = recovery; numExploits=1; throwAway = 'nonEpidemics', SEpidemic=60; %cutoff;
 %agent
 [figMeanNei figNeiOneRun fighistSpRun fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetanew fighistGammanew figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            meanNeiIndexDynamic betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean gammameanmean betameanmean2 betameanmeanNew betameanmeanNew2 gammameanmeanNew Smean Imean Rmean Bmean NSmean Smedian Imedian Rmedian S I R NS SmeanN ImeanN1 RmeanN1 BmeanN1 Snewendmean Snewendstd Rnewendmean Rnewendstd Rnew_nonendmean Rnew_nonendstd propSimsNonEp propSimsNonEp_theoretical...
            numRunsMinor measuredProbMinor numRunsMajor measuredProbMajor meanmdist meannumInfectedVM...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,modePar, numLoci, casetype);
   close all; 
   
    %peak time
    FinalSize = max(Imean,[],2); %mean final size SI
    maska = Imean >= FinalSize-1; %when within 1 of the final value
    [C Index] = max(maska);
    MLa(j,z) = Index; %T1
    %resistance
    Resistance = (numNodes-FinalSize)./numNodes;
   
%data logging
 fprintf(data.logOutput(data.m), '%f,', Imean);     % infection
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', meanmdist); %distance spread
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Index);     %T1
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Resistance); %resistance
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', FinalSize); %final size
fprintf(data.logOutput(data.m), '\n'); %new line
end %range
end %VR

mode = 'Dynamic-random' %mode - can be 'static' or 'dynamic' 'Dynamic-random' 'Dynamic-favScore'
%monte carlo random genotype SIR model
%set up diversity mode parameters
modePar.modeStr = mode
modePar.stopUpdatingTime = modePar.virusStartTime %time at which to stop updating the genotypes
for j = 1:size(VR,2)%do for all VR
    variantRichness =  VR(j)
    numTimesteps = numTimestepsVR(1);
for z= 1:size(range,2)
    modePar.dinr = range(z);
    modePar.numEncountersBetweenUpdates = numEncountersBetweenUpdates(1);
 betan = 0.02;  plotGraphs = 0; gamma = recovery; numExploits=1; throwAway = 'nonEpidemics', SEpidemic=60; %cutoff;
 %agent
 [figMeanNei figNeiOneRun fighistSpRun fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetanew fighistGammanew figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            meanNeiIndexDynamic betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean gammameanmean betameanmean2 betameanmeanNew betameanmeanNew2 gammameanmeanNew Smean Imean Rmean Bmean NSmean Smedian Imedian Rmedian S I R NS SmeanN ImeanN1 RmeanN1 BmeanN1 Snewendmean Snewendstd Rnewendmean Rnewendstd Rnew_nonendmean Rnew_nonendstd propSimsNonEp propSimsNonEp_theoretical...
            numRunsMinor measuredProbMinor numRunsMajor measuredProbMajor meanmdist meannumInfectedVM...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,modePar, numLoci, casetype);
   close all; 
   
    %peak time
    FinalSize = max(Imean,[],2); %mean final size SI
    maska = Imean >= FinalSize-1; %when within 1 of the final value
    [C Index] = max(maska);
    MLa(j,z) = Index; %T1
    %resistance
    Resistance = (numNodes-FinalSize)./numNodes;
   
%data logging
 fprintf(data.logOutput(data.m), '%f,', Imean);     % infection
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', meanmdist); %distance spread
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Index);     %T1
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Resistance); %resistance
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', FinalSize); %final size
fprintf(data.logOutput(data.m), '\n'); %new line
end %range
end %VR

%**********************
%3. no constraints
modePar.numToConstrain = 0; %fix(numNodes*1/4); %constain a number of nodes to 1 variant
modePar.areaConstrainrwp='no'
modePar.nodesln  = 0;     % Number of limited nodes (absolute) - always first
%**********************
mode = 'Dynamic-favScore' %mode - can be 'static' or 'dynamic' 'Dynamic-random' 'Dynamic-favScore'
%monte carlo random genotype SIR model
%set up diversity mode parameters
modePar.modeStr = mode
modePar.stopUpdatingTime = 200000 %time at which to stop updating the genotypes
for j = 1:size(VR,2)%do for all VR
    variantRichness =  VR(j)
    numTimesteps = numTimestepsVR(1);
for z= 1:size(range,2)
    modePar.dinr = range(z);
    modePar.numEncountersBetweenUpdates = numEncountersBetweenUpdates(1);
 betan = 0.02;  plotGraphs = 0; gamma = recovery; numExploits=1; throwAway = 'nonEpidemics', SEpidemic=60; %cutoff;
 %agent
 [figMeanNei figNeiOneRun fighistSpRun fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetanew fighistGammanew figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            meanNeiIndexDynamic betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean gammameanmean betameanmean2 betameanmeanNew betameanmeanNew2 gammameanmeanNew Smean Imean Rmean Bmean NSmean Smedian Imedian Rmedian S I R NS SmeanN ImeanN1 RmeanN1 BmeanN1 Snewendmean Snewendstd Rnewendmean Rnewendstd Rnew_nonendmean Rnew_nonendstd propSimsNonEp propSimsNonEp_theoretical...
            numRunsMinor measuredProbMinor numRunsMajor measuredProbMajor meanmdist meannumInfectedVM...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,modePar, numLoci, casetype);
   close all; 
   
    %peak time
    FinalSize = max(Imean,[],2); %mean final size SI
    maska = Imean >= FinalSize-1; %when within 1 of the final value
    [C Index] = max(maska);
    MLa(j,z) = Index; %T1
    %resistance
    Resistance = (numNodes-FinalSize)./numNodes;
   
%data logging
 fprintf(data.logOutput(data.m), '%f,', Imean);     % infection
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', meanmdist); %distance spread
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Index);     %T1
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Resistance); %resistance
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', FinalSize); %final size
fprintf(data.logOutput(data.m), '\n'); %new line
end %range
end %VR

mode = 'Dynamic-random' %mode - can be 'static' or 'dynamic' 'Dynamic-random' 'Dynamic-favScore'
%monte carlo random genotype SIR model
%set up diversity mode parameters
modePar.modeStr = mode
modePar.stopUpdatingTime = modePar.virusStartTime %time at which to stop updating the genotypes
for j = 1:size(VR,2)%do for all VR
    variantRichness =  VR(j)
    numTimesteps = numTimestepsVR(1);
for z= 1:size(range,2)
    modePar.dinr = range(z);
    modePar.numEncountersBetweenUpdates = numEncountersBetweenUpdates(1);
 betan = 0.02;  plotGraphs = 0; gamma = recovery; numExploits=1; throwAway = 'nonEpidemics', SEpidemic=60; %cutoff;
 %agent
 [figMeanNei figNeiOneRun fighistSpRun fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetanew fighistGammanew figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            meanNeiIndexDynamic betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean gammameanmean betameanmean2 betameanmeanNew betameanmeanNew2 gammameanmeanNew Smean Imean Rmean Bmean NSmean Smedian Imedian Rmedian S I R NS SmeanN ImeanN1 RmeanN1 BmeanN1 Snewendmean Snewendstd Rnewendmean Rnewendstd Rnew_nonendmean Rnew_nonendstd propSimsNonEp propSimsNonEp_theoretical...
            numRunsMinor measuredProbMinor numRunsMajor measuredProbMajor meanmdist meannumInfectedVM...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,modePar, numLoci, casetype);
   close all; 
   
    %peak time
    FinalSize = max(Imean,[],2); %mean final size SI
    maska = Imean >= FinalSize-1; %when within 1 of the final value
    [C Index] = max(maska);
    MLa(j,z) = Index; %T1
    %resistance
    Resistance = (numNodes-FinalSize)./numNodes;
   
%data logging
 fprintf(data.logOutput(data.m), '%f,', Imean);     % infection
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', meanmdist); %distance spread
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Index);     %T1
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Resistance); %resistance
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', FinalSize); %final size
fprintf(data.logOutput(data.m), '\n'); %new line
end %range
end %VR



%close data logger
fclose(data.logOutput(data.m));

toc
end%313


%stationary nodes -  4 loci AND
if doSIRNTests(312) == 1
data.m = 312 %data file number
%1 locus model
%outputDirectory = 'Graphs\'
outputDirectory = 'Graphs\'
tic
modePar.netStr = 'rwp'; %'rwp' random waypoint 're' random encounter
%rwp par
modePar.genotypeTimeOutPeriod = 10; %rwp
modePar.xminln   = 1;     % Minimum x-axis location limited nodes (du)
modePar.xmaxln   = 16;    % Maximum x-axis location limited nodes (du)
modePar.yminln   = 1;     % Minimum y-axis location limited nodes (du)
modePar.ymaxln   = 16;    % Maximum y-axis location limited nodes (du)
modePar.xmin     = 1;     % Minimum x-axis location (du)
modePar.xmax     = 31;    % Maximum x-axis location (du)
modePar.ymin     = 1;     % Minimum y-axis location (du)
modePar.ymax     = 31;    % Maximum y-axis location (du)
modePar.smin     = 0;  % Minimum speed (du / tu)
modePar.smax     = 0;  % Maximum speed (du / tu)
modePar.pmin     = 0;     % Minimum time for pause (integer tu)
modePar.pmax     = 0;     % Maximum time for pause (integer tu)
%modePar.dinr     = 3; %7    % Radial distance for in-range (du)
modePar.dexceed  = 1000;  % Value larger than any possible distance (du)
modePar.tinrg    = 1;     % Time in-range for genotype transfer (tu)
modePar.tinrv    = 1;     % Time in-range for virus transfer (tu)
% Mode
% rwph.mode = rwph.MODE_RIR;  1    % Select Random in range mode
% rwph.mode = rwph.MODE_NIR;  2    % Select Nearest in range mode
modePar.moderwp = 1; %MODE_AIR;  % 4    % Select (Random) Available in range mode
%rwp constrain
modePar.xboxc=0.5
modePar.yboxc=0.5
modePar.rwpcontrainplotGraph=0 %draw init constraints graph
% Log vector
modePar.logena = [1 2];
% Log directory for rwp
modePar.logDir = sprintf('%s\\log', pwd);
%data logging of output
data.logDir = sprintf('%s\\outputdata', pwd)
s = sprintf('%s\\data%i.txt', data.logDir,data.m)
data.logOutput(data.m) = fopen(s, 'wt');
%numEncountersBetweenUpdates = [2 5 10 20 50]
numEncountersBetweenUpdates = [1]
modePar.bufferlength = 10 %length of genotype buffer for favscore algorithm
modePar.genoInit = 'mindiv'  %initial conditions of genotypes maxdiv (maximum diversity - random variant chosen) mindiv (minimum diversity - all nodes same)
%blacklist mode
modePar.blklist = 'no'; %yes or no
modePar.numToBlklist = 1
modePar.blklistrate = 0.004 %0.001
%update mode
modePar.update = 'yes' %stop updating mode
modePar.bufferlengthNei=10 %buffer for buffering local nei diversity,
modePar.stdNeiUpdateThreshold = 0.15
%recovery
recovery = 0; 0.0005 %0.001

%main par
numRuns = 30 %20 %100 %200
numNodes = 100;
numLoci=4; 
casetype = 'AND';
modePar.virusStartTime = 140;
numTimestepsVR = [350];
range = [1 3 5 7 10];
%VR = [1 2 4 6 8 10];
VR = [1 2 3 4 5 6 7];
modePar.statnodesrwp = 'stationary'; %'dynamic' or 'stationary' nodes, if dynamic box constraints applied at point of virus start
Imean = zeros(1,numTimestepsVR(1)); %init if runs all tests
%**********************
%1. rwp box constraints
modePar.numToConstrain = 0;%fix(numNodes*1/4); %constain a number of nodes to 1 variant
modePar.areaConstrainrwp='yes'
modePar.nodesln  = fix(numNodes/4);     % Number of limited nodes (absolute) - always first
%**********************
mode = 'Dynamic-favScore' %mode - can be 'static' or 'dynamic' 'Dynamic-random' 'Dynamic-favScore'
%monte carlo random genotype SIR model
%set up diversity mode parameters
modePar.modeStr = mode
modePar.stopUpdatingTime = 200000 %time at which to stop updating the genotypes
for j = 1:size(VR,2)%do for all VR
    variantRichness =  VR(j)
    numTimesteps = numTimestepsVR(1);
for z= 1:size(range,2)
    modePar.dinr = range(z);
    modePar.numEncountersBetweenUpdates = numEncountersBetweenUpdates(1);
 betan = 0.02;  plotGraphs = 0; gamma = recovery; numExploits=1; throwAway = 'nonEpidemics', SEpidemic=60; %cutoff;
 %agent
 [figMeanNei figNeiOneRun fighistSpRun fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetanew fighistGammanew figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            meanNeiIndexDynamic betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean gammameanmean betameanmean2 betameanmeanNew betameanmeanNew2 gammameanmeanNew Smean Imean Rmean Bmean NSmean Smedian Imedian Rmedian S I R NS SmeanN ImeanN1 RmeanN1 BmeanN1 Snewendmean Snewendstd Rnewendmean Rnewendstd Rnew_nonendmean Rnew_nonendstd propSimsNonEp propSimsNonEp_theoretical...
            numRunsMinor measuredProbMinor numRunsMajor measuredProbMajor meanmdist meannumInfectedVM...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,modePar, numLoci, casetype);
   close all; 
   
    %peak time
    FinalSize = max(Imean,[],2); %mean final size SI
    maska = Imean >= FinalSize-1; %when within 1 of the final value
    [C Index] = max(maska);
    MLa(j,z) = Index; %T1
    %resistance
    Resistance = (numNodes-FinalSize)./numNodes;
   
%data logging
 fprintf(data.logOutput(data.m), '%f,', Imean);     % infection
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', meanmdist); %distance spread
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Index);     %T1
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Resistance); %resistance
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', FinalSize); %final size
fprintf(data.logOutput(data.m), '\n'); %new line
end %range
end %VR

mode = 'Dynamic-random' %mode - can be 'static' or 'dynamic' 'Dynamic-random' 'Dynamic-favScore'
%monte carlo random genotype SIR model
%set up diversity mode parameters
modePar.modeStr = mode
modePar.stopUpdatingTime = modePar.virusStartTime %time at which to stop updating the genotypes
for j = 1:size(VR,2)%do for all VR
    variantRichness =  VR(j)
    numTimesteps = numTimestepsVR(1);
for z= 1:size(range,2)
    modePar.dinr = range(z);
    modePar.numEncountersBetweenUpdates = numEncountersBetweenUpdates(1);
 betan = 0.02;  plotGraphs = 0; gamma = recovery; numExploits=1; throwAway = 'nonEpidemics', SEpidemic=60; %cutoff;
 %agent
 [figMeanNei figNeiOneRun fighistSpRun fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetanew fighistGammanew figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            meanNeiIndexDynamic betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean gammameanmean betameanmean2 betameanmeanNew betameanmeanNew2 gammameanmeanNew Smean Imean Rmean Bmean NSmean Smedian Imedian Rmedian S I R NS SmeanN ImeanN1 RmeanN1 BmeanN1 Snewendmean Snewendstd Rnewendmean Rnewendstd Rnew_nonendmean Rnew_nonendstd propSimsNonEp propSimsNonEp_theoretical...
            numRunsMinor measuredProbMinor numRunsMajor measuredProbMajor meanmdist meannumInfectedVM...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,modePar, numLoci, casetype);
   close all; 
   
    %peak time
    FinalSize = max(Imean,[],2); %mean final size SI
    maska = Imean >= FinalSize-1; %when within 1 of the final value
    [C Index] = max(maska);
    MLa(j,z) = Index; %T1
    %resistance
    Resistance = (numNodes-FinalSize)./numNodes;
   
%data logging
 fprintf(data.logOutput(data.m), '%f,', Imean);     % infection
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', meanmdist); %distance spread
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Index);     %T1
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Resistance); %resistance
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', FinalSize); %final size
fprintf(data.logOutput(data.m), '\n'); %new line
end %range
end %VR

%**********************
%2. random constraints
modePar.numToConstrain = fix(numNodes*1/4); %constain a number of nodes to 1 variant
modePar.areaConstrainrwp='no'
modePar.nodesln  = 0;     % Number of limited nodes (absolute) - always first
%**********************
mode = 'Dynamic-favScore' %mode - can be 'static' or 'dynamic' 'Dynamic-random' 'Dynamic-favScore'
%monte carlo random genotype SIR model
%set up diversity mode parameters
modePar.modeStr = mode
modePar.stopUpdatingTime = 200000 %time at which to stop updating the genotypes
for j = 1:size(VR,2)%do for all VR
    variantRichness =  VR(j)
    numTimesteps = numTimestepsVR(1);
for z= 1:size(range,2)
    modePar.dinr = range(z);
    modePar.numEncountersBetweenUpdates = numEncountersBetweenUpdates(1);
 betan = 0.02;  plotGraphs = 0; gamma = recovery; numExploits=1; throwAway = 'nonEpidemics', SEpidemic=60; %cutoff;
 %agent
 [figMeanNei figNeiOneRun fighistSpRun fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetanew fighistGammanew figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            meanNeiIndexDynamic betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean gammameanmean betameanmean2 betameanmeanNew betameanmeanNew2 gammameanmeanNew Smean Imean Rmean Bmean NSmean Smedian Imedian Rmedian S I R NS SmeanN ImeanN1 RmeanN1 BmeanN1 Snewendmean Snewendstd Rnewendmean Rnewendstd Rnew_nonendmean Rnew_nonendstd propSimsNonEp propSimsNonEp_theoretical...
            numRunsMinor measuredProbMinor numRunsMajor measuredProbMajor meanmdist meannumInfectedVM...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,modePar, numLoci, casetype);
   close all; 
   
    %peak time
    FinalSize = max(Imean,[],2); %mean final size SI
    maska = Imean >= FinalSize-1; %when within 1 of the final value
    [C Index] = max(maska);
    MLa(j,z) = Index; %T1
    %resistance
    Resistance = (numNodes-FinalSize)./numNodes;
   
%data logging
 fprintf(data.logOutput(data.m), '%f,', Imean);     % infection
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', meanmdist); %distance spread
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Index);     %T1
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Resistance); %resistance
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', FinalSize); %final size
fprintf(data.logOutput(data.m), '\n'); %new line
end %range
end %VR

mode = 'Dynamic-random' %mode - can be 'static' or 'dynamic' 'Dynamic-random' 'Dynamic-favScore'
%monte carlo random genotype SIR model
%set up diversity mode parameters
modePar.modeStr = mode
modePar.stopUpdatingTime = modePar.virusStartTime %time at which to stop updating the genotypes
for j = 1:size(VR,2)%do for all VR
    variantRichness =  VR(j)
    numTimesteps = numTimestepsVR(1);
for z= 1:size(range,2)
    modePar.dinr = range(z);
    modePar.numEncountersBetweenUpdates = numEncountersBetweenUpdates(1);
 betan = 0.02;  plotGraphs = 0; gamma = recovery; numExploits=1; throwAway = 'nonEpidemics', SEpidemic=60; %cutoff;
 %agent
 [figMeanNei figNeiOneRun fighistSpRun fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetanew fighistGammanew figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            meanNeiIndexDynamic betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean gammameanmean betameanmean2 betameanmeanNew betameanmeanNew2 gammameanmeanNew Smean Imean Rmean Bmean NSmean Smedian Imedian Rmedian S I R NS SmeanN ImeanN1 RmeanN1 BmeanN1 Snewendmean Snewendstd Rnewendmean Rnewendstd Rnew_nonendmean Rnew_nonendstd propSimsNonEp propSimsNonEp_theoretical...
            numRunsMinor measuredProbMinor numRunsMajor measuredProbMajor meanmdist meannumInfectedVM...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,modePar, numLoci, casetype);
   close all; 
   
    %peak time
    FinalSize = max(Imean,[],2); %mean final size SI
    maska = Imean >= FinalSize-1; %when within 1 of the final value
    [C Index] = max(maska);
    MLa(j,z) = Index; %T1
    %resistance
    Resistance = (numNodes-FinalSize)./numNodes;
   
%data logging
 fprintf(data.logOutput(data.m), '%f,', Imean);     % infection
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', meanmdist); %distance spread
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Index);     %T1
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Resistance); %resistance
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', FinalSize); %final size
fprintf(data.logOutput(data.m), '\n'); %new line
end %range
end %VR

%**********************
%3. no constraints
modePar.numToConstrain = 0; %fix(numNodes*1/4); %constain a number of nodes to 1 variant
modePar.areaConstrainrwp='no'
modePar.nodesln  = 0;     % Number of limited nodes (absolute) - always first
%**********************
mode = 'Dynamic-favScore' %mode - can be 'static' or 'dynamic' 'Dynamic-random' 'Dynamic-favScore'
%monte carlo random genotype SIR model
%set up diversity mode parameters
modePar.modeStr = mode
modePar.stopUpdatingTime = 200000 %time at which to stop updating the genotypes
for j = 1:size(VR,2)%do for all VR
    variantRichness =  VR(j)
    numTimesteps = numTimestepsVR(1);
for z= 1:size(range,2)
    modePar.dinr = range(z);
    modePar.numEncountersBetweenUpdates = numEncountersBetweenUpdates(1);
 betan = 0.02;  plotGraphs = 0; gamma = recovery; numExploits=1; throwAway = 'nonEpidemics', SEpidemic=60; %cutoff;
 %agent
 [figMeanNei figNeiOneRun fighistSpRun fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetanew fighistGammanew figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            meanNeiIndexDynamic betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean gammameanmean betameanmean2 betameanmeanNew betameanmeanNew2 gammameanmeanNew Smean Imean Rmean Bmean NSmean Smedian Imedian Rmedian S I R NS SmeanN ImeanN1 RmeanN1 BmeanN1 Snewendmean Snewendstd Rnewendmean Rnewendstd Rnew_nonendmean Rnew_nonendstd propSimsNonEp propSimsNonEp_theoretical...
            numRunsMinor measuredProbMinor numRunsMajor measuredProbMajor meanmdist meannumInfectedVM...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,modePar, numLoci, casetype);
   close all; 
   
    %peak time
    FinalSize = max(Imean,[],2); %mean final size SI
    maska = Imean >= FinalSize-1; %when within 1 of the final value
    [C Index] = max(maska);
    MLa(j,z) = Index; %T1
    %resistance
    Resistance = (numNodes-FinalSize)./numNodes;
   
%data logging
 fprintf(data.logOutput(data.m), '%f,', Imean);     % infection
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', meanmdist); %distance spread
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Index);     %T1
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Resistance); %resistance
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', FinalSize); %final size
fprintf(data.logOutput(data.m), '\n'); %new line
end %range
end %VR

mode = 'Dynamic-random' %mode - can be 'static' or 'dynamic' 'Dynamic-random' 'Dynamic-favScore'
%monte carlo random genotype SIR model
%set up diversity mode parameters
modePar.modeStr = mode
modePar.stopUpdatingTime = modePar.virusStartTime %time at which to stop updating the genotypes
for j = 1:size(VR,2)%do for all VR
    variantRichness =  VR(j)
    numTimesteps = numTimestepsVR(1);
for z= 1:size(range,2)
    modePar.dinr = range(z);
    modePar.numEncountersBetweenUpdates = numEncountersBetweenUpdates(1);
 betan = 0.02;  plotGraphs = 0; gamma = recovery; numExploits=1; throwAway = 'nonEpidemics', SEpidemic=60; %cutoff;
 %agent
 [figMeanNei figNeiOneRun fighistSpRun fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetanew fighistGammanew figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            meanNeiIndexDynamic betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean gammameanmean betameanmean2 betameanmeanNew betameanmeanNew2 gammameanmeanNew Smean Imean Rmean Bmean NSmean Smedian Imedian Rmedian S I R NS SmeanN ImeanN1 RmeanN1 BmeanN1 Snewendmean Snewendstd Rnewendmean Rnewendstd Rnew_nonendmean Rnew_nonendstd propSimsNonEp propSimsNonEp_theoretical...
            numRunsMinor measuredProbMinor numRunsMajor measuredProbMajor meanmdist meannumInfectedVM...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,modePar, numLoci, casetype);
   close all; 
   
    %peak time
    FinalSize = max(Imean,[],2); %mean final size SI
    maska = Imean >= FinalSize-1; %when within 1 of the final value
    [C Index] = max(maska);
    MLa(j,z) = Index; %T1
    %resistance
    Resistance = (numNodes-FinalSize)./numNodes;
   
%data logging
 fprintf(data.logOutput(data.m), '%f,', Imean);     % infection
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', meanmdist); %distance spread
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Index);     %T1
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Resistance); %resistance
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', FinalSize); %final size
fprintf(data.logOutput(data.m), '\n'); %new line
end %range
end %VR

%close data logger
fclose(data.logOutput(data.m));

toc
end%312


%stationary nodes -  1 locus
if doSIRNTests(311) == 1
data.m = 311 %data file number
%1 locus model
%outputDirectory = 'Graphs\'
outputDirectory = 'Graphs\'
tic
modePar.netStr = 'rwp'; %'rwp' random waypoint 're' random encounter
%rwp par
modePar.genotypeTimeOutPeriod = 10; %rwp
%modePar.nodesln  = fix(numNodes/4);     % Number of limited nodes (absolute) - always first
modePar.xminln   = 1;     % Minimum x-axis location limited nodes (du)
modePar.xmaxln   = 16;    % Maximum x-axis location limited nodes (du)
modePar.yminln   = 1;     % Minimum y-axis location limited nodes (du)
modePar.ymaxln   = 16;    % Maximum y-axis location limited nodes (du)
modePar.xmin     = 1;     % Minimum x-axis location (du)
modePar.xmax     = 31;    % Maximum x-axis location (du)
modePar.ymin     = 1;     % Minimum y-axis location (du)
modePar.ymax     = 31;    % Maximum y-axis location (du)
modePar.smin     = 0;  % Minimum speed (du / tu)
modePar.smax     = 0;  % Maximum speed (du / tu)
modePar.pmin     = 0;     % Minimum time for pause (integer tu)
modePar.pmax     = 0;     % Maximum time for pause (integer tu)
%modePar.dinr     = 3; %7    % Radial distance for in-range (du)
modePar.dexceed  = 1000;  % Value larger than any possible distance (du)
modePar.tinrg    = 1;     % Time in-range for genotype transfer (tu)
modePar.tinrv    = 1;     % Time in-range for virus transfer (tu)
% Mode
% rwph.mode = rwph.MODE_RIR;  1    % Select Random in range mode
% rwph.mode = rwph.MODE_NIR;  2    % Select Nearest in range mode
modePar.moderwp = 1; %MODE_AIR;  % 4    % Select (Random) Available in range mode
%rwp constrain
modePar.xboxc=0.5
modePar.yboxc=0.5
modePar.rwpcontrainplotGraph=0 %draw init constraints graph
% Log vector
modePar.logena = [1 2];
% Log directory for rwp
modePar.logDir = sprintf('%s\\log', pwd);
%data logging of output
data.logDir = sprintf('%s\\outputdata', pwd)
s = sprintf('%s\\data%i.txt', data.logDir,data.m)
data.logOutput(data.m) = fopen(s, 'wt');
%numEncountersBetweenUpdates = [2 5 10 20 50]
numEncountersBetweenUpdates = [1]
modePar.bufferlength = 10 %length of genotype buffer for favscore algorithm
modePar.genoInit = 'mindiv'  %initial conditions of genotypes maxdiv (maximum diversity - random variant chosen) mindiv (minimum diversity - all nodes same)
%blacklist mode
modePar.blklist = 'no'; %yes or no
modePar.numToBlklist = 1
modePar.blklistrate = 0.004 %0.001
%update mode
modePar.update = 'yes' %stop updating mode
modePar.bufferlengthNei=10 %buffer for buffering local nei diversity,
modePar.stdNeiUpdateThreshold = 0.15
%recovery
recovery = 0; 0.0005 %0.001

%main par
numRuns = 30 %20 %100 %200
numNodes = 100;
numLoci=1; 
casetype = 'OR';
modePar.virusStartTime = 140;
numTimestepsVR = [350];
range = [1 3 5 7 10];
VR = [1 2 3 4 5 6 8 10 12 14 16];

modePar.statnodesrwp = 'stationary'; %'dynamic' or 'stationary' nodes, if dynamic box constraints applied at point of virus start
Imean = zeros(1,numTimestepsVR(1)); %init if runs all tests
%**********************
%1. rwp box constraints
modePar.numToConstrain = 0;%fix(numNodes*1/4); %constain a number of nodes to 1 variant
modePar.areaConstrainrwp='yes'
modePar.nodesln  = fix(numNodes/4);     % Number of limited nodes (absolute) - always first
%**********************
mode = 'Dynamic-favScore' %mode - can be 'static' or 'dynamic' 'Dynamic-random' 'Dynamic-favScore'
%monte carlo random genotype SIR model
%set up diversity mode parameters
modePar.modeStr = mode
modePar.stopUpdatingTime = 200000 %time at which to stop updating the genotypes
for j = 1:size(VR,2)%do for all VR
    variantRichness =  VR(j)
    numTimesteps = numTimestepsVR(1);
for z= 1:size(range,2)
    modePar.dinr = range(z);
    modePar.numEncountersBetweenUpdates = numEncountersBetweenUpdates(1);
 betan = 0.02;  plotGraphs = 0; gamma = recovery; numExploits=1; throwAway = 'nonEpidemics', SEpidemic=60; %cutoff;
 %agent
 [figMeanNei figNeiOneRun fighistSpRun fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetanew fighistGammanew figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            meanNeiIndexDynamic betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean gammameanmean betameanmean2 betameanmeanNew betameanmeanNew2 gammameanmeanNew Smean Imean Rmean Bmean NSmean Smedian Imedian Rmedian S I R NS SmeanN ImeanN1 RmeanN1 BmeanN1 Snewendmean Snewendstd Rnewendmean Rnewendstd Rnew_nonendmean Rnew_nonendstd propSimsNonEp propSimsNonEp_theoretical...
            numRunsMinor measuredProbMinor numRunsMajor measuredProbMajor meanmdist meannumInfectedVM...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,modePar, numLoci, casetype);
   close all; 
   
    %peak time
    FinalSize = max(Imean,[],2); %mean final size SI
    maska = Imean >= FinalSize-1; %when within 1 of the final value
    [C Index] = max(maska);
    MLa(j,z) = Index; %T1
    %resistance
    Resistance = (numNodes-FinalSize)./numNodes;
   
%data logging
 fprintf(data.logOutput(data.m), '%f,', Imean);     % infection
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', meanmdist); %distance spread
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Index);     %T1
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Resistance); %resistance
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', FinalSize); %final size
fprintf(data.logOutput(data.m), '\n'); %new line
end %range
end %VR

mode = 'Dynamic-random' %mode - can be 'static' or 'dynamic' 'Dynamic-random' 'Dynamic-favScore'
%monte carlo random genotype SIR model
%set up diversity mode parameters
modePar.modeStr = mode
modePar.stopUpdatingTime = modePar.virusStartTime %time at which to stop updating the genotypes
for j = 1:size(VR,2)%do for all VR
    variantRichness =  VR(j)
    numTimesteps = numTimestepsVR(1);
for z= 1:size(range,2)
    modePar.dinr = range(z);
    modePar.numEncountersBetweenUpdates = numEncountersBetweenUpdates(1);
 betan = 0.02;  plotGraphs = 0; gamma = recovery; numExploits=1;throwAway = 'nonEpidemics', SEpidemic=60; %cutoff;
 %agent
 [figMeanNei figNeiOneRun fighistSpRun fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetanew fighistGammanew figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            meanNeiIndexDynamic betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean gammameanmean betameanmean2 betameanmeanNew betameanmeanNew2 gammameanmeanNew Smean Imean Rmean Bmean NSmean Smedian Imedian Rmedian S I R NS SmeanN ImeanN1 RmeanN1 BmeanN1 Snewendmean Snewendstd Rnewendmean Rnewendstd Rnew_nonendmean Rnew_nonendstd propSimsNonEp propSimsNonEp_theoretical...
            numRunsMinor measuredProbMinor numRunsMajor measuredProbMajor meanmdist meannumInfectedVM...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,modePar, numLoci, casetype);
   close all; 
   
    %peak time
    FinalSize = max(Imean,[],2); %mean final size SI
    maska = Imean >= FinalSize-1; %when within 1 of the final value
    [C Index] = max(maska);
    MLa(j,z) = Index; %T1
    %resistance
    Resistance = (numNodes-FinalSize)./numNodes;
   
%data logging
 fprintf(data.logOutput(data.m), '%f,', Imean);     % infection
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', meanmdist); %distance spread
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Index);     %T1
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Resistance); %resistance
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', FinalSize); %final size
fprintf(data.logOutput(data.m), '\n'); %new line
end %range
end %VR

%**********************
%2. random constraints
modePar.numToConstrain = fix(numNodes*1/4); %constain a number of nodes to 1 variant
modePar.areaConstrainrwp='no'
modePar.nodesln  = 0;     % Number of limited nodes (absolute) - always first
%**********************
mode = 'Dynamic-favScore' %mode - can be 'static' or 'dynamic' 'Dynamic-random' 'Dynamic-favScore'
%monte carlo random genotype SIR model
%set up diversity mode parameters
modePar.modeStr = mode
modePar.stopUpdatingTime = 200000 %time at which to stop updating the genotypes
for j = 1:size(VR,2)%do for all VR
    variantRichness =  VR(j)
    numTimesteps = numTimestepsVR(1);
for z= 1:size(range,2)
    modePar.dinr = range(z);
    modePar.numEncountersBetweenUpdates = numEncountersBetweenUpdates(1);
 betan = 0.02;  plotGraphs = 0; gamma = recovery; numExploits=1; throwAway = 'nonEpidemics', SEpidemic=60; %cutoff;
 %agent
 [figMeanNei figNeiOneRun fighistSpRun fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetanew fighistGammanew figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            meanNeiIndexDynamic betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean gammameanmean betameanmean2 betameanmeanNew betameanmeanNew2 gammameanmeanNew Smean Imean Rmean Bmean NSmean Smedian Imedian Rmedian S I R NS SmeanN ImeanN1 RmeanN1 BmeanN1 Snewendmean Snewendstd Rnewendmean Rnewendstd Rnew_nonendmean Rnew_nonendstd propSimsNonEp propSimsNonEp_theoretical...
            numRunsMinor measuredProbMinor numRunsMajor measuredProbMajor meanmdist meannumInfectedVM...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,modePar, numLoci, casetype);
   close all; 
   
    %peak time
    FinalSize = max(Imean,[],2); %mean final size SI
    maska = Imean >= FinalSize-1; %when within 1 of the final value
    [C Index] = max(maska);
    MLa(j,z) = Index; %T1
    %resistance
    Resistance = (numNodes-FinalSize)./numNodes;
   
%data logging
 fprintf(data.logOutput(data.m), '%f,', Imean);     % infection
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', meanmdist); %distance spread
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Index);     %T1
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Resistance); %resistance
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', FinalSize); %final size
fprintf(data.logOutput(data.m), '\n'); %new line
end %range
end %VR

mode = 'Dynamic-random' %mode - can be 'static' or 'dynamic' 'Dynamic-random' 'Dynamic-favScore'
%monte carlo random genotype SIR model
%set up diversity mode parameters
modePar.modeStr = mode
modePar.stopUpdatingTime = modePar.virusStartTime %time at which to stop updating the genotypes
for j = 1:size(VR,2)%do for all VR
    variantRichness =  VR(j)
    numTimesteps = numTimestepsVR(1);
for z= 1:size(range,2)
    modePar.dinr = range(z);
    modePar.numEncountersBetweenUpdates = numEncountersBetweenUpdates(1);
 betan = 0.02;  plotGraphs = 0; gamma = recovery; numExploits=1; throwAway = 'nonEpidemics', SEpidemic=60; %cutoff;
 %agent
 [figMeanNei figNeiOneRun fighistSpRun fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetanew fighistGammanew figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            meanNeiIndexDynamic betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean gammameanmean betameanmean2 betameanmeanNew betameanmeanNew2 gammameanmeanNew Smean Imean Rmean Bmean NSmean Smedian Imedian Rmedian S I R NS SmeanN ImeanN1 RmeanN1 BmeanN1 Snewendmean Snewendstd Rnewendmean Rnewendstd Rnew_nonendmean Rnew_nonendstd propSimsNonEp propSimsNonEp_theoretical...
            numRunsMinor measuredProbMinor numRunsMajor measuredProbMajor meanmdist meannumInfectedVM...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,modePar, numLoci, casetype);
   close all; 
   
    %peak time
    FinalSize = max(Imean,[],2); %mean final size SI
    maska = Imean >= FinalSize-1; %when within 1 of the final value
    [C Index] = max(maska);
    MLa(j,z) = Index; %T1
    %resistance
    Resistance = (numNodes-FinalSize)./numNodes;
   
%data logging
 fprintf(data.logOutput(data.m), '%f,', Imean);     % infection
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', meanmdist); %distance spread
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Index);     %T1
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Resistance); %resistance
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', FinalSize); %final size
fprintf(data.logOutput(data.m), '\n'); %new line
end %range
end %VR

%**********************
%3. no constraints
modePar.numToConstrain = 0; %fix(numNodes*1/4); %constain a number of nodes to 1 variant
modePar.areaConstrainrwp='no'
modePar.nodesln  = 0;     % Number of limited nodes (absolute) - always first
%**********************
mode = 'Dynamic-favScore' %mode - can be 'static' or 'dynamic' 'Dynamic-random' 'Dynamic-favScore'
%monte carlo random genotype SIR model
%set up diversity mode parameters
modePar.modeStr = mode
modePar.stopUpdatingTime = 200000 %time at which to stop updating the genotypes
for j = 1:size(VR,2)%do for all VR
    variantRichness =  VR(j)
    numTimesteps = numTimestepsVR(1);
for z= 1:size(range,2)
    modePar.dinr = range(z);
    modePar.numEncountersBetweenUpdates = numEncountersBetweenUpdates(1);
 betan = 0.02;  plotGraphs = 0; gamma = recovery; numExploits=1; throwAway = 'nonEpidemics', SEpidemic=60; %cutoff;
 %agent
 [figMeanNei figNeiOneRun fighistSpRun fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetanew fighistGammanew figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            meanNeiIndexDynamic betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean gammameanmean betameanmean2 betameanmeanNew betameanmeanNew2 gammameanmeanNew Smean Imean Rmean Bmean NSmean Smedian Imedian Rmedian S I R NS SmeanN ImeanN1 RmeanN1 BmeanN1 Snewendmean Snewendstd Rnewendmean Rnewendstd Rnew_nonendmean Rnew_nonendstd propSimsNonEp propSimsNonEp_theoretical...
            numRunsMinor measuredProbMinor numRunsMajor measuredProbMajor meanmdist meannumInfectedVM...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,modePar, numLoci, casetype);
   close all; 
   
    %peak time
    FinalSize = max(Imean,[],2); %mean final size SI
    maska = Imean >= FinalSize-1; %when within 1 of the final value
    [C Index] = max(maska);
    MLa(j,z) = Index; %T1
    %resistance
    Resistance = (numNodes-FinalSize)./numNodes;
   
%data logging
 fprintf(data.logOutput(data.m), '%f,', Imean);     % infection
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', meanmdist); %distance spread
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Index);     %T1
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Resistance); %resistance
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', FinalSize); %final size
fprintf(data.logOutput(data.m), '\n'); %new line
end %range
end %VR

mode = 'Dynamic-random' %mode - can be 'static' or 'dynamic' 'Dynamic-random' 'Dynamic-favScore'
%monte carlo random genotype SIR model
%set up diversity mode parameters
modePar.modeStr = mode
modePar.stopUpdatingTime = modePar.virusStartTime %time at which to stop updating the genotypes
for j = 1:size(VR,2)%do for all VR
    variantRichness =  VR(j)
    numTimesteps = numTimestepsVR(1);
for z= 1:size(range,2)
    modePar.dinr = range(z);
    modePar.numEncountersBetweenUpdates = numEncountersBetweenUpdates(1);
 betan = 0.02;  plotGraphs = 0; gamma = recovery; numExploits=1; throwAway = 'nonEpidemics', SEpidemic=60; %cutoff;
 %agent
 [figMeanNei figNeiOneRun fighistSpRun fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetanew fighistGammanew figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            meanNeiIndexDynamic betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean gammameanmean betameanmean2 betameanmeanNew betameanmeanNew2 gammameanmeanNew Smean Imean Rmean Bmean NSmean Smedian Imedian Rmedian S I R NS SmeanN ImeanN1 RmeanN1 BmeanN1 Snewendmean Snewendstd Rnewendmean Rnewendstd Rnew_nonendmean Rnew_nonendstd propSimsNonEp propSimsNonEp_theoretical...
            numRunsMinor measuredProbMinor numRunsMajor measuredProbMajor meanmdist meannumInfectedVM...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,modePar, numLoci, casetype);
   close all; 
   
    %peak time
    FinalSize = max(Imean,[],2); %mean final size SI
    maska = Imean >= FinalSize-1; %when within 1 of the final value
    [C Index] = max(maska);
    MLa(j,z) = Index; %T1
    %resistance
    Resistance = (numNodes-FinalSize)./numNodes;
   
%data logging
 fprintf(data.logOutput(data.m), '%f,', Imean);     % infection
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', meanmdist); %distance spread
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Index);     %T1
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', Resistance); %resistance
fprintf(data.logOutput(data.m), '\n'); %new line
 fprintf(data.logOutput(data.m), '%f,', FinalSize); %final size
fprintf(data.logOutput(data.m), '\n'); %new line
end %range
end %VR

%close data logger
fclose(data.logOutput(data.m));

toc
end%311
%**************************************************************
%End RWP comparative box constraints tests
%*************************************************************


%*********************************
%Dynamic Diversity reaching highest diversity level-
%diversity under geographical constraints, random in range
%Random  in range
%***********************************
if doSIRNTests(310) == 1
%1 locus model
%outputDirectory = 'Graphs\'
outputDirectory = 'Graphs\'
tic
modePar.netStr = 'rwp'; %'rwp' random waypoint 're' random encounter
%rwp par
modePar.genotypeTimeOutPeriod = 10; %rwp
modePar.xmin     = 1;     % Minimum x-axis location (du)
modePar.xmax     = 31;    % Maximum x-axis location (du)
modePar.ymin     = 1;     % Minimum y-axis location (du)
modePar.ymax     = 31;    % Maximum y-axis location (du)
modePar.smin     = 1.4;  % Minimum speed (du / tu)
modePar.smax     = 1.4;  % Maximum speed (du / tu)
modePar.pmin     = 0;     % Minimum time for pause (integer tu)
modePar.pmax     = 0;     % Maximum time for pause (integer tu)
modePar.dinr     = 1; %7    % Radial distance for in-range (du)
modePar.dexceed  = 1000;  % Value larger than any possible distance (du)
modePar.tinrg    = 1;     % Time in-range for genotype transfer (tu)
modePar.tinrv    = 1;     % Time in-range for virus transfer (tu)
% Mode
% rwph.mode = rwph.MODE_RIR;  1    % Select Random in range mode
% rwph.mode = rwph.MODE_NIR;  2    % Select Nearest in range mode
modePar.moderwp = 1; %MODE_AIR;  % 4    % Select (Random) Available in range mode

numRuns = 1 %20 %100 %200
numNodes = 100;
 casetype = 'OR';
modePar.virusStartTime = 100;
%rwp constrain
numTimestepsVR = [200];
modePar.areaConstrainrwp='yes';
modePar.statnodesrwp = 'stationary'; %'dynamic' or 'stationary' nodes, if dynamic box constraints applied at point of virus start
modePar.numToConstrain = 0;fix(numNodes*1/4); %constain a number of nodes to 1 variant
modePar.xboxc=0.5
modePar.yboxc=0.5
modePar.rwpcontrainplotGraph=1 %draw init constraints graph
% Log vector
modePar.logena = [1 2 3 4];
% Log directory for rwp
modePar.logDir = sprintf('%s\\log', pwd);
%data logging of output
data.logDir = sprintf('%s\\outputdata', pwd)
data.m = 304 %data file number
s = sprintf('%s\\data%i.txt', data.logDir,data.m)
data.logOutput(data.m) = fopen(s, 'wt');



%VRmin = 5 %1
%VRmax = 5 %10
%VR = [2 5 10 20 50]
VR = [10]

meanTimeStart = 100;
%numEncountersBetweenUpdates = [2 5 10 20 50]
numEncountersBetweenUpdates = [1]

%blacklist mode
modePar.blklist = 'no'; %yes or no
modePar.numToBlklist = 1
modePar.blklistrate = 0.004 %0.001
%update mode
modePar.update = 'yes' %stop updating mode
modePar.bufferlengthNei=10 %buffer for buffering local nei diversity,
modePar.stdNeiUpdateThreshold = 0.15
%recovery
recovery = 0; 0.0005 %0.001
dec = 10 %decimation of output data
%constraints
modePar.stopUpdatingTime = modePar.virusStartTime %time at which to stop updating the genotypes
%modePar.numEncountersBetweenUpdates = 1 % number of encounters between updates for random and favscore algorithms
modePar.bufferlength = 10 %length of genotype buffer for favscore algorithm
modePar.genoInit = 'mindiv'  %initial conditions of genotypes maxdiv (maximum diversity - random variant chosen) mindiv (minimum diversity - all nodes same)

%initialise output arrays
meanNeiIndexDynamic = zeros(1,numTimestepsVR(1));
meanNeiIndexDynamicF = zeros(1,numTimestepsVR(1));



mode = 'Dynamic-favScore' %mode - can be 'static' or 'dynamic' 'Dynamic-random' 'Dynamic-favScore'
%monte carlo random genotype SIR model
%set up diversity mode parameters
modePar.modeStr = mode

for j = 1:size(VR,2)%do for all VR
    variantRichness =  VR(j)
    numTimesteps = numTimestepsVR(1)
for z= 1:size(numEncountersBetweenUpdates,2)
    numUpdates = numEncountersBetweenUpdates(z)
    modePar.numEncountersBetweenUpdates = numEncountersBetweenUpdates(z);
 betan = 0.02;  plotGraphs = 0; gamma = recovery; numExploits=1; numLoci=1; throwAway = 'nonEpidemics', SEpidemic=60; %cutoff;

 %agent
 [figMeanNei figNeiOneRun fighistSpRun fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetanew fighistGammanew figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            meanNeiIndexDynamic betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean gammameanmean betameanmean2 betameanmeanNew betameanmeanNew2 gammameanmeanNew Smean Imean Rmean Bmean NSmean Smedian Imedian Rmedian S I R NS SmeanN ImeanN1 RmeanN1 BmeanN1 Snewendmean Snewendstd Rnewendmean Rnewendstd Rnew_nonendmean Rnew_nonendstd propSimsNonEp propSimsNonEp_theoretical...
            numRunsMinor measuredProbMinor numRunsMajor measuredProbMajor meanmdist meannumInfectedVM...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,modePar, numLoci, casetype);
meanDiv(j) = mean(meanNeiIndexDynamic(meanTimeStart:end));
   %close all; 
%data logging
 fprintf(data.logOutput(data.m), '%f,', meanNeiIndexDynamic);
fprintf(data.logOutput(data.m), '\n'); %new line
return
   
end

end

mode = 'Dynamic-favScore' %mode - can be 'static' or 'dynamic' 'Dynamic-random' 'Dynamic-favScore'
modePar.modeStr = mode

for j = 1:size(VR,2)%do for all tx times
    variantRichness =  VR(j)
    numTimesteps = numTimestepsVR(1)
for z= 1:size(numEncountersBetweenUpdates,2) %do for all update rates
    numUpdates=numEncountersBetweenUpdates(z)
    modePar.numEncountersBetweenUpdates = numEncountersBetweenUpdates(z);
 betan = 0.02;  plotGraphs = 0; gamma = recovery; numExploits=1; numLoci=1; throwAway = 'nonEpidemics', SEpidemic=60; %cutoff;

 %agent
 [figMeanNei figNeiOneRun fighistSpRun fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetanew fighistGammanew figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            meanNeiIndexDynamicF betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean gammameanmean betameanmean2 betameanmeanNew betameanmeanNew2 gammameanmeanNew Smean Imean Rmean NSmean Bmean Smedian Imedian Rmedian S I R NS SmeanN ImeanN1 RmeanN1 BmeanN1 Snewendmean Snewendstd Rnewendmean Rnewendstd Rnew_nonendmean Rnew_nonendstd propSimsNonEp propSimsNonEp_theoretical...
            numRunsMinor measuredProbMinor numRunsMajor measuredProbMajor meanmdist meannumInfectedVM...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,modePar, numLoci, casetype);
meanDivF(j) = mean(meanNeiIndexDynamicF(meanTimeStart:end));
numAlleles = repmat(variantRichness,[1,numLoci]);
            [ Dmax(j) DmaxLocus ] = getMaxNeiCalc( numNodes, numAlleles );
            %close all;
%data logging
 fprintf(data.logOutput(data.m), '%f,', meanNeiIndexDynamicF);
fprintf(data.logOutput(data.m), '\n'); %new line
end

end

            %outputs
            meanDiv
            meanDivF
            Dmax

%close data logger
fclose(data.logOutput(data.m));

%plot data
data.numRows= (j * z)*2;
data.numTimesteps=numTimesteps;
data.numNodes=numNodes;
data.numLoci=numLoci;
data.variantRichness=variantRichness;
dataplot(data,0,0);

%output data

             
            figMeanNei=figure
            plot(VR,meanDiv,'k','Linewidth',2)
            hold on
            plot(VR,meanDivF,'k--','Linewidth',2)             
            %plot(meanDiv_1000devices_ThreeQuarters,'k','Linewidth',2)
            %hold on
            %plot(meanDivF_1000devices_ThreeQuarters,'k--','Linewidth',2)
            plot(Dmax,'k.-','Linewidth',2)
            %plot(meanDiv_1000devices_quarter,'k','Linewidth',2)
            %plot(meanDivF_1000devices_quarter,'k--','Linewidth',2)
            %plot(meanDiv_1000devices_half,'k','Linewidth',2)
            %plot(meanDivF_1000devices_half,'k--','Linewidth',2)
           % plot(Dmax,'k.-','Linewidth',2)
            xlabel('Variant Richness')
            ylabel('Measured Variant Diversity (mean across time after maximum reached)')
            %legend('update: every 1 encounter','update: every 10 encounters','update: every 20 encounters','update: every 50 encounters')
            legend('Diversity: random','Diversity: favscore', 'Calculated maximum')
            title('constrained Dynamic')

 
toc
end%310



%*********************************
%%peak infection and time to transmit the virus
%Random  in range
%***********************************
%*********************************
%Dynamic Diversity reaching highest diversity level-
%max div vs time to tx genotype
%Random  in range
%***********************************
if doSIRNTests(309) == 1
%1 locus model
%outputDirectory = 'Graphs\'
outputDirectory = 'Graphs\'
tic
modePar.netStr = 'rwp'; %'rwp' random waypoint 're' random encounter
%rwp par
modePar.genotypeTimeOutPeriod = 10; %rwp
modePar.xmin     = 1;     % Minimum x-axis location (du)
modePar.xmax     = 600;    % Maximum x-axis location (du)
modePar.ymin     = 1;     % Minimum y-axis location (du)
modePar.ymax     = 600;    % Maximum y-axis location (du)
modePar.smin     = 1.4;  % Minimum speed (du / tu)
modePar.smax     = 1.4;  % Maximum speed (du / tu)
modePar.pmin     = 0;     % Minimum time for pause (integer tu)
modePar.pmax     = 0;     % Maximum time for pause (integer tu)
modePar.dinr     = 50;     % Radial distance for in-range (du)
modePar.dexceed  = 1000;  % Value larger than any possible distance (du)
%modePar.tinrg    = 2;     % Time in-range for genotype transfer (tu)
%modePar.tinrv    = 2;     % Time in-range for virus transfer (tu)
% Mode
% rwph.mode = rwph.MODE_RIR;  1    % Select Random in range mode
% rwph.mode = rwph.MODE_NIR;  2    % Select Nearest in range mode
modePar.moderwp = 1; %MODE_AIR;  % 4    % Select (Random) Available in range mode
% Log vector
modePar.logena = [1 2 3 4];
% Log directory for rwp
modePar.logDir = sprintf('%s\\log', pwd);
%data logging of output
data.logDir = sprintf('%s\\outputdata', pwd)
data.m = 309 %data file number
s = sprintf('%s\\data%i.txt', data.logDir,data.m)
data.logOutput(data.m) = fopen(s, 'wt');

numRuns = 5 %20 %100 %200
numNodes = 1000;

%tinrg    = [1 1 1 1 1 1 1 1 1 1 1];     % Time in-range for genotype transfer (tu)
%tinrv    = [1 3 5 7 9 11 13 15 17 19 20];     % Time in-range for virus transfer (tu)

tinrg    = [1 1 1 1 1 1];     % Time in-range for genotype transfer (tu)
tinrv    = [1 3 5 7 9 10];     % Time in-range for virus transfer (tu)

%VRmin = 5 %1
%VRmax = 5 %10
%VR = [2 5 10 20 50]
VR = [5]
numTimestepsVR = [1200];
%numEncountersBetweenUpdates = [2 5 10 20 50]
numEncountersBetweenUpdates = [1]

%blacklist mode
modePar.blklist = 'no'; %yes or no
modePar.numToBlklist = 1
modePar.blklistrate = 0.004 %0.001
%update mode
modePar.update = 'no' %stop updating mode
modePar.bufferlengthNei=10 %buffer for buffering local nei diversity,
modePar.stdNeiUpdateThreshold = 0.05
%recovery
recovery = 0; 0.0005 %0.001
dec = 10 %decimation of output data
%constraints
modePar.numToConstrain = 0;%fix(numNodes/2) %constain a number of nodes to 1 variant
modePar.stopUpdatingTime = 200000 %time at which to stop updating the genotypes
%modePar.numEncountersBetweenUpdates = 1 % number of encounters between updates for random and favscore algorithms
modePar.bufferlength = 10 %length of genotype buffer for favscore algorithm
modePar.genoInit = 'maxdiv'  %initial conditions of genotypes maxdiv (maximum diversity - random variant chosen) mindiv (minimum diversity - all nodes same)

%initialise output arrays
%meanNeiIndexDynamic = zeros(1,numTimestepsVR(1));
%meanNeiIndexDynamicF = zeros(1,numTimestepsVR(1));



mode = 'Dynamic-random' %mode - can be 'static' or 'dynamic' 'Dynamic-random' 'Dynamic-favScore'
%monte carlo random genotype SIR model
%set up diversity mode parameters
modePar.modeStr = mode

for j = 1:size(tinrg,2)%do for all tx times
    variantRichness =  VR(1)
    numTimesteps = numTimestepsVR(1)
    modePar.tinrg    = tinrg(j);  
    modePar.tinrv    = tinrv(j);
for z= 1:size(numEncountersBetweenUpdates,2)
    numUpdates = numEncountersBetweenUpdates(z)
    modePar.numEncountersBetweenUpdates = numEncountersBetweenUpdates(z);
 betan = 0.02;  plotGraphs = 0; gamma = recovery; numExploits=1; numLoci=1; casetype = 'OR'; throwAway = 'nonEpidemics', SEpidemic=60; %cutoff;

 %agent
 [figMeanNei figNeiOneRun fighistSpRun fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetanew fighistGammanew figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            meanNeiIndexDynamic betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean gammameanmean betameanmean2 betameanmeanNew betameanmeanNew2 gammameanmeanNew Smean Imean Rmean Bmean NSmean Smedian Imedian Rmedian S I R NS SmeanN ImeanN1 RmeanN1 BmeanN1 Snewendmean Snewendstd Rnewendmean Rnewendstd Rnew_nonendmean Rnew_nonendstd propSimsNonEp propSimsNonEp_theoretical...
            numRunsMinor measuredProbMinor numRunsMajor measuredProbMajor meanmdist meannumInfectedVM...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,modePar, numLoci, casetype);
maxI = max(Imean,[],2); %mean final size SI
    %time
    size(maxI)
     maska = Imean >= maxI-1; %when within 1 of the final value
    [C Index] = max(maska);
    MLaRE(j,z) = Index; %T1
   close all; 
   

   
%data logging
 fprintf(data.logOutput(data.m), '%f,', Imean);
fprintf(data.logOutput(data.m), '\n'); %new line
   
end

end


mode = 'Dynamic-favScore' %mode - can be 'static' or 'dynamic' 'Dynamic-random' 'Dynamic-favScore'
modePar.modeStr = mode

for j = 1:size(tinrg,2)%do for all tx times
    variantRichness =  VR(1)
    numTimesteps = numTimestepsVR(1)
    modePar.tinrg    = tinrg(j);  
    modePar.tinrv    = tinrv(j);
for z= 1:size(numEncountersBetweenUpdates,2) %do for all update rates
    numUpdates=numEncountersBetweenUpdates(z)
    modePar.numEncountersBetweenUpdates = numEncountersBetweenUpdates(z);
 betan = 0.02;  plotGraphs = 0; gamma = recovery; numExploits=1; numLoci=1; casetype = 'OR'; throwAway = 'nonEpidemics', SEpidemic=60; %cutoff;

 %agent
 [figMeanNei figNeiOneRun fighistSpRun fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetanew fighistGammanew figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            meanNeiIndexDynamicF betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean gammameanmean betameanmean2 betameanmeanNew betameanmeanNew2 gammameanmeanNew Smean ImeanF Rmean NSmean Bmean Smedian Imedian Rmedian S I R NS SmeanN ImeanN1 RmeanN1 BmeanN1 Snewendmean Snewendstd Rnewendmean Rnewendstd Rnew_nonendmean Rnew_nonendstd propSimsNonEp propSimsNonEp_theoretical...
            numRunsMinor measuredProbMinor numRunsMajor measuredProbMajor meanmdist meannumInfectedVM...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,modePar, numLoci, casetype);
maxI = max(ImeanF,[],2); %mean final size SI
    %timemeanmdist
     maska = ImeanF >= maxI-1; %when first reaches max with small tolerance
    [C Index] = max(maska);
    MLaFS(j,z) = Index; %T1
close all;

%data logging
 fprintf(data.logOutput(data.m), '%f,', ImeanF);
fprintf(data.logOutput(data.m), '\n'); %new line
end
end

mode = 'static' %mode - can be 'static' or 'dynamic' 'Dynamic-random' 'Dynamic-favScore'
modePar.modeStr = mode

for j = 1:size(tinrg,2)%do for all tx times
    variantRichness =  VR(1)
    numTimesteps = numTimestepsVR(1)
    modePar.tinrg    = tinrg(j);  
    modePar.tinrv    = tinrv(j);
for z= 1:size(numEncountersBetweenUpdates,2) %do for all update rates
    numUpdates=numEncountersBetweenUpdates(z)
    modePar.numEncountersBetweenUpdates = numEncountersBetweenUpdates(z);
 betan = 0.02;  plotGraphs = 0; gamma = recovery; numExploits=1; numLoci=1; casetype = 'OR'; throwAway = 'nonEpidemics', SEpidemic=60; %cutoff;

 %agent
 [figMeanNei figNeiOneRun fighistSpRun fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetanew fighistGammanew figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            meanNeiIndexDynamicF betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean gammameanmean betameanmean2 betameanmeanNew betameanmeanNew2 gammameanmeanNew Smean ImeanF Rmean NSmean Bmean Smedian Imedian Rmedian S I R NS SmeanN ImeanN1 RmeanN1 BmeanN1 Snewendmean Snewendstd Rnewendmean Rnewendstd Rnew_nonendmean Rnew_nonendstd propSimsNonEp propSimsNonEp_theoretical...
            numRunsMinor measuredProbMinor numRunsMajor measuredProbMajor meanmdist meannumInfectedVM...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,modePar, numLoci, casetype);
maxI = max(ImeanF,[],2); %mean final size SI
    %time
     maska = ImeanF >= maxI-1; %when first reaches max with small tolerance
    [C Index] = max(maska);
    MLaFS_static(j,z) = Index; %T1
close all;
%data logging
 fprintf(data.logOutput(data.m), '%f,', ImeanF);
fprintf(data.logOutput(data.m), '\n'); %new line

end
end
%no diversity
mode = 'static' %mode - can be 'static' or 'dynamic' 'Dynamic-random' 'Dynamic-favScore'
modePar.modeStr = mode

for j = 1:size(tinrg,2)%do for all tx times
    variantRichness =  1
    numTimesteps = numTimestepsVR(1)
    modePar.tinrg    = tinrg(j);  
    modePar.tinrv    = tinrv(j);
for z= 1:size(numEncountersBetweenUpdates,2) %do for all update rates
    numUpdates=numEncountersBetweenUpdates(z)
    modePar.numEncountersBetweenUpdates = numEncountersBetweenUpdates(z);
 betan = 0.02;  plotGraphs = 0; gamma = recovery; numExploits=1; numLoci=1; casetype = 'OR'; throwAway = 'nonEpidemics', SEpidemic=60; %cutoff;

 %agent
 [figMeanNei figNeiOneRun fighistSpRun fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetanew fighistGammanew figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            meanNeiIndexDynamicF betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean gammameanmean betameanmean2 betameanmeanNew betameanmeanNew2 gammameanmeanNew Smean ImeanF Rmean NSmean Bmean Smedian Imedian Rmedian S I R NS SmeanN ImeanN1 RmeanN1 BmeanN1 Snewendmean Snewendstd Rnewendmean Rnewendstd Rnew_nonendmean Rnew_nonendstd propSimsNonEp propSimsNonEp_theoretical...
            numRunsMinor measuredProbMinor numRunsMajor measuredProbMajor meanmdist meannumInfectedVM...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,modePar, numLoci, casetype);
maxI = max(ImeanF,[],2); %mean final size SI
    %time
     maska = ImeanF >= maxI-1; %when first reaches max with small tolerance
    [C Index] = max(maska);
    MLaFS_nodiv(j,z) = Index; %T1
close all;

%data logging
 fprintf(data.logOutput(data.m), '%f,', ImeanF);
fprintf(data.logOutput(data.m), '\n'); %new line
end

end



%close data logger
fclose(data.logOutput(data.m));

data.logDir = sprintf('%s\\outputdata', pwd)
%plot data
data.numRows= (j * z)*4;
data.numTimesteps=numTimesteps;
data.numNodes=numNodes;
data.numLoci=numLoci;
data.variantRichness=variantRichness;
dataplot(data,MLaFS,MLaRE);

%output data

 %time output data
MLaRE
MLaFS
MLaFS_static
MLaFS_nodiv

              %Time relationships
              figure
              plot(tinrv,MLaRE,'k+-') %all update rates RE
              hold on
              plot(tinrv,MLaFS,'k+--') %all update rates FS
              plot(tinrv,MLaFS_static,'r+--') %all update rates FS
              plot(tinrv,MLaFS_nodiv,'b+--') %all update rates FS
              
             % for j = 1:size(tinrg,2)%do for all tx times
             % plot(numEncountersBetweenUpdates,MLaRE(j,:),'k') %all update rates RE
             % hold on
             % plot(numEncountersBetweenUpdates,MLaFS(j,:),'k--') %all update rates FS
             % end
             legend('RV-E','FS', 'Static', 'No diversity')
            xlabel('time to tx virus')
            ylabel('Time to reach peak infection')
             title('numUpdates vs Time')

 
toc
end



%*********************************
%Dynamic Diversity reaching highest diversity level-
%max div vs time to tx genotype
%Random  in range
%***********************************
if doSIRNTests(307) == 1
%1 locus model
%outputDirectory = 'Graphs\'
outputDirectory = 'Graphs\'
tic
modePar.netStr = 'rwp'; %'rwp' random waypoint 're' random encounter
%rwp par
modePar.genotypeTimeOutPeriod = 10; %rwp
modePar.xmin     = 1;     % Minimum x-axis location (du)
modePar.xmax     = 600;    % Maximum x-axis location (du)
modePar.ymin     = 1;     % Minimum y-axis location (du)
modePar.ymax     = 600;    % Maximum y-axis location (du)
modePar.smin     = 1.4;  % Minimum speed (du / tu)
modePar.smax     = 1.4;  % Maximum speed (du / tu)
modePar.pmin     = 0;     % Minimum time for pause (integer tu)
modePar.pmax     = 0;     % Maximum time for pause (integer tu)
modePar.dinr     = 10;     % Radial distance for in-range (du)
modePar.dexceed  = 1000;  % Value larger than any possible distance (du)
modePar.tinrg    = 1;     % Time in-range for genotype transfer (tu)
modePar.tinrv    = 1;     % Time in-range for virus transfer (tu)
% Mode
% rwph.mode = rwph.MODE_RIR;  1    % Select Random in range mode
% rwph.mode = rwph.MODE_NIR;  2    % Select Nearest in range mode
modePar.moderwp = 1; %MODE_AIR;  % 4    % Select (Random) Available in range mode
% Log vector
modePar.logena = [1 2 3 4];
% Log directory for rwp
modePar.logDir = sprintf('%s\\log', pwd);
%data logging of output
data.logDir = sprintf('%s\\outputdata', pwd)
data.m = 304 %data file number
s = sprintf('%s\\data%i.txt', data.logDir,data.m)
data.logOutput(data.m) = fopen(s, 'wt');

numRuns = 1 %20 %100 %200
numNodes = 1000;

%VRmin = 5 %1
%VRmax = 5 %10
%VR = [2 5 10 20 50]
VR = [1 2 3 4 5 6 7 8 9 10 12 14 16 20 25 30 35 40 45 50]
numTimestepsVR = [300];
meanTimeStart = 100;
%numEncountersBetweenUpdates = [2 5 10 20 50]
numEncountersBetweenUpdates = [1]

%blacklist mode
modePar.blklist = 'no'; %yes or no
modePar.numToBlklist = 1
modePar.blklistrate = 0.004 %0.001
%update mode
modePar.update = 'no' %stop updating mode
modePar.bufferlengthNei=10 %buffer for buffering local nei diversity,
modePar.stdNeiUpdateThreshold = 0.05
%recovery
recovery = 0; 0.0005 %0.001
dec = 10 %decimation of output data
%constraints
modePar.numToConstrain = fix(numNodes*3/4) %constain a number of nodes to 1 variant
modePar.stopUpdatingTime = 200000 %time at which to stop updating the genotypes
%modePar.numEncountersBetweenUpdates = 1 % number of encounters between updates for random and favscore algorithms
modePar.bufferlength = 10 %length of genotype buffer for favscore algorithm
modePar.genoInit = 'mindiv'  %initial conditions of genotypes maxdiv (maximum diversity - random variant chosen) mindiv (minimum diversity - all nodes same)

%initialise output arrays
meanNeiIndexDynamic = zeros(1,numTimestepsVR(1));
meanNeiIndexDynamicF = zeros(1,numTimestepsVR(1));



mode = 'Dynamic-random' %mode - can be 'static' or 'dynamic' 'Dynamic-random' 'Dynamic-favScore'
%monte carlo random genotype SIR model
%set up diversity mode parameters
modePar.modeStr = mode

for j = 1:size(VR,2)%do for all VR
    variantRichness =  VR(j)
    numTimesteps = numTimestepsVR(1)
for z= 1:size(numEncountersBetweenUpdates,2)
    numUpdates = numEncountersBetweenUpdates(z)
    modePar.numEncountersBetweenUpdates = numEncountersBetweenUpdates(z);
 betan = 0.02;  plotGraphs = 0; gamma = recovery; numExploits=1; numLoci=1; casetype = 'OR'; throwAway = 'nonEpidemics', SEpidemic=60; %cutoff;

 %agent
 [figMeanNei figNeiOneRun fighistSpRun fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetanew fighistGammanew figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            meanNeiIndexDynamic betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean gammameanmean betameanmean2 betameanmeanNew betameanmeanNew2 gammameanmeanNew Smean Imean Rmean Bmean NSmean Smedian Imedian Rmedian S I R NS SmeanN ImeanN1 RmeanN1 BmeanN1 Snewendmean Snewendstd Rnewendmean Rnewendstd Rnew_nonendmean Rnew_nonendstd propSimsNonEp propSimsNonEp_theoretical...
            numRunsMinor measuredProbMinor numRunsMajor measuredProbMajor meanmdist meannumInfectedVM...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,modePar, numLoci, casetype);
meanDiv(j) = mean(meanNeiIndexDynamic(meanTimeStart:end));
   close all; 
   
%data logging
 fprintf(data.logOutput(data.m), '%f,', meanNeiIndexDynamic);
fprintf(data.logOutput(data.m), '\n'); %new line
   
end

end

mode = 'Dynamic-favScore' %mode - can be 'static' or 'dynamic' 'Dynamic-random' 'Dynamic-favScore'
modePar.modeStr = mode

for j = 1:size(VR,2)%do for all tx times
    variantRichness =  VR(j)
    numTimesteps = numTimestepsVR(1)
for z= 1:size(numEncountersBetweenUpdates,2) %do for all update rates
    numUpdates=numEncountersBetweenUpdates(z)
    modePar.numEncountersBetweenUpdates = numEncountersBetweenUpdates(z);
 betan = 0.02;  plotGraphs = 0; gamma = recovery; numExploits=1; numLoci=1; casetype = 'OR'; throwAway = 'nonEpidemics', SEpidemic=60; %cutoff;

 %agent
 [figMeanNei figNeiOneRun fighistSpRun fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetanew fighistGammanew figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            meanNeiIndexDynamicF betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean gammameanmean betameanmean2 betameanmeanNew betameanmeanNew2 gammameanmeanNew Smean Imean Rmean NSmean Bmean Smedian Imedian Rmedian S I R NS SmeanN ImeanN1 RmeanN1 BmeanN1 Snewendmean Snewendstd Rnewendmean Rnewendstd Rnew_nonendmean Rnew_nonendstd propSimsNonEp propSimsNonEp_theoretical...
            numRunsMinor measuredProbMinor numRunsMajor measuredProbMajor meanmdist meannumInfectedVM...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,modePar, numLoci, casetype);
meanDivF(j) = mean(meanNeiIndexDynamicF(meanTimeStart:end));
numAlleles = repmat(variantRichness,[1,numLoci]);
            [ Dmax(j) DmaxLocus ] = getMaxNeiCalc( numNodes, numAlleles );
            close all;
%data logging
 fprintf(data.logOutput(data.m), '%f,', meanNeiIndexDynamicF);
fprintf(data.logOutput(data.m), '\n'); %new line
end

end

            %outputs
            meanDiv
            meanDivF
            Dmax

%close data logger
fclose(data.logOutput(data.m));

%plot data
data.numRows= (j * z)*2;
data.numTimesteps=numTimesteps;
data.numNodes=numNodes;
data.numLoci=numLoci;
data.variantRichness=variantRichness;
dataplot(data,0,0);

%output data

             
            figMeanNei=figure
            plot(VR,meanDiv,'k','Linewidth',2)
            hold on
            plot(VR,meanDivF,'k--','Linewidth',2)             
            %plot(meanDiv_1000devices_ThreeQuarters,'k','Linewidth',2)
            %hold on
            %plot(meanDivF_1000devices_ThreeQuarters,'k--','Linewidth',2)
            plot(Dmax,'k.-','Linewidth',2)
            %plot(meanDiv_1000devices_quarter,'k','Linewidth',2)
            %plot(meanDivF_1000devices_quarter,'k--','Linewidth',2)
            %plot(meanDiv_1000devices_half,'k','Linewidth',2)
            %plot(meanDivF_1000devices_half,'k--','Linewidth',2)
           % plot(Dmax,'k.-','Linewidth',2)
            xlabel('Variant Richness')
            ylabel('Measured Variant Diversity (mean across time after maximum reached)')
            %legend('update: every 1 encounter','update: every 10 encounters','update: every 20 encounters','update: every 50 encounters')
            legend('Diversity: random','Diversity: favscore', 'Calculated maximum')
            title('constrained Dynamic')

 
toc
end



%*********************************
%Dynamic Diversity reaching highest diversity level-
%max div vs time to tx genotype
%using data from 303 to 305 AgentModels\randomGenotypesGraphs\rwp
%***********************************
if doSIRNTests(306) == 1  
%303 available in range
MLaRE303 =[49 45 57 83 182 284 317 446 554 750 727];
MLaFS303 =[33 48 47 66 176 260 308 443 526 587 679];
txTime303 = [1 3 5 7 10 11 13 15 17 19 20]; 
%304 random in range
%MLaRE304 = [36 49 57 77 157 166 174];
%MLaFS304 = [34 35 47 67 130 165 179];
MLaRE304 = [36 49 57 77 157 166 288 327 403 459 605 589];
MLaFS304 = [34 35 47 67 130 165 228 288 357 464 538 592];
txTime304 = [1 3 5 7 9 10 11 13 15 17 19 20];
%305 nearest in range
MLaRE305 =[46 46 70 80 135 164 246 368 451 471 598 638];
%MLaFS305 =[31 36 47 62 143 173 191];
MLaFS305 =[31 36 47 62 143 173 242 311 364 452 612 635];
txTime305 = [1 3 5 7 9 10 11 13 15 17 19 20];



              %Time relationships
              figure
              plot(txTime303,MLaRE303,'b+-') %all update rates RE
              hold on
              plot(txTime303,MLaFS303,'b+--') %all update rates FS
              plot(txTime304,MLaRE304,'r+-') %all update rates RE
              hold on
              plot(txTime304,MLaFS304,'r+--') %all update rates FS
              plot(txTime305,MLaRE305,'g+-') %all update rates RE
              hold on
              plot(txTime305,MLaFS305,'g+--') %all update rates FS
              
             legend('Available in range - RE','Available in range - FS', 'Random in range - RE', 'Random in range - FS', 'Nearest in range - RE', 'Nearest in range - FS')
            xlabel('Time to tx genotype')
            ylabel('Time to reach maximum')
             title('numUpdates vs Time')


end

%*********************************
%Dynamic Diversity reaching highest diversity level-
%max div vs time to tx genotype
%Random  in range
%***********************************
if doSIRNTests(305) == 1
%1 locus model
%outputDirectory = 'Graphs\'
outputDirectory = 'Graphs\'
tic
modePar.netStr = 'rwp'; %'rwp' random waypoint 're' random encounter
%rwp par
modePar.genotypeTimeOutPeriod = 10; %rwp
modePar.xmin     = 1;     % Minimum x-axis location (du)
modePar.xmax     = 600;    % Maximum x-axis location (du)
modePar.ymin     = 1;     % Minimum y-axis location (du)
modePar.ymax     = 600;    % Maximum y-axis location (du)
modePar.smin     = 1.4;  % Minimum speed (du / tu)
modePar.smax     = 1.4;  % Maximum speed (du / tu)
modePar.pmin     = 0;     % Minimum time for pause (integer tu)
modePar.pmax     = 0;     % Maximum time for pause (integer tu)
modePar.dinr     = 10;     % Radial distance for in-range (du)
modePar.dexceed  = 1000;  % Value larger than any possible distance (du)
%modePar.tinrg    = 2;     % Time in-range for genotype transfer (tu)
%modePar.tinrv    = 2;     % Time in-range for virus transfer (tu)
% Mode
% rwph.mode = rwph.MODE_RIR;  1    % Select Random in range mode
% rwph.mode = rwph.MODE_NIR;  2    % Select Nearest in range mode
modePar.moderwp = 2; %MODE_AIR;  % 4    % Select (Random) Available in range mode
% Log vector
modePar.logena = [1 2 3 4];
% Log directory for rwp
modePar.logDir = sprintf('%s\\log', pwd);
%data logging of output
data.logDir = sprintf('%s\\outputdata', pwd)
data.m = 305 %data file number
s = sprintf('%s\\data%i.txt', data.logDir,data.m)
data.logOutput(data.m) = fopen(s, 'wt');

numRuns = 5 %20 %100 %200
numNodes = 1000;

tinrg    = [11 13 15 17 19 20];     % Time in-range for genotype transfer (tu)
tinrv    = [11 13 15 17 19 20];     % Time in-range for virus transfer (tu)

%VRmin = 5 %1
%VRmax = 5 %10
%VR = [2 5 10 20 50]
VR = [5]
numTimestepsVR = [800];
%numEncountersBetweenUpdates = [2 5 10 20 50]
numEncountersBetweenUpdates = [1]

%blacklist mode
modePar.blklist = 'no'; %yes or no
modePar.numToBlklist = 1
modePar.blklistrate = 0.004 %0.001
%update mode
modePar.update = 'no' %stop updating mode
modePar.bufferlengthNei=10 %buffer for buffering local nei diversity,
modePar.stdNeiUpdateThreshold = 0.05
%recovery
recovery = 0; 0.0005 %0.001
dec = 10 %decimation of output data
%constraints
modePar.numToConstrain = 0;%fix(numNodes/2) %constain a number of nodes to 1 variant
modePar.stopUpdatingTime = 200000 %time at which to stop updating the genotypes
%modePar.numEncountersBetweenUpdates = 1 % number of encounters between updates for random and favscore algorithms
modePar.bufferlength = 10 %length of genotype buffer for favscore algorithm
modePar.genoInit = 'mindiv'  %initial conditions of genotypes maxdiv (maximum diversity - random variant chosen) mindiv (minimum diversity - all nodes same)

%initialise output arrays
meanNeiIndexDynamic = zeros(1,numTimestepsVR(1));
meanNeiIndexDynamicF = zeros(1,numTimestepsVR(1));



mode = 'Dynamic-random' %mode - can be 'static' or 'dynamic' 'Dynamic-random' 'Dynamic-favScore'
%monte carlo random genotype SIR model
%set up diversity mode parameters
modePar.modeStr = mode

for j = 1:size(tinrg,2)%do for all tx times
    variantRichness =  VR(1)
    numTimesteps = numTimestepsVR(1)
    modePar.tinrg    = tinrg(j);  
    modePar.tinrv    = tinrv(j);
for z= 1:size(numEncountersBetweenUpdates,2)
    numUpdates = numEncountersBetweenUpdates(z)
    modePar.numEncountersBetweenUpdates = numEncountersBetweenUpdates(z);
 betan = 0.02;  plotGraphs = 0; gamma = recovery; numExploits=1; numLoci=1; casetype = 'OR'; throwAway = 'nonEpidemics', SEpidemic=60; %cutoff;

 %agent
 [figMeanNei figNeiOneRun fighistSpRun fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetanew fighistGammanew figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            meanNeiIndexDynamic betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean gammameanmean betameanmean2 betameanmeanNew betameanmeanNew2 gammameanmeanNew Smean Imean Rmean Bmean NSmean Smedian Imedian Rmedian S I R NS SmeanN ImeanN1 RmeanN1 BmeanN1 Snewendmean Snewendstd Rnewendmean Rnewendstd Rnew_nonendmean Rnew_nonendstd propSimsNonEp propSimsNonEp_theoretical...
            numRunsMinor measuredProbMinor numRunsMajor measuredProbMajor meanmdist meannumInfectedVM...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,modePar, numLoci, casetype);
maxNei = max(meanNeiIndexDynamic,[],2); %mean final size SI
    %time
    size(maxNei)
     maska = meanNeiIndexDynamic >= maxNei-0.001; %when first reaches max with small tolerance
    [C Index] = max(maska);
    MLaRE(j,z) = Index; %T1
   close all; 
   
%data logging
 fprintf(data.logOutput(data.m), '%f,', meanNeiIndexDynamic);
fprintf(data.logOutput(data.m), '\n'); %new line
   
end

end


mode = 'Dynamic-favScore' %mode - can be 'static' or 'dynamic' 'Dynamic-random' 'Dynamic-favScore'
modePar.modeStr = mode

for j = 1:size(tinrg,2)%do for all tx times
    variantRichness =  VR(1)
    numTimesteps = numTimestepsVR(1)
    modePar.tinrg    = tinrg(j);  
    modePar.tinrv    = tinrv(j);
for z= 1:size(numEncountersBetweenUpdates,2) %do for all update rates
    numUpdates=numEncountersBetweenUpdates(z)
    modePar.numEncountersBetweenUpdates = numEncountersBetweenUpdates(z);
 betan = 0.02;  plotGraphs = 0; gamma = recovery; numExploits=1; numLoci=1; casetype = 'OR'; throwAway = 'nonEpidemics', SEpidemic=60; %cutoff;

 %agent
 [figMeanNei figNeiOneRun fighistSpRun fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetanew fighistGammanew figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            meanNeiIndexDynamicF betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean gammameanmean betameanmean2 betameanmeanNew betameanmeanNew2 gammameanmeanNew Smean Imean Rmean NSmean Bmean Smedian Imedian Rmedian S I R NS SmeanN ImeanN1 RmeanN1 BmeanN1 Snewendmean Snewendstd Rnewendmean Rnewendstd Rnew_nonendmean Rnew_nonendstd propSimsNonEp propSimsNonEp_theoretical...
            numRunsMinor measuredProbMinor numRunsMajor measuredProbMajor meanmdist meannumInfectedVM...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,modePar, numLoci, casetype);
maxNei = max(meanNeiIndexDynamicF,[],2); %mean final size SI
    %time
     maska = meanNeiIndexDynamicF >= maxNei-0.001; %when first reaches max with small tolerance
    [C Index] = max(maska);
    MLaFS(j,z) = Index; %T1
close all;
%data logging
 fprintf(data.logOutput(data.m), '%f,', meanNeiIndexDynamicF);
fprintf(data.logOutput(data.m), '\n'); %new line
end

end

%close data logger
fclose(data.logOutput(data.m));

%plot data
data.numRows= (j * z)*2;
data.numTimesteps=numTimesteps;
data.numNodes=numNodes;
data.numLoci=numLoci;
data.variantRichness=variantRichness;
dataplot(data,MLaFS,MLaRE);

%output data

 %time output data
MLaRE
MLaFS

              %Time relationships
              figure
              plot(tinrg,MLaRE,'k+-') %all update rates RE
              hold on
              plot(tinrg,MLaFS,'k+--') %all update rates FS
             % for j = 1:size(tinrg,2)%do for all tx times
             % plot(numEncountersBetweenUpdates,MLaRE(j,:),'k') %all update rates RE
             % hold on
             % plot(numEncountersBetweenUpdates,MLaFS(j,:),'k--') %all update rates FS
             % end
             legend('Diversity: random','Diversity: favscore')
            xlabel('time to tx genotype')
            ylabel('Time to reach maximum')
             title('numUpdates vs Time')

 
toc
end



%*********************************
%Dynamic Diversity reaching highest diversity level-
%max div vs time to tx genotype
%Random  in range
%***********************************
if doSIRNTests(304) == 1
%1 locus model
%outputDirectory = 'Graphs\'
outputDirectory = 'Graphs\'
tic
modePar.netStr = 'rwp'; %'rwp' random waypoint 're' random encounter
%rwp par
modePar.genotypeTimeOutPeriod = 10; %rwp
modePar.xmin     = 1;     % Minimum x-axis location (du)
modePar.xmax     = 600;    % Maximum x-axis location (du)
modePar.ymin     = 1;     % Minimum y-axis location (du)
modePar.ymax     = 600;    % Maximum y-axis location (du)
modePar.smin     = 1.4;  % Minimum speed (du / tu)
modePar.smax     = 1.4;  % Maximum speed (du / tu)
modePar.pmin     = 0;     % Minimum time for pause (integer tu)
modePar.pmax     = 0;     % Maximum time for pause (integer tu)
modePar.dinr     = 10;     % Radial distance for in-range (du)
modePar.dexceed  = 1000;  % Value larger than any possible distance (du)
%modePar.tinrg    = 2;     % Time in-range for genotype transfer (tu)
%modePar.tinrv    = 2;     % Time in-range for virus transfer (tu)
% Mode
% rwph.mode = rwph.MODE_RIR;  1    % Select Random in range mode
% rwph.mode = rwph.MODE_NIR;  2    % Select Nearest in range mode
modePar.moderwp = 1; %MODE_AIR;  % 4    % Select (Random) Available in range mode
% Log vector
modePar.logena = [1 2 3 4];
% Log directory for rwp
modePar.logDir = sprintf('%s\\log', pwd);
%data logging of output
data.logDir = sprintf('%s\\outputdata', pwd)
data.m = 304 %data file number
s = sprintf('%s\\data%i.txt', data.logDir,data.m)
data.logOutput(data.m) = fopen(s, 'wt');

numRuns = 5 %20 %100 %200
numNodes = 1000;

tinrg    = [1 3 5 7 9 10 11];     % Time in-range for genotype transfer (tu)
tinrv    = [1 3 5 7 9 10 11];     % Time in-range for virus transfer (tu)

%VRmin = 5 %1
%VRmax = 5 %10
%VR = [2 5 10 20 50]
VR = [5]
numTimestepsVR = [200];
%numEncountersBetweenUpdates = [2 5 10 20 50]
numEncountersBetweenUpdates = [1]

%blacklist mode
modePar.blklist = 'no'; %yes or no
modePar.numToBlklist = 1
modePar.blklistrate = 0.004 %0.001
%update mode
modePar.update = 'no' %stop updating mode
modePar.bufferlengthNei=10 %buffer for buffering local nei diversity,
modePar.stdNeiUpdateThreshold = 0.05
%recovery
recovery = 0; 0.0005 %0.001
dec = 10 %decimation of output data
%constraints
modePar.numToConstrain = 0;%fix(numNodes/2) %constain a number of nodes to 1 variant
modePar.stopUpdatingTime = 200000 %time at which to stop updating the genotypes
%modePar.numEncountersBetweenUpdates = 1 % number of encounters between updates for random and favscore algorithms
modePar.bufferlength = 10 %length of genotype buffer for favscore algorithm
modePar.genoInit = 'mindiv'  %initial conditions of genotypes maxdiv (maximum diversity - random variant chosen) mindiv (minimum diversity - all nodes same)

%initialise output arrays
meanNeiIndexDynamic = zeros(1,numTimestepsVR(1));
meanNeiIndexDynamicF = zeros(1,numTimestepsVR(1));



mode = 'Dynamic-random' %mode - can be 'static' or 'dynamic' 'Dynamic-random' 'Dynamic-favScore'
%monte carlo random genotype SIR model
%set up diversity mode parameters
modePar.modeStr = mode

for j = 1:size(tinrg,2)%do for all tx times
    variantRichness =  VR(1)
    numTimesteps = numTimestepsVR(1)
    modePar.tinrg    = tinrg(j);  
    modePar.tinrv    = tinrv(j);
for z= 1:size(numEncountersBetweenUpdates,2)
    numUpdates = numEncountersBetweenUpdates(z)
    modePar.numEncountersBetweenUpdates = numEncountersBetweenUpdates(z);
 betan = 0.02;  plotGraphs = 0; gamma = recovery; numExploits=1; numLoci=1; casetype = 'OR'; throwAway = 'nonEpidemics', SEpidemic=60; %cutoff;

 %agent
 [figMeanNei figNeiOneRun fighistSpRun fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetanew fighistGammanew figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            meanNeiIndexDynamic betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean gammameanmean betameanmean2 betameanmeanNew betameanmeanNew2 gammameanmeanNew Smean Imean Rmean Bmean NSmean Smedian Imedian Rmedian S I R NS SmeanN ImeanN1 RmeanN1 BmeanN1 Snewendmean Snewendstd Rnewendmean Rnewendstd Rnew_nonendmean Rnew_nonendstd propSimsNonEp propSimsNonEp_theoretical...
            numRunsMinor measuredProbMinor numRunsMajor measuredProbMajor meanmdist meannumInfectedVM...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,modePar, numLoci, casetype);
maxNei = max(meanNeiIndexDynamic,[],2); %mean final size SI
    %time
    size(maxNei)
     maska = meanNeiIndexDynamic >= maxNei-0.001; %when first reaches max with small tolerance
    [C Index] = max(maska);
    MLaRE(j,z) = Index; %T1
   close all; 
   
%data logging
 fprintf(data.logOutput(data.m), '%f,', meanNeiIndexDynamic);
fprintf(data.logOutput(data.m), '\n'); %new line
   
end

end


mode = 'Dynamic-favScore' %mode - can be 'static' or 'dynamic' 'Dynamic-random' 'Dynamic-favScore'
modePar.modeStr = mode

for j = 1:size(tinrg,2)%do for all tx times
    variantRichness =  VR(1)
    numTimesteps = numTimestepsVR(1)
    modePar.tinrg    = tinrg(j);  
    modePar.tinrv    = tinrv(j);
for z= 1:size(numEncountersBetweenUpdates,2) %do for all update rates
    numUpdates=numEncountersBetweenUpdates(z)
    modePar.numEncountersBetweenUpdates = numEncountersBetweenUpdates(z);
 betan = 0.02;  plotGraphs = 0; gamma = recovery; numExploits=1; numLoci=1; casetype = 'OR'; throwAway = 'nonEpidemics', SEpidemic=60; %cutoff;

 %agent
 [figMeanNei figNeiOneRun fighistSpRun fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetanew fighistGammanew figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            meanNeiIndexDynamicF betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean gammameanmean betameanmean2 betameanmeanNew betameanmeanNew2 gammameanmeanNew Smean Imean Rmean NSmean Bmean Smedian Imedian Rmedian S I R NS SmeanN ImeanN1 RmeanN1 BmeanN1 Snewendmean Snewendstd Rnewendmean Rnewendstd Rnew_nonendmean Rnew_nonendstd propSimsNonEp propSimsNonEp_theoretical...
            numRunsMinor measuredProbMinor numRunsMajor measuredProbMajor meanmdist meannumInfectedVM...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,modePar, numLoci, casetype);
maxNei = max(meanNeiIndexDynamicF,[],2); %mean final size SI
    %time
     maska = meanNeiIndexDynamicF >= maxNei-0.001; %when first reaches max with small tolerance
    [C Index] = max(maska);
    MLaFS(j,z) = Index; %T1
close all;
%data logging
 fprintf(data.logOutput(data.m), '%f,', meanNeiIndexDynamicF);
fprintf(data.logOutput(data.m), '\n'); %new line
end

end

%close data logger
fclose(data.logOutput(data.m));

%plot data
data.numRows= (j * z)*2;
data.numTimesteps=numTimesteps;
data.numNodes=numNodes;
data.numLoci=numLoci;
data.variantRichness=variantRichness;
dataplot(data,MLaFS,MLaRE);

%output data

 %time output data
MLaRE
MLaFS

              %Time relationships
              figure
              plot(tinrg,MLaRE,'k+-') %all update rates RE
              hold on
              plot(tinrg,MLaFS,'k+--') %all update rates FS
             % for j = 1:size(tinrg,2)%do for all tx times
             % plot(numEncountersBetweenUpdates,MLaRE(j,:),'k') %all update rates RE
             % hold on
             % plot(numEncountersBetweenUpdates,MLaFS(j,:),'k--') %all update rates FS
             % end
             legend('Diversity: random','Diversity: favscore')
            xlabel('time to tx genotype')
            ylabel('Time to reach maximum')
             title('numUpdates vs Time')

 
toc
end







%*********************************
%Dynamic Diversity reaching highest diversity level-
%max div vs time to tx genotype
%Random Available in range
%***********************************
if doSIRNTests(303) == 1
%1 locus model
%outputDirectory = 'Graphs\'
outputDirectory = 'Graphs\'
tic
modePar.netStr = 'rwp'; %'rwp' random waypoint 're' random encounter
%rwp par
modePar.genotypeTimeOutPeriod = 10; %rwp
modePar.xmin     = 1;     % Minimum x-axis location (du)
modePar.xmax     = 600;    % Maximum x-axis location (du)
modePar.ymin     = 1;     % Minimum y-axis location (du)
modePar.ymax     = 600;    % Maximum y-axis location (du)
modePar.smin     = 1.4;  % Minimum speed (du / tu)
modePar.smax     = 1.4;  % Maximum speed (du / tu)
modePar.pmin     = 0;     % Minimum time for pause (integer tu)
modePar.pmax     = 0;     % Maximum time for pause (integer tu)
modePar.dinr     = 10;     % Radial distance for in-range (du)
modePar.dexceed  = 1000;  % Value larger than any possible distance (du)
%modePar.tinrg    = 2;     % Time in-range for genotype transfer (tu)
%modePar.tinrv    = 2;     % Time in-range for virus transfer (tu)
% Mode
% rwph.mode = rwph.MODE_RIR;  1    % Select Random in range mode
% rwph.mode = rwph.MODE_NIR;  2    % Select Nearest in range mode
modePar.moderwp = 4; %MODE_AIR;  % 4    % Select (Random) Available in range mode
% Log vector
modePar.logena = [1 2 3 4];
% Log directory for rwp
modePar.logDir = sprintf('%s\\log', pwd);
%data logging of output
data.logDir = sprintf('%s\\outputdata', pwd)
data.m = 303 %data file number
s = sprintf('%s\\data%i.txt', data.logDir,data.m)
data.logOutput(data.m) = fopen(s, 'wt');

numRuns = 5 %20 %100 %200
numNodes = 1000;

tinrg    = [1 3 5 7 10];     % Time in-range for genotype transfer (tu)
tinrv    = [1 3 5 7 10];     % Time in-range for virus transfer (tu)

%VRmin = 5 %1
%VRmax = 5 %10
%VR = [2 5 10 20 50]
VR = [5]
numTimestepsVR = [200];
%numEncountersBetweenUpdates = [2 5 10 20 50]
numEncountersBetweenUpdates = [1]

%blacklist mode
modePar.blklist = 'no'; %yes or no
modePar.numToBlklist = 1
modePar.blklistrate = 0.004 %0.001
%update mode
modePar.update = 'no' %stop updating mode
modePar.bufferlengthNei=10 %buffer for buffering local nei diversity,
modePar.stdNeiUpdateThreshold = 0.05
%recovery
recovery = 0; 0.0005 %0.001
dec = 10 %decimation of output data
%constraints
modePar.numToConstrain = 0;%fix(numNodes/2) %constain a number of nodes to 1 variant
modePar.stopUpdatingTime = 200000 %time at which to stop updating the genotypes
%modePar.numEncountersBetweenUpdates = 1 % number of encounters between updates for random and favscore algorithms
modePar.bufferlength = 10 %length of genotype buffer for favscore algorithm
modePar.genoInit = 'mindiv'  %initial conditions of genotypes maxdiv (maximum diversity - random variant chosen) mindiv (minimum diversity - all nodes same)

%initialise output arrays
meanNeiIndexDynamic = zeros(1,numTimestepsVR(1));
meanNeiIndexDynamicF = zeros(1,numTimestepsVR(1));



mode = 'Dynamic-random' %mode - can be 'static' or 'dynamic' 'Dynamic-random' 'Dynamic-favScore'
%monte carlo random genotype SIR model
%set up diversity mode parameters
modePar.modeStr = mode

for j = 1:size(tinrg,2)%do for all tx times
    variantRichness =  VR(1)
    numTimesteps = numTimestepsVR(1)
    modePar.tinrg    = tinrg(j);  
    modePar.tinrv    = tinrv(j);
for z= 1:size(numEncountersBetweenUpdates,2)
    numUpdates = numEncountersBetweenUpdates(z)
    modePar.numEncountersBetweenUpdates = numEncountersBetweenUpdates(z);
 betan = 0.02;  plotGraphs = 0; gamma = recovery; numExploits=1; numLoci=1; casetype = 'OR'; throwAway = 'nonEpidemics', SEpidemic=60; %cutoff;

 %agent
 [figMeanNei figNeiOneRun fighistSpRun fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetanew fighistGammanew figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            meanNeiIndexDynamic betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean gammameanmean betameanmean2 betameanmeanNew betameanmeanNew2 gammameanmeanNew Smean Imean Rmean Bmean NSmean Smedian Imedian Rmedian S I R NS SmeanN ImeanN1 RmeanN1 BmeanN1 Snewendmean Snewendstd Rnewendmean Rnewendstd Rnew_nonendmean Rnew_nonendstd propSimsNonEp propSimsNonEp_theoretical...
            numRunsMinor measuredProbMinor numRunsMajor measuredProbMajor meanmdist meannumInfectedVM...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,modePar, numLoci, casetype);
maxNei = max(meanNeiIndexDynamic,[],2); %mean final size SI
    %time
    size(maxNei)
     maska = meanNeiIndexDynamic >= maxNei-0.001; %when first reaches max with small tolerance
    [C Index] = max(maska);
    MLaRE(j,z) = Index; %T1
   close all; 
   
%data logging
 fprintf(data.logOutput(data.m), '%f,', meanNeiIndexDynamic);
fprintf(data.logOutput(data.m), '\n'); %new line
   
end

end


mode = 'Dynamic-favScore' %mode - can be 'static' or 'dynamic' 'Dynamic-random' 'Dynamic-favScore'
modePar.modeStr = mode

for j = 1:size(tinrg,2)%do for all tx times
    variantRichness =  VR(1)
    numTimesteps = numTimestepsVR(1)
    modePar.tinrg    = tinrg(j);  
    modePar.tinrv    = tinrv(j);
for z= 1:size(numEncountersBetweenUpdates,2) %do for all update rates
    numUpdates=numEncountersBetweenUpdates(z)
    modePar.numEncountersBetweenUpdates = numEncountersBetweenUpdates(z);
 betan = 0.02;  plotGraphs = 0; gamma = recovery; numExploits=1; numLoci=1; casetype = 'OR'; throwAway = 'nonEpidemics', SEpidemic=60; %cutoff;

 %agent
 [figMeanNei figNeiOneRun fighistSpRun fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetanew fighistGammanew figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            meanNeiIndexDynamicF betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean gammameanmean betameanmean2 betameanmeanNew betameanmeanNew2 gammameanmeanNew Smean Imean Rmean NSmean Bmean Smedian Imedian Rmedian S I R NS SmeanN ImeanN1 RmeanN1 BmeanN1 Snewendmean Snewendstd Rnewendmean Rnewendstd Rnew_nonendmean Rnew_nonendstd propSimsNonEp propSimsNonEp_theoretical...
            numRunsMinor measuredProbMinor numRunsMajor measuredProbMajor meanmdist meannumInfectedVM...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,modePar, numLoci, casetype);
maxNei = max(meanNeiIndexDynamicF,[],2); %mean final size SI
    %time
     maska = meanNeiIndexDynamicF >= maxNei-0.001; %when first reaches max with small tolerance
    [C Index] = max(maska);
    MLaFS(j,z) = Index; %T1
close all;
%data logging
 fprintf(data.logOutput(data.m), '%f,', meanNeiIndexDynamicF);
fprintf(data.logOutput(data.m), '\n'); %new line
end

end

%close data logger
fclose(data.logOutput(data.m));

%plot data
data.numRows= (j * z)*2;
data.numTimesteps=numTimesteps;
data.numNodes=numNodes;
data.numLoci=numLoci;
data.variantRichness=variantRichness;
dataplot(data,MLaFS,MLaRE);

%output data

 %time output data
MLaRE
MLaFS

              %Time relationships
              figure
              plot(tinrg,MLaRE,'k+-') %all update rates RE
              hold on
              plot(tinrg,MLaFS,'k+--') %all update rates FS
             % for j = 1:size(tinrg,2)%do for all tx times
             % plot(numEncountersBetweenUpdates,MLaRE(j,:),'k') %all update rates RE
             % hold on
             % plot(numEncountersBetweenUpdates,MLaFS(j,:),'k--') %all update rates FS
             % end
             legend('Diversity: random','Diversity: favscore')
            xlabel('time to tx genotype')
            ylabel('Time to reach maximum')
             title('numUpdates vs Time')

 
toc
end



%*********************************
%Dynamic Diversity reaching highest diversity level-
%time graphs
%RANDOM IN RANGE
%***********************************
if doSIRNTests(301) == 1
    plotprevResults=1
    if plotprevResults==1 %plot saved data
        %VR=5
        numEncountersBetweenUpdates = [1 10 20 50]
        %available in range
        MLaFS_av = [38   219   320   736];
        MLaRE_av = [62   323   538   784];
        %random in range
        MLaFS_ra =[64   139   478   678];
        MLaRE_ra =[54   265   446   773];
        %nearest in range
        MLaFS_ne =[43   209   283   681];
        MLaRE_ne =[63   281   426   787];
        
        %Time relationships
        figure
        plot(numEncountersBetweenUpdates,MLaRE_av(1,:),'k+-','linewidth', 2) %all update rates RE
        hold on
        plot(numEncountersBetweenUpdates,MLaFS_av(1,:),'r+--','linewidth', 2) %all update rates FS
        plot(numEncountersBetweenUpdates,MLaRE_ra(1,:),'ks-','linewidth', 2) %all update rates RE
        hold on
        plot(numEncountersBetweenUpdates,MLaFS_ra(1,:),'rs--','linewidth', 2) %all update rates FS
        plot(numEncountersBetweenUpdates,MLaRE_ne(1,:),'ko-','linewidth', 2) %all update rates RE
        plot(numEncountersBetweenUpdates,MLaFS_ne(1,:),'ro--','linewidth', 2) %all update rates FS

        legend('RV-E - Available in range', 'FS - Available in range','RV-E - Random in range','FS -  Random in range','RV-E - Nearest in range','FS - Nearest in range')
        xlabel('Number of encounters before an update')
        ylabel('Mean time to reach maximum')
        %title('numUpdates vs Time')
     
    else
%1 locus model
%outputDirectory = 'Graphs\'
outputDirectory = 'Graphs\'
tic
modePar.netStr = 'rwp'; %'rwp' random waypoint 're' random encounter
%rwp par
modePar.genotypeTimeOutPeriod = 10; %rwp
modePar.xmin     = 1;     % Minimum x-axis location (du)
modePar.xmax     = 600;    % Maximum x-axis location (du)
modePar.ymin     = 1;     % Minimum y-axis location (du)
modePar.ymax     = 600;    % Maximum y-axis location (du)
modePar.smin     = 0.25;  % Minimum speed (du / tu)
modePar.smax     = 1.40;  % Maximum speed (du / tu)
modePar.pmin     = 2;     % Minimum time for pause (integer tu)
modePar.pmax     = 4;     % Maximum time for pause (integer tu)
modePar.dinr     = 10;     % Radial distance for in-range (du)
modePar.dexceed  = 1000;  % Value larger than any possible distance (du)
modePar.tinrg    = 2;     % Time in-range for genotype transfer (tu)
modePar.tinrv    = 2;     % Time in-range for virus transfer (tu)
% Mode
% rwph.mode = rwph.MODE_RIR;  1    % Select Random in range mode
% rwph.mode = rwph.MODE_NIR;  2    % Select Nearest in range mode
modePar.moderwp = 4; %MODE_AIR;  % 4    % Select (Random) Available in range mode
% Log vector
modePar.logena = [1 2 3 4];
% Log directory
modePar.logDir = sprintf('%s\\log', pwd);

numRuns = 1 %20 %100 %200
numNodes = 1000;
%VRmin = 5 %1
%VRmax = 5 %10
%VR = [2 5 10 20 50]
VR = [5]
numTimestepsVR = [800 800 800 800 800];
%numEncountersBetweenUpdates = [2 5 10 20 50]
numEncountersBetweenUpdates = [1 10 20 50]

%rwp constrain
modePar.xboxc=0.5
modePar.yboxc=0.5
modePar.rwpcontrainplotGraph=0 %draw init constraints graph
modePar.virusStartTime = 1;
modePar.statnodesrwp = 'dynamic'; %'dynamic' or 'stationary' nodes, if dynamic box constraints applied at point of virus start
Imean = zeros(1,numTimestepsVR(1)); %init if runs all tests
%**********************
%1. rwp box constraints
%modePar.numToConstrain = 0;%fix(numNodes*1/4); %constain a number of nodes to 1 variant
modePar.areaConstrainrwp='no'
%**********************


%blacklist mode
modePar.blklist = 'no'; %yes or no
modePar.numToBlklist = 1
modePar.blklistrate = 0.004 %0.001
%update mode
modePar.update = 'no' %stop updating mode
modePar.bufferlengthNei=10 %buffer for buffering local nei diversity,
modePar.stdNeiUpdateThreshold = 0.05
%recovery
recovery = 0; 0.0005 %0.001
dec = 10 %decimation of output data
%constraints
modePar.numToConstrain = 0;%fix(numNodes/2) %constain a number of nodes to 1 variant
modePar.stopUpdatingTime = 200000 %time at which to stop updating the genotypes
%modePar.numEncountersBetweenUpdates = 1 % number of encounters between updates for random and favscore algorithms
modePar.bufferlength = 10 %length of genotype buffer for favscore algorithm
modePar.genoInit = 'mindiv'  %initial conditions of genotypes maxdiv (maximum diversity - random variant chosen) mindiv (minimum diversity - all nodes same)

%initialise output arrays
meanNeiIndexDynamic = zeros(1,800);
meanNeiIndexDynamicF = zeros(1,800);



mode = 'Dynamic-random' %mode - can be 'static' or 'dynamic' 'Dynamic-random' 'Dynamic-favScore'
%monte carlo random genotype SIR model
%set up diversity mode parameters
modePar.modeStr = mode

for j = 1:size(VR,2)%do for all VRs
    variantRichness =  VR(j)
    numTimesteps = numTimestepsVR(j)
for z= 1:size(numEncountersBetweenUpdates,2)
    numUpdates = numEncountersBetweenUpdates(z)
    modePar.numEncountersBetweenUpdates = numEncountersBetweenUpdates(z);
 betan = 0.02;  plotGraphs = 0; gamma = recovery; numExploits=1; numLoci=1; casetype = 'OR'; throwAway = 'nonEpidemics', SEpidemic=60; %cutoff;
VRNumberSimOn=j
NumEncountersSimOn=z
 %agent
 [figMeanNei figNeiOneRun fighistSpRun fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetanew fighistGammanew figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            meanNeiIndexDynamic(z,1:numTimesteps) betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean gammameanmean betameanmean2 betameanmeanNew betameanmeanNew2 gammameanmeanNew Smean Imean Rmean Bmean NSmean Smedian Imedian Rmedian S I R NS SmeanN ImeanN1 RmeanN1 BmeanN1 Snewendmean Snewendstd Rnewendmean Rnewendstd Rnew_nonendmean Rnew_nonendstd propSimsNonEp propSimsNonEp_theoretical...
            numRunsMinor measuredProbMinor numRunsMajor measuredProbMajor meanmdist meannumInfectedVM...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,modePar, numLoci, casetype);
maxNei = max(meanNeiIndexDynamic(z,1:numTimesteps),[],2); %mean final size SI
    %time
    size(maxNei)
     maska = meanNeiIndexDynamic(z,1:numTimesteps) >= maxNei-0.001; %when first reaches max with small tolerance
    [C Index] = max(maska);
    MLaRE(j,z) = Index; %T1
   close all; 
end
if j == 1 %VR=5 save data for graph
    meanNeiIndexDynamic_VR5 = meanNeiIndexDynamic;
end

end



mode = 'Dynamic-favScore' %mode - can be 'static' or 'dynamic' 'Dynamic-random' 'Dynamic-favScore'
modePar.modeStr = mode

for j = 1:size(VR,2)%do for all VRs
    variantRichness =  VR(j)
    numTimesteps = numTimestepsVR(j)
for z= 1:size(numEncountersBetweenUpdates,2) %do for all update rates
    numUpdates=numEncountersBetweenUpdates(z)
    modePar.numEncountersBetweenUpdates = numEncountersBetweenUpdates(z);
 betan = 0.02;  plotGraphs = 0; gamma = recovery; numExploits=1; numLoci=1; casetype = 'OR'; throwAway = 'nonEpidemics', SEpidemic=60; %cutoff;

 %agent
 [figMeanNei figNeiOneRun fighistSpRun fighistRecovered fighistSusceptible fighistpeakValue fighistpeakTime fighistBeta fighistGamma fighistBetanew fighistGammanew figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            meanNeiIndexDynamicF(z,1:numTimesteps) betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean gammameanmean betameanmean2 betameanmeanNew betameanmeanNew2 gammameanmeanNew Smean Imean Rmean NSmean Bmean Smedian Imedian Rmedian S I R NS SmeanN ImeanN1 RmeanN1 BmeanN1 Snewendmean Snewendstd Rnewendmean Rnewendstd Rnew_nonendmean Rnew_nonendstd propSimsNonEp propSimsNonEp_theoretical...
            numRunsMinor measuredProbMinor numRunsMajor measuredProbMajor meanmdist meannumInfectedVM...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,modePar, numLoci, casetype);
maxNei = max(meanNeiIndexDynamicF(z,1:numTimesteps),[],2); %mean final size SI
    %time
     maska = meanNeiIndexDynamicF(z,1:numTimesteps) >= maxNei-0.001; %when first reaches max with small tolerance
    [C Index] = max(maska);
    MLaFS(j,z) = Index; %T1
close all;    
end
if j == 1 %VR=5 save data for graph
    meanNeiIndexDynamic_VR5F = meanNeiIndexDynamicF;
end
end

%output data
%decimated output
 meanNeiIndexDynamic_VR5Output1 = meanNeiIndexDynamic_VR5(1,1:dec:numTimesteps)%update rate = 1
 meanNeiIndexDynamic_VR5Output10 = meanNeiIndexDynamic_VR5(2,1:dec:numTimesteps)%update rate = 10
 meanNeiIndexDynamic_VR5Output20 = meanNeiIndexDynamic_VR5(3,1:dec:numTimesteps)%update rate = 20
 meanNeiIndexDynamic_VR5Output50 = meanNeiIndexDynamic_VR5(4,1:dec:numTimesteps)%update rate = 50

 
 meanNeiIndexDynamic_VR5FOutput1 = meanNeiIndexDynamic_VR5F(1,1:dec:numTimesteps)%update rate = 1
 meanNeiIndexDynamic_VR5FOutput10 = meanNeiIndexDynamic_VR5F(2,1:dec:numTimesteps)%update rate = 10
 meanNeiIndexDynamic_VR5FOutput20 = meanNeiIndexDynamic_VR5F(3,1:dec:numTimesteps)%update rate = 20
 meanNeiIndexDynamic_VR5FOutput50 = meanNeiIndexDynamic_VR5F(4,1:dec:numTimesteps)%update rate = 50

 %time output data
MLaFS
MLaRE

              %Time relationships
              figure
              plot(numEncountersBetweenUpdates,MLaRE(1,:),'k') %all update rates RE
              hold on
              plot(numEncountersBetweenUpdates,MLaFS(1,:),'k--') %all update rates FS
              for j = 1:size(VR,2)%do for all VRs
              plot(numEncountersBetweenUpdates,MLaRE(j,:),'k') %all update rates RE
              hold on
              plot(numEncountersBetweenUpdates,MLaFS(j,:),'k--') %all update rates FS
              end
             legend('Diversity: random','Diversity: favscore')
            xlabel('Number of encounters before an update')
            ylabel('Time to reach maximum')
             title('numUpdates vs Time')
             
             
             

              figure
              plot(VR,MLaRE(:,1),'k') %all update rates RE
              hold on
              plot(VR,MLaFS(:,1),'k--') %all update rates FS
              for z = 1:size(numEncountersBetweenUpdates,2)%do for all VRs
              plot(VR,MLaRE(:,z),'k') %all update rates RE
              hold on
              plot(VR,MLaFS(:,z),'k--') %all update rates FS
              end
             legend('Diversity: random','Diversity: favscore')
            xlabel('Number of Variants')
            ylabel('Time to reach maximum')
             title('numUpdates vs Time')
 

                       %diversity mean DISPLAYS VR value (5)
            figMeanNei=figure
            for s= 1:size(numEncountersBetweenUpdates,2)
            plot(meanNeiIndexDynamic_VR5(s,:),'k','Linewidth',2)
            hold on
            plot(meanNeiIndexDynamic_VR5F(s,:),'k--','Linewidth',2)
            end
            numAlleles = repmat(variantRichness,[1,numLoci]);
            [ Dmax DmaxLocus ] = getMaxNeiCalc( numNodes, numAlleles );
            DmaxLine = repmat(Dmax,[1 numTimesteps]);
            plot(DmaxLine,'k','Linewidth',2)
            xlabel('Time (seconds)')
            ylabel('Variant Diversity')
            axis([0 numTimesteps 0 1])
            %legend('update: every 1 encounter','update: every 10 encounters','update: every 20 encounters','update: every 50 encounters')
            legend('RV-E','FS')
            title('VR=5')
toc
    end %end sim
end %310



%*****************************
%%testing rwp algorithm
%*****************************
if doSIRNTests(300) == 1
%****************************************
%***************************************
numNodes = 100 %50;
modePar.netStr = 'rwp'; %'rwp' random waypoint 're' random encounter
%rwp par
%new rwp box constraints
modePar.nodesln  = 0; %fix(numNodes/4);     % Number of limited nodes (absolute) - always first
modePar.xminln   = 1;     % Minimum x-axis location limited nodes (du)
modePar.xmaxln   = 16;    % Maximum x-axis location limited nodes (du)
modePar.yminln   = 1;     % Minimum y-axis location limited nodes (du)
modePar.ymaxln   = 16;    % Maximum y-axis location limited nodes (du)
modePar.genotypeTimeOutPeriod = 10; %rwp
modePar.xmin     = 1;     % Minimum x-axis location (du)
modePar.xmax     = 31; %50   % Maximum x-axis location (du)
modePar.ymin     = 1;     % Minimum y-axis location (du)
modePar.ymax     = 31; %50   % Maximum y-axis location (du)
modePar.smin     = 0; %0.25;  % Minimum speed (du / tu)
modePar.smax     = 0; %1.00;  % Maximum speed (du / tu)
modePar.pmin     = 2;     % Minimum time for pause (integer tu)
modePar.pmax     = 4;     % Maximum time for pause (integer tu)
modePar.dinr     = 10;     % Radial distance for in-range (du)
modePar.dexceed  = 1000;  % Value larger than any possible distance (du)
modePar.tinrg    = 1;     % Time in-range for genotype transfer (tu)
modePar.tinrv    = 1;     % Time in-range for virus transfer (tu)
% Mode
% rwph.mode = rwph.MODE_RIR;  1    % Select Random in range mode
% rwph.mode = rwph.MODE_NIR;  2    % Select Nearest in range mode
modePar.moderwp = 1; %MODE_AIR;  % 4    % Select (Random) Available in range mode
% Log vector
modePar.logena = [1 2 3 4 5 6];
% Log directory
modePar.logDir = sprintf('%s\\log', pwd);

%rwp constrain
modePar.xboxc=0.5
modePar.yboxc=0.5
modePar.rwpcontrainplotGraph=1 %draw init constraints graph
modePar.virusStartTime = 500;
modePar.statnodesrwp = 'stationary'; %rwp change keep as stationary to set initially all time%'dynamic' or 'stationary' nodes, if dynamic box constraints applied at point of virus start
%Imean = zeros(1,numTimestepsVR(1)); %init if runs all tests
%**********************
%1. rwp box constraints
modePar.numToConstrain = 0; %fix(numNodes*1/4); %constain a number of nodes to 1 variant
modePar.areaConstrainrwp='no'
%**********************



%
numRuns = 1 %100 %200
numTimesteps=30 %120
VRmin = 5 ;%1
VRmax = 5; %10
modePar.update = 'yes' %stop updating mode
modePar.bufferlengthNei=10 %buffer for buffering local nei diversity,
modePar.stdNeiUpdateThreshold = 0.03
modePar.blklistStartTime = 2000000
modePar.numToBlklist = 1
modePar.blklistrate = 0.002 %0.001
recovery = 0; 0.0005 %0.001
dec = 100 %decimation of output data
%modePar.numToConstrain = 0; %constain a number of nodes to 1 variant
modePar.stopUpdatingTime = 200000; %time at which to stop updating the genotypes
modePar.numEncountersBetweenUpdates = 10; % number of encounters between updates for random and favscore algorithms
modePar.bufferlength = 10; %length of genotype buffer for favscore algorithm
modePar.genoInit = 'mindiv';  %initial conditions of genotypes maxdiv (maximum diversity - random variant chosen) mindiv (minimum diversity - all nodes same)
tic
%5.dynamic diversity
modePar.blklist = 'no'; %yes or no
mode = 'Dynamic-favScore'; %mode - can be 'static' or 'dynamic' 'Dynamic-random' 'Dynamic-favScore'
%monte carlo random genotype SIR model
%set up diversity mode parameters
modePar.modeStr = mode;
for z= VRmin:VRmax
 betan = 0.2; variantRichness =  z;   plotGraphs = 1; gamma = recovery; numExploits=1; numLoci=1; casetype = 'OR'; throwAway = 'nonEpidemics'; SEpidemic=60; %cutoff;
%agent
 [figMeanNei(z) figNeiOneRun(z) fighistSpRun(z) fighistRecovered(z) fighistSusceptible(z) fighistpeakValue(z) fighistpeakTime(z) fighistBeta(z) fighistGamma(z) fighistBetanew(z) fighistGammanew(z) figInfectedRun1 figBetaRun1 figGammaRun1 figRoRun1 figBetaGamma1 figBetaGamma2...
            meanNeiIndexDynamicF  betastd gammastd betastd2 betastdNew betastdNew2 gammastdNew peakTimemean peakTimestd peakValuemean peakValuestd...
            betameanmean(z) gammameanmean betameanmean2(z) betameanmeanNew(z) betameanmeanNew2 gammameanmeanNew(z) Smean Imean Rmean Bmean NSmean(z,:) Smedian Imedian Rmedian S I R NS SmeanN1 ImeanN1 RmeanN1 BmeanN1 Snewendmean Snewendstd Rnewendmean(z) Rnewendstd(z) Rnew_nonendmean Rnew_nonendstd propSimsNonEp(z) propSimsNonEp_theoretical(z)...
            numRunsMinor(z) measuredProbMinor(z) numRunsMajor(z) measuredProbMajor(z) meanmdist meannumInfectedVM...
            ] = monteCarloSIRN2(numRuns,numNodes, betan, variantRichness, numTimesteps, plotGraphs,gamma,throwAway,SEpidemic,numExploits,modePar, numLoci, casetype);
FinalSize(z) = max(Imean,[],2); %mean final size SI

    %time
     maska = Imean >= FinalSize(z)-1; %when within 1 of the final value
    [C Index] = max(maska);
    MLa(z) = Index; %T1
%close(fighistSpRun); close(fighistRecovered); close(fighistSusceptible); close(fighistpeakValue); close(fighistpeakTime);
%close(fighistBeta); close(fighistGamma); close(fighistBetanew); close(fighistGammanew); 
%close(figInfectedRun1);%close(figBetaRun1); %close(figGammaRun1); %close(figRoRun1); %close(figBetaGamma1); %close(figBetaGamma2); 
%close all;
    text = ['done ' num2str(z) ' out of ' num2str(VRmax)];
    disp(text)
end    
%save outputs
FinalSizeDynamic = FinalSize(z);
TimeDynamic=MLa(z);
ImeanDynamic=Imean(1:dec:end);
RmeanDynamic=Rmean(1:dec:end);
SmeanDynamic=Smean(1:dec:end);
BmeanDynamic=Bmean(1:dec:end);
disp('done dynamic')


figure
plot(meanNeiIndexDynamicF,'k')
xlabel('Time (seconds)')
ylabel('Variant Diversity')

figure
plot(FinalSize,'ko-','linewidth',2)
hold on
title('final size')

%*****************************************************
toc
end %30



end

