%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Jennifer Jackson 2017                                                        %
% A Biodiversity Approach to Cyber Security                                    %
% PhD Complexity Science                                                       %
% University of Warwick                                                        %
% http://wrap.warwick.ac.uk/98458                                              %
% This software is distributed under the  Creative Commons Attribution License %
% https://creativecommons.org/licenses/by/4.0/                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [sp numSusceptibles numInfected numRecovered numNotSusceptibles numBlacklisting neiIndex mdist numInfectedVM] = randomGenotypesSIRN(numNodes, betan, variantRichness, numTimesteps, plotGraphs, gamma, numExploits, modePar,numLoci,casetype)
%NOTES: when blacklisting one variant VR needs to be at least 3, since FS
%algorithm sets FS score of blacklist to zero but if there is only one left
%to choose from will set this to zero also because considers it used to
%much already and then assigns the blacklisted one as a default measure 
%(as its the first in the list) as it doesn't know which one to pick.
%plotGraphs = 1
numNodeCapture=5
%MULTIPLE LOCI
%sp is the initial number of nodes susceptible to given exploit
%case type - malware type AND / OR
%mode - can be 'static' or 'dynamic' 'Dynamic-random' 'Dynamic-favScore'
%Dynamic model - random genotypes model
%each node selects a random genotype at every time step
%numNodes -number of susceptible nodes
%for a single locus variantRichness is the number of genotypes
%for multiple loci variantRichness is the number of variants at each locus
%which is the same
%***********************************
%choose initial genotype value, when in static mode , genotypes remain at these values
%when chosen randomly from a uniform distribution assume maximally diverse(ON AVERAGE)
%subject to the minimum number of devices constraint
%for a single locus variantRichness is the number of genotypes
%for multiple loci variantRichness is the number of variants at each locus
%which is the same
gammarep = repmat(gamma,1,numNodes); %recovery / patching rate
blklistrep = repmat(modePar.blklistrate,1,numNodes); %rate at which blacklisting information is disseminated
blklistrepPermanent = zeros(1,numNodes); %which nodes are currently blacklisting variants
vmupdate = zeros(1,numNodes); %which nodes are currently reseting vms and binning infection
vmItrack = zeros(1,numNodes); %which nodes have ever been infected to track resistance for vm updates
mode=modePar.modeStr %mode of diversity %mode - can be 'static' or 'dynamic' 'Dynamic-random' 'Dynamic-favScore'
numToConstrain = modePar.numToConstrain; %fix(numNodes/2) %number of nodes to constrain to a single variant and not allow to update
numNotConstrain = numNodes-numToConstrain;
stopUpdating = modePar.stopUpdatingTime; %2000000 %stop updating algorithm after 30 timesteps
neiIndex = zeros(1,numTimesteps); %measured diversity index over time
%*********************************
%Malware model set up
%SIR state set up
stateNode = ones(1,numNodes).*1;%SIRN states 1, 2,3, start in Susceptibles
%stateNode(1,1)=2;%1 node infected
%malware set up
exploitGenotype=ones(numLoci,numNodes).*numExploits;
%******************************
%Network models set up
%******************************
%Random encounters
if strcmp(modePar.netStr,'re')
    betasrep = repmat(betan,1,numNodes); %contact rate
end
%******************************
%Random waypoint
if strcmp(modePar.netStr,'rwp')
rwph = rwpHandle; % Create the reference class
% Setup simulation parameters
% -- Base parameters --
% du = distance units
% tu = time units
% -- Derived parameters --
% su = du / tu units
rwph.nodes    = numNodes;     % Number of nodes (absolute)
rwph.xmin     = modePar.xmin;     % Minimum x-axis location (du)
rwph.xmax     = modePar.xmax;    % Maximum x-axis location (du)
rwph.ymin     = modePar.ymin;     % Minimum y-axis location (du)
rwph.ymax     = modePar.ymax;    % Maximum y-axis location (du)
rwph.smin     = modePar.smin;  % Minimum speed (du / tu)
rwph.smax     = modePar.smax;  % Maximum speed (du / tu)
rwph.pmin     = modePar.pmin;     % Minimum time for pause (integer tu)
rwph.pmax     = modePar.pmax;     % Maximum time for pause (integer tu)
rwph.dinr     = modePar.dinr;     % Radial distance for in-range (du)
rwph.dexceed  = modePar.dexceed;  % Value larger than any possible distance (du)
rwph.tinrg    = modePar.tinrg ;     % Time in-range for genotype transfer (tu)
rwph.tinrv    = modePar.tinrv ;     % Time in-range for virus transfer (tu)
%new rwp box constraints
rwph.nodesln  = modePar.nodesln;     % Number of limited nodes (absolute) - always first
rwph.xminln   = modePar.xminln;     % Minimum x-axis location limited nodes (du)
rwph.xmaxln   = modePar.xmaxln;    % Maximum x-axis location limited nodes (du)
rwph.yminln   = modePar.yminln;     % Minimum y-axis location limited nodes (du)
rwph.ymaxln   = modePar.ymaxln;    % Maximum y-axis location limited nodes (du)
% Mode
rwph.mode = modePar.moderwp;      % Random in range mode, Nearest in range mode, (Random) Available in range mode
% Log vector
rwph.logena = modePar.logena;
% Log directory
rwph.logDir = sprintf('%s\\log', pwd);
% Initialise the rwp simulation
fprintf('Initialisingrwp\n');
rwpInit(rwph);
%graphs
%precision 1(nearest metre), 100(nearest cm), 1000(nearest mm)
if plotGraphs == 1; 
precision = 1
LocalDivz = zeros(rwph.xmax*precision,rwph.ymax*precision); %set granularity of plot
LocalDivzH = figure % local diversity plot
%mov(1:numTimesteps)= struct('cdata',[],'colormap',[]); %added Sept 2017 to capture animation
LocalStateH = figure %local state plot
genotypeH=figure %local variants
virusDistanceH=figure %distance spread of virus from origin
end
%******************************************
%If running in geographical constraints mode
%set constraints
%set initial infected to bottom left
t=0;

if and(strcmp(modePar.areaConstrainrwp,'yes'),strcmp(modePar.statnodesrwp,'stationary'))
    %if stationary constraints start straight away
    [ nearestNodeToZero nodeMaskConstrainrwp nodeMaskConstrainrwp_not] = rwpConstrain(t, rwph, modePar.xboxc, modePar.yboxc, modePar.rwpcontrainplotGraph);
    %stateNode(1,nearestNodeToZero)=2;%1 node infected
     nodeMaskConstrainrwp_notrep = repmat(nodeMaskConstrainrwp_not,[numLoci,1]);
     nodeMaskConstrainrwprep = repmat(nodeMaskConstrainrwp,[numLoci,1]);
elseif and(strcmp(modePar.areaConstrainrwp,'yes'),strcmp(modePar.statnodesrwp,'dynamic'))
    %if dynamic constraints start when virus starts later
    nodeMaskConstrainrwp = zeros(1,numNodes); %dont constrain anything
    nodeMaskConstrainrwp_not=ones(1,numNodes);
    nodeMaskConstrainrwp_notrep = repmat(nodeMaskConstrainrwp_not,[numLoci,1]);
    nodeMaskConstrainrwprep = repmat(nodeMaskConstrainrwp,[numLoci,1]);
else
   % stateNode(1,1)=2;%1 node infected
end
    %constrain a nodes rwp
    %if strcmp(modePar.areaConstrainrwp,'yes'); genotypeNode(:,nodeMaskConstrainrwp_not) = randi(variantRichness,[numLoci,sum(nodeMaskConstrainrwp_not)]); end; %constrain
%*******************************************
end %rwp
%initialise genotypes

genotypeNode = ones(numLoci,numNodes); %genotypes - all set the same initially
    %constrain a specified number areaConstrainrwp must be set to 'no' - no geographical area info
    if and(strcmp(modePar.genoInit,'maxdiv'),strcmp(modePar.areaConstrainrwp,'no'))  %initial conditions of genotypes maxdiv (maximum diversity - random variant chosen) mindiv (minimum diversity - all nodes same)
        genotypeNode(:,1:numNotConstrain) = randi(variantRichness,[numLoci,numNotConstrain]); %number not to constrain
    
        if and(plotGraphs == 1,strcmp(modePar.netStr,'rwp')) 
            figure
            hold on
            numNotConstrain
            plot(rwph.cx(numNotConstrain+1:end), rwph.cy(numNotConstrain+1:end), 'b+') %constrained
            plot(rwph.cx(1:numNotConstrain), rwph.cy(1:numNotConstrain), 'k+') %not constrained
             plot(rwph.cx(1), rwph.cy(1), 'r+') %random
            legend('Constrained','Not constrained','Malware source (random)')
            axis([rwph.xmin rwph.xmax rwph.ymin rwph.ymax]);
            title('constraints assigned randomly')
            xlabel('Distance (metres)')
            ylabel('Distance (metres)')
        end
    
    
    
    end
    %constrain nodes rwp to area, genoInit must be set to 'maxdiv' for
    %initialising unconstrained to random value, otherwise constraints are
    %just applied each time step
    if and(and(strcmp(modePar.areaConstrainrwp,'yes'),strcmp(modePar.netStr,'rwp')),strcmp(modePar.genoInit,'maxdiv'))
        genotypeNode(:,:) = (genotypeNode.*nodeMaskConstrainrwprep) + (randi(variantRichness,[numLoci,numNodes]).*nodeMaskConstrainrwp_notrep); 
    end;

%min Network size
minNetworkSize = variantRichness.^numLoci
%*****************************
%Diversity Algorithms set up
%****************************
%Static diversity
if strcmp(mode,'static')
    updateStopFlag = zeros(numNodes,1); %NOT USED
    %extra diversity algorithm parameters required for rwp network model
    if strcmp(modePar.netStr,'rwp')
        bufferCount = ones(1,numNodes); %length of genotype buffer
        bufferLength=modePar.bufferlength; %buffer for buffering genotypes encountered, genotypes can start updating when buffer reached
        bufferG = zeros(numNodes,bufferLength,numLoci);
        genotypeTimeOutPeriod = modePar.genotypeTimeOutPeriod; %time out before getting genotype from same node
        bufferGnode = zeros(numNodes,bufferLength); %genotypebuffer - node genotype from
        bufferGTimeOutEnd = zeros(numNodes,bufferLength); %end time of genotype time out period
    end
end
%dynamic - RV-T
if strcmp(mode,'dynamic')
updateStopFlag = zeros(numNodes,1); %NOT USED
genoUpdateCount = ones(1,numNodes); %only used by vm update
numEncountersBetweenUpdates = modePar.numEncountersBetweenUpdates; %only used by vm update
%extra diversity algorithm parameters required for rwp network model
if strcmp(modePar.netStr,'rwp')
    bufferCount = ones(1,numNodes); %length of genotype buffer
    bufferLength=modePar.bufferlength; %buffer for buffering genotypes encountered, genotypes can start updating when buffer reached
    bufferG = zeros(numNodes,bufferLength,numLoci);
    genotypeTimeOutPeriod = modePar.genotypeTimeOutPeriod; %time out before getting genotype from same node
    bufferGnode = zeros(numNodes,bufferLength); %genotypebuffer - node genotype from
    bufferGTimeOutEnd = zeros(numNodes,bufferLength); %end time of genotype time out period
end
end
%*******************************
%Random update diversity algorithm - update after a specified number of encounters
if strcmp(mode,'Dynamic-random') 
updateStopFlag = zeros(numNodes,1); %latched Flag to stop updating - once set stays there unless reset
numEncountersBetweenUpdates = modePar.numEncountersBetweenUpdates; %number of encounters between updates 
genoUpdateCount = ones(1,numNodes); %number of encounters before a genotype update
%extra diversity algorithm parameters required for rwp network model
if strcmp(modePar.netStr,'rwp')
    bufferCount = ones(1,numNodes); %length of genotype buffer
    bufferLength=modePar.bufferlength; %buffer for buffering genotypes encountered, genotypes can start updating when buffer reached
    bufferG = zeros(numNodes,bufferLength,numLoci);
    genotypeTimeOutPeriod = modePar.genotypeTimeOutPeriod; %time out before getting genotype from same node
    bufferGnode = zeros(numNodes,bufferLength); %genotypebuffer - node genotype from
    bufferGTimeOutEnd = zeros(numNodes,bufferLength); %end time of genotype time out period
end
end
%*******************************
%Dynamic diversity algorithm - update after a specified number of encounters
%Fav score alg init
if strcmp(mode,'Dynamic-favScore')
bufferLength=modePar.bufferlength; %buffer for buffering genotypes encountered, genotypes can start updating when buffer reached
bufferLengthNei=modePar.bufferlengthNei; %buffer for buffering local nei diversity,
stdNeiUpdateThreshold = modePar.stdNeiUpdateThreshold;%threshold for stopping the update process
stdNei = zeros(numNodes,1);
numEncountersBetweenUpdates = modePar.numEncountersBetweenUpdates; %number of encounters between updates can be same or different to buffer length
bufferCount = ones(1,numNodes); %length of genotype buffer
neiLocal = zeros(1,numNodes); %current local nei value
bufferNeiCountLocal = ones(1,numNodes); %length of local nei buffer
bufferNeiLocal = zeros(numNodes,bufferLength); %local nei buffer
NeiBufferFull = zeros(numNodes,1); %latched Flag to say Nei buffer has been filled after initial start up
updateStopFlag = zeros(numNodes,1); %latched Flag to stop updating - once set stays there unless reset
genoUpdateCount = ones(1,numNodes); %number of encounters before a genotype update
bufferG = zeros(numNodes,bufferLength,numLoci);
%extra diversity algorithm parameters required for rwp network model
if strcmp(modePar.netStr,'rwp')
    genotypeTimeOutPeriod = modePar.genotypeTimeOutPeriod; %time out before getting genotype from same node
    bufferGnode = zeros(numNodes,bufferLength); %genotypebuffer - node genotype from
    bufferGTimeOutEnd = zeros(numNodes,bufferLength); %end time of genotype time out period
end
end
%********************************
%***************************
%SETUP Malware
%***************************
%Genetic matching to exploits
%1 locus (also needed for multipe loci as does each locus first)
exploitMatch = (genotypeNode <= exploitGenotype);%own genotype matches to exploit (equal to or less than number of exploits)
%check for two loci, AND/OR case
if and(numLoci == 2, strcmp(casetype, 'AND')); exploitMatch = and(exploitMatch(1,:), exploitMatch(2,:)); end;
if and(numLoci == 2, strcmp(casetype, 'OR')); exploitMatch = or(exploitMatch(1,:), exploitMatch(2,:)); end;
%check for three loci, AND/OR case
if and(numLoci == 3, strcmp(casetype, 'AND')); exploitMatch = and((and(exploitMatch(1,:),exploitMatch(2,:))),exploitMatch(3,:)); end;
if and(numLoci == 3, strcmp(casetype, 'OR')); exploitMatch = or((or(exploitMatch(1,:), exploitMatch(2,:))),exploitMatch(3,:)); end;
%check for four loci, AND/OR case
if and(numLoci == 4, strcmp(casetype, 'AND')); exploitMatch = and((and((and(exploitMatch(1,:),exploitMatch(2,:))),exploitMatch(3,:))),exploitMatch(4,:)); end;
if and(numLoci == 4, strcmp(casetype, 'OR')); exploitMatch = or((or((or(exploitMatch(1,:), exploitMatch(2,:))),exploitMatch(3,:))),exploitMatch(4,:)); end;
%********************************************
%initial nodes 'truly' susceptible for a given run
sp=sum(exploitMatch);
%********************************************
%********************************************
%DO CODE EVERY TIMESTEP
for t = 1: numTimesteps
%*************************************
%virus start time
if t == modePar.virusStartTime
    if strcmp(modePar.netStr,'rwp')
    if strcmp(modePar.areaConstrainrwp,'yes')
        %dont need this now as constraints set at start, but need to know
        %nearest node at origin
        [ nearestNodeToZero nodeMaskConstrainrwp nodeMaskConstrainrwp_not] = rwpConstrain(t, rwph, modePar.xboxc, modePar.yboxc, 1);
        %nodeMaskConstrainrwp_notrep = repmat(nodeMaskConstrainrwp_not,[numLoci,1]);
        %nodeMaskConstrainrwprep = repmat(nodeMaskConstrainrwp,[numLoci,1]);
        %genotypeNode(:,:) = (1.*nodeMaskConstrainrwprep) + genotypeNode.*nodeMaskConstrainrwp_notrep; %set to variant 1
        stateNode(1,nearestNodeToZero)=2;%1 node infected
        vmItrack(1,nearestNodeToZero)=1; %track infected in vm mode
    else
        [ nearestNodeToZero nodeMaskConstrainrwp nodeMaskConstrainrwp_not] = rwpConstrain(t, rwph, modePar.xboxc, modePar.yboxc, 0);
        stateNode(1,nearestNodeToZero)=2;%1 node infected
        vmItrack(1,nearestNodeToZero)=1;%track infected in vm mode
           %if and(plotGraphs == 1,strcmp(modePar.netStr,'rwp')) 
            figure
            hold on
            plot(rwph.cx(numNotConstrain+1:end), rwph.cy(numNotConstrain+1:end), 'b+') %constrained
            plot(rwph.cx(1:numNotConstrain), rwph.cy(1:numNotConstrain), 'k+') %not constrained
             plot(rwph.cx(nearestNodeToZero), rwph.cy(nearestNodeToZero), 'r+') %random
            legend('Constrained','Not constrained','Malware source (nearest origin)')
            axis([rwph.xmin rwph.xmax rwph.ymin rwph.ymax]);
            title('constraints assigned randomly')
            xlabel('Distance (metres)')
            ylabel('Distance (metres)')
        %end
        
    end
    else
        stateNode(1,1)=2;%1 node infected
        vmItrack(1,1)=1;%track infected in vm mode
    end %rwp
end
%********************************************
%Performance measurements of the whole network
%Get dynamic diversity information
numAlleles = repmat(variantRichness,[1,numLoci]);
genotypeMatrix = zeros(numNodes,(numLoci*variantRichness));%binary form of genotype matrix
%convert to binary matrix
for k = 1:numNodes
for j = 1:numLoci
start = (j*variantRichness)-variantRichness;
genotypeMatrix(k,(start+genotypeNode(j,k)))=1;
end
end
%get measured variant diversity of whole network
[ neiIndex(t) neiIndexSingleLocus ] = getNeiIndex( genotypeMatrix, numAlleles ); 
%get blacklisting information
numBlacklisting(t) = sum(blklistrepPermanent);
%get updating information
numUpdateStopFlag(t) = sum(updateStopFlag);
%Get current state information
Notsusceptibles = (stateNode == 4);
numNotSusceptibles(t) = sum(Notsusceptibles);
susceptibles = (stateNode == 1);
numSusceptibles(t) = sum(susceptibles);
infected = (stateNode ==2);
numInfected(t) = sum(infected);
recovered = (stateNode ==3);
numRecovered(t) = sum(recovered);
%measure and graph distance spread of virus
if and(plotGraphs == 1,strcmp(modePar.netStr,'rwp')); graphMalwareDist(t,virusDistanceH,infected,rwph,precision,numTimesteps); end;
if strcmp(modePar.netStr,'rwp')
    distance = sqrt(rwph.cx.^2 + rwph.cy.^2);
    mdist = max(distance.*infected);
else
    mdist = 0;
end
%****************************************************
%Network model - random encounter
%*********************************************
if strcmp(modePar.netStr,'re')
    randomEncounter = randi(numNodes,[1,numNodes]); %random encounter with another node
    %code added - must not encounter itself
        for k = 1:numNodes
                while randomEncounter(k) == k
                 randomEncounter(k) = randi(numNodes,1);
                end;
        end;
    %contact/infection rate
    randomNumberBeta = rand(1,numNodes); %choose a random number for probability comparison to beta    
    encounterYes = randomNumberBeta < betasrep;%only valid encounter with certain probability RANDOM ENCOUNTER NETWORK MODEL ONLY
    %update to bring in-line with rwp model with two independent outputs
    txexploitYes=encounterYes;
    txgenotypeYes=encounterYes;
    if plotGraphs == 1; encounterYesGraph(t,1:numNodeCapture)= txgenotypeYes(1:numNodeCapture); end
end
%****************************************************
%Network model - random waypoint
%*********************************************
if strcmp(modePar.netStr,'rwp')
    %****************************
    %1. iterate  rwp model to get x y positions and encounter info
    %****************************
    rwpIterate(rwph); %if rwp mode iterate random waypoint points
    %****************************
    %2. buffer genotypes encountered 
    %****************************
    %************************************
    %random waypoint mode - FS and RV-E algorithm
    %only record genotypes in buffer if valid genotype
    %encounter and not seen that node recently
    txgenotypeYes_temp = rwph.mlg>=1; %is there a genotype encounter
    txgenotypeNode = rwph.mlg; %node number of encountered with genotype data (array 1 x numNodes)   
    for n = 1:numNodes
        if txgenotypeYes_temp(n)==1 %if there is actually a genotype encounter
            %check for genotype captured from node already
            indexExist = find(bufferGnode(n,:)==txgenotypeNode(n)); %index of node in buffer
            %if node not listed or t is equal to or greater than the end time, and there is actually a genotype encounter then
            %save genotype data
            notListed = isempty(indexExist);%node not listed in buffer
            listed = not(notListed);%node is listed in buffer
            if listed 
                exceedGenotypeTimeOut = t>=bufferGTimeOutEnd(n,indexExist) ;
            else
                exceedGenotypeTimeOut = 0;
            end; % if listed check if greater than end time otherwise get empty matrix
            %may be listed more than once so need to exceed all times
            exceedGenotypeTimeOut = sum(exceedGenotypeTimeOut)==size(exceedGenotypeTimeOut,2);
            txgenotypeYes(n) = or(notListed,exceedGenotypeTimeOut); %if not listed or exceeds timeout period, then save new genotype data
            %set buffer if valid genotype encounter
            if txgenotypeYes(n)==1
                genotypeOfEncounter(:,n)=genotypeNode(:,txgenotypeNode(n)); %genotype of encounter    
                bufferG(n,bufferCount(n),:) = genotypeOfEncounter(:,n); %buffer of genotypes encountered
                bufferGnode(n,bufferCount(n))= txgenotypeNode(n); %buffer of nodes genotype came from
                bufferGTimeOutEnd(n,bufferCount(n))=t+genotypeTimeOutPeriod; %buffer of time out end times
            end %if
        else
         txgenotypeYes(n)=0;   %don't save genotype data
        end %if
    end %for n
    %txgenotypeYes
    %bufferG
    %bufferGnode
    %bufferGTimeOutEnd
    %bufferG counter
     bufferCount = bufferCount+txgenotypeYes; %add 1 to buffer when valid encounter
    reachedBufferCount = bufferCount==bufferLength+1; %reached bufferCount and update genotype
    reset = not(reachedBufferCount);
    bufferCount = (bufferCount.*reset)+ reachedBufferCount; %reset those reached max count, but reset to 1, hence + reachedBufferCount
    %****************************************
    %graphs in-line
     %****************************************
    %%surf plot of local diversity
    %   %neiLocal = ones(1,numNodes).*randi(100,[1,numNodes])./100; %current local nei diversity value set as random numbers at mo for testing
       if strcmp(mode,'Dynamic-favScore')
    %       neiLocal
           
           if plotGraphs == 1; 
               %set(LocalDivzH,'nextplot','replacechildren') %added Sept 2017 to get animation
               graphLocalDiversity(LocalDivzH,neiLocal,numNodes,rwph,precision); 
               figure(LocalDivzH)
               %mov(t)=getframe(gcf); %added Sept 2017 to get animation
           end
       end
    %****************************************
    %plot of node states
    if plotGraphs == 1; graphNodeStates(LocalStateH,stateNode,numNodes,rwph,precision,exploitMatch); end
    %plot of node variants
    lociNum=1;
    if plotGraphs == 1; graphLocalVariant(genotypeH,genotypeNode,lociNum, variantRichness, numNodes,rwph,precision); end
end %rwp
%********************************************
%Recovery rate
%random number parameters for rates
randomNumberGamma = rand(1,numNodes); %choose a random number for probability comparison to gamma (recovery)
gammaYes = randomNumberGamma < gammarep;%only valid gamma with certain probability (move to recovered state)
if plotGraphs == 1; gammaYesGraph(t,1:numNodeCapture)= gammaYes(1:numNodeCapture); end
%********************************************
%Blacklisting rate
randomNumberBlklist = rand(1,numNodes); %choose a random number for probability comparison to blacklist rate
if t >=modePar.blklistStartTime %only start blacklisting mode at predefined time
blklistYes = randomNumberBlklist < blklistrep; %only valid to start blacklisting with certain probability
else
blklistYes = zeros(1,numNodes); %otherwise dont blacklist yet    
end
if plotGraphs == 1; blklistYesGraph(t,1:numNodeCapture) = blklistYes(1,1:numNodeCapture); end
if strcmp(modePar.blklist,'yes') %only enable if blacklisting mode on
    if t >=modePar.blklistStartTime %only start blacklisting mode at predefined time
    blklistrepPermanent = (blklistrepPermanent + blklistYes) >=1; %as soon as valid to start blacklisting it becomes permanent at every valid update until reset by recovery
    %if recovered, blacklisting can be released
    blklistrepPermanent = (1-(stateNode == 3)).*blklistrepPermanent;
     if plotGraphs == 1; blklistrepPermanentGraph(t,1:numNodeCapture)= blklistrepPermanent(1:numNodeCapture); end
    end
end
%*******************************************
%Genetic matching to exploits
%1 locus (also needed for multipe loci as does each locus first)
exploitMatch = (genotypeNode <= exploitGenotype);%own genotype matches to exploit (equal to or less than number of exploits)
%check for two loci, AND/OR case
if and(numLoci == 2, strcmp(casetype, 'AND')); exploitMatch = and(exploitMatch(1,:), exploitMatch(2,:)); end;
if and(numLoci == 2, strcmp(casetype, 'OR')); exploitMatch = or(exploitMatch(1,:), exploitMatch(2,:)); end;
%check for three loci, AND/OR case
if and(numLoci == 3, strcmp(casetype, 'AND')); exploitMatch = and((and(exploitMatch(1,:),exploitMatch(2,:))),exploitMatch(3,:)); end;
if and(numLoci == 3, strcmp(casetype, 'OR')); exploitMatch = or((or(exploitMatch(1,:), exploitMatch(2,:))),exploitMatch(3,:)); end;
%check for four loci, AND/OR case
if and(numLoci == 4, strcmp(casetype, 'AND')); exploitMatch = and((and((and(exploitMatch(1,:),exploitMatch(2,:))),exploitMatch(3,:))),exploitMatch(4,:)); end;
if and(numLoci == 4, strcmp(casetype, 'OR')); exploitMatch = or((or((or(exploitMatch(1,:), exploitMatch(2,:))),exploitMatch(3,:))),exploitMatch(4,:)); end;
%********************************************
%check if successfully encountered a node and it is infected
%Random encounter network model
if strcmp(modePar.netStr,'re')
    EncounterInfectedAndEncountered = checkIfEncNodeInfected(numNodes,stateNode,randomEncounter,txexploitYes);
end
%Random waypoint network model
if strcmp(modePar.netStr,'rwp')
    txexploitYes = rwph.mlv>=1; %is there an exploit encounter pulse flag
    txexploitNode = rwph.mlv; %node number of encountered with exploit data (array 1 x numNodes)
    EncounterInfectedAndEncountered = checkIfEncNodeInfected(numNodes,stateNode,txexploitNode,txexploitYes);
end
%********************************************
%SIR state machine
%if susceptible AND genotype matches to exploit AND the encountered node is infected, then
%self becomes infected
ownInfected = and(susceptibles,and(exploitMatch,EncounterInfectedAndEncountered));
ownNotInfected = not(ownInfected);
gotoInfectedState = ownInfected.*2; %goto infected state
%stayInSameStateS = ownNotInfected.*stateNode %stay in same state
vmItrack=or((gotoInfectedState==2),vmItrack);

%if infected AND gamma valid then self becomes recovered
ownRecovered = and(infected,gammaYes);
ownNotRecovered = not(ownRecovered);
gotoRecoveredState = ownRecovered.*3; %goto infected state
%stayInSameStateI = ownNotRecovered.*stateNode %stay in same state

%if infected and vmupdate valid change back to susceptible, unless already
%due to recover
if strcmp(modePar.vm,'yes')
    vmSusceptible = and(and(infected,vmupdate),not(gammaYes));
    vmNotSusceptible = not(vmSusceptible);
    gotoSusceptibleState = vmSusceptible.*1; %goto susceptible state
    stayInSameState = and(and(ownNotInfected,ownNotRecovered),vmNotSusceptible).*stateNode; %stay in same state
    stateNode = gotoSusceptibleState + gotoInfectedState + gotoRecoveredState + stayInSameState;
else
    %ignore vm update mode
    stayInSameState = and(ownNotInfected,ownNotRecovered).*stateNode; %stay in same state
    stateNode = gotoInfectedState + gotoRecoveredState + stayInSameState;
end

%for plotting function capture node state info at each timestep t

if plotGraphs == 1; stateNodeGraph(t,1:numNodeCapture) = stateNode(1,1:numNodeCapture); end

%end SIR
%*************************************************************************

%**************************************************
%DYNAMIC DIVERSITY ALGORITHMS TO UPDATE GENOTYPES
%***************************************************
%DYNAMIC-RANDOM (TIME)
if strcmp(mode,'dynamic') %only change genotype at every timestep when in dynamic mode
   
%genotypeNode = randi(variantRichness,[numLoci,numNodes]); %choose genotype value
if t < stopUpdating %update within specified timeframe
%genotypeNode(:,1:numNotConstrain) = randi(variantRichness,[numLoci,numNotConstrain]); %number not to constrain
    %rwp constraints applied each timestep
    if and(strcmp(modePar.areaConstrainrwp,'yes'),strcmp(modePar.netStr,'rwp'))
        genotypeNode(:,:) = (genotypeNode.*nodeMaskConstrainrwprep) + (randi(variantRichness,[numLoci,numNodes]).*nodeMaskConstrainrwp_notrep); 
    else
        genotypeNode(:,1:numNotConstrain) = randi(variantRichness,[numLoci,numNotConstrain]); %number not to constrain
    end;
end %stop updating
    %for blacklisting mode, and number of variants available is greater than number to blacklist
    if and(strcmp(modePar.blklist,'yes'),(variantRichness >= modePar.numToBlklist+1))
        %start blacklisting straight away or continue blacklisting at everyfurther update cycle until reset
        blklistNow = or(blklistYes,blklistrepPermanent); %those nodes to blacklist
        blklistNow_not = 1-blklistNow; %those nodes to keep as they are
        if and(strcmp(modePar.areaConstrainrwp,'yes'),strcmp(modePar.netStr,'rwp'))
            genotypeNode_blkList(:,:) = randi([modePar.numToBlklist+1 variantRichness],[numLoci,numNodes]).*nodeMaskConstrainrwp_notrep;    
            genotypeNode(:,:) = (genotypeNode(:,:).*nodeMaskConstrainrwprep) + (genotypeNode(:,:).*blklistNow_not.*nodeMaskConstrainrwp_notrep) + (genotypeNode_blkList(:,:).*blklistNow.*nodeMaskConstrainrwp_notrep);
        else
            genotypeNode_blkList(:,1:numNotConstrain) = randi([modePar.numToBlklist+1 variantRichness],[numLoci,numNotConstrain]);
            genotypeNode(:,1:numNotConstrain) = genotypeNode(:,1:numNotConstrain).*blklistNow_not + genotypeNode_blkList(:,1:numNotConstrain).*blklistNow;
        end
    end
%end %stop updating
end
%*********************************************
%DYNAMIC-RANDOM (ENCOUNTER)
%Random update Dynamic algorithm
if strcmp(mode,'Dynamic-random')
    %****
    %count the number of genotypes encountered, for re mode do not need a
    %buffer since every encounter is used, for the rwp mode not all encounters
    %are used since may have only just seen the same one. Use a buffer so the
    %encounter rate is the same as the FS algorithm for comparison reasons even
    %though the genotypes are never used
    %The txgenotypeYes gets either directly passed from re network model or
    %through buffer for rwp network model
    %encounter counter for updating the genotypes
    %reachedgenoUpdateCount = genoUpdateCount>=numEncountersBetweenUpdates+1; %reached numEncountersBetweenUpdates and update genotype
    %resetg = not(reachedgenoUpdateCount);
    %genoUpdateCount=genoUpdateCount.*resetg+reachedgenoUpdateCount; %reset bufferCount if bufferLength reached
    %genoUpdateCount = genoUpdateCount+txgenotypeYes.*resetg; %increment if valid genotype encounter
    
    genoUpdateCount = genoUpdateCount+txgenotypeYes; %add 1  when valid encounter
    reachedgenoUpdateCount=genoUpdateCount==numEncountersBetweenUpdates+1;
    resetg = not(reachedgenoUpdateCount);
    genoUpdateCount=(genoUpdateCount.*resetg)+reachedgenoUpdateCount;
    %****
%select new genotypes
%random new
newGenotypeNode = randi(variantRichness,[numLoci,numNodes]); %genotypes
newGenotypeNodeValid = repmat(reachedgenoUpdateCount,[numLoci,1]); %when reached fixed no. of genotype encounters, new genotype is valid
oldGenotypeNodeValid = (1-newGenotypeNodeValid);

%update genotype - only update when reached a fixed number of encounters
%and only update those not constrained
if t < stopUpdating
%informs which ones to keep and which ones to update (without blacklisting)
%genotypeNode(:,1:numNotConstrain) = (oldGenotypeNodeValid(:,1:numNotConstrain) .* genotypeNode(:,1:numNotConstrain) + newGenotypeNodeValid(:,1:numNotConstrain) .* newGenotypeNode(:,1:numNotConstrain));
    %rwp constraints applied each timestep
    if and(strcmp(modePar.areaConstrainrwp,'yes'),strcmp(modePar.netStr,'rwp'))
                a(:,:)=(genotypeNode(:,:).*nodeMaskConstrainrwprep);%those to stay as they are
        b(:,:)=(oldGenotypeNodeValid(:,:).*genotypeNode(:,:).*nodeMaskConstrainrwp_notrep);%those stay old
        c(:,:)=(newGenotypeNodeValid(:,:).*newGenotypeNode(:,:).*nodeMaskConstrainrwp_notrep);%those to update
        genotypeNode=a+b+c;
        
        
        %genotypeNode(:,:)=(oldGenotypeNodeValid(:,:).*genotypeNode(:,:).*nodeMaskConstrainrwprep) + (newGenotypeNodeValid(:,:).*newGenotypeNode(:,:).*nodeMaskConstrainrwp_notrep);
    else
        genotypeNode(:,1:numNotConstrain) = (oldGenotypeNodeValid(:,1:numNotConstrain) .* genotypeNode(:,1:numNotConstrain) + newGenotypeNodeValid(:,1:numNotConstrain) .* newGenotypeNode(:,1:numNotConstrain));
    end;
else %changed jtd tues
    updateStopFlag(1:numNodes) = 1;
end
%for blacklisting mode, and number of variants available is greater than number to blacklist
    if and(strcmp(modePar.blklist,'yes'),(variantRichness >= modePar.numToBlklist+1))
        %start blacklisting straight away (regardless of current update) or continue blacklisting at everyfurther update cycle until reset
        %blklistNow = or(blklistYes,(and(blklistrepPermanent,newGenotypeNodeValid))); %those nodes to blacklist
         blklistNow = or(blklistYes,(and(blklistrepPermanent,reachedgenoUpdateCount))); %those nodes to blacklist
         blklistNow_not = 1-blklistNow; %those nodes to keep as they are
         blklistNowrep = repmat(blklistNow,[numLoci,1]);%masking to number of loci
         blklistNow_notrep = repmat(blklistNow_not,[numLoci,1]);%masking to number of loci
         
        updateStopFlag = updateStopFlag - blklistYes'; %JTD restart update straight away if blklist encounter
         if plotGraphs == 1; blklistNowGraph(t,1:numNodeCapture) = blklistNow(1:numNodeCapture); end
        
        if and(strcmp(modePar.areaConstrainrwp,'yes'),strcmp(modePar.netStr,'rwp'))
            genotypeNode_blkList(:,:) = randi([modePar.numToBlklist+1 variantRichness],[numLoci,numNodes]).*nodeMaskConstrainrwp_notrep;                
            genotypeNode(:,:) = (genotypeNode(:,:).*nodeMaskConstrainrwprep)+(genotypeNode(:,:).*blklistNow_notrep.*nodeMaskConstrainrwp_notrep) + (genotypeNode_blkList(:,:).*blklistNowrep.*nodeMaskConstrainrwp_notrep);
        else
            genotypeNode_blkList(:,1:numNotConstrain) = randi([modePar.numToBlklist+1 variantRichness],[numLoci,numNotConstrain]);
            genotypeNode(:,1:numNotConstrain) = genotypeNode(:,1:numNotConstrain).*blklistNow_notrep + genotypeNode_blkList(:,1:numNotConstrain).*blklistNowrep;
        end
    end
%else
 %   updateStopFlag(1:numNodes) = 1;
%end
if plotGraphs == 1; updateStopFlagGraph(1:numNodeCapture,t) = updateStopFlag(1:numNodeCapture); end
end %randomUpdate
%**********************************
%*********************************************
%DYNAMIC FAVSCORE ALGORITHM
%Dynamic diversity algorithm
%Fav Score alg - no constraints of compatability
if strcmp(mode,'Dynamic-favScore')
%****************************
%1. buffer genotypes encountered 
%****************************
%re network model
if strcmp(modePar.netStr,'re') %random encounter network model
    genotypeOfEncounter=genotypeNode(:,randomEncounter);
    for n = 1:numNodes
    bufferG(n,bufferCount(n),:) = genotypeOfEncounter(:,n); %buffer of genotypes encountered
    end
    bufferCount = bufferCount+txgenotypeYes; %add 1 to buffer when valid encounter
    reachedBufferCount = bufferCount==bufferLength+1; %reached bufferCount and update genotype
    reset = not(reachedBufferCount);
    bufferCount = (bufferCount.*reset)+ reachedBufferCount; %reset those reached max count, but reset to 1, hence + reachedBufferCount
end
%rwp network model
%generated in rwp code above as same as RV-E alg
%*****************************
%2. calculate diversity metric
%****************************
%get freq for all var
for varNum = 1:variantRichness
    var = bufferG==varNum; %identify a variant out of the buffer
    sumS = sum(var,2); %sum across samples for each node at each locus - number of times variant is present
    freq(:,varNum,:) = sumS./bufferLength;
end

metric_tmp=1-freq; %(numNodes,numVariants,numLoci) give variants used a lot a small number
%normalisation of metric
S = sum(metric_tmp,2); %sum values at each node
Srep=repmat(S,[1,variantRichness]);
metric = metric_tmp./Srep; %divide values by sum
%******************************
%3. update mode
%****************************
%Calculate local nei diversity 
%each locus 1 - sum(freq)
nei_1locusLocal=1-sum(freq.^2,2); %(numNodes,numVariants,numLoci)
check = nei_1locusLocal<0;
if sum(check)>1; disp('freq sum error'); end
%across all loci sum(each locus)/numLoci
neiLocal = sum(nei_1locusLocal,3)./numLoci;
%buffer nei values
for n = 1:numNodes
bufferNeiLocal(n,bufferNeiCountLocal(n),:) = neiLocal(n); %buffer of local nei measurement
end
reachedBufferNeiCountLocal = bufferNeiCountLocal>=bufferLengthNei+1; %reached bufferCount 
NeiBufferFull = (NeiBufferFull + reachedBufferNeiCountLocal')>=1; %buffer filled up after initial start up
resetNeiLocal = not(reachedBufferNeiCountLocal);
bufferNeiCountLocal=bufferNeiCountLocal.*resetNeiLocal+reachedBufferNeiCountLocal; %reset bufferCount if bufferLength reached
bufferNeiCountLocal = bufferNeiCountLocal+txgenotypeYes.*resetNeiLocal; %increment if wish to save one above in current memory slot
%surf plot of local diversity
       %neiLocal = ones(1,numNodes).*randi(100,[1,numNodes])./100; %current local nei diversity value set as random numbers at mo for testing
       if strcmp(modePar.netStr,'rwp')
          % graphLocalDiversity(LocalDivzH,neiLocal,numNodes,rwph,precision)
       end

if strcmp(modePar.update,'yes')
%calculate std of nei, if less than threshold and current std greater than preious std(minimum point) stop updating
stdOld = stdNei; %save previous Nei value
stdNei = std(bufferNeiLocal,[],2); %new Nei value
if plotGraphs == 1; stdNeiGraph(:,t) = stdNei; end
if plotGraphs == 1; NeiBufferFullGraph(:,t) = NeiBufferFull; end
updateStopFlag_temp = and(and(stdNei<stdNeiUpdateThreshold,NeiBufferFull),stdNei>stdOld); %pulse flag
%updateStopFlag_temp = and(stdNei<stdNeiUpdateThreshold,NeiBufferFull); %pulse flag
updateStopFlag = (updateStopFlag + updateStopFlag_temp)>=1;%latched flag - once set stays there

%calculate std of nei, if less than threshold and current std greater than preious std(minimum point) stop updating
%need to control updates elsewhere in code, so need a flag for each node.
end%update mode
%***********************************
%4. calculate desirability score
%***********************************
%add desirability constraints
if strcmp(modePar.desirabilities,'yes')
metric = addDesirabilityConstraints(numNodes,numLoci,variantRichness,metric);
end
%***********************************
%5. calculate favourability score
%***********************************
favScore = metric;

%encounter counter for updating the genotypes
%%reachedgenoUpdateCount = genoUpdateCount>=numEncountersBetweenUpdates+1; %reached numEncountersBetweenUpdates and update genotype
%%resetg = not(reachedgenoUpdateCount);
%%genoUpdateCount=genoUpdateCount.*resetg+reachedgenoUpdateCount; %reset bufferCount if bufferLength reached
%%genoUpdateCount = genoUpdateCount+txgenotypeYes.*resetg; %increment if wish to save one above in current memory slot

    genoUpdateCount = genoUpdateCount+txgenotypeYes; %add 1  when valid encounter
    reachedgenoUpdateCount=genoUpdateCount==numEncountersBetweenUpdates+1;
    resetg = not(reachedgenoUpdateCount);
    genoUpdateCount=(genoUpdateCount.*resetg)+reachedgenoUpdateCount;

newGenotypeNodeValid = repmat(reachedgenoUpdateCount,[numLoci,1]); %validate genotypeUpdate
oldGenotypeNodeValid = (1-newGenotypeNodeValid);
%***********************************
 %**************************
 %6. for blacklisting mode
 %**************************
    %for blacklisting mode, and number of variants available is greater than number to blacklist
    if and(strcmp(modePar.blklist,'yes'),(variantRichness >= modePar.numToBlklist+1))
        %start blacklisting straight away (regardless of current update) or continue blacklisting at everyfurther update cycle until reset
        %blklistNow = or(blklistYes,(and(blklistrepPermanent,newGenotypeNodeValid))); %those nodes to blacklist
        blklistNow = or(blklistYes,(and(blklistrepPermanent,reachedgenoUpdateCount))); %those nodes to blacklist
        if plotGraphs == 1; blklistNowGraph(t,1:numNodeCapture) = blklistNow(1:numNodeCapture); end
        blklistNow_not = 1-blklistNow; %those nodes to keep as they are
        %blklistNowrep=repmat(blklistNow',[1,variantRichness])
        blklistNowrepFav=repmat(blklistNow',[1,variantRichness,numLoci]);%masking favScore
        blklistNow_notrepFav=repmat(blklistNow_not',[1,variantRichness,numLoci]); %masking favScore
        blklistNowrep = repmat(blklistNow,[numLoci,1]);%masking rest
        blklistNow_notrep = repmat(blklistNow_not,[numLoci,1]);%masking rest
        
        favScore_blklist = favScore;
        favScore_blklist(:,1:modePar.numToBlklist,:) = 0; %favour blacklisted variants as '0'        
        favScore = favScore.*blklistNow_notrepFav + favScore_blklist.*blklistNowrepFav; %new favscore
         if plotGraphs == 1; favScore_blklistGraph(t,1:numNodeCapture,1) = favScore_blklist(1:numNodeCapture,1); end %variant 1 score
         if plotGraphs == 1; favScore_blklistGraph(t,1:numNodeCapture,2) = favScore_blklist(1:numNodeCapture,2); end %variant 2 score
        if plotGraphs == 1; favScoreGraph(t,1:numNodeCapture,:) = favScore(1:numNodeCapture,:); end %variant 1 score
        %reset updateStopFlag to restart updating to incorporate
        %blacklisting and when recover re-update try to re-maximise diversity level
       % if plotGraphs == 1; blklistYesGraph(t,:)=blklistYes; end ;%plotting graphs
        if strcmp(modePar.update,'yes')
            %disp('reset updateStopFlag')
            resetUpdate = not(or(gammaYes,blklistYes))';
            if plotGraphs == 1; resetUpdateGraph(t,1:numNodeCapture)=resetUpdate(1:numNodeCapture)'; end ;%plotting graphs
            %updateStopFlag = updateStopFlag.*not(blklistYes');
            updateStopFlag = updateStopFlag.*resetUpdate;
        end
    end
    if plotGraphs == 1; updateStopFlagGraph(1:numNodeCapture,t) = updateStopFlag(1:numNodeCapture); end
%*********************************
%7. probablistic choice of genotype
%*********************************
%if filtering, then filter with specific mask
 if strcmp(modePar.filter,'yes')
    chosenVariant = filterFavScore(numNodes,numLoci,variantRichness,favScore);
%otherwise select as normal
 else
    %do for each locus independently with no compatability filtering
    for L = 1:numLoci
     [chosenVal chosenVariant(:,L)] = probSol(favScore(:,:,L)); %use function from big model
    end
 end
newGenotypeNode = chosenVariant';  %variants to choose for each locus for each node
    %work out itself if stop updating - updateStopFlag (0 if update, 1 if stop)
 if strcmp(modePar.update,'yes')
     updateStopFlagrep=repmat(updateStopFlag,[1,numLoci]);
   newGenotypeNode = newGenotypeNode.*not(updateStopFlagrep') +  genotypeNode.*updateStopFlagrep';
 end
%*******************************
%8.Choose to update
%*******************************
%update genotype - only update when reached a fixed number of encounters
%and only update those not constrained
if t < stopUpdating %hard stop updating override  
%genotypeNode(:,1:numNotConstrain) = (oldGenotypeNodeValid(:,1:numNotConstrain) .* genotypeNode(:,1:numNotConstrain) + newGenotypeNodeValid(:,1:numNotConstrain) .* newGenotypeNode(:,1:numNotConstrain));
    %rwp constraints applied each timestep
    if and(strcmp(modePar.areaConstrainrwp,'yes'),strcmp(modePar.netStr,'rwp'))
        a(:,:)=(genotypeNode(:,:).*nodeMaskConstrainrwprep);%those to stay as they are
        b(:,:)=(oldGenotypeNodeValid(:,:).*genotypeNode(:,:).*nodeMaskConstrainrwp_notrep);%those stay old
        c(:,:)=(newGenotypeNodeValid(:,:).*newGenotypeNode(:,:).*nodeMaskConstrainrwp_notrep);%those to update
        genotypeNode=a+b+c;
    else
        genotypeNode(:,1:numNotConstrain) = (oldGenotypeNodeValid(:,1:numNotConstrain) .* genotypeNode(:,1:numNotConstrain) + newGenotypeNodeValid(:,1:numNotConstrain) .* newGenotypeNode(:,1:numNotConstrain));
    end;
%for blacklisting mode, and number of variants available is greater than number to blacklist, 
    %update those blacklisting as well as those reached update count
    if and(strcmp(modePar.blklist,'yes'),(variantRichness >= modePar.numToBlklist+1))  
    %genotypeNode(:,1:numNotConstrain) = genotypeNode(:,1:numNotConstrain).*blklistNow_not + newGenotypeNode(:,1:numNotConstrain).*blklistNow;
          if and(strcmp(modePar.areaConstrainrwp,'yes'),strcmp(modePar.netStr,'rwp'))
              genotypeNode(:,:) = (genotypeNode(:,:).*nodeMaskConstrainrwprep)+(genotypeNode(:,:).*blklistNow_notrep.*nodeMaskConstrainrwp_notrep) + (newGenotypeNode(:,:).*blklistNow.*nodeMaskConstrainrwp_notrep);
          else
              genotypeNode(:,1:numNotConstrain) = genotypeNode(:,1:numNotConstrain).*blklistNow_notrep + newGenotypeNode(:,1:numNotConstrain).*blklistNowrep;
          end
    end
else
    updateStopFlag(1:numNodes) = 1;
end

    if plotGraphs == 1; genotypeNodeGraph(:,:,t)=genotypeNode; end
end %favScoreEnable
%**********************************************
%vmupdate
if strcmp(modePar.vm,'yes')
    if strcmp(mode,'dynamic')
        vmupdate = and((blklistYes)>=1,not(updateStopFlag')) ;
    else
%vmupdate = and((newGenotypeNodeValid + blklistYes)>=1,not(updateStopFlag')) ;
vmupdate = and((reachedgenoUpdateCount + blklistYes)>=1,not(updateStopFlag')) ;
    end
end

if strcmp(modePar.vm,'yes'); if plotGraphs == 1; vmupdateGraph(t,1:numNodeCapture) = vmupdate(1:numNodeCapture); end; end;
if strcmp(modePar.vm,'yes'); if plotGraphs == 1; numvmupdateGraph(t) = sum(vmupdate); end; end;
if strcmp(mode,'Dynamic-favScore'); if plotGraphs == 1; newGenotypeNodeValidGraph(t,1:numNodeCapture) = newGenotypeNodeValid(1:numNodeCapture); end; end;
%blklistNow(t,:)=blklistNow;

end %time end
%track how many get infected at least once when in vm update mode
numInfectedVM = sum(vmItrack);

%***********
if strcmp(modePar.netStr,'rwp')
%clean up memory from rwp
% Cleanup
fprintf('Clean-up\n');
rwpCleanup(rwph);
% Log and validate waypoint extremes
fprintf('Extremes : (%.3f, %.3f) - (%.3f, %.3f)\n', ...
        rwph.xlow, rwph.ylow, rwph.xhigh, rwph.yhigh);
if( rwph.xlow  >= rwph.xmin && rwph.ylow  >= rwph.ymin && ...
    rwph.xhigh <= rwph.xmax && rwph.yhigh <= rwph.ymax)
    fprintf('\tOK - waypoints all within allowed range\n');
else
    fprintf('\tERROR - some waypoints exceed allowed range\n');
end;
% Plot all logged entries by reloading file data ONLY - can limit timestep
% when not set to zero
fprintf('Plotting\n');
%rwpPlot(rwph.logDir, rwph.logena, rwph.xmin, rwph.xmax, rwph.ymin, rwph.ymax, 0, 0);
%rwpPlot(rwph.logDir, [1 2 3 4], rwph.xmin, rwph.xmax, rwph.ymin, rwph.ymax, 0, 0);
%rwpPlot(rwph.logDir, [1 2 3 4], rwph.xmin, rwph.xmax, rwph.ymin, rwph.ymax, 500, 550);
end
%***********
if plotGraphs == 1
    %save animation
%movie2avi(mov, 'E:\Users\Jen\Documents\ComplexityScience\PhD\VIVAPreparation\Presentation\presentation\animations\diversitymovie3.avi', 'compression', 'None'); %added Sept 2017
end

%**************************
%graphs to plot
if plotGraphs == 1
    dispGraphs = 0
    if dispGraphs == 1
%update graphs    
if strcmp(modePar.update,'yes')
    bufferOffset=1.2;
    updateStopOffset=2.4;
    blkListOffset=3.8;
    recoverOffset = 8.8;
%node 1 ***********************
%figure %var 1
%plot(favScore_blklistGraph(:,1,1),'g--') %favscore blacklisted variant1
%figure
%plot(favScore_blklistGraph(:,1,2),'r--') %favscore blacklisted variant2
%figure
%plot(favScoreGraph(:,1,1),'m--') %favscore variant 1 after blacklisting
%figure
%plot(favScoreGraph(:,1,2),'c--') %favscore variant 2 after blacklisting

figure
plot(stdNeiGraph(1,:),'k.-','linewidth',2) %TEST - plot std result of node 1
hold on
plotG(1,:)=genotypeNodeGraph(1,1,:);
plot(plotG,'k','linewidth',2)%display locus 1 variant value - see if stops changing
plot([1 numTimesteps],[stdNeiUpdateThreshold stdNeiUpdateThreshold],'k--')
%plot(blklistNowGraph(:,1),'b') %blacklist now pulse
%plot(favScore_blklistGraph(:,1,1),'g--') %favscore blacklisted variant1
%plot(favScore_blklistGraph(:,1,2),'r--') %favscore blacklisted variant2
%plot(favScoreGraph(:,1,1),'m--') %favscore variant 1 after blacklisting
%plot(favScoreGraph(:,1,2),'c--') %favscore variant 2 after blacklisting
%plot(updateStopFlagGraph(1,:)-updateStopOffset,'g')%indicates when to stop updating
%plot(blklistYesGraph(:,1)-blkListOffset,'m')%blacklist now pulse  
xlabel('Time')
ylabel('Std of local diversity    /    variant choice')
legend('Std Nei','Variant','Stop threshold')
%legend('Std Nei','Variant','Stop threshold','blklist now in this time-step','favscore var1 blk','favscore var2 blk','after blacklisting var 1','after blacklisting var 2','update stop','blacklist encounter')
title('Node 1')

figure
divVal=4
plot((NeiBufferFullGraph(1,:)-bufferOffset)./divVal,'b') %indicates when nei buffer full
hold on
%plot(favScore_blklistGraph(:,1,2),'r--') %favscore blacklisted variant2
plot((updateStopFlagGraph(1,:)-updateStopOffset)./divVal,'g')%indicates when to stop updating
%plot(encounterYesGraph(:,1),'m') %pulse every encounter
if strcmp(modePar.blklist,'yes')
plot((blklistYesGraph(:,1)-blkListOffset)./divVal,'m')%blacklist now pulse   
plot((blklistrepPermanentGraph(:,1)-blkListOffset-1.4)./divVal,'c')%this node to blacklist
end
%plot([1 numTimesteps],[stdNeiUpdateThreshold stdNeiUpdateThreshold],'y--')
plotstateNodeGraph(:,1)=stateNodeGraph(:,1)==3; %when node changed to recovered state plot
plot((gammaYesGraph(:,1)-recoverOffset+2)./divVal,'g')%recover pulse
plot((stateNodeGraph(:,1)-recoverOffset-1.4)./divVal,'g--')%recover
if and(strcmp(modePar.blklist,'yes'),strcmp(modePar.update,'yes')); plot(resetUpdateGraph(:,1),'k'); end; %reset update end
xlabel('Time')
ylabel('Std of local diversity of one node')
legend('Buffer full', 'stop Updating','BlkListYes pulse','Blacklisting','Recover pulse','Recover state')
title('Node 1')

figure
plot(stdNeiGraph(1,:),'k.-','linewidth',2) %TEST - plot std result of node 1
hold on
plotG(1,:)=genotypeNodeGraph(1,1,:);
plot(plotG,'k','linewidth',2)%display locus 1 variant value - see if stops changing
plot([1 numTimesteps],[stdNeiUpdateThreshold stdNeiUpdateThreshold],'k--')
plot((NeiBufferFullGraph(1,:)-bufferOffset)./divVal,'b') %indicates when nei buffer full
hold on
%plot(favScore_blklistGraph(:,1,2),'r--') %favscore blacklisted variant2
plot((updateStopFlagGraph(1,:)-updateStopOffset)./divVal,'g')%indicates when to stop updating
%plot(encounterYesGraph(:,1),'m') %pulse every encounter
if strcmp(modePar.blklist,'yes')
plot((blklistYesGraph(:,1)-blkListOffset)./divVal,'m')%blacklist now pulse   
plot((blklistrepPermanentGraph(:,1)-blkListOffset-1.4)./divVal,'c')%this node to blacklist
end
%plot([1 numTimesteps],[stdNeiUpdateThreshold stdNeiUpdateThreshold],'y--')
plotstateNodeGraph(:,1)=stateNodeGraph(:,1)==3; %when node changed to recovered state plot
plot((gammaYesGraph(:,1)-recoverOffset+2)./divVal,'g')%recover pulse
plot((stateNodeGraph(:,1)-recoverOffset-1.4)./divVal,'g--')%recover
xlabel('Time')
ylabel('Std of local diversity    /    variant choice')
legend('Std Nei','Variant','Stop threshold','Buffer full', 'stop Updating','BlkListYes pulse','Blacklisting','Recover pulse','Recover state')
title('Node 1')

%*******************************
figure
plot(stdNeiGraph(1,:),'k.-') %TEST - plot std result of node 1
hold on
plot(NeiBufferFullGraph(1,:)-bufferOffset,'b') %indicates when nei buffer full
plot(updateStopFlagGraph(1,:)-updateStopOffset,'g')%indicates when to stop updating
plotG(1,:)=genotypeNodeGraph(1,1,:);
plot(plotG,'r')%display locus 1 variant value - see if stops changing
if strcmp(modePar.netStr,'re'); plot(encounterYesGraph(:,1),'m'); end; %pulse every encounter
if strcmp(modePar.blklist,'yes')
plot(blklistrepPermanentGraph(:,1)-blkListOffset,'c')%this node to blacklist
plot(blklistYesGraph(:,1),'y')%blacklist now pulse
end
plot([1 numTimesteps],[stdNeiUpdateThreshold stdNeiUpdateThreshold],'y--')
plotstateNodeGraph(:,1)=stateNodeGraph(:,1)==3; %when node changed to recovered state plot
plot(stateNodeGraph(:,1)-recoverOffset,'g--')%recover
xlabel('Time')
ylabel('Std of local diversity of one node')
legend('Std Nei','Buffer full', 'stop Updating', 'Variant','Encounter pulse','Blacklisting','BlkListYes pulse','Stop Threshold','Recover')
title('Node 1')

figure
plot(updateStopFlagGraph(1,:),'g')%indicates when to stop updating
title('node 1')
figure
if strcmp(modePar.netStr,'re'); plot(encounterYesGraph(:,1),'m'); end; %pulse every encounter
hold on
plotG(1,:)=genotypeNodeGraph(1,1,:); %loci,numNodes,time
plot(plotG,'r')%display locus 1 variant value - see if stops changing
figure
plotstateNodeGraph(:,1)=stateNodeGraph(:,1)==3;
plot(stateNodeGraph(:,1),'g--')%recover
xlabel('Time')
ylabel('Recovery')
legend('Recover')
title('Node 1')
%node 2 ***********************
figure
plot(stdNeiGraph(2,:),'k.-') %TEST - plot std result of node 2
hold on
plot(NeiBufferFullGraph(2,:),'b') %indicates when nei buffer full
plot(updateStopFlagGraph(2,:),'g')%indicates when to stop updating
plotG(1,:)=genotypeNodeGraph(1,2,:); %loci,numNodes,time
plot(plotG,'r')%display locus 1 variant value - see if stops changing
if strcmp(modePar.netStr,'re'); plot(encounterYesGraph(:,2),'m'); end; %pulse every encounter
if strcmp(modePar.blklist,'yes')
plot(blklistrepPermanentGraph(:,2),'c')%this node to blacklist
plot(blklistYesGraph(:,2),'y')%blacklist now pulse
end
plot([1 numTimesteps],[stdNeiUpdateThreshold stdNeiUpdateThreshold],'y--')
plotstateNodeGraph(:,1)=stateNodeGraph(:,2)==3;
plot(stateNodeGraph(:,2),'g--')%recover
xlabel('Time')
ylabel('Std of local diversity of one node')
legend('Std Nei','Buffer full', 'stop Updating', 'Variant','Encounter pulse','Blacklisting','BlkListYes pulse','Stop Threshold','Recover')
title('Node 2')
figure
plot(updateStopFlagGraph(2,:),'g')%indicates when to stop updating
title('node 2')
figure
if strcmp(modePar.netStr,'re'); plot(encounterYesGraph(:,2),'m'); end; %pulse every encounter
hold on
plotG(1,:)=genotypeNodeGraph(1,2,:);
plot(plotG,'r')%display locus 1 variant value - see if stops changing
figure
plotstateNodeGraph(:,1)=stateNodeGraph(:,2)==3;
plot(stateNodeGraph(:,2),'g--')%recover
xlabel('Time')
ylabel('Recovery')
legend('Recover')
title('Node 2')
%*************************
%node 3 ***********************
figure
plot(stdNeiGraph(3,:),'k.-') %TEST - plot std result of node 3
hold on
plot(NeiBufferFullGraph(3,:),'b') %indicates when nei buffer full
plot(updateStopFlagGraph(3,:),'g')%indicates when to stop updating
plotG(1,:)=genotypeNodeGraph(1,3,:); %loci,numNodes,time
plot(plotG,'r')%display locus 1 variant value - see if stops changing
if strcmp(modePar.netStr,'re'); plot(encounterYesGraph(:,3),'m'); end; %pulse every encounter
if strcmp(modePar.blklist,'yes')
plot(blklistrepPermanentGraph(:,3),'c')%this node to blacklist
plot(blklistYesGraph(:,3),'y')%blacklist now pulse
end
plot([1 numTimesteps],[stdNeiUpdateThreshold stdNeiUpdateThreshold],'y--')
plotstateNodeGraph(:,1)=stateNodeGraph(:,3)==3;
plot(stateNodeGraph(:,3),'g--')%recover
xlabel('Time')
ylabel('Std of local diversity of one node')
legend('Std Nei','Buffer full', 'stop Updating', 'Variant','Encounter pulse','Blacklisting','BlkListYes pulse','Stop Threshold','Recover')
title('Node 3')
figure
plot(updateStopFlagGraph(3,:),'g')%indicates when to stop updating
title('node 3')
figure
if strcmp(modePar.netStr,'re'); plot(encounterYesGraph(:,3),'m'); end; %pulse every encounter
hold on
plotG(1,:)=genotypeNodeGraph(1,3,:);
plot(plotG,'r')%display locus 1 variant value - see if stops changing
figure
plotstateNodeGraph(:,1)=stateNodeGraph(:,3)==3;
plot(stateNodeGraph(:,3),'g--')%recover
xlabel('Time')
ylabel('Recovery')
legend('Recover')
title('Node 3')
%*************************
%node 4 ***********************
nodeNum=4
figure
plot(stdNeiGraph(nodeNum,:),'k.-') %TEST - plot std result of node 4
hold on
plot(NeiBufferFullGraph(nodeNum,:),'b') %indicates when nei buffer full
plot(updateStopFlagGraph(nodeNum,:),'g')%indicates when to stop updating
plotG(1,:)=genotypeNodeGraph(1,nodeNum,:); %loci,numNodes,time
plot(plotG,'r')%display locus 1 variant value - see if stops changing
if strcmp(modePar.netStr,'re'); plot(encounterYesGraph(:,nodeNum),'m'); end; %pulse every encounter
if strcmp(modePar.blklist,'yes')
plot(blklistrepPermanentGraph(:,nodeNum),'c')%this node to blacklist
plot(blklistYesGraph(:,nodeNum),'y')%blacklist now pulse
end
plot([1 numTimesteps],[stdNeiUpdateThreshold stdNeiUpdateThreshold],'y--')
plotstateNodeGraph(:,1)=stateNodeGraph(:,nodeNum)==3;
plot(stateNodeGraph(:,nodeNum),'g--')%recover
xlabel('Time')
ylabel('Std of local diversity of one node')
legend('Std Nei','Buffer full', 'stop Updating', 'Variant','Encounter pulse','Blacklisting','BlkListYes pulse','Stop Threshold','Recover')
title('Node 4')
figure
plot(updateStopFlagGraph(nodeNum,:),'g')%indicates when to stop updating
title('node 4')
figure
if strcmp(modePar.netStr,'re'); plot(encounterYesGraph(:,nodeNum),'m'); end; %pulse every encounter
hold on
plotG(1,:)=genotypeNodeGraph(1,nodeNum,:);
plot(plotG,'r')%display locus 1 variant value - see if stops changing
figure
plotstateNodeGraph(:,1)=stateNodeGraph(:,nodeNum)==3;
plot(stateNodeGraph(:,nodeNum),'g--')%recover
xlabel('Time')
ylabel('Recovery')
legend('Recover')
title('Node 4')
%*************************
end %update

%**********
%blacklist graphs only
if strcmp(modePar.blklist,'yes')
    %node1
figure
plotG(1,:)=genotypeNodeGraph(1,1,:); %loci,numNodes,time
plot(plotG,'r')%display locus 1 variant value - see if stops changing
hold on
if strcmp(modePar.netStr,'re'); plot(encounterYesGraph(:,1),'m'); end;%pulse every encounter
plot(blklistrepPermanentGraph(:,1),'c')%this node to blacklist
plot(blklistYesGraph(:,1),'y')%blacklist now pulse
legend('Variant','encounter pulse','blacklisting','blkListYes pulse')
title('node 1')
%node2
figure
plotG(1,:)=genotypeNodeGraph(1,2,:); %loci,numNodes,time
plot(plotG,'r')%display locus 1 variant value - see if stops changing
hold on
if strcmp(modePar.netStr,'re'); plot(encounterYesGraph(:,2),'m'); end;%pulse every encounter
plot(blklistrepPermanentGraph(:,2),'c')%this node to blacklist
plot(blklistYesGraph(:,2),'y')%blacklist now pulse
legend('Variant','encounter pulse','blacklisting','blkListYes pulse')
title('node 2')
%node3
figure
plotG(1,:)=genotypeNodeGraph(1,3,:); %loci,numNodes,time
plot(plotG,'r')%display locus 1 variant value - see if stops changing
hold on
if strcmp(modePar.netStr,'re'); plot(encounterYesGraph(:,3),'m'); end;%pulse every encounter
plot(blklistrepPermanentGraph(:,3),'c')%this node to blacklist
plot(blklistYesGraph(:,3),'y')%blacklist now pulse
legend('Variant','encounter pulse','blacklisting','blkListYes pulse')
title('node 3')
%node4
figure
plotG(1,:)=genotypeNodeGraph(1,4,:); %loci,numNodes,time
plot(plotG,'r')%display locus 1 variant value - see if stops changing
hold on
if strcmp(modePar.netStr,'re'); plot(encounterYesGraph(:,4),'m'); end;%pulse every encounter
plot(blklistrepPermanentGraph(:,4),'c')%this node to blacklist
plot(blklistYesGraph(:,4),'y')%blacklist now pulse
legend('Variant','encounter pulse','blacklisting','blkListYes pulse')
title('node 4')
%node5
figure
plotG(1,:)=genotypeNodeGraph(1,5,:); %loci,numNodes,time
plot(plotG,'r')%display locus 1 variant value - see if stops changing
hold on
if strcmp(modePar.netStr,'re'); plot(encounterYesGraph(:,5),'m'); end;%pulse every encounter
plot(blklistrepPermanentGraph(:,5),'c')%this node to blacklist
plot(blklistYesGraph(:,5),'y')%blacklist now pulse
legend('Variant','encounter pulse','blacklisting','blkListYes pulse')
title('node 5')
end%blacklist graphs
%**************


figure
plot(neiIndex.*1000,'k')
hold on
plot(numUpdateStopFlag,'c')
plot(numBlacklisting,'g--')
plot(numRecovered,'g')
plot([1 numTimesteps],[800 800],'k','linewidth',2) %max div value
legend('Variant Diversity','Number stopped updating','Number blacklisting','Number recovered')
end %disp graphs


%SIR graph
figure
plot(numSusceptibles,'b')
hold on
plot(numInfected,'r')
plot(neiIndex.*numNodes,'k')
%plot(numNotSusceptibles,'k')
%plot(numRecovered,'g')
plot(numBlacklisting,'g--')
plot(numUpdateStopFlag,'c')
if strcmp(modePar.vm,'yes'); plot(numvmupdateGraph,'m'); end;
plot([1 numTimesteps],[0.8*numNodes 0.8*numNodes],'k','linewidth',2) %max div value VR=5
%legend('numSusceptibles','numInfected','numRecovered','Variant Diversity','numBlacklisting', 'numStoppedUpdating','numvmupdate')
legend('Susceptible','Infected','Variant Diversity','Blacklisting','Stopped Updating','VM update')
xlabel('Time')
ylabel('Number of devices')

%variant diversity / percentage stopped updating
figure
plot(neiIndex,'k','linewidth',2)
hold on
plot(numUpdateStopFlag./numNodes,'k--','linewidth',2)
plot([1 numTimesteps],[0.8 0.8],'k','linewidth',1) %max div value VR=5
legend('Variant Diversity', 'numStoppedUpdating')
xlabel('Time (seconds)')
ylabel('Variant Diversity (network) Percentage stopped updating')
axis([1 numTimesteps 0 1]);
%title('1 run, variant div and number stopped updating')


figure
if strcmp(modePar.netStr,'re'); plot(encounterYesGraph(:,1),'c'); end;%node 1 
hold on
if strcmp(mode,'Dynamic-favScore'); plot(newGenotypeNodeValidGraph(:,1)+1.5,'b'); end;
plot(blklistYesGraph(:,1)+3,'r')
if strcmp(mode,'Dynamic-favScore'); plot(updateStopFlagGraph(1,:)+4.5,'m'); end;
if strcmp(modePar.vm,'yes'); plot(vmupdateGraph(:,1)+6,'g'); end;
plot(gammaYesGraph(:,1)+7.5,'k')
%plot(resetUpdateGraph(:,1)+9,'k--')
title('vm update')
axis([1 numTimesteps 0 11]);
if strcmp(modePar.netStr,'re');
    legend('encounterYesGraph','newGenotypeNodeValid','blklistYes','updateStopFlag','vmupdate','gammaYes')
else
    legend('newGenotypeNodeValid','blklistYes','updateStopFlag','vmupdate','gammaYes')
end





end%plot graphs
%***************************
end

%******************************
%Subfunctions
%*******************************

%compatability filtering
function chosenVariantOS = filterFavScore(numNodes,numLoci,variantRichness,favScore)
%create masks
%assume loci1 is OS locus
for i = 1 : variantRichness
    %each OS has two options in each locus to choose from
    maskOS(i).mask = zeros(numNodes,variantRichness,numLoci); %mask matrix for each variant
    maskOS(i).mask(:,i,:)=1; %set current variant number
    if i < variantRichness
        maskOS(i).mask(:,i+1,:)=1;  %set current variant number
        maskOS(i).mask(:,i+1,1)=0; %only set 1 os
    else
        maskOS(i).mask(:,1,:)=1;  %set current variant number
        maskOS(i).mask(:,1,1)=0; %only set 1 os
    end
end
%choose OS variant first
filterScore = zeros(numNodes,variantRichness,numLoci);
 [chosenValOS chosenVariantOS(:,1)] = probSol(favScore(:,:,1)); %use function from big model
%apply chosen mask to all devices
%only do if num Loci bigger than 1, otherwise variant already chosen
if numLoci > 1
    %for every node
    for j = 1:numNodes
        %xtract mask for chosen variant and multiply by favScore
        mask=maskOS(chosenVariantOS(j,1)).mask;
        maskSize=size(maskOS(chosenVariantOS(j,1)).mask);
        filterScore(j,:,:) = maskOS(chosenVariantOS(j,1)).mask(j,:,:) .* favScore(j,:,:);
    end
    %choose remaining variants
    for k=1:numLoci
        [chosenValOS chosenVariantOS(:,k)] = probSol(filterScore(:,:,k));
    end
end
end

%desirability constraints
function desScore = addDesirabilityConstraints(numNodes,numLoci,variantRichness,metric)
commDes = zeros(variantRichness,1);
commDesrep = zeros(numNodes,variantRichness,numLoci);
wmetric=0; %weight of metric value
wcommdes=1; %weight of community value
%for one locus
commDes(1,1)=0.84;
commDes(2:end,1)=0.16/(variantRichness-1); %rest share equally
commDesrep = repmat(commDes,1,numNodes)';
%apply to all loci
for i = 1:numLoci
    commAll(:,:,i)=commDesrep;
end
desScore = metric.*wmetric + commAll.*wcommdes;
%disp('setting desirability constraints')
end

%check that encountered node is infected
function nodeEncounterInfected = checkIfEncNodeInfected(numNodes,stateNode,txexploitNode,txexploitYes)
    %check if successfully encountered an exploit node and it is infected
    %if successful encounter, infection can be transmitted
    %txexploitNode is the node transmitting the exploit
    %txexploitYes is a mask denoting a successful encounter with a
    %node transmitting exploit data
    for j = 1:numNodes
        if txexploitYes(j)==1 %if there is actually a malware encounter
            nodeEncounterInfected(j) = (stateNode(txexploitNode(j))==2);%if encountered node is infected(state 2)
        else
            nodeEncounterInfected(j) = 0; %otherwise 0 if no encounter
        end
    end
end
%******************************
%plot functions for rwp
%*****************************
%plot of malware distance from origin
function graphMalwareDist(t,virusDistanceH,infected,rwph,precision,numTimesteps)

distance = sqrt(rwph.cx.^2 + rwph.cy.^2);
mdist = max(distance.*infected);

   figure(virusDistanceH)
   plot(t,mdist, 'k+', 'linewidth',2)
   hold on
   ymaxgr=sqrt((rwph.xmax*precision).^2+(rwph.xmax*precision).^2);
   axis([1 numTimesteps 0 ymaxgr]);
    title('Distance malware travelled')
end


%surf plot of genotypes
function graphLocalVariant(genotypeH,genotypeNode,lociNum, numVariants, numNodes,rwph,precision)
    cx=round(rwph.cx.*precision); %current x get precision and round to nearest point
    cy=round(rwph.cy.*precision); %current y get precision and round to nearest point
    LocalDivz = zeros(rwph.xmax*precision,rwph.ymax*precision); %reset values each time
    for n = 1:numNodes
    Localgen(cy(n),cx(n))=genotypeNode(lociNum,n)+1;
    end 
    figure(genotypeH)
    pcolor(Localgen)
    colormap(jet)
    colorbar('location','EastOutside')
    set(gca, 'CLim', [0.0, numVariants+1]);
    title('Genotypes')
end
%surf plot of local diversity measurements
function graphLocalDiversity(LocalDivzH,neiLocal,numNodes,rwph,precision)
    cx=round(rwph.cx.*precision); %current x get precision and round to nearest point
    cy=round(rwph.cy.*precision); %current y get precision and round to nearest point
    LocalDivz = zeros(rwph.xmax*precision,rwph.ymax*precision); %reset values each time
    for n = 1:numNodes
    LocalDivz(cy(n),cx(n))=neiLocal(n)+1;
    end 
    figure(LocalDivzH)
    pcolor(LocalDivz)
    colormap(jet)
    colorbar('location','EastOutside')
    set(gca, 'CLim', [0.0, 2.0]);
    title('Local diversity')
end
%graph of local states
function graphNodeStates(LocalStateH,stateNode,numNodes,rwph,precision,exploitMatch)
    cx=round(rwph.cx.*precision); %current x get precision and round to nearest point
    cy=round(rwph.cy.*precision); %current y get precision and round to nearest point
   figure(LocalStateH)
    clf(LocalStateH) %clear current figure
    for n = 1:numNodes
        if stateNode(n)==1; nodeShape = '+'; nodeColor = 'b'; end; %susceptible
        if exploitMatch(n)==1; nodeShape = '+'; nodeColor = 'c'; end; %instantaneous susceptible
        if stateNode(n)==2; nodeShape = '*'; nodeColor = 'r'; end; %infected
        if stateNode(n)==3; nodeShape = 'o'; nodeColor = 'g'; end; %recovered
        
        plotOne(n) = plot(cx(n),cy(n)); %plot point
        set(plotOne(n),'marker',nodeShape); %set marker
        set(plotOne(n),'color',nodeColor); %set colour
        hold on
    end
    axis([rwph.xmin*precision rwph.xmax*precision rwph.ymin*precision rwph.ymax*precision]);
    title('State of devices')
end

