%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Jennifer Jackson 2017                                                        %
% A Biodiversity Approach to Cyber Security                                    %
% PhD Complexity Science                                                       %
% University of Warwick                                                        %
% http://wrap.warwick.ac.uk/98458                                              %
% This software is distributed under the  Creative Commons Attribution License %
% https://creativecommons.org/licenses/by/4.0/                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [ output_args ] = SIPeakTime( input_args )

clear all;
close all;

%**********************************
% PEAK TIME GRAPHS - CALCULATED - mostly unused in results chapter
%*********************************
%1 locus set number of variants
Nn=1000
Bn=0.02
I0=1 %I(0)
V = 1:160; %number of variants
x=1 %1 exploit
Sp = x./V;

%With diversity
term1 = 1./(Sp.*Bn);
term2 = log((((Sp.*Nn)-1).*((Sp.*Nn)-I0))./(I0));
T = term1.*term2;

%with no diversity
NoDiversity_1 = (1/(0.02));
innerlog = (Nn-1)*(Nn-I0)/I0;
NoDiversity_2 = log(innerlog);
NoDiversity = NoDiversity_1 * NoDiversity_2;

TimeMins = T./60;
TimeHrs = TimeMins./60;

figure
plot(TimeHrs,'k')
hold on
%plot(NoDiversity,'r+')
xlabel('Average Variant Richness')
ylabel('Time of peak infection (Hours)')
title('1 locus - peak infection time up to 160 variants, N=1000')
%*************************************************************
%multi loci AND
%1 locus set number of variants
Nn=20000
Bn=0.02
I0=1 %I(0)
V = 1:8 %number of variants
%Nn=(17^4)*5
numExploits = 4
for x=1:numExploits %1 exploit
Sp1(x,:) = (x)./(V) %1locus
Sp2(x,:) = (x.^2)./(V.^2) %2loci
Sp3(x,:) = (x.^3)./(V.^3) %3loci
Sp4(x,:) = (x.^4)./(V.^4) %4loci

%With diversity
term11(x,:) = 1./(Sp1(x,:).*Bn);
term21(x,:) = log((((Sp1(x,:).*Nn)-1).*((Sp1(x,:).*Nn)-I0))./(I0));
Ta1(x,:) = term11(x,:).*term21(x,:);

term12(x,:) = 1./(Sp2(x,:).*Bn);
term22(x,:) = log((((Sp2(x,:).*Nn)-1).*((Sp2(x,:).*Nn)-I0))./(I0));
Ta2(x,:) = term12(x,:).*term22(x,:);

term13(x,:) = 1./(Sp3(x,:).*Bn);
term23(x,:) = log((((Sp3(x,:).*Nn)-1).*((Sp3(x,:).*Nn)-I0))./(I0));
Ta3(x,:) = term13(x,:).*term23(x,:);

term14(x,:) = 1./(Sp4(x,:).*Bn);
term24(x,:) = log((((Sp4(x,:).*Nn)-1).*((Sp4(x,:).*Nn)-I0))./(I0));
Ta4(x,:) = term14(x,:).*term24(x,:);
end
TimeMinsa1 = Ta1./60;
TimeMinsa2 = Ta2./60;
TimeHrsa2 = TimeMinsa2./60;
TimeMinsa3 = Ta3./60;
TimeMinsa4 = Ta4./60;

figure
for x = 1:1% exploits
plot(TimeMinsa1(x,:)./60./24,'ko-')
hold on
plot(TimeMinsa2(x,:)./60./24,'k+-')
plot(TimeMinsa3(x,:)./60./24,'k--')
plot(TimeMinsa4(x,:)./60./24,'k.-')
end
%plot(NoDiversity,'r+')
xlabel('Average Variant Richness')
ylabel('Time of peak infection (Days)')
title('2 loci AND - peak infection time up to 8 variants, 1 exploit per locus, N=20,000')
legend('1 locus','2 loci AND','3 loci AND','4 loci AND')

figure
for x = 2:2% exploits
plot(TimeMinsa1(x,:)./60./24,'ko-')
hold on
plot(TimeMinsa2(x,:)./60./24,'k+-')
plot(TimeMinsa3(x,:)./60./24,'k--')
plot(TimeMinsa4(x,:)./60./24,'k.-')
end
%plot(NoDiversity,'r+')
xlabel('Average Variant Richness')
ylabel('Time of peak infection (Days)')
title('multi loci AND - peak infection time up to 8 variants, 2 exploits per locus, N=20,000')
legend('1 locus','2 loci AND','3 loci AND','4 loci AND')

figure
for x = 4:4% exploits
plot(TimeMinsa1(x,:)./60,'ko-')
hold on
plot(TimeMinsa2(x,:)./60,'k+-')
plot(TimeMinsa3(x,:)./60,'k--')
plot(TimeMinsa4(x,:)./60,'k.-')
end
%plot(NoDiversity,'r+')
xlabel('Average Variant Richness')
ylabel('Time of peak infection (Hours)')
title('multi loci AND - peak infection time up to 8 variants, 4 exploits per locus, N=20,000')
legend('1 locus','2 loci AND','3 loci AND','4 loci AND')

%1 locus set susceptibility
%Nn=1000
Bn=0.02
I0=1 %I(0)
%V = 1:160 %number of variants
x=1 %1 exploit
Sp = [0.0002 0.0005 0.001 0.005 0.01 0.05 0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1];
QoS1 = (1-Sp)*100;
term1 = 1./(Sp.*Bn);
term2 = log((((Sp.*Nn)-1).*((Sp.*Nn)-I0))./(I0));
T = term1.*term2

n95 = ones(1,16)*3.837;
n90 = ones(1,16)*2.111;
n80 = ones(1,16)*1.152;

NoDiversity_1 = (1/(0.02))
innerlog = (Nn-1)*(Nn-I0)/I0
NoDiversity_2 = log(innerlog)
NoDiversity = NoDiversity_1 * NoDiversity_2

TimeMins = T./60;
TimeHrs1 = TimeMins./60;
figure
plot(QoS1,TimeHrs1,'k+-', 'Linewidth',2)
hold on
plot([80 80],[0 1.5],'k--')
plot([90 90],[0 2.5],'k--')
plot([95 95],[0 4],'k--')
plot([77 95],[3.837 3.837],'k--')
plot([77 90],[2.111 2.111],'k--')
plot([77 80],[1.152 1.152],'k--')
hold on
%plot(NoDiversity,'r+')
xlabel('Quality of Service Tolerance')
ylabel('Time of peak infection (Hours)')
title('peak infection time, N=20,000')
axis([77 97 0 5])

%1 locus set susceptibility
Nn=100
Bn=0.02
I0=1 %I(0)
%V = 1:160 %number of variants
%x=1 %1 exploit
Sp = [0.05 0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1];
QoS = (1-Sp)*100;
term1 = 1./(Sp.*Bn);
term2 = log((((Sp.*Nn)-1).*((Sp.*Nn)-I0))./(I0));
T = term1.*term2

NoDiversity_1 = (1/(0.02))
innerlog = (Nn-1)*(Nn-I0)/I0
NoDiversity_2 = log(innerlog)
NoDiversity = NoDiversity_1 * NoDiversity_2

n95 = ones(1,11)*46.2;
n90 = ones(1,11)*36.6;
n80 = ones(1,11)*24.5;

TimeMins = T./60
TimeHrs = TimeMins./60;
figure
plot(QoS,TimeMins,'k','Linewidth', 2)
hold on
plot([80 80],[1 30],'k--')
plot([90 90],[1 40],'k--')
plot([95 95],[1 50],'k--')
plot(QoS,n95,'k--')
plot(QoS,n90,'k--')
plot(QoS,n80,'k--')
%plot(NoDiversity,'r+')
xlabel('Quality of Service Tolerance')
ylabel('Time of peak infection (Minutes)')
title('Nn=100, Bn=0.02, T= 0 to 95%')

%return

%**********************************************************************
%SI SIMULATION COMPARE Peak Time of stochastic and Deterministic with
%calculated, Fix quality of service tolerance, Sp=0.2, 0.1, 0.05
%**********************************************************************

%*********
%SI test 1 - Fix quality of service tolerance, Sp=0.2, 0.1, 0.05
%*********
outputDirectory = 'graphs\'
%calculated
Nn=20000
Bn=0.02
I0=1
Sp = [0.0002 0.0005 0.001 0.005 0.01 0.05 0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1];
QoS1 = (1-Sp)*100;
term1 = 1./(Sp.*Bn);
term2 = log((((Sp.*Nn)-1).*((Sp.*Nn)-I0))./(I0));
T = term1.*term2
TimeMins = T./60;
TimeHrs1 = TimeMins./60;
%*******************SIR Deterministic
N=20001
Sprop = [0.2 0.1 0.05]
S = N.*Sprop %Ns
B = 0.02 %bn
beta = B.*Sprop %bs
gamma = 0; %1/48*60*60 %2 days
R=0
I=1
drawPlots=0
timeMax=15000
timeHours = (1:timeMax)./60./60;
c = ['r', 'm', 'c', 'k','b', 'g', 'r', 'm']
for i = 1:3
    sd = S(i)
    [Ro(i) St(i,:) It(i,:) Rt(i,:)] = SIRDeterministic(beta(i), gamma, S(i), I, R, timeMax, drawPlots);   
     mask(i,:) = It(i,:) >= S(i)-1; %when It within 1 of the final value
    [C(i) Index(i)] = max(mask(i,:));
    T1(i) = Index(i)./60./60 %T1
end

%figure
% plot(timeHours,It(1,:),'k')      
% hold on
%  plot(timeHours,It(2,:),'k','Linewidth',2) 
%   plot(timeHours,It(3,:),'k--') 
% plot(T1(1),(S(1)-1),'Ko','Linewidth',2) 
% plot(T1(2),(S(2)-1),'Ko','Linewidth',2) 
% plot(T1(3),(S(3)-1),'Ko','Linewidth',2) 
% xlabel('Time (Hours)')
% ylabel('Number Infected (It)')
% legend('80% Tolerance', '90% Tolerance', '95% Tolerance')
 
 figure
plot(QoS1,TimeHrs1,'k+-', 'Linewidth',2)
hold on
%calculated
plot([80 80],[0 1.5],'k--')
plot([90 90],[0 2.5],'k--')
plot([95 95],[0 4],'k--')
plot([77 95],[3.837 3.837],'k--')
plot([77 90],[2.111 2.111],'k--')
plot([77 80],[1.152 1.152],'k--')
%measured from simulation
plot([80],[T1(1)],'ko', 'Linewidth',2)
plot([90],[T1(2)],'ko', 'Linewidth',2)
plot([95],[T1(3)],'ko', 'Linewidth',2)
xlabel('Quality of Service Tolerance')
ylabel('Time of peak infection (Hours)')
title('peak infection time, N=20,000')
axis([77 97 0 5])

%*******************SIR stochastic
numNodes = N.*Sprop
beta = B.*Sprop %bs
gamma = 0
maxTime = 20000 %17000;
numSims = 3 %500;
dtt=0.1%0.001; % Time step
numTimesteps = floor(maxTime/dtt); %7000
time = 0:dtt:numTimesteps*dtt-dtt;
 disp('calculating Stochastic result.....')
for j = 1:3 %3 Sprop values
for i = 1:numSims
    i
    j
 [ Ss(j,i,:), Is(j,i,:), Rs(j,i,:) ] = stochasticSIR(numNodes(j), beta(j), numTimesteps, dtt, gamma);
end
end
 disp('finished calculating stochastic result.....')
 %figS = figure
 %for i = 1:5
 %    plots(i,:) = Ss(1,i,:);
 %plot(time,plots,'k')
 %hold on
 %end
 %xlabel('Time')
 %ylabel('Susceptible')

 %figI = figure
 %for i = 1:50
 %    ploti(i,:) = Is(1,i,:);
 %plot(time,ploti,'k')
 %hold on
 %end
 %xlabel('Time')
 %ylabel('Infected')
 
for j = 1: 3 
meanI(j,:)=mean(Is(j,:,:));
meanS(j,:)=mean(Ss(j,:,:));
end
 %compare to deterministic
 Sd = numNodes
 Id = 1
 Rd = 0
 timeMax = time(length(time));
 for j = 1: 3
 [Rod(j,:) Std(j,:) Itd(j,:) Rtd(j,:)] = SIRDeterministic(beta(j), gamma, Sd(j), Id, Rd, timeMax, 0);
 step = timeMax / length(Itd(j,:));
 ts = [0:step:timeMax-step];
 size(ts)
 size(Itd)
 end
 disp('calculating T1s.....')
 %within 1 of final values
 for j = 1:3
 %stochastic
      masks(j,:) = meanI(j,:) >= numNodes(j)-1; %when It within 1 of the final value
    [Cs(j) Indexs(j)] = max(masks(j,:))
    T1s(j) = Indexs(j).*dtt./60./60 %T1
 %deterministic
    maskd(j,:) = Itd(j,:) >= numNodes(j)-1; %when It within 1 of the final value
    [Cd(j) Indexd(j)] = max(maskd(j,:));
    T1d(j) = Indexd(j)./60./60 %T1
 end
 %calculated
 T1c=[1.152 2.111 3.837]
 
  disp('plotting.....')
 %plot 
 figmean = figure
 timeHours = time./60./60;

 %Deterministic
 plot(ts./60./60,Itd(1,:),'k--', 'Linewidth',1)
  hold on
  plot(time./60./60,meanI(1,:),'k')
  plot(T1d(1),(numNodes(1)-1),'K+','Linewidth',2) 
  plot(T1s(1),(numNodes(1)-1),'Ko','Linewidth',2)
   plot(1.152,(numNodes(1)-1),'ks')
   
  plot(ts./60./60,Itd(2,:),'k--', 'Linewidth',1)
 plot(ts./60./60,Itd(3,:),'k--', 'Linewidth',1)
 %stochastic
 plot(time./60./60,meanI(2,:),'k')
 plot(time./60./60,meanI(3,:),'k')
 %Deterministic Peak points
  plot(T1d(1),(numNodes(1)-1),'K+','Linewidth',2) 
 plot(T1d(2),(numNodes(2)-1),'K+','Linewidth',2) 
 plot(T1d(3),(numNodes(3)-1),'K+','Linewidth',2) 
 %stochastic Peak points
  plot(T1s(1),(numNodes(1)-1),'Ko','Linewidth',2) 
 plot(T1s(2),(numNodes(2)-1),'Ko','Linewidth',2) 
 plot(T1s(3),(numNodes(3)-1),'Ko','Linewidth',2) 
 %calculated peak points
 plot(T1c(1),(numNodes(1)-1),'ks')
plot(T1c(2),(numNodes(2)-1),'ks')
plot(T1c(3), (numNodes(3)-1),'ks')
 legend('Deterministic It','Stochastic It', 'Deterministic T1', 'Stochastic T1', 'Calculated T1')
 xlabel('Time (Hours)')
 ylabel('Number Infected (It)')
 axis([0 5 0 4500])
 titleText = ['Td = ' num2str(T1d(1)) ' ' num2str(T1d(2)) ' ' num2str(T1d(3)) ' Ts = '  num2str(T1s(1)) ' ' num2str(T1s(2)) ' ' num2str(T1s(3)) ' Tc = '  num2str(T1c(1)) ' ' num2str(T1c(2)) ' ' num2str(T1c(3))]
 title(titleText)
%*************************************************************************************************



end

