%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Jennifer Jackson 2017                                                        %
% A Biodiversity Approach to Cyber Security                                    %
% PhD Complexity Science                                                       %
% University of Warwick                                                        %
% http://wrap.warwick.ac.uk/98458                                              %
% This software is distributed under the  Creative Commons Attribution License %
% https://creativecommons.org/licenses/by/4.0/                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [ output_args ] = SITolerance( input_args )
%   calculates the number of variants in each locus needed to tolerate the
%   specified number of exploits
close all;
clear all;
runApprox = 0 %run approx graphs
if runApprox==1
%*********************************************************************************************************************
%***********************************************
%EPIDEMIC MODEL CHAPTER
%***********************************************
%***********************************************
%OUTPUTS - NO RECOVERY
%optimum Diversity for Quality of Service Tolerance
%*******************
%2 loci AND example - general case, exact solution
%*******************
%minimum solution for specified exploits in each locus
x1 = 2; x2 = 4; tol = 0.2;
mw1 = 2; mw2 = 4;
numGenotypes = x1*x2/tol
w1=1:12
w2 = ceil(numGenotypes./w1)
minw1 = ones(1,12)*mw1
minw2 = ones(1,12)*mw2
figure
plot(w1,w2,'k')
hold on
plot(minw2,'k--')
plot(mw1,w2,'k+-')
xlabel('v1')
ylabel('v2')
title('Number of variants - different solutions')

figure
sumVariants = w1+w2
plot(w1,sumVariants,'k')
hold on
xlabel('v1')
ylabel('sum of Variants')
title('Sum of variants - optimum solutions')

figure
VR = sumVariants/2
plot(w1,VR,'k')
hold on
xlabel('v1')
ylabel('Variant Richness')
%title('Sum of variants - optimum solutions')
%*******************
%2 loci OR example - general case, exact solution
%*******************
s=0.2;x1=2;x2=4;
for v1 = 1:40
v2(v1)=ceil(((x1*x2/v1)-x2)/((x1/v1)-s)); %exact solution
v2Approx(v1) =ceil(x2./(s-(x1/v1)));%approx solution when only use first term
sumV(v1) = v1+v2(v1);
VR = sumV/2;
multV(v1) = v1*v2(v1);
end
figure
plot(v2,'k')
hold on
%plot(v2Approx,'b')
xlabel('v1')
ylabel('v2')
title('OR 2 loci - Number of variants')

figure
plot(VR,'k')
xlabel('v1')
ylabel('Variant Richness')
%title('OR 2 loci - VR')

figure
plot(multV,'k')
xlabel('v1')
ylabel('Total number of genotypes')
title('OR 2 loci - Number of genotypes')

%*************************************************************
%OUTPUTS - NO RECOVERY
%************************************************************
%APPROXIMATION GRAPHS - OR type

%*************************************************************
%********************************
%Approximate solution example for two loci, comparing the exact solution to
%the approximate solution
%2 loci OR plot equation - maximum diversity when x1 = x2 and v1 = v2
%2x/v - x^2/v^2
%x=1;
maxV=500;
for x=1:20
v=1:maxV;
a(x,:)=(2.*x./v);
b(x,:)=(x.^2./v.^2);
size(a);
size(b);
sp = a - b;
end
sp2=sp;
a2=a;
figure
plot(sp(1,:),'k', 'LineWidth',2)
hold on
plot(a(1,:),'k--', 'LineWidth',2)
%plot(b(1,:),'ko-', 'LineWidth',2)

%plot(2:maxV,sp(2,2:end),'b') %only valide for v=>x
%plot(2:maxV,a(2,2:end),'b--')
%plot(2:maxV,b(2,2:end),'b.-')

plot(3:maxV,sp(3,3:end),'k')
plot(3:maxV,a(3,3:end),'k--')
%plot(3:maxV,b(3,3:end),'c.-')

%plot(4:maxV,sp(4,4:end),'g')
%plot(4:maxV,a(4,4:end),'g--')
%plot(4:maxV,b(4,4:end),'g.-')

plot(5:maxV,sp(5,5:end),'k')
plot(5:maxV,a(5,5:end),'k--')
%plot(5:maxV,b(5,5:end),'ko-')

plot(7:maxV,sp(7,7:end),'k')
plot(7:maxV,a(7,7:end),'k--')
%plot(5:maxV,b(5,5:end),'ko-')

title('OR 2 loci - Equation 2x/v - x^2/v^2')
xlabel('v1 and v2')
ylabel('Sp')
%legend ('Susceptibility', '2x/v', 'x^2/v^2')
%legend ('Exact (x1/v1 + x2/v2 - x1x2/v1v2)', 'Term 1 (x1/v1 + x2/v2)', 'Term 2 (x1x2/v1v2)')
legend ('Exact (x1/v1 + x2/v2 - x1x2/v1v2)', 'Approx (x1/v1 + x2/v2)')
axis([0 70 0 1.2])
%********************************
%2 loci OR plot equation - NOT maximum diversity
%x1/v1 +x2/v2 - x1x2/v1v2
%x=1;
maxV1=500;
x2=1;
v2=6;
for x1=1:5
v1=1:maxV1;
a(x1,:)=(x1./v1)+(x2./v2);
b(x1,:)=((x1.*x2)./(v1.*v2));
size(a);
size(b);
sp = a - b;
end
%figure
%plot(sp(1,:),'k')
%hold on
%plot(a(1,:),'k--')
%plot(b(1,:),'k.-')
%plot(5:maxV1,sp(5,5:end),'b')
%hold on
%plot(5:maxV1,a(5,5:end),'b--')
%plot(5:maxV1,b(5,5:end),'b.-')
%title('OR 2 loci - Equation x1/v1 + x2/v2 - x1x2/v1v2')
%xlabel('v')
%ylabel('sp')
%legend ('susceptibility', 'x1/v1 + x2/v2', 'x1x2/v1v2')
%********************************
%3 loci OR plot equation - maximum diversity when x1 = x2 = x3 and v1 = v2
%= v3
%3x/v - 3x^2/v^2 + x^3/v^3
%x=1;
maxV=500;
%c=zeros(20,maxV)
for x=1:20
v=1:maxV;
a(x,:)=((3.*x)./v);
b(x,:)=((3.*(x.^2))./(v.^2));
c(x,:)=(x.^3)./(v.^3);
size(a);
size(b);
size(c);
end
sp = a - b + c;
sp3=sp;
a3=a;
figure
plot(sp(1,:),'k', 'LineWidth',2)
hold on
plot(a(1,:),'k--', 'LineWidth',2)
%plot(b(1,:),'ko-', 'LineWidth',2)
%plot(c(1,:),'k.-', 'LineWidth',2)

%plot(2:maxV,sp(2,2:end),'b') %only valide for v=>x
%plot(2:maxV,a(2,2:end),'b--')
%plot(2:maxV,b(2,2:end),'b.-')

plot(3:maxV,sp(3,3:end),'k')
plot(3:maxV,a(3,3:end),'k--')
%plot(3:maxV,b(3,3:end),'ko-')
%plot(3:maxV,c(3,3:end),'k.-')

%plot(4:maxV,sp(4,4:end),'g')
%plot(4:maxV,a(4,4:end),'g--')
%plot(4:maxV,b(4,4:end),'g.-')

plot(5:maxV,sp(5,5:end),'k')
plot(5:maxV,a(5,5:end),'k--')
%plot(5:maxV,b(5,5:end),'ko-')
%plot(5:maxV,c(5,5:end),'k.-')

plot(7:maxV,sp(7,7:end),'k')
plot(7:maxV,a(7,7:end),'k--')
%plot(7:maxV,b(7,7:end),'ko-')
%plot(7:maxV,c(7,7:end),'k.-')

title('OR 3 loci - Equation 3x/v - 3x^2/v^2 + x^3/v^3')
xlabel('v1,v2,v3')
ylabel('Sp')
%legend ('Susceptibility', '2x/v', 'x^2/v^2')
%legend ('Exact (x1/v1 + x2/v2 - x1x2/v1v2)', 'Term 1 (x1/v1 + x2/v2)', 'Term 2 (x1x2/v1v2)')
legend ('Exact ', 'Approx')
axis([0 80 0 1.2])
%********************************
%4 loci OR plot equation - maximum diversity when x1 = x2 = x3=x4 and v1 = v2
%= v3=v4
%4x/v - 6x^2/v^2 + 4x^3/v^3 - x^4/v^4
%x=1;
maxV=500;
%c=zeros(20,maxV)
for x=1:20
v=1:maxV;
a(x,:)=((4.*x)./v);
b(x,:)=((6.*(x.^2))./(v.^2));
c(x,:)=(4.*x.^3)./(v.^3);
d(x,:)=(x.^4)./(v.^4);
size(a);
size(b);
size(c);
end
sp = a - b + c - d;
sp4=sp;
a4=a;
figure
plot(sp(1,:),'k', 'LineWidth',2)
hold on
plot(a(1,:),'k--', 'LineWidth',2)
%plot(b(1,:),'ko-', 'LineWidth',2)
%plot(c(1,:),'k.-', 'LineWidth',2)

%plot(2:maxV,sp(2,2:end),'b') %only valide for v=>x
%plot(2:maxV,a(2,2:end),'b--')
%plot(2:maxV,b(2,2:end),'b.-')

plot(3:maxV,sp(3,3:end),'k')
plot(3:maxV,a(3,3:end),'k--')
%plot(3:maxV,b(3,3:end),'ko-')
%plot(3:maxV,c(3,3:end),'k.-')

%plot(4:maxV,sp(4,4:end),'g')
%plot(4:maxV,a(4,4:end),'g--')
%plot(4:maxV,b(4,4:end),'g.-')

plot(5:maxV,sp(5,5:end),'k')
plot(5:maxV,a(5,5:end),'k--')
%plot(5:maxV,b(5,5:end),'ko-')
%plot(5:maxV,c(5,5:end),'k.-')

plot(7:maxV,sp(7,7:end),'k')
plot(7:maxV,a(7,7:end),'k--')
%plot(7:maxV,b(7,7:end),'ko-')
%plot(7:maxV,c(7,7:end),'k.-')

title('OR 4 loci - Equation 4x/v - 6x^2/v^2 + 4x^3/v^3 - x^4/v^4')
xlabel('v1,v2,v3,v4')
ylabel('Sp')
%legend ('Susceptibility', '2x/v', 'x^2/v^2')
%legend ('Exact (x1/v1 + x2/v2 - x1x2/v1v2)', 'Term 1 (x1/v1 + x2/v2)', 'Term 2 (x1x2/v1v2)')
legend ('Exact ', 'Approx')
axis([0 80 0 1.2])

%Compare loci between exact and approximate solutions OR MALWARE AVERAGE results*******************
maxV=500; for x=1:20 v=1:maxV; a1(x,:)=((x)./v); sp1=a1; end; %1 loci
figure %x=1
plot(sp1(1,:),'k', 'LineWidth',1)
hold on
plot(a1(1,:),'k--', 'LineWidth',1)
plot(sp2(1,:),'k', 'LineWidth',1)
plot(a2(1,:),'k--', 'LineWidth',1)
plot(sp3(1,:),'k', 'LineWidth',1)
plot(a3(1,:),'k--', 'LineWidth',1)
plot(sp4(1,:),'k', 'LineWidth',1)
plot(a4(1,:),'k--', 'LineWidth',1)

title('OR All loci x=1')
xlabel('v1,v2,v3,v4')
ylabel('Sp')
%legend ('Susceptibility', '2x/v', 'x^2/v^2')
%legend ('Exact (x1/v1 + x2/v2 - x1x2/v1v2)', 'Term 1 (x1/v1 + x2/v2)', 'Term 2 (x1x2/v1v2)')
legend ('Exact ', 'Approx')
axis([0 40 0 1.1])


figure %x=20
plot(sp1(20,:),'k', 'LineWidth',1)
hold on
plot(a1(20,:),'k--', 'LineWidth',1)
plot(sp2(20,:),'k', 'LineWidth',1)
plot(a2(20,:),'k--', 'LineWidth',1)
plot(sp3(20,:),'k', 'LineWidth',1)
plot(a3(20,:),'k--', 'LineWidth',1)
plot(sp4(20,:),'k', 'LineWidth',1)
plot(a4(20,:),'k--', 'LineWidth',1)

title('OR All loci x=20')
xlabel('v1,v2,v3,v4')
ylabel('Sp')
%legend ('Susceptibility', '2x/v', 'x^2/v^2')
%legend ('Exact (x1/v1 + x2/v2 - x1x2/v1v2)', 'Term 1 (x1/v1 + x2/v2)', 'Term 2 (x1x2/v1v2)')
legend ('Exact ', 'Approx')
axis([0 500 0 1.1])

figure %x=8
plot(sp1(8,:),'k', 'LineWidth',1)
hold on
plot(a1(8,:),'k--', 'LineWidth',1)
plot(sp2(8,:),'k', 'LineWidth',1)
plot(a2(8,:),'k--', 'LineWidth',1)
plot(sp3(8,:),'k', 'LineWidth',1)
plot(a3(8,:),'k--', 'LineWidth',1)
plot(sp4(8,:),'k', 'LineWidth',1)
plot(a4(8,:),'k--', 'LineWidth',1)
axis([0 300 0 1.1])
title('OR All loci x=8')
xlabel('v1,v2,v3,v4')
ylabel('Sp')
%legend ('Susceptibility', '2x/v', 'x^2/v^2')
%legend ('Exact (x1/v1 + x2/v2 - x1x2/v1v2)', 'Term 1 (x1/v1 + x2/v2)', 'Term 2 (x1x2/v1v2)')
legend ('Exact ', 'Approx')
%end %if runApprox==1
%***************************************************
%END OF APPROXIMATION OR GRAPHS
%***************************************************

%*************************************************************************************************************************************

%********************************************************************************************************************************
clear all;
%************************************************
%EPIDEMIC RESULTS
%************************************************
%************************************************
%4 loci AND example - general case exact solution
%Determine optimum number of variants at each locus using the GENERAL
%EQUATION

%************************************************
x = [2 3 2 4;...
     4 3 5 4;...
     5 4 6 5;...
     6 4 6 7;...
     6 5 8 7;...
     8 5 8 8;...
     8 8 7 8;...
     8 8 8 8] % exploits
 
 Sp = [0.2 0.1 0.05]
 maxV = 20; %max number of variants
 for j = 1:3 %tolerance
 for i = 1:8 %exploits
[ v1(j,i) v2(j,i) v3(j,i) v4(j,i) ] = Loci4Min( x(i,1),x(i,2),x(i,3),x(i,4),Sp(j), maxV)
%xM(i) = x(i,1).*x(i,2).*x(i,3).*x(i,4); %graph values
xM(i) = x(i,1)+x(i,2)+x(i,3)+x(i,4); %graph values
 end
 end
 minX=9
maxX=34
  %80%
 figure
 plot(xM,v1(1,:),'k+','Linewidth',2,'markersize',10)
 hold on
 plot(xM,v2(1,:),'ko','Linewidth',2,'markersize',8)
 plot(xM,v3(1,:),'ks','Linewidth',2,'markersize',10)
 plot(xM,v4(1,:),'k.','Linewidth',2,'markersize',10)
 %xlabel('x1x2x3x4')
 xlabel('Total number of exploits')
 ylabel('Number of variants')
 legend('v1', 'v2', 'v3', 'v4')
 title('80% Tolerance, 4 loci AND')
 % axis([0 4500 0 20])
axis([minX maxX 0 20])
    %90%
 figure
 plot(xM,v1(2,:),'k+','Linewidth',2,'markersize',10)
 hold on
 plot(xM,v2(2,:),'ko','Linewidth',2,'markersize',8)
 plot(xM,v3(2,:),'ks','Linewidth',2,'markersize',10)
 plot(xM,v4(2,:),'k.','Linewidth',2,'markersize',10)
 %xlabel('x1x2x3x4')
 xlabel('Total number of exploits')
 ylabel('Number of variants')
 legend('v1', 'v2', 'v3', 'v4')
 title('90% Tolerance, 4 loci AND')
 % axis([0 4500 0 20])
 axis([minX maxX 0 20])
 %95%
 figure
 plot(xM,v1(3,:),'k+','Linewidth',2,'markersize',10)
 hold on
 plot(xM,v2(3,:),'ko','Linewidth',2,'markersize',8)
 plot(xM,v3(3,:),'ks','Linewidth',2,'markersize',10)
 plot(xM,v4(3,:),'k.','Linewidth',2,'markersize',10)
 %xlabel('x1x2x3x4')
 xlabel('Total number of exploits')
 ylabel('Number of variants')
 legend('v1', 'v2', 'v3', 'v4')
 title('95% Tolerance, 4 loci AND')
 % axis([0 4500 0 20])
 axis([minX maxX 0 20])
  
%************************************************
%4 loci OR example - general case APPROXIMATION solution
%Determine optimum number of variants at each locus using the GENERAL
%APPROXIMATION
%************************************************
clear all;
x = [2 3 2 4;...
     4 3 5 4;...
     5 4 6 5;...
     6 4 6 7;...
     6 5 8 7;...
     8 5 8 8;...
     8 8 7 8;...
     8 8 8 8] % exploits
 Sp = [0.2 0.1 0.05]
 
 %calc max V values
 xsum = sum(x,2) %sumrows
 %minSp = Sp(3)
 %maxV = ceil(xsum./Sp) + 20 %180; %max number of variants, AT LEAST 4*8/0.05 = 640 for 4 loci, 8 exploits
 for j = 1:3 %tolerance
     j
 for i = 1:8 %exploits
     i
     maxV(i,j) = ceil(xsum(i)./Sp(j)) + 20
[ v1(j,i) v2(j,i) v3(j,i) v4(j,i) ] = Loci4MinOR( x(i,1),x(i,2),x(i,3),x(i,4),Sp(j), maxV(i,j))
%xM(i) = x(i,1).*x(i,2).*x(i,3).*x(i,4); %graph values
xM(i) = x(i,1)+x(i,2)+x(i,3)+x(i,4); %graph values
 end
 end
  minX=9
maxX=34
  %80%
 figure
 plot(xM,v1(1,:),'k+','Linewidth',2,'markersize',10)
 hold on
 plot(xM,v2(1,:),'ko','Linewidth',2,'markersize',8)
 plot(xM,v3(1,:),'ks','Linewidth',2,'markersize',10)
 plot(xM,v4(1,:),'k.','Linewidth',2,'markersize',10)
 %xlabel('x1x2x3x4')
 xlabel('Total number of exploits')
 ylabel('Number of variants')
 legend('v1', 'v2', 'v3', 'v4')
 title('80% Tolerance, 4 loci OR')
 % axis([0 4500 0 180])
 axis([minX maxX 0 180])
    %90%
 figure
 plot(xM,v1(2,:),'k+','Linewidth',2,'markersize',10)
 hold on
 plot(xM,v2(2,:),'ko','Linewidth',2,'markersize',8)
 plot(xM,v3(2,:),'ks','Linewidth',2,'markersize',10)
 plot(xM,v4(2,:),'k.','Linewidth',2,'markersize',10)
 %xlabel('x1x2x3x4')
 xlabel('Total number of exploits')
 ylabel('Number of variants')
 legend('v1', 'v2', 'v3', 'v4')
 title('90% Tolerance, 4 loci OR')
 % axis([0 4500 0 350])
 axis([minX maxX 0 350])
 %95%
 figure
 plot(xM,v1(3,:),'k+','Linewidth',2,'markersize',10)
 hold on
 plot(xM,v2(3,:),'ko','Linewidth',2,'markersize',8)
 plot(xM,v3(3,:),'ks','Linewidth',2,'markersize',10)
 plot(xM,v4(3,:),'k.','Linewidth',2,'markersize',10)
 %xlabel('x1x2x3x4')
 xlabel('Total number of exploits')
 ylabel('Number of variants')
 legend('v1', 'v2', 'v3', 'v4')
 title('95% Tolerance, 4 loci OR')
%  axis([0 4500 0 670])
axis([minX maxX 0 670])

  end %if runApprox==1
%******************************************************
%OUTPUTS - NO RECOVERY
%******************************************************
%Quality of service tolerance graphs AVERAGE - AND and OR TYPEs
%Average variant richness for a given tolerance 
%And peak infection time
%******************************************************

Tolerance = [0.8 0.9 0.95]
Susceptibility = [0.2 0.1 0.05]
numExploits = 1:8; %number of exploits at each locus

%**************
%single locus 
%**************
for T = 1:3 %tolerance loop
    for E = 1:8%exploit loop     
   v1(E,T) = ceil(E/(Susceptibility(T)));      %use ceil as needs to be rounded UP to a whole number   
   Sp1(E,T)=(E)./((v1(E,T)))%1loci - true SP for peak time
    end
end
%1 y label
figure %NOT USED - Average variant richness for a given tolerance
plot(v1(:,1),'k*-')
hold on
plot(v1(:,2),'k+-')
plot(v1(:,3),'ko-')
legend('80% Tolerance','90% Tolerance','95% Tolerance')
xlabel('Number of exploits per locus')
ylabel('Average Variant Richness')
title('Resistance Tolerance - 1 locus')
%2 y labels - trying to plot genotypic richness as well
%figure
%%X = [1:8];
%A=plotyy(X,v1(:,1),X,v1(:,1));
%hold on
%plotyy(X,v1(:,2),X,v1(:,2));
%plotyy(X,v1(:,3),X,v1(:,3));
%legend('80% Tolerance','90% Tolerance','95% Tolerance')
%xlabel('Number of exploits per locus')
%title('Resistance Tolerance - 1 locus')
%set(A(2),'ytick',[],'yticklabel',[20 40 60 80 100 120 140 160]);
% ylabel(A(1),'Average Variant Richness','color','k')
% ylabel(A(2),'Genotypic Richness','color','k')
%set(A(1), 'YColor', 'black');
%set(A(2), 'YColor', 'black');

%Peak infection time With diversity - 1 locus model
Bn=0.02;
Nn = 20000;
I0 = 1
term1 = 1./(Sp1.*Bn);
term2 = log((((Sp1.*Nn)-1).*((Sp1.*Nn)-I0))./(I0));
T1 = term1.*term2;
figure
plot(T1(:,1)./60./60,'k*-','Linewidth',2)
hold on
plot(T1(:,2)./60./60,'k+-','Linewidth',2)
plot(T1(:,3)./60./60,'ko-','Linewidth',2)
legend('80% Tolerance','90% Tolerance','95% Tolerance')
xlabel('Number of exploits per locus')
ylabel('Peak Infection Time (Hours)')
title('Peak Infection Time - 1 locus AND')

%**************
%AND two loci 
%**************
J=2 %2 loci
for T = 1:3 %tolerance loop
    for E = 1:8%exploit loop     
     va2(E,T) =ceil(nthroot((E^J)/(Susceptibility(T)),J)); %use ceil as needs to be rounded UP to a whole number 
     Sp2(E,T)=(E.^2)./((va2(E,T)).^2)%4loci - true SP for peak time
    end
end

figure
plot(va2(:,1),'k*-','Linewidth',2)
hold on
plot(va2(:,2),'k+-','Linewidth',2)
plot(va2(:,3),'ko-','Linewidth',2)
legend('80% Tolerance','90% Tolerance','95% Tolerance')
xlabel('Number of exploits per locus')
ylabel('Average Variant Richness')
title('Resistance Tolerance - 2 loci AND')

%Peak time With diversity
Bn=0.02;
Nn = 20000;
I0 = 1
term1 = 1./(Sp2.*Bn);
term2 = log((((Sp2.*Nn)-1).*((Sp2.*Nn)-I0))./(I0));
T2 = term1.*term2;
figure
plot(T2(:,1)./60./60,'k*-','Linewidth',2)
hold on
plot(T2(:,2)./60./60,'k+-','Linewidth',2)
plot(T2(:,3)./60./60,'ko-','Linewidth',2)
legend('80% Tolerance','90% Tolerance','95% Tolerance')
xlabel('Number of exploits per locus')
ylabel('Peak Infection Time (Hours)')
title('Peak Infection Time - 2 loci AND')

%**************
%OR two loci  - using average approximation
%**************
J=2 %2 loci
for T = 1:3 %tolerance loop
    for E = 1:8%exploit loop     
     vo2(E,T) =ceil((E.*J)./(Susceptibility(T))); %use ceil as needs to be rounded UP to a whole number 
     Spo2(E,T)=(E.*J)./((vo2(E,T)));%4loci - true SP for peak time
    end
end
vo2
Spo2
figure
plot(vo2(:,1),'k*-','Linewidth',2)
hold on
plot(vo2(:,2),'k+-','Linewidth',2)
plot(vo2(:,3),'ko-','Linewidth',2)
legend('80% Tolerance','90% Tolerance','95% Tolerance')
xlabel('Number of exploits per locus')
ylabel('Average Variant Richness')
title('Resistance Tolerance - 2 loci OR  - using average approximation')

%Peak time With diversity
Bn=0.02;
Nn = 20000;
I0 = 1
term1 = 1./(Spo2.*Bn);
term2 = log((((Spo2.*Nn)-1).*((Spo2.*Nn)-I0))./(I0));
T2 = term1.*term2;
figure
plot(T2(:,1)./60./60,'k*-','Linewidth',2)
hold on
plot(T2(:,2)./60./60,'k+-','Linewidth',2)
plot(T2(:,3)./60./60,'ko-','Linewidth',2)
legend('80% Tolerance','90% Tolerance','95% Tolerance')
xlabel('Number of exploits per locus')
ylabel('Peak Infection Time (Hours)')
title('Peak Infection Time - 2 loci OR  - using average approximation')


%**************
%AND three loci 
%**************
J=3 %2 loci
for T = 1:3 %tolerance loop
    for E = 1:8%exploit loop     
     va3(E,T) =ceil(nthroot((E^J)/(Susceptibility(T)),J)); %use ceil as needs to be rounded UP to a whole number 
     Sp3(E,T)=(E.^3)./((va3(E,T)).^3)%3loci - true SP for peak time
    end
end

figure
plot(va3(:,1),'k*-','Linewidth',2)
hold on
plot(va3(:,2),'k+-','Linewidth',2)
plot(va3(:,3),'ko-','Linewidth',2)
legend('80% Tolerance','90% Tolerance','95% Tolerance')
xlabel('Number of exploits per locus')
ylabel('Average Variant Richness')
title('Resistance Tolerance - 3 loci AND')

%Peak time With diversity
Bn=0.02;
Nn = 20000;
I0 = 1
term1 = 1./(Sp3.*Bn);
term2 = log((((Sp3.*Nn)-1).*((Sp3.*Nn)-I0))./(I0));
T3 = term1.*term2;
figure
plot(T3(:,1)./60./60,'k*-','Linewidth',2)
hold on
plot(T3(:,2)./60./60,'k+-','Linewidth',2)
plot(T3(:,3)./60./60,'ko-','Linewidth',2)
legend('80% Tolerance','90% Tolerance','95% Tolerance')
xlabel('Number of exploits per locus')
ylabel('Peak Infection Time (Hours)')
title('Peak Infection Time - 3 loci AND')

%**************
%OR three loci  - using average approximation
%**************
J=3 %2 loci
for T = 1:3 %tolerance loop
    for E = 1:8%exploit loop     
     vo3(E,T) =ceil((E.*J)/(Susceptibility(T))); %use ceil as needs to be rounded UP to a whole number 
     Spo3(E,T)=(E.*J)./((vo3(E,T)))%3loci - true SP for peak time
    end
end

figure
plot(vo3(:,1),'k*-','Linewidth',2)
hold on
plot(vo3(:,2),'k+-','Linewidth',2)
plot(vo3(:,3),'ko-','Linewidth',2)
legend('80% Tolerance','90% Tolerance','95% Tolerance')
xlabel('Number of exploits per locus')
ylabel('Average Variant Richness')
title('Resistance Tolerance - 3 loci OR - using average approximation')

%Peak time With diversity
Bn=0.02;
Nn = 20000;
I0 = 1
term1 = 1./(Spo3.*Bn);
term2 = log((((Spo3.*Nn)-1).*((Spo3.*Nn)-I0))./(I0));
T3 = term1.*term2;
figure
plot(T3(:,1)./60./60,'k*-','Linewidth',2)
hold on
plot(T3(:,2)./60./60,'k+-','Linewidth',2)
plot(T3(:,3)./60./60,'ko-','Linewidth',2)
legend('80% Tolerance','90% Tolerance','95% Tolerance')
xlabel('Number of exploits per locus')
ylabel('Peak Infection Time (Hours)')
title('Peak Infection Time - 3 loci OR - using average approximation')

%**************
%AND four loci 
%**************
J=4 %2 loci
for T = 1:3 %tolerance loop
    for E = 1:8%exploit loop     
     va4(E,T) =ceil(nthroot((E^J)/(Susceptibility(T)),J)); %use ceil as needs to be rounded UP to a whole number 
     Sp4(E,T)=(E.^4)./((va4(E,T)).^4)%4loci - true SP for peak time
    end
end
%Average variant Richness
figure
plot(va4(:,1),'k*-','Linewidth',2)
hold on
plot(va4(:,2),'k+-','Linewidth',2)
plot(va4(:,3),'ko-','Linewidth',2)
legend('80% Tolerance','90% Tolerance','95% Tolerance')
xlabel('Number of exploits per locus')
ylabel('Average Variant Richness')
title('Resistance Tolerance - 4 loci AND')

%Peak time With diversity
Bn=0.02;
Nn = 20000;
I0 = 1
term1 = 1./(Sp4.*Bn);
term2 = log((((Sp4.*Nn)-1).*((Sp4.*Nn)-I0))./(I0));
T4 = term1.*term2;
figure
plot(T4(:,1)./60./60,'k*-','Linewidth',2)
hold on
plot(T4(:,2)./60./60,'k+-','Linewidth',2)
plot(T4(:,3)./60./60,'ko-','Linewidth',2)
legend('80% Tolerance','90% Tolerance','95% Tolerance')
xlabel('Number of exploits per locus')
ylabel('Peak Infection Time (Hours)')
title('Peak Infection Time - 4 loci AND')

%**************
%OR four loci - using average approximation
%**************
J=4 %4 loci
for T = 1:3 %tolerance loop
    for E = 1:8%exploit loop     
     vo4(E,T) =ceil((E.*J)/(Susceptibility(T))); %use ceil as needs to be rounded UP to a whole number 
     Spo4(E,T)=(E.*J)./((vo4(E,T)))%4loci - true SP for peak time
    end
end
%Average variant Richness
figure
plot(vo4(:,1),'k*-','Linewidth',2)
hold on
plot(vo4(:,2),'k+-','Linewidth',2)
plot(vo4(:,3),'ko-','Linewidth',2)
legend('80% Tolerance','90% Tolerance','95% Tolerance')
xlabel('Number of exploits per locus')
ylabel('Average Variant Richness')
title('Resistance Tolerance - 4 loci OR - using average approximation')

%Peak time With diversity
Bn=0.02;
Nn = 20000;
I0 = 1
term1 = 1./(Spo4.*Bn);
term2 = log((((Spo4.*Nn)-1).*((Spo4.*Nn)-I0))./(I0));
T4 = term1.*term2;
figure
plot(T4(:,1)./60./60,'k*-','Linewidth',2)
hold on
plot(T4(:,2)./60./60,'k+-','Linewidth',2)
plot(T4(:,3)./60./60,'ko-','Linewidth',2)
legend('80% Tolerance','90% Tolerance','95% Tolerance')
xlabel('Number of exploits per locus')
ylabel('Peak Infection Time (Hours)')
title('Peak Infection Time - 4 loci OR - using average approximation')




%***********************************************************
%combined loci graphs of average variant richness - AND TYPE
figure
plot(v1(:,1),'k*-','Linewidth',2)
hold on
plot(v1(:,2),'k+-','Linewidth',2)
plot(v1(:,3),'ko-','Linewidth',2)
plot(va2(:,1),'k*-')
plot(va2(:,2),'k+-')
plot(va2(:,3),'ko-')
legend('80% Tolerance 1 locus','90% Tolerance 1 locus','95% Tolerance 1 locus','80% Tolerance 2 loci','90% Tolerance 2 loci','95% Tolerance 2 loci')
%legend('80% Tolerance 1 locus','80% Tolerance 2 loci','80% Tolerance 3 loci','80% Tolerance 4 loci')

xlabel('Number of exploits per locus')
ylabel('Average Variant Richness')
title('Resistance Tolerance - 1 locus and 2 loci AND')
axis([1 8 0 160])

figure
plot(v1(:,1),'k*-','Linewidth',2)
hold on
plot(v1(:,2),'k+-','Linewidth',2)
plot(v1(:,3),'ko-','Linewidth',2)
plot(va3(:,1),'k*-')
plot(va3(:,2),'k+-')
plot(va3(:,3),'ko-')
legend('80% Tolerance 1 locus','90% Tolerance 1 locus','95% Tolerance 1 locus','80% Tolerance 3 loci','90% Tolerance 3 loci','95% Tolerance 3 loci')
%legend('90% Tolerance 1 locus','90% Tolerance 2 loci','90% Tolerance 3 loci','90% Tolerance 4 loci')
xlabel('Number of exploits per locus')
ylabel('Average Variant Richness')
title('Resistance Tolerance - 1 locus and 3 loci AND')
axis([1 8 0 160])

figure
plot(v1(:,1),'k*-','Linewidth',2)
hold on
plot(v1(:,2),'k+-','Linewidth',2)
plot(v1(:,3),'ko-','Linewidth',2)
plot(va4(:,1),'k*-')
plot(va4(:,2),'k+-')
plot(va4(:,3),'ko-')
legend('80% Tolerance 1 locus','90% Tolerance 1 locus','95% Tolerance 1 locus','80% Tolerance 4 loci','90% Tolerance 4 loci','95% Tolerance 4 loci')
%legend('95% Tolerance 1 locus','95% Tolerance 2 loci','95% Tolerance 3 loci','95% Tolerance 4 loci')
xlabel('Number of exploits per locus')
ylabel('Average Variant Richness')
title('Resistance Tolerance - 1 locus and 4 loci AND')
axis([1 8 0 160])

%***********************************************************
%combined loci graphs of average variant richness - OR TYPE
figure
plot(v1(:,1),'k*-','Linewidth',2)
hold on
plot(v1(:,2),'k+-','Linewidth',2)
plot(v1(:,3),'ko-','Linewidth',2)
plot(vo2(:,1),'k*-')%using average approximation
plot(vo2(:,2),'k+-')
plot(vo2(:,3),'ko-')
legend('80% Tolerance 1 locus','90% Tolerance 1 locus','95% Tolerance 1 locus','80% Tolerance 2 loci','90% Tolerance 2 loci','95% Tolerance 2 loci')
%legend('80% Tolerance 1 locus','80% Tolerance 2 loci','80% Tolerance 3 loci','80% Tolerance 4 loci')

xlabel('Number of exploits per locus')
ylabel('Average Variant Richness')
title('Resistance Tolerance - 1 locus and 2 loci OR')
axis([1 8 0 700])

figure
plot(v1(:,1),'k*-','Linewidth',2)
hold on
plot(v1(:,2),'k+-','Linewidth',2)
plot(v1(:,3),'ko-','Linewidth',2)
plot(vo3(:,1),'k*-')%using average approximation
plot(vo3(:,2),'k+-')
plot(vo3(:,3),'ko-')
legend('80% Tolerance 1 locus','90% Tolerance 1 locus','95% Tolerance 1 locus','80% Tolerance 3 loci','90% Tolerance 3 loci','95% Tolerance 3 loci')
%legend('90% Tolerance 1 locus','90% Tolerance 2 loci','90% Tolerance 3 loci','90% Tolerance 4 loci')
xlabel('Number of exploits per locus')
ylabel('Average Variant Richness')
title('Resistance Tolerance - 1 locus and 3 loci OR')
axis([1 8 0 700])

figure
plot(v1(:,1),'k*-','Linewidth',2)
hold on
plot(v1(:,2),'k+-','Linewidth',2)
plot(v1(:,3),'ko-','Linewidth',2)
plot(vo4(:,1),'k*-')%using average approximation
plot(vo4(:,2),'k+-')
plot(vo4(:,3),'ko-')
legend('80% Tolerance 1 locus','90% Tolerance 1 locus','95% Tolerance 1 locus','80% Tolerance 4 loci','90% Tolerance 4 loci','95% Tolerance 4 loci')
%legend('95% Tolerance 1 locus','95% Tolerance 2 loci','95% Tolerance 3 loci','95% Tolerance 4 loci')
xlabel('Number of exploits per locus')
ylabel('Average Variant Richness')
title('Resistance Tolerance - 1 locus and 4 loci OR')
axis([1 8 0 700])

%***********************************************************
%single loci graphs of approximated average variant richness - OR TYPE
figure
hold on
plot(vo2(:,1),'k*-', 'linewidth', 2)%using average approximation
plot(vo2(:,2),'k+-', 'linewidth', 2)
plot(vo2(:,3),'ko-', 'linewidth', 2)
legend('80% Tolerance','90% Tolerance','95% Tolerance')
%legend('80% Tolerance 1 locus','80% Tolerance 2 loci','80% Tolerance 3 loci','80% Tolerance 4 loci')
xlabel('Number of exploits per locus')
ylabel('Variant Richness')
title('Resistance Tolerance - 2 loci OR average approximation')
%axis([1 8 0 700])

figure
hold on
plot(vo3(:,1),'k*-', 'linewidth', 2)%using average approximation
plot(vo3(:,2),'k+-', 'linewidth', 2)
plot(vo3(:,3),'ko-', 'linewidth', 2)
legend('80% Tolerance','90% Tolerance','95% Tolerance')
%legend('90% Tolerance 1 locus','90% Tolerance 2 loci','90% Tolerance 3 loci','90% Tolerance 4 loci')
xlabel('Number of exploits per locus')
ylabel('Variant Richness')
title('Resistance Tolerance - 3 loci OR average approximation')
%axis([1 8 0 700])

figure
hold on
plot(vo4(:,1),'k*-', 'linewidth', 2)%using average approximation
plot(vo4(:,2),'k+-', 'linewidth', 2)
plot(vo4(:,3),'ko-', 'linewidth', 2)
legend('80% Tolerance','90% Tolerance','95% Tolerance')
%legend('95% Tolerance 1 locus','95% Tolerance 2 loci','95% Tolerance 3 loci','95% Tolerance 4 loci')
xlabel('Number of exploits per locus')
ylabel('Variant Richness')
title('Resistance Tolerance - 4 loci OR average approximation')
%axis([1 8 0 700])


%*****************************
% AND total number of variants
%*****************************
v1Sum = v1.*1 %1 locus
va2Sum = va2.*2 %2 loci
va3Sum = va3.*3 %3 loci
va4Sum = va4.*4 %4 loci

figure
plot(v1Sum(:,1),'k*-','Linewidth',2)
hold on
plot(v1Sum(:,2),'k+-','Linewidth',2)
plot(v1Sum(:,3),'ko-','Linewidth',2)
plot(va4Sum(:,1),'k*-')
plot(va4Sum(:,2),'k+-')
plot(va4Sum(:,3),'ko-')
legend('80% Tolerance 1 locus','90% Tolerance 1 locus','95% Tolerance 1 locus','80% Tolerance 4 loci','90% Tolerance 4 loci','95% Tolerance 4 loci')
xlabel('Number of exploits per locus')
ylabel('Total number of variants (V)')
title('Resistance Tolerance - 1 locus and 4 loci - AND')

figure
plot(v1Sum(:,1),'k*-','Linewidth',2)
hold on
plot(v1Sum(:,2),'k+-','Linewidth',2)
plot(v1Sum(:,3),'ko-','Linewidth',2)
plot(va2Sum(:,1),'k*-')
plot(va2Sum(:,2),'k+-')
plot(va2Sum(:,3),'ko-')
legend('80% Tolerance 1 locus','90% Tolerance 1 locus','95% Tolerance 1 locus','80% Tolerance 2 loci','90% Tolerance 2 loci','95% Tolerance 2 loci')
xlabel('Number of exploits per locus')
ylabel('Total number of variants (V)')
title('Resistance Tolerance - 1 locus and 2 loci - AND')

figure
plot(v1Sum(:,1),'k*-','Linewidth',2)
hold on
plot(v1Sum(:,2),'k+-','Linewidth',2)
plot(v1Sum(:,3),'ko-','Linewidth',2)
plot(va3Sum(:,1),'k*-')
plot(va3Sum(:,2),'k+-')
plot(va3Sum(:,3),'ko-')
legend('80% Tolerance 1 locus','90% Tolerance 1 locus','95% Tolerance 1 locus','80% Tolerance 3 loci','90% Tolerance 3 loci','95% Tolerance 3 loci')
xlabel('Number of exploits per locus')
ylabel('Total number of variants (V)')
title('Resistance Tolerance - 1 locus and 3 loci - AND')

%*****************************
% OR total number of variants
%*****************************
v1Sum = v1.*1 %1 locus
vo2Sum = vo2.*2 %2 loci
vo3Sum = vo3.*3 %3 loci
vo4Sum = vo4.*4 %4 loci

figure
plot(v1Sum(:,1),'k*-','Linewidth',2)
hold on
plot(v1Sum(:,2),'k+-','Linewidth',2)
plot(v1Sum(:,3),'ko-','Linewidth',2)
plot(vo4Sum(:,1),'k*-')
plot(vo4Sum(:,2),'k+-')
plot(vo4Sum(:,3),'ko-')
legend('80% Tolerance 1 locus','90% Tolerance 1 locus','95% Tolerance 1 locus','80% Tolerance 4 loci','90% Tolerance 4 loci','95% Tolerance 4 loci')
xlabel('Number of exploits per locus')
ylabel('Total number of variants (V)')
title('Resistance Tolerance - 1 locus and 4 loci - OR')
axis([1 8 0 2600])

figure
plot(v1Sum(:,1),'k*-','Linewidth',2)
hold on
plot(v1Sum(:,2),'k+-','Linewidth',2)
plot(v1Sum(:,3),'ko-','Linewidth',2)
plot(vo2Sum(:,1),'k*-')
plot(vo2Sum(:,2),'k+-')
plot(vo2Sum(:,3),'ko-')
legend('80% Tolerance 1 locus','90% Tolerance 1 locus','95% Tolerance 1 locus','80% Tolerance 2 loci','90% Tolerance 2 loci','95% Tolerance 2 loci')
xlabel('Number of exploits per locus')
ylabel('Total number of variants (V)')
title('Resistance Tolerance - 1 locus and 2 loci - OR')
axis([1 8 0 2600])

figure
plot(v1Sum(:,1),'k*-','Linewidth',2)
hold on
plot(v1Sum(:,2),'k+-','Linewidth',2)
plot(v1Sum(:,3),'ko-','Linewidth',2)
plot(vo3Sum(:,1),'k*-')
plot(vo3Sum(:,2),'k+-')
plot(vo3Sum(:,3),'ko-')
legend('80% Tolerance 1 locus','90% Tolerance 1 locus','95% Tolerance 1 locus','80% Tolerance 3 loci','90% Tolerance 3 loci','95% Tolerance 3 loci')
xlabel('Number of exploits per locus')
ylabel('Total number of variants (V)')
title('Resistance Tolerance - 1 locus and 3 loci - OR')
axis([1 8 0 2600])


%*****************************
% And total number of genotypes
%*****************************
v1G = v1.*1 %1 locus
va2G = va2.^2 %2 loci
va3G= va3.^3 %3 loci
va4G = va4.^4 %4 loci
figure
plot(v1G(:,1),'k*-','Linewidth',2)
hold on
plot(v1G(:,2),'k+-','Linewidth',2)
plot(v1G(:,3),'ko-','Linewidth',2)
plot(va2G(:,1),'k*-')
hold on
plot(va2G(:,2),'k+-')
plot(va2G(:,3),'ko-')
legend('80% Tolerance 1 locus','90% Tolerance 1 locus','95% Tolerance 1 locus','80% Tolerance 2 loci','90% Tolerance 2 loci','95% Tolerance 2 loci')
xlabel('Number of exploits per locus')
ylabel('Total number of genotypes (G)')
title('Resistance Tolerance - 1 locus and 2 loci AND')

figure
plot(va3G(:,1),'k*-','Linewidth',2)
hold on
plot(va3G(:,2),'k+-','Linewidth',2)
plot(va3G(:,3),'ko-','Linewidth',2)
plot(va4G(:,1),'k*-')
hold on
plot(va4G(:,2),'k+-')
plot(va4G(:,3),'ko-')
legend('80% Tolerance 3 loci','90% Tolerance 3 loci','95% Tolerance 3 loci','80% Tolerance 4 loci','90% Tolerance 4 loci','95% Tolerance 4 loci')
xlabel('Number of exploits per locus')
ylabel('Total number of genotypes (G)')
title('Resistance Tolerance - 3 locus and 4 loci AND')

%*****************************
% And total number of genotypes
%*****************************
v1G = v1.*1 %1 locus
vo2G = vo2.^2 %2 loci
vo3G= vo3.^3 %3 loci
vo4G = vo4.^4 %4 loci
figure
plot(v1G(:,1),'k*-','Linewidth',2)
hold on
plot(v1G(:,2),'k+-','Linewidth',2)
plot(v1G(:,3),'ko-','Linewidth',2)
plot(vo2G(:,1),'k*-')
hold on
plot(vo2G(:,2),'k+-')
plot(vo2G(:,3),'ko-')
legend('80% Tolerance 1 locus','90% Tolerance 1 locus','95% Tolerance 1 locus','80% Tolerance 2 loci','90% Tolerance 2 loci','95% Tolerance 2 loci')
xlabel('Number of exploits per locus')
ylabel('Total number of genotypes (G)')
title('Resistance Tolerance - 1 locus and 2 loci OR')

figure
plot(vo3G(:,1),'k*-','Linewidth',2)
hold on
plot(vo3G(:,2),'k+-','Linewidth',2)
plot(vo3G(:,3),'ko-','Linewidth',2)
plot(vo4G(:,1),'k*-')
hold on
plot(vo4G(:,2),'k+-')
plot(vo4G(:,3),'ko-')
legend('80% Tolerance 3 loci','90% Tolerance 3 loci','95% Tolerance 3 loci','80% Tolerance 4 loci','90% Tolerance 4 loci','95% Tolerance 4 loci')
xlabel('Number of exploits per locus')
ylabel('Total number of genotypes (G)')
title('Resistance Tolerance - 3 locus and 4 loci OR')

figure
plot(v1G(:,1),'k*-','Linewidth',2)
hold on
plot(v1G(:,2),'k+-','Linewidth',2)
plot(v1G(:,3),'ko-','Linewidth',2)
legend('80% Tolerance','90% Tolerance','95% Tolerance')
xlabel('Number of exploits per locus')
ylabel('Total number of genotypes (G)')
title('Resistance Tolerance - 1 locus AND')

figure
plot(vo2G(:,1),'k*-','Linewidth',2)
hold on
plot(vo2G(:,2),'k+-','Linewidth',2)
plot(vo2G(:,3),'ko-','Linewidth',2)
legend('80% Tolerance','90% Tolerance','95% Tolerance')
xlabel('Number of exploits per locus')
ylabel('Total number of genotypes (G)')
title('Resistance Tolerance - 2 loci AND')

figure
plot(vo3G(:,1),'k*-','Linewidth',2)
hold on
plot(vo3G(:,2),'k+-','Linewidth',2)
plot(vo3G(:,3),'ko-','Linewidth',2)
legend('80% Tolerance','90% Tolerance','95% Tolerance')
xlabel('Number of exploits per locus')
ylabel('Total number of genotypes (G)')
title('Resistance Tolerance - 3 loci AND')

figure
plot(vo4G(:,1),'k*-','Linewidth',2)
hold on
plot(vo4G(:,2),'k+-','Linewidth',2)
plot(vo4G(:,3),'ko-','Linewidth',2)
legend('80% Tolerance','90% Tolerance','95% Tolerance')
xlabel('Number of exploits per locus')
ylabel('Total number of genotypes (G)')
title('Resistance Tolerance - 4 loci AND')



%**************
%OR two loci 
%**************
J=2 %2 loci
for T = 1:3 %tolerance loop
    for E = 1:8%exploit loop 
        %solve('((2*x/v)-(x^2/v^2)=s)','v')
        vo2_1 = E/(sqrt(1-Susceptibility(T))+1);
        vo2_2 = E/(sqrt(1-Susceptibility(T))-1);
     vo2(E,T) =ceil(vo2_1-vo2_2); %use ceil as needs to be rounded UP to a whole number 
    end
end
        %solve('((2*x/v)-(x^2/v^2)=s)','v')
        %T = 80%
        result = solve('((2*1/v)-(1^2/v^2)=0.2)','v'); vo2_2(1,1) = result(1); %S=0.2,E=1
        result = solve('((2*2/v)-(2^2/v^2)=0.2)','v'); vo2_2(2,1) = result(1); %S=0.2,E=1
        result = solve('((2*3/v)-(3^2/v^2)=0.2)','v'); vo2_2(3,1) = result(1); %S=0.2,E=1
        result = solve('((2*4/v)-(4^2/v^2)=0.2)','v'); vo2_2(4,1) = result(1); %S=0.2,E=1
        result = solve('((2*5/v)-(5^2/v^2)=0.2)','v'); vo2_2(5,1) = result(1); %S=0.2,E=1
        result = solve('((2*6/v)-(6^2/v^2)=0.2)','v'); vo2_2(6,1) = result(1); %S=0.2,E=1
        result = solve('((2*7/v)-(7^2/v^2)=0.2)','v'); vo2_2(7,1) = result(1); %S=0.2,E=1
        result = solve('((2*8/v)-(8^2/v^2)=0.2)','v'); vo2_2(8,1) = result(1); %S=0.2,E=1
        %T = 90%
        result = solve('((2*1/v)-(1^2/v^2)=0.1)','v'); vo2_2(1,2) = result(1); %S=0.1,E=1
        result = solve('((2*2/v)-(2^2/v^2)=0.1)','v'); vo2_2(2,2) = result(1); %S=0.1,E=1
        result = solve('((2*3/v)-(3^2/v^2)=0.1)','v'); vo2_2(3,2) = result(1); %S=0.1,E=1
        result = solve('((2*4/v)-(4^2/v^2)=0.1)','v'); vo2_2(4,2) = result(1); %S=0.1,E=1
        result = solve('((2*5/v)-(5^2/v^2)=0.1)','v'); vo2_2(5,2) = result(1); %S=0.1,E=1
        result = solve('((2*6/v)-(6^2/v^2)=0.1)','v'); vo2_2(6,2) = result(1); %S=0.1,E=1
        result = solve('((2*7/v)-(7^2/v^2)=0.1)','v'); vo2_2(7,2) = result(1); %S=0.1,E=1
        result = solve('((2*8/v)-(8^2/v^2)=0.1)','v'); vo2_2(8,2) = result(1); %S=0.1,E=1
        %T = 95%
        result = solve('((2*1/v)-(1^2/v^2)=0.05)','v'); vo2_2(1,3) = result(1); %S=0.05,E=1
        result = solve('((2*2/v)-(2^2/v^2)=0.05)','v'); vo2_2(2,3) = result(1); %S=0.05,E=1
        result = solve('((2*3/v)-(3^2/v^2)=0.05)','v'); vo2_2(3,3) = result(1); %S=0.05,E=1
        result = solve('((2*4/v)-(4^2/v^2)=0.05)','v'); vo2_2(4,3) = result(1); %S=0.05,E=1
        result = solve('((2*5/v)-(5^2/v^2)=0.05)','v'); vo2_2(5,3) = result(1); %S=0.05,E=1
        result = solve('((2*6/v)-(6^2/v^2)=0.05)','v'); vo2_2(6,3) = result(1); %S=0.05,E=1
        result = solve('((2*7/v)-(7^2/v^2)=0.05)','v'); vo2_2(7,3) = result(1); %S=0.05,E=1
        result = solve('((2*8/v)-(8^2/v^2)=0.05)','v'); vo2_2(8,3) = result(1); %S=0.05,E=1

figure
plot(vo2(:,1),'k*-') %approx
hold on
plot(vo2(:,2),'k+-')
plot(vo2(:,3),'ko-')

plot(vo2_2(:,1),'b*-')%exact
plot(vo2_2(:,2),'b+-')
plot(vo2_2(:,3),'bo-')
legend('80% Tolerance','90% Tolerance','95% Tolerance')
xlabel('Number of exploits per locus')
ylabel('Average Variant Richness')
title('Resistance Tolerance - 2 loci OR')

solve('((x1/v1)+(x2/v2)-(x1*x2/v1*v2)=s)','v1') %with different x and v values

%peak time with exact solution
%exact solution
vo2_2Round=ceil(vo2_2)%round up to nearest
for j = 1:3
for i = 1:8
sptrue2(i,j) =(2*i/vo2_2Round(i,j))-(i^2/(vo2_2Round(i,j))^2)
end
end
%Peak time With diversity
Bn=0.02;
Nn = 20000;
I0 = 1
term1 = 1./(sptrue2.*Bn);
term2 = log((((sptrue2.*Nn)-1).*((sptrue2.*Nn)-I0))./(I0));
T2 = term1.*term2;
figure
plot(T2(:,1)./60./60,'k*-','Linewidth',2)
hold on
plot(T2(:,2)./60./60,'k+-','Linewidth',2)
plot(T2(:,3)./60./60,'ko-','Linewidth',2)
legend('80% Tolerance','90% Tolerance','95% Tolerance')
xlabel('Number of exploits per locus')
ylabel('Peak Infection Time (Hours)')
title('Peak Infection Time - 2 loci OR - using average exact')


%**************
%OR three loci 
%**************
        %solve('((3*x/v)-(3*x^2/v^2)+(x^3/v^3)=s)','v')
        %T = 80%
        result = solve('((3*1/v)-(3*1^2/v^2)+(1^3/v^3)=0.2)','v'); vo3e(1,1) = result(1); %S=0.2,E=1
        result = solve('((3*2/v)-(3*2^2/v^2)+(2^3/v^3)=0.2)','v'); vo3e(2,1) = result(1); %S=0.2,E=2
        result = solve('((3*3/v)-(3*3^2/v^2)+(3^3/v^3)=0.2)','v'); vo3e(3,1) = result(1); %S=0.2,E=3
        result = solve('((3*4/v)-(3*4^2/v^2)+(4^3/v^3)=0.2)','v'); vo3e(4,1) = result(1); %S=0.2,E=4
        result = solve('((3*5/v)-(3*5^2/v^2)+(5^3/v^3)=0.2)','v'); vo3e(5,1) = result(1); %S=0.2,E=5
        result = solve('((3*6/v)-(3*6^2/v^2)+(6^3/v^3)=0.2)','v'); vo3e(6,1) = result(1); %S=0.2,E=6
        result = solve('((3*7/v)-(3*7^2/v^2)+(7^3/v^3)=0.2)','v'); vo3e(7,1) = result(1); %S=0.2,E=7
        result = solve('((3*8/v)-(3*8^2/v^2)+(8^3/v^3)=0.2)','v'); vo3e(8,1) = result(1); %S=0.2,E=8
        %T = 90%
        result = solve('((3*1/v)-(3*1^2/v^2)+(1^3/v^3)=0.1)','v'); vo3e(1,2) = result(1); %S=0.1,E=1
        result = solve('((3*2/v)-(3*2^2/v^2)+(2^3/v^3)=0.1)','v'); vo3e(2,2) = result(1); %S=0.1,E=2
        result = solve('((3*3/v)-(3*3^2/v^2)+(3^3/v^3)=0.1)','v'); vo3e(3,2) = result(1); %S=0.1,E=3
        result = solve('((3*4/v)-(3*4^2/v^2)+(4^3/v^3)=0.1)','v'); vo3e(4,2) = result(1); %S=0.1,E=4
        result = solve('((3*5/v)-(3*5^2/v^2)+(5^3/v^3)=0.1)','v'); vo3e(5,2) = result(1); %S=0.1,E=5
        result = solve('((3*6/v)-(3*6^2/v^2)+(6^3/v^3)=0.1)','v'); vo3e(6,2) = result(1); %S=0.1,E=6
        result = solve('((3*7/v)-(3*7^2/v^2)+(7^3/v^3)=0.1)','v'); vo3e(7,2) = result(1); %S=0.1,E=7
        result = solve('((3*8/v)-(3*8^2/v^2)+(8^3/v^3)=0.1)','v'); vo3e(8,2) = result(1); %S=0.1,E=8
        %T = 95%
        result = solve('((3*1/v)-(3*1^2/v^2)+(1^3/v^3)=0.05)','v'); vo3e(1,3) = result(1); %S=0.05,E=1
        result = solve('((3*2/v)-(3*2^2/v^2)+(2^3/v^3)=0.05)','v'); vo3e(2,3) = result(1); %S=0.05,E=2
        result = solve('((3*3/v)-(3*3^2/v^2)+(3^3/v^3)=0.05)','v'); vo3e(3,3) = result(1); %S=0.05,E=3
        result = solve('((3*4/v)-(3*4^2/v^2)+(4^3/v^3)=0.05)','v'); vo3e(4,3) = result(1); %S=0.05,E=4
        result = solve('((3*5/v)-(3*5^2/v^2)+(5^3/v^3)=0.05)','v'); vo3e(5,3) = result(1); %S=0.05,E=5
        result = solve('((3*6/v)-(3*6^2/v^2)+(6^3/v^3)=0.05)','v'); vo3e(6,3) = result(1); %S=0.05,E=6
        result = solve('((3*7/v)-(3*7^2/v^2)+(7^3/v^3)=0.05)','v'); vo3e(7,3) = result(1); %S=0.05,E=7
        result = solve('((3*8/v)-(3*8^2/v^2)+(8^3/v^3)=0.05)','v'); vo3e(8,3) = result(1); %S=0.05,E=8

figure
plot(vo3e(:,1),'k*-')
hold on
plot(vo3e(:,2),'k+-')
plot(vo3e(:,3),'ko-')
legend('80% Tolerance','90% Tolerance','95% Tolerance')
xlabel('Number of exploits per locus')
ylabel('Average Variant Richness')
title('Resistance Tolerance - 3 loci OR')

figure
plot(vo3e(:,1),'k*-')%exact
hold on
plot(vo3e(:,2),'k+-')
plot(vo3e(:,3),'ko-')
plot(vo3(:,1),'r*-')%approx
plot(vo3(:,2),'r+-')
plot(vo3(:,3),'ro-')
legend('80% Tolerance','90% Tolerance','95% Tolerance')
xlabel('Number of exploits per locus')
ylabel('Average Variant Richness')
title('Resistance Tolerance - 3 loci OR')

solve('((x1/v1)+(x2/v2)+(x3/v3)-(x1*x2/v1*v2)-(x1*x3/v1*v3)-(x2*x3/v2*v3)+(x1*x2*x3/v1*v2*v3)=s)','v1') %with different x and v values
 
%peak time with exact solution
%exact solution
vo3eRound=ceil(vo3e)%round up to nearest
for j = 1:3
for i = 1:8
sptrue3(i,j) = (3*i/(vo3eRound(i,j)))-(3*i^2/(vo3eRound(i,j))^2)+(i^3/(vo3eRound(i,j))^3);
end
end
%Peak time With diversity
Bn=0.02;
Nn = 20000;
I0 = 1
term1 = 1./(sptrue3.*Bn);
term2 = log((((sptrue3.*Nn)-1).*((sptrue3.*Nn)-I0))./(I0));
T3 = term1.*term2
figure
plot(T3(:,1)./60./60,'k*-','Linewidth',2)
hold on
plot(T3(:,2)./60./60,'k+-','Linewidth',2)
plot(T3(:,3)./60./60,'ko-','Linewidth',2)
legend('80% Tolerance','90% Tolerance','95% Tolerance')
xlabel('Number of exploits per locus')
ylabel('Peak Infection Time (Hours)')
title('Peak Infection Time - 3 loci OR - using average exact')

%**************
%OR four loci 
%**************
        %solve('((4*x/v)-(6*x^2/v^2)+(4*x^3/v^3)-(x^4/v^4)=s)','v')
        %get result(1) to ensure largest v solution which needs to be
        %greater than x and not imaginary as the solve function returns
        %range of possible solutions
        %T = 80%
        result = solve('((4*1/v)-(6*1^2/v^2)+(4*1^3/v^3)-(1^4/v^4)=0.2)','v'); vo4e(1,1) = result(1); %S=0.2,E=1
        result = solve('((4*2/v)-(6*2^2/v^2)+(4*2^3/v^3)-(2^4/v^4)=0.2)','v'); vo4e(2,1) = result(1); %S=0.2,E=2
        result = solve('((4*3/v)-(6*3^2/v^2)+(4*3^3/v^3)-(3^4/v^4)=0.2)','v'); vo4e(3,1) = result(1); %S=0.2,E=3
        result = solve('((4*4/v)-(6*4^2/v^2)+(4*4^3/v^3)-(4^4/v^4)=0.2)','v'); vo4e(4,1) = result(1); %S=0.2,E=4
        result = solve('((4*5/v)-(6*5^2/v^2)+(4*5^3/v^3)-(5^4/v^4)=0.2)','v'); vo4e(5,1) = result(1); %S=0.2,E=5
        result = solve('((4*6/v)-(6*6^2/v^2)+(4*6^3/v^3)-(6^4/v^4)=0.2)','v'); vo4e(6,1) = result(1); %S=0.2,E=6
        result = solve('((4*7/v)-(6*7^2/v^2)+(4*7^3/v^3)-(7^4/v^4)=0.2)','v'); vo4e(7,1) = result(1); %S=0.2,E=7
        result = solve('((4*8/v)-(6*8^2/v^2)+(4*8^3/v^3)-(8^4/v^4)=0.2)','v'); vo4e(8,1) = result(1); %S=0.2,E=8
        %T = 90%
        result = solve('((4*1/v)-(6*1^2/v^2)+(4*1^3/v^3)-(1^4/v^4)=0.1)','v'); vo4e(1,2) = result(1); %S=0.1,E=1
        result = solve('((4*2/v)-(6*2^2/v^2)+(4*2^3/v^3)-(2^4/v^4)=0.1)','v'); vo4e(2,2) = result(1); %S=0.1,E=2
        result = solve('((4*3/v)-(6*3^2/v^2)+(4*3^3/v^3)-(3^4/v^4)=0.1)','v'); vo4e(3,2) = result(1); %S=0.1,E=3
        result = solve('((4*4/v)-(6*4^2/v^2)+(4*4^3/v^3)-(4^4/v^4)=0.1)','v'); vo4e(4,2) = result(1); %S=0.1,E=4
        result = solve('((4*5/v)-(6*5^2/v^2)+(4*5^3/v^3)-(5^4/v^4)=0.1)','v'); vo4e(5,2) = result(1); %S=0.1,E=5
        result = solve('((4*6/v)-(6*6^2/v^2)+(4*6^3/v^3)-(6^4/v^4)=0.1)','v'); vo4e(6,2) = result(1); %S=0.1,E=6
        result = solve('((4*7/v)-(6*7^2/v^2)+(4*7^3/v^3)-(7^4/v^4)=0.1)','v'); vo4e(7,2) = result(1); %S=0.1,E=7
        result = solve('((4*8/v)-(6*8^2/v^2)+(4*8^3/v^3)-(8^4/v^4)=0.1)','v'); vo4e(8,2) = result(1); %S=0.1,E=8
        %T = 95%
        result = solve('((4*1/v)-(6*1^2/v^2)+(4*1^3/v^3)-(1^4/v^4)=0.05)','v'); vo4e(1,3) = result(1); %S=0.05,E=1
        result = solve('((4*2/v)-(6*2^2/v^2)+(4*2^3/v^3)-(2^4/v^4)=0.05)','v'); vo4e(2,3) = result(1); %S=0.05,E=2
        result = solve('((4*3/v)-(6*3^2/v^2)+(4*3^3/v^3)-(3^4/v^4)=0.05)','v'); vo4e(3,3) = result(1); %S=0.05,E=3
        result = solve('((4*4/v)-(6*4^2/v^2)+(4*4^3/v^3)-(4^4/v^4)=0.05)','v'); vo4e(4,3) = result(1); %S=0.05,E=4
        result = solve('((4*5/v)-(6*5^2/v^2)+(4*5^3/v^3)-(5^4/v^4)=0.05)','v'); vo4e(5,3) = result(1); %S=0.05,E=5
        result = solve('((4*6/v)-(6*6^2/v^2)+(4*6^3/v^3)-(6^4/v^4)=0.05)','v'); vo4e(6,3) = result(1); %S=0.05,E=6
        result = solve('((4*7/v)-(6*7^2/v^2)+(4*7^3/v^3)-(7^4/v^4)=0.05)','v'); vo4e(7,3) = result(1); %S=0.05,E=7
        result = solve('((4*8/v)-(6*8^2/v^2)+(4*8^3/v^3)-(8^4/v^4)=0.05)','v'); vo4e(8,3) = result(1); %S=0.05,E=8
figure
plot(vo4e(:,1),'k*-')
hold on
plot(vo4e(:,2),'k+-')
plot(vo4e(:,3),'ko-')
legend('80% Tolerance','90% Tolerance','95% Tolerance')
xlabel('Number of exploits per locus')
ylabel('Average Variant Richness')
title('Resistance Tolerance - 4 loci OR')
        
figure
plot(vo4e(:,1),'k*-')%exact
hold on
plot(vo4e(:,2),'k+-')
plot(vo4e(:,3),'ko-')
plot(vo4(:,1),'r*-')%approx
hold on
plot(vo4(:,2),'r+-')
plot(vo4(:,3),'ro-')
legend('80% Tolerance','90% Tolerance','95% Tolerance')
xlabel('Number of exploits per locus')
ylabel('Average Variant Richness')
title('Resistance Tolerance - 4 loci OR')

%peak time with exact solution
%exact solution
vo4eRound=ceil(vo4e)%round up to nearest
for j = 1:3
for i = 1:8
sptrue4(i,j) =(4*i/(vo4eRound(i,j)))-(6*i^2/(vo4eRound(i,j))^2)+(4*i^3/(vo4eRound(i,j))^3)-(i^4/(vo4eRound(i,j))^4)
end
end
%Peak time With diversity
Bn=0.02;
Nn = 20000;
I0 = 1
term1 = 1./(sptrue4.*Bn);
term2 = log((((sptrue4.*Nn)-1).*((sptrue4.*Nn)-I0))./(I0));
T4 = term1.*term2;
figure
plot(T4(:,1)./60./60,'k*-','Linewidth',2)
hold on
plot(T4(:,2)./60./60,'k+-','Linewidth',2)
plot(T4(:,3)./60./60,'ko-','Linewidth',2)
legend('80% Tolerance','90% Tolerance','95% Tolerance')
xlabel('Number of exploits per locus')
ylabel('Peak Infection Time (Hours)')
title('Peak Infection Time - 4 loci OR - using average exact')


%*************
% AND only - 80%
%*************
figure
plot(v1(:,1),'k*-')
hold on
plot(va2(:,1),'k+-')
plot(va3(:,1),'ko-')
plot(va4(:,1),'ks-')
legend('1 locus','2 loci','3 loci', '4 loci')
xlabel('Number of exploits per locus')
ylabel('Average Variant Richness')
title('Resistance Tolerance - 80% - AND type')
%*************
% AND only - 90%
%*************
figure
plot(v1(:,2),'k*-')
hold on
plot(va2(:,2),'k+-')
plot(va3(:,2),'ko-')
plot(va4(:,2),'ks-')
legend('1 locus','2 loci','3 loci', '4 loci')
xlabel('Number of exploits per locus')
ylabel('Average Variant Richness')
title('Resistance Tolerance - 90% - AND type')
%*************
% AND only - 95%
%*************
figure
plot(v1(:,3),'k*-')
hold on
plot(va2(:,3),'k+-')
plot(va3(:,3),'ko-')
plot(va4(:,3),'ks-')
legend('1 locus','2 loci','3 loci', '4 loci')
xlabel('Number of exploits per locus')
ylabel('Average Variant Richness')
title('Resistance Tolerance - 95% - AND type')

%*************
%AND all
%*************
figure
plot(v1(:,1),'k*-')
hold on
plot(va2(:,1),'k+-')
plot(va3(:,1),'ko-')
plot(va4(:,1),'ks-')

plot(v1(:,2),'r*-')
hold on
plot(va2(:,2),'r+-')
plot(va3(:,2),'ro-')
plot(va4(:,2),'rs-')

plot(v1(:,3),'b*-')
hold on
plot(va2(:,3),'b+-')
plot(va3(:,3),'bo-')
plot(va4(:,3),'bs-')
legend('1 locus','2 loci','3 loci', '4 loci')
xlabel('Number of exploits per locus')
ylabel('Average Variant Richness')
title('Resistance Tolerance - All - AND type')

%*************
% OR only - 80%
%*************
figure
plot(v1(:,1),'k*-')
hold on
plot(vo2e(:,1),'k+-')
plot(vo3e(:,1),'ko-')
plot(vo4e(:,1),'ks-')
legend('1 locus','2 loci','3 loci', '4 loci')
xlabel('Number of exploits per locus')
ylabel('Average Variant Richness')
title('Resistance Tolerance - 80% - OR type')
%*************
% OR only - 90%
%*************
figure
plot(v1(:,2),'k*-')
hold on
plot(vo2e(:,2),'k+-')
plot(vo3e(:,2),'ko-')
plot(vo4e(:,2),'ks-')
legend('1 locus','2 loci','3 loci', '4 loci')
xlabel('Number of exploits per locus')
ylabel('Average Variant Richness')
title('Resistance Tolerance - 90% - OR type')
%*************
% OR only - 95%
%*************
figure
plot(v1(:,3),'k*-')
hold on
plot(vo2e(:,3),'k+-')
plot(vo3e(:,3),'ko-')
plot(vo4e(:,3),'ks-')
legend('1 locus','2 loci','3 loci', '4 loci')
xlabel('Number of exploits per locus')
ylabel('Average Variant Richness')
title('Resistance Tolerance - 95% - OR type')

%*************
%OR all
%*************
figure
plot(v1(:,1),'k*-')
hold on
plot(vo2e(:,1),'k+-')
plot(vo3e(:,1),'ko-')
plot(vo4e(:,1),'ks-')

plot(v1(:,2),'r*-')
hold on
plot(vo2e(:,2),'r+-')
plot(vo3e(:,2),'ro-')
plot(vo4e(:,2),'rs-')

plot(v1(:,3),'b*-')
hold on
plot(vo2e(:,3),'b+-')
plot(vo3e(:,3),'bo-')
plot(vo4e(:,3),'bs-')
legend('1 locus','2 loci','3 loci', '4 loci')
xlabel('Number of exploits per locus')
ylabel('Average Variant Richness')
title('Resistance Tolerance - All - OR type')

%solve('(4/v) - (6/v^2) + (4/v^3) - (1/v^4) =0.2','v')
  
 

















end

